@@ -11,6 +11,10 @@ import com.unciv.ui.components.fonts.Fonts
 import com.unciv.utils.Display
 import com.unciv.utils.Log
 import java.io.File
+import java.lang.Exception
+import kotlinx.coroutines.CoroutineScope
+import kotlinx.coroutines.Dispatchers
+import kotlinx.coroutines.launch
 
 open class AndroidLauncher : AndroidApplication() {
 
@@ -43,7 +47,9 @@ open class AndroidLauncher : AndroidApplication() {
         // Create notification channels for Multiplayer notificator
         MultiplayerTurnCheckWorker.createNotificationChannels(applicationContext)
 
-        copyMods()
+        CoroutineScope(Dispatchers.IO).launch {
+            copyMods()
+        }
 
         game = AndroidGame(this)
         initialize(game, config)
@@ -65,9 +71,11 @@ open class AndroidLauncher : AndroidApplication() {
         val externalPath = getExternalFilesDir(null)?.path ?: return
         val externalModsDir = File("$externalPath/mods")
 
-        // Copy external mod directory (with data user put in it) to internal (where it can be read)
-        if (!externalModsDir.exists()) externalModsDir.mkdirs() // this can fail sometimes, which is why we check if it exists again in the next line
-        if (externalModsDir.exists()) externalModsDir.copyRecursively(internalModsDir, true)
+        try { // Rarely we get a kotlin.io.AccessDeniedException, if so - no biggie
+            // Copy external mod directory (with data user put in it) to internal (where it can be read)
+            if (!externalModsDir.exists()) externalModsDir.mkdirs() // this can fail sometimes, which is why we check if it exists again in the next line
+            if (externalModsDir.exists()) externalModsDir.copyRecursively(internalModsDir, true)
+        } catch (ex: Exception) {}
     }
 
     override fun onPause() {
@@ -77,8 +85,14 @@ open class AndroidLauncher : AndroidApplication() {
                 && game.settings.multiplayer.turnCheckerEnabled
                 && game.files.getMultiplayerSaves().any()
         ) {
-            MultiplayerTurnCheckWorker.startTurnChecker(
-                applicationContext, game.files, game.gameInfo!!, game.settings.multiplayer)
+            CoroutineScope(Dispatchers.IO).launch {
+                MultiplayerTurnCheckWorker.startTurnChecker(
+                    applicationContext,
+                    game.files,
+                    game.gameInfo!!,
+                    game.settings.multiplayer
+                )
+            }
         }
         super.onPause()
     }
