@@ -4,6 +4,7 @@ object Constants {
     const val settler = "Settler"
     const val eraSpecificUnit = "Era Starting Unit"
     val all = setOf("All", "all")
+    const val NO_ID = -1
 
     const val english = "English"
 
@@ -41,11 +42,20 @@ object Constants {
 
     const val barbarianEncampment = "Barbarian encampment"
     const val cityCenter = "City center"
-
+    
+    // Treaties
     const val peaceTreaty = "Peace Treaty"
     const val researchAgreement = "Research Agreement"
-    const val openBorders = "Open Borders"
     const val defensivePact = "Defensive Pact"
+    
+    // Agreements
+    const val openBorders = "Open Borders"
+    
+    // Other trade items
+    const val acceptEmbassy = "Accept Embassy"
+    const val goldPerTurn = "Gold per turn"
+    const val flatGold = "Gold"
+    
     /** Used as origin in StatMap or ResourceSupplyList, or the toggle button in DiplomacyOverviewTab */
     const val cityStates = "City-States"
     /** Used as origin in ResourceSupplyList */
@@ -67,6 +77,8 @@ object Constants {
 
     const val cancelImprovementOrder = "Cancel improvement order"
     const val tutorialPopupNamePrefix = "Tutorial: "
+    const val thisUnit = "This Unit"
+    const val targetUnit = "Target Unit"
 
     const val OK = "OK"
     const val close = "Close"
@@ -86,6 +98,11 @@ object Constants {
     const val repair = "Repair"
 
     const val uniqueOrDelimiter = "\" OR \""
+    const val stringSplitCharacter = '␟' // U+241 - Unit separator character. Used to join texts and split them with a char that is virtually guaranteed to not be used in normal text. 
+
+
+    const val simulationCiv1 = "SimulationCiv1"
+    const val simulationCiv2 = "SimulationCiv2"
 
     const val dropboxMultiplayerServer = "Dropbox"
     const val uncivXyzServer = "https://uncivserver.xyz"
@@ -95,9 +112,10 @@ object Constants {
     const val defaultFallbackTileset = "FantasyHex"
     const val defaultUnitset = "AbsoluteUnits"
     const val defaultSkin = "Minimal"
+    const val defaultFallbackSkin = "Minimal"
 
     /**
-     * Use this to determine whether a [MapUnit][com.unciv.logic.map.MapUnit]'s movement is exhausted
+     * Use this to determine whether a [MapUnit][com.unciv.logic.map.mapunit.MapUnit]'s movement is exhausted
      * (currentMovement <= this) if and only if a fuzzy comparison is needed to account for Float rounding errors.
      * _Most_ checks do compare to 0!
      */
