@@ -6,9 +6,10 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.models.metadata.GameSettings
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.worldscreen.UndoHandler.Companion.clearUndoCheckpoints
-import com.unciv.ui.screens.worldscreen.WorldMapHolder
+import com.unciv.ui.screens.worldscreen.worldmap.WorldMapHolder
 import com.unciv.ui.screens.worldscreen.WorldScreen
 import com.unciv.ui.screens.worldscreen.unit.UnitTable
+import yairm210.purity.annotations.Readonly
 
 object GUI {
 
@@ -24,46 +25,29 @@ object GUI {
         UncivGame.Current.resetToWorldScreen()
     }
 
-    fun getSettings(): GameSettings {
-        return UncivGame.Current.settings
-    }
+    @Readonly fun getSettings(): GameSettings = UncivGame.Current.settings
 
-    fun isWorldLoaded(): Boolean {
-        return UncivGame.Current.worldScreen != null
-    }
+    @Readonly fun isWorldLoaded(): Boolean = UncivGame.Current.worldScreen != null
 
+    @Readonly
     fun isMyTurn(): Boolean {
         if (!UncivGame.isCurrentInitialized() || !isWorldLoaded()) return false
         return UncivGame.Current.worldScreen!!.isPlayersTurn
     }
 
-    fun isAllowedChangeState(): Boolean {
-        return UncivGame.Current.worldScreen!!.canChangeState
-    }
+    @Readonly fun isAllowedChangeState(): Boolean = UncivGame.Current.worldScreen!!.canChangeState
 
-    fun getWorldScreen(): WorldScreen {
-        return UncivGame.Current.worldScreen!!
-    }
+    @Readonly fun getWorldScreen(): WorldScreen = UncivGame.Current.worldScreen!!
 
-    fun getWorldScreenIfActive(): WorldScreen? {
-        return UncivGame.Current.getWorldScreenIfActive()
-    }
+    @Readonly fun getWorldScreenIfActive(): WorldScreen? = UncivGame.Current.getWorldScreenIfActive()
 
-    fun getMap(): WorldMapHolder {
-        return UncivGame.Current.worldScreen!!.mapHolder
-    }
+    @Readonly fun getMap(): WorldMapHolder = UncivGame.Current.worldScreen!!.mapHolder
 
-    fun getUnitTable(): UnitTable {
-        return UncivGame.Current.worldScreen!!.bottomUnitTable
-    }
+    @Readonly fun getUnitTable(): UnitTable = UncivGame.Current.worldScreen!!.bottomUnitTable
 
-    fun getViewingPlayer(): Civilization {
-        return UncivGame.Current.worldScreen!!.viewingCiv
-    }
+    @Readonly fun getViewingPlayer(): Civilization = UncivGame.Current.worldScreen!!.viewingCiv
 
-    fun getSelectedPlayer(): Civilization {
-        return UncivGame.Current.worldScreen!!.selectedCiv
-    }
+    @Readonly fun getSelectedPlayer(): Civilization = UncivGame.Current.worldScreen!!.selectedCiv
 
     /** Disable Undo (as in: forget the way back, but allow future undo checkpoints) */
     fun clearUndoCheckpoints() {
@@ -77,7 +61,7 @@ object GUI {
     }
 
     private var keyboardAvailableCache: Boolean? = null
-    /** Tests availability of a physical keyboard */
+    /** Tests availability of a physical keyboard - cached (connecting a keyboard while the game is running won't be recognized until relaunch) */
     val keyboardAvailable: Boolean
         get() {
             // defer decision if Gdx.input not yet initialized
