@@ -1,17 +1,14 @@
 package com.unciv
 
-import com.badlogic.gdx.Application
-import com.badlogic.gdx.Game
-import com.badlogic.gdx.Gdx
-import com.badlogic.gdx.Input
-import com.badlogic.gdx.Screen
-import com.badlogic.gdx.scenes.scene2d.actions.Actions
+import com.badlogic.gdx.*
+import com.unciv.UncivGame.Companion.Current
+import com.unciv.UncivGame.Companion.isCurrentInitialized
 import com.unciv.logic.GameInfo
-import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.UncivShowableException
+import com.unciv.logic.Version
 import com.unciv.logic.civilization.PlayerType
 import com.unciv.logic.files.UncivFiles
-import com.unciv.logic.multiplayer.OnlineMultiplayer
+import com.unciv.logic.multiplayer.Multiplayer
 import com.unciv.models.metadata.GameSettings
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.skins.SkinCache
@@ -21,13 +18,13 @@ import com.unciv.ui.audio.MusicController
 import com.unciv.ui.audio.MusicMood
 import com.unciv.ui.audio.MusicTrackChooserFlags
 import com.unciv.ui.audio.SoundPlayer
-import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.crashhandling.CrashScreen
 import com.unciv.ui.crashhandling.wrapCrashHandlingUnit
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.popups.Popup
+import com.unciv.ui.screens.GameStartScreen
 import com.unciv.ui.screens.LanguagePickerScreen
 import com.unciv.ui.screens.LoadingScreen
 import com.unciv.ui.screens.basescreen.BaseScreen
@@ -36,19 +33,19 @@ import com.unciv.ui.screens.savescreens.LoadGameScreen
 import com.unciv.ui.screens.worldscreen.PlayerReadyScreen
 import com.unciv.ui.screens.worldscreen.WorldScreen
 import com.unciv.ui.screens.worldscreen.unit.AutoPlay
-import com.unciv.utils.Concurrency
-import com.unciv.utils.DebugUtils
-import com.unciv.utils.Display
-import com.unciv.utils.Log
-import com.unciv.utils.PlatformSpecific
-import com.unciv.utils.debug
-import com.unciv.utils.launchOnGLThread
-import com.unciv.utils.withGLContext
-import com.unciv.utils.withThreadPoolContext
+import com.unciv.utils.*
 import kotlinx.coroutines.CancellationException
+import yairm210.purity.annotations.Readonly
 import java.io.PrintWriter
-import java.util.EnumSet
-import java.util.UUID
+import java.util.*
+import kotlin.collections.ArrayDeque
+import kotlin.collections.asSequence
+import kotlin.collections.count
+import kotlin.collections.filter
+import kotlin.collections.forEach
+import kotlin.collections.listOf
+import kotlin.collections.none
+import kotlin.collections.removeAll
 import kotlin.reflect.KClass
 
 /** Represents the Unciv app itself:
@@ -59,13 +56,14 @@ import kotlin.reflect.KClass
 open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpecific {
 
     var deepLinkedMultiplayerGame: String? = null
+    override var customDataDirectory: String? = null
 
     /** The game currently in progress */
     var gameInfo: GameInfo? = null
 
     lateinit var settings: GameSettings
     lateinit var musicController: MusicController
-    lateinit var onlineMultiplayer: OnlineMultiplayer
+    lateinit var onlineMultiplayer: Multiplayer
     lateinit var files: UncivFiles
 
     var isTutorialTaskCollapsed = false
@@ -92,7 +90,12 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
             DebugUtils.VISIBLE_MAP = false
         }
         Current = this
-        files = UncivFiles(Gdx.files)
+        files = UncivFiles(Gdx.files, customDataDirectory)
+        Concurrency.run {
+            // Delete temporary files created when downloading mods
+            val tempFiles = files.getLocalFile("mods").list().filter { !it.isDirectory && it.name().startsWith("temp-") }
+            for (file in tempFiles) file.delete()
+        }
 
         // If this takes too long players, especially with older phones, get ANR problems.
         // Whatever needs graphics needs to be done on the main thread,
@@ -113,7 +116,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         musicController = MusicController()  // early, but at this point does only copy volume from settings
         installAudioHooks()
 
-        onlineMultiplayer = OnlineMultiplayer()
+        onlineMultiplayer = Multiplayer()
 
         Concurrency.run {
             // Check if the server is available in case the feature set has changed
@@ -126,12 +129,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
 
         ImageGetter.resetAtlases()
         ImageGetter.reloadImages()  // This needs to come after the settings, since we may have default visual mods
-        val imageGetterTilesets = ImageGetter.getAvailableTilesets()
-        val availableTileSets = TileSetCache.getAvailableTilesets(imageGetterTilesets)
-        if (settings.tileSet !in availableTileSets) { // If the configured tileset is no longer available, default back
-            settings.tileSet = Constants.defaultTileset
-        }
-
+        
         Gdx.graphics.isContinuousRendering = settings.continuousRendering
 
         Concurrency.run("LoadJSON") {
@@ -139,12 +137,16 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
             translations.tryReadTranslationForCurrentLanguage()
             translations.loadPercentageCompleteOfLanguages()
             TileSetCache.loadTileSetConfigs()
+            if (settings.tileSet !in TileSetCache) { // The configured tileset is no longer available, default back
+                settings.tileSet = Constants.defaultTileset
+            }
+
             SkinCache.loadSkinConfigs()
 
             val vanillaRuleset = RulesetCache.getVanillaRuleset()
 
-            if (settings.multiplayer.userId.isEmpty()) { // assign permanent user id
-                settings.multiplayer.userId = UUID.randomUUID().toString()
+            if (settings.multiplayer.getUserId().isEmpty()) { // assign permanent user id
+                settings.multiplayer.setUserId(UUID.randomUUID().toString())
                 settings.save()
             }
 
@@ -189,11 +191,11 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
 
         if (gameInfo?.gameParameters?.isOnlineMultiplayer == true
                 && gameInfo?.gameParameters?.anyoneCanSpectate == false
-                && gameInfo!!.civilizations.none { it.playerId == settings.multiplayer.userId }) {
+                && gameInfo!!.civilizations.none { it.playerId == settings.multiplayer.getUserId() }) {
             throw UncivShowableException("You are not allowed to spectate!")
         }
 
-        initializeResources(prevGameInfo, newGameInfo)
+        initializeResources(newGameInfo)
 
         val isLoadingSameGame = worldScreen != null && prevGameInfo != null && prevGameInfo.gameId == newGameInfo.gameId
         val worldScreenRestoreState = if (!callFromLoadScreen && isLoadingSameGame) worldScreen!!.getRestoreState() else null
@@ -205,6 +207,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
             // we do it all in one swoop on the same thread and the application just "freezes" without loading screen for the duration.
             loadingScreen = LoadingScreen(getScreen())
             setScreen(loadingScreen)
+            Gdx.input.inputProcessor = null // It's just been set by setScreen, so unset it to avoid ANRs while loading
         }
 
         return@toplevel withGLContext {
@@ -224,7 +227,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
             }
 
             screenStack.addLast(screenToShow)
-            setScreen(screenToShow)
+            setScreen(screenToShow) // Only here do we set the inputProcessor again 
             loadingScreen.dispose()
 
             return@withGLContext newWorldScreen
@@ -232,16 +235,12 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
     }
 
     /** The new game info may have different mods or rulesets, which may use different resources that need to be loaded. */
-    private suspend fun initializeResources(prevGameInfo: GameInfo?, newGameInfo: GameInfo) {
-        if (prevGameInfo == null
-                || prevGameInfo.gameParameters.baseRuleset != newGameInfo.gameParameters.baseRuleset
-                || prevGameInfo.gameParameters.mods != newGameInfo.gameParameters.mods) {
-            withGLContext {
-                ImageGetter.setNewRuleset(newGameInfo.ruleset)
-            }
-            val fullModList = newGameInfo.gameParameters.getModsAndBaseRuleset()
-            musicController.setModList(fullModList)
+    private suspend fun initializeResources(newGameInfo: GameInfo) {
+        withGLContext {
+            ImageGetter.setNewRuleset(newGameInfo.ruleset, true)
         }
+        val fullModList = newGameInfo.gameParameters.getModsAndBaseRuleset()
+        musicController.setModList(fullModList)
     }
 
     /** Re-creates the current [worldScreen], if there is any. */
@@ -293,17 +292,22 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
      *
      * Automatically [disposes][BaseScreen.dispose] the old screen.
      *
+     * @param silentQuit Don't ask for exit confirmation when popping the last screen (for FasterUIDevelopment, where musicController is not initialized)
      * @return the new screen
      */
-    fun popScreen(): BaseScreen? {
+    fun popScreen(silentQuit: Boolean = false): BaseScreen? {
         if (screenStack.size == 1) {
+            if (silentQuit) {
+                Gdx.app.exit()
+                return null
+            }
             musicController.pause()
             worldScreen?.autoPlay?.stopAutoPlay()
             ConfirmPopup(
                 screen = screenStack.last(),
                 question = "Do you want to exit the game?",
                 confirmText = "Exit",
-                restoreDefault = { musicController.resume() },
+                restoreDefault = { musicController.resumeFromShutdown() },
                 action = { Gdx.app.exit() }
             ).open(force = true)
             return null
@@ -333,7 +337,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         // Re-initialize translations, images etc. that may have been 'lost' when we were playing around in NewGameScreen
         val ruleset = worldScreen.gameInfo.ruleset
         translations.translationActiveMods = ruleset.mods
-        ImageGetter.setNewRuleset(ruleset)
+        ImageGetter.setNewRuleset(ruleset, true)
 
         setScreen(worldScreen)
         return worldScreen
@@ -342,7 +346,9 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
     /** Get all currently existing screens of type [clazz]
      *  - Not a generic to allow screenStack to be private
      */
-    fun getScreensOfType(clazz: KClass<out BaseScreen>): Sequence<BaseScreen> = screenStack.asSequence().filter { it::class == clazz }
+    @Readonly
+    fun getScreensOfType(clazz: KClass<out BaseScreen>): Sequence<BaseScreen> = 
+        screenStack.asSequence().filter { it::class == clazz }
 
     /** Dispose and remove all currently existing screens of type [clazz]
      *  - Not a generic to allow screenStack to be private
@@ -362,7 +368,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
             }
         }
         try {
-            onlineMultiplayer.loadGame(deepLinkedMultiplayerGame!!)
+            onlineMultiplayer.downloadGame(deepLinkedMultiplayerGame!!)
         } catch (ex: Exception) {
             launchOnGLThread {
                 val mainMenu = MainMenuScreen()
@@ -383,7 +389,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
     override fun resume() {
         super.resume()
         if (!isInitialized) return // The stuff from Create() is still happening, so the main screen will load eventually
-        musicController.resume()
+        musicController.resumeFromShutdown()
 
         // This is also needed in resume to open links and notifications
         // correctly when the app was already running. The handling in onCreate
@@ -396,7 +402,8 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         // already has, but if we do _our_ pause before the MusicController timer notices, it will at least remember the current track.
         if (::musicController.isInitialized) musicController.pause()
         val curGameInfo = gameInfo
-        if (curGameInfo != null) files.autosaves.requestAutoSave(curGameInfo)
+        // Since we're pausing the game, we don't need to clone it before autosave - no one else will touch it
+        if (curGameInfo != null) files.autosaves.requestAutoSaveUnCloned(curGameInfo)
         super.pause()
     }
 
@@ -444,25 +451,8 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         }
     }
 
-    /** Handles an uncaught exception or error. First attempts a platform-specific handler, and if that didn't handle the exception or error, brings the game to a [CrashScreen]. */
-    fun handleUncaughtThrowable(ex: Throwable) {
-        if (ex is CancellationException) {
-            return // kotlin coroutines use this for control flow... so we can just ignore them.
-        }
-        Log.error("Uncaught throwable", ex)
-        try {
-            PrintWriter(files.fileWriter("lasterror.txt")).use {
-                ex.printStackTrace(it)
-            }
-        } catch (_: Exception) {
-            // ignore
-        }
-        Gdx.app.postRunnable {
-            setAsRootScreen(CrashScreen(ex))
-        }
-    }
-
     /** Returns the [worldScreen] if it is the currently active screen of the game */
+    @Readonly
     fun getWorldScreenIfActive(): WorldScreen? {
         return if (screen == worldScreen) worldScreen else null
     }
@@ -477,43 +467,54 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         return mainMenuScreen
     }
 
-    /** Sets a simulated [GameInfo] object this game should run on */
-    fun startSimulation(simulatedGameInfo: GameInfo) {
-        gameInfo = simulatedGameInfo
-    }
-
     companion object {
         //region AUTOMATICALLY GENERATED VERSION DATA - DO NOT CHANGE THIS REGION, INCLUDING THIS COMMENT
-        val VERSION = Version("4.11.16", 1001)
+        val VERSION = Version("4.19.11", 1199)
         //endregion
 
         /** Global reference to the one Gdx.Game instance created by the platform launchers - do not use without checking [isCurrentInitialized] first. */
         // Set by Gdx Game.create callback, or the special cases ConsoleLauncher and unit tests make do with out Gdx and set this themselves.
         lateinit var Current: UncivGame
         /** @return `true` if [Current] has been set yet and can be accessed */
-        fun isCurrentInitialized() = this::Current.isInitialized
+        @Readonly fun isCurrentInitialized() = this::Current.isInitialized
         /** Get the game currently in progress safely - null either if [Current] has not yet been set or if its gameInfo field has no game */
-        fun getGameInfoOrNull() = if (isCurrentInitialized()) Current.gameInfo else null
-        fun isCurrentGame(gameId: String): Boolean = isCurrentInitialized() && Current.gameInfo != null && Current.gameInfo!!.gameId == gameId
-        fun isDeepLinkedGameLoading() = isCurrentInitialized() && Current.deepLinkedMultiplayerGame != null
-    }
+        @Readonly fun getGameInfoOrNull() = if (isCurrentInitialized()) Current.gameInfo else null
+        @Readonly fun isCurrentGame(gameId: String): Boolean = isCurrentInitialized() && Current.gameInfo != null && Current.gameInfo!!.gameId == gameId
+        @Readonly fun isDeepLinkedGameLoading() = isCurrentInitialized() && Current.deepLinkedMultiplayerGame != null
 
-    data class Version(
-        val text: String,
-        val number: Int
-    ) : IsPartOfGameInfoSerialization {
-        @Suppress("unused") // used by json serialization
-        constructor() : this("", -1)
-        fun toNiceString() = "$text (Build $number)"
-    }
-}
+        @Readonly
+        fun getUserAgent(fallbackStr: String = "Unknown"): String = if (isCurrentInitialized()) {
+            "Unciv/${VERSION.toNiceString()}-GNU-Terry-Pratchett"
+        } else "Unciv/$fallbackStr-GNU-Terry-Pratchett"
 
-private class GameStartScreen : BaseScreen() {
-    init {
-        val logoImage = ImageGetter.getExternalImage("banner.png")
-        logoImage.center(stage)
-        logoImage.color.a = 0f
-        logoImage.addAction(Actions.alpha(1f, 0.3f))
-        stage.addActor(logoImage)
+        /** Handles an uncaught exception or error. First attempts a platform-specific handler, and if that didn't handle the exception or error, brings the game to a [CrashScreen]. */
+        fun handleUncaughtThrowable(ex: Throwable) {
+            if (ex is CancellationException) {
+                return // kotlin coroutines use this for control flow... so we can just ignore them.
+            }
+            Log.error("Uncaught throwable", ex)
+            dumpLastError(ex)
+            Gdx.app.postRunnable {
+                Gdx.input.inputProcessor =
+                    null // CrashScreen needs to toJson which can take a while
+                // This may not be enough, we may need to run "generate crash text" in a different thread,
+                //   but for now let's try this.
+                Current.setAsRootScreen(CrashScreen(ex))
+            }
+        }
+
+        private fun dumpLastError(ex: Throwable) {
+            try {
+                val files =
+                    if (isCurrentInitialized() && Current::files.isInitialized) Current.files
+                    else UncivFiles(Gdx.files, null)
+                PrintWriter(files.fileWriter("lasterror.txt")).use {
+                    ex.printStackTrace(it)
+                }
+            } catch (_: Exception) {
+                Log.debug("Failed to write exception to lasterror.txt", ex)
+                // ignore
+            }
+        }
     }
 }
