@@ -5,11 +5,8 @@ import com.badlogic.gdx.files.FileHandle
 import com.badlogic.gdx.utils.Json
 import com.badlogic.gdx.utils.JsonWriter
 import com.badlogic.gdx.utils.SerializationException
-import com.unciv.logic.civilization.CivRankingHistory
-import com.unciv.logic.civilization.Notification
-import com.unciv.logic.map.tile.TileHistory
+import com.unciv.logic.map.HexCoord
 import com.unciv.ui.components.input.KeyCharAndCode
-import com.unciv.ui.components.input.KeyboardBindings
 import java.time.Duration
 
 
@@ -24,9 +21,9 @@ fun json() = Json(JsonWriter.OutputType.json).apply {
     setIgnoreDeprecated(true)
     ignoreUnknownFields = true
 
-    setSerializer(HashMapVector2.getSerializerClass(), HashMapVector2.createSerializer())
     setSerializer(Duration::class.java, DurationSerializer())
     setSerializer(KeyCharAndCode::class.java, KeyCharAndCode.Serializer())
+    setSerializer(HexCoord::class.java, HexCoord.Serializer())
 }
 
 /**
