@@ -16,7 +16,7 @@ import com.unciv.models.metadata.GameSetupInfo
 import com.unciv.models.metadata.Player
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
@@ -24,6 +24,8 @@ import com.unciv.models.stats.Stats
 import com.unciv.models.translations.equalsPlaceholderText
 import com.unciv.models.translations.getPlaceholderParameters
 import com.unciv.utils.debug
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
 
 object GameStarter {
     // temporary instrumentation while tuning/debugging
@@ -90,7 +92,7 @@ object GameStarter {
             // Remove units for civs that aren't in this game
             for (tile in tileMap.values)
                 for (unit in tile.getUnits())
-                    if (gameInfo.civilizations.none { it.civName == unit.owner }) {
+                    if (gameInfo.civilizations.none { it.civID == unit.owner }) {
                         unit.currentTile = tile
                         unit.setTransients(ruleset)
                         unit.removeFromTile()
@@ -173,7 +175,7 @@ object GameStarter {
         }
 
         for (civInfo in gameInfo.civilizations) {
-            if (civInfo.isBarbarian()) continue
+            if (civInfo.isBarbarian) continue
 
             for (tech in ruleset.technologies.values.filter { it.hasUnique(UniqueType.StartingTech) })
                 civInfo.addTechSilently(tech.name)
@@ -195,6 +197,7 @@ object GameStarter {
             val startingEraNumber = ruleset.eras[gameSetupInfo.gameParameters.startingEra]!!.eraNumber
             for (tech in ruleset.technologies.values) {
                 if (ruleset.eras[tech.era()]!!.eraNumber >= startingEraNumber) continue
+                if (civInfo.tech.isUnresearchable(tech)) continue
                 civInfo.addTechSilently(tech.name)
             }
 
@@ -204,7 +207,7 @@ object GameStarter {
     }
 
     private fun addCivPolicies(gameInfo: GameInfo, ruleset: Ruleset) {
-        for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian() }) {
+        for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian }) {
 
             // generic start with policy unique
             for (unique in civInfo.getMatchingUniques(UniqueType.StartsWithPolicy)) {
@@ -228,37 +231,37 @@ object GameStarter {
         val ruleSet = gameInfo.ruleset
         val startingEra = gameInfo.gameParameters.startingEra
         val era = ruleSet.eras[startingEra]!!
-        for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian() && !it.isSpectator() }) {
+        for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian && !it.isSpectator() }) {
             civInfo.addGold((era.startingGold * gameInfo.speed.goldCostModifier).toInt())
             civInfo.policies.addCulture((era.startingCulture * gameInfo.speed.cultureCostModifier).toInt())
         }
     }
 
+    @Readonly
     private fun chooseCivilizations(
         newGameParameters: GameParameters,
         gameInfo: GameInfo,
         ruleset: Ruleset,
         existingMap: Boolean
     ): List<Player> {
-        val chosenPlayers = mutableListOf<Player>()  // Yes this preserves order
-        val dequeCapacity = ruleset.nations.size
-
         val selectedPlayerNames = newGameParameters.players
             .map { it.chosenCiv }.toSet()
-        val randomNationsPool = (
-                if (gameSetupInfo.gameParameters.enableRandomNationsPool)
-                    gameSetupInfo.gameParameters.randomNationsPool.asSequence()
-                else
-                    ruleset.nations.filter { it.value.isMajorCiv }.keys.asSequence()
-                ).filter { it !in selectedPlayerNames }
-            .shuffled().toCollection(ArrayDeque(dequeCapacity))
+        @LocalState val randomNationsPool = (
+            if (gameSetupInfo.gameParameters.enableRandomNationsPool)
+                ruleset.nations.asSequence().map { it.value }
+                    .filter { it.name in gameSetupInfo.gameParameters.randomNationsPool }
+            else
+                ruleset.nations.asSequence().map { it.value }
+                    .filter { it.isMajorCiv && !it.hasUnique(UniqueType.WillNotBeChosenForNewGames) }
+            ).filter { it.name !in selectedPlayerNames }
+            .shuffled().let { ArrayDeque(it.toList()) }
 
         val civNamesWithStartingLocations =
             if (existingMap) gameInfo.tileMap.startingLocationsByNation.keys
             else emptySet()
-        val presetRandomNationsPool = randomNationsPool
-            .filter { it in civNamesWithStartingLocations }
-            .shuffled().toCollection(ArrayDeque(dequeCapacity))
+        @LocalState val presetRandomNationsPool = randomNationsPool
+            .filter { it.name in civNamesWithStartingLocations }
+            .shuffled().let { ArrayDeque(it) }
         randomNationsPool.removeAll(presetRandomNationsPool)
 
         // At this point the civ names in newGameParameters.players, randomNationsPool and presetRandomNationsPool
@@ -289,23 +292,35 @@ object GameStarter {
         }
 
         // Add player entries to the result
-        (
+
+        val chosenPlayers = (
             // Join two Sequences, one the explicitly chosen players...
             newGameParameters.players.asSequence()
-            .filterNot { it in selectedAIToSkip }
-            .sortedWith(compareBy<Player> { it.chosenCiv == Constants.random } // Nonrandom before random
-                .thenBy { it.playerType == PlayerType.AI }) // Human before AI
-            // ...another for the extra random ones
-            + (0 until extraRandomAIPlayers).asSequence().map { Player() }
-        ).mapNotNull {
-            // Resolve random players
-            when {
-                it.chosenCiv != Constants.random -> it
-                presetRandomNationsPool.isNotEmpty() -> Player(presetRandomNationsPool.removeLast(), it.playerType, it.playerId)
-                randomNationsPool.isNotEmpty() -> Player(randomNationsPool.removeLast(), it.playerType, it.playerId)
-                else -> null
-            }
-        }.toCollection(chosenPlayers)
+                .filterNot { it in selectedAIToSkip }
+                .sortedWith(compareBy<Player> { it.chosenCiv == Constants.random } // Nonrandom before random
+                    .thenBy { it.playerType == PlayerType.AI }) // Human before AI
+                // ...another for the extra random ones
+                + (0 until extraRandomAIPlayers).asSequence().map { Player() }
+            ).mapNotNull {
+                // Resolve random players
+                when {
+                    it.chosenCiv != Constants.random -> Player(ruleset.nations[it.chosenCiv]!!, it.playerType, it.playerId)
+                    presetRandomNationsPool.isNotEmpty() -> Player(presetRandomNationsPool.removeLast(), it.playerType, it.playerId)
+                    randomNationsPool.isNotEmpty() -> Player(randomNationsPool.removeLast(), it.playerType, it.playerId)
+                    else -> null
+                }
+            }.toMutableList()
+
+        // ensure Spectators always first players
+        val spectators = chosenPlayers.filter { it.chosenCiv == Constants.spectator }
+        val otherPlayers = chosenPlayers.filterNot { it.chosenCiv == Constants.spectator }.toMutableList()
+        
+        // Shuffle Major Civs
+        if (newGameParameters.shufflePlayerOrder) otherPlayers.shuffle()
+
+        chosenPlayers.clear()
+        chosenPlayers.addAll(spectators)
+        chosenPlayers.addAll(otherPlayers)
 
         // Add CityStates to result - disguised as normal AI, but addCivilizations will detect them
         val numberOfCityStates = if (newGameParameters.randomNumberOfCityStates) {
@@ -317,16 +332,15 @@ object GameStarter {
             newGameParameters.numberOfCityStates
         }
 
-        ruleset.nations.asSequence()
+        chosenPlayers += ruleset.nations.asSequence()
             .filter {
                 it.value.isCityState &&
-                        !it.value.hasUnique(UniqueType.CityStateDeprecated)
-            }.map { it.key }
+                    !it.value.hasUnique(UniqueType.WillNotBeChosenForNewGames)
+            }.map { it.value }
             .shuffled()
-            .sortedByDescending { it in civNamesWithStartingLocations }  // please those with location first
+            .sortedByDescending { it.name in civNamesWithStartingLocations }  // please those with location first
             .take(numberOfCityStates)
             .map { Player(it) }
-            .toCollection(chosenPlayers)
 
         return chosenPlayers
     }
@@ -337,28 +351,28 @@ object GameStarter {
         ruleset: Ruleset,
         chosenPlayers: List<Player>
     ) {
-        if (!newGameParameters.noBarbarians && ruleset.nations.containsKey(Constants.barbarians)) {
-            val barbarianCivilization = Civilization(Constants.barbarians)
-            gameInfo.civilizations.add(barbarianCivilization)
+        if (!newGameParameters.noBarbarians) {
+            val barbs = ruleset.nations[Constants.barbarians]
+            if (barbs != null) {
+                val barbarianCivilization = Civilization(barbs)
+                gameInfo.civilizations.add(barbarianCivilization)
+            }
         }
 
-        val usedCivNames = chosenPlayers.map { it.chosenCiv }.toSet()
-        val (usedMajorCivs, unusedMajorCivs) = ruleset.nations.asSequence()
-            .filter { it.value.isMajorCiv }
-            .map { it.key }
-            .partition { it in usedCivNames }
+        val usedCivNations = chosenPlayers.map { it.chosenNation }.toSet()
+        val usedMajorCivs = ruleset.nations.asSequence()
+            .map { it.value }
+            .filter { it.isMajorCiv && it in usedCivNations }
 
         for (player in chosenPlayers) {
-            val civ = Civilization(player.chosenCiv)
-            when (player.chosenCiv) {
-                in usedMajorCivs, Constants.spectator -> {
-                    civ.playerType = player.playerType
-                    civ.playerId = player.playerId
-                }
-                else ->
-                    if (!civ.cityStateFunctions.initCityState(ruleset, newGameParameters.startingEra, unusedMajorCivs))
-                        continue
+            val civ = Civilization(player.chosenNation)
+            if (civ.isMajorCiv() || civ.isSpectator()) {
+                civ.playerType = player.playerType
+                civ.playerId = player.playerId
+                civ.playerMinutesBeforeForceResign = newGameParameters.minutesUntilForceResign
             }
+            else if (!civ.cityStateFunctions.initCityState(ruleset, newGameParameters.startingEra, usedMajorCivs))
+                continue
             gameInfo.civilizations.add(civ)
         }
     }
@@ -370,7 +384,7 @@ object GameStarter {
 
         val cityCenterMinStats = sequenceOf(ruleSet.tileImprovements[Constants.cityCenter])
             .filterNotNull()
-            .flatMap { it.getMatchingUniques(UniqueType.EnsureMinimumStats, StateForConditionals.IgnoreConditionals) }
+            .flatMap { it.getMatchingUniques(UniqueType.EnsureMinimumStats, GameContext.IgnoreConditionals) }
             .firstOrNull()
             ?.stats ?: Stats.DefaultCityCenterMinimum
 
@@ -378,19 +392,19 @@ object GameStarter {
         for (tile in tileMap.values) {
             startScores[tile] = tile.stats.getTileStartScore(cityCenterMinStats)
         }
-        val allCivs = gameInfo.civilizations.filter { !it.isBarbarian() }
+        val allCivs = gameInfo.civilizations.filter { !it.isBarbarian }
         val landTilesInBigEnoughGroup = getCandidateLand(allCivs.size, tileMap, startScores)
 
         // First we get start locations for the major civs, on the second pass the city states (without predetermined starts) can squeeze in wherever
         val civNamesWithStartingLocations = tileMap.startingLocationsByNation.keys
-        val bestCivs = allCivs.filter { (!it.isCityState() || it.civName in civNamesWithStartingLocations)
+        val bestCivs = allCivs.filter { (!it.isCityState || it.civID in civNamesWithStartingLocations)
             && !it.isSpectator()}
         val bestLocations = getStartingLocations(bestCivs, tileMap, landTilesInBigEnoughGroup, startScores)
         for ((civ, tile) in bestLocations) {
             // A nation can have multiple marked starting locations, of which the first pass may have chosen one
-            tileMap.removeStartingLocations(civ.civName)
+            tileMap.removeStartingLocations(civ.civID)
             // Mark the best start locations so we remember them for the second pass
-            tileMap.addStartingLocation(civ.civName, tile)
+            tileMap.addStartingLocation(civ.civID, tile)
         }
 
         val startingLocations = getStartingLocations(allCivs, tileMap, landTilesInBigEnoughGroup, startScores)
@@ -417,7 +431,7 @@ object GameStarter {
         val startingEra = gameInfo.gameParameters.startingEra
         val settlerLikeUnits = ruleset.units.filter { it.value.isCityFounder() }
 
-        for (civ in gameInfo.civilizations.filter { !it.isBarbarian() && !it.isSpectator() }) {
+        for (civ in gameInfo.civilizations.filter { !it.isBarbarian && !it.isSpectator() }) {
             val startingLocation = startingLocations[civ]!!
 
             removeAncientRuinsNearStartingLocation(startingLocation)
@@ -425,16 +439,23 @@ object GameStarter {
             adjustStartingUnitsForCityStatesAndOneCityChallenge(civ, gameInfo, startingUnits, settlerLikeUnits)
             placeStartingUnits(civ, startingLocation, startingUnits, ruleset, ruleset.eras[startingEra]!!.startingMilitaryUnit, settlerLikeUnits)
 
-            //Trigger any global or nation uniques that should triggered.
-            //We may need the starting location for some uniques, which is why we're doing it now
-            val startingTriggers = (ruleset.globalUniques.uniqueObjects + civ.nation.uniqueObjects)
-            for (unique in startingTriggers.filter { !it.hasTriggerConditional() && it.conditionalsApply(civ) })
-                UniqueTriggerActivation.triggerUnique(unique, civ, tile = startingLocation)
+            // Trigger any global or nation uniques that should triggered.
+            // We may need the starting location for some uniques, which is why we're doing it now
+            // This relies on gameInfo.ruleset already being initialized
+            val startingTriggers = gameInfo.getGlobalUniques().uniqueObjects + civ.nation.uniqueObjects
+            for (unique in startingTriggers) {
+                if (unique.hasTriggerConditional() || !unique.conditionalsApply(civ.state)) continue
+                repeat(unique.getUniqueMultiplier(civ.state)) {
+                    UniqueTriggerActivation.triggerUnique(unique, civ, tile = startingLocation)
+                }
+            }
         }
     }
 
+    @Readonly
     private fun getStartingUnitsForEraAndDifficulty(civ: Civilization, gameInfo: GameInfo, ruleset: Ruleset, startingEra: String): MutableList<String> {
-        val startingUnits = ruleset.eras[startingEra]!!.getStartingUnits(ruleset)
+        @LocalState val startingUnits = ruleset.eras[startingEra]?.getStartingUnits(ruleset)
+            ?: throw Exception("Era $startingEra does not exist in the ruleset!")
 
         // Add extra units granted by difficulty
         startingUnits.addAll(when {
@@ -446,6 +467,7 @@ object GameStarter {
         return startingUnits
     }
 
+    @Readonly
     private fun getEquivalentUnit(
         civ: Civilization,
         unitParam: String,
@@ -482,7 +504,7 @@ object GameStarter {
         settlerLikeUnits: Map<String, BaseUnit>
     ) {
         // Adjust starting units for city states
-        if (civ.isCityState() && !gameInfo.ruleset.modOptions.hasUnique(UniqueType.AllowCityStatesSpawnUnits)) {
+        if (civ.isCityState && !gameInfo.ruleset.modOptions.hasUnique(UniqueType.AllowCityStatesSpawnUnits)) {
             val startingSettlers = startingUnits.filter { settlerLikeUnits.contains(it) }
 
             startingUnits.clear()
@@ -547,11 +569,12 @@ object GameStarter {
         throw Exception("Didn't manage to get starting tiles even with distance of 1?")
     }
 
+    @Readonly
     private fun getCivsOrderedByAvailableLocations(civs: List<Civilization>, tileMap: TileMap): List<Civilization> {
         return civs.shuffled()   // Order should be random since it determines who gets best start
             .sortedBy { civ ->
                 when {
-                    civ.civName in tileMap.startingLocationsByNation -> 1 // harshest requirements
+                    civ.civID in tileMap.startingLocationsByNation -> 1 // harshest requirements
                     civ.nation.startBias.any { it in tileMap.naturalWonders } && !gameSetupInfo.gameParameters.noStartBias -> 2
                     civ.nation.startBias.contains(Constants.tundra) && !gameSetupInfo.gameParameters.noStartBias -> 3    // Tundra starts are hard to find, so let's do them first
                     civ.nation.startBias.isNotEmpty() && !gameSetupInfo.gameParameters.noStartBias -> 4 // less harsh
@@ -560,6 +583,7 @@ object GameStarter {
             }.sortedByDescending { it.isHuman() } // More important for humans to get their start biases!
     }
 
+    @Readonly
     private fun getFreeTiles(tileMap: TileMap, landTilesInBigEnoughGroup: Map<Tile, Float>, minimumDistanceBetweenStartingLocations: Int): MutableList<Tile> {
         return landTilesInBigEnoughGroup.asSequence()
             .filter {
@@ -570,6 +594,7 @@ object GameStarter {
             .toMutableList()
     }
 
+    // Mutating - updates freeTiles
     private fun getStartingLocationsForCivs(
         civsOrderedByAvailableLocations: List<Civilization>,
         tileMap: TileMap,
@@ -586,7 +611,7 @@ object GameStarter {
             startingLocations[civ] = startingLocation
 
             val distanceToNext = minimumDistanceBetweenStartingLocations /
-                (if (civ.isCityState()) 2 else 1) // We allow city states to squeeze in tighter
+                (if (civ.isCityState) 2 else 1) // We allow city states to squeeze in tighter
             freeTiles.removeAll(tileMap.getTilesInDistance(startingLocation.position, distanceToNext)
                 .toSet())
         }
@@ -599,7 +624,7 @@ object GameStarter {
         freeTiles: MutableList<Tile>,
         startScores: HashMap<Tile, Float>,
     ): Tile? {
-        var startingLocation = tileMap.startingLocationsByNation[civ.civName]?.randomOrNull()
+        var startingLocation = tileMap.startingLocationsByNation[civ.civID]?.randomOrNull()
         if (startingLocation == null) {
             startingLocation = tileMap.startingLocationsByNation[Constants.spectator]?.randomOrNull()
             if (startingLocation != null) {
@@ -612,6 +637,7 @@ object GameStarter {
         return startingLocation
     }
 
+    @Readonly
     private fun getOneStartingLocation(
         civ: Civilization,
         tileMap: TileMap,
@@ -637,14 +663,14 @@ object GameStarter {
                     val tileToAvoid = startBias.getPlaceholderParameters()[0]
                     preferredTiles.filter { tile ->
                         !tile.getTilesInDistance(1).any {
-                            it.matchesTerrainFilter(tileToAvoid)
+                            it.matchesTerrainFilter(tileToAvoid, null)
                         }
                     }
                 }
                 startBias in tileMap.naturalWonders -> preferredTiles  // passthrough: already failed
                 else -> preferredTiles.filter { tile ->
                     tile.getTilesInDistance(1).any {
-                        it.matchesTerrainFilter(startBias)
+                        it.matchesTerrainFilter(startBias, null)
                     }
                 }
             }
