@@ -2,32 +2,37 @@ package com.unciv.logic.automation
 
 import com.unciv.logic.city.City
 import com.unciv.logic.city.CityFocus
+import com.unciv.logic.city.CityStats
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.BFS
 import com.unciv.logic.map.TileMap
-import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.INonPerpetualConstruction
-import com.unciv.models.ruleset.Victory
-import com.unciv.models.ruleset.nation.PersonalityValue
+import com.unciv.models.ruleset.PerpetualConstruction
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.unique.LocalUniqueCache
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.ui.screens.victoryscreen.RankingType
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
+import kotlin.math.min
+
 
 object Automation {
 
-    fun rankTileForCityWork(tile: Tile, city: City, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Float {
+    @Readonly
+    fun rankTileForCityWork(tile: Tile, city: City, localUniqueCache: LocalUniqueCache): Float {
         val stats = tile.stats.getTileStats(city, city.civ, localUniqueCache)
         return rankStatsForCityWork(stats, city, false, localUniqueCache)
     }
 
+    @Readonly
     fun rankSpecialist(specialist: String, city: City, localUniqueCache: LocalUniqueCache): Float {
         val stats = city.cityStats.getStatsOfSpecialist(specialist, localUniqueCache)
         var rank = rankStatsForCityWork(stats, city, true, localUniqueCache)
@@ -38,16 +43,40 @@ object Automation {
             gpp = specialistInfo.greatPersonPoints.sumValues().toFloat()
         }
         gpp = gpp * (100 + city.currentGPPBonus) / 100
-        rank += gpp * 3 // GPP weight
+        rank += gpp // GPP weight
         return rank
     }
 
+    // More complicated logic to properly weigh Food vs other Stats (esp Production)
+    @Readonly
+    private fun getFoodModWeight(city: City, surplusFood: Float): Float {
+        val speed = city.civ.gameInfo.speed.modifier
+        // Zero out Growth if close to Unhappiness limit
+        if (city.civ.getHappiness() < -8)
+            return 0f
+        if (city.civ.isAI()) {
+            // Value 1 Growth at less than 2 Production, but above 1 Production + 1 Gold
+            return 1.5f
+        }
+        // Human weights. May be different since AI Happiness is always "easier"
+        // Only apply these for Default to not interfere with Focus weights
+        if (city.getCityFocus() == CityFocus.NoFocus) {
+            if (city.population.population < 5)
+                return 2f
+            if (surplusFood > city.population.getFoodToNextPopulation() / (10 * speed))
+                return 0.75f // get Growth just under Production
+        }
 
+        return 1f
+    }
+
+    @Readonly
     fun rankStatsForCityWork(stats: Stats, city: City, areWeRankingSpecialist: Boolean, localUniqueCache: LocalUniqueCache): Float {
         val cityAIFocus = city.getCityFocus()
-        val yieldStats = stats.clone()
-        val civPersonality = city.civ.getPersonality()
+        @LocalState val yieldStats = stats.clone()
         val cityStatsObj = city.cityStats
+        val civInfo = city.civ
+        val allTechsAreResearched = civInfo.tech.allTechsAreResearched()
 
         if (areWeRankingSpecialist) {
             // If you have the Food Bonus, count as 1 extra food production (base is 2food)
@@ -56,115 +85,133 @@ object Automation {
                     yieldStats.food -= (unique.params[0].toFloat() / 100f) * 2f // base 2 food per Pop
             // Specialist Happiness Percentage Change 0f-1f
             for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.UnhappinessFromPopulationTypePercentageChange))
-                if (city.matchesFilter(unique.params[2]) && unique.params[1] == "Specialists")
+                if (unique.params[1] == "Specialists" && city.matchesFilter(unique.params[2]))
                     yieldStats.happiness -= (unique.params[0].toFloat() / 100f)  // relative val is negative, make positive
+            if (yieldStats.science == 3f || yieldStats.science >= 5f )
+                yieldStats.science *= 1.3f
         }
 
         val surplusFood = city.cityStats.currentCityStats[Stat.Food]
+        val starving = surplusFood < 0
         // If current Production converts Food into Production, then calculate increased Production Yield
         if (cityStatsObj.canConvertFoodToProduction(surplusFood, city.cityConstructions.getCurrentConstruction())) {
             // calculate delta increase of food->prod. This isn't linear
             yieldStats.production += cityStatsObj.getProductionFromExcessiveFood(surplusFood+yieldStats.food) - cityStatsObj.getProductionFromExcessiveFood(surplusFood)
             yieldStats.food = 0f  // all food goes to 0
         }
+
+        // Split Food Yield into feedFood, amount needed to not Starve
+        // and growthFood, any amount above that
+        var feedFood = 0f
+        if (starving)
+            feedFood = min(yieldStats.food, -surplusFood).coerceAtLeast(0f)
+        var growthFood = yieldStats.food - feedFood // how much extra Food we yield
+        // Avoid Growth, only count Food that gets you not-starving, but no more
+        if (city.avoidGrowth) {
+            growthFood = 0f
+        }
+        yieldStats.food = 1f
+        
         // Apply base weights
         yieldStats.applyRankingWeights()
 
-        if (surplusFood > 0 && city.avoidGrowth) {
-            yieldStats.food = 0f // don't need more food!
-        } else if (cityAIFocus in CityFocus.zeroFoodFocuses()) {
-            // Focus on non-food/growth
-            if (surplusFood < 0)
-                yieldStats.food *= 8 // Starving, need Food, get to 0
-            else
-                yieldStats.food /= 2
-        } else if (!city.avoidGrowth) {
-            // NoFocus or Food/Growth Focus. Target +2 Food Surplus
-            if (surplusFood < 2)
-                yieldStats.food *= 8
-            else if (city.population.population < 5)
-                yieldStats.food *= 3
+        val foodBaseWeight = yieldStats.food
+
+        // If starving, need Food, so feedFood > 0
+        // scale feedFood by 14(base weight)*8(super important)
+        // By only scaling what we need to reach Not Starving by x8, we can pick a tile that gives
+        // exactly as much Food as we need to Not Starve that also has other good yields instead of
+        // always picking the Highest Food tile until Not Starving
+        yieldStats.food = feedFood * (foodBaseWeight * 8)
+        // growthFood is any additional food not required to meet Starvation
+
+        // Growth is penalized when Unhappy, see GlobalUniques.json
+        // No Growth if <-10, 1/4 if <0
+        // Reusing food growth code from CityStats.updateFinalStatList()
+        val growthNullifyingUnique =
+            localUniqueCache.forCityGetMatchingUniques(city, UniqueType.NullifiesGrowth).firstOrNull()
+        if (growthNullifyingUnique == null) { // if not nullified
+            var newGrowthFood = growthFood  // running count of growthFood
+            val cityStats = CityStats(city)
+            val growthBonuses = cityStats.getGrowthBonus(growthFood, localUniqueCache)
+            for (growthBonus in growthBonuses) {
+                newGrowthFood += growthBonus.value.food
+            }
+            if (city.isWeLoveTheKingDayActive() && city.civ.getHappiness() >= 0) {
+                newGrowthFood += growthFood / 4
+            }
+            newGrowthFood = newGrowthFood.coerceAtLeast(0f) // floor to 0 for safety
+            
+            yieldStats.food += newGrowthFood * foodBaseWeight * getFoodModWeight(city, surplusFood)
         }
 
-        if (city.population.population < 5) {
+        if (city.population.population < 10) {
             // "small city" - we care more about food and less about global problems like gold science and culture
-            // Food already handled above. Science/Culture have low weights in Stats already
-            yieldStats.gold /= 2 // it's barely worth anything at this point
-        } else {
-            if (city.civ.gold < 0 && city.civ.stats.statsForNextTurn.gold <= 0)
-                yieldStats.gold *= 2 // We have a global problem
-
-            if (city.tiles.size < 12)
-                yieldStats.culture *= 2
-
-            if (city.civ.getHappiness() < 0)
-                yieldStats.happiness *= 2
+            // Food already handled above. Gold/Culture have low weights in Stats already
+            yieldStats.science /= 2
         }
 
-        for (stat in Stat.values()) {
-            if (city.civ.wantsToFocusOn(stat))
-                yieldStats[stat] *= 2f
+        if (city.civ.stats.statsForNextTurn.gold < 0) {
+            // We have a global problem, we need to deal with it before it leads to science loss
+            yieldStats.gold *= 2
+        }
 
-            yieldStats[stat] *= civPersonality.scaledFocus(PersonalityValue[stat])
+        if (city.civ.getHappiness() < 0) {
+            yieldStats.happiness *= 2
+        }
+
+        if (allTechsAreResearched) {
+            // Science is useless at this point
+            yieldStats.science *= 0
+        }
+
+        if (city.cityConstructions.getCurrentConstruction() is PerpetualConstruction) {
+            // With 4:1 conversion of production to science, production is overvalued by a factor (12*4)/7 = 6.9
+            yieldStats.production /= 6
         }
 
         // Apply City focus
-        cityAIFocus.applyWeightTo(yieldStats)
+        for (stat in cityAIFocus.statValuesForFocus) {
+            val currentStat = yieldStats[stat]
+            if (currentStat == 0f) continue
+            val statMultiplier = cityAIFocus.getStatMultiplier(stat)
+            yieldStats[stat] = currentStat * statMultiplier
+        }
 
         return yieldStats.values.sum()
     }
 
     fun tryTrainMilitaryUnit(city: City) {
         if (city.isPuppet) return
-        if ((city.cityConstructions.getCurrentConstruction() as? BaseUnit)?.isMilitary() == true)
+        if ((city.cityConstructions.getCurrentConstruction() as? BaseUnit)?.isMilitary == true)
             return // already training a military unit
-        val chosenUnitName = chooseMilitaryUnit(city, city.civ.gameInfo.ruleset.units.values.asSequence())
-        if (chosenUnitName != null)
-            city.cityConstructions.currentConstructionFromQueue = chosenUnitName
+        val chosenUnit = chooseMilitaryUnit(city, city.civ.gameInfo.ruleset.units.values.asSequence())
+        if (chosenUnit != null)
+            city.cityConstructions.setCurrentConstruction(chosenUnit.name)
     }
 
-    @Suppress("MemberVisibilityCanBePrivate")
-    fun providesUnneededCarryingSlots(baseUnit: BaseUnit, civInfo: Civilization): Boolean {
-        // Simplified, will not work for crazy mods with more than one carrying filter for a unit
-        val carryUnique = baseUnit.getMatchingUniques(UniqueType.CarryAirUnits).first()
-        val carryFilter = carryUnique.params[1]
-
-        fun getCarryAmount(mapUnit: MapUnit): Int {
-            val mapUnitCarryUnique =
-                mapUnit.getMatchingUniques(UniqueType.CarryAirUnits).firstOrNull() ?: return 0
-            if (mapUnitCarryUnique.params[1] != carryFilter) return 0 //Carries a different type of unit
-            return mapUnitCarryUnique.params[0].toInt() +
-                    mapUnit.getMatchingUniques(UniqueType.CarryExtraAirUnits)
-                        .filter { it.params[1] == carryFilter }.sumOf { it.params[0].toInt() }
-        }
-
-        val totalCarriableUnits =
-            civInfo.units.getCivUnits().count { it.matchesFilter(carryFilter) }
-        val totalCarryingSlots = civInfo.units.getCivUnits().sumOf { getCarryAmount(it) }
-        return totalCarriableUnits < totalCarryingSlots
-    }
-
-    fun chooseMilitaryUnit(city: City, availableUnits: Sequence<BaseUnit>): String? {
+    @Readonly
+    fun chooseMilitaryUnit(city: City, availableUnits: Sequence<BaseUnit>): BaseUnit? {
         val currentChoice = city.cityConstructions.getCurrentConstruction()
-        if (currentChoice is BaseUnit && !currentChoice.isCivilian()) return city.cityConstructions.currentConstructionFromQueue
+        if (currentChoice is BaseUnit && !currentChoice.isCivilian()) return currentChoice
 
         // if not coastal, removeShips == true so don't even consider ships
         var removeShips = true
         var isMissingNavalUnitsForCityDefence = false
 
-        fun isNavalMeleeUnit(unit: BaseUnit) = unit.isMelee() && unit.type.isWaterUnit()
+        @Readonly fun isNavalMeleeUnit(unit: BaseUnit) = unit.isMelee() && unit.type.isWaterUnit()
         if (city.isCoastal()) {
             // in the future this could be simplified by assigning every distinct non-lake body of
             // water their own ID like a continent ID
             val findWaterConnectedCitiesAndEnemies =
-                    BFS(city.getCenterTile()) { it.isWater || it.isCityCenter() }
+                    BFS(city.getCenterTile()) { it.isWater && !it.isImpassible() || it.isCityCenter() }
             findWaterConnectedCitiesAndEnemies.stepToEnd()
 
             val numberOfOurConnectedCities = findWaterConnectedCitiesAndEnemies.getReachedTiles()
                 .count { it.isCityCenter() && it.getOwner() == city.civ }
-            val numberOfOurNavalMeleeUnits = findWaterConnectedCitiesAndEnemies.getReachedTiles().asSequence()
-                .flatMap { it.getUnits() }
-                .count { isNavalMeleeUnit(it.baseUnit) }
+            val numberOfOurNavalMeleeUnits = findWaterConnectedCitiesAndEnemies.getReachedTiles()
+                .sumOf { it.getUnits().count { isNavalMeleeUnit(it.baseUnit) } }
+                
             isMissingNavalUnitsForCityDefence = numberOfOurConnectedCities > numberOfOurNavalMeleeUnits
 
             removeShips = findWaterConnectedCitiesAndEnemies.getReachedTiles().none {
@@ -174,17 +221,20 @@ object Automation {
         }
 
         val militaryUnits = availableUnits
-            .filter { it.isMilitary() }
-            .filterNot { removeShips && it.isWaterUnit() }
+            .filter { it.isMilitary }
+            .filterNot { removeShips && it.isWaterUnit }
             .filter { allowSpendingResource(city.civ, it) }
             .filterNot {
-                // filter out carrier-type units that can't attack if we don't need them
-                (it.hasUnique(UniqueType.CarryAirUnits) && it.hasUnique(UniqueType.CannotAttack))
-                        && providesUnneededCarryingSlots(it, city.civ)
+                // units that can't attack may be very strong meatshields, but current AI
+                // has trouble using them properly, and this conveniently filters out aircraft carriers
+                // (they're kinda useless compared to investing in battleships or extended-range bombers,
+                // and are obsoleted at Stealth. Excluding units based on their carrying slots being
+                // not needed filters out nuclear submarines and missile cruisers, which is not correct)
+                it.hasUnique(UniqueType.CannotAttack)
             }
             // Only now do we filter out the constructable units because that's a heavier check
             .filter { it.isBuildable(city.cityConstructions) }
-            .toList()
+            .toList().asSequence()
 
         val chosenUnit: BaseUnit
         if (!city.civ.isAtWar()
@@ -211,12 +261,12 @@ object Automation {
             val bestForce = bestUnitsForType.maxOfOrNull { it.value.getForceEvaluation() } ?: return null
             chosenUnit = bestUnitsForType.filterValues { it.uniqueTo != null || it.getForceEvaluation() > bestForce / 3 }.values.random()
         }
-        return chosenUnit.name
+        return chosenUnit
     }
 
     /** Determines whether [civInfo] should be allocating military to fending off barbarians */
     fun afraidOfBarbarians(civInfo: Civilization): Boolean {
-        if (civInfo.isCityState() || civInfo.isBarbarian())
+        if (civInfo.isCityState || civInfo.isBarbarian)
             return false
 
         if (civInfo.gameInfo.gameParameters.noBarbarians)
@@ -233,8 +283,8 @@ object Automation {
         if (civInfo.gameInfo.turns > 120 * speed.barbarianModifier * multiplier)
             multiplier /= 2
 
-        // If we have a lot of, or no cities we are not afraid
-        if (civInfo.cities.isEmpty() || civInfo.cities.size >= 4 * multiplier)
+        // If we have no cities or a lot of units we are not afraid
+        if (civInfo.cities.isEmpty() || civInfo.units.getCivUnits().count() >= 4 * multiplier)
             return false
 
         // If we have vision of our entire starting continent (ish) we are not afraid
@@ -248,39 +298,23 @@ object Automation {
         return true
     }
 
-    /** Checks both feasibility of Buildings with a CreatesOneImprovement unique
-     *  and resource scarcity making a construction undesirable.
-     */
+    /** Checks resource scarcity making a construction undesirable. */
+    @Readonly
     fun allowAutomatedConstruction(
         civInfo: Civilization,
         city: City,
         construction: INonPerpetualConstruction
     ): Boolean {
-        return allowCreateImprovementBuildings(civInfo, city, construction)
-            && allowSpendingResource(civInfo, construction, city)
+        return allowSpendingResource(civInfo, construction, city)
     }
 
-    @Suppress("MemberVisibilityCanBePrivate")
-    /** Checks both feasibility of Buildings with a [UniqueType.CreatesOneImprovement] unique (appropriate tile available).
-     *  Constructions without pass uncontested. */
-    fun allowCreateImprovementBuildings(
-        civInfo: Civilization,
-        city: City,
-        construction: INonPerpetualConstruction
-    ): Boolean {
-        if (construction !is Building) return true
-        if (!construction.hasCreateOneImprovementUnique()) return true  // redundant but faster???
-        val improvement = construction.getImprovementToCreate(city.getRuleset(), civInfo) ?: return true
-        return city.getTiles().any {
-            it.improvementFunctions.canBuildImprovement(improvement, civInfo)
-        }
-    }
 
     /** Determines whether the AI should be willing to spend strategic resources to build
      *  [construction] for [civInfo], assumes that we are actually able to do so. */
+    @Readonly
     fun allowSpendingResource(civInfo: Civilization, construction: INonPerpetualConstruction, cityInfo: City? = null): Boolean {
         // City states do whatever they want
-        if (civInfo.isCityState())
+        if (civInfo.isCityState)
             return true
 
         // Spaceships are always allowed
@@ -288,8 +322,8 @@ object Automation {
             return true
 
         val requiredResources = if (construction is BaseUnit)
-            construction.getResourceRequirementsPerTurn(StateForConditionals(civInfo))
-        else construction.getResourceRequirementsPerTurn(StateForConditionals(civInfo, cityInfo))
+            construction.getResourceRequirementsPerTurn(civInfo.state)
+        else construction.getResourceRequirementsPerTurn(cityInfo?.state ?: civInfo.state)
         // Does it even require any resources?
         if (requiredResources.isEmpty())
             return true
@@ -307,16 +341,13 @@ object Automation {
             for (city in civInfo.cities) {
                 val otherConstruction = city.cityConstructions.getCurrentConstruction()
                 if (otherConstruction is Building)
-                    futureForBuildings += otherConstruction.getResourceRequirementsPerTurn(
-                        StateForConditionals(civInfo, city))[resource]
+                    futureForBuildings += otherConstruction.getResourceRequirementsPerTurn(city.state)[resource]
                 else
-                    futureForUnits += otherConstruction.getResourceRequirementsPerTurn(
-                        StateForConditionals(civInfo))[resource]
+                    futureForUnits += otherConstruction.getResourceRequirementsPerTurn(civInfo.state)[resource]
             }
 
             // Make sure we have some for space
-            if (resource in civInfo.gameInfo.spaceResources && civResources[resource]!! - amount - futureForBuildings - futureForUnits
-                < getReservedSpaceResourceAmount(civInfo)) {
+            if (resource in civInfo.gameInfo.spaceResources && civResources[resource]!! - amount - futureForBuildings - futureForUnits < 2) {
                 return false
             }
 
@@ -349,10 +380,7 @@ object Automation {
         return true
     }
 
-    fun getReservedSpaceResourceAmount(civInfo: Civilization): Int {
-        return if (civInfo.wantsToFocusOn(Victory.Focus.Science)) 3 else 2
-    }
-
+    @Readonly
     fun threatAssessment(assessor: Civilization, assessed: Civilization): ThreatLevel {
         val powerLevelComparison =
             assessed.getStatForRanking(RankingType.Force) / assessor.getStatForRanking(RankingType.Force).toFloat()
@@ -364,18 +392,31 @@ object Automation {
             else -> ThreatLevel.Medium
         }
     }
+    @Readonly
+    private fun improvementIsRemovable(city: City, tile: Tile): Boolean {
+        val gameContext = GameContext(city.civ, city, tile = tile)
+        return (tile.getTileImprovement()?.hasUnique(UniqueType.AutomatedUnitsWillNotReplace, gameContext) == false  && tile.getTileImprovement()?.hasUnique(UniqueType.Irremovable, gameContext) == false)
+    }
 
     /** Support [UniqueType.CreatesOneImprovement] unique - find best tile for placement automation */
+    @Readonly
     fun getTileForConstructionImprovement(city: City, improvement: TileImprovement): Tile? {
         val localUniqueCache = LocalUniqueCache()
+        val civ = city.civ
         return city.getTiles().filter {
-            it.improvementFunctions.canBuildImprovement(improvement, city.civ)
-        }.maxByOrNull {
-            rankTileForCityWork(it, city, localUniqueCache)
+            (it.getTileImprovement() == null || improvementIsRemovable(city, it))
+                && it.improvementFunctions.canBuildImprovement(improvement, city.state)
+        }.maxByOrNull { 
+            // Needs to take into account future improvement layouts, and better placement of citadel-like improvements
+            rankStatsValue(it.stats.getStatDiffForImprovement(improvement, civ, city, localUniqueCache, it.stats.getTileStats(city, civ, localUniqueCache)), civ) + (
+                if (improvement.hasUnique(UniqueType.DefensiveBonus)) { 
+                    it.aerialDistanceTo(city.getCenterTile()) + it.getDefensiveBonus(false) } 
+                else 0).toFloat()
         }
     }
 
     // Ranks a tile for any purpose except the expansion algorithm of cities
+    @Readonly
     internal fun rankTile(tile: Tile?, civInfo: Civilization,
                           localUniqueCache: LocalUniqueCache): Float {
         if (tile == null) return 0f
@@ -385,8 +426,8 @@ object Automation {
         var rank = rankStatsValue(stats, civInfo)
         if (tile.improvement == null) rank += 0.5f // improvement potential!
         if (tile.isPillaged()) rank += 0.6f
-        if (tile.hasViewableResource(civInfo)) {
-            val resource = tile.tileResource
+        val resource = tile.tileResource
+        if (civInfo.canSeeResource(resource)) {
             if (resource.resourceType != ResourceType.Bonus) rank += 1f // for usage
             if (tile.improvement == null) rank += 1f // improvement potential - resources give lots when improved!
             if (tile.isPillaged()) rank += 1.1f // even better, repair is faster
@@ -395,6 +436,7 @@ object Automation {
     }
 
     // Ranks a tile for the expansion algorithm of cities
+    @Readonly
     internal fun rankTileForExpansion(tile: Tile, city: City,
                                       localUniqueCache: LocalUniqueCache): Int {
         // https://github.com/Gedemon/Civ5-DLL/blob/aa29e80751f541ae04858b6d2a2c7dcca454201e/CvGameCoreDLL_Expansion1/CvCity.cpp#L10301
@@ -409,14 +451,15 @@ object Automation {
         var score = distance * 100
 
         // Resources are good: less points
-        if (tile.hasViewableResource(city.civ)) {
-            if (tile.tileResource.resourceType != ResourceType.Bonus) score -= 105
-            else if (distance <= 3) score -= 104
+        val resource = tile.tileResource
+        if (city.civ.canSeeResource(resource)) {
+            if (resource.resourceType != ResourceType.Bonus) score -= 105
+            else if (distance <= city.getWorkRange()) score -= 104
         } else {
-            // Water tiles without resources aren't great
-            if (tile.isWater) score += 25
+            // Water tiles without resources aren't great unless they're Atolls
+            if (tile.isWater) score += 3
             // Can't work it anyways
-            if (distance > 3) score += 100
+            if (distance > city.getWorkRange()) score += 100
         }
 
         if (tile.naturalWonder != null) score -= 105
@@ -429,13 +472,12 @@ object Automation {
 
         for (adjacentTile in tile.neighbors.filter { it.getOwner() == null }) {
             val adjacentDistance = city.getCenterTile().aerialDistanceTo(adjacentTile)
-            if (adjacentTile.hasViewableResource(city.civ) &&
-                (adjacentDistance < 3 ||
-                    adjacentTile.tileResource.resourceType != ResourceType.Bonus
-                )
+            val adjacentResource = adjacentTile.tileResource
+            if (city.civ.canSeeResource(adjacentResource) &&
+                (adjacentDistance < city.getWorkRange() || adjacentResource.resourceType != ResourceType.Bonus)
             ) score -= 1
             if (adjacentTile.naturalWonder != null) {
-                if (adjacentDistance < 3) adjacentNaturalWonder = true
+                if (adjacentDistance < city.getWorkRange()) adjacentNaturalWonder = true
                 score -= 1
             }
         }
@@ -448,18 +490,19 @@ object Automation {
         return score
     }
 
+    @Readonly
     fun rankStatsValue(stats: Stats, civInfo: Civilization): Float {
         var rank = 0.0f
-        rank += if (stats.food <= 2)
-                    (stats.food * 1.2f) //food get more value to keep city growing
-                else
-                    (2.4f + (stats.food - 2) / 2) // 1.2 point for each food up to 2, from there on half a point
+        rank += stats.food * 1.2f //food get more value to keep city growing
 
         rank += if (civInfo.gold < 0 && civInfo.stats.statsForNextTurn.gold <= 0)
-                    stats.gold
+                    stats.gold //build more gold infrastructure if in serious gold problems
+        // This could lead to oscilliatory behaviour however: gold problem -> build trade post -> no gold problem -> replace trade posts -> gold problem
                 else
-                    stats.gold / 3 // 3 gold is much worse than 2 production
-
+                    stats.gold / 3 // Gold is valued less than is the case for citizen assignment,
+        //otherwise the AI would replace tiles with trade posts upon entering a golden age,
+        //and replace the trade post again when the golden age ends.
+        // We need a way to take golden age gold into account before the GA actually takes place
         rank += stats.happiness
         rank += stats.production
         rank += stats.science
