@@ -8,10 +8,10 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.BeliefType
-import com.unciv.models.ruleset.Victory
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
+import yairm210.purity.annotations.Readonly
 import kotlin.math.min
 import kotlin.math.pow
 import kotlin.random.Random
@@ -30,15 +30,15 @@ object ReligionAutomation {
             buyGreatProphetInAnyCity(civInfo)
             return
         }
-
-        // We don't have a religion and no more change of getting it :(
-        if (civInfo.religionManager.religionState <= ReligionState.Pantheon) {
+        
+        if (civInfo.religionManager.remainingFoundableReligions() == 0 ) {
+            buyGreatPerson(civInfo)
             tryBuyAnyReligiousBuilding(civInfo)
-            // Todo: buy Great People post industrial era
             return
         }
 
         // If we don't have majority in all our own cities, build missionaries and inquisitors to solve this
+        //TODO: in late game, this may not be worth it
         val citiesWithoutOurReligion = civInfo.cities.filter { it.religion.getMajorityReligion() != civInfo.religionManager.religion!! }
         // The original had a cap at 4 missionaries total, but 1/4 * the number of cities should be more appropriate imo
         if (citiesWithoutOurReligion.count() >
@@ -55,7 +55,6 @@ object ReligionAutomation {
             return
         }
 
-
         // Get an inquisitor to defend our holy city
         val holyCity = civInfo.religionManager.getHolyCity()
         if (holyCity != null
@@ -66,25 +65,13 @@ object ReligionAutomation {
             buyInquisitorNear(civInfo, holyCity)
             return
         }
-
-        // Buy religious buildings in cities if possible
-        val citiesWithMissingReligiousBuildings = civInfo.cities.filter { city ->
-            city.religion.getMajorityReligion() != null
-            && !city.cityConstructions.isAllBuilt(city.religion.getMajorityReligion()!!.buildingsPurchasableByBeliefs)
-        }
-        if (citiesWithMissingReligiousBuildings.any()) {
-            tryBuyAnyReligiousBuilding(civInfo)
-            return
-        }
-
-        // Todo: buy Great People post industrial era
-
+        
         // Just buy missionaries to spread our religion outside of our civ
         if (civInfo.units.getCivUnits().count { it.hasUnique(UniqueType.CanSpreadReligion) } < 4) {
             buyMissionaryInAnyCity(civInfo)
             return
         }
-        // Todo: buy inquisitors for defence of other cities
+        // Todo: declare war if enemy missionaries enter our civ without permission
     }
 
     private fun tryBuyAnyReligiousBuilding(civInfo: Civilization) {
@@ -193,20 +180,43 @@ object ReligionAutomation {
         cityToBuy.cityConstructions.purchaseConstruction(inquisitorConstruction, -1, true, Stat.Faith)
     }
 
+    private fun buyGreatPerson(civInfo: Civilization) {
+        val greatPersonUnit = civInfo.gameInfo.ruleset.units.values.filter {
+            it.hasUnique(UniqueType.GreatPerson) && !it.hasUnique(UniqueType.MayFoundReligion) //we want to exclude great prophets from the list
+        }
+        val greatPersonConstruction = greatPersonUnit
+            // Get list of cities it can be built in
+            .associateBy({unit -> unit}) { unit -> civInfo.cities.filter { unit.isPurchasable(it.cityConstructions) && unit.canBePurchasedWithStat(it, Stat.Faith) } }
+            .filter { it.value.isNotEmpty() }
+            // And from that list determine the cheapest price
+            .minByOrNull { it.value.minOf { city -> it.key.getStatBuyCost(city, Stat.Faith)!!  }}?.key
+            ?: return
+
+        val validCitiesToBuy = civInfo.cities.filter {
+            (greatPersonConstruction.getStatBuyCost(it, Stat.Faith) ?: return@filter false) <= civInfo.religionManager.storedFaith
+        }
+        
+        if (validCitiesToBuy.isEmpty()) return
+
+        val cityToBuy = validCitiesToBuy.first()
+
+        cityToBuy.cityConstructions.purchaseConstruction(greatPersonConstruction, -1, true, Stat.Faith)
+    }
+
     // endregion
 
     // region rate beliefs
-
+    @Readonly
     fun rateBelief(civInfo: Civilization, belief: Belief): Float {
-        var score = 0f
+        var score = 0f // Roughly equivalent to the sum of stats gained across all cities
 
         for (city in civInfo.cities) {
-            for (tile in city.getCenterTile().getTilesInDistance(3)) {
+            for (tile in city.getCenterTile().getTilesInDistance(city.getWorkRange())) {
                 val tileScore = beliefBonusForTile(belief, tile, city)
                 score += tileScore * when {
-                    city.workedTiles.contains(tile.position) -> 8
-                    tile.getCity() == city -> 5
-                    else -> 3
+                    city.workedTiles.contains(tile.position) -> 1f // worked
+                    tile.getCity() == city -> 0.7f // workable
+                    else -> 0.5f // unavailable - for now
                 } * (Random.nextFloat() * 0.05f + 0.975f)
             }
 
@@ -219,39 +229,54 @@ object ReligionAutomation {
 
         if (belief.type == BeliefType.Pantheon)
             score *= 0.9f
+        
+        score *= belief.getWeightForAiDecision(GameContext(civInfo))
 
         return score
     }
 
+    @Readonly
     private fun beliefBonusForTile(belief: Belief, tile: Tile, city: City): Float {
         var bonusYield = 0f
         for (unique in belief.uniqueObjects) {
             when (unique.type) {
-                UniqueType.StatsFromObject -> if (tile.matchesFilter(unique.params[1]))
-                    bonusYield += unique.stats.values.sum()
-                UniqueType.StatsFromTilesWithout ->
-                    if (city.matchesFilter(unique.params[3])
-                        && tile.matchesFilter(unique.params[1])
-                        && !tile.matchesFilter(unique.params[2])
+                UniqueType.StatsFromObject -> {
+                    val resource = tile.tileResource
+                    if (tile.matchesFilter(unique.params[1])) {
+                        if (!tile.lastTerrain.hasUnique(UniqueType.ProductionBonusWhenRemoved) ||
+                            !tile.lastTerrain.matchesFilter(unique.params[1]) //forest pantheons are bad, as we want to remove the forests
+                        ) bonusYield += unique.stats.values.sum()
+                        else if (resource != null && (resource.matchesFilter(unique.params[1]) ||
+                                resource.isImprovedBy(unique.params[1]))
+                        ) bonusYield += unique.stats.values.sum() //resource pantheons are good, as we want to work the tile anyways
+                    } else if (resource != null && (resource.matchesFilter(unique.params[1]) ||
+                            resource.isImprovedBy(unique.params[1]))
+                    ) bonusYield += unique.stats.values.sum() //resource pantheons are good, as we want to work the tile anyways
+                }
+                UniqueType.StatsFromTilesWithout -> {
+                    if (city.matchesFilter(unique.params[3]) &&
+                        tile.matchesFilter(unique.params[1]) &&
+                        !tile.matchesFilter(unique.params[2])
                     ) bonusYield += unique.stats.values.sum()
-                // ToDo: Also calculate add stats for improvements that will be buildable
+                }
                 else -> {}
             }
         }
         return bonusYield
     }
 
+    @Readonly
     private fun beliefBonusForCity(civInfo: Civilization, belief: Belief, city: City): Float {
         var score = 0f
         val ruleSet = civInfo.gameInfo.ruleset
         for (unique in belief.uniqueObjects) {
-            val modifier = 0.5f.pow(unique.conditionals.size)
+            val modifier = 0.5f.pow(unique.modifiers.size)
             // Multiply by 3/10 if has an obsoleted era
             // Multiply by 2 if enough pop/followers (best implemented with conditionals, so left open for now)
             // If obsoleted, continue
             score += modifier * when (unique.type) {
                 UniqueType.GrowthPercentBonus -> unique.params[0].toFloat() / 3f
-                UniqueType.BorderGrowthPercentage -> -unique.params[0].toFloat() * 2f / 10f
+                UniqueType.BorderGrowthPercentage -> -unique.params[0].toFloat() / 10f
                 UniqueType.StrengthForCities -> unique.params[0].toFloat() / 10f // Modified by personality
                 UniqueType.CityHealingUnits -> unique.params[1].toFloat() / 10f
                 UniqueType.PercentProductionBuildings -> unique.params[0].toFloat() / 3f
@@ -264,27 +289,27 @@ object ReligionAutomation {
                 UniqueType.StatsFromObject ->
                     when {
                         ruleSet.buildings.containsKey(unique.params[1]) -> {
-                            unique.stats.values.sum() /
-                                if (ruleSet.buildings[unique.params[1]]!!.isWonder) 2f
-                                else 1f
+                            unique.stats.values.sum() *
+                                if (ruleSet.buildings[unique.params[1]]!!.isNationalWonder) 0.25f //there's at most 1 copy of each of these in our empire, and the AI is slow at getting it
+                                else 1f // Yields from regular buildings won't need the upfront purchase cost as is the case with religion buildings, but they may have weird requirements (gardens etc.)
 
                         }
                         ruleSet.specialists.containsKey(unique.params[1]) -> {
                             unique.stats.values.sum() *
-                                if (city.population.population > 8f) 3f
+                                if (city.population.population > 8f) 2f
                                 else 1f
                         }
-                        else -> 0f
+                        else -> unique.stats.values.sum() * 0f //yields from world wonders and great improvements - the latter needs additional AI logic to be used correctly
                     }
                 UniqueType.StatsFromTradeRoute ->
                     unique.stats.values.sum() *
-                        if (city.isConnectedToCapital()) 2f
-                        else 1f
+                        if (city.isConnectedToCapital()) 1f
+                        else 0f //no yields from the belief yet, also for pantheons it's quite low-tempo
                 UniqueType.StatPercentFromReligionFollowers ->
                     min(unique.params[0].toFloat() * city.population.population, unique.params[2].toFloat())
                 UniqueType.StatsPerCity ->
                     if (city.matchesFilter(unique.params[1]))
-                        unique.stats.values.sum()
+                        unique.stats.values.sum() * 1f //free yields
                     else 0f
                 else -> 0f
             }
@@ -293,6 +318,7 @@ object ReligionAutomation {
         return score
     }
 
+    @Readonly
     private fun beliefBonusForPlayer(civInfo: Civilization, belief: Belief): Float {
         var score = 0f
         val numberOfFoundedReligions = civInfo.gameInfo.civilizations.count {
@@ -329,57 +355,48 @@ object ReligionAutomation {
 
         for (unique in belief.uniqueObjects) {
             val modifier =
-                if (unique.conditionals.any { it.type == UniqueType.ConditionalOurUnit && it.params[0] == civInfo.religionManager.getGreatProphetEquivalent()?.name }) 1/2f
+                if (unique.getModifiers(UniqueType.ConditionalOurUnit).any { it.params[0] == civInfo.religionManager.getGreatProphetEquivalent()?.name }) 1/2f
                 else 1f
             // Some city-filters are modified by personality (non-enemy foreign cities)
             score += modifier * when (unique.type) {
-                UniqueType.KillUnitPlunderNearCity ->
-                    unique.params[0].toFloat() * 4f *
-                        if (civInfo.wantsToFocusOn(Victory.Focus.Military)) 2f
-                        else 1f
+                UniqueType.KillUnitPlunderNearCity -> 0f //can be very strong, but the AI currently isn't farming barb camps
                 UniqueType.BuyUnitsForAmountStat, UniqueType.BuyBuildingsForAmountStat ->
                     if (civInfo.religionManager.religion != null
-                        && civInfo.religionManager.religion!!.getFollowerUniques()
-                            .any { it.type == unique.type }
+                        && civInfo.religionManager.religion!!.followerBeliefUniqueMap.getUniques(unique.type).any()
                     ) 0f
                     // This is something completely different from the original, but I have no idea
                     // what happens over there
-                    else civInfo.stats.statsForNextTurn[Stat.valueOf(unique.params[2])] * 5f / unique.params[1].toFloat()
+                    else civInfo.stats.statsForNextTurn[Stat.valueOf(unique.params[2])] * 300f / unique.params[1].toFloat() //the costs of these are probably similar to the baseUnitBuyCost
                 UniqueType.BuyUnitsWithStat, UniqueType.BuyBuildingsWithStat ->
                     if (civInfo.religionManager.religion != null
-                        && civInfo.religionManager.religion!!.getFollowerUniques()
-                            .any { it.type == unique.type }
+                        && civInfo.religionManager.religion!!.followerBeliefUniqueMap.getUniques(unique.type).any()
                     ) 0f
                     // This is something completely different from the original, but I have no idea
                     // what happens over there
-                    else civInfo.stats.statsForNextTurn[Stat.valueOf(unique.params[1])] * 10f / civInfo.getEra().baseUnitBuyCost
+                    else civInfo.stats.statsForNextTurn[Stat.valueOf(unique.params[1])] * 300f / civInfo.getEra().baseUnitBuyCost
+                    //baseUnitBuyCost is 200 in Standard speed pre-Renaissance, but overvalue such as to let the high-faith civs pick the good faith sinks 
                 UniqueType.BuyUnitsByProductionCost ->
-                    15f * if (civInfo.wantsToFocusOn(Victory.Focus.Military)) 2f else 1f
+                    0f //Holy Warriors is a waste if we don't buy units with it, and if we buy units with it'll cost us great persons
                 UniqueType.StatsWhenSpreading ->
-                    unique.params[0].toFloat() / 5f
-                UniqueType.StatsWhenAdoptingReligion, UniqueType.StatsWhenAdoptingReligionSpeed ->
+                    unique.params[0].toFloat() / 15f
+                UniqueType.StatsWhenAdoptingReligion ->
                     unique.stats.values.sum() / 50f
                 UniqueType.RestingPointOfCityStatesFollowingReligionChange ->
-                    if (civInfo.wantsToFocusOn(Victory.Focus.CityStates))
-                        unique.params[0].toFloat() / 3.5f
-                    else
-                        unique.params[0].toFloat() / 7f
+                    unique.params[0].toFloat() / 8f
                 UniqueType.StatsFromGlobalCitiesFollowingReligion ->
-                    unique.stats.values.sum()
+                    unique.stats.values.sum() * 2f //free yields that are potentially more than our own number of cities would allow
                 UniqueType.StatsFromGlobalFollowers ->
-                    4f * (unique.stats.values.sum() / unique.params[1].toFloat())
-                UniqueType.ProvidesStatsWheneverGreatPersonExpended ->
-                    unique.stats.values.sum() / 2f
+                    10f * (unique.stats.values.sum() / unique.params[1].toFloat())
                 UniqueType.Strength ->
-                    unique.params[0].toFloat() / 4f
+                    unique.params[0].toFloat() * 3f//combat strength from beliefs is very strong
                 UniqueType.ReligionSpreadDistance ->
                     (10f + unique.params[0].toFloat()) * goodEarlyModifier
                 UniqueType.NaturalReligionSpreadStrength ->
-                    unique.params[0].toFloat() * goodEarlyModifier / 5f
+                    unique.params[0].toFloat() * goodEarlyModifier / 10f //We should weigh this according to cityFilter; Religious Texts is way stronger than Religious Unity
                 UniqueType.SpreadReligionStrength ->
-                    unique.params[0].toFloat() * goodLateModifier / 5f
+                    unique.params[0].toFloat() * goodLateModifier / 10f //Cheaper missionaries are generally better than stronger missionaries
                 UniqueType.FaithCostOfGreatProphetChange ->
-                    -unique.params[0].toFloat() * goodLateModifier / 2f
+                    -unique.params[0].toFloat() * goodLateModifier / 10f //It's only about 1 more prophet, due to the increasing costs.
                 UniqueType.BuyBuildingsDiscount, UniqueType.BuyUnitsDiscount ->
                     -unique.params[2].toFloat() * goodLateModifier / 5f
                 UniqueType.BuyItemsDiscount ->
@@ -407,7 +424,7 @@ object ReligionAutomation {
         // line 4426 through 4870.
         // This is way too much work for now, so I'll just choose a random pantheon instead.
         // Should probably be changed later, but it works for now.
-        val chosenPantheon = chooseBeliefOfType(civInfo, BeliefType.Pantheon)
+        val chosenPantheon = pickBeliefOfType(civInfo, BeliefType.Pantheon)
             ?: return // panic!
         civInfo.religionManager.chooseBeliefs(
             listOf(chosenPantheon),
@@ -417,16 +434,17 @@ object ReligionAutomation {
 
     private fun foundReligion(civInfo: Civilization) {
         if (civInfo.religionManager.religionState != ReligionState.FoundingReligion) return
-        val availableReligionIcons = civInfo.gameInfo.ruleset.religions
-            .filterNot { civInfo.gameInfo.religions.values.map { religion -> religion.name }.contains(it) }
-        val favoredReligion = civInfo.nation.favoredReligion
-        val religionIcon =
-            if (favoredReligion != null && favoredReligion in availableReligionIcons
-                && (1..10).random() <= 5) favoredReligion
-            else availableReligionIcons.randomOrNull()
-                ?: return // Wait what? How did we pass the checking when using a great prophet but not this?
+        val usedReligions = civInfo.gameInfo.religions.values.mapTo(mutableSetOf()) { it.name }
+        val availableReligions = civInfo.gameInfo.ruleset.religions.filterNot { it in usedReligions }
+        val favoredReligion = civInfo.nation.favoredReligion?.takeIf { it in availableReligions }
+        val allFavoredReligions = civInfo.gameInfo.civilizations.mapNotNullTo(mutableSetOf()) { it.nation.favoredReligion}
+        val nonFavoredReligions = availableReligions.filterNot { it in allFavoredReligions }
+        val chosenReligion = favoredReligion
+            ?: nonFavoredReligions.randomOrNull() // allow other civs to found their own favoured religion when possible
+            ?: availableReligions.randomOrNull()
+            ?: return // Wait what? How did we pass the checking when using a great prophet but not this?
 
-        civInfo.religionManager.foundReligion(religionIcon, religionIcon)
+        civInfo.religionManager.foundReligion(chosenReligion, chosenReligion)
 
         val chosenBeliefs = chooseBeliefs(civInfo, civInfo.religionManager.getBeliefsToChooseAtFounding()).toList()
         civInfo.religionManager.chooseBeliefs(chosenBeliefs)
@@ -453,27 +471,28 @@ object ReligionAutomation {
         // belief less than make the game crash. The `continue`s should only be reached whenever
         // there are not enough beliefs to choose, but there should be, as otherwise we could
         // not have used a great prophet to found/enhance our religion.
-        for (belief in BeliefType.values()) {
+        for (belief in BeliefType.entries) {
             if (belief == BeliefType.None) continue
             repeat(beliefsToChoose[belief]) {
                 chosenBeliefs.add(
-                    chooseBeliefOfType(civInfo, belief, chosenBeliefs) ?: return@repeat
+                    pickBeliefOfType(civInfo, belief, chosenBeliefs) ?: return@repeat
                 )
             }
         }
         return chosenBeliefs
     }
 
-    private fun chooseBeliefOfType(civInfo: Civilization, beliefType: BeliefType, additionalBeliefsToExclude: HashSet<Belief> = hashSetOf()): Belief? {
+    @Readonly
+    private fun pickBeliefOfType(civInfo: Civilization, beliefType: BeliefType, additionalBeliefsToExclude: HashSet<Belief> = hashSetOf()): Belief? {
         return civInfo.gameInfo.ruleset.beliefs.values
             .filter {
                 (it.type == beliefType || beliefType == BeliefType.Any)
                     && !additionalBeliefsToExclude.contains(it)
                     && civInfo.religionManager.getReligionWithBelief(it) == null
-                    && it.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
-                    .none { unique -> !unique.conditionalsApply(civInfo) }
+                    && it.getMatchingUniques(UniqueType.OnlyAvailable, GameContext.IgnoreConditionals)
+                    .none { unique -> !unique.conditionalsApply(civInfo.state) }
             }
-            .maxByOrNull { ReligionAutomation.rateBelief(civInfo, it) }
+            .maxByOrNull { rateBelief(civInfo, it) }
     }
 
 
