@@ -1,33 +1,37 @@
 package com.unciv.logic.automation.civilization
 
+import com.unciv.logic.automation.unit.UnitAutomation
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.BFS
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.INonPerpetualConstruction
-import com.unciv.models.ruleset.PerpetualConstruction
-import com.unciv.models.ruleset.Victory
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
-import java.util.SortedMap
-import java.util.TreeMap
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
+import java.util.*
 
 object UseGoldAutomation {
 
 
     /** allow AI to spend money to purchase city-state friendship, buildings & unit */
     fun useGold(civ: Civilization) {
+        for (unit in civ.units.getCivUnits())
+            UnitAutomation.tryUpgradeUnit(unit)
+
         if (civ.isMajorCiv())
             useGoldForCityStates(civ)
 
-        for (city in civ.cities.sortedByDescending { it.population.population }) {
+        for (city in civ.cities.sortedByDescending { it.cityStats.currentCityStats.production }) {
+            // Low production cities are disallowed from building military and world wonders, and have low priority for xp buildings
             val construction = city.cityConstructions.getCurrentConstruction()
-            if (construction is PerpetualConstruction) continue
-            if ((construction as INonPerpetualConstruction).canBePurchasedWithStat(city, Stat.Gold)
-                && city.civ.gold / 3 >= construction.getStatBuyCost(city, Stat.Gold)!!) {
-                city.cityConstructions.purchaseConstruction(construction, 0, true)
-            }
+            if (construction !is INonPerpetualConstruction) continue
+            val statBuyCost = construction.getStatBuyCost(city, Stat.Gold) ?: continue
+            if (!city.cityConstructions.isConstructionPurchaseAllowed(construction, Stat.Gold, statBuyCost)) continue
+            if (civ.gold < statBuyCost || construction.hurryCostModifier > 10) continue // Don't buy things that are more expensive than they need to be
+            city.cityConstructions.purchaseConstruction(construction, 0, true)
         }
 
         maybeBuyCityTiles(civ)
@@ -37,7 +41,7 @@ object UseGoldAutomation {
     private fun useGoldForCityStates(civ: Civilization) {
         // RARE EDGE CASE: If you ally with a city-state, you may reveal more map that includes ANOTHER civ!
         // So if we don't lock this list, we may later discover that there are more known civs, concurrent modification exception!
-        val knownCityStates = civ.getKnownCivs().filter { it.isCityState() }.toList()
+        val knownCityStates = civ.getKnownCivs().filter { it.isCityState && MotivationToAttackAutomation.hasAtLeastMotivationToAttack(civ, it, 0f) <= 0 }.toList()
 
         // canBeMarriedBy checks actual cost, but it can't be below 500*speedmodifier, and the later check is expensive
         if (civ.gold >= 330 && civ.getHappiness() > 0 && civ.hasUnique(UniqueType.CityStateCanBeBoughtForGold)) {
@@ -48,19 +52,9 @@ object UseGoldAutomation {
             }
         }
 
-        if (civ.gold < 250) return  // skip checks if tryGainInfluence will bail anyway
-        if (civ.wantsToFocusOn(Victory.Focus.Culture)) {
-            for (cityState in knownCityStates.filter { it.cityStateFunctions.canProvideStat(Stat.Culture) }) {
-                val diploManager = cityState.getDiplomacyManager(civ)
-                if (diploManager.getInfluence() < 40) { // we want to gain influence with them
-                    tryGainInfluence(civ, cityState)
-                }
-            }
-        }
-
-        if (civ.gold < 250 || knownCityStates.none()) return
+        if (civ.gold < 500 || knownCityStates.none()) return // skip checks if tryGainInfluence will bail anyway
         val cityState = knownCityStates
-            .filter { it.getAllyCiv() != civ.civName }
+            .filter { it.allyCiv != civ }
             .associateWith { NextTurnAutomation.valueCityStateAlliance(civ, it, true) }
             .maxByOrNull { it.value }?.takeIf { it.value > 0 }?.key
         if (cityState != null) {
@@ -121,58 +115,63 @@ object UseGoldAutomation {
         }
     }
 
+    @Readonly
     private fun getHighlyDesirableTilesToCityMap(civInfo: Civilization): SortedMap<Tile, MutableSet<City>> {
         val highlyDesirableTiles: SortedMap<Tile, MutableSet<City>> = TreeMap(
             compareByDescending<Tile?> { it?.naturalWonder != null }
-                .thenByDescending { it?.resource != null && it.tileResource.resourceType == ResourceType.Luxury }
-                .thenByDescending { it?.resource != null && it.tileResource.resourceType == ResourceType.Strategic }
+                .thenByDescending { it?.tileResource?.resourceType == ResourceType.Luxury }
+                .thenByDescending { it?.tileResource?.resourceType == ResourceType.Strategic }
                 // This is necessary, so that the map keeps Tiles with the same resource as two
                 // separate entries.
                 .thenBy { it.hashCode() }
         )
 
         for (city in civInfo.cities.filter { !it.isPuppet && !it.isBeingRazed }) {
-            val highlyDesirableTilesInCity = city.tilesInRange.filter {
+            val highlyDesirableTilesInCity = city.getCenterTile().getTilesAtDistance(2).filter {
+                // Only consider second ring tiles: further tiles may be as good or better, but have much higher gold cost 
                 isHighlyDesirableTile(it, civInfo, city)
             }
             for (highlyDesirableTileInCity in highlyDesirableTilesInCity) {
-                highlyDesirableTiles.getOrPut(highlyDesirableTileInCity) { mutableSetOf() }
-                    .add(city)
+                @LocalState val desirableTiles = highlyDesirableTiles.getOrPut(highlyDesirableTileInCity) { mutableSetOf() }
+                desirableTiles.add(city)
             }
         }
         return highlyDesirableTiles
     }
 
+    @Readonly
     private fun isHighlyDesirableTile(it: Tile, civInfo: Civilization, city: City): Boolean {
         if (!it.isVisible(civInfo)) return false
         if (it.getOwner() != null) return false
         if (it.neighbors.none { neighbor -> neighbor.getCity() == city }) return false
 
-        fun hasNaturalWonder() = it.naturalWonder != null
+        @Readonly fun hasNaturalWonder() = it.naturalWonder != null
 
-        fun hasLuxuryCivDoesntOwn() =
-            it.hasViewableResource(civInfo)
-                && it.tileResource.resourceType == ResourceType.Luxury
-                && !civInfo.hasResource(it.resource!!)
+        @Readonly  fun hasLuxury(): Boolean {
+            val resource = it.tileResource
+            return civInfo.canSeeResource(resource) &&
+                resource.resourceType == ResourceType.Luxury &&
+                civInfo.getResourceAmount(resource) < 2 // At 2 or more, we haven't been able to trade it away for another duplicate...
+        }
 
-        fun hasResourceCivHasNoneOrLittle() =
-            it.hasViewableResource(civInfo)
-                && it.tileResource.resourceType == ResourceType.Strategic
-                && civInfo.getResourceAmount(it.resource!!) <= 3
+        @Readonly fun hasHighYields(): Boolean {
+            val tileStats = it.stats.getTileStats(civInfo)
+            return tileStats.food + tileStats.production >= 3
+        }
 
-        return (hasNaturalWonder() || hasLuxuryCivDoesntOwn() || hasResourceCivHasNoneOrLittle())
+        return (hasNaturalWonder() || hasLuxury() || hasHighYields())
     }
 
     private fun tryGainInfluence(civInfo: Civilization, cityState: Civilization) {
-        if (civInfo.gold < 250) return // Save up
-        if (cityState.getDiplomacyManager(civInfo).getInfluence() >= 20
-            && civInfo.gold < 500) {
-            // Only make a small investment if we have a bit of influence already to build off of so we don't waste our money
-            cityState.cityStateFunctions.receiveGoldGift(civInfo, 250)
-            return
-        }
-        if (civInfo.gold < 500) return // It's not worth it to invest now, wait until you have enough for 2
-        cityState.cityStateFunctions.receiveGoldGift(civInfo, 500)
+        if (civInfo.gold < 500) return // Save up, giving 500 gold in one go typically grants +5 influence compared to giving 2×250 gold
+        val influence = cityState.getDiplomacyManager(civInfo)!!.getInfluence()
+        val stopSpending =  influence > 60 + 2 * NextTurnAutomation.valueCityStateAlliance(civInfo, cityState, true)
+        // Don't go into a gold gift race: be content with friendship for cheap, or use the gold on more productive uses,
+        // for example upgrading an army to conquer the player who's contesting our city states
+        if (influence < 10 || stopSpending) return
+        // Only make an investment if we got our Pledge to Protect influence at the highest level
+        if (civInfo.gold >= 1000) cityState.cityStateFunctions.receiveGoldGift(civInfo, 1000)
+        else cityState.cityStateFunctions.receiveGoldGift(civInfo, 500)
         return
     }
 
