@@ -1,23 +1,60 @@
 package com.unciv.logic.automation.unit
 
+import com.unciv.logic.battle.AirInterception
+import com.unciv.logic.battle.Battle
 import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.battle.Nuke
 import com.unciv.logic.battle.TargetHelper
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
+import com.unciv.models.ruleset.unique.UniqueType
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 
 object AirUnitAutomation {
 
     fun automateFighter(unit: MapUnit) {
+        if (unit.health < 75) return // Wait and heal
+
         val tilesWithEnemyUnitsInRange = unit.civ.threatManager.getTilesWithEnemyUnitsInDistance(unit.getTile(), unit.getRange())
+        // TODO: Optimize [friendlyAirUnitsInRange] by creating an alternate [ThreatManager.getTilesWithEnemyUnitsInDistance] that handles only friendly units
+        val friendlyAirUnitsInRange = unit.getTile().getTilesInDistance(unit.getRange()).flatMap { it.airUnits }.filter { it.civ == unit.civ }
+        // Find all visible enemy air units
         val enemyAirUnitsInRange = tilesWithEnemyUnitsInRange
             .flatMap { it.airUnits.asSequence() }.filter { it.civ.isAtWarWith(unit.civ) }
+        val enemyFighters = enemyAirUnitsInRange.size / 2 // Assume half the planes are fighters
+        val friendlyUnusedFighterCount = friendlyAirUnitsInRange.count { it.canAttack() }
+        val friendlyUsedFighterCount = friendlyAirUnitsInRange.count { !it.canAttack() }
 
-        if (enemyAirUnitsInRange.any()) return // we need to be on standby in case they attack
+        // We need to be on standby in case they attack
+        if (friendlyUnusedFighterCount < enemyFighters) return
+
+        if (friendlyUsedFighterCount <= enemyFighters) {
+            @Readonly fun airSweepDamagePercentBonus(): Int {
+                return unit.getMatchingUniques(UniqueType.StrengthWhenAirsweep)
+                    .sumOf { it.params[0].toInt() }
+            }
+
+            // If we are outnumbered, don't heal after attacking and don't have an Air Sweep bonus
+            // Then we shouldn't speed the air battle by killing our fighters, instead, focus on defending
+            if (friendlyUsedFighterCount + friendlyUnusedFighterCount < enemyFighters
+                && !unit.hasUnique(UniqueType.HealsEvenAfterAction)
+                && airSweepDamagePercentBonus() <= 0) {
+                return
+            } else {
+                if (tryAirSweep(unit, tilesWithEnemyUnitsInRange)) return
+            }
+        }
+
+        if (unit.health < 80) {
+            return // Wait and heal up, no point in moving closer to battle if we aren't healed
+        }
 
         if (BattleHelper.tryAttackNearbyEnemy(unit)) return
 
+        if (unit.cache.cannotMove) return // from here on it's all "try to move somewhere else"
+
         if (tryRelocateToCitiesWithEnemyNearBy(unit)) return
 
         val pathsToCities = unit.movement.getAerialPathsToCities()
@@ -43,12 +80,28 @@ object AirUnitAutomation {
 
         // no city needs fighters to defend, so let's attack stuff from the closest possible location
         tryMoveToCitiesToAerialAttackFrom(pathsToCities, unit)
+    }
 
+    private fun tryAirSweep(unit: MapUnit, tilesWithEnemyUnitsInRange: List<Tile>): Boolean {
+        val targetTile = tilesWithEnemyUnitsInRange.filter {
+            tile -> tile.getUnits().any { it.civ.isAtWarWith(unit.civ)
+                || (tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(unit.civ)) }
+            }.minByOrNull { it.aerialDistanceTo(unit.getTile()) } ?: return false
+        AirInterception.airSweep(MapUnitCombatant(unit),targetTile)
+        return !unit.hasMovement()
     }
 
     fun automateBomber(unit: MapUnit) {
+        if (unit.health < 75) return // Wait and heal
+
         if (BattleHelper.tryAttackNearbyEnemy(unit)) return
 
+        if (unit.health <= 90 || (unit.health < 100 && !unit.civ.isAtWar())) {
+            return // Wait and heal
+        }
+
+        if (unit.cache.cannotMove) return // from here on it's all "try to move somewhere else"
+
         if (tryRelocateToCitiesWithEnemyNearBy(unit)) return
 
         val pathsToCities = unit.movement.getAerialPathsToCities()
@@ -76,26 +129,29 @@ object AirUnitAutomation {
     fun automateNukes(unit: MapUnit) {
         if (!unit.civ.isAtWar()) return
         // We should *Almost* never want to nuke our own city, so don't consider it
-        val tilesInRange = unit.currentTile.getTilesInDistanceRange(2..unit.getRange())
-        var highestTileNukeValue = 0
-        var tileToNuke: Tile? = null
-        tilesInRange.forEach {
-            val value = getNukeLocationValue(unit, it)
-            if (value > highestTileNukeValue) {
-                highestTileNukeValue = value
-                tileToNuke = it
-            }
+        if (unit.type.isAirUnit()) {
+            val tilesInRange = unit.currentTile.getTilesInDistanceRange(2..unit.getRange())
+            val highestTileNukeValue = tilesInRange.map { it to getNukeLocationValue(unit, it) }
+                .maxByOrNull { it.second }
+            if (highestTileNukeValue != null && highestTileNukeValue.second > 0)
+                Nuke.NUKE(MapUnitCombatant(unit), highestTileNukeValue.first)
+
+            tryRelocateMissileToNearbyAttackableCities(unit)
+        } else {
+            val attackableTiles = TargetHelper.getAttackableEnemies(unit, unit.movement.getDistanceToTiles())
+            val highestTileNukeValue = attackableTiles.map { it to getNukeLocationValue(unit, it.tileToAttack) }
+                .maxByOrNull { it.second }
+            if (highestTileNukeValue != null && highestTileNukeValue.second > 0)
+                Battle.moveAndAttack(MapUnitCombatant(unit), highestTileNukeValue.first)
+            HeadTowardsEnemyCityAutomation.tryHeadTowardsEnemyCity(unit)
         }
-        if (highestTileNukeValue > 0) {
-            Nuke.NUKE(MapUnitCombatant(unit), tileToNuke!!)
-        }
-        tryRelocateMissileToNearbyAttackableCities(unit)
     }
 
     /**
      * Ranks the tile to nuke based off of all tiles in it's blast radius
      * By default the value is -500 to prevent inefficient nuking.
      */
+    @Readonly
     private fun getNukeLocationValue(nuke: MapUnit, tile: Tile): Int {
         val civ = nuke.civ
         if (!Nuke.mayUseNuke(MapUnitCombatant(nuke), tile)) return Int.MIN_VALUE
@@ -113,6 +169,7 @@ object AirUnitAutomation {
         var explosionValue = -500
 
         // Returns either ourValue or thierValue depending on if the input Civ matches the Nuke's Civ
+        @Pure
         fun evaluateCivValue(targetCiv: Civilization, ourValue: Int, theirValue: Int): Int {
             if (targetCiv == civ) // We are nuking something that we own!
                 return ourValue
@@ -125,7 +182,7 @@ object AirUnitAutomation {
                     if (targetUnit.isInvisible(civ)) continue
                     // If we are nuking a unit at ground zero, it is more likely to be destroyed
                     val tileExplosionValue = if (targetTile == tile) 80 else 50
-                    
+
                     if (targetUnit.isMilitary()) {
                         explosionValue += if (targetTile == tile) evaluateCivValue(targetUnit.civ, -200, tileExplosionValue)
                         else evaluateCivValue(targetUnit.civ, -150, 50)
@@ -142,7 +199,7 @@ object AirUnitAutomation {
             else if (targetTile.owningCity != null) {
                 val owningCiv = targetTile.owningCity?.civ!!
                 // If there is a tile to add fallout to there is a 50% chance it will get fallout
-                if (!(tile.isWater || tile.isImpassible() || targetTile.terrainFeatures.any { it == "Fallout" }))
+                if (!(tile.isWater || tile.isImpassible() || targetTile.hasFalloutEquivalent()))
                     explosionValue += evaluateCivValue(owningCiv, -40, 10)
                 // If there is an improvment to pillage
                 if (targetTile.improvement != null && !targetTile.improvementIsPillaged)
