@@ -5,19 +5,37 @@ import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitActionType
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionModifiers
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionModifiers.canUse
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
+import yairm210.purity.annotations.Readonly
 
 object CivilianUnitAutomation {
 
+    @Readonly
     fun shouldClearTileForAddInCapitalUnits(unit: MapUnit, tile: Tile) =
-        tile.getCity()?.isCapital() == true
+        tile.isCityCenter() && tile.getCity()!!.isCapital()
         && !unit.hasUnique(UniqueType.AddInCapital)
         && unit.civ.units.getCivUnits().any { unit.hasUnique(UniqueType.AddInCapital) }
 
     fun automateCivilianUnit(unit: MapUnit, dangerousTiles: HashSet<Tile>) {
+        // To allow "found city" actions that can only trigger a limited number of times
+        
+        // Slightly modified getUsableUnitActionUniques() to allow for settlers with *conditional* settling uniques
+        @Readonly
+        fun hasSettlerAction(uniqueType: UniqueType) =
+            unit.getMatchingUniques(uniqueType, GameContext.IgnoreConditionals)
+                .filter { unique -> !unique.hasModifier(UniqueType.UnitActionExtraLimitedTimes) }
+                .any { canUse(unit, it) }
+        
+        val hasSettlerUnique = hasSettlerAction(UniqueType.FoundCity) || hasSettlerAction(UniqueType.FoundPuppetCity)
+        
+        if (hasSettlerUnique && !(unit.civ.isCityState && unit.isMilitary()))
+            return SpecificUnitAutomation.automateSettlerActions(unit, dangerousTiles)
+
         if (tryRunAwayIfNeccessary(unit)) return
 
         if (shouldClearTileForAddInCapitalUnits(unit, unit.currentTile)) {
@@ -25,12 +43,9 @@ object CivilianUnitAutomation {
             val tilesCanMoveTo = unit.movement.getDistanceToTiles()
                 .filter { unit.movement.canMoveTo(it.key) }
             if (tilesCanMoveTo.isNotEmpty())
-                unit.movement.moveToTile(tilesCanMoveTo.minByOrNull { it.value.totalDistance }!!.key)
+                unit.movement.moveToTile(tilesCanMoveTo.minByOrNull { it.value.totalMovement }!!.key)
         }
 
-        if (unit.hasUnique(UniqueType.FoundCity))
-            return SpecificUnitAutomation.automateSettlerActions(unit, dangerousTiles)
-
         if (unit.isAutomatingRoadConnection())
             return unit.civ.getWorkerAutomation().roadToAutomation.automateConnectRoad(unit, dangerousTiles)
 
@@ -48,6 +63,13 @@ object CivilianUnitAutomation {
             && unit.civ.religionManager.mayFoundReligionAtAll()
         )
             return ReligiousUnitAutomation.foundReligion(unit)
+        
+        if (unit.hasUnique(UniqueType.MayFoundReligion) && unit.civ.isCityState){
+            // We have literally nothing to do with this unit, at least stop costing money
+            unit.disband()
+            return 
+        }
+            
 
         if (unit.hasUnique(UniqueType.MayEnhanceReligion)
             && unit.civ.religionManager.religionState < ReligionState.EnhancedReligion
@@ -67,8 +89,6 @@ object CivilianUnitAutomation {
             return
         if (unit.cache.hasCitadelPlacementUnique && SpecificUnitAutomation.automateCitadelPlacer(unit))
             return
-        if (unit.cache.hasCitadelPlacementUnique || unit.cache.hasStrengthBonusInRadiusUnique)
-            return SpecificUnitAutomation.automateGreatGeneralFallback(unit)
 
         if (unit.civ.religionManager.maySpreadReligionAtAll(unit))
             return ReligiousUnitAutomation.automateMissionary(unit)
@@ -85,15 +105,16 @@ object CivilianUnitAutomation {
 
             val hurriedPolicy = UnitActions.invokeUnitAction(unit, UnitActionType.HurryPolicy)
             if (hurriedPolicy) return
+            //TODO: save up great scientists/writers for late game (8 turns after research labs/broadcast towers resp.)
         }
 
         // Great merchant -> Conduct trade mission if late game and if not at war.
         // TODO: This could be more complex to walk to the city state that is most beneficial to
         //  also have more influence.
         if (unit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence)
-            // Don't wander around with the great merchant when at war. Barbs might also be a
-            // problem, but hopefully by the time we have a great merchant, they're under control.
-            && !unit.civ.isAtWar()
+            // There's a risk our merchant gets intercepted and killed by the enemy during war.
+            // If such happens, it is a failure of our military unit movement to protect our merchant.
+            // Barbs might also be a problem, but hopefully by the time we have a great merchant, they're under control.
             && isLateGame
         ) {
             val tradeMissionCanBeConductedEventually =
@@ -102,24 +123,14 @@ object CivilianUnitAutomation {
                 return
         }
 
-        // Great engineer -> Try to speed up wonder construction if late game
-        if (isLateGame &&
-            (unit.hasUnique(UniqueType.CanSpeedupConstruction)
-                || unit.hasUnique(UniqueType.CanSpeedupWonderConstruction))) {
+        // Great engineer -> Try to speed up wonder construction
+        if (unit.hasUnique(UniqueType.CanSpeedupConstruction)
+                || unit.hasUnique(UniqueType.CanSpeedupWonderConstruction)) {
             val wonderCanBeSpedUpEventually = SpecificUnitAutomation.speedupWonderConstruction(unit)
             if (wonderCanBeSpedUpEventually)
                 return
         }
 
-        // This has to come after the individual abilities for the great people that can also place
-        // instant improvements (e.g. great scientist).
-        if (unit.hasUnique(UniqueType.ConstructImprovementInstantly)) {
-            // catch great prophet for civs who can't found/enhance/spread religion
-            // includes great people plus moddable units
-            val improvementCanBePlacedEventually =
-                SpecificUnitAutomation.automateImprovementPlacer(unit)
-        }
-
         if (unit.hasUnique(UniqueType.GainFreeBuildings)) {
             val unique = unit.getMatchingUniques(UniqueType.GainFreeBuildings).first()
             val buildingName = unique.params[0]
@@ -127,10 +138,12 @@ object CivilianUnitAutomation {
             val cityToGainBuilding = unit.civ.cities.filter {
                 !it.cityConstructions.containsBuildingOrEquivalent(buildingName)
                     && (unit.movement.canMoveTo(it.getCenterTile()) || unit.currentTile == it.getCenterTile())
-            }.minByOrNull {
+            }.map {
                 val path = unit.movement.getShortestPath(it.getCenterTile())
-                path.size
-            }
+                // We want to calc path once, but still filter out unreachable cities
+                it to path.size
+            }.filter { it.second > 0 }.minByOrNull { it.second }?.first
+            
 
             if (cityToGainBuilding != null) {
                 if (unit.currentTile == cityToGainBuilding.getCenterTile()) {
@@ -147,23 +160,34 @@ object CivilianUnitAutomation {
         //  (depending on number of cities) and after that they should just be used to start golden
         //  ages?
 
+        if (SpecificUnitAutomation.automateImprovementPlacer(unit)) return
+        
+        val goldenAgeAction = UnitActions.getUnitActions(unit, UnitActionType.TriggerUnique)
+            .filter { it.action != null && it.associatedUnique?.type in listOf(UniqueType.OneTimeEnterGoldenAge,
+                UniqueType.OneTimeEnterGoldenAgeTurns) }.firstOrNull()
+        if (goldenAgeAction != null) {
+            goldenAgeAction.action?.invoke()
+            return
+        }
+
         return // The AI doesn't know how to handle unknown civilian units
     }
 
-    private fun isLateGame(civ: Civilization): Boolean {
+    @Readonly
+    fun isLateGame(civ: Civilization): Boolean {
         val researchCompletePercent =
             (civ.tech.researchedTechnologies.size * 1.0f) / civ.gameInfo.ruleset.technologies.size
-        return researchCompletePercent >= 0.8f
+        return researchCompletePercent >= 0.55f
     }
 
     /** Returns whether the civilian spends its turn hiding and not moving */
-    private fun tryRunAwayIfNeccessary(unit: MapUnit): Boolean {
+    fun tryRunAwayIfNeccessary(unit: MapUnit): Boolean {
         // This is a little 'Bugblatter Beast of Traal': Run if we can attack an enemy
         // Cheaper than determining which enemies could attack us next turn
         val enemyUnitsInWalkingDistance = unit.movement.getDistanceToTiles().keys
             .filter { unit.civ.threatManager.doesTileHaveMilitaryEnemy(it) }
 
-        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary()
+        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary
             && unit.getTile().militaryUnit == null && !unit.getTile().isCityCenter()) {
             runAway(unit)
             return true
@@ -188,11 +212,22 @@ object CivilianUnitAutomation {
             unit.movement.moveToTile(defensiveUnit)
             return
         }
-        val tileFurthestFromEnemy = reachableTiles.keys
-            .filter { unit.movement.canMoveTo(it) && unit.getDamageFromTerrain(it) < unit.health }
-            .maxByOrNull { unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(), 4, false) }
-            ?: return // can't move anywhere!
-        unit.movement.moveToTile(tileFurthestFromEnemy)
-    }
 
+        val unitTile = unit.getTile()
+        val dangerousTiles = unit.civ.threatManager.getDangerousTiles(unit)
+        val tileClosestToDanger = dangerousTiles
+            // Priotirize capture threat over ranged attack
+            .sortedByDescending { unit.civ.threatManager.getEnemyUnitsOnTiles(listOf(it)).isNotEmpty() }
+            .minByOrNull { it.aerialDistanceTo(unitTile) } ?: unitTile
+        val tileFurthestFromDanger = reachableTiles.keys
+            .filter {
+                unit.movement.canMoveTo(it)
+                    && unit.getDamageFromTerrain(it) < unit.health
+                    && it !in dangerousTiles }
+            .sortedWith(compareByDescending<Tile> { it.aerialDistanceTo(tileClosestToDanger) } // As far away from threat
+                .thenByDescending { it.isFriendlyTerritory(unit.civ) }) // Priotirize friendly territory
+            .firstOrNull() ?: return // can't move anywhere!
+
+        unit.movement.moveToTile(tileFurthestFromDanger)
+    }
 }
