@@ -1,6 +1,7 @@
-﻿package com.unciv.logic.automation.unit
+package com.unciv.logic.automation.unit
 
 import com.unciv.logic.automation.Automation
+import com.unciv.logic.automation.unit.CivilianUnitAutomation.tryRunAwayIfNeccessary
 import com.unciv.logic.battle.GreatGeneralImplementation
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
@@ -8,11 +9,13 @@ import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitActionType
 import com.unciv.models.ruleset.Building
+import com.unciv.models.ruleset.tile.TerrainType
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsFromUniques
+import yairm210.purity.annotations.Readonly
 import kotlin.math.roundToInt
 
 object SpecificUnitAutomation {
@@ -27,9 +30,13 @@ object SpecificUnitAutomation {
     }
 
     fun automateCitadelPlacer(unit: MapUnit): Boolean {
+        // Keep at least 2 generals alive
+        if (unit.hasUnique(UniqueType.StrengthBonusInRadius) 
+                && unit.civ.units.getCivUnits().count { it.hasUnique(UniqueType.StrengthBonusInRadius) } < 3) 
+            return false
         // try to revenge and capture their tiles
         val enemyCities = unit.civ.getKnownCivs()
-                .filter { unit.civ.getDiplomacyManager(it).hasModifier(DiplomaticModifiers.StealingTerritory) }
+                .filter { unit.civ.getDiplomacyManager(it)!!.hasModifier(DiplomaticModifiers.StealingTerritory) }
                 .flatMap { it.cities }
         // find the suitable tiles (or their neighbours)
         val tileToSteal = enemyCities.flatMap { it.getTiles() } // City tiles
@@ -52,51 +59,19 @@ object SpecificUnitAutomation {
         // if there is a good tile to steal - go there
         if (tileToSteal != null) {
             unit.movement.headTowards(tileToSteal)
-            if (unit.currentMovement > 0 && unit.currentTile == tileToSteal)
+            if (unit.hasMovement() && unit.currentTile == tileToSteal)
                 UnitActionsFromUniques.getImprovementConstructionActionsFromGeneralUnique(unit, unit.currentTile).firstOrNull()?.action?.invoke()
             return true
         }
 
-        // try to build a citadel for defensive purposes
-        if (unit.civ.getWorkerAutomation().evaluateFortPlacement(unit.currentTile, true)) {
-            UnitActionsFromUniques.getImprovementConstructionActionsFromGeneralUnique(unit, unit.currentTile).firstOrNull()?.action?.invoke()
-            return true
-        }
+        // keep in mind that citadels can be stolen by other citadels.
+        // you need 1 more general than your opponent, or be able to place it on a totally unreachable tile,
+        // for it to make sense to make the first move as the defender.
+        // so, let's focus only on the stealing part for now (the old defensive citadel placement logic turned out negative in simulation result)
+
         return false
     }
 
-    fun automateGreatGeneralFallback(unit: MapUnit) {
-        // if no unit to follow, take refuge in city or build citadel there.
-        val reachableTest: (Tile) -> Boolean = {
-            it.civilianUnit == null &&
-                    unit.movement.canMoveTo(it)
-                    && unit.movement.canReach(it)
-        }
-        val cityToGarrison = unit.civ.cities.asSequence().map { it.getCenterTile() }
-                .sortedBy { it.aerialDistanceTo(unit.currentTile) }
-                .firstOrNull { reachableTest(it) }
-            ?: return
-        if (!unit.cache.hasCitadelPlacementUnique) {
-            unit.movement.headTowards(cityToGarrison)
-            return
-        }
-
-        // try to find a good place for citadel nearby
-        val tileForCitadel = cityToGarrison.getTilesInDistanceRange(3..4)
-            .firstOrNull {
-                reachableTest(it) &&
-                    unit.civ.getWorkerAutomation().evaluateFortPlacement(it, true)
-            }
-        if (tileForCitadel == null) {
-            unit.movement.headTowards(cityToGarrison)
-            return
-        }
-        unit.movement.headTowards(tileForCitadel)
-        if (unit.currentMovement > 0 && unit.currentTile == tileForCitadel)
-            UnitActionsFromUniques.getImprovementConstructionActionsFromGeneralUnique(unit, unit.currentTile)
-                .firstOrNull()?.action?.invoke()
-    }
-
     fun automateSettlerActions(unit: MapUnit, dangerousTiles: HashSet<Tile>) {
         // If we don't have any cities, we are probably at the start of the game with only one settler
         // If we are at the start of the game lets spend a maximum of 3 turns to settle our first city
@@ -107,13 +82,14 @@ object SpecificUnitAutomation {
 
         // It's possible that we'll see a tile "over the sea" that's better than the tiles close by, but that's not a reason to abandon the close tiles!
         // Also this lead to some routing problems, see https://github.com/yairm210/Unciv/issues/3653
-        val bestTilesInfo = CityLocationTileRanker.getBestTilesToFoundCity(unit, rangeToSearch, 50f)
+        val minimumTileValue = unit.civ.gameInfo.ruleset.modOptions.constants.minimumCityLocationTileValue // To prevent AI from settling snow regions, could use a better implementation
+        val bestTilesInfo = CityLocationTileRanker.getBestTilesToFoundCity(unit, rangeToSearch, minimumTileValue)
         var bestCityLocation: Tile? = null
 
         if (unit.civ.gameInfo.turns == 0 && unit.civ.cities.isEmpty() && bestTilesInfo.tileRankMap.containsKey(unit.getTile())) {   // Special case, we want AI to settle in place on turn 1.
             val foundCityAction = UnitActionsFromUniques.getFoundCityAction(unit, unit.getTile())
             // Depending on era and difficulty we might start with more than one settler. In that case settle the one with the best location
-            val allUnsettledSettlers = unit.civ.units.getCivUnits().filter { it.currentMovement > 0 && it.baseUnit == unit.baseUnit }
+            val allUnsettledSettlers = unit.civ.units.getCivUnits().filter { it.hasMovement() && it.baseUnit == unit.baseUnit }
 
             // Don't settle immediately if we only have one settler, look for a better location
             val bestSettlerInRange = allUnsettledSettlers.maxByOrNull {
@@ -144,6 +120,7 @@ object SpecificUnitAutomation {
 
         if (bestCityLocation == null) {
             // Find the best tile that is within
+            @Readonly
             fun isTileRankOK(it: Map.Entry<Tile, Float>): Boolean {
                 if (it.key in dangerousTiles && it.key != unit.getTile()) return false
                 val pathSize = unit.movement.getShortestPath(it.key).size
@@ -167,9 +144,10 @@ object SpecificUnitAutomation {
             // Try to move towards the frontier
 
             /** @return the number of tiles 4 (un-modded) out from this city that could hold a city, ie how lonely this city is */
+            @Readonly
             fun getFrontierScore(city: City) = city.getCenterTile()
                 .getTilesAtDistance(city.civ.gameInfo.ruleset.modOptions.constants.minimalCityDistance + 1)
-                .count { it.canBeSettled() && (it.getOwner() == null || it.getOwner() == city.civ ) }
+                .count { it.canBeSettled(unit.civ) }
 
             val frontierCity = unit.civ.cities.maxByOrNull { getFrontierScore(it) }
             if (frontierCity != null && getFrontierScore(frontierCity) > 0  && unit.movement.canReach(frontierCity.getCenterTile()))
@@ -181,22 +159,27 @@ object SpecificUnitAutomation {
 
         val foundCityAction = UnitActionsFromUniques.getFoundCityAction(unit, bestCityLocation)
         if (foundCityAction?.action == null) { // this means either currentMove == 0 or city within 3 tiles
-            if (unit.currentMovement > 0) // therefore, city within 3 tiles
+            if (unit.hasMovement() && !unit.civ.isOneCityChallenger()) // therefore, city within 3 tiles
                 throw Exception("City within distance")
             return
         }
 
+        val shouldSettle = (unit.getTile() == bestCityLocation && unit.hasMovement())
+        if (shouldSettle) return foundCityAction.action.invoke()
+        //Settle if we're already on the best tile, before looking if we should retreat from barbarians
+        if (tryRunAwayIfNeccessary(unit)) return 
         unit.movement.headTowards(bestCityLocation)
-        if (unit.getTile() == bestCityLocation && unit.currentMovement > 0)
-            foundCityAction.action.invoke()
+        val shouldSettleNow = (unit.getTile() == bestCityLocation && unit.hasMovement())
+        if (shouldSettleNow) foundCityAction.action.invoke() 
     }
 
     /** @return whether there was any progress in placing the improvement. A return value of `false`
      * can be interpreted as: the unit doesn't know where to place the improvement or is stuck. */
     fun automateImprovementPlacer(unit: MapUnit) : Boolean {
-        val improvementBuildingUniques = unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
+        val improvementBuildingUnique = unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly).firstOrNull()
+            ?: return false
 
-        val improvementName = improvementBuildingUniques.first().params[0]
+        val improvementName = improvementBuildingUnique.params[0]
         val improvement = unit.civ.gameInfo.ruleset.tileImprovements[improvementName]
             ?: return false
         val relatedStat = improvement.maxByOrNull { it.value }?.key ?: Stat.Culture
@@ -205,13 +188,22 @@ object SpecificUnitAutomation {
             it.cityStats.statPercentBonusTree.totalStats[relatedStat]
         }
 
+        val averageTerrainStatsValue = unit.civ.gameInfo.ruleset.terrains.values.asSequence()
+            .filter { it.type == TerrainType.Land }
+            .map { Automation.rankStatsValue(it, unit.civ) }
+            .average()
 
+        val localUniqueCache = LocalUniqueCache()
         for (city in citiesByStatBoost) {
             val applicableTiles = city.getWorkableTiles().filter {
-                it.isLand && it.resource == null && !it.isCityCenter()
-                        && (unit.currentTile == it || unit.movement.canMoveTo(it))
-                        && !it.containsGreatImprovement() && it.improvementFunctions.canBuildImprovement(improvement, unit.civ)
+                it.isLand && (it.tileResource?.isImprovedBy(improvementName) != false) && !it.isCityCenter()
+                    && (unit.currentTile == it || unit.movement.canMoveTo(it))
+                    // okay if we replace regular improvements by great improvements, but not the other way around
+                    && (it.improvement == null || !it.getTileImprovement()!!.hasUnique(UniqueType.GreatImprovement))
+                    && it.improvementFunctions.canBuildImprovement(improvement, unit.cache.state)
+                    && (!improvement.hasUnique(UniqueType.GreatImprovement) || Automation.rankStatsValue(it.getBaseTerrain().cloneStats(), unit.civ) > averageTerrainStatsValue)
             }
+
             if (applicableTiles.none()) continue
 
             val pathToCity = unit.movement.getShortestPath(city.getCenterTile())
@@ -233,7 +225,6 @@ object SpecificUnitAutomation {
             }
 
             // if we got here, we're pretty close, start looking!
-            val localUniqueCache = LocalUniqueCache()
             val chosenTile = applicableTiles.sortedByDescending {
                 Automation.rankTile(
                     it,
@@ -262,26 +253,28 @@ object SpecificUnitAutomation {
      * `false` can be interpreted as: the unit doesn't know where to go or there are no city
      * states. */
     fun conductTradeMission(unit: MapUnit): Boolean {
-        val closestCityStateTile =
+        val conductedTradeMission = UnitActions.invokeUnitAction(unit, UnitActionType.ConductTradeMission)
+        if (conductedTradeMission) return true
+        
+        val relevantTiles = 
                 unit.civ.gameInfo.civilizations
                     .filter {
                         it != unit.civ
                             && !unit.civ.isAtWarWith(it)
-                            && it.isCityState()
+                            && it.isCityState
                             && it.cities.isNotEmpty()
                     }
                     .flatMap { it.cities[0].getTiles() }
                     .filter { unit.civ.hasExplored(it) }
-                    .mapNotNull { tile ->
-                        val path = unit.movement.getShortestPath(tile)
-                        // 0 is unreachable, 10 is too far away
-                        if (path.size in 1..10) tile to path.size else null
-                    }
-                    .minByOrNull { it.second }?.first
-                    ?: return false
+                    .sortedBy { unit.currentTile.aerialDistanceTo(it) }
 
-        val conductedTradeMission = UnitActions.invokeUnitAction(unit, UnitActionType.ConductTradeMission)
-        if (conductedTradeMission) return true
+        val closestCityStateTile = relevantTiles.firstOrNull { tile ->
+            if (!unit.movement.canPassThrough(tile)) return@firstOrNull false
+            val path = unit.movement.getShortestPath(tile)
+            // 0 is unreachable, 10 is too far away
+            path.size in 1..10
+        }
+        if (closestCityStateTile == null) return false
 
         val unitTileBeforeMovement = unit.currentTile
         unit.movement.headTowards(closestCityStateTile)
@@ -329,6 +322,7 @@ object SpecificUnitAutomation {
         return tileBeforeMoving != unit.currentTile
     }
 
+    @Readonly
     private fun getWonderThatWouldBenefitFromBeingSpedUp(city: City): Building? {
         return city.cityConstructions.getBuildableBuildings().filter { building ->
             building.isWonder && !building.hasUnique(UniqueType.CannotBeHurried)
@@ -341,6 +335,8 @@ object SpecificUnitAutomation {
         val capitalTile = unit.civ.getCapital()!!.getCenterTile()
         if (unit.movement.canReach(capitalTile))
             unit.movement.headTowards(capitalTile)
+        if (unit.movement.canUnitSwapTo(capitalTile))
+            unit.movement.swapMoveToTile(capitalTile)
         if (unit.getTile() == capitalTile) {
             UnitActions.invokeUnitAction(unit, UnitActionType.AddInCapital)
         }
