@@ -10,26 +10,118 @@ import com.unciv.logic.battle.ICombatant
 import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.battle.TargetHelper
 import com.unciv.logic.city.City
+import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.civilization.MapUnitAction
 import com.unciv.logic.civilization.NotificationCategory
+import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UpgradeUnitAction
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsPillage
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
+import yairm210.purity.annotations.Readonly
 
 object UnitAutomation {
 
     private const val CLOSE_ENEMY_TILES_AWAY_LIMIT = 5
     private const val CLOSE_ENEMY_TURNS_AWAY_LIMIT = 3f
 
-    private fun isGoodTileToExplore(unit: MapUnit, tile: Tile): Boolean {
-        return (tile.getOwner() == null || !tile.getOwner()!!.isCityState())
-                && tile.neighbors.any { !unit.civ.hasExplored(it) }
-                && (!unit.civ.isCityState() || tile.neighbors.any { it.getOwner() == unit.civ }) // Don't want city-states exploring far outside their borders
+    fun automateUnitMoves(unit: MapUnit) {
+        check(!unit.civ.isBarbarian) { "Barbarians is not allowed here." }
+
+        // Might die next turn - move!
+        if (unit.getDamageFromTerrain() > 0 && tryHealUnit(unit)) return
+
+
+        if (unit.isCivilian()) {
+            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
+            return
+        }
+
+        // AI upgrades units via UseGoldAutomation in NextTurnAutomation
+        if (unit.civ.isHuman() && tryUpgradeUnit(unit)) return
+
+        //This allows for military units with certain civilian abilities to behave as civilians in peace and soldiers in war
+        if ((unit.hasUnique(UniqueType.BuildImprovements) || unit.hasUnique(UniqueType.FoundCity) || 
+                    unit.hasUnique(UniqueType.FoundPuppetCity) ||
+                    unit.hasUnique(UniqueType.ReligiousUnit) || unit.hasUnique(UniqueType.CreateWaterImprovements))
+            && !unit.civ.isAtWar()){
+            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
+            return
+        }
+
+        // Note that not all nukes have to be air units
+        if (unit.isNuclearWeapon()) {
+            return AirUnitAutomation.automateNukes(unit)
+        }
+
+        if (unit.baseUnit.isAirUnit()) {
+            if (unit.canIntercept())
+                return AirUnitAutomation.automateFighter(unit)
+
+            if (unit.hasUnique(UniqueType.SelfDestructs))
+                return AirUnitAutomation.automateMissile(unit)
+
+            return AirUnitAutomation.automateBomber(unit)
+        }
+
+        // Accompany settlers
+        if (tryAccompanySettlerOrGreatPerson(unit)) return
+
+        if (tryGoToRuin(unit) && !unit.hasMovement()) return
+
+        if (unit.health < 50 && (tryRetreat(unit) || tryHealUnit(unit))) return // do nothing but heal
+
+        // If there are no enemies nearby and we can heal here, wait until we are at full health
+        if (unit.health < 100 && canUnitHealInTurnsOnCurrentTile(unit,2, 3)) return
+
+        if (tryHeadTowardsOurSiegedCity(unit)) return
+
+        // if a embarked melee unit can land and attack next turn, do not attack from water.
+        if (BattleHelper.tryDisembarkUnitToAttackPosition(unit)) return
+
+        // if there is an attackable unit in the vicinity, attack!
+        if (tryAttacking(unit)) return
+
+        if (tryTakeBackCapturedCity(unit)) return
+
+        // Focus all units without a specific target on the enemy city closest to one of our cities
+        if (HeadTowardsEnemyCityAutomation.tryHeadTowardsEnemyCity(unit)) return
+
+        if (tryHeadTowardsEncampment(unit)) return
+
+        if (tryGarrisoningLandUnit(unit)) return
+
+        if (unit.health < 80 && tryHealUnit(unit)) return
+
+        // move towards the closest reasonably attackable enemy unit within 3 turns of movement (and 5 tiles range)
+        if (tryAdvanceTowardsCloseEnemy(unit)) return
+
+        if (unit.health < 100 && tryHealUnit(unit)) return
+
+        if (tryPrepare(unit)) return
+
+        // else, try to go to unreached tiles
+        if (tryExplore(unit)) return
+
+        if (tryFogBust(unit)) return
+
+        // Idle CS units should wander so they don't obstruct players so much
+        if (unit.civ.isCityState)
+            wander(unit, stayInTerritory = true)
+    }
+
+
+    @Readonly
+    private fun isGoodTileToExplore(unit: MapUnit, tile: Tile, unitVisibilityRange: Int): Boolean {
+        // These should be ordered by increasing computational cost
+        return (tile.getOwner() == null || !tile.getOwner()!!.isCityState)
+                && tile.getTilesInDistance(unitVisibilityRange).any { !unit.civ.hasExplored(it) }
+                && (!unit.civ.isCityState || tile.neighbors.any { it.getOwner() == unit.civ }) // Don't want city-states exploring far outside their borders
                 && unit.getDamageFromTerrain(tile) <= 0    // Don't take unnecessary damage
                 && unit.civ.threatManager.getDistanceToClosestEnemyUnit(tile, 3) > 3 // don't walk in range of enemy units
                 && unit.movement.canMoveTo(tile) // expensive, evaluate last
@@ -37,39 +129,39 @@ object UnitAutomation {
     }
 
     internal fun tryExplore(unit: MapUnit): Boolean {
-        if (tryGoToRuinAndEncampment(unit) && (unit.currentMovement == 0f || unit.isDestroyed)) return true
+        if (tryGoToRuin(unit) && (!unit.hasMovement() || unit.isDestroyed)) return true
 
+        val unitVisibilityRange = unit.getVisibilityRange()
         val explorableTilesThisTurn =
-                unit.movement.getDistanceToTiles().keys.filter { isGoodTileToExplore(unit, it) }
+                unit.movement.getDistanceToTiles().keys.filter { isGoodTileToExplore(unit, it, unitVisibilityRange) }
         if (explorableTilesThisTurn.any()) {
             val bestTile = explorableTilesThisTurn
-                .sortedByDescending { it.tileHeight }  // secondary sort is by 'how far can you see'
-                .maxByOrNull { it.aerialDistanceTo(unit.currentTile) }!! // primary sort is by 'how far can you go'
+                .maxBy { it.tileHeight + it.getTilesAtDistance(unit.getVisibilityRange()).count { tile -> !tile.isExplored(unit.civ) }}
+            // Assign each tile a score for "explore value"
+            // This could be more elaborate: for example add a malus for distant tiles such as to move not too far away from capital (barb control)
+            // or bonus according to tile yields (likely candidates for city locations), but this comes at a cost of performance
             unit.movement.headTowards(bestTile)
             return true
         }
 
         // Nothing immediate, lets look further. Number increases exponentially with distance - at 10 this took a looong time
         for (tile in unit.currentTile.getTilesInDistance(5))
-            if (isGoodTileToExplore(unit, tile)) {
+            if (isGoodTileToExplore(unit, tile, unitVisibilityRange)) {
                 unit.movement.headTowards(tile)
                 return true
             }
         return false
     }
 
-    private fun tryGoToRuinAndEncampment(unit: MapUnit): Boolean {
+    private fun tryGoToRuin(unit: MapUnit): Boolean {
         if (!unit.civ.isMajorCiv()) return false // barbs don't have anything to do in ruins
 
-        val tileWithRuinOrEncampment = unit.viewableTiles
+        val tileWithRuin = unit.viewableTiles
             .firstOrNull {
-                (
-                        (it.improvement != null && it.getTileImprovement()!!.isAncientRuinsEquivalent())
-                                || it.improvement == Constants.barbarianEncampment
-                        )
+                (it.getTileImprovement()?.isAncientRuinsEquivalent(unit.cache.state) == true)
                         && unit.movement.canMoveTo(it) && unit.movement.canReach(it)
             } ?: return false
-        unit.movement.headTowards(tileWithRuinOrEncampment)
+        unit.movement.headTowards(tileWithRuin)
         return true
     }
 
@@ -99,6 +191,7 @@ object UnitAutomation {
         return false
     }
 
+    @Readonly
     private fun isGoodTileForFogBusting(unit: MapUnit, tile: Tile): Boolean {
         return unit.movement.canMoveTo(tile)
                 && tile.getOwner() == null
@@ -110,20 +203,19 @@ object UnitAutomation {
     }
 
     fun wander(unit: MapUnit, stayInTerritory: Boolean = false, tilesToAvoid: Set<Tile> = setOf()) {
-        val unitDistanceToTiles = unit.movement.getDistanceToTiles()
+        if (!unit.hasMovement()) return // return in case we can't move anyways
+        val unitDistanceToTiles = unit.currentTile.getTilesAtDistance(1)
+        // We could walk further, but wander() is meant to let units not stay on the same tile permanently,
+        // to avoid obstructing human scouts and workers, moving just one tile should be enough
         val reachableTiles = unitDistanceToTiles
                 .filter {
-                    it.key !in tilesToAvoid
-                    && unit.movement.canMoveTo(it.key)
-                    && unit.movement.canReach(it.key)
+                    it !in tilesToAvoid
+                        && unit.movement.canMoveTo(it)
+                        && unit.movement.canReachInCurrentTurn(it) // Yes this is required despite the above line - see #14461
+                        && unit.getDamageFromTerrain(it) <= 0 // Don't end turn on damaging terrain for no good reason
+                        && (!stayInTerritory || it.getOwner() == unit.civ || unit.currentTile.getOwner() != unit.civ)
                 }
-
-        val reachableTilesMaxWalkingDistance = reachableTiles
-                .filter { it.value.totalDistance == unit.currentMovement
-                        && unit.getDamageFromTerrain(it.key) <= 0 // Don't end turn on damaging terrain for no good reason
-                        && (!stayInTerritory || it.key.getOwner() == unit.civ) }
-        if (reachableTilesMaxWalkingDistance.any()) unit.movement.moveToTile(reachableTilesMaxWalkingDistance.toList().random().first)
-        else if (reachableTiles.any()) unit.movement.moveToTile(reachableTiles.keys.random())
+        if (reachableTiles.any()) unit.movement.moveToTile(reachableTiles.toList().random())
     }
 
     internal fun tryUpgradeUnit(unit: MapUnit): Boolean {
@@ -134,7 +226,7 @@ object UnitAutomation {
         if (upgradeUnits.none()) return false // for resource reasons, usually
         val upgradedUnit = upgradeUnits.minBy { it.cost }
 
-        if (upgradedUnit.getResourceRequirementsPerTurn(StateForConditionals(unit.civ, unit = unit)).keys.any { !unit.requiresResource(it) }) {
+        if (upgradedUnit.getResourceRequirementsPerTurn(unit.cache.state).keys.any { !unit.requiresResource(it) }) {
             // The upgrade requires new resource types, so check if we are willing to invest them
             if (!Automation.allowSpendingResource(unit.civ, upgradedUnit)) return false
         }
@@ -149,114 +241,24 @@ object UnitAutomation {
     /** Get the base unit this map unit could upgrade to, respecting researched tech and nation uniques only.
      *  Note that if the unit can't upgrade, the current BaseUnit is returned.
      */
+    @Readonly
     private fun getUnitsToUpgradeTo(unit: MapUnit): Sequence<BaseUnit> {
 
+        @Readonly
         fun isInvalidUpgradeDestination(baseUnit: BaseUnit): Boolean {
             if (!unit.civ.tech.isResearched(baseUnit))
                 return true
-            return baseUnit.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
-                .any { !it.conditionalsApply(StateForConditionals(unit.civ, unit = unit)) }
+            if (unit.civ.isBarbarian && baseUnit.hasUnique(UniqueType.CannotBeBarbarian))
+                return true
+            return baseUnit.getMatchingUniques(UniqueType.OnlyAvailable, GameContext.IgnoreConditionals)
+                .any { !it.conditionalsApply(unit.cache.state) }
         }
 
-        return unit.baseUnit.getRulesetUpgradeUnits(StateForConditionals(unit.civ, unit = unit))
+        return unit.baseUnit.getRulesetUpgradeUnits(unit.cache.state)
             .map { unit.civ.getEquivalentUnit(it) }
             .filter { !isInvalidUpgradeDestination(it) && unit.upgrade.canUpgrade(it) }
     }
 
-    fun automateUnitMoves(unit: MapUnit) {
-        check(!unit.civ.isBarbarian()) { "Barbarians is not allowed here." }
-
-        // Might die next turn - move!
-        if (unit.health <= unit.getDamageFromTerrain() && tryHealUnit(unit)) return
-
-
-        if (unit.isCivilian()) {
-            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
-            return
-        }
-
-        while (unit.promotions.canBePromoted() &&
-            // Restrict Human automated units from promotions via setting
-                (UncivGame.Current.settings.automatedUnitsChoosePromotions || unit.civ.isAI())) {
-            val availablePromotions = unit.promotions.getAvailablePromotions()
-                .filterNot { it.hasUnique(UniqueType.SkipPromotion) }
-            if (availablePromotions.none()) break
-            unit.promotions.addPromotion(
-                availablePromotions.filter { it.hasUnique(UniqueType.FreePromotion) }.toList().randomOrNull()?.name
-                    ?: availablePromotions.toList().random().name)
-        }
-
-        //This allows for military units with certain civilian abilities to behave as civilians in peace and soldiers in war
-        if ((unit.hasUnique(UniqueType.BuildImprovements) || unit.hasUnique(UniqueType.FoundCity) ||
-                unit.hasUnique(UniqueType.ReligiousUnit) || unit.hasUnique(UniqueType.CreateWaterImprovements))
-                && !unit.civ.isAtWar()){
-            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
-            return
-        }
-
-        // Note that not all nukes have to be air units
-        if (unit.isNuclearWeapon()) {
-            return AirUnitAutomation.automateNukes(unit)
-        }
-
-        if (unit.baseUnit.isAirUnit()) {
-            if (unit.canIntercept())
-                return AirUnitAutomation.automateFighter(unit)
-
-            if (unit.hasUnique(UniqueType.SelfDestructs))
-                return AirUnitAutomation.automateMissile(unit)
-
-            return AirUnitAutomation.automateBomber(unit)
-        }
-
-        // Accompany settlers
-        if (tryAccompanySettlerOrGreatPerson(unit)) return
-
-        if (tryGoToRuinAndEncampment(unit) && unit.currentMovement == 0f) return
-
-        if (tryUpgradeUnit(unit)) return
-
-        if (unit.health < 50 && (tryRetreat(unit) || tryHealUnit(unit))) return // do nothing but heal
-
-        // If there are no enemies nearby and we can heal here, wait until we are at full health
-        if (unit.health < 100 && canUnitHealInTurnsOnCurrentTile(unit,2, 4)) return
-
-        if (tryHeadTowardsOurSiegedCity(unit)) return
-
-        // if a embarked melee unit can land and attack next turn, do not attack from water.
-        if (BattleHelper.tryDisembarkUnitToAttackPosition(unit)) return
-
-        // if there is an attackable unit in the vicinity, attack!
-        if (tryAttacking(unit)) return
-
-        if (tryTakeBackCapturedCity(unit)) return
-
-        // Focus all units without a specific target on the enemy city closest to one of our cities
-        if (HeadTowardsEnemyCityAutomation.tryHeadTowardsEnemyCity(unit)) return
-
-        if (tryGarrisoningRangedLandUnit(unit)) return
-
-        if (tryStationingMeleeNavalUnit(unit)) return
-
-        if (unit.health < 80 && tryHealUnit(unit)) return
-
-        // move towards the closest reasonably attackable enemy unit within 3 turns of movement (and 5 tiles range)
-        if (tryAdvanceTowardsCloseEnemy(unit)) return
-
-        if (tryHeadTowardsEncampment(unit)) return
-
-        if (unit.health < 100 && tryHealUnit(unit)) return
-
-        // else, try to go to unreached tiles
-        if (tryExplore(unit)) return
-
-        if (tryFogBust(unit)) return
-
-        // Idle CS units should wander so they don't obstruct players so much
-        if (unit.civ.isCityState())
-            wander(unit, stayInTerritory = true)
-    }
-
 
     /** @return true only if the unit has 0 movement left */
     private fun tryAttacking(unit: MapUnit): Boolean {
@@ -270,11 +272,12 @@ object UnitAutomation {
 
     private fun tryHeadTowardsEncampment(unit: MapUnit): Boolean {
         if (unit.hasUnique(UniqueType.SelfDestructs)) return false // don't use single-use units against barbarians...
-        val knownEncampments = unit.civ.gameInfo.tileMap.values.asSequence()
-                .filter { it.improvement == Constants.barbarianEncampment && unit.civ.hasExplored(it) }
         val cities = unit.civ.cities
-        val encampmentsCloseToCities = knownEncampments
-            .filter { cities.any { city -> city.getCenterTile().aerialDistanceTo(it) < 6 } }
+        val knownEncampments = cities.asSequence()
+            .flatMap { it.getCenterTile().getTilesInDistance(6) }
+                .filter { it.improvement == Constants.barbarianEncampment && unit.civ.hasExplored(it) }
+            .distinct()
+        val encampmentsCloseToCities = knownEncampments.asSequence()
             .sortedBy { it.aerialDistanceTo(unit.currentTile) }
         val encampmentToHeadTowards = encampmentsCloseToCities.firstOrNull { unit.movement.canReach(it) }
             ?: return false
@@ -283,12 +286,11 @@ object UnitAutomation {
     }
 
     private fun tryRetreat(unit: MapUnit): Boolean {
-        if (!unit.civ.isAtWar()) return false
         // Precondition: This must be a military unit
         if (unit.isCivilian()) return false
         if (unit.baseUnit.isAirUnit()) return false
         // Better to do a more healing oriented move then
-        if (unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),6, true) > 4) return false
+        if (unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),4, true) > 3) return false
 
 
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
@@ -305,32 +307,34 @@ object UnitAutomation {
             unitDistanceToTiles.asSequence().map { it.key }.sortedByDescending { unit.civ.threatManager.getDistanceToClosestEnemyUnit(it, 3, false) }
         }
 
-        val ourDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),6, false)
+        val ourDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),4, false)
         // Lets check all tiles and swap with the first one
         for (retreatTile in sortedTilesToRetreatTo) {
-            val tileDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(retreatTile,6,false)
+            val tileDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(retreatTile,4,false)
             if (ourDistanceToClosestEnemy >= tileDistanceToClosestEnemy) continue
 
             val otherUnit = retreatTile.militaryUnit
             if (otherUnit == null) {
                 // See if we can retreat to the tile
-                if (unit.movement.canMoveTo(retreatTile)) {
-                    unit.movement.moveToTile(retreatTile)
-                    return true
-                }
+                if (!unit.movement.canMoveTo(retreatTile)) continue
+                unit.movement.moveToTile(retreatTile)
+                return true
             } else if (otherUnit.civ == unit.civ) {
                 // The tile is taken, lets see if we want to swap retreat to it
-                if (otherUnit.health > 80) {
-                    if (otherUnit.baseUnit.isRanged()) {
-                        // Don't swap ranged units closer than they have to be
-                        val range = otherUnit.baseUnit.range
-                        if (ourDistanceToClosestEnemy < range)
-                            continue
-                    }
-                    if (unit.movement.canUnitSwapTo(retreatTile)) {
+                if (otherUnit.health <= 80) continue
+                if (otherUnit.baseUnit.isRanged()) {
+                    // Don't swap ranged units closer than they have to be
+                    val range = otherUnit.baseUnit.range
+                    if (ourDistanceToClosestEnemy < range)
+                        continue
+                }
+                if (unit.movement.canUnitSwapTo(retreatTile)) {
+                    unit.movement.headTowards(retreatTile) // we need to move through the intermediate tiles
+                    // if nothing changed
+                    if (unit.currentTile.neighbors.contains(otherUnit.currentTile) && unit.movement.canUnitSwapTo(retreatTile)) {
                         unit.movement.swapMoveToTile(retreatTile)
-                        return true
                     }
+                    return true
                 }
             }
         }
@@ -342,27 +346,29 @@ object UnitAutomation {
             return false // will heal anyway, and attacks don't hurt
 
         // Try pillage improvements until healed
-        while(tryPillageImprovement(unit, false)) {
+        while (tryPillageImprovement(unit, false)) {
             // If we are fully healed and can still do things, lets keep on going by returning false
-            if (unit.currentMovement == 0f || unit.health == 100) return unit.currentMovement == 0f
+            if (!unit.hasMovement() || unit.health == 100) return !unit.hasMovement()
         }
 
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         if (unitDistanceToTiles.isEmpty()) return true // can't move, so...
 
+        val dangerousTiles = unit.civ.threatManager.getDangerousTiles(unit, 3)
+        
         // If the unit can heal on this tile in two turns, just heal here
-        if (canUnitHealInTurnsOnCurrentTile(unit,3,)) return true
+        if (unit.currentTile !in dangerousTiles 
+            && canUnitHealInTurnsOnCurrentTile(unit,3)) return true
 
         val currentUnitTile = unit.getTile()
 
-        val dangerousTiles = unit.civ.threatManager.getDangerousTiles(unit, 4)
 
         val viableTilesForHealing = unitDistanceToTiles.keys
                 .filter { it !in dangerousTiles && unit.movement.canMoveTo(it) }
         val tilesByHealingRate = viableTilesForHealing.groupBy { unit.rankTileForHealing(it) }
 
         if (tilesByHealingRate.keys.all { it == 0 }) { // We can't heal here at all! We're probably embarked
-            if (!unit.baseUnit.movesLikeAirUnits()) {
+            if (!unit.baseUnit.movesLikeAirUnits) {
                 val reachableCityTile = unit.civ.cities.asSequence()
                     .map { it.getCenterTile() }
                     .sortedBy { it.aerialDistanceTo(unit.currentTile) }
@@ -390,7 +396,7 @@ object UnitAutomation {
         }
 
         val bestTilesForHealing = tilesByHealingRate.maxByOrNull { it.key }!!.value
-        val bestTileForHealing = bestTilesForHealing.maxByOrNull { it.getDefensiveBonus() }!!
+        val bestTileForHealing = bestTilesForHealing.maxByOrNull { it.getDefensiveBonus(unit = unit) }!!
         val bestTileForHealingRank = unit.rankTileForHealing(bestTileForHealing)
 
         if (currentUnitTile != bestTileForHealing
@@ -404,6 +410,7 @@ object UnitAutomation {
     /**
      * @return true if the tile is safe and the unit can heal to full within [turns]
      */
+    @Readonly
     private fun canUnitHealInTurnsOnCurrentTile(unit: MapUnit, turns: Int, noEnemyDistance: Int = 3): Boolean {
         if (unit.hasUnique(UniqueType.HealsEvenAfterAction)) return false // We can keep on moving
         // Check if we are not in a safe city and there is an enemy nearby this isn't a good tile to heal on
@@ -414,6 +421,7 @@ object UnitAutomation {
         return healthRequiredPerTurn <= unit.rankTileForHealing(unit.getTile())
     }
 
+    @Readonly
     private fun getDangerousTiles(unit: MapUnit): HashSet<Tile> {
         val nearbyEnemyUnits = unit.currentTile.getTilesInDistance(3)
             .flatMap { tile -> tile.getUnits().filter { unit.civ.isAtWarWith(it.civ) } }
@@ -423,7 +431,7 @@ object UnitAutomation {
 
         val tilesWithinBombardmentRange = unit.currentTile.getTilesInDistance(3)
             .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
-            .flatMap { it.getTilesInDistance(it.getCity()!!.range) }
+            .flatMap { it.getTilesInDistance(it.getCity()!!.getBombardRange()) }
 
         val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(3)
             .filter { unit.getDamageFromTerrain(it) > 0 }
@@ -438,13 +446,13 @@ object UnitAutomation {
         if (unit.isCivilian()) return false
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         val tilesThatCanWalkToAndThenPillage = unitDistanceToTiles
-            .filter { it.value.totalDistance < unit.currentMovement }.keys
+            .filter { it.value.totalMovement < unit.currentMovement }.keys
             .filter { unit.movement.canMoveTo(it) && UnitActionsPillage.canPillage(unit, it)
                     && (it.canPillageTileImprovement()
                     || (!onlyPillageToHeal && it.canPillageRoad() && it.getRoadOwner() != null && unit.civ.isAtWarWith(it.getRoadOwner()!!))) }
 
         if (tilesThatCanWalkToAndThenPillage.isEmpty()) return false
-        val tileToPillage = tilesThatCanWalkToAndThenPillage.maxByOrNull { it.getDefensiveBonus(false) }!!
+        val tileToPillage = tilesThatCanWalkToAndThenPillage.maxByOrNull { it.getDefensiveBonus(false, unit) }!!
         if (unit.getTile() != tileToPillage)
             unit.movement.moveToTile(tileToPillage)
 
@@ -462,32 +470,56 @@ object UnitAutomation {
      *  Tiles attack from which would result in instant death of the [unit] are ignored. */
     private fun tryAdvanceTowardsCloseEnemy(unit: MapUnit): Boolean {
         // this can be sped up if we check each layer separately
-        val unitDistanceToTiles = unit.movement.getDistanceToTilesWithinTurn(
+        val unitDistanceToTiles = unit.movement.getMovementToTilesAtPosition(
                 unit.getTile().position,
                 unit.getMaxMovement() * CLOSE_ENEMY_TURNS_AWAY_LIMIT
         )
-        var closeEnemies = TargetHelper.getAttackableEnemies(
-            unit,
-            unitDistanceToTiles,
-            tilesToCheck = unit.getTile().getTilesInDistance(CLOSE_ENEMY_TILES_AWAY_LIMIT).toList()
-        ).filter {
-            // Ignore units that would 1-shot you if you attacked. Account for taking terrain damage after the fact.
-            BattleDamage.calculateDamageToAttacker(
-                MapUnitCombatant(unit),
-                Battle.getMapCombatantOfTile(it.tileToAttack)!!
-            )
-                    + unit.getDamageFromTerrain(it.tileToAttackFrom) < unit.health
-        }
-
-        if (unit.baseUnit.isRanged())
-            closeEnemies = closeEnemies.filterNot { it.tileToAttack.isCityCenter() && it.tileToAttack.getCity()!!.health == 1 }
-
-        val closestEnemy = closeEnemies
-            .filter { unit.getDamageFromTerrain(it.tileToAttackFrom) <= 0 }  // Don't attack from a mountain
+        val closeEnemy = TargetHelper.getAttackableEnemies(unit, unitDistanceToTiles, tilesToCheck = unit.getTile().getTilesInDistance(CLOSE_ENEMY_TILES_AWAY_LIMIT).toList())
+            .filterNot { unit.baseUnit.isRanged() && it.tileToAttack.isCityCenter() && it.tileToAttack.getCity()!!.health == 1 } // occurs fairly often probably because AI dumb
+            .filter { unit.getDamageFromTerrain(it.tileToAttackFrom) <= 0 }  // Don't attack from a mountain or near enemy citadels
+            .filter {
+            // Ignore units that would 1-shot you if you attacked
+            BattleDamage.calculateDamageToAttacker(MapUnitCombatant(unit), Battle.getMapCombatantOfTile(it.tileToAttack)!!) < unit.health }
             .minByOrNull { it.tileToAttack.aerialDistanceTo(unit.getTile()) }
 
-        if (closestEnemy != null) {
-            unit.movement.headTowards(closestEnemy.tileToAttackFrom)
+        if (closeEnemy != null) {
+            unit.movement.headTowards(closeEnemy.tileToAttackFrom)
+            return true
+        }
+        return false
+    }
+
+    /**
+     * Tries to move the unit to the closest city that is close to a target civilization
+     */
+    private fun tryPrepare(unit: MapUnit): Boolean {
+        val civInfo = unit.civ
+
+        @Readonly
+        fun hasPreparationFlag(targetCiv: Civilization): Boolean {
+            val diploManager = civInfo.getDiplomacyManager(targetCiv)!!
+            if (diploManager.hasFlag(DiplomacyFlags.Denunciation)
+                    || diploManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.Denunciation)) return true
+            if (diploManager.hasFlag(DiplomacyFlags.WaryOf) && diploManager.getFlag(DiplomacyFlags.WaryOf) < 0) return true
+            return false
+        }
+
+        val hostileCivs = civInfo.getKnownCivs().filter { it.isAtWarWith(civInfo) || hasPreparationFlag(it) }.toSet()
+        val closeCities = civInfo.threatManager.getNeighboringCitiesOfOtherCivs().filter { it.second.civ in hostileCivs }
+        val closestDistance = closeCities.minOfOrNull { it.first.getCenterTile().aerialDistanceTo(it.second.getCenterTile()) }
+            ?: return false
+        val citiesToDefend = closeCities
+            .filter { it.first.getCenterTile().aerialDistanceTo(it.second.getCenterTile()) <= closestDistance + 2 }
+            .map { it.first }
+            .distinct() // Remove duplicate cities
+            .sortedBy { unit.getTile().aerialDistanceTo(it.getCenterTile()) }
+
+        // Move to the closest city with a tile we can enter nearby
+        for (city in citiesToDefend) {
+            if (unit.getTile().aerialDistanceTo(city.getCenterTile()) <= 2) return true
+            val tileToMoveTo = city.getCenterTile().getTilesInDistance(2)
+                .firstOrNull { unit.movement.canMoveTo(it) && unit.movement.canReach(it) } ?: continue
+            unit.movement.headTowards(tileToMoveTo)
             return true
         }
         return false
@@ -500,6 +532,7 @@ object UnitAutomation {
                 val tile = it.currentTile
                 it.isCivilian() &&
                         (it.hasUnique(UniqueType.FoundCity) || unit.isGreatPerson())
+                        && !it.hasUnique(UniqueType.StrengthBonusInRadius) // Exlude great generals, as they move independently after all military units
                         && (tile == unit.currentTile || tile.militaryUnit == null && unit.movement.canMoveTo(tile))
                         && distanceToTiles.containsKey(tile)
             } ?: return false
@@ -511,9 +544,8 @@ object UnitAutomation {
         val siegedCities = unit.civ.cities
                 .asSequence()
                 .filter {
-                    unit.civ == it.civ &&
-                            it.health < it.getMaxHealth() * 0.75
-                } //Weird health issues and making sure that not all forces move to good defenses
+                    unit.civ == it.civ && it.health < it.getMaxHealth()
+                }
 
         if (siegedCities.any { it.getCenterTile().aerialDistanceTo(unit.getTile()) <= 2 })
             return false
@@ -527,7 +559,7 @@ object UnitAutomation {
         if (reachableTileNearSiegedCity != null) {
             unit.movement.headTowards(reachableTileNearSiegedCity)
         }
-        return unit.currentMovement == 0f
+        return !unit.hasMovement()
     }
 
 
@@ -550,29 +582,13 @@ object UnitAutomation {
         return true
     }
 
+    @Readonly
     private fun chooseBombardTarget(city: City): ICombatant? {
-        var targets = TargetHelper.getBombardableTiles(city).map { Battle.getMapCombatantOfTile(it)!! }
-            .filterNot { it.isCivilian() && !it.getUnitType().hasUnique(UniqueType.Uncapturable) } // Don't bombard capturable civilians
+        val targets = TargetHelper.getBombardableTiles(city).map { Battle.getMapCombatantOfTile(it)!! }
+            .filterNot { it is MapUnitCombatant && it.isCivilian() }
+            // Don't bombard civilians (they're more efficiently captured/killed by military units if they're indeed this close)
         if (targets.none()) return null
-
-        val siegeUnits = targets
-                .filter { it is MapUnitCombatant && it.unit.baseUnit.isProbablySiegeUnit() }
-        val nonEmbarkedSiege = siegeUnits.filter { it is MapUnitCombatant && !it.unit.isEmbarked() }
-        if (nonEmbarkedSiege.any()) targets = nonEmbarkedSiege
-        else if (siegeUnits.any()) targets = siegeUnits
-        else {
-            val rangedUnits = targets
-                    .filter { it.isRanged() }
-            if (rangedUnits.any()) targets = rangedUnits
-        }
-
-        val hitsToKill = targets.associateWith { it.getHealth().toFloat() / BattleDamage.calculateDamageToDefender(
-            CityCombatant(city),
-            it
-        ).toFloat().coerceAtLeast(1f) }
-        val target = hitsToKill.filter { it.value <= 1 }.maxByOrNull { it.key.getAttackingStrength() }?.key
-        if (target != null) return target
-        return hitsToKill.minByOrNull { it.value }?.key
+        return targets.maxByOrNull { BattleDamage.calculateDamageToDefender(CityCombatant(city), it) }
     }
 
     private fun tryTakeBackCapturedCity(unit: MapUnit): Boolean {
@@ -580,7 +596,7 @@ object UnitAutomation {
                 .flatMap { it.cities.asSequence() }
                 .filter {
                     unit.civ.isAtWarWith(it.civ) &&
-                            unit.civ.civName == it.foundingCiv &&
+                            unit.civ == it.foundingCivObject &&
                             it.isInResistance() &&
                             it.health < it.getMaxHealth()
                 } //Most likely just been captured
@@ -605,15 +621,14 @@ object UnitAutomation {
         return false
     }
 
-    private fun tryGarrisoningRangedLandUnit(unit: MapUnit): Boolean {
-        if (unit.baseUnit.isMelee() || unit.baseUnit.isWaterUnit()) return false // don't garrison melee units, they're not that good at it
+    private fun tryGarrisoningLandUnit(unit: MapUnit): Boolean {
+        if (unit.baseUnit.isWaterUnit) return false // Water units don't count for Garrison bonus
         val citiesWithoutGarrison = unit.civ.cities.filter {
             val centerTile = it.getCenterTile()
             centerTile.militaryUnit == null
                     && unit.movement.canMoveTo(centerTile)
         }
 
-
         val citiesToTry = if (!unit.civ.isAtWar()) {
             if (unit.getTile().isCityCenter()) return true // It's always good to have a unit in the city center, so if you haven't found anyone around to attack, forget it.
             citiesWithoutGarrison.asSequence()
@@ -633,52 +648,23 @@ object UnitAutomation {
         unit.movement.headTowards(closestReachableCityNeedsDefending.getCenterTile())
         return true
     }
-
+    @Readonly
     private fun isCityThatNeedsDefendingInWartime(city: City): Boolean {
         if (city.health < city.getMaxHealth()) return true // this city is under attack!
         for (enemyCivCity in city.civ.diplomacy.values
             .filter { it.diplomaticStatus == DiplomaticStatus.War }
-            .map { it.otherCiv() }.flatMap { it.cities })
+            .map { it.otherCiv }.flatMap { it.cities })
             if (city.getCenterTile().aerialDistanceTo(enemyCivCity.getCenterTile()) <= 5) return true // this is an edge city that needs defending
         return false
     }
 
-    private fun tryStationingMeleeNavalUnit(unit: MapUnit): Boolean {
-        fun isMeleeNaval(mapUnit: MapUnit) = mapUnit.baseUnit.isMelee() && mapUnit.type.isWaterUnit()
-
-        if (!isMeleeNaval(unit)) return false
-        val closeCity = unit.getTile().getTilesInDistance(3)
-            .firstOrNull { it.isCityCenter() }
-
-        // We're the closest unit to this city, we should stay here :)
-        if (closeCity != null && closeCity.getTilesInDistance(3)
-                .flatMap { it.getUnits() }
-                .firstOrNull {isMeleeNaval(it)} == unit
-            && unit.movement.canReach(closeCity)) {
-            unit.movement.headTowards(closeCity)
-            return true
-        }
-
-        val citiesWithoutNavalDefence = unit.civ.cities.filter { it.isCoastal() }
-            .filter { it.getCenterTile().aerialDistanceTo(unit.getTile()) < 20 } // Not too far away
-            .filter { it.getCenterTile().getTilesInDistance(3)
-                .flatMap { it.getUnits() }
-                .none { isMeleeNaval(it) }}
-
-        val reachableCity = citiesWithoutNavalDefence.firstOrNull {
-            unit.movement.canReach(it.getCenterTile())
-        } ?: return false
-        unit.movement.headTowards(reachableCity.getCenterTile())
-        return true
-    }
-
     /** This is what a unit with the 'explore' action does.
     It also explores, but also has other functions, like healing if necessary. */
     fun automatedExplore(unit: MapUnit) {
-        if (tryGoToRuinAndEncampment(unit) && (unit.currentMovement == 0f || unit.isDestroyed)) return
+        if (tryGoToRuin(unit) && (!unit.hasMovement() || unit.isDestroyed)) return
         if (unit.health < 80 && tryHealUnit(unit)) return
         if (tryExplore(unit)) return
-        unit.civ.addNotification("${unit.shortDisplayName()} finished exploring.", unit.currentTile.position, NotificationCategory.Units, unit.name, "OtherIcons/Sleep")
+        unit.civ.addNotification("${unit.shortDisplayName()} finished exploring.", MapUnitAction(unit), NotificationCategory.Units, unit.name, "OtherIcons/Sleep")
         unit.action = null
     }
 
