@@ -5,9 +5,12 @@ import com.unciv.logic.city.City
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.mapunit.movement.PathsToTilesWithinTurn
 import com.unciv.logic.map.tile.Tile
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
+import yairm210.purity.annotations.Readonly
 
 object TargetHelper {
+    @Readonly
     fun getAttackableEnemies(
         unit: MapUnit,
         unitDistanceToTiles: PathsToTilesWithinTurn,
@@ -18,7 +21,7 @@ object TargetHelper {
         val attackableTiles = ArrayList<AttackableTile>()
 
         val unitMustBeSetUp = unit.hasUnique(UniqueType.MustSetUp)
-        val tilesToAttackFrom = if (stayOnTile || unit.baseUnit.movesLikeAirUnits())
+        val tilesToAttackFrom = if (stayOnTile || unit.baseUnit.movesLikeAirUnits)
             sequenceOf(Pair(unit.currentTile, unit.currentMovement))
         else getTilesToAttackFromWhenUnitMoves(unitDistanceToTiles, unitMustBeSetUp, unit)
 
@@ -30,13 +33,16 @@ object TargetHelper {
             if (unit.baseUnit.isMelee() && unit.isEscorting()) {
                 val escortingUnit = unit.getOtherEscortUnit()!!
                 if (!escortingUnit.movement.canReachInCurrentTurn(reachableTile)
-                    || escortingUnit.currentMovement - escortingUnit.movement.getDistanceToTiles()[reachableTile]!!.totalDistance <= 0f) 
+                    || escortingUnit.currentMovement - escortingUnit.movement.getDistanceToTiles()[reachableTile]!!.totalMovement <= 0f) 
                     continue
             }
+
             val tilesInAttackRange =
-                if (unit.hasUnique(UniqueType.IndirectFire) || unit.baseUnit.movesLikeAirUnits())
+                if (unit.baseUnit.isMelee()) reachableTile.neighbors
+                else if (unit.baseUnit.movesLikeAirUnits || unit.hasUnique(UniqueType.IndirectFire, checkCivInfoUniques = true))
                     reachableTile.getTilesInDistance(rangeOfAttack)
                 else reachableTile.tileMap.getViewableTiles(reachableTile.position, rangeOfAttack, true).asSequence()
+
             for (tile in tilesInAttackRange) {
                 when {
                     // Since military units can technically enter tiles with enemy civilians,
@@ -64,6 +70,7 @@ object TargetHelper {
         return attackableTiles
     }
 
+    @Readonly
     private fun getTilesToAttackFromWhenUnitMoves(unitDistanceToTiles: PathsToTilesWithinTurn, unitMustBeSetUp: Boolean, unit: MapUnit) =
         unitDistanceToTiles.asSequence()
             .map { (tile, distance) ->
@@ -73,7 +80,7 @@ object TargetHelper {
                 val movementPointsToExpendBeforeAttack =
                     if (tile == unit.currentTile) movementPointsToExpendHere else movementPointsToExpendAfterMovement
                 val movementLeft =
-                    unit.currentMovement - distance.totalDistance - movementPointsToExpendBeforeAttack
+                    unit.currentMovement - distance.totalMovement - movementPointsToExpendBeforeAttack
                 Pair(tile, movementLeft)
             }
             // still got leftover movement points after all that, to attack
@@ -82,6 +89,7 @@ object TargetHelper {
                 it.first == unit.getTile() || unit.movement.canMoveTo(it.first)
             }
 
+    @Readonly
     private fun tileContainsAttackableEnemy(unit: MapUnit, tile: Tile, tilesToCheck: List<Tile>?): Boolean {
         if (tile !in (tilesToCheck ?: unit.civ.viewableTiles) || !containsAttackableEnemy(tile, MapUnitCombatant(unit)) )
             return false
@@ -90,6 +98,7 @@ object TargetHelper {
         return (!unit.baseUnit.isMelee() || mapCombatant !is MapUnitCombatant || !mapCombatant.unit.isCivilian() || unit.movement.canPassThrough(tile))
     }
 
+    @Readonly
     fun containsAttackableEnemy(tile: Tile, combatant: ICombatant): Boolean {
         if (combatant is MapUnitCombatant && combatant.unit.isEmbarked() && !combatant.hasUnique(UniqueType.AttackOnSea)) {
             // Can't attack water units while embarked, only land
@@ -109,22 +118,27 @@ object TargetHelper {
         )
             return false
 
-        if (combatant is MapUnitCombatant && combatant.hasUnique(UniqueType.CannotAttack))
-            return false
+        
+        if (combatant is MapUnitCombatant) {
+            val gameContext = GameContext(
+                unit = (combatant as? MapUnitCombatant)?.unit, tile = tile, 
+                ourCombatant = combatant, theirCombatant = tileCombatant, combatAction = CombatAction.Attack)
 
-        if (combatant is MapUnitCombatant &&
-            combatant.unit.getMatchingUniques(UniqueType.CanOnlyAttackUnits).run {
-                any() && none { tileCombatant.matchesFilter(it.params[0]) }
-            }
-        )
-            return false
+            if (combatant.hasUnique(UniqueType.CannotAttack, gameContext))
+                return false
 
-        if (combatant is MapUnitCombatant &&
-            combatant.unit.getMatchingUniques(UniqueType.CanOnlyAttackTiles).run {
-                any() && none { tile.matchesFilter(it.params[0]) }
-            }
-        )
-            return false
+            if (combatant.unit.getMatchingUniques(UniqueType.CanOnlyAttackUnits, gameContext).run {
+                    any() && none { tileCombatant.matchesFilter(it.params[0]) }
+                }
+            )
+                return false
+
+            if (combatant.unit.getMatchingUniques(UniqueType.CanOnlyAttackTiles, gameContext).run {
+                    any() && none { tile.matchesFilter(it.params[0]) }
+                }
+            )
+                return false
+        }
 
         // Only units with the right unique can view submarines (or other invisible units) from more then one tile away.
         // Garrisoned invisible units can be attacked by anyone, as else the city will be in invincible.
@@ -132,12 +146,14 @@ object TargetHelper {
             return combatant is MapUnitCombatant
                 && combatant.getCivInfo().viewableInvisibleUnitsTiles.map { it.position }.contains(tile.position)
         }
+        
         return true
     }
 
     /** Get a list of visible tiles which have something attackable */
+    @Readonly
     fun getBombardableTiles(city: City): Sequence<Tile> =
-            city.getCenterTile().getTilesInDistance(city.range)
+            city.getCenterTile().getTilesInDistance(city.getBombardRange())
                     .filter { it.isVisible(city.civ) && containsAttackableEnemy(it, CityCombatant(city)) }
 
 }
