@@ -1,6 +1,5 @@
 package com.unciv.logic.city
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.GUI
 import com.unciv.logic.IsPartOfGameInfoSerialization
@@ -12,16 +11,27 @@ import com.unciv.logic.city.managers.CityPopulationManager
 import com.unciv.logic.city.managers.CityReligionManager
 import com.unciv.logic.city.managers.SpyFleeReason
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.civilization.transients.CapitalConnectionsFinder.CapitalConnectionMedium
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.TileMap
 import com.unciv.logic.map.mapunit.MapUnit
+import com.unciv.logic.map.mapunit.UnitPromotions
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
+import com.unciv.models.Counter
 import com.unciv.models.ruleset.Building
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.tile.TileResource
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
+import com.unciv.models.stats.GameResource
+import com.unciv.models.stats.INamed
 import com.unciv.models.stats.Stat
+import com.unciv.models.stats.SubStat
+import com.unciv.utils.withoutItem
+import yairm210.purity.annotations.Readonly
+import java.util.EnumSet
 import java.util.UUID
 import kotlin.math.roundToInt
 
@@ -32,30 +42,30 @@ enum class CityFlags {
 }
 
 
-class City : IsPartOfGameInfoSerialization {
+class City : IsPartOfGameInfoSerialization, INamed {
     @Transient
     lateinit var civ: Civilization
 
     @Transient
     private lateinit var centerTile: Tile  // cached for better performance
 
-    @Transient
-    val range = 2
-
     @Transient
     lateinit var tileMap: TileMap
 
     @Transient
     lateinit var tilesInRange: HashSet<Tile>
+    
+    @Transient var state = GameContext.EmptyState
 
     @Transient
     // This is so that military units can enter the city, even before we decide what to do with it
     var hasJustBeenConquered = false
 
-    var location: Vector2 = Vector2.Zero
+    var location = HexCoord()
     var id: String = UUID.randomUUID().toString()
-    var name: String = ""
-    var foundingCiv = ""
+    override var name: String = ""
+    /** Serialization field for [foundingCivObject]. Is equivalent to `foundingCivObject.civName` */
+    private var foundingCiv = ""
     // This is so that cities in resistance that are recaptured aren't in resistance anymore
     var previousOwner = ""
     var turnAcquired = 0
@@ -70,31 +80,55 @@ class City : IsPartOfGameInfoSerialization {
 
     @Transient  // CityStats has no serializable fields
     var cityStats = CityStats(this)
+    
+    var resourceStockpiles = Counter<String>()
 
     /** All tiles that this city controls */
-    var tiles = HashSet<Vector2>()
+    var tiles = HashSet<HexCoord>()
 
     /** Tiles that have population assigned to them */
-    var workedTiles = HashSet<Vector2>()
+    var workedTiles = HashSet<HexCoord>()
 
     /** Tiles that the population in them won't be reassigned */
-    var lockedTiles = HashSet<Vector2>()
+    var lockedTiles = HashSet<HexCoord>()
     var manualSpecialists = false
     var isBeingRazed = false
     var attackedThisTurn = false
     var hasSoldBuildingThisTurn = false
     var isPuppet = false
-    var updateCitizens = false  // flag so that on startTurn() the Governor reassigns Citizens
+    var shouldReassignPopulation = false  // flag so that on startTurn() we reassign population
+    
+    var unitShouldUseSavedPromotion = HashMap<String, Boolean>()
+    
+    var unitToPromotions = HashMap<String, UnitPromotions>()
 
+    /** Neighboring explored cities, in radius of 12 tiles */
     @delegate:Transient
     val neighboringCities: List<City> by lazy { 
-        civ.gameInfo.getCities().filter { it != this && it.getCenterTile().aerialDistanceTo(getCenterTile()) <= 8 }.toList()
+        civ.gameInfo.getCities().filter { it != this && it.getCenterTile().isExplored(civ) && it.getCenterTile().aerialDistanceTo(getCenterTile()) <= 12 }.toList()
     }
 
-    var cityAIFocus: String = CityFocus.NoFocus.name
-    fun getCityFocus() = CityFocus.values().firstOrNull { it.name == cityAIFocus } ?: CityFocus.NoFocus
+    private var cityAIFocus: String = CityFocus.NoFocus.name
+    @Readonly fun getCityFocus() = CityFocus.entries.firstOrNull { it.name == cityAIFocus } ?: CityFocus.NoFocus
     fun setCityFocus(cityFocus: CityFocus){ cityAIFocus = cityFocus.name }
 
+    /**
+     * Civ object for the original founder of this city
+     * 
+     * Setting this also sets its backing serialization string ``foundingCiv``
+     * */
+    @Transient
+    @get:Readonly
+    var foundingCivObject: Civilization? = null
+        get() {
+            if (field == null && foundingCiv.isNotEmpty()) field = civ.gameInfo.getCivilization(foundingCiv)
+            return field
+        }
+        set(value) {
+            field = value
+            foundingCiv = value?.civID ?: ""
+        }
+
 
 
     var avoidGrowth: Boolean = false
@@ -111,13 +145,9 @@ class City : IsPartOfGameInfoSerialization {
     internal var flagsCountdown = HashMap<String, Int>()
 
     /** Persisted connected-to-capital (by any medium) to allow "disconnected" notifications after loading */
-    // Unknown only exists to support older saves, so those do not generate spurious connected/disconnected messages.
-    // The other names are chosen so serialization is compatible with a Boolean to allow easy replacement in the future.
-    @Suppress("EnumEntryName")
-    enum class ConnectedToCapitalStatus { Unknown, `false`, `true` }
-    var connectedToCapitalStatus = ConnectedToCapitalStatus.Unknown
+    var connectedToCapitalStatus = false
 
-    fun hasDiplomaticMarriage(): Boolean = foundingCiv == ""
+    @Readonly fun hasDiplomaticMarriage(): Boolean = foundingCivObject == null
 
     //region pure functions
     fun clone(): City {
@@ -133,6 +163,7 @@ class City : IsPartOfGameInfoSerialization {
         toReturn.tiles = tiles
         toReturn.workedTiles = workedTiles
         toReturn.lockedTiles = lockedTiles
+        toReturn.resourceStockpiles = resourceStockpiles.clone()
         toReturn.isBeingRazed = isBeingRazed
         toReturn.attackedThisTurn = attackedThisTurn
         toReturn.foundingCiv = foundingCiv
@@ -141,47 +172,55 @@ class City : IsPartOfGameInfoSerialization {
         toReturn.isOriginalCapital = isOriginalCapital
         toReturn.flagsCountdown.putAll(flagsCountdown)
         toReturn.demandedResource = demandedResource
-        toReturn.updateCitizens = updateCitizens
+        toReturn.shouldReassignPopulation = shouldReassignPopulation
         toReturn.cityAIFocus = cityAIFocus
         toReturn.avoidGrowth = avoidGrowth
         toReturn.manualSpecialists = manualSpecialists
         toReturn.connectedToCapitalStatus = connectedToCapitalStatus
+        toReturn.unitShouldUseSavedPromotion = unitShouldUseSavedPromotion
+        toReturn.unitToPromotions = unitToPromotions
         return toReturn
     }
 
-    fun canBombard() = !attackedThisTurn && !isInResistance()
-    fun getCenterTile(): Tile = centerTile
-    fun getTiles(): Sequence<Tile> = tiles.asSequence().map { tileMap[it] }
-    fun getWorkableTiles() = tilesInRange.asSequence().filter { it.getOwner() == civ }
-    fun isWorked(tile: Tile) = workedTiles.contains(tile.position)
+    @Readonly fun canBombard() = !attackedThisTurn && !isInResistance()
+    @Readonly fun getCenterTile(): Tile = centerTile
+    @Readonly fun getCenterTileOrNull(): Tile? = if (::centerTile.isInitialized) centerTile else null
+    @Readonly fun getTiles(): Sequence<Tile> = tiles.asSequence().map { tileMap[it] }
+    @Readonly fun getWorkableTiles() = tilesInRange.asSequence().filter { it.getOwner() == civ }
+    @Readonly fun getWorkedTiles(): Sequence<Tile> = workedTiles.asSequence().map { tileMap[it] }
+    @Readonly fun isWorked(tile: Tile) = workedTiles.contains(tile.position)
 
-    fun isCapital(): Boolean = cityConstructions.getBuiltBuildings().any { it.hasUnique(UniqueType.IndicatesCapital) }
-    fun isCoastal(): Boolean = centerTile.isCoastalTile()
+    @Readonly fun isCapital(): Boolean = cityConstructions.builtBuildingUniqueMap.hasUnique(UniqueType.IndicatesCapital, state)
+    @Readonly fun isCoastal(): Boolean = centerTile.isCoastalTile()
+    @Readonly fun isNaval(): Boolean = centerTile.isWater || isCoastal()
+    
+    @Readonly fun getBombardRange(): Int = civ.gameInfo.ruleset.modOptions.constants.baseCityBombardRange
+    @Readonly fun getWorkRange(): Int = civ.gameInfo.ruleset.modOptions.constants.cityWorkRange
+    @Readonly fun getExpandRange(): Int = civ.gameInfo.ruleset.modOptions.constants.cityExpandRange
 
-    fun capitalCityIndicator(): Building? {
-        val indicatorBuildings = getRuleset().buildings.values.asSequence()
-            .filter { it.hasUnique(UniqueType.IndicatesCapital) }
-
-        val civSpecificBuilding = indicatorBuildings.firstOrNull { it.uniqueTo == civ.civName }
-        return civSpecificBuilding ?: indicatorBuildings.firstOrNull()
-    }
-
-    fun isConnectedToCapital(connectionTypePredicate: (Set<String>) -> Boolean = { true }): Boolean {
+    @Readonly
+    fun isConnectedToCapital() = this in civ.cache.citiesConnectedToCapitalToMediums
+    @Readonly
+    fun isConnectedToCapital(
+        @Readonly connectionTypePredicate: (EnumSet<CapitalConnectionMedium>) -> Boolean
+    ): Boolean {
         val mediumTypes = civ.cache.citiesConnectedToCapitalToMediums[this] ?: return false
         return connectionTypePredicate(mediumTypes)
     }
 
-    fun isGarrisoned() = getGarrison() != null
+    @Readonly fun isGarrisoned() = getGarrison() != null
+    @Readonly
     fun getGarrison(): MapUnit? =
             getCenterTile().militaryUnit?.takeIf {
                 it.civ == this.civ && it.canGarrison()
             }
 
-    fun hasFlag(flag: CityFlags) = flagsCountdown.containsKey(flag.name)
-    fun getFlag(flag: CityFlags) = flagsCountdown[flag.name]!!
+    @Readonly fun hasFlag(flag: CityFlags) = flagsCountdown.containsKey(flag.name)
+    @Readonly fun getFlag(flag: CityFlags) = flagsCountdown[flag.name]!!
 
-    fun isWeLoveTheKingDayActive() = hasFlag(CityFlags.WeLoveTheKing)
-    fun isInResistance() = hasFlag(CityFlags.Resistance)
+    @Readonly fun isWeLoveTheKingDayActive() = hasFlag(CityFlags.WeLoveTheKing)
+    @Readonly fun isInResistance() = hasFlag(CityFlags.Resistance)
+    @Readonly
     fun isBlockaded(): Boolean {
         // Coastal cities are blocked if every adjacent water tile is blocked
         if (!isCoastal()) return false
@@ -190,22 +229,51 @@ class City : IsPartOfGameInfoSerialization {
         }
     }
 
-    fun getRuleset() = civ.gameInfo.ruleset
+    @Readonly fun getRuleset() = civ.gameInfo.ruleset
 
-    fun getResourcesGeneratedByCity() = CityResources.getResourcesGeneratedByCity(this)
-    fun getAvailableResourceAmount(resourceName: String) = CityResources.getAvailableResourceAmount(this, resourceName)
+    @Readonly fun getResourcesGeneratedByCity(resourceModifier: (TileResource) -> Float = ::getResourceModifier) = CityResources.getResourcesGeneratedByCity(this, resourceModifier)
+    @Readonly fun getAvailableResourceAmount(resourceName: String) = CityResources.getAvailableResourceAmount(this, resourceName)
+    @Readonly fun getAvailableResourceAmount(resource: TileResource) = CityResources.getAvailableResourceAmount(this, resource)
 
-    fun isGrowing() = foodForNextTurn() > 0
-    fun isStarving() = foodForNextTurn() < 0
+    /**
+     * Returns the resource production modifier as a multiplier.
+     *
+     * For example: 1.0f means no change, 2.0f results in double production.
+     *
+     * @param resource The resource for which to calculate the modifier.
+     * @return The production modifier as a multiplier.
+     */
+    @Readonly
+    fun getResourceModifier(resource: TileResource): Float {
+        var finalModifier = 1f
 
-    fun foodForNextTurn() = cityStats.currentCityStats.food.roundToInt()
+        for (unique in getMatchingUniques(UniqueType.PercentResourceProduction))
+            if (resource.matchesFilter(unique.params[1], state))
+                finalModifier += unique.params[0].toFloat() / 100f
 
+        return finalModifier
+    }
+    /** Gets modifiers for ALL resources */
+    @Readonly
+    fun getResourceModifiers(): Map<String, Float> =
+        civ.gameInfo.ruleset.tileResources.values.associate { it.name to getResourceModifier(it) }
 
-    fun containsBuildingUnique(uniqueType: UniqueType, state: StateForConditionals = StateForConditionals(this)) =
+    @Readonly fun isGrowing() = foodForNextTurn() > 0
+    @Readonly fun isStarving() = foodForNextTurn() < 0
+
+    @Readonly fun foodForNextTurn() = cityStats.currentCityStats.food.roundToInt()
+
+    @Readonly
+    fun containsBuildingUnique(uniqueType: UniqueType, state: GameContext = this.state) =
         cityConstructions.builtBuildingUniqueMap.getMatchingUniques(uniqueType, state).any()
 
-    fun getGreatPersonPercentageBonus() = GreatPersonPointsBreakdown.getGreatPersonPercentageBonus(this)
-    fun getGreatPersonPoints() = GreatPersonPointsBreakdown(this).sum()
+    @Readonly fun getGreatPersonPercentageBonus() = GreatPersonPointsBreakdown.getGreatPersonPercentageBonus(this)
+    @Readonly fun getGreatPersonPoints() = GreatPersonPointsBreakdown(this).sum()
+
+    fun gainStockpiledResource(resource: TileResource, amount: Int) {
+        if (resource.isCityWide) resourceStockpiles.add(resource.name, amount)
+        else civ.resourceStockpiles.add(resource.name, amount)
+    }
 
     fun addStat(stat: Stat, amount: Int) {
         when (stat) {
@@ -215,6 +283,20 @@ class City : IsPartOfGameInfoSerialization {
         }
     }
 
+    fun addGameResource(stat: GameResource, amount: Int) {
+        if (stat is TileResource) {
+            if (!stat.isStockpiled) return
+            gainStockpiledResource(stat, amount)
+            return
+        }
+        when (stat) {
+            Stat.Production -> cityConstructions.addProductionPoints(amount)
+            Stat.Food, SubStat.StoredFood -> population.foodStored += amount
+            else -> civ.addGameResource(stat, amount)
+        }
+    }
+    
+    @Readonly
     fun getStatReserve(stat: Stat): Int {
         return when (stat) {
             Stat.Production -> cityConstructions.getWorkDone(cityConstructions.getCurrentConstruction().name)
@@ -223,6 +305,7 @@ class City : IsPartOfGameInfoSerialization {
         }
     }
 
+    @Readonly
     fun hasStatToBuy(stat: Stat, price: Int): Boolean {
         return when {
             civ.gameInfo.gameParameters.godMode -> true
@@ -231,16 +314,22 @@ class City : IsPartOfGameInfoSerialization {
         }
     }
 
-    internal fun getMaxHealth() =
-        200 + cityConstructions.getBuiltBuildings().sumOf { it.cityHealth }
+    @Readonly internal fun getMaxHealth() = 200 + cityConstructions.getBuiltBuildings().sumOf { it.cityHealth }
 
-    fun getStrength() = cityConstructions.getBuiltBuildings().sumOf { it.cityStrength }.toFloat() 
+    @Readonly fun getStrength() = cityConstructions.getBuiltBuildings().sumOf { it.cityStrength }.toFloat()
+
+    /** Gets max air units that can remain in the city untransported */
+    @Readonly fun getMaxAirUnits(): Int = civ.gameInfo.ruleset.modOptions.constants.cityAirUnitCapacity +
+        getMatchingUniques(UniqueType.CarryExtraAirUnits)
+            .filter { it.params[1] == "Air" }
+            .sumOf { it.params[0].toInt() }
 
     override fun toString() = name // for debug
 
-    fun isHolyCity(): Boolean = religion.religionThisIsTheHolyCityOf != null && !religion.isBlockedHolyCity
-    fun isHolyCityOf(religionName: String?) = isHolyCity() && religion.religionThisIsTheHolyCityOf == religionName
+    @Readonly fun isHolyCity(): Boolean = religion.religionThisIsTheHolyCityOf != null && !religion.isBlockedHolyCity
+    @Readonly fun isHolyCityOf(religionName: String?) = isHolyCity() && religion.religionThisIsTheHolyCityOf == religionName
 
+    @Readonly
     fun canBeDestroyed(justCaptured: Boolean = false): Boolean {
         if (civ.gameInfo.gameParameters.noCityRazing) return false
 
@@ -261,7 +350,8 @@ class City : IsPartOfGameInfoSerialization {
         this.civ = civInfo
         tileMap = civInfo.gameInfo.tileMap
         centerTile = tileMap[location]
-        tilesInRange = getCenterTile().getTilesInDistance(3).toHashSet()
+        state = GameContext(this)
+        tilesInRange = getCenterTile().getTilesInDistance(getWorkRange()).toHashSet()
         population.city = this
         expansion.city = this
         expansion.setTransients()
@@ -307,19 +397,19 @@ class City : IsPartOfGameInfoSerialization {
         }
         if (!manualSpecialists)
             population.specialistAllocations.clear()
-        updateCitizens = false
+        shouldReassignPopulation = false
         population.autoAssignPopulation()
     }
 
     /** Apply worked tiles optimization (aka CityFocus) -
      *  immediately for a human player whoes turn it is (interactive),
      *  or deferred to the next startTurn while nextTurn is running (for AI)
-     *  @see reassignPopulation
+     *  @see shouldReassignPopulation
      */
     fun reassignPopulationDeferred() {
         // TODO - is this the best (or even correct) way to detect "interactive" UI calls?
         if (GUI.isMyTurn() && GUI.getViewingPlayer() == civ) reassignPopulation()
-        else updateCitizens = true
+        else shouldReassignPopulation = true
     }
 
     fun destroyCity(overrideSafeties: Boolean = false) {
@@ -340,8 +430,10 @@ class City : IsPartOfGameInfoSerialization {
         // Must be before removing existing capital because we may be annexing a puppet which means city stats update - see #8337
         if (isCapital()) civ.moveCapitalToNextLargest(null)
 
-        civ.cities = civ.cities.toMutableList().apply { remove(this@City) }
-        getCenterTile().changeImprovement("City ruins")
+        civ.cities = civ.cities.withoutItem(this)
+
+        if (getRuleset().tileImprovements.containsKey("City ruins"))
+            getCenterTile().setImprovement("City ruins")
 
         // Edge case! What if a water unit is in a city, and you raze the city?
         // Well, the water unit has to return to the water!
@@ -361,8 +453,6 @@ class City : IsPartOfGameInfoSerialization {
             civ.updateProximity(otherCiv,
                 otherCiv.updateProximity(civ))
         }
-
-        civ.gameInfo.cityDistances.setDirty()
     }
 
     fun annexCity() = CityConquestFunctions(this).annexCity()
@@ -379,17 +469,15 @@ class City : IsPartOfGameInfoSerialization {
         CityConquestFunctions(this).moveToCiv(newCivInfo)
 
     internal fun tryUpdateRoadStatus() {
-        if (getCenterTile().roadStatus == RoadStatus.None) {
-            val roadImprovement = getRuleset().roadImprovement
-            if (roadImprovement != null && roadImprovement.techRequired in civ.tech.techsResearched)
-                getCenterTile().roadStatus = RoadStatus.Road
-        } else if (getCenterTile().roadStatus != RoadStatus.Railroad) {
-            val railroadImprovement = getRuleset().railroadImprovement
-            if (railroadImprovement != null && railroadImprovement.techRequired in civ.tech.techsResearched)
-                getCenterTile().roadStatus = RoadStatus.Railroad
+        val requiredRoad = when{
+            getRuleset().railroadImprovement?.let { it.techRequired == null || it.techRequired in civ.tech.techsResearched } == true -> RoadStatus.Railroad
+            getRuleset().roadImprovement?.let { it.techRequired == null || it.techRequired in civ.tech.techsResearched } == true -> RoadStatus.Road
+            else -> RoadStatus.None
         }
+        getCenterTile().setRoadStatus(requiredRoad, civ)
     }
 
+    @Readonly
     fun getGoldForSellingBuilding(buildingName: String) =
         getRuleset().buildings[buildingName]!!.cost / 10
 
@@ -403,25 +491,29 @@ class City : IsPartOfGameInfoSerialization {
         hasSoldBuildingThisTurn = true
 
         population.unassignExtraPopulation() // If the building provided specialists, release them to other work
-        population.autoAssignPopulation()
-        cityStats.update()
+        population.autoAssignPopulation() // also updates city stats
         civ.cache.updateCivResources() // this building could be a resource-requiring one
     }
-
+    
+    @Readonly
     fun canPlaceNewUnit(construction: BaseUnit): Boolean {
         val tile = getCenterTile()
         return when {
             construction.isCivilian() -> tile.civilianUnit == null
-            construction.movesLikeAirUnits() -> tile.airUnits.count { !it.isTransported } < 6
+            construction.movesLikeAirUnits -> return true // Dealt with in MapUnit.getRejectionReasons
             else -> tile.militaryUnit == null
         }
     }
 
     /** Implements [UniqueParameterType.CityFilter][com.unciv.models.ruleset.unique.UniqueParameterType.CityFilter] */
-    fun matchesFilter(filter: String, viewingCiv: Civilization? = civ): Boolean {
-        return MultiFilter.multiFilter(filter, { matchesSingleFilter(it, viewingCiv) })
+    @Readonly
+    fun matchesFilter(filter: String, viewingCiv: Civilization? = civ, multiFilter: Boolean = true): Boolean {
+        return if (multiFilter)
+            MultiFilter.multiFilter(filter, { matchesSingleFilter(it, viewingCiv) })
+        else matchesSingleFilter(filter, viewingCiv)
     }
 
+    @Readonly
     private fun matchesSingleFilter(filter: String, viewingCiv: Civilization? = civ): Boolean {
         return when (filter) {
             "in this city" -> true // Filtered by the way uniques are found
@@ -446,18 +538,18 @@ class City : IsPartOfGameInfoSerialization {
                 && !civ.isAtWarWith(viewingCiv)
             "in enemy cities", "Enemy" -> civ.isAtWarWith(viewingCiv ?: civ)
             "in foreign cities", "Foreign" -> viewingCiv != null && viewingCiv != civ
-            "in annexed cities", "Annexed" -> foundingCiv != civ.civName && !isPuppet
+            "in annexed cities", "Annexed" -> foundingCivObject != civ && !isPuppet
             "in puppeted cities", "Puppeted" -> isPuppet
             "in resisting cities", "Resisting" -> isInResistance()
             "in cities being razed", "Razing" -> isBeingRazed
             "in holy cities", "Holy" -> isHolyCity()
-            "in City-State cities" -> civ.isCityState()
+            "in City-State cities" -> civ.isCityState
             // This is only used in communication to the user indicating that only in cities with this
             // religion a unique is active. However, since religion uniques only come from the city itself,
             // this will always be true when checked.
             "in cities following this religion" -> true
             "in cities following our religion" -> viewingCiv?.religionManager?.religion == religion.getMajorityReligion()
-            else -> civ.matchesFilter(filter)
+            else -> civ.matchesFilter(filter, state, false)
         }
     }
 
@@ -470,39 +562,70 @@ class City : IsPartOfGameInfoSerialization {
     // Sadly, due to the large disparity between use cases, there needed to be lots of functions.
 
     // Finds matching uniques provided from both local and non-local sources.
+    @Readonly
     fun getMatchingUniques(
         uniqueType: UniqueType,
-        stateForConditionals: StateForConditionals = StateForConditionals(this),
+        gameContext: GameContext = state,
         includeCivUniques: Boolean = true
     ): Sequence<Unique> {
         return if (includeCivUniques)
-            civ.getMatchingUniques(uniqueType, stateForConditionals) +
-                getLocalMatchingUniques(uniqueType, stateForConditionals)
+            civ.getMatchingUniques(uniqueType, gameContext) +
+                getLocalMatchingUniques(uniqueType, gameContext)
         else (
             cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType)
-                + religion.getUniques().filter { it.type == uniqueType }
+                + religion.getUniques(uniqueType)
             ).filter {
-                !it.isTimedTriggerable && it.conditionalsApply(stateForConditionals)
-            }.flatMap { it.getMultiplied(stateForConditionals) }
+                !it.isTimedTriggerable && it.conditionalsApply(gameContext)
+            }.flatMap { it.getMultiplied(gameContext) }
     }
 
     // Uniques special to this city
-    fun getLocalMatchingUniques(uniqueType: UniqueType, stateForConditionals: StateForConditionals = StateForConditionals(this)): Sequence<Unique> {
+    @Readonly
+    fun getLocalMatchingUniques(uniqueType: UniqueType, gameContext: GameContext = state): Sequence<Unique> {
         val uniques = cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType).filter { it.isLocalEffect } +
-            religion.getUniques().filter { it.type == uniqueType }
-        return if (uniques.any()) uniques.filter { !it.isTimedTriggerable && it.conditionalsApply(stateForConditionals) }
-            .flatMap { it.getMultiplied(stateForConditionals) }
-        else uniques
+            religion.getUniques(uniqueType)
+        return uniques.filter { !it.isTimedTriggerable && it.conditionalsApply(gameContext) }
+                .flatMap { it.getMultiplied(gameContext) }
     }
 
     // Uniques coming from this city, but that should be provided globally
-    fun getMatchingUniquesWithNonLocalEffects(uniqueType: UniqueType, stateForConditionals: StateForConditionals): Sequence<Unique> {
+    @Readonly
+    fun getMatchingUniquesWithNonLocalEffects(uniqueType: UniqueType, gameContext: GameContext = state): Sequence<Unique> {
         val uniques = cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType)
         // Memory performance showed that this function was very memory intensive, thus we only create the filter if needed
         return if (uniques.any()) uniques.filter { !it.isLocalEffect && !it.isTimedTriggerable
-            && it.conditionalsApply(stateForConditionals) }.flatMap { it.getMultiplied(stateForConditionals) }
+            && it.conditionalsApply(gameContext) }.flatMap { it.getMultiplied(gameContext) }
         else uniques
     }
 
+    @Readonly
+    fun getTriggeredUniques(
+        trigger: UniqueType,
+        gameContext: GameContext = state,
+        triggerFilter: (Unique) -> Boolean = { true },
+        includeCivUniques: Boolean = true): Sequence<Unique> {
+        if (includeCivUniques) {
+            return civ.getTriggeredUniques(trigger, gameContext, triggerFilter).asSequence() +
+                getLocalTriggeredUniques(trigger, gameContext, triggerFilter)
+        }
+        else {
+            val uniques =
+                cityConstructions.builtBuildingUniqueMap.getAllUniques() + religion.getAllUniques()
+            return uniques.filter {
+                it.getModifiers(trigger).any(triggerFilter) && it.conditionalsApply(gameContext)
+            }.flatMap { it.getMultiplied(gameContext) }
+        }
+    }
+
+    @Readonly
+    fun getLocalTriggeredUniques(trigger: UniqueType, gameContext: GameContext = state,
+        triggerFilter: (Unique) -> Boolean = { true }): Sequence<Unique> {
+        val uniques =
+            cityConstructions.builtBuildingUniqueMap.getAllUniques().filter { it.isLocalEffect } + religion.getAllUniques()
+        return uniques.filter {
+            it.getModifiers(trigger).any(triggerFilter) && it.conditionalsApply(gameContext)
+        }.flatMap { it.getMultiplied(gameContext) }
+    }
+
     //endregion
 }
