@@ -4,10 +4,9 @@ import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.city.managers.CityPopulationManager
 import com.unciv.models.stats.Stat
-import com.unciv.models.stats.Stats
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.screens.cityscreen.CitizenManagementTable
-import com.unciv.ui.images.ImageGetter
+import yairm210.purity.annotations.Pure
 
 /**
  *  Controls automatic worker-to-tile assignment
@@ -41,13 +40,15 @@ enum class CityFocus(
     FaithFocus("${Stat.Faith.character}", true, Stat.Faith),
     GoldGrowthFocus("${Stat.Gold.character} ${Stat.Food.character}", true) {
         override fun getStatMultiplier(stat: Stat) = when (stat) {
-            Stat.Gold, Stat.Food -> 2f
+            Stat.Gold -> 2f
+            Stat.Food -> 1.5f
             else -> 1f
         }
     },
     ProductionGrowthFocus("${Stat.Production.character} ${Stat.Food.character}", true) {
         override fun getStatMultiplier(stat: Stat) = when (stat) {
-            Stat.Production, Stat.Food -> 2f
+            Stat.Production -> 2f
+            Stat.Food -> 1.5f
             else -> 1f
         }
     },
@@ -58,32 +59,27 @@ enum class CityFocus(
 
     val binding: KeyboardBinding =
         binding ?:
-        KeyboardBinding.values().firstOrNull { it.name == name } ?:
+        KeyboardBinding.entries.firstOrNull { it.name == name } ?:
         KeyboardBinding.None
 
+    @Pure
     open fun getStatMultiplier(stat: Stat) = when (this.stat) {
-        stat -> 3f
+        stat -> 3.05f // on ties, prefer the Focus
         else -> 1f
     }
 
-    fun applyWeightTo(stats: Stats) {
-        for (stat in Stat.values()) {
-            stats[stat] *= getStatMultiplier(stat)
-        }
+    val statValuesForFocus: List<Stat> by lazy {
+        Stat.entries.filter { getStatMultiplier(it) != 1f }
     }
 
     companion object {
-        fun safeValueOf(stat: Stat): CityFocus {
-            return values().firstOrNull { it.stat == stat } ?: NoFocus
-        }
+        @Pure fun safeValueOf(stat: Stat): CityFocus = entries.firstOrNull { it.stat == stat } ?: NoFocus
 
         // set used in Automation. All non-Food Focuses, so targets 0 Surplus Food
-        fun zeroFoodFocuses(): Set<CityFocus> {
-            return setOf(
-                CultureFocus, FaithFocus, GoldFocus,
-                HappinessFocus, ProductionFocus, ScienceFocus
-            )
-        }
+        val zeroFoodFocuses = setOf(
+            CultureFocus, FaithFocus, GoldFocus,
+            HappinessFocus, ProductionFocus, ScienceFocus
+        )
     }
 }
 
