@@ -6,7 +6,6 @@ import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.IConstruction
 import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.models.ruleset.unique.LocalUniqueCache
-import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
@@ -16,9 +15,13 @@ import com.unciv.models.stats.StatMap
 import com.unciv.models.stats.Stats
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.utils.DebugUtils
+import yairm210.purity.annotations.InternalState
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 import kotlin.math.min
 
-
+@InternalState
 class StatTreeNode {
     val children = LinkedHashMap<String, StatTreeNode>()
     private var innerStats: Stats? = null
@@ -33,7 +36,9 @@ class StatTreeNode {
         else innerStats!!.add(stats) // What happens if we add 2 stats to the same leaf?
     }
 
-    fun addStats(newStats: Stats, vararg hierarchyList: String) {
+    fun addStats(newStats: Stats?, vararg hierarchyList: String) {
+        if (newStats == null) return
+        if (newStats.isEmpty()) return
         if (hierarchyList.isEmpty()) {
             addInnerStats(newStats)
             return
@@ -55,7 +60,7 @@ class StatTreeNode {
     fun clone() : StatTreeNode {
         val new = StatTreeNode()
         new.innerStats = this.innerStats?.clone()
-        new.children.putAll(this.children)
+        new.children.putAll(this.children.mapValues { it.value.clone() })
         return new
     }
 
@@ -92,12 +97,12 @@ class CityStats(val city: City) {
     //endregion
     //region Pure Functions
 
-
+    @Readonly
     private fun getStatsFromTradeRoute(): Stats {
         val stats = Stats()
-        if (!city.isCapital() && city.isConnectedToCapital()) {
-            val civInfo = city.civ
-            stats.gold = civInfo.getCapital()!!.population.population * 0.15f + city.population.population * 1.1f - 1 // Calculated by http://civilization.wikia.com/wiki/Trade_route_(Civ5)
+        val capitalForTradeRoutePurposes = city.civ.getCapital()!!
+        if (city != capitalForTradeRoutePurposes && city.isConnectedToCapital()) {
+            stats.gold = capitalForTradeRoutePurposes.population.population * 0.15f + city.population.population * 1.1f - 1 // Calculated by http://civilization.wikia.com/wiki/Trade_route_(Civ5)
             for (unique in city.getMatchingUniques(UniqueType.StatsFromTradeRoute))
                 stats.add(unique.stats)
             val percentageStats = Stats()
@@ -110,16 +115,18 @@ class CityStats(val city: City) {
         return stats
     }
 
-    private fun getStatsFromProduction(production: Float): Stats {
-        val stats = Stats()
-
-        if (city.cityConstructions.currentConstructionFromQueue in Stat.statsWithCivWideField.map { it.name }) {
-            val stat = Stat.valueOf(city.cityConstructions.currentConstructionFromQueue)
+    @Readonly
+    private fun getStatsFromProduction(production: Float): Stats? {
+        if (city.cityConstructions.currentConstructionName() in Stat.statsWithCivWideField.map { it.name }) {
+            val stats = Stats()
+            val stat = Stat.valueOf(city.cityConstructions.currentConstructionName())
             stats[stat] = production * getStatConversionRate(stat)
+            return stats
         }
-        return stats
+        return null
     }
 
+    @Readonly
     fun getStatConversionRate(stat: Stat): Float {
         var conversionRate = 1 / 4f
         val conversionUnique = city.civ.getMatchingUniques(UniqueType.ProductionToCivWideStatConversionBonus).firstOrNull { it.params[0] == stat.name }
@@ -129,39 +136,30 @@ class CityStats(val city: City) {
         return conversionRate
     }
 
-    private fun getStatPercentBonusesFromRailroad(): Stats {
-        val stats = Stats()
+    @Readonly
+    private fun getStatPercentBonusesFromRailroad(): Stats? {
         val railroadImprovement = city.getRuleset().railroadImprovement
-            ?: return stats // for mods
+            ?: return null // for mods
         val techEnablingRailroad = railroadImprovement.techRequired
         // If we conquered enemy cities connected by railroad, but we don't yet have that tech,
         // we shouldn't get bonuses, it's as if the tracks are laid out but we can't operate them.
         if ( (techEnablingRailroad == null || city.civ.tech.isResearched(techEnablingRailroad))
                 && (city.isCapital() || isConnectedToCapital(RoadStatus.Railroad)))
-            stats.production += 25f
-        return stats
+            return Stats(production = 25f)
+        return null
     }
 
-    private fun addStatPercentBonusesFromBuildings(statPercentBonusTree: StatTreeNode) {
-        val localUniqueCache = LocalUniqueCache()
-        for (building in city.cityConstructions.getBuiltBuildings())
-            statPercentBonusTree.addStats(building.getStatPercentageBonuses(city, localUniqueCache), "Buildings", building.name)
+    @Readonly
+    private fun getStatPercentBonusesFromPuppetCity(): Stats? {
+        if (!city.isPuppet) return null
+        return Stats(science = -25f, culture = -25f)
     }
 
-    private fun getStatPercentBonusesFromPuppetCity(): Stats {
-        val stats = Stats()
-        if (city.isPuppet) {
-            stats.science -= 25f
-            stats.culture -= 25f
-        }
-        return stats
-    }
-
-    private fun getGrowthBonus(totalFood: Float): StatMap {
+    @Readonly
+    fun getGrowthBonus(totalFood: Float, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): StatMap {
         val growthSources = StatMap()
-        val stateForConditionals = StateForConditionals(city.civ, city)
         // "[amount]% growth [cityFilter]"
-        for (unique in city.getMatchingUniques(UniqueType.GrowthPercentBonus, stateForConditionals = stateForConditionals)) {
+        for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.GrowthPercentBonus)) {
             if (!city.matchesFilter(unique.params[1])) continue
 
             growthSources.add(
@@ -172,15 +170,17 @@ class CityStats(val city: City) {
         return growthSources
     }
 
+    @Readonly
     fun hasExtraAnnexUnhappiness(): Boolean {
-        if (city.civ.civName == city.foundingCiv || city.isPuppet) return false
-        return !city.containsBuildingUnique(UniqueType.RemoveAnnexUnhappiness)
+        if (city.civ == city.foundingCivObject || city.isPuppet) return false
+        return !city.containsBuildingUnique(UniqueType.RemovesAnnexUnhappiness)
     }
 
+    @Readonly
     fun getStatsOfSpecialist(specialistName: String, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Stats {
         val specialist = city.getRuleset().specialists[specialistName]
             ?: return Stats()
-        val stats = specialist.cloneStats()
+        @LocalState val stats = specialist.cloneStats()
         for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.StatsFromSpecialist))
             if (city.matchesFilter(unique.params[1]))
                 stats.add(unique.stats)
@@ -190,22 +190,24 @@ class CityStats(val city: City) {
         return stats
     }
 
+    @Readonly
     private fun getStatsFromSpecialists(specialists: Counter<String>): Stats {
         val stats = Stats()
         val localUniqueCache = LocalUniqueCache()
-        for (entry in specialists.filter { it.value > 0 })
-            stats.add(getStatsOfSpecialist(entry.key, localUniqueCache) * entry.value)
+        for ((key, value) in specialists.filter { it.value > 0 }.toList()) // avoid concurrent modification when calculating construction costs
+            stats.add(getStatsOfSpecialist(key, localUniqueCache) * value)
         return stats
     }
 
 
+    @Readonly
     private fun getStatsFromUniquesBySource(): StatTreeNode {
         val sourceToStats = StatTreeNode()
 
         val cityStateStatsMultipliers = city.civ.getMatchingUniques(UniqueType.BonusStatsFromCityStates).toList()
 
         fun addUniqueStats(unique: Unique) {
-            val stats = unique.stats.clone()
+            @LocalState val stats = unique.stats.clone()
             if (unique.sourceObjectType==UniqueTarget.CityState)
                 for (multiplierUnique in cityStateStatsMultipliers)
                     stats[Stat.valueOf(multiplierUnique.params[1])] *= multiplierUnique.params[0].toPercent()
@@ -224,7 +226,7 @@ class CityStats(val city: City) {
             }
 
         for (unique in city.getMatchingUniques(UniqueType.StatsFromCitiesOnSpecificTiles))
-            if (city.getCenterTile().matchesTerrainFilter(unique.params[1]))
+            if (city.getCenterTile().matchesTerrainFilter(unique.params[1], city.civ))
                 addUniqueStats(unique)
 
 
@@ -232,20 +234,20 @@ class CityStats(val city: City) {
         return sourceToStats
     }
 
-    private fun getStatPercentBonusesFromGoldenAge(isGoldenAge: Boolean): Stats {
-        val stats = Stats()
-        if (isGoldenAge) {
-            stats.production += 20f
-            stats.culture += 20f
-        }
-        return stats
+    @Pure
+    private fun getStatPercentBonusesFromGoldenAge(isGoldenAge: Boolean): Stats? {
+        if (!isGoldenAge) return null
+        return Stats(production = 20f, culture = 20f)
     }
 
+    @Readonly
     private fun getStatsPercentBonusesFromUniquesBySource(currentConstruction: IConstruction): StatTreeNode {
         val sourceToStats = StatTreeNode()
 
         fun addUniqueStats(unique: Unique, stat: Stat, amount: Float) {
-            sourceToStats.addStats(Stats().add(stat, amount), unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
+            val stats = Stats()
+            stats.add(stat, amount)
+            sourceToStats.addStats(stats, unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
         }
 
         for (unique in city.getMatchingUniques(UniqueType.StatPercentBonus)) {
@@ -266,7 +268,7 @@ class CityStats(val city: City) {
                     city.getMatchingUniques(UniqueType.PercentProductionWonders)
                 currentConstruction is Building && !currentConstruction.isAnyWonder() ->
                     city.getMatchingUniques(UniqueType.PercentProductionBuildings)
-                else -> sequenceOf() // Science/Gold production
+                else -> emptySequence() // Science/Gold production
             }
 
         for (unique in uniquesToCheck) {
@@ -280,14 +282,12 @@ class CityStats(val city: City) {
         for (unique in city.getMatchingUniques(UniqueType.StatPercentFromReligionFollowers))
             addUniqueStats(unique, Stat.valueOf(unique.params[1]),
                 min(
-                    unique.params[0].toFloat() * city.religion.getFollowersOfOurReligion(),
+                    unique.params[0].toFloat() * city.religion.getFollowersOfMajorityReligion(),
                     unique.params[2].toFloat()
                 ))
 
         if (currentConstruction is Building
-            && city.civ.cities.isNotEmpty()
-            && city.civ.getCapital() != null
-            && city.civ.getCapital()!!.cityConstructions.isBuilt(currentConstruction.name)
+            && city.civ.getCapital()?.cityConstructions?.isBuilt(currentConstruction.name) == true
         ) {
             for (unique in city.getMatchingUniques(UniqueType.PercentProductionBuildingsInCapital))
                 addUniqueStats(unique, Stat.Production, unique.params[0].toFloat())
@@ -296,43 +296,49 @@ class CityStats(val city: City) {
         return sourceToStats
     }
 
-    private fun getStatPercentBonusesFromUnitSupply(): Stats {
-        val stats = Stats()
+    @Readonly
+    private fun getStatPercentBonusesFromUnitSupply(): Stats? {
         val supplyDeficit = city.civ.stats.getUnitSupplyDeficit()
         if (supplyDeficit > 0)
-            stats.production = city.civ.stats.getUnitSupplyProductionPenalty()
-        return stats
+            return Stats(production = city.civ.stats.getUnitSupplyProductionPenalty())
+        return null
     }
 
+    @Readonly
     private fun constructionMatchesFilter(construction: IConstruction, filter: String): Boolean {
-        if (construction is Building) return construction.matchesFilter(filter)
-        if (construction is BaseUnit) return construction.matchesFilter(filter)
+        val state = city.state
+        if (construction is Building) return construction.matchesFilter(filter, state)
+        if (construction is BaseUnit) return construction.matchesFilter(filter, state)
         return false
     }
 
+    @Readonly
     fun isConnectedToCapital(roadType: RoadStatus): Boolean {
-        if (city.civ.cities.size < 2) return false// first city!
+        if (city.civ.cities.size < 2) return false // first city!
 
         // Railroad, or harbor from railroad
         return if (roadType == RoadStatus.Railroad)
                 city.isConnectedToCapital {
-                    roadTypes ->
-                    roadTypes.any { it.contains(RoadStatus.Railroad.name) }
+                    mediums ->
+                    mediums.any { it.roadType == RoadStatus.Railroad }
                 }
             else city.isConnectedToCapital()
     }
 
+    @Readonly
+    fun getRoadTypeOfConnectionToCapital(): RoadStatus {
+        return city.civ.cache.citiesConnectedToCapitalToMediums[city]?.maxOfOrNull { it.roadType }
+            ?: RoadStatus.None
+    }
+
+    @Readonly
     private fun getBuildingMaintenanceCosts(): Float {
         // Same here - will have a different UI display.
-        var buildingsMaintenance = city.cityConstructions.getMaintenanceCosts().toFloat() // this is AFTER the bonus calculation!
+        var buildingsMaintenance = city.cityConstructions.getMaintenanceCosts() // this is AFTER the bonus calculation!
         if (!city.civ.isHuman()) {
             buildingsMaintenance *= city.civ.gameInfo.getDifficulty().aiBuildingMaintenanceModifier
         }
 
-        for (unique in city.getMatchingUniques(UniqueType.BuildingMaintenance)) {
-            buildingsMaintenance *= unique.params[0].toPercent()
-        }
-
         return buildingsMaintenance
     }
 
@@ -343,17 +349,17 @@ class CityStats(val city: City) {
         val stats = Stats()
         val workedTiles = city.tilesInRange.asSequence()
             .filter {
-                city.location == it.position
+                city.location.toHexCoord() == it.position
                         || city.isWorked(it)
                         || it.owningCity == city && (it.getUnpillagedTileImprovement()
-                    ?.hasUnique(UniqueType.TileProvidesYieldWithoutPopulation) == true
-                        || it.terrainHasUnique(UniqueType.TileProvidesYieldWithoutPopulation))
+                    ?.hasUnique(UniqueType.TileProvidesYieldWithoutPopulation, it.stateThisTile) == true
+                        || it.terrainHasUnique(UniqueType.TileProvidesYieldWithoutPopulation, it.stateThisTile))
             }
         for (tile in workedTiles) {
             if (tile.isBlockaded() && city.isWorked(tile)) {
                 city.workedTiles.remove(tile.position)
                 city.lockedTiles.remove(tile.position)
-                city.updateCitizens = true
+                city.shouldReassignPopulation = true
                 continue
             }
             val tileStats = tile.stats.getTileStats(city, city.civ, localUniqueCache)
@@ -448,32 +454,40 @@ class CityStats(val city: City) {
         newBaseStatTree.add(getStatsFromUniquesBySource())
         baseStatTree = newBaseStatTree
     }
-
-
-    private fun updateStatPercentBonusList(currentConstruction: IConstruction) {
+    
+    @Readonly
+    private fun getStatPercentBonusList(currentConstruction: IConstruction): StatTreeNode {
         val newStatsBonusTree = StatTreeNode()
 
         newStatsBonusTree.addStats(getStatPercentBonusesFromGoldenAge(city.civ.goldenAges.isGoldenAge()),"Golden Age")
-        addStatPercentBonusesFromBuildings(newStatsBonusTree)
         newStatsBonusTree.addStats(getStatPercentBonusesFromRailroad(), "Railroad")
         newStatsBonusTree.addStats(getStatPercentBonusesFromPuppetCity(), "Puppet City")
         newStatsBonusTree.addStats(getStatPercentBonusesFromUnitSupply(), "Unit Supply")
-
         newStatsBonusTree.add(getStatsPercentBonusesFromUniquesBySource(currentConstruction))
+        
+        val localUniqueCache = LocalUniqueCache()
+        for (building in city.cityConstructions.getBuiltBuildings())
+            newStatsBonusTree.addStats(building.getStatPercentageBonuses(city, localUniqueCache),
+                "Buildings", building.name)
+
 
         if (DebugUtils.SUPERCHARGED) {
             val stats = Stats()
-            for (stat in Stat.values()) stats[stat] = 10000f
+            for (stat in Stat.entries) stats[stat] = 10000f
             newStatsBonusTree.addStats(stats, "Supercharged")
         }
-
-        statPercentBonusTree = newStatsBonusTree
+        return newStatsBonusTree
+    }
+    
+    private fun updateStatPercentBonusList(currentConstruction: IConstruction){
+        statPercentBonusTree = getStatPercentBonusList(currentConstruction)
     }
 
     fun update(currentConstruction: IConstruction = city.cityConstructions.getCurrentConstruction(),
                updateTileStats:Boolean = true,
                updateCivStats:Boolean = true,
-               localUniqueCache:LocalUniqueCache = LocalUniqueCache()) {
+               localUniqueCache:LocalUniqueCache = LocalUniqueCache(),
+               calculateGrowthModifiers:Boolean = true) {
 
         if (updateTileStats) updateTileStats(localUniqueCache)
 
@@ -484,7 +498,7 @@ class CityStats(val city: City) {
         updateCityHappiness(statsFromBuildings)
         updateStatPercentBonusList(currentConstruction)
 
-        updateFinalStatList(currentConstruction) // again, we don't edit the existing currentCityStats directly, in order to avoid concurrency exceptions
+        updateFinalStatList(currentConstruction, calculateGrowthModifiers) // again, we don't edit the existing currentCityStats directly, in order to avoid concurrency exceptions
 
         val newCurrentCityStats = Stats()
         for (stat in finalStatList.values) newCurrentCityStats.add(stat)
@@ -493,7 +507,7 @@ class CityStats(val city: City) {
         if (updateCivStats) city.civ.updateStatsForNextTurn()
     }
 
-    private fun updateFinalStatList(currentConstruction: IConstruction) {
+    private fun updateFinalStatList(currentConstruction: IConstruction, calculateGrowthModifiers: Boolean = true) {
         val newFinalStatList = StatMap() // again, we don't edit the existing currentCityStats directly, in order to avoid concurrency exceptions
 
         for ((key, value) in baseStatTree.children)
@@ -506,7 +520,7 @@ class CityStats(val city: City) {
 
         // We only add the 'extra stats from production' AFTER we calculate the production INCLUDING BONUSES
         val statsFromProduction = getStatsFromProduction(newFinalStatList.values.map { it.production }.sum())
-        if (!statsFromProduction.isEmpty()) {
+        if (statsFromProduction != null && !statsFromProduction.isEmpty()) {
             baseStatTree = StatTreeNode().apply {
                 children.putAll(baseStatTree.children)
                 addStats(statsFromProduction, "Production")
@@ -557,7 +571,7 @@ class CityStats(val city: City) {
         var totalFood = newFinalStatList.values.map { it.food }.sum()
 
         // Apply growth modifier only when positive food
-        if (totalFood > 0) {
+        if (totalFood > 0 && calculateGrowthModifiers) {
             // Since growth bonuses are special, (applied afterwards) they will be displayed separately in the user interface as well.
             // All bonuses except We Love The King do apply even when unhappy
             val growthBonuses = getGrowthBonus(totalFood)
@@ -584,12 +598,13 @@ class CityStats(val city: City) {
 
         val growthNullifyingUnique = city.getMatchingUniques(UniqueType.NullifiesGrowth).firstOrNull()
         if (growthNullifyingUnique != null) {
-            // Note that negative food will also be nullified. Pretty sure that's conform civ V, but haven't checked.
-            val amountToRemove = -newFinalStatList.values.sumOf { it[Stat.Food].toDouble() }
-            newFinalStatList.add(
-                growthNullifyingUnique.getSourceNameForUser(),
-                Stats(food = amountToRemove.toFloat())
-            )
+            // Does not nullify negative growth (starvation)
+            val currentGrowth = newFinalStatList.values.sumOf { it[Stat.Food].toDouble() }
+            if (currentGrowth > 0)
+                newFinalStatList.add(
+                    growthNullifyingUnique.getSourceNameForUser(),
+                    Stats(food = -currentGrowth.toFloat())
+                )
         }
 
         if (city.isInResistance())
@@ -600,6 +615,7 @@ class CityStats(val city: City) {
         finalStatList = newFinalStatList
     }
 
+    @Readonly
     fun canConvertFoodToProduction(food: Float, currentConstruction: IConstruction): Boolean {
         return (food > 0
             && currentConstruction is INonPerpetualConstruction
@@ -613,6 +629,7 @@ class CityStats(val city: City) {
      * See for details: https://civilization.fandom.com/wiki/Settler_(Civ5)
      * @see calcFoodEaten as well for Food consumed this turn
      */
+    @Pure
     fun getProductionFromExcessiveFood(food : Float): Float {
         return if (food >= 4.0f ) 2.0f + (food / 4.0f).toInt()
           else if (food >= 2.0f ) 2.0f
@@ -620,27 +637,25 @@ class CityStats(val city: City) {
         else 0.0f
     }
 
+    @Readonly
     private fun calcFoodEaten(): Float {
-        var foodEaten = city.population.population.toFloat() * 2
         var foodEatenBySpecialists = 2f * city.population.getNumberOfSpecialists()
-
+        var foodEaten = city.population.population.toFloat() * 2 - foodEatenBySpecialists
+        
         for (unique in city.getMatchingUniques(UniqueType.FoodConsumptionBySpecialists))
             if (city.matchesFilter(unique.params[1]))
                 foodEatenBySpecialists *= unique.params[0].toPercent()
 
-        foodEaten -= 2f * city.population.getNumberOfSpecialists() - foodEatenBySpecialists
+        foodEaten += foodEatenBySpecialists
+        
+        for (unique in city.getMatchingUniques(UniqueType.FoodConsumptionByPopulation)) {
+            if (!city.matchesFilter(unique.params[2])) continue
+            val foodEatenByPopulationFilter = 2f * city.population.getPopulationFilterAmount(unique.params[1])
+            foodEaten -= foodEatenByPopulationFilter * (1f - unique.params[0].toPercent())
+        }
+        
         return foodEaten
     }
 
     //endregion
-
-    fun getStatDifferenceFromBuilding(building: String): Stats {
-        val newCity = city.clone()
-        newCity.setTransients(city.civ) // Will break the owned tiles. Needs to be reverted before leaving this function
-        newCity.cityConstructions.builtBuildings.add(building)
-        newCity.cityConstructions.setTransients()
-        newCity.cityStats.update(updateCivStats = false)
-        city.expansion.setTransients() // Revert owned tiles to original city
-        return newCity.cityStats.currentCityStats - currentCityStats
-    }
 }
