@@ -1,18 +1,19 @@
 package com.unciv.logic.city.managers
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.toPercent
-import com.unciv.ui.components.extensions.withItem
-import com.unciv.ui.components.extensions.withoutItem
+import com.unciv.utils.withItem
+import com.unciv.utils.withoutItem
+import yairm210.purity.annotations.Readonly
 import kotlin.math.ceil
 import kotlin.math.floor
 import kotlin.math.pow
@@ -25,10 +26,9 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
         private set
     var foodStored = 0
 
-    // In favor of this bad boy
     val specialistAllocations = Counter<String>()
 
-    fun getNewSpecialists() = specialistAllocations //convertStatsToSpecialistHashmap(specialists)
+    fun getNewSpecialists() = specialistAllocations 
 
 
     //region pure functions
@@ -40,20 +40,22 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
         return toReturn
     }
 
-    fun getNumberOfSpecialists() = getNewSpecialists().values.sum()
+    @Readonly fun getNumberOfSpecialists() = getNewSpecialists().values.sum()
 
+    @Readonly
     fun getFreePopulation(): Int {
         val workingPopulation = city.workedTiles.size
         return population - workingPopulation - getNumberOfSpecialists()
     }
-
+    
+    @Readonly
     fun getFoodToNextPopulation(): Int {
         // civ v math, civilization.wikia
-        var foodRequired = 15 + 6 * (population - 1) + floor((population - 1).toDouble().pow(1.8))
+        var foodRequired = 15 + 8 * (population - 1) + floor((population - 1).toDouble().pow(1.5))
 
         foodRequired *= city.civ.gameInfo.speed.modifier
 
-        if (city.civ.isCityState())
+        if (city.civ.isCityState)
             foodRequired *= 1.5f
         if (!city.civ.isHuman())
             foodRequired *= city.civ.gameInfo.getDifficulty().aiCityGrowthModifier
@@ -61,6 +63,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
     }
 
     /** Take null to mean infinity. */
+    @Readonly
     fun getNumTurnsToStarvation(): Int? {
         if (!city.isStarving()) return null
         return foodStored / -city.foodForNextTurn() + 1
@@ -68,6 +71,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
 
 
     /** Take null to mean infinity. */
+    @Readonly
     fun getNumTurnsToNewPopulation(): Int? {
         if (!city.isGrowing()) return null
         val roundedFoodPerTurn = city.foodForNextTurn().toFloat()
@@ -81,13 +85,14 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
     //endregion
 
     /** Implements [UniqueParameterType.PopulationFilter][com.unciv.models.ruleset.unique.UniqueParameterType.PopulationFilter] */
+    @Readonly
     fun getPopulationFilterAmount(filter: String): Int {
         return when (filter) {
             "Specialists" -> getNumberOfSpecialists()
             "Population" -> population
             "Followers of the Majority Religion", "Followers of this Religion" -> city.religion.getFollowersOfMajorityReligion()
             "Unemployed" -> getFreePopulation()
-            else -> 0
+            else -> specialistAllocations[filter]
         }
     }
 
@@ -109,6 +114,12 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
         if (city.getMatchingUniques(UniqueType.NullifiesGrowth).any())
             return
 
+        // Hard block growth when using Avoid Growth, cap stored food
+        if (city.avoidGrowth) {
+            foodStored = foodNeededToGrow
+            return
+        }
+
         // growth!
         foodStored -= foodNeededToGrow
         val percentOfFoodCarriedOver =
@@ -118,19 +129,18 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
                 .coerceAtMost(95)  // Try to avoid runaway food gain in mods, just in case
         foodStored += (foodNeededToGrow * percentOfFoodCarriedOver / 100f).toInt()
         addPopulation(1)
-        city.updateCitizens = true
+        city.shouldReassignPopulation = true
         city.civ.addNotification("[${city.name}] has grown!", city.location,
             NotificationCategory.Cities, NotificationIcon.Growth)
     }
 
     fun addPopulation(count: Int) {
-        val changedAmount =
-            if (population + count < 0) -population
-            else count
+        val changedAmount = count.coerceAtLeast(1 - population)
         population += changedAmount
         val freePopulation = getFreePopulation()
         if (freePopulation < 0) {
             unassignExtraPopulation()
+            city.cityStats.update()
         } else {
             autoAssignPopulation()
         }
@@ -166,6 +176,8 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
                 .filterNot { it.providesYield() }
                 .associateWith { it.stats.getTileStats(city, city.civ, localUniqueCache)}
 
+        val maxSpecialists = getMaxSpecialists().asSequence()
+
         repeat(freePopulation) {
             //evaluate tiles
             val bestTileAndRank = tilesToEvaluate
@@ -179,7 +191,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
             val valueBestTile = bestTileAndRank?.value ?: 0f
 
             val bestJobAndRank = if (city.manualSpecialists) null
-                else getMaxSpecialists().asSequence()
+                else maxSpecialists
                     .filter { specialistAllocations[it.key] < it.value }
                     .map { it.key }
                     .associateWith { Automation.rankSpecialist(it, city, localUniqueCache) }
@@ -191,7 +203,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
             if (valueBestTile > valueBestSpecialist) {
                 if (bestTile != null) {
                     city.workedTiles = city.workedTiles.withItem(bestTile.position)
-                    cityStats.food += bestTile.stats.getTileStats(city, city.civ, localUniqueCache).food
+                    cityStats.food += tileStats[bestTile]!!.food
                 }
             } else if (bestJob != null) {
                 specialistAllocations.add(bestJob, 1)
@@ -201,7 +213,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
         city.cityStats.update()
     }
 
-    fun stopWorkingTile(position: Vector2) {
+    fun stopWorkingTile(position: HexCoord) {
         city.workedTiles = city.workedTiles.withoutItem(position)
         city.lockedTiles.remove(position)
     }
@@ -209,7 +221,7 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
     fun unassignExtraPopulation() {
         for (tile in city.workedTiles.map { city.tileMap[it] }) {
             if (tile.getOwner() != city.civ || tile.getWorkingCity() != city
-                    || tile.aerialDistanceTo(city.getCenterTile()) > 3)
+                    || tile.aerialDistanceTo(city.getCenterTile()) > city.getWorkRange())
                 city.population.stopWorkingTile(tile.position)
         }
 
@@ -265,10 +277,9 @@ class CityPopulationManager : IsPartOfGameInfoSerialization {
                 }
             }
         }
-
-        city.cityStats.update()
     }
 
+    @Readonly
     fun getMaxSpecialists(): Counter<String> {
         val counter = Counter<String>()
         for (building in city.cityConstructions.getBuiltBuildings())
