@@ -9,15 +9,23 @@ import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.civilization.OverviewAction
 import com.unciv.models.ruleset.tile.ResourceType
+import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
-import kotlin.math.min
 import kotlin.random.Random
 
 class CityTurnManager(val city: City) {
 
 
     fun startTurn() {
+        for (resource in city.getResourcesGeneratedByCity()) {
+            if (resource.resource.isStockpiled && resource.resource.isCityWide)
+                city.gainStockpiledResource(resource.resource, resource.amount)
+        }
+        for (unique in city.getTriggeredUniques(UniqueType.TriggerUponTurnStart, includeCivUniques = false).toList()) {
+            UniqueTriggerActivation.triggerUnique(unique, city)
+        }
+
         // Construct units at the beginning of the turn,
         // so they won't be generated out in the open and vulnerable to enemy attacks before you can control them
         city.cityConstructions.constructIfEnough()
@@ -34,7 +42,7 @@ class CityTurnManager(val city: City) {
         if (city.isPuppet) {
             city.setCityFocus(CityFocus.GoldFocus)
             city.reassignAllPopulation()
-        } else if (city.updateCitizens) {
+        } else if (city.shouldReassignPopulation || city.civ.isAI()) {
             city.reassignPopulation()  // includes cityStats.update
         } else
             city.cityStats.update()
@@ -77,7 +85,7 @@ class CityTurnManager(val city: City) {
                         demandNewResource()
                     }
                     CityFlags.Resistance.name -> {
-                        city.updateCitizens = true
+                        city.shouldReassignPopulation = true
                         city.civ.addNotification(
                             "The resistance in [${city.name}] has ended!",
                             CityAction.withLocation(city), NotificationCategory.General, "StatIcons/Resistance")
@@ -93,20 +101,23 @@ class CityTurnManager(val city: City) {
             it.resourceType == ResourceType.Luxury && // Must be luxury
                     !it.hasUnique(UniqueType.CityStateOnlyResource) && // Not a city-state only resource eg jewelry
                     it.name != city.demandedResource && // Not same as last time
-                    !city.civ.hasResource(it.name) && // Not one we already have
-                    it.name in city.tileMap.resources && // Must exist somewhere on the map
-                    city.getCenterTile().getTilesInDistance(3).none { nearTile -> nearTile.resource == it.name } // Not in this city's radius
+                    it in city.tileMap.resourceObjects && // Must exist somewhere on the map
+                    city.getCenterTile().getTilesInDistance(city.getWorkRange()).none { nearTile -> nearTile.tileResource == it } // Not in this city's radius
+        }
+        val missingResources = candidates.filter { !city.civ.hasResource(it) }
+        
+        if (missingResources.isEmpty()) { // hooray happpy day forever!
+            city.demandedResource = candidates.randomOrNull()?.name ?: ""
+            return // actually triggering "wtlk" is done in tryWeLoveTheKing(), *next turn*
         }
 
-        val chosenResource = candidates.randomOrNull()
-        /* What if we had a WLTKD before but now the player has every resource in the game? We can't
-           pick a new resource, so the resource will stay stay the same and the city will demand it
-           again even if the player still has it. But we shouldn't punish success. */
-        if (chosenResource != null)
-            city.demandedResource = chosenResource.name
-        if (city.demandedResource == "") // Failed to get a valid resource, try again some time later
-            city.setFlag(CityFlags.ResourceDemand, 15 + Random.Default.nextInt(10))
-        else
+        val chosenResource = missingResources.randomOrNull()
+        
+        city.demandedResource = chosenResource?.name ?: "" // mods may have no resources as candidates even
+        // Get a new resource in ~20 turns
+        city.setFlag(CityFlags.ResourceDemand, 15 + Random.Default.nextInt(10))
+        
+        if (city.demandedResource != "") // Failed to get a valid resource, try again some time later
             city.civ.addNotification("[${city.name}] demands [${city.demandedResource}]!",
                 listOf(LocationAction(city.location), OverviewAction(EmpireOverviewCategories.Resources)),
                 NotificationCategory.General, NotificationIcon.City, "ResourceIcons/${city.demandedResource}")
@@ -114,6 +125,9 @@ class CityTurnManager(val city: City) {
 
 
     fun endTurn() {
+        for (unique in city.getTriggeredUniques(UniqueType.TriggerUponTurnEnd, includeCivUniques = false).toList()) {
+            UniqueTriggerActivation.triggerUnique(unique, city)
+        }
         val stats = city.cityStats.currentCityStats
 
         city.cityConstructions.endTurn(stats)
@@ -122,10 +136,9 @@ class CityTurnManager(val city: City) {
             val removedPopulation =
                     1 + city.civ.getMatchingUniques(UniqueType.CitiesAreRazedXTimesFaster)
                         .sumOf { it.params[0].toInt() - 1 }
-            city.population.addPopulation(-1 * removedPopulation)
 
-            if (city.population.population <= 0) {
-                city.espionage.removeAllPresentSpies(SpyFleeReason.CityCaptured)
+            if (city.population.population <= removedPopulation) {
+                city.espionage.removeAllPresentSpies(SpyFleeReason.Other)
                 city.civ.addNotification(
                     "[${city.name}] has been razed to the ground!",
                     city.location, NotificationCategory.General,
@@ -133,6 +146,7 @@ class CityTurnManager(val city: City) {
                 )
                 city.destroyCity()
             } else { //if not razed yet:
+                city.population.addPopulation(-removedPopulation)
                 if (city.population.foodStored >= city.population.getFoodToNextPopulation()) { //if surplus in the granary...
                     city.population.foodStored =
                             city.population.getFoodToNextPopulation() - 1 //...reduce below the new growth threshold
@@ -144,7 +158,7 @@ class CityTurnManager(val city: City) {
         if (city.civ.gameInfo.isReligionEnabled()) city.religion.endTurn()
 
         if (city in city.civ.cities) { // city was not destroyed
-            city.health = min(city.health + 20, city.getMaxHealth())
+            city.health = (city.health + 20).coerceAtMost(city.getMaxHealth())
             city.population.unassignExtraPopulation()
         }
     }
