@@ -1,14 +1,15 @@
 package com.unciv.logic.civilization
 
-import com.badlogic.gdx.math.Vector2
+import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.utils.Json
 import com.badlogic.gdx.utils.JsonValue
+import com.unciv.Constants
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.city.City
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.ui.components.MayaCalendar
 import com.unciv.ui.screens.cityscreen.CityScreen
-import com.unciv.ui.screens.civilopediascreen.CivilopediaScreen
 import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewScreen
@@ -25,7 +26,7 @@ import com.unciv.ui.screens.worldscreen.WorldScreen
  * there that should not be serialized to the saved game.
  *
  * IsPartOfGameInfoSerialization is just a marker class and not actually tested for, so inheriting it
- * _indirectly_ is OK (the NotificationAction subclasses need not re-implement, a `is`test would still succeed).
+ * _indirectly_ is OK (the NotificationAction subclasses need not re-implement, a `is` test would still succeed).
  *
  * Also note all implementations need the default no-args constructor for deserialization,
  * therefore the otherwise unused default initializers.
@@ -35,24 +36,24 @@ interface NotificationAction : IsPartOfGameInfoSerialization {
 }
 
 /** A notification action that shows map places. */
-// Note location is nonprivate only for writeOldFormatAction
-class LocationAction(internal val location: Vector2 = Vector2.Zero) : NotificationAction {
+class LocationAction(private val location: HexCoord = HexCoord.Zero) : NotificationAction {
     override fun execute(worldScreen: WorldScreen) {
         worldScreen.mapHolder.setCenterPosition(location, selectUnit = false)
     }
+
+    /**
+     *  This Companion implements constructor-like factories through [invoke] to simulate the old [LocationAction]
+     *  which stored several locations (back then in turn there was only one action per Notification).
+     *  - Meant to be used in [Civilization.addNotification] calls.
+     *  - Note there are overloads accepting `Iterable`, `Sequence` or `vararg` of [HexCoord]
+     *  - The `vararg` version accepts and ignores `null`s, often making pre-filtering of conditionally available locations unnecessary.
+     *
+     *  Example: `addNotification("Bob hit alice", LocationAction(bob.position, alice.position), NotificationCategory.War)`
+     */
     companion object {
-        /*
-            These are constructor-like factories to simulate the old LocationAction which stored
-            several locations (back then in turn there was only one action per Notification).
-            Example: addNotification("Bob hit alice", LocationAction(bob.position, alice.position), NotificationCategory.War)
-            This maps to the (String, Sequence<NotificationAction>, NotificationCategory, vararg String)
-            overload of addNotification through the last invoke below.
-         */
-        operator fun invoke(locations: Sequence<Vector2>): Sequence<LocationAction> =
+        operator fun invoke(locations: Sequence<HexCoord>): Sequence<LocationAction> =
             locations.map { LocationAction(it) }
-        operator fun invoke(locations: Iterable<Vector2>): Sequence<LocationAction> =
-            locations.asSequence().map { LocationAction(it) }
-        operator fun invoke(vararg locations: Vector2?): Sequence<LocationAction> =
+        operator fun invoke(vararg locations: HexCoord?): Sequence<LocationAction> =
             locations.asSequence().filterNotNull().map { LocationAction(it) }
     }
 }
@@ -66,7 +67,7 @@ class TechAction(private val techName: String = "") : NotificationAction {
 }
 
 /** enter city */
-class CityAction(private val city: Vector2 = Vector2.Zero) : NotificationAction {
+class CityAction(private val city: HexCoord = HexCoord.Zero) : NotificationAction {
     override fun execute(worldScreen: WorldScreen) {
         val cityObject = worldScreen.mapHolder.tileMap[city].getCity()
             ?: return
@@ -79,20 +80,37 @@ class CityAction(private val city: Vector2 = Vector2.Zero) : NotificationAction
 }
 
 /** enter diplomacy screen */
-class DiplomacyAction(
-    private val otherCivName: String = "",
-    private var showTrade: Boolean = false
-) : NotificationAction {
+class DiplomacyAction : NotificationAction {
+    private val otherCivName: String
+    private var showTrade: Boolean
+    @Transient
+    private lateinit var otherCiv: Civilization
+
+    @Suppress("unused") // Used for deserialization
+    constructor(): this("", false)
+
+    constructor(otherCivName: String, showTrade: Boolean = false) {
+        this.otherCivName = otherCivName
+        this.showTrade = showTrade
+    }
+
+    constructor(otherCiv: Civilization, showTrade: Boolean = false) {
+        this.otherCiv = otherCiv
+        this.otherCivName = otherCiv.civID
+        this.showTrade = showTrade
+    }
+
     override fun execute(worldScreen: WorldScreen) {
         val currentCiv = worldScreen.selectedCiv
-        val otherCiv = worldScreen.gameInfo.getCivilization(otherCivName)
+        if (!::otherCiv.isInitialized)
+            otherCiv = worldScreen.gameInfo.getCivilization(otherCivName)
 
         if (showTrade && otherCiv == currentCiv)
             // Because TradeTable will set up otherCiv against that one,
             // not the one we pass below, and two equal civs will crash - can't look up a DiplomacyManager.
             return
         // We should not be able to trade with city-states
-        if (showTrade && (otherCiv.isCityState() || currentCiv.isCityState()))
+        if (showTrade && (otherCiv.isCityState || currentCiv.isCityState))
             showTrade = false
 
         if (showTrade && currentCiv.isAtWarWith(otherCiv))
@@ -109,24 +127,37 @@ class MayaLongCountAction : NotificationAction {
     }
 }
 
-/** A notification action that shows and selects units on the map.
+/** A notification action that shows **and selects** things on the map.
  *
- *  Saves and serializes only the location. Activation will select the tile which will select any unit
- *  on it or cycle through selections if this NotificationAction is the only one on the Notification.
+ *  Saves and serializes only the location and optionally a MapUnit id.
+ *  When an id is present, activation will select that unit if it is still in the given position - otherwise, it will behave like LocationAction.
+ *  Without id, activation will select the tile which will select any unit on it or cycle through selections if this NotificationAction is the only one on the Notification.
  *  When the unit has been moved away, activation still shows the tile and not the unit.
- *
- *  As MapUnits do not have any persistent ID differentiating them from other units of same Civ and BaseUnit,
- *  this cannot be done significantly better. Should someone add a persisted UUID to MapUnit, please change this too.
+ *  Activation without unit id also works for cities, selecting them - so a bombard is one click less.
  */
-class MapUnitAction(private val location: Vector2 = Vector2.Zero) : NotificationAction {
-    constructor(unit: MapUnit) : this(unit.currentTile.position)
+class MapUnitAction(
+    private val location: HexCoord = HexCoord.Zero,
+    private val id: Int = Constants.NO_ID
+) : NotificationAction {
+    constructor(unit: MapUnit) : this(unit.currentTile.position.toHexCoord(), unit.id)
     override fun execute(worldScreen: WorldScreen) {
-        worldScreen.mapHolder.setCenterPosition(location, selectUnit = true)
+        val selectUnit = id != Constants.NO_ID // This is the unspecific "select any unit on that tile", specific works without this being on
+        val unit = if (selectUnit) 
+            worldScreen.selectedCiv.units.getUnitById(id) 
+        else
+            worldScreen.gameInfo.tileMap[location].getUnits().firstOrNull { it.id == id }
+        if (unit != null) {
+            val unitLocation = unit.currentTile.position.toHexCoord()
+            worldScreen.mapHolder.setCenterPosition(unitLocation, selectUnit = selectUnit, forceSelectUnit = unit)
+        }
+        else {
+            worldScreen.mapHolder.setCenterPosition(location.toHexCoord(), selectUnit = false)
+        }
     }
     companion object {
         // Convenience shortcut as it makes replacing LocationAction calls easier (see above)
-        operator fun invoke(locations: Iterable<Vector2>): Sequence<MapUnitAction> =
-            locations.asSequence().map { MapUnitAction(it) }
+        operator fun invoke(units: Iterable<MapUnit>): Sequence<MapUnitAction> =
+            units.asSequence().map { MapUnitAction(it) }
     }
 }
 
@@ -137,12 +168,25 @@ class CivilopediaAction(private val link: String = "") : NotificationAction {
     }
 }
 
-/** Show Promotion picker for a MapUnit - by name and location, as they lack a serialized unique ID */
-class PromoteUnitAction(private val name: String = "", private val location: Vector2 = Vector2.Zero) : NotificationAction {
+/** Show Promotion picker for a MapUnit - by name and location, as they lack a serialized unique ID
+ *  TODO that's no longer true - migration with backward compat is in order - this implements the first step
+ */
+class PromoteUnitAction(
+    private val name: String,
+    private val location: HexCoord,
+    val id: Int
+) : NotificationAction {
+    @Suppress("unused")
+    constructor() : this("", HexCoord.Zero, Constants.NO_ID)
+    constructor(unit: MapUnit) : this(unit.name, unit.currentTile.position, unit.id)
+
     override fun execute(worldScreen: WorldScreen) {
-        val tile = worldScreen.gameInfo.tileMap[location]
-        val unit = tile.militaryUnit?.takeIf { it.name == name && it.civ == worldScreen.selectedCiv }
-            ?: return
+        val unit = if (id != Constants.NO_ID) {
+            worldScreen.selectedCiv.units.getUnitById(id)
+        } else {
+            val tile = worldScreen.gameInfo.tileMap[location]
+            tile.militaryUnit?.takeIf { it.name == name && it.civ == worldScreen.selectedCiv }
+        } ?: return
         worldScreen.game.pushScreen(PromotionPickerScreen(unit))
     }
 }
@@ -153,7 +197,7 @@ class OverviewAction(
     private val select: String = ""
 ) : NotificationAction {
     override fun execute(worldScreen: WorldScreen) {
-        worldScreen.game.pushScreen(EmpireOverviewScreen(worldScreen.selectedCiv, page, select))
+        worldScreen.openEmpireOverview(page, select)
     }
 }
 
@@ -172,24 +216,41 @@ class EspionageAction : NotificationAction {
         worldScreen.game.pushScreen(EspionageOverviewScreen(worldScreen.selectedCiv, worldScreen))
     }
     companion object {
-        fun withLocation(location: Vector2?): Sequence<NotificationAction> =
+        fun withLocation(location: HexCoord?): Sequence<NotificationAction> =
             LocationAction(location) + EspionageAction()
     }
 }
 
+/** Open [url] externally in the browser */
+class LinkAction(private val url: String = "") : NotificationAction {
+    override fun execute(worldScreen: WorldScreen) {
+        if (url.isNotEmpty()) Gdx.net.openURI(url)
+    }
+}
+
+/** Open [EmpireOverviewScreen] on the [Religion][EmpireOverviewCategories.Religion] tab */
+class ReligionAction(private val religionName: String? = null) : NotificationAction {
+    override fun execute(worldScreen: WorldScreen) {
+        worldScreen.openEmpireOverview(EmpireOverviewCategories.Religion, religionName.orEmpty())
+    }
+    companion object {
+        fun withLocation(location: HexCoord?, religionName: String?): Sequence<NotificationAction> =
+            LocationAction(location) + ReligionAction(religionName)
+    }
+}
+
 
 @Suppress("PrivatePropertyName")  // These names *must* match their class name, see below
+/** This exists as trick to leverage readFields for Json deserialization.
+ *
+ *  The serializer writes each [NotificationAction] as json object (within the [actions][Notification.actions] array),
+ *  containing the class `simpleName` as subfield name, which carries any (or none) subclass-specific data as its object value.
+ *  So, reading this from json data will fill just one of the fields, and the `listOfNotNull` will output that field only.
+ *  Even though we know there's only one result, no need to `first()` since it's no advantage to the caller.
+ *
+ *  In a way, this is like asking the class loader to resolve the class by compilation instead of via the reflection API, like Gdx would try unaided.
+ */
 internal class NotificationActionsDeserializer {
-    /* This exists as trick to leverage readFields for Json deserialization.
-    // The serializer writes each NotificationAction as json object (within the actions array),
-    // containing the class simpleName as subfield name, which carries any (or none) subclass-
-    // specific data as its object value. So, reading this from json data will fill just one of the
-    // fields, and the listOfNotNull will output that field only.
-    // Even though we know there's only one result, no need to first() since it's no advantage to the caller.
-    //
-    // In a way, this is like asking the class loader to resolve the class by compilation
-    // instead of via the reflection API, like Gdx would try unaided.
-    */
     private val LocationAction: LocationAction? = null
     private val TechAction: TechAction? = null
     private val CityAction: CityAction? = null
@@ -201,13 +262,15 @@ internal class NotificationActionsDeserializer {
     private val OverviewAction: OverviewAction? = null
     private val PolicyAction: PolicyAction? = null
     private val EspionageAction: EspionageAction? = null
+    private val LinkAction: LinkAction? = null
+    private val ReligionAction: ReligionAction? = null
 
     fun read(json: Json, jsonData: JsonValue): List<NotificationAction> {
         json.readFields(this, jsonData)
         return listOfNotNull(
             LocationAction, TechAction, CityAction, DiplomacyAction, MayaLongCountAction,
             MapUnitAction, CivilopediaAction, PromoteUnitAction, OverviewAction, PolicyAction,
-            EspionageAction
+            EspionageAction, LinkAction, ReligionAction
         )
     }
 }
