@@ -11,16 +11,27 @@ enum class AlertType : IsPartOfGameInfoSerialization {
     CityConquered,
     CityTraded,
     BorderConflict,
+    TilesStolen,
+
     DemandToStopSettlingCitiesNear,
     CitySettledNearOtherCivDespiteOurPromise,
+
+    DemandToStopSpreadingReligion,
+    ReligionSpreadDespiteOurPromise,
+    
+    DemandToStopSpyingOnUs,
+    SpyingOnUsDespiteOurPromise,
+
     GoldenAge,
     DeclarationOfFriendship,
     StartIntro,
     DiplomaticMarriage,
     BulliedProtectedMinor,
     AttackedProtectedMinor,
+    AttackedAllyMinor,
     RecapturedCivilian,
-    GameHasBeenWon
+    GameHasBeenWon,
+    Event
 }
 
 class PopupAlert : IsPartOfGameInfoSerialization {
