@@ -3,29 +3,23 @@ package com.unciv.logic.civilization.managers
 import com.unciv.Constants
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.city.City
-import com.unciv.logic.civilization.AlertType
-import com.unciv.logic.civilization.Civilization
-import com.unciv.logic.civilization.LocationAction
-import com.unciv.logic.civilization.MayaLongCountAction
-import com.unciv.logic.civilization.NotificationCategory
-import com.unciv.logic.civilization.NotificationIcon
-import com.unciv.logic.civilization.PlayerType
-import com.unciv.logic.civilization.PolicyAction
-import com.unciv.logic.civilization.PopupAlert
-import com.unciv.logic.civilization.TechAction
-import com.unciv.logic.map.MapSize
+import com.unciv.logic.civilization.*
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.models.ruleset.tech.Era
 import com.unciv.models.ruleset.tech.Technology
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.tile.TileResource
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
+import com.unciv.models.translations.tr
 import com.unciv.ui.components.MayaCalendar
-import com.unciv.ui.components.extensions.withItem
+import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.components.fonts.Fonts
+import com.unciv.utils.withItem
+import yairm210.purity.annotations.Readonly
 import kotlin.math.ceil
 import kotlin.math.max
 import kotlin.math.min
@@ -57,13 +51,15 @@ class TechManager : IsPartOfGameInfoSerialization {
     var movementSpeedOnRoads = 1f
     @Transient
     var roadsConnectAcrossRivers = false
+    @Transient
+    var allTechsAreResearched = false
 
     var freeTechs = 0
     // For calculating score
     var repeatingTechsResearched = 0
 
     /** For calculating Great Scientist yields - see https://civilization.fandom.com/wiki/Great_Scientist_(Civ5)  */
-    var scienceOfLast8Turns = IntArray(8) { 0 }
+    var scienceOfLast8Turns = IntArray(8)
     var scienceFromResearchAgreements = 0
     /** This is the list of strings, which is serialized */
     var techsResearched = HashSet<String>()
@@ -91,97 +87,115 @@ class TechManager : IsPartOfGameInfoSerialization {
         return toReturn
     }
 
-    fun getNumberOfTechsResearched(): Int = techsResearched.size
+    @Readonly fun getNumberOfTechsResearched(): Int = techsResearched.size
 
-    fun getOverflowScience(techName: String): Int {
-        return if (overflowScience == 0) 0
-            else (getScienceModifier(techName) * overflowScience).toInt()
-    }
+    @Readonly fun getOverflowScience(): Int = overflowScience
 
+    @Readonly
     private fun getScienceModifier(techName: String): Float { // https://forums.civfanatics.com/threads/the-mechanics-of-overflow-inflation.517970/
-        val techsResearchedKnownCivs = civInfo.getKnownCivs()
+        val numberOfCivsResearchedThisTech = civInfo.getKnownCivs()
             .count { it.isMajorCiv() && it.tech.isResearched(techName) }
-        val undefeatedCivs = civInfo.gameInfo.civilizations
+        val numberOfCivsRemaining = civInfo.gameInfo.civilizations
             .count { it.isMajorCiv() && !it.isDefeated() }
-        return 1 + techsResearchedKnownCivs / undefeatedCivs.toFloat() * 0.3f
+        return 1 + numberOfCivsResearchedThisTech / numberOfCivsRemaining.toFloat() * 0.3f
     }
 
-    private fun getRuleset() = civInfo.gameInfo.ruleset
+    @Readonly private fun getRuleset() = civInfo.gameInfo.ruleset
 
+    @Readonly
     fun costOfTech(techName: String): Int {
         var techCost = getRuleset().technologies[techName]!!.cost.toFloat()
         if (civInfo.isHuman())
             techCost *= civInfo.getDifficulty().researchCostModifier
         techCost *= civInfo.gameInfo.speed.scienceCostModifier
         techCost /= getScienceModifier(techName)
-        // https://civilization.fandom.com/wiki/Map_(Civ5)
-        val worldSizeModifier = with (civInfo.gameInfo.tileMap.mapParameters.mapSize) {
-            when {
-                radius >= MapSize.Huge.radius -> floatArrayOf(1.3f, 0.025f)
-                radius >= MapSize.Large.radius -> floatArrayOf(1.2f, 0.0375f)
-                radius >= MapSize.Medium.radius -> floatArrayOf(1.1f, 0.05f)
-                else -> floatArrayOf(1f, 0.05f)
-            }
-        }
-        techCost *= worldSizeModifier[0]
-        techCost *= 1 + (civInfo.cities.size - 1) * worldSizeModifier[1]
+        val mapSizePredef = civInfo.gameInfo.tileMap.mapParameters.mapSize.getPredefinedOrNextSmaller()
+        techCost *= mapSizePredef.techCostMultiplier
+        var cityModifier = (civInfo.cities.count { !it.isPuppet } - 1) * mapSizePredef.techCostPerCityModifier
+        for (unique in civInfo.getMatchingUniques(UniqueType.LessTechCostFromCities)) cityModifier *= 1 - unique.params[0].toFloat() / 100
+        for (unique in civInfo.getMatchingUniques(UniqueType.LessTechCost)) techCost *= unique.params[0].toPercent()
+        techCost *= 1 + cityModifier
         return techCost.toInt()
     }
 
+    @Readonly
     fun currentTechnology(): Technology? {
         val currentTechnologyName = currentTechnologyName() ?: return null
         return getRuleset().technologies[currentTechnologyName]
     }
 
+    @Readonly
     fun currentTechnologyName(): String? {
         return if (techsToResearch.isEmpty()) null else techsToResearch[0]
     }
 
-    fun researchOfTech(techName: String?) = techsInProgress[techName] ?: 0
+    @Readonly fun researchOfTech(techName: String?) = techsInProgress[techName] ?: 0
     // Was once duplicated as fun scienceSpentOnTech(tech: String): Int
 
+    @Readonly
     fun remainingScienceToTech(techName: String): Int {
-        val spareScience = if (canBeResearched(techName)) getOverflowScience(techName) else 0
+        val spareScience = if (canBeResearched(techName)) getOverflowScience() else 0
         return costOfTech(techName) - researchOfTech(techName) - spareScience
     }
 
+    @Readonly
     fun turnsToTech(techName: String): String {
         val remainingCost = remainingScienceToTech(techName).toDouble()
         return when {
-            remainingCost <= 0f -> "0"
+            remainingCost <= 0f -> (0).tr()
             civInfo.stats.statsForNextTurn.science <= 0f -> Fonts.infinity.toString()
-            else -> max(1, ceil(remainingCost / civInfo.stats.statsForNextTurn.science).toInt()).toString()
+            else -> max(
+                1,
+                ceil(remainingCost / civInfo.stats.statsForNextTurn.science).toInt()
+            ).tr()
         }
     }
+    
+    @Readonly fun isResearched(techName: String): Boolean = techsResearched.contains(techName)
+    @Readonly fun isResearched(construction: INonPerpetualConstruction): Boolean = construction.requiredTechs().all{ requiredTech -> isResearched(requiredTech) }
 
-    fun isResearched(techName: String): Boolean = techsResearched.contains(techName)
+    /** resources which need no research count as researched */
+    @Readonly
+    fun isRevealed(resource: TileResource): Boolean {
+        val revealedBy = resource.revealedBy ?: return true
+        return isResearched(revealedBy)
+    }
 
-    fun isResearched(construction: INonPerpetualConstruction): Boolean = construction.requiredTechs().all{ requiredTech -> isResearched(requiredTech) }
+    @Readonly fun isObsolete(unit: BaseUnit): Boolean = unit.techsThatObsoleteThis().any { obsoleteTech -> isResearched(obsoleteTech) }
 
-    fun isObsolete(unit: BaseUnit): Boolean = unit.techsThatObsoleteThis().any{ obsoleteTech -> isResearched(obsoleteTech) }
+    @Readonly
+    fun isUnresearchable(tech: Technology): Boolean {
+        if (tech.getMatchingUniques(UniqueType.OnlyAvailable, GameContext.IgnoreConditionals).any { !it.conditionalsApply(civInfo.state) })
+            return true
+        if (tech.hasUnique(UniqueType.Unavailable, civInfo.state)) return true
+        return false
+    }
 
+    @Readonly
     fun canBeResearched(techName: String): Boolean {
         val tech = getRuleset().technologies[techName]!!
-        if (tech.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals).any { !it.conditionalsApply(civInfo) })
-            return false
-        if (tech.hasUnique(UniqueType.Unavailable, StateForConditionals(civInfo))) return false
 
-        if (isResearched(tech.name) && !tech.isContinuallyResearchable())
-            return false
+        if (isUnresearchable(tech)) return false
+        if (isResearched(tech.name) && !tech.isContinuallyResearchable()) return false
 
         return tech.prerequisites.all { isResearched(it) }
     }
 
+    @Readonly fun allTechsAreResearched() = allTechsAreResearched
+
     //endregion
 
+    /** Returns empty list if no path exists */
     fun getRequiredTechsToDestination(destinationTech: Technology): List<Technology> {
         val prerequisites = mutableListOf<Technology>()
 
         val checkPrerequisites = ArrayDeque<Technology>()
+        if (isUnresearchable(destinationTech)) return listOf()
         checkPrerequisites.add(destinationTech)
 
         while (!checkPrerequisites.isEmpty()) {
             val techToCheck = checkPrerequisites.removeFirst()
+            if (isUnresearchable(techToCheck)) return listOf()
             // future tech can have been researched even when we're researching it,
             // so...if we skip it we'll end up with 0 techs in the "required techs", which will mean that we don't have anything to research. Yeah.
             if (!techToCheck.isContinuallyResearchable() &&
@@ -195,6 +209,7 @@ class TechManager : IsPartOfGameInfoSerialization {
         return prerequisites.sortedBy { it.column!!.columnNumber }
     }
 
+    @Readonly
     fun getScienceFromGreatScientist(): Int {
         // https://civilization.fandom.com/wiki/Great_Scientist_(Civ5)
         return (scienceOfLast8Turns.sum() * civInfo.gameInfo.speed.scienceCostModifier).toInt()
@@ -204,6 +219,7 @@ class TechManager : IsPartOfGameInfoSerialization {
         scienceOfLast8Turns[civInfo.gameInfo.turns % 8] = science
     }
 
+    @Readonly
     private fun limitOverflowScience(overflowScience: Int): Int {
         // http://www.civclub.net/bbs/forum.php?mod=viewthread&tid=123976
         // Apparently yes, we care about the absolute tech cost, not the actual calculated-for-this-player tech cost,
@@ -212,6 +228,7 @@ class TechManager : IsPartOfGameInfoSerialization {
                 getRuleset().technologies[currentTechnologyName()]!!.cost))
     }
 
+    @Readonly
     private fun scienceFromResearchAgreements(): Int {
         // https://forums.civfanatics.com/resources/research-agreements-bnw.25568/
         var researchAgreementModifier = 0.5f
@@ -236,7 +253,7 @@ class TechManager : IsPartOfGameInfoSerialization {
                 NotificationIcon.Science)
         }
         if (overflowScience != 0) {
-            finalScienceToAdd += getOverflowScience(currentTechnologyName()!!)
+            finalScienceToAdd += getOverflowScience()
             overflowScience = 0
         }
 
@@ -262,7 +279,7 @@ class TechManager : IsPartOfGameInfoSerialization {
      */
     fun updateResearchProgress() {
         val currentTechnology = currentTechnologyName() ?: return
-        val realOverflow = getOverflowScience(currentTechnology)
+        val realOverflow = getOverflowScience()
         val scienceSpent = researchOfTech(currentTechnology) + realOverflow
         if (scienceSpent >= costOfTech(currentTechnology)) {
             overflowScience = 0
@@ -298,23 +315,22 @@ class TechManager : IsPartOfGameInfoSerialization {
             civInfo.popupAlerts.add(PopupAlert(AlertType.TechResearched, techName))
 
         val triggerNotificationText = "due to researching [$techName]"
-        for (unique in newTech.uniqueObjects)
-            if (!unique.hasTriggerConditional() && unique.conditionalsApply(StateForConditionals(civInfo)))
+        for (unique in newTech.uniqueObjects) {
+            if (!unique.isTriggerable || unique.hasTriggerConditional() || !unique.conditionalsApply(civInfo.state))
+                continue
+            repeat(unique.getUniqueMultiplier(civInfo.state)) {
                 UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = triggerNotificationText)
+            }
+        }
 
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponResearchOld))
-            if (unique.conditionals.any {it.type == UniqueType.TriggerUponResearchOld && it.params[0] == techName})
-                UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = triggerNotificationText)
-
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponResearch))
-            if (unique.conditionals.any {it.type == UniqueType.TriggerUponResearch && newTech.matchesFilter(it.params[0]) })
-                UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = triggerNotificationText)
+        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponResearch) { newTech.matchesFilter(it.params[0], civInfo.state) })
+            UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = triggerNotificationText)
 
 
         val revealedResources = getRuleset().tileResources.values.filter { techName == it.revealedBy }
         if (civInfo.playerType == PlayerType.Human) {
             for (revealedResource in revealedResources) {
-                civInfo.gameInfo.notifyExploredResources(civInfo, revealedResource.name, 5)
+                civInfo.gameInfo.notifyExploredResources(civInfo, revealedResource, 5)
             }
         }
 
@@ -341,7 +357,8 @@ class TechManager : IsPartOfGameInfoSerialization {
     }
 
     /** A variant of kotlin's [associateBy] that omits null values */
-    private inline fun <T, K, V> Iterable<T>.associateByNotNull(keySelector: (T) -> K, valueTransform: (T) -> V?): Map<K, V> {
+    @Readonly
+    private inline fun <T, K, V> Iterable<T>.associateByNotNull(@Readonly keySelector: (T) -> K, @Readonly valueTransform: (T) -> V?): Map<K, V> {
         val destination = LinkedHashMap<K, V>()
         for (element in this) {
             val value = valueTransform(element) ?: continue
@@ -352,6 +369,7 @@ class TechManager : IsPartOfGameInfoSerialization {
 
     private fun obsoleteOldUnits(techName: String) {
         // First build a map with obsoleted units to their (nation-specific) upgrade
+        @Readonly 
         fun BaseUnit.getEquivalentUpgradeOrNull(techName: String): BaseUnit? {
             val unitUpgradesTo = automaticallyUpgradedInProductionToUnitByTech(techName)
                 ?: return null
@@ -370,13 +388,7 @@ class TechManager : IsPartOfGameInfoSerialization {
             return obsoleteUnits[old]?.name  // Replacement, or pass through null to remove from queue
         }
         for (city in civInfo.cities) {
-            // Replace queue - the sequence iteration and finalization happens before the result
-            // is reassigned, therefore no concurrent modification worries
-            city.cityConstructions.constructionQueue =
-                city.cityConstructions.constructionQueue
-                .asSequence()
-                .mapNotNull { transformConstruction(it, city) }
-                .toMutableList()
+            city.cityConstructions.transformQueue(::transformConstruction)
         }
 
         // Add notifications for obsolete units/constructions
@@ -390,7 +402,7 @@ class TechManager : IsPartOfGameInfoSerialization {
             for(city in cities)
                 city.cityConstructions.validateInProgressConstructions()
 
-            val locationAction = LocationAction(cities.asSequence().map { it.location })
+            val locationAction = LocationAction(cities.asSequence().map { it.location.toHexCoord() })
             val cityText = if (cities.size == 1) "[${cities.first().name}]"
                 else "[${cities.size}] cities"
             val newUnit = obsoleteUnits[unit]?.name
@@ -408,66 +420,60 @@ class TechManager : IsPartOfGameInfoSerialization {
         val previousEra = civInfo.getEra()
         updateEra()
         val currentEra = civInfo.getEra()
-        if (previousEra != currentEra) {
-            if(showNotification) {
-                if(!civInfo.isSpectator())
-                    civInfo.addNotification(
-                        "You have entered the [$currentEra]!",
-                        NotificationCategory.General,
-                        NotificationIcon.Science
+        if (previousEra == currentEra) return
+        
+        if (showNotification) {
+            if (!civInfo.isSpectator())
+                civInfo.addNotification(
+                    "You have entered the [$currentEra]!",
+                    NotificationCategory.General,
+                    NotificationIcon.Science
+                )
+            if (civInfo.isMajorCiv()) {
+                for (knownCiv in civInfo.getKnownCivsWithSpectators()) {
+                    knownCiv.addNotification(
+                        "[${civInfo.civName}] has entered the [$currentEra]!",
+                        NotificationCategory.General, civInfo.civName, NotificationIcon.Science
                     )
-                if (civInfo.isMajorCiv()) {
-                    for (knownCiv in civInfo.getKnownCivsWithSpectators()) {
-                        knownCiv.addNotification(
-                            "[${civInfo.civName}] has entered the [$currentEra]!",
-                            NotificationCategory.General, civInfo.civName, NotificationIcon.Science
-                        )
-                    }
-                }
-                for (policyBranch in getRuleset().policyBranches.values.filter {
-                    it.era == currentEra.name && civInfo.policies.isAdoptable(it)
-                }) {
-                    if (!civInfo.isSpectator())
-                        civInfo.addNotification(
-                            "[${policyBranch.name}] policy branch unlocked!",
-                            PolicyAction(policyBranch.name),
-                            NotificationCategory.General,
-                            NotificationIcon.Culture
-                        )
                 }
             }
-
-            val erasPassed = getRuleset().eras.values
-                .filter { it.eraNumber > previousEra.eraNumber && it.eraNumber <= currentEra.eraNumber }
-                .sortedBy { it.eraNumber }
-
-
-            for (era in erasPassed)
-                for (unique in era.uniqueObjects)
-                    if (!unique.hasTriggerConditional() && unique.conditionalsApply(StateForConditionals(civInfo)))
-                        UniqueTriggerActivation.triggerUnique(
-                            unique,
-                            civInfo,
-                            triggerNotificationText = "due to entering the [${era.name}]"
-                        )
-
-            val eraNames = erasPassed.map { it.name }.toHashSet()
-            for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnteringEra))
-                for (eraName in eraNames)
-                    if (unique.conditionals.any { it.type == UniqueType.TriggerUponEnteringEra && it.params[0] == eraName })
-                        UniqueTriggerActivation.triggerUnique(
-                            unique,
-                            civInfo,
-                            triggerNotificationText = "due to entering the [$eraName]"
-                        )
-
-            // The unfiltered version
-            for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnteringEraUnfiltered))
-                UniqueTriggerActivation.triggerUnique(
-                    unique,
-                    civInfo,
-                    triggerNotificationText = "due to entering the [${currentEra.name}]")
+            
+            for (policyBranch in getRuleset().policyBranches.values.filter {
+                it.era == currentEra.name && civInfo.policies.isAdoptable(it)
+            }) {
+                if (!civInfo.isSpectator())
+                    civInfo.addNotification(
+                        "[${policyBranch.name}] policy branch unlocked!",
+                        PolicyAction(policyBranch.name),
+                        NotificationCategory.General,
+                        NotificationIcon.Culture
+                    )
+            }
         }
+
+        val erasPassed = getRuleset().eras.values
+            .filter { it.eraNumber > previousEra.eraNumber && it.eraNumber <= currentEra.eraNumber }
+            .sortedBy { it.eraNumber }
+
+        for (era in erasPassed) {
+            for (unique in era.uniqueObjects) {
+                if (unique.hasTriggerConditional() || !unique.conditionalsApply(civInfo.state)) continue
+                repeat(unique.getUniqueMultiplier(civInfo.state)) {
+                    UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = "due to entering the [${era.name}]")
+                }
+            }
+            
+            for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnteringEra) { it.params[0] == era.name }) {
+                UniqueTriggerActivation.triggerUnique(unique, civInfo, triggerNotificationText = "due to entering the [${era.name}]")
+            }
+        }
+
+        // The unfiltered version
+        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnteringEraUnfiltered))
+            UniqueTriggerActivation.triggerUnique(
+                unique,
+                civInfo,
+                triggerNotificationText = "due to entering the [${currentEra.name}]")
     }
 
     private fun updateEra() {
@@ -522,21 +528,24 @@ class TechManager : IsPartOfGameInfoSerialization {
         movementSpeedOnRoads = if (civInfo.hasUnique(UniqueType.RoadMovementSpeed))
             RoadStatus.Road.movementImproved else RoadStatus.Road.movement
         roadsConnectAcrossRivers = civInfo.hasUnique(UniqueType.RoadsConnectAcrossRivers)
+        allTechsAreResearched = civInfo.gameInfo.ruleset.technologies.values
+            .all { isResearched(it.name) || !canBeResearched(it.name)}
     }
 
+    @Readonly
     fun getBestRoadAvailable(): RoadStatus {
         val railroadImprovement = getRuleset().railroadImprovement  // May not exist in mods
-        if (railroadImprovement != null && (railroadImprovement.techRequired == null || isResearched(railroadImprovement.techRequired!!)))
+        if (railroadImprovement != null && (railroadImprovement.techRequired == null || isResearched(railroadImprovement.techRequired!!))
+            && ImprovementFunctions.getImprovementBuildingProblems(railroadImprovement, civInfo.state).none())
             return RoadStatus.Railroad
 
         val roadImprovement = getRuleset().roadImprovement
-        if (roadImprovement != null && (roadImprovement.techRequired == null || isResearched(roadImprovement.techRequired!!)))
+        if (roadImprovement != null && (roadImprovement.techRequired == null || isResearched(roadImprovement.techRequired!!))
+            && ImprovementFunctions.getImprovementBuildingProblems(roadImprovement, civInfo.state).none())
             return RoadStatus.Road
 
         return RoadStatus.None
     }
 
-    fun canResearchTech(): Boolean {
-        return getRuleset().technologies.values.any { canBeResearched(it.name) }
-    }
+    @Readonly fun canResearchTech(): Boolean = getRuleset().technologies.values.any { canBeResearched(it.name) }
 }
