@@ -1,8 +1,11 @@
 package com.unciv.logic.civilization.managers
 
+import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
+import com.unciv.ui.screens.victoryscreen.RankingType
+import yairm210.purity.annotations.Readonly
 
 /**
  * Handles optimised operations related to finding threats or allies in an area.
@@ -27,6 +30,7 @@ class ThreatManager(val civInfo: Civilization) {
      * The result value is cached and since it is called each turn in NextTurnAutomation.getUnitPriority 
      * each subsequent calls are likely to be free.
      */
+    @Readonly @Suppress("purity") //tilesWithEnemies is implicit cache
     fun getDistanceToClosestEnemyUnit(tile: Tile, maxDist: Int, takeLargerValues: Boolean = true): Int {
         val tileData = distanceToClosestEnemyTiles[tile]
         // Needs to be a high value, but not the max value so we can still add to it. Example: nextTurnAutomation sorting
@@ -43,7 +47,7 @@ class ThreatManager(val civInfo: Civilization) {
                     else enemyTile.second.coerceAtMost(maxDist)
                 } else {
                     // This tile is no longer valid
-                    tilesWithEnemies.removeFirst()
+                    tilesWithEnemies.removeAt(0)
                 }
             }
 
@@ -81,6 +85,7 @@ class ThreatManager(val civInfo: Civilization) {
      * May be quicker than a manual search because of caching.
      * Also ends up calculating and caching [getDistanceToClosestEnemyUnit].
      */
+    @Readonly @Suppress("purity") // actually mutates cached data :think:
     fun getTilesWithEnemyUnitsInDistance(tile: Tile, maxDist: Int): MutableList<Tile> {
         val tileData = distanceToClosestEnemyTiles[tile]
 
@@ -128,16 +133,14 @@ class ThreatManager(val civInfo: Civilization) {
     }
 
     /**
-     * Returns all enemy military units within maxDistance of the tile.
+     * Returns all enemy military units on tiles
      */
-    fun getEnemyMilitaryUnitsInDistance(tile: Tile, maxDist: Int): List<MapUnit> = 
-        getEnemyUnitsOnTiles(getTilesWithEnemyUnitsInDistance(tile, maxDist))
-
+    @Readonly
     fun getEnemyUnitsOnTiles(tilesWithEnemyUnitsInDistance:List<Tile>): List<MapUnit> =
         tilesWithEnemyUnitsInDistance.flatMap { enemyTile -> enemyTile.getUnits()
             .filter { it.isMilitary() && civInfo.isAtWarWith(it.civ) } }
 
-    
+    @Readonly
     fun getDangerousTiles(unit: MapUnit, distance: Int = 3): HashSet<Tile> {
         val tilesWithEnemyUnits = getTilesWithEnemyUnitsInDistance(unit.getTile(), distance)
         val nearbyRangedEnemyUnits = getEnemyUnitsOnTiles(tilesWithEnemyUnits)
@@ -147,7 +150,7 @@ class ThreatManager(val civInfo: Civilization) {
 
         val tilesWithinBombardmentRange = tilesWithEnemyUnits
             .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
-            .flatMap { it.getTilesInDistance(it.getCity()!!.range) }
+            .flatMap { it.getTilesInDistance(it.getCity()!!.getBombardRange()) }
 
         val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(distance)
             .filter { unit.getDamageFromTerrain(it) > 0 }
@@ -158,6 +161,7 @@ class ThreatManager(val civInfo: Civilization) {
     /**
      * Returns true if the tile has a visible enemy, otherwise returns false.
      */
+    @Readonly
     fun doesTileHaveMilitaryEnemy(tile: Tile): Boolean {
         if (!tile.isExplored(civInfo)) return false
         if (tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(civInfo)) return true
@@ -169,8 +173,20 @@ class ThreatManager(val civInfo: Civilization) {
         return false
     }
 
+    /** @return a sequence of pairs of cities, the first city is our city and the second city is a nearby city that is not from our civ. */
+    @Readonly
+    fun getNeighboringCitiesOfOtherCivs(): Sequence<Pair<City,City>> = civInfo.cities.flatMap {
+        ourCity -> ourCity.neighboringCities.filter { it.civ != civInfo }.map { Pair(ourCity, it) } 
+    }.asSequence()
+
+    @Readonly fun getNeighboringCivilizations(): Set<Civilization> = civInfo.cities
+        .flatMap { it.neighboringCities }
+        .filter { it.civ != civInfo && civInfo.knows(it.civ) }
+        .map { it.civ }.toSet()
+
+    @Readonly fun getCombinedForceOfWarringCivs(): Int = civInfo.getCivsAtWarWith().sumOf { it.getStatForRanking(RankingType.Force) } 
+
     fun clear() {
         distanceToClosestEnemyTiles.clear()
     }
 }
-
