@@ -6,9 +6,10 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.Milestone
 import com.unciv.models.ruleset.Victory
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
+import yairm210.purity.annotations.Readonly
 
 class VictoryManager : IsPartOfGameInfoSerialization {
     @Transient
@@ -26,6 +27,7 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return toReturn
     }
 
+    @Readonly
     private fun calculateDiplomaticVotingResults(votesCast: HashMap<String, String?>): Counter<String> {
         val results = Counter<String>()
         // UN Owner gets 2 votes in G&K
@@ -37,20 +39,23 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return results
     }
 
+    @Readonly
     private fun getVotingCivs() = civInfo.gameInfo.civilizations.asSequence()
-        .filterNot { it.isBarbarian() || it.isSpectator() || it.isDefeated() }
+        .filterNot { it.isBarbarian || it.isSpectator() || it.isDefeated() }
 
     /** Finds the Building and Owner of the United Nations (or whatever the Mod called it)
      *  - if it's built at all and only if the owner is alive
      *  @return `first`: Building name, `second`: Owner civ name; both null if not found
      */
+    @Readonly
     fun getUNBuildingAndOwnerNames(): Pair<String?, String?> = getVotingCivs()
             .flatMap { civ -> civ.cities.asSequence()
                 .flatMap { it.cityConstructions.getBuiltBuildings() }
-                .filter { it.hasUnique(UniqueType.OneTimeTriggerVoting, stateForConditionals = StateForConditionals.IgnoreConditionals) }
-                .map { it.name to civ.civName }
+                .filter { it.hasUnique(UniqueType.OneTimeTriggerVoting, GameContext.IgnoreConditionals) }
+                .map { it.name to civ.civID }
             }.firstOrNull() ?: (null to null)
 
+    @Readonly
     private fun votesNeededForDiplomaticVictory(): Int {
         // The original counts "teams ever alive", which excludes Observer and Barbarians.
         // The "ever alive" part sounds unfair - could make a Vote unwinnable?
@@ -67,12 +72,13 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return voteCount * (67 - (1.1 * voteCount).toInt()) / 100 + 1
     }
 
+    @Readonly
     fun hasEnoughVotesForDiplomaticVictory(): Boolean {
         val results = calculateDiplomaticVotingResults(civInfo.gameInfo.diplomaticVictoryVotesCast)
         val bestCiv = results.maxByOrNull { it.value } ?: return false
 
         // If we don't have the highest score, we have not won anyway
-        if (bestCiv.key != civInfo.civName) return false
+        if (bestCiv.key != civInfo.civID) return false
 
         // If we don't have enough votes, we haven't won
         if (bestCiv.value < votesNeededForDiplomaticVictory()) return false
@@ -81,13 +87,15 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return (results.none { it != bestCiv && it.value == bestCiv.value })
     }
 
-    fun getDiplomaticVictoryVoteBreakdown(): String {
+    data class DiplomaticVictoryVoteBreakdown(val results: Counter<String>, val winnerText: String)
+    @Readonly
+    fun getDiplomaticVictoryVoteBreakdown(): DiplomaticVictoryVoteBreakdown {
         val results = calculateDiplomaticVotingResults(civInfo.gameInfo.diplomaticVictoryVotesCast)
         val (voteCount, winnerList) = results.asSequence()
             .groupBy({ it.value }, { it.key }).asSequence()
             .sortedByDescending { it.key }  // key is vote count here
             .firstOrNull()
-            ?: return "No valid votes were cast."
+            ?: return DiplomaticVictoryVoteBreakdown(results, "No valid votes were cast.")
 
         val lines = arrayListOf<String>()
         val minVotes = votesNeededForDiplomaticVictory()
@@ -99,11 +107,12 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         lines += when {
             lines.isNotEmpty() -> "No world leader was elected."
             winnerCiv == civInfo -> "You have been elected world leader!"
-            else -> "${civInfo.nation.getLeaderDisplayName()} has been elected world leader!"
+            else -> "${winnerCiv.nation.getLeaderDisplayName()} has been elected world leader!"
         }
-        return lines.joinToString("\n") { "{$it}" }
+        return DiplomaticVictoryVoteBreakdown(results, lines.joinToString("\n") { "{$it}" })
     }
 
+    @Readonly
     fun getVictoryTypeAchieved(): String? {
         if (!civInfo.isMajorCiv()) return null
         val enabledVictories = civInfo.gameInfo.gameParameters.victoryTypes
@@ -116,7 +125,8 @@ class VictoryManager : IsPartOfGameInfoSerialization {
             return Constants.neutralVictoryType
         return null
     }
-
+    
+    @Readonly
     fun getNextMilestone(victory: Victory): Milestone? {
         for (milestone in victory.milestoneObjects) {
             if (!milestone.hasBeenCompletedBy(civInfo))
@@ -125,6 +135,7 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return null
     }
 
+    @Readonly
     fun amountMilestonesCompleted(victory: Victory): Int {
         var completed = 0
         for (milestone in victory.milestoneObjects) {
@@ -136,5 +147,5 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return completed
     }
 
-    fun hasWon() = getVictoryTypeAchieved() != null
+    @Readonly fun hasWon() = getVictoryTypeAchieved() != null
 }
