@@ -7,33 +7,40 @@ import com.unciv.logic.map.BFS
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.unique.UniqueType
+import yairm210.purity.annotations.Readonly
+import java.util.EnumSet
 import kotlin.collections.set
 
 class CapitalConnectionsFinder(private val civInfo: Civilization) {
-    private val citiesReachedToMediums = HashMap<City, MutableSet<String>>()
-    private var citiesToCheck = mutableListOf(civInfo.getCapital()!!)
+    enum class CapitalConnectionMedium(
+        val roadType: RoadStatus = RoadStatus.None,
+        val checkRailroad: Boolean = false
+    ) {
+        Unknown, // future use only
+        Start(checkRailroad = true),
+        Road(RoadStatus.Road),
+        Railroad(RoadStatus.Railroad, true),
+        Harbor,
+        HarborFromRoad(RoadStatus.Road),
+        HarborFromRailroad(RoadStatus.Railroad, true),
+    }
+
+    private var citiesToCheck = listOfNotNull(civInfo.getCapital()).toMutableList()
+    private val citiesReachedToMediums = citiesToCheck.associateWithTo(
+        HashMap<City, EnumSet<CapitalConnectionMedium>>()
+    ) { city -> EnumSet.of(CapitalConnectionMedium.Start) }
     private lateinit var newCitiesToCheck: MutableList<City>
 
     private val openBordersCivCities = civInfo.gameInfo.getCities().filter { canEnterBordersOf(it.civ) }
 
-    private val harbor = "Harbor"   // hardcoding at least centralized for this class for now
-    private val road = RoadStatus.Road.name
-    private val railroad = RoadStatus.Railroad.name
-    private val harborFromRoad = "$harbor-$road"
-    private val harborFromRailroad = "$harbor-$railroad"
-
     private val ruleset = civInfo.gameInfo.ruleset
-    private val roadIsResearched = ruleset.tileImprovements[road].let {
+    private val roadIsResearched = ruleset.tileImprovements[RoadStatus.Road.name].let {
         it != null && (it.techRequired==null || civInfo.tech.isResearched(it.techRequired!!)) }
 
-    private val railroadIsResearched = ruleset.tileImprovements[railroad].let {
+    private val railroadIsResearched = ruleset.tileImprovements[RoadStatus.Railroad.name].let {
         it != null && (it.techRequired==null || civInfo.tech.isResearched(it.techRequired!!)) }
 
-    init {
-        citiesReachedToMediums[civInfo.getCapital()!!] = hashSetOf("Start")
-    }
-
-    fun find(): Map<City, Set<String>> {
+    fun find(): Map<City, EnumSet<CapitalConnectionMedium>> {
         // We map which cities we've reached, to the mediums they've been reached by -
         // this is so we know that if we've seen which cities can be connected by port A, and one
         // of those is city B, then we don't need to check the cities that B can connect to by port,
@@ -45,8 +52,8 @@ class CapitalConnectionsFinder(private val civInfo: Civilization) {
                     checkHarbor(cityToConnectFrom)
                 }
                 if (railroadIsResearched) {
-                    val mediumsReached= citiesReachedToMediums[cityToConnectFrom]!!
-                    if(mediumsReached.contains("Start") || mediumsReached.contains(railroad) || mediumsReached.contains(harborFromRailroad))
+                    val mediumsReached = citiesReachedToMediums[cityToConnectFrom]!!
+                    if (mediumsReached.any { it.checkRailroad })
                         checkRailroad(cityToConnectFrom) // This is only relevant for city connection if there is an unbreaking line from the capital
                 }
                 if (roadIsResearched) {
@@ -60,37 +67,39 @@ class CapitalConnectionsFinder(private val civInfo: Civilization) {
 
     private fun checkRoad(cityToConnectFrom: City) {
         check(
-                cityToConnectFrom,
-                transportType = road,
-                overridingTransportType = railroad,
-                tileFilter = { tile -> tile.hasConnection(civInfo) }
+            cityToConnectFrom,
+            transportType = CapitalConnectionMedium.Road,
+            overridingTransportType = CapitalConnectionMedium.Railroad,
+            tileFilter = { tile -> tile.hasConnection(civInfo) }
         )
     }
 
     private fun checkRailroad(cityToConnectFrom: City) {
         check(
-                cityToConnectFrom,
-                transportType = railroad,
-                tileFilter = { tile -> tile.getUnpillagedRoad() == RoadStatus.Railroad }
+            cityToConnectFrom,
+            transportType = CapitalConnectionMedium.Railroad,
+            tileFilter = { tile -> tile.getUnpillagedRoad() == RoadStatus.Railroad }
         )
     }
 
     private fun checkHarbor(cityToConnectFrom: City) {
         check(
-                cityToConnectFrom,
-                transportType = if(cityToConnectFrom.wasPreviouslyReached(railroad,null)) harborFromRailroad else harborFromRoad,
-                overridingTransportType = harborFromRailroad,
-                tileFilter = { tile -> tile.isWater },
-                cityFilter = { city -> city.civ == civInfo && city.containsHarbor() && !city.isBlockaded() } // use only own harbors
+            cityToConnectFrom,
+            transportType = if (cityToConnectFrom.wasPreviouslyReached(CapitalConnectionMedium.Railroad,null))
+                CapitalConnectionMedium.HarborFromRailroad else CapitalConnectionMedium.HarborFromRoad,
+            overridingTransportType = CapitalConnectionMedium.HarborFromRailroad,
+            tileFilter = { tile -> tile.isWater },
+            cityFilter = { city -> city.civ == civInfo && city.containsHarbor() && !city.isBlockaded() } // use only own harbors
         )
     }
 
-    private fun City.containsHarbor() = 
+    @Readonly
+    private fun City.containsHarbor() =
         this.containsBuildingUnique(UniqueType.ConnectTradeRoutes)
 
     private fun check(cityToConnectFrom: City,
-                      transportType: String,
-                      overridingTransportType: String? = null,
+                      transportType: CapitalConnectionMedium,
+                      overridingTransportType: CapitalConnectionMedium? = null,
                       tileFilter: (Tile) -> Boolean,
                       cityFilter: (City) -> Boolean = { true }) {
         // This is the time-saving mechanism we discussed earlier - If I arrived at this city via a certain BFS,
@@ -117,26 +126,29 @@ class CapitalConnectionsFinder(private val civInfo: Civilization) {
     private fun addCityIfFirstEncountered(reachedCity: City) {
         if (!citiesReachedToMediums.containsKey(reachedCity)) {
             newCitiesToCheck.add(reachedCity)
-            citiesReachedToMediums[reachedCity] = mutableSetOf()
+            citiesReachedToMediums[reachedCity] = EnumSet.noneOf(CapitalConnectionMedium::class.java)
         }
     }
 
-    private fun City.wasPreviouslyReached(transportType: String, overridingTransportType: String?): Boolean {
+    @Readonly
+    private fun City.wasPreviouslyReached(
+        transportType: CapitalConnectionMedium, overridingTransportType: CapitalConnectionMedium?
+    ): Boolean {
         val mediums = citiesReachedToMediums[this]!!
-        return mediums.contains(transportType) || mediums.contains(overridingTransportType)
+        return transportType in mediums || overridingTransportType in mediums
     }
 
-    private fun City.addMedium(transportType: String) {
+    private fun City.addMedium(transportType: CapitalConnectionMedium) {
         citiesReachedToMediums[this]!!.add(transportType)
     }
 
-
+    @Readonly
     private fun canEnterBordersOf(otherCiv: Civilization): Boolean {
         if (otherCiv == civInfo) return true // own borders are always open
-        if (otherCiv.isBarbarian() || civInfo.isBarbarian()) return false // barbarians blocks the routes
-        val diplomacyManager = civInfo.diplomacy[otherCiv.civName]
+        if (otherCiv.isBarbarian || civInfo.isBarbarian) return false // barbarians blocks the routes
+        val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)
             ?: return false // not encountered yet
-        if (otherCiv.isCityState() && diplomacyManager.diplomaticStatus != DiplomaticStatus.War) return true
+        if (otherCiv.isCityState && diplomacyManager.diplomaticStatus != DiplomaticStatus.War) return true
         return diplomacyManager.hasOpenBorders
     }
 
