@@ -1,6 +1,5 @@
 package com.unciv.logic.civilization.transients
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
@@ -9,19 +8,19 @@ import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.civilization.PlayerType
 import com.unciv.logic.civilization.Proximity
+import com.unciv.logic.civilization.transients.CapitalConnectionsFinder.CapitalConnectionMedium
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.MapShape
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.tile.ResourceSupplyList
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileImprovement
-import com.unciv.models.ruleset.unique.StateForConditionals
-import com.unciv.models.ruleset.unique.UniqueTarget
-import com.unciv.models.ruleset.unique.UniqueTriggerActivation
-import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unique.*
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stats
 import com.unciv.utils.DebugUtils
+import java.util.EnumSet
 
 /** CivInfo class was getting too crowded */
 class CivInfoTransientCache(val civInfo: Civilization) {
@@ -44,18 +43,23 @@ class CivInfoTransientCache(val civInfo: Civilization) {
 
     /** Contains mapping of cities to travel mediums from ALL civilizations connected by trade routes to the capital */
     @Transient
-    var citiesConnectedToCapitalToMediums = mapOf<City, Set<String>>()
+    var citiesConnectedToCapitalToMediums = mapOf<City, EnumSet<CapitalConnectionMedium>>()
+
+    fun updateState() {
+        civInfo.state = GameContext(civInfo)
+    }
 
     fun setTransients() {
         val ruleset = civInfo.gameInfo.ruleset
 
+        val state = civInfo.state
         val buildingsToRequiredResources = ruleset.buildings.values
                 .filter { civInfo.getEquivalentBuilding(it) == it }
-                .associateWith { it.requiredResources() }
+                .associateWith { it.requiredResources(state) }
 
         val unitsToRequiredResources = ruleset.units.values
                 .filter { civInfo.getEquivalentUnit(it) == it }
-                .associateWith { it.requiredResources() }
+                .associateWith { it.requiredResources(state) }
 
         for (resource in ruleset.tileResources.values.asSequence().filter { it.resourceType == ResourceType.Strategic }.map { it.name }) {
             val applicableBuildings = buildingsToRequiredResources.filter { it.value.contains(resource) }.map { it.key }
@@ -71,17 +75,17 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         }
 
         for (building in ruleset.buildings.values) {
-            if (building.uniqueTo == civInfo.civName) {
+            if (building.uniqueTo != null && civInfo.matchesFilter(building.uniqueTo!!)) {
                 uniqueBuildings.add(building)
             }
         }
 
         for (improvement in ruleset.tileImprovements.values)
-            if (improvement.uniqueTo == civInfo.civName)
+            if (improvement.uniqueTo != null && civInfo.matchesFilter(improvement.uniqueTo!!))
                 uniqueImprovements.add(improvement)
 
         for (unit in ruleset.units.values) {
-            if (unit.uniqueTo == civInfo.civName) {
+            if (unit.uniqueTo != null && civInfo.matchesFilter(unit.uniqueTo!!)) {
                 uniqueUnits.add(unit)
             }
         }
@@ -94,7 +98,7 @@ class CivInfoTransientCache(val civInfo: Civilization) {
     }
 
     // This is a big performance
-    fun updateViewableTiles(explorerPosition: Vector2? = null) {
+    fun updateViewableTiles(explorerPosition: HexCoord? = null) {
         setNewViewableTiles()
 
         updateViewableInvisibleTiles()
@@ -115,13 +119,14 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         for (tile in civInfo.viewableTiles) {
             val tileOwner = tile.getOwner()
             if (tileOwner != null) viewedCivs[tileOwner] = tile
-            for (unit in tile.getUnits()) viewedCivs[unit.civ] = tile
+            val unitOwner = tile.getFirstUnit()?.civ
+            if (unitOwner != null) viewedCivs[unitOwner] = tile
         }
 
-        if (!civInfo.isBarbarian()) {
+        if (!civInfo.isBarbarian) {
             for (entry in viewedCivs) {
                 val metCiv = entry.key
-                if (metCiv == civInfo || metCiv.isBarbarian() || civInfo.diplomacy.containsKey(metCiv.civName)) continue
+                if (metCiv == civInfo || metCiv.isBarbarian || civInfo.diplomacy.containsKey(metCiv.civID)) continue
                 civInfo.diplomacyFunctions.makeCivilizationsMeet(metCiv)
                 if(!civInfo.isSpectator())
                     civInfo.addNotification("We have encountered [${metCiv.civName}]!",
@@ -196,7 +201,7 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         newViewableTiles.addAll(civInfo.units.getCivUnits().flatMap { unit -> unit.viewableTiles.asSequence().filter { it.getOwner() != civInfo } })
 
         for (otherCiv in civInfo.getKnownCivs()) {
-            if (otherCiv.getAllyCiv() == civInfo.civName || otherCiv.civName == civInfo.getAllyCiv()) {
+            if (otherCiv.allyCiv == civInfo || otherCiv == civInfo.allyCiv) {
                 newViewableTiles.addAll(otherCiv.cities.asSequence().flatMap { it.getTiles() })
             }
         }
@@ -209,12 +214,8 @@ class CivInfoTransientCache(val civInfo: Civilization) {
     private fun updateLastSeenImprovements() {
         if (civInfo.playerType == PlayerType.AI) return // don't bother for AI, they don't really use the info anyway
 
-        for (tile in civInfo.viewableTiles) {
-            if (tile.improvement == null)
-                civInfo.lastSeenImprovement.remove(tile.position)
-            else
-                civInfo.lastSeenImprovement[tile.position] = tile.improvement!!
-        }
+        for (tile in civInfo.viewableTiles)
+            civInfo.setLastSeenImprovement(tile.position, tile.improvement)
     }
 
     /** Visible for DevConsole use only */
@@ -258,28 +259,6 @@ class CivInfoTransientCache(val civInfo: Civilization) {
                     statsGained.add(firstDiscoveredBonus)
             }
 
-            // Variable for support of twooo deprecated uniques
-            var goldGained = 0
-
-            // Support for depreciated GoldWhenDiscoveringNaturalWonder unique
-            for (unique in civInfo.getMatchingUniques(UniqueType.GoldWhenDiscoveringNaturalWonder)) {
-
-                goldGained += if (discoveredNaturalWonders.contains(tile.naturalWonder!!)) {
-                    100
-                } else {
-                    500
-                }
-            }
-
-            // Support for depreciated GrantsGoldToFirstToDiscover unique
-            if (tile.terrainHasUnique(UniqueType.GrantsGoldToFirstToDiscover)
-                && !discoveredNaturalWonders.contains(tile.naturalWonder!!)) {
-
-                for (unique in tile.getTerrainMatchingUniques(UniqueType.GoldWhenDiscoveringNaturalWonder)) {
-                    goldGained += 500
-                }
-            }
-
             var naturalWonder: String? = null
 
             if (!statsGained.isEmpty()) {
@@ -293,18 +272,8 @@ class CivInfoTransientCache(val civInfo: Civilization) {
                     )
             }
 
-            if (goldGained > 0) {
-                naturalWonder = tile.naturalWonder
-            }
-
-            if (goldGained > 0 && naturalWonder != null) {
-                civInfo.addGold(goldGained)
-                civInfo.addNotification("We have received [$goldGained] Gold for discovering [${naturalWonder}]",
-                    Notification.NotificationCategory.General, NotificationIcon.Gold)
-            }
-
             for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponDiscoveringNaturalWonder,
-                StateForConditionals(civInfo, tile = tile)
+                GameContext(civInfo, tile = tile)
             ))
                 UniqueTriggerActivation.triggerUnique(unique, civInfo, tile=tile, triggerNotificationText = "due to discovering a Natural Wonder")
         }
@@ -320,19 +289,15 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         if (civInfo.cities.isEmpty()) return // No cities to connect
 
         val oldConnectedCities = if (initialSetup)
-            civInfo.cities.filter { it.connectedToCapitalStatus == City.ConnectedToCapitalStatus.`true` }
+            civInfo.cities.filterTo(mutableSetOf()) { it.connectedToCapitalStatus }
             else citiesConnectedToCapitalToMediums.keys
-        val oldMaybeConnectedCities = if (initialSetup)
-            civInfo.cities.filter { it.connectedToCapitalStatus != City.ConnectedToCapitalStatus.`false` }
-        else citiesConnectedToCapitalToMediums.keys
 
-        citiesConnectedToCapitalToMediums = if (civInfo.getCapital() == null) mapOf()
-        else CapitalConnectionsFinder(civInfo).find()
+        citiesConnectedToCapitalToMediums = CapitalConnectionsFinder(civInfo).find()
 
         val newConnectedCities = citiesConnectedToCapitalToMediums.keys
 
         for (city in newConnectedCities)
-            if (city !in oldMaybeConnectedCities && city.civ == civInfo && city != civInfo.getCapital())
+            if (city !in oldConnectedCities && city.civ == civInfo && city != civInfo.getCapital())
                 civInfo.addNotification("[${city.name}] has been connected to your capital!",
                     city.location, NotificationCategory.Cities, NotificationIcon.Gold
                 )
@@ -345,23 +310,22 @@ class CivInfoTransientCache(val civInfo: Civilization) {
                 )
 
         for (city in civInfo.cities)
-            city.connectedToCapitalStatus = if (city in newConnectedCities)
-                City.ConnectedToCapitalStatus.`true` else City.ConnectedToCapitalStatus.`false`
+            city.connectedToCapitalStatus = city in newConnectedCities
     }
 
     fun updateCivResources() {
         val newDetailedCivResources = ResourceSupplyList()
         for (city in civInfo.cities) newDetailedCivResources.add(city.getResourcesGeneratedByCity())
 
-        if (!civInfo.isCityState()) {
+        if (!civInfo.isCityState) {
             // First we get all these resources of each city state separately
             val cityStateProvidedResources = ResourceSupplyList()
             var resourceBonusPercentage = 1f
             for (unique in civInfo.getMatchingUniques(UniqueType.CityStateResources))
                 resourceBonusPercentage += unique.params[0].toFloat() / 100
-            for (cityStateAlly in civInfo.getKnownCivs().filter { it.getAllyCiv() == civInfo.civName }) {
+            for (cityStateAlly in civInfo.getKnownCivs().filter { it.allyCiv == civInfo }) {
                 for (resourceSupply in cityStateAlly.cityStateFunctions.getCityStateResourcesForAlly()) {
-                    if (resourceSupply.resource.hasUnique(UniqueType.CannotBeTraded, StateForConditionals(cityStateAlly))) continue
+                    if (resourceSupply.resource.hasUnique(UniqueType.CannotBeTraded, cityStateAlly.state)) continue
                     val newAmount = (resourceSupply.amount * resourceBonusPercentage).toInt()
                     cityStateProvidedResources.add(resourceSupply.copy(amount = newAmount))
                 }
@@ -387,13 +351,21 @@ class CivInfoTransientCache(val civInfo: Civilization) {
             newDetailedCivResources.subtractResourceRequirements(
                 unit.getResourceRequirementsPerTurn(), civInfo.gameInfo.ruleset, "Units")
 
-        newDetailedCivResources.removeAll { it.resource.hasUnique(UniqueType.CityResource) }
+        newDetailedCivResources.removeAll { it.resource.isCityWide }
 
         // Check if anything has actually changed so we don't update stats for no reason - this uses List equality which means it checks the elements
         if (civInfo.detailedCivResources == newDetailedCivResources) return
 
+        val summarizedResourceSupply = newDetailedCivResources.sumByResource("All")
+
+        val newResourceUniqueMap = UniqueMap()
+        for (resource in summarizedResourceSupply)
+            if (resource.amount > 0)
+                newResourceUniqueMap.addUniques(resource.resource.uniqueObjects)
+        
         civInfo.detailedCivResources = newDetailedCivResources
-        civInfo.summarizedCivResourceSupply = newDetailedCivResources.sumByResource("All")
+        civInfo.summarizedCivResourceSupply = summarizedResourceSupply
+        civInfo.civResourcesUniqueMap = newResourceUniqueMap
 
         civInfo.updateStatsForNextTurn() // More or less resources = more or less happiness, with potential domino effects
     }
@@ -404,11 +376,11 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         if (preCalculated != null) {
             // We usually want to update this for a pair of civs at the same time
             // Since this function *should* be symmetrical for both civs, we can just do it once
-            civInfo.proximity[otherCiv.civName] = preCalculated
+            civInfo.proximity[otherCiv.civID] = preCalculated
             return preCalculated
         }
         if (civInfo.cities.isEmpty() || otherCiv.cities.isEmpty()) {
-            civInfo.proximity[otherCiv.civName] = Proximity.None
+            civInfo.proximity[otherCiv.civID] = Proximity.None
             return Proximity.None
         }
 
@@ -467,7 +439,7 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         if (numMajors <= 4 && proximity > Proximity.Far)
             proximity = Proximity.Far
 
-        civInfo.proximity[otherCiv.civName] = proximity
+        civInfo.proximity[otherCiv.civID] = proximity
 
         return proximity
     }
