@@ -3,44 +3,47 @@ package com.unciv.logic.files
 import com.badlogic.gdx.Files
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
+import com.badlogic.gdx.scenes.scene2d.ui.TextField
 import com.badlogic.gdx.utils.GdxRuntimeException
 import com.badlogic.gdx.utils.JsonReader
 import com.badlogic.gdx.utils.SerializationException
-import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
-import com.unciv.logic.CompatibilityVersion
+import com.unciv.logic.BackwardCompatibility.migrateCivID
 import com.unciv.logic.GameInfo
 import com.unciv.logic.GameInfoPreview
-import com.unciv.logic.GameInfoSerializationVersion
-import com.unciv.logic.HasGameInfoSerializationVersion
 import com.unciv.logic.UncivShowableException
-import com.unciv.logic.civilization.PlayerType
-import com.unciv.logic.civilization.managers.TurnManager
 import com.unciv.models.metadata.GameSettings
 import com.unciv.models.metadata.doMigrations
 import com.unciv.models.metadata.isMigrationNecessary
 import com.unciv.models.ruleset.RulesetCache
+import com.unciv.ui.screens.modmanager.ModUIData
 import com.unciv.ui.screens.savescreens.Gzip
+import com.unciv.logic.CompatibilityVersion
 import com.unciv.utils.Concurrency
+import com.unciv.logic.GameInfoSerializationVersion
+import com.unciv.logic.HasGameInfoSerializationVersion
 import com.unciv.utils.Log
 import com.unciv.utils.debug
 import kotlinx.coroutines.Job
-import java.io.File
 import java.io.Writer
 
 private const val SAVE_FILES_FOLDER = "SaveFiles"
 private const val MULTIPLAYER_FILES_FOLDER = "MultiplayerGames"
 private const val AUTOSAVE_FILE_NAME = "Autosave"
 const val SETTINGS_FILE_NAME = "GameSettings.json"
+const val MOD_LIST_CACHE_FILE_NAME = "ModListCache.json"
 
 class UncivFiles(
     /**
      * This is necessary because the Android turn check background worker does not hold any reference to the actual [com.badlogic.gdx.Application],
      * which is normally responsible for keeping the [Gdx] static variables from being garbage collected.
      */
-    private val files: Files
+    private val files: Files,
+
+    /** If not null, this is the path to the directory in which to store the local files - mods, saves, maps, etc */
+    val customDataDirectory: String? = null
 ) {
     init {
         debug("Creating UncivFiles, localStoragePath: %s, externalStoragePath: %s",
@@ -51,6 +54,17 @@ class UncivFiles(
 
     //region Helpers
 
+    fun getLocalFile(fileName: String): FileHandle {
+        return if (customDataDirectory == null) files.local(fileName)
+        else files.absolute(customDataDirectory).child(fileName)
+    }
+
+    fun getModsFolder() = getLocalFile("mods")
+    fun getModFolder(modName: String): FileHandle = getModsFolder().child(modName)
+
+    /** The folder that holds data that the game changes while running - all the mods, maps, save files, etc */
+    fun getDataFolder() = getLocalFile("")
+
     fun getSave(gameName: String): FileHandle {
         return getSave(SAVE_FILES_FOLDER, gameName)
     }
@@ -62,13 +76,15 @@ class UncivFiles(
         debug("Getting save %s from folder %s, preferExternal: %s",
             gameName, saveFolder, preferExternalStorage, files.externalStoragePath)
         val location = "${saveFolder}/$gameName"
-        val localFile = files.local(location)
+        val localFile = getLocalFile(location)
         val externalFile = files.external(location)
 
         val toReturn = if (files.isExternalStorageAvailable && (
                 externalFile.exists() && !localFile.exists() || // external file is only valid choice
                 preferExternalStorage && (externalFile.exists() || !localFile.exists()) // unless local file is only valid choice, choose external
                 ) ) {
+            if (externalFile.isDirectory) externalFile.deleteDirectory() // fix for #13113
+            externalFile.parent().mkdirs()
             externalFile
         } else {
             localFile
@@ -82,13 +98,13 @@ class UncivFiles(
      * @throws GdxRuntimeException if the [path] represents a directory
      */
     fun fileWriter(path: String, append: Boolean = false): Writer {
-        val file = pathToFileHandler(path)
+        val file = pathToFileHandle(path)
         return file.writer(append, Charsets.UTF_8.name())
     }
 
-    fun pathToFileHandler(path: String): FileHandle {
+    fun pathToFileHandle(path: String): FileHandle {
         return if (preferExternalStorage && files.isExternalStorageAvailable) files.external(path)
-        else files.local(path)
+        else getLocalFile(path)
     }
 
 
@@ -98,20 +114,21 @@ class UncivFiles(
 
     fun getSaves(autoSaves: Boolean = true): Sequence<FileHandle> {
         val saves = getSaves(SAVE_FILES_FOLDER)
-        val filteredSaves = if (autoSaves) { saves } else { saves.filter { !it.name().startsWith(
-            AUTOSAVE_FILE_NAME
-        ) }}
-        return filteredSaves
+        if (autoSaves) return saves
+        return saves.filter { !it.name().startsWith(AUTOSAVE_FILE_NAME) }
     }
 
     private fun getSaves(saveFolder: String): Sequence<FileHandle> {
         debug("Getting saves from folder %s, externalStoragePath: %s", saveFolder, files.externalStoragePath)
-        val localFiles = files.local(saveFolder).list().asSequence()
+        // This construct instead of asSequence causes the actual list() to happen when the
+        // first element is pulled, not right now before a Sequence is wrapped around the result.
+        // Note that any performance gains are moot when logging is on: See the the `debug` below.
+        val localFiles = Sequence { getLocalFile(saveFolder).list().iterator() }
 
-        val externalFiles = if (files.isExternalStorageAvailable && files.local("").file().absolutePath != files.external("").file().absolutePath) {
-            files.external(saveFolder).list().asSequence()
-        } else {
-            emptySequence()
+        val externalFiles = when {
+            !files.isExternalStorageAvailable -> emptySequence()
+            getDataFolder().file().absolutePath == files.external("").file().absolutePath -> emptySequence()
+            else -> Sequence { files.external(saveFolder).list().iterator() }
         }
 
         debug("Local files: %s, external files: %s",
@@ -128,14 +145,6 @@ class UncivFiles(
         return deleteSave(getSave(gameName))
     }
 
-    /**
-     * @return `true` if successful.
-     * @throws SecurityException when delete access was denied
-     */
-    fun deleteMultiplayerSave(gameName: String): Boolean {
-        return deleteSave(getMultiplayerSave(gameName))
-    }
-
     /**
      * Only use this with a [FileHandle] obtained by one of the methods of this class!
      *
@@ -148,9 +157,14 @@ class UncivFiles(
     }
 
     //endregion
+
     //region Saving
 
-    fun saveGame(game: GameInfo, gameName: String, saveCompletionCallback: (Exception?) -> Unit = { if (it != null) throw it }): FileHandle {
+    private fun rethrowIfNotNull(ex: Exception?) {
+        if (ex != null) throw ex
+    }
+
+    fun saveGame(game: GameInfo, gameName: String, saveCompletionCallback: (Exception?) -> Unit = ::rethrowIfNotNull): FileHandle {
         val file = getSave(gameName)
         saveGame(game, file, saveCompletionCallback)
         return file
@@ -159,7 +173,7 @@ class UncivFiles(
     /**
      * Only use this with a [FileHandle] obtained by one of the methods of this class!
      */
-    fun saveGame(game: GameInfo, file: FileHandle, saveCompletionCallback: (Exception?) -> Unit = { if (it != null) throw it }) {
+    fun saveGame(game: GameInfo, file: FileHandle, saveCompletionCallback: (Exception?) -> Unit = ::rethrowIfNotNull) {
         try {
             debug("Saving GameInfo %s to %s", game.gameId, file.path())
             val string = gameInfoToString(game)
@@ -173,16 +187,16 @@ class UncivFiles(
     /**
      * Overload of function saveGame to save a GameInfoPreview in the MultiplayerGames folder
      */
-    fun saveGame(game: GameInfoPreview, gameName: String, saveCompletionCallback: (Exception?) -> Unit = { if (it != null) throw it }): FileHandle {
+    fun saveMultiplayerGamePreview(game: GameInfoPreview, gameName: String, saveCompletionCallback: (Exception?) -> Unit = ::rethrowIfNotNull): FileHandle {
         val file = getMultiplayerSave(gameName)
-        saveGame(game, file, saveCompletionCallback)
+        saveMultiplayerGamePreview(game, file, saveCompletionCallback)
         return file
     }
 
     /**
      * Only use this with a [FileHandle] obtained by one of the methods of this class!
      */
-    fun saveGame(game: GameInfoPreview, file: FileHandle, saveCompletionCallback: (Exception?) -> Unit = { if (it != null) throw it }) {
+    fun saveMultiplayerGamePreview(game: GameInfoPreview, file: FileHandle, saveCompletionCallback: (Exception?) -> Unit = ::rethrowIfNotNull) {
         try {
             debug("Saving GameInfoPreview %s to %s", game.gameId, file.path())
             json().toJson(game, file)
@@ -205,7 +219,7 @@ class UncivFiles(
         onSaved: () -> Unit,
         onError: (Exception) -> Unit
     ) {
-        val saveLocation = game.customSaveLocation ?: Gdx.files.local(gameName).path()
+        val saveLocation = game.customSaveLocation ?: UncivGame.Current.files.getLocalFile(gameName).path()
 
         try {
             val data = gameInfoToString(game)
@@ -239,11 +253,11 @@ class UncivFiles(
         return gameInfoFromString(gameData)
     }
 
-    fun loadGamePreviewByName(gameName: String) =
-            loadGamePreviewFromFile(getMultiplayerSave(gameName))
-
     fun loadGamePreviewFromFile(gameFile: FileHandle): GameInfoPreview {
-        return json().fromJson(GameInfoPreview::class.java, gameFile) ?: throw emptyFile(gameFile)
+        val preview = json().fromJson(GameInfoPreview::class.java, gameFile)
+            ?: throw emptyFile(gameFile)
+        preview.migrateCivID()
+        return preview
     }
 
     /**
@@ -282,11 +296,12 @@ class UncivFiles(
 
 
     //endregion
+
     //region Settings
 
     private fun getGeneralSettingsFile(): FileHandle {
         return if (UncivGame.Current.isConsoleMode) FileHandle(SETTINGS_FILE_NAME)
-        else files.local(SETTINGS_FILE_NAME)
+        else getLocalFile(SETTINGS_FILE_NAME)
     }
 
     fun getGeneralSettings(): GameSettings {
@@ -313,9 +328,11 @@ class UncivFiles(
         getGeneralSettingsFile().writeString(json().toJson(gameSettings), false, Charsets.UTF_8.name())
     }
 
-    val scenarioFolder = "scenarios"
-    fun getScenarioFiles() = sequence {
+    //endregion
 
+    //region Scenarios
+    private val scenarioFolder = "scenarios"
+    fun getScenarioFiles() = sequence {
         for (mod in RulesetCache.values) {
             val modFolder = mod.folderLocation ?: continue
             val scenarioFolder = modFolder.child(scenarioFolder)
@@ -324,22 +341,35 @@ class UncivFiles(
                     yield(Pair(file, mod))
         }
     }
+    //endregion
 
-    fun loadScenario(gameFile: FileHandle): GameInfo {
-        val game = loadGameFromFile(gameFile)
-        game.civilizations.removeAll { it.isSpectator() }
-        for (civ in game.civilizations)
-            civ.diplomacy.remove(Constants.spectator)
-        if (game.civilizations.none { it.isHuman() })
-            game.civilizations.first { it.isMajorCiv() }.playerType = PlayerType.Human
-
-        game.currentPlayerCiv = game.civilizations.first { it.playerType == PlayerType.Human }
-        game.currentPlayer = game.currentPlayerCiv.civName
-        TurnManager(game.currentPlayerCiv).startTurn()
-
-        return game
+    //region Mod caching
+    fun saveModCache(modDataList: List<ModUIData>) {
+        val file = getLocalFile(MOD_LIST_CACHE_FILE_NAME)
+        try {
+            json().toJson(modDataList, file)
+        }
+        catch (ex: Exception){ // Not a huge deal if this fails
+            Log.error("Error saving mod cache", ex)
+        }
     }
 
+
+    fun loadModCache(): Iterable<ModUIData> {
+        val file = getLocalFile(MOD_LIST_CACHE_FILE_NAME)
+        if (!file.exists()) return emptyList()
+        try {
+            return json().fromJsonFile(Array<ModUIData>::class.java, file).asIterable()
+        }
+        catch (ex: Exception){ // Not a huge deal if this fails
+            Log.error("Error loading mod cache", ex)
+            return emptyList()
+        }
+    }
+
+
+    //endregion
+
     companion object {
 
         var saveZipped = false
@@ -354,30 +384,32 @@ class UncivFiles(
          * Platform dependent saver-loader to custom system locations
          */
         var saverLoader: PlatformSaverLoader = PlatformSaverLoader.None
-            get() {
-                if (field.javaClass.simpleName == "DesktopSaverLoader" && LinuxX11SaverLoader.isRequired())
-                    field = LinuxX11SaverLoader()
-                return field
-            }
 
         /** Specialized function to access settings before Gdx is initialized.
          *
-         * @param base Path to the directory where the file should be - if not set, the OS current directory is used (which is "/" on Android)
+         * @param baseDirectory Path to the directory where the file should be - if not set, the OS current directory is used (which is "/" on Android)
          */
-        fun getSettingsForPlatformLaunchers(base: String = "."): GameSettings {
+        fun getSettingsForPlatformLaunchers(baseDirectory: String): GameSettings {
             // FileHandle is Gdx, but the class and JsonParser are not dependent on app initialization
             // In fact, at this point Gdx.app or Gdx.files are null but this still works.
-            val file = FileHandle(base + File.separator + SETTINGS_FILE_NAME)
-            return if (file.exists()) json().fromJsonFile(GameSettings::class.java, file)
-            else GameSettings().apply { isFreshlyCreated = true }
+            val file = FileHandle(baseDirectory).child(SETTINGS_FILE_NAME)
+            if (file.exists()){
+                try {
+                    return json().fromJson(GameSettings::class.java, file)
+                } catch (ex: Exception) {
+                    Log.error("Exception while deserializing GameSettings JSON", ex)
+                }
+            }
+            return GameSettings().apply { isFreshlyCreated = true }
         }
 
         /** @throws IncompatibleGameInfoVersionException if the [gameData] was created by a version of this game that is incompatible with the current one. */
         fun gameInfoFromString(gameData: String): GameInfo {
+            val fixedData = gameData.trim().replace("\r", "").replace("\n", "")
             val unzippedJson = try {
-                Gzip.unzip(gameData.trim())
-            } catch (_: Exception) {
-                gameData.trim()
+                Gzip.unzip(fixedData)
+            } catch (ex: Exception) {
+                fixedData
             }
             val gameInfo = try {
                 json().fromJson(GameInfo::class.java, unzippedJson)
@@ -387,7 +419,7 @@ class UncivFiles(
                 throw IncompatibleGameInfoVersionException(onlyVersion.version, ex)
             } ?: throw UncivShowableException("The file data seems to be corrupted.")
 
-            if (gameInfo.version > GameInfo.CURRENT_COMPATIBILITY_VERSION) {
+            if (gameInfo.version > CompatibilityVersion.CURRENT_COMPATIBILITY_VERSION) {
                 // this means there wasn't an immediate error while serializing, but this version will cause other errors later down the line
                 throw IncompatibleGameInfoVersionException(gameInfo.version)
             }
@@ -400,15 +432,18 @@ class UncivFiles(
          * @throws SerializationException
          */
         fun gameInfoPreviewFromString(gameData: String): GameInfoPreview {
-            return json().fromJson(GameInfoPreview::class.java, Gzip.unzip(gameData))
+            val preview = json().fromJson(GameInfoPreview::class.java, Gzip.unzip(gameData))
+            preview.migrateCivID()
+            return preview
         }
 
         /** Returns gzipped serialization of [game], optionally gzipped ([forceZip] overrides [saveZipped]) */
-        fun gameInfoToString(game: GameInfo, forceZip: Boolean? = null, updateChecksum:Boolean=false): String {
-            game.version = GameInfo.CURRENT_COMPATIBILITY_VERSION
+        fun gameInfoToString(game: GameInfo, forceZip: Boolean? = null, updateChecksum: Boolean = false): String {
+            game.version = CompatibilityVersion.CURRENT_COMPATIBILITY_VERSION
 
             if (updateChecksum) game.checksum = game.calculateChecksum()
             val plainJson = json().toJson(game)
+
             return if (forceZip ?: saveZipped) Gzip.zip(plainJson) else plainJson
         }
 
@@ -417,6 +452,25 @@ class UncivFiles(
             return Gzip.zip(json().toJson(game))
         }
 
+        private val charsForbiddenInFileNames = setOf('\\', '/', ':')
+        private val _fileNameTextFieldFilter = TextField.TextFieldFilter { _, char ->
+            char !in charsForbiddenInFileNames
+        }
+        /** Check characters typed into a file name TextField: Disallows both Unix and Windows path separators, plus the
+         *  ['NTFS alternate streams'](https://learn.microsoft.com/en-us/openspecs/windows_protocols/ms-fscc/b134f29a-6278-4f3f-904f-5e58a713d2c5)
+         *  indicator, irrespective of platform, in case players wish to exchange files cross-platform.
+         *  @see isValidFileName
+         *  @return A `TextFieldFilter` appropriate for `TextField`s used to enter a file name for saving
+         */
+        fun fileNameTextFieldFilter() = _fileNameTextFieldFilter
+
+        /** Determines whether a filename is acceptable.
+         *  - Forbids trailing blanks because Windows has trouble with them.
+         *  - Forbids leading blanks because they might confuse users (neither Windows nor Linux have noteworthy problems with them).
+         *  - Does **not** deal with problems that can be recognized inspecting a single character, use [fileNameTextFieldFilter] for that.
+         *  @param  fileName A base file name, not a path.
+         */
+        fun isValidFileName(fileName: String) = fileName.isNotEmpty() && !fileName.endsWith(' ') && !fileName.startsWith(' ')
     }
 }
 
@@ -446,6 +500,9 @@ class Autosaves(val files: UncivFiles) {
     }
 
     fun autoSave(gameInfo: GameInfo, nextTurn: Boolean = false) {
+        // get GameSettings to check the maxAutosavesStored in the autoSave function
+        val settings = files.getGeneralSettings()
+
         try {
             files.saveGame(gameInfo, AUTOSAVE_FILE_NAME)
         } catch (oom: OutOfMemoryError) {
@@ -453,32 +510,40 @@ class Autosaves(val files: UncivFiles) {
             return  // not much we can do here
         }
 
-        // keep auto-saves for the last 10 turns for debugging purposes
-        if (nextTurn) {
-            val newAutosaveFilename =
-                SAVE_FILES_FOLDER + File.separator + AUTOSAVE_FILE_NAME + "-${gameInfo.currentPlayer}-${gameInfo.turns}"
-            val file = files.pathToFileHandler(newAutosaveFilename)
-            files.getSave(AUTOSAVE_FILE_NAME).copyTo(file)
+        if (!nextTurn) return
 
-            fun getAutosaves(): Sequence<FileHandle> {
-                return files.getSaves().filter { it.name().startsWith(AUTOSAVE_FILE_NAME) }
-            }
-            while (getAutosaves().count() > 10) {
-                val saveToDelete = getAutosaves().minByOrNull { it.lastModified() }!!
-                files.deleteSave(saveToDelete.name())
-            }
-        }
+        // keep auto-saves for the last `settings.maxAutosavesStored` turns
+        val newAutosaveFile = files.pathToFileHandle(SAVE_FILES_FOLDER)
+            .child("$AUTOSAVE_FILE_NAME-${gameInfo.currentPlayer}-${gameInfo.turns}")
+        files.getSave(AUTOSAVE_FILE_NAME).copyTo(newAutosaveFile)
+
+        purgeOldAutosaves(settings.maxAutosavesStored)
+    }
+
+    private fun purgeOldAutosaves(maxAutosavesStored: Int) {
+        // Since the latest numbered autosave is a copy of the un-numbered autosave file,
+        // we add 1 so `maxAutosavesStored` numbered files AND the un-numbered one are kept (see #12790).
+        val oldAutoSaves = files.getSaves()
+            .filter { it.name().startsWith(AUTOSAVE_FILE_NAME) }
+            .sortedByDescending { it.lastModified() }   // youngest to the top
+            .drop(maxAutosavesStored + 1)  // dropping those we want to keep
+            // The following only has an effect if an exception is thrown halfway through the list
+            // We delete the oldest first - principle of least surprise
+            .asIterable().reversed()
+        for (file in oldAutoSaves)
+            files.deleteSave(file)
     }
 
     fun loadLatestAutosave(): GameInfo {
         return try {
             files.loadGameByName(AUTOSAVE_FILE_NAME)
         } catch (_: Exception) {
-            // silent fail if we can't read the autosave for any reason - try to load the last autosave by turn number first
-            val autosaves = files.getSaves().filter { it.name() != AUTOSAVE_FILE_NAME && it.name().startsWith(
-                AUTOSAVE_FILE_NAME
-            ) }
-            files.loadGameFromFile(autosaves.maxByOrNull { it.lastModified() }!!)
+            // silent fail if we can't read the autosave for any reason - try to load the last autosave by timestamp first
+            val autosaves = files.getSaves().filter {
+                it.name() != AUTOSAVE_FILE_NAME &&
+                it.name().startsWith(AUTOSAVE_FILE_NAME)
+            }
+            files.loadGameFromFile(autosaves.maxBy { it.lastModified() })
         }
     }
 
