@@ -1,9 +1,10 @@
 package com.unciv.logic.map
 
+import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.mapunit.MapUnit
-import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.utils.Log
+import yairm210.purity.annotations.Readonly
 
 //TODO: Eventually, all path generation in the game should be moved into here.
 object MapPathing {
@@ -13,26 +14,31 @@ object MapPathing {
      * to upgrade it to a railroad, we consider it to be a railroad for pathing since it will be upgraded.
      * Otherwise, we set every tile to have equal value since building a road on any of them makes the original movement cost irrelevant.
      */
-    private fun roadPreferredMovementCost(unit: MapUnit, from: Tile, to: Tile): Float{
+    @Suppress("UNUSED_PARAMETER") // While `from` is unused, this function should stay close to the signatures expected by the AStar and getPath `heuristic` parameter.
+    @Readonly
+    private fun roadPreferredMovementCost(civ: Civilization, from: Tile, to: Tile): Float{
         // hasRoadConnection accounts for civs that treat jungle/forest as roads
         // Ignore road over river penalties.
-        if ((to.hasRoadConnection(unit.civ, false) || to.hasRailroadConnection(false)))
+        if ((to.hasRoadConnection(civ, false) || to.hasRailroadConnection(false)))
             return .5f
 
         return 1f
     }
 
-    fun isValidRoadPathTile(unit: MapUnit, tile: Tile): Boolean {
-        val roadImprovement = tile.ruleset.roadImprovement ?: return false
-        val railRoadImprovement = tile.ruleset.railroadImprovement ?: return false
-        return tile.isLand
-            && !tile.isImpassible()
-            && unit.civ.hasExplored(tile)
-            && tile.canCivPassThrough(unit.civ)
-            && (tile.hasRoadConnection(unit.civ, false) 
-                || tile.hasRailroadConnection(false) 
-                || tile.improvementFunctions.canBuildImprovement(roadImprovement, unit.civ))
-                || tile.improvementFunctions.canBuildImprovement(railRoadImprovement, unit.civ)
+    @Readonly
+    fun isValidRoadPathTile(civ: Civilization, tile: Tile): Boolean {
+        val roadImprovement = tile.ruleset.roadImprovement
+        val railRoadImprovement = tile.ruleset.railroadImprovement
+        
+        if (tile.isWater) return false
+        if (tile.isImpassible()) return false
+        if (!civ.hasExplored(tile)) return false
+        if (!tile.canCivPassThrough(civ)) return false
+        
+        return tile.hasRoadConnection(civ, false)
+                || tile.hasRailroadConnection(false)
+                || roadImprovement != null && tile.improvementFunctions.canBuildImprovement(roadImprovement, civ.state)
+                || railRoadImprovement != null && tile.improvementFunctions.canBuildImprovement(railRoadImprovement,civ.state)
     }
 
     /**
@@ -40,19 +46,19 @@ object MapPathing {
      *
      * This function uses the A* search algorithm to find an optimal path for road construction between two specified tiles.
      *
-     * @param unit The unit that will construct the road.
+     * @param civ The civlization that will construct the road.
      * @param startTile The starting tile of the path.
      * @param endTile The destination tile of the path.
      * @return A sequence of tiles representing the path from startTile to endTile, or null if no valid path is found.
      */
-    fun getRoadPath(unit: MapUnit, startTile: Tile, endTile: Tile): List<Tile>?{
-        return getPath(unit,
+    @Readonly
+    fun getRoadPath(civ: Civilization, startTile: Tile, endTile: Tile): List<Tile>? {
+        return getConnection(civ,
             startTile,
             endTile,
             ::isValidRoadPathTile,
-            ::roadPreferredMovementCost,
-            {_, _, _ -> 0f}
-            )
+            ::roadPreferredMovementCost
+        ) { _, _, _ -> 0f }
     }
 
     /**
@@ -70,6 +76,7 @@ object MapPathing {
      * It takes a MapUnit, a 'from' Tile, and a 'to' Tile, returning a Float value representing the heuristic cost estimate.
      * @return A list of tiles representing the path from the startTile to the endTile. Returns null if no valid path is found.
      */
+    @Readonly
     private fun getPath(unit: MapUnit,
                 startTile: Tile,
                 endTile: Tile,
@@ -83,7 +90,7 @@ object MapPathing {
         while (true) {
             if (astar.hasEnded()) {
                 // We failed to find a path
-                Log.debug("getRoadPath failed at AStar search size ${astar.size()}")
+                Log.debug("getPath failed at AStar search size ${astar.size()}")
                 return null
             }
             if (!astar.hasReachedTile(endTile)) {
@@ -97,4 +104,39 @@ object MapPathing {
         }
     }
 
+    /**
+     * Gets the connection to the end tile. This does not take into account tile movement costs.
+     * Takes in a civilization instead of a specific unit.
+     */
+    @Readonly
+    fun getConnection(civ: Civilization,
+        startTile: Tile,
+        endTile: Tile,
+        predicate: (Civilization, Tile) -> Boolean,
+        cost: (Civilization, Tile, Tile) -> Float = { _, _, _ -> 1f },
+        heuristic: (Civilization, Tile, Tile) -> Float = { _, from, to -> from.aerialDistanceTo(to).toFloat() }
+    ): List<Tile>? {
+        val astar = AStar(
+                startTile,
+                predicate = { tile -> predicate(civ, tile) },
+                cost = { from, to -> cost(civ, from, to) },
+                heuristic = { from, to -> heuristic(civ, from, to) }
+        )
+        while (true) {
+            if (astar.hasEnded()) {
+                // We failed to find a path
+                Log.debug("getConnection failed at AStar search size ${astar.size()}")
+                return null
+            }
+            if (!astar.hasReachedTile(endTile)) {
+                astar.nextStep()
+                continue
+            }
+            // Found a path.
+            return astar.getPathTo(endTile)
+                    .toList()
+                    .reversed()
+        }
+    }
+
 }
