@@ -12,14 +12,17 @@ import com.unciv.models.metadata.Player
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.ruleset.tile.TerrainType
+import com.unciv.models.ruleset.tile.TileResource
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
-import com.unciv.ui.components.extensions.addToMapOfSets
-import com.unciv.ui.components.extensions.contains
-import java.lang.Integer.max
+import com.unciv.utils.addToMapOfSets
+import com.unciv.utils.contains
+import yairm210.purity.annotations.Readonly
 import java.util.concurrent.ConcurrentHashMap
 import kotlin.math.abs
+import kotlin.math.max
 
 /** An Unciv map with all properties as produced by the [map editor][com.unciv.ui.screens.mapeditorscreen.MapEditorScreen]
  * or [MapGenerator][com.unciv.logic.map.mapgenerator.MapGenerator]; or as part of a running [game][GameInfo].
@@ -40,7 +43,7 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * @param nation Name of the nation
      */
     data class StartingLocation(
-        val position: Vector2 = Vector2.Zero,
+        val position: HexCoord = HexCoord.Zero,
         val nation: String = "",
         val usage: Usage = Usage.default // default for maps saved pior to this feature
     ) : IsPartOfGameInfoSerialization {
@@ -74,29 +77,39 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     @Transient
     var ruleset: Ruleset? = null
 
+    data class TerrainListData(
+        val uniques: UniqueMap,
+        val terrainNameSet: Set<String>
+    ){
+        companion object{
+            val EMPTY = TerrainListData(UniqueMap.EMPTY, emptySet())
+        }
+    }
+    
     @Transient
-    var tileUniqueMapCache = ConcurrentHashMap<List<String>, UniqueMap>()
+    var tileUniqueMapCache = ConcurrentHashMap<List<String>, TerrainListData>()
 
     @Transient
     var tileMatrix = ArrayList<ArrayList<Tile?>>() // this works several times faster than a hashmap, the performance difference is really astounding
 
-    @Transient
-    var leftX = 0
-
-    @Transient
-    var bottomY = 0
+    @Transient var leftX = 0
+    @Transient var bottomY = 0
+    @Transient var width = 0
 
     @delegate:Transient
-    val maxLatitude: Float by lazy { if (values.isEmpty()) 0f else values.maxOf { abs(it.latitude) } }
+    val maxLatitude: Int by lazy { if (values.isEmpty()) 0 else values.maxOf { abs(it.latitude) } }
 
     @delegate:Transient
-    val maxLongitude: Float by lazy { if (values.isEmpty()) 0f else values.maxOf { abs(it.longitude) } }
+    val maxLongitude: Int by lazy { if (values.isEmpty()) 0 else values.maxOf { abs(it.longitude) } }
 
     @delegate:Transient
     val naturalWonders: Set<String> by lazy { tileList.asSequence().filter { it.isNaturalWonder() }.map { it.naturalWonder!! }.toSet() }
 
     @delegate:Transient
-    val resources: Set<String> by lazy { tileList.asSequence().filter { it.resource != null }.map { it.resource!! }.toSet() }
+    val resources: Set<String> by lazy { tileList.asSequence().mapNotNull { it.resource }.toSet() }
+
+    @delegate:Transient
+    val resourceObjects: Set<TileResource> by lazy { tileList.asSequence().mapNotNull { it.tileResource }.toSet() }
 
     // Excluded from Serialization by having no own backing field
     val values: Collection<Tile>
@@ -115,8 +128,9 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     /**
      * creates a hexagonal map of given radius (filled with grassland)
      *
-     * To help you visualize how UnCiv hexagonal coordinate system works, here's a small example:
-     *
+     * To help you visualize how Unciv hexagonal coordinate system works, here's a small example:
+     * 
+     * ```
      *          _____         _____         _____
      *         /     \       /     \       /     \
      *   _____/ 2, 0  \_____/  1, 1 \_____/  0,2  \_____
@@ -132,7 +146,7 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * / 0,-3  \_____/ -1,-2 \_____/ -2,-1 \_____/ -3,0  \
      * \       /     \       /     \       /     \       /
      *  \_____/       \_____/       \_____/       \_____/
-     *
+     * ```
      *
      * The rules are simple if you think about your X and Y axis as diagonal w.r.t. a standard carthesian plane. As such:
      *
@@ -148,8 +162,8 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
             : this (HexMath.getNumberOfTilesInHexagon(radius)) {
         startingLocations.clear()
         val firstAvailableLandTerrain = MapLandmassGenerator.getInitializationTerrain(ruleset, TerrainType.Land)
-        for (vector in HexMath.getVectorsInDistance(Vector2.Zero, radius, worldWrap))
-            tileList.add(Tile().apply { position = vector; baseTerrain = firstAvailableLandTerrain })
+        for (vector in HexMath.getHexCoordsInDistance(HexCoord.Zero, radius, worldWrap))
+            tileList.add(Tile().apply { position = vector.asSerializable(); baseTerrain = firstAvailableLandTerrain })
         setTransients(ruleset)
     }
 
@@ -167,7 +181,7 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         for (column in -wrapAdjustedWidth / 2 .. (wrapAdjustedWidth-1) / 2)
             for (row in -height / 2 .. (height-1) / 2)
                 tileList.add(Tile().apply {
-                    position = HexMath.getTileCoordsFromColumnRow(column, row)
+                    position = HexMath.getTileCoordsFromColumnRow(column, row).asSerializable()
                     baseTerrain = firstAvailableLandTerrain
                 })
 
@@ -195,44 +209,47 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         return toReturn
     }
 
-    operator fun contains(vector: Vector2) =
-        contains(vector.x.toInt(), vector.y.toInt())
+    @Readonly operator fun contains(vector: Vector2) = contains(vector.x.toInt(), vector.y.toInt())
+    @Readonly operator fun contains(hexCoord: HexCoord) = contains(hexCoord.x, hexCoord.y)
+    @Readonly fun contains(x: Int, y: Int) = getOrNull(x, y) != null
 
-    operator fun get(vector: Vector2) =
-        get(vector.x.toInt(), vector.y.toInt())
-
-    fun contains(x: Int, y: Int) =
-        getOrNull(x, y) != null
-
-    operator fun get(x: Int, y: Int) =
-        tileMatrix[x - leftX][y - bottomY]!!
+    @Readonly operator fun get(vector: Vector2) = get(vector.x.toInt(), vector.y.toInt())
+    @Readonly operator fun get(hexCoord: HexCoord) = get(hexCoord.x, hexCoord.y)
+    @Readonly operator fun get(x: Int, y: Int) = tileMatrix[x - leftX][y - bottomY]!!
 
     /** @return tile at hex coordinates ([x],[y]) or null if they are outside the map. Does *not* respect world wrap, use [getIfTileExistsOrNull] for that. */
+    @Readonly
     private fun getOrNull (x: Int, y: Int): Tile? =
             tileMatrix.getOrNull(x - leftX)?.getOrNull(y - bottomY)
 
     //endregion
     //region Pure Functions
 
+    /** Can we access [gameInfo]? e.g. for MapEditor use where there is a map but no game */
+    fun hasGameInfo() = ::gameInfo.isInitialized
+
     /** @return All tiles in a hexagon of radius [distance], including the tile at [origin] and all up to [distance] steps away.
      *  Respects map edges and world wrap. */
-    fun getTilesInDistance(origin: Vector2, distance: Int): Sequence<Tile> =
+    @Readonly
+    fun getTilesInDistance(origin: HexCoord, distance: Int): Sequence<Tile> =
             getTilesInDistanceRange(origin, 0..distance)
 
     /** @return All tiles in a hexagonal ring around [origin] with the distances in [range]. Excludes the [origin] tile unless [range] starts at 0.
      *  Respects map edges and world wrap. */
-    fun getTilesInDistanceRange(origin: Vector2, range: IntRange): Sequence<Tile> =
+    @Readonly
+    fun getTilesInDistanceRange(origin: HexCoord, range: IntRange): Sequence<Tile> =
             range.asSequence().flatMap { getTilesAtDistance(origin, it) }
 
     /** @return All tiles in a hexagonal ring 1 tile wide around [origin] with the [distance]. Contains the [origin] if and only if [distance] is <= 0.
      *  Respects map edges and world wrap. */
-    fun getTilesAtDistance(origin: Vector2, distance: Int): Sequence<Tile> =
+    @Readonly
+    fun getTilesAtDistance(origin: HexCoord, distance: Int): Sequence<Tile> =
             if (distance <= 0) // silently take negatives.
                 sequenceOf(get(origin))
             else
                 sequence {
-                    val centerX = origin.x.toInt()
-                    val centerY = origin.y.toInt()
+                    val centerX = origin.x
+                    val centerY = origin.y
 
                     // Start from 6 O'clock point which means (-distance, -distance) away from the center point
                     var currentX = centerX - distance
@@ -267,15 +284,16 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
             for (worldColumnNumber in x until x + rectangle.width.toInt()) {
                 for (worldRowNumber in y until y + rectangle.height.toInt()) {
                     val hexCoords = HexMath.getTileCoordsFromColumnRow(worldColumnNumber, worldRowNumber)
-                    yield(getIfTileExistsOrNull(hexCoords.x.toInt(), hexCoords.y.toInt()))
+                    yield(getIfTileExistsOrNull(hexCoords.x, hexCoords.y))
                 }
             }
         }.filterNotNull()
 
     /** @return tile at hex coordinates ([x],[y]) or null if they are outside the map. Respects map edges and world wrap. */
+    @Readonly
     fun getIfTileExistsOrNull(x: Int, y: Int): Tile? {
-        if (contains(x, y))
-            return get(x, y)
+        val tile = getOrNull(x, y)
+        if (tile != null) return tile
 
         if (!mapParameters.worldWrap)
             return null
@@ -284,30 +302,33 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         if (mapParameters.shape == MapShape.rectangular)
             radius = mapParameters.mapSize.width / 2
 
-        //tile is outside of the map
-        if (contains(x + radius, y - radius)) { //tile is on right side
-            //get tile wrapped around from right to left
-            return get(x + radius, y - radius)
-        } else if (contains(x - radius, y + radius)) { //tile is on left side
-            //get tile wrapped around from left to right
-            return get(x - radius, y + radius)
-        }
+        // Maybe tile is "outside of the map" in world wrap.
+
+        // A. Get tile wrapped around from right to left
+        val rightSideTile = getOrNull(x + radius, y - radius)
+        if (rightSideTile != null) return rightSideTile
+
+        // B. Get tile wrapped around from left to right
+        val leftSideTile = getOrNull(x - radius, y + radius)
+        if (leftSideTile != null) return leftSideTile
 
         return null
     }
 
+
     /**
      * Returns the clock position of [otherTile] seen from [tile]'s position
      * Returns -1 if not neighbors
      */
+    @Readonly
     fun getNeighborTileClockPosition(tile: Tile, otherTile: Tile): Int {
         val radius = if (mapParameters.shape == MapShape.rectangular)
             mapParameters.mapSize.width / 2
         else mapParameters.mapSize.radius
-        val x1 = tile.position.x.toInt()
-        val y1 = tile.position.y.toInt()
-        val x2 = otherTile.position.x.toInt()
-        val y2 = otherTile.position.y.toInt()
+        val x1 = tile.position.x
+        val y1 = tile.position.y
+        val x2 = otherTile.position.x
+        val y2 = otherTile.position.y
 
         val xDifference = x1 - x2
         val yDifference = y1 - y2
@@ -332,17 +353,19 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * Takes world wrap into account
      * Returns null if there is no such neighbor tile or if [clockPosition] is not a valid clock position
      */
+    @Readonly
     fun getClockPositionNeighborTile(tile: Tile, clockPosition: Int): Tile? {
-        val difference = HexMath.getClockPositionToHexVector(clockPosition)
-        if (difference == Vector2.Zero) return null
-        val possibleNeighborPosition = tile.position.cpy().add(difference)
-        return getIfTileExistsOrNull(possibleNeighborPosition.x.toInt(), possibleNeighborPosition.y.toInt())
+        val difference = HexMath.getClockPositionToHexcoord(clockPosition)
+        if (difference == HexCoord.Zero) return null
+        val possibleNeighborPosition = tile.position.plus(difference)
+        return getIfTileExistsOrNull(possibleNeighborPosition.x, possibleNeighborPosition.y)
     }
 
     /** Convert relative direction of [otherTile] seen from [tile]'s position into a vector
      * in world coordinates of length sqrt(3), so that it can be used to go from tile center to
      * the edge of the hex in that direction (meaning the center of the border between the hexes)
      */
+    @Readonly
     fun getNeighborTilePositionAsWorldCoords(tile: Tile, otherTile: Tile): Vector2 =
         HexMath.getClockPositionToWorldVector(getNeighborTileClockPosition(tile, otherTile))
 
@@ -350,7 +373,8 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * Returns the closest position to (0, 0) outside the map which can be wrapped
      * to the position of the given vector
      */
-    fun getUnWrappedPosition(position: Vector2): Vector2 {
+    @Readonly
+    fun getUnwrappedPosition(position: HexCoord): HexCoord {
         if (!contains(position))
             return position //The position is outside the map so its unwrapped already
 
@@ -358,10 +382,16 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
             mapParameters.mapSize.width / 2
         else mapParameters.mapSize.radius
 
-        val vectorUnwrappedLeft = Vector2(position.x + radius, position.y - radius)
-        val vectorUnwrappedRight = Vector2(position.x - radius, position.y + radius)
+        val vectorUnwrappedLeft = HexCoord.of(position.x + radius, position.y - radius)
+        val vectorUnwrappedRight = HexCoord.of(position.x - radius, position.y + radius)
+        
+        fun squareSum(hexCoord: HexCoord): Int {
+            val x = hexCoord.x
+            val y = hexCoord.y
+            return x * x + y * y
+        }  
 
-        return if (vectorUnwrappedRight.len() < vectorUnwrappedLeft.len())
+        return if (squareSum(vectorUnwrappedRight) < squareSum(vectorUnwrappedLeft))
             vectorUnwrappedRight
         else
             vectorUnwrappedLeft
@@ -370,7 +400,8 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     data class ViewableTile(val tile: Tile, val maxHeightSeenToTile: Int, val isVisible: Boolean, val isAttackable: Boolean)
 
     /** @return List of tiles visible from location [position] for a unit with sight range [sightDistance] */
-    fun getViewableTiles(position: Vector2, sightDistance: Int, forAttack: Boolean = false): List<Tile> {
+    @Readonly
+    fun getViewableTiles(position: HexCoord, sightDistance: Int, forAttack: Boolean = false): List<Tile> {
         val aUnitHeight = get(position).unitHeight
         val viewableTiles = mutableListOf(ViewableTile(
             get(position),
@@ -401,7 +432,8 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
 
             This can all be summed up as "I can see c if a=>b || c>b"
             */
-                val bMinimumHighestSeenTerrainSoFar = viewableTiles.filter { it.tile in cTile.neighbors }
+                val bMinimumHighestSeenTerrainSoFar = viewableTiles
+                    .filter { it.tile.aerialDistanceTo(cTile) == 1 }
                     .minOf { it.maxHeightSeenToTile }
 
                 tilesToAddInDistanceI.add(ViewableTile(
@@ -449,6 +481,12 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         val players = gameInfo.gameParameters.players.size
         return bigIslands >= players
     }
+    
+    fun usingArchipelagoRegions(): Boolean {
+        val totalLand = continentSizes.values.sum().toFloat()
+        val largestContinent = continentSizes.values.maxOf { it }.toFloat()
+        return largestContinent / totalLand < 0.25f
+    }
 
     //endregion
     //region State-Changing Methods
@@ -460,12 +498,13 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     fun setTransients(ruleset: Ruleset? = null, setUnitCivTransients: Boolean = true) {
         if (ruleset != null) this.ruleset = ruleset
         check(this.ruleset != null) { "TileMap.setTransients called without ruleset" }
+        check(tileList.isNotEmpty()) { "No tiles were found in the save?!" }
 
         if (tileMatrix.isEmpty()) {
-            val topY = tileList.asSequence().map { it.position.y.toInt() }.maxOrNull()!!
-            bottomY = tileList.asSequence().map { it.position.y.toInt() }.minOrNull()!!
-            val rightX = tileList.asSequence().map { it.position.x.toInt() }.maxOrNull()!!
-            leftX = tileList.asSequence().map { it.position.x.toInt() }.minOrNull()!!
+            val topY = tileList.asSequence().map { it.position.y }.max()
+            bottomY = tileList.asSequence().map { it.position.y }.min()
+            val rightX = tileList.asSequence().map { it.position.x }.max()
+            leftX = tileList.asSequence().map { it.position.x }.min()
 
             // Initialize arrays with enough capacity to avoid re-allocations (+Arrays.copyOf).
             // We have just calculated the dimensions above, so we know the final size.
@@ -484,19 +523,24 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         }
 
         for (tileInfo in values) {
-            tileMatrix[tileInfo.position.x.toInt() - leftX][tileInfo.position.y.toInt() - bottomY] = tileInfo
+            tileMatrix[tileInfo.position.x - leftX][tileInfo.position.y - bottomY] = tileInfo
         }
-        for (tileInfo in values) {
+        for ((index, tileInfo) in values.withIndex()) {
             // Do ***NOT*** call Tile.setTerrainTransients before the tileMatrix is complete -
             // setting transients might trigger the neighbors lazy (e.g. thanks to convertHillToTerrainFeature).
             // When that lazy runs, some directions might be omitted because getIfTileExistsOrNull
             // looks at tileMatrix. Thus filling Tiles into tileMatrix and setting their
             // transients in the same loop will leave incomplete cached `neighbors`.
             tileInfo.tileMap = this
+            tileInfo.zeroBasedIndex = index
             tileInfo.ruleset = this.ruleset!!
             tileInfo.setTerrainTransients()
             tileInfo.setUnitTransients(setUnitCivTransients)
         }
+        
+        val minColumn = tileList.asSequence().map { HexMath.getColumn(it.position) }.min()
+        val maxColumn = tileList.asSequence().map { HexMath.getColumn(it.position) }.max()
+        width = maxColumn - minColumn + 1
     }
 
     /** Initialize Civilization.neutralRoads based on Tile.roadOwner
@@ -525,12 +569,13 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * @return created [MapUnit] or null if no suitable location was found
      * */
     fun placeUnitNearTile(
-        position: Vector2,
+        position: HexCoord,
         unitName: String,
-        civInfo: Civilization
+        civInfo: Civilization,
+        unitId: Int? = null
     ): MapUnit? {
         val unit = gameInfo.ruleset.units[unitName]!!
-        return placeUnitNearTile(position, unit, civInfo)
+        return placeUnitNearTile(position, unit, civInfo, unitId)
     }
 
     /** Tries to place the [baseUnit] into the [Tile] closest to the given [position]
@@ -540,11 +585,12 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * @return created [MapUnit] or null if no suitable location was found
      * */
     fun placeUnitNearTile(
-            position: Vector2,
+            position: HexCoord,
             baseUnit: BaseUnit,
-            civInfo: Civilization
+            civInfo: Civilization,
+            unitId: Int? = null
     ): MapUnit? {
-        val unit = baseUnit.getMapUnit(civInfo)
+        val unit = baseUnit.newMapUnit(civInfo, unitId)
 
         fun getPassableNeighbours(tile: Tile): Set<Tile> =
                 tile.neighbors.filter { unit.movement.canPassThrough(it) }.toSet()
@@ -552,12 +598,13 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
         // both the civ name and actual civ need to be in here in order to calculate the canMoveTo...Darn
         unit.assignOwner(civInfo, false)
         // remember our first owner
-        unit.originalOwner = civInfo.civName
+        unit.originalOwner = civInfo.civID
 
         var unitToPlaceTile: Tile? = null
         // try to place at the original point (this is the most probable scenario)
         val currentTile = get(position)
         unit.currentTile = currentTile  // temporary
+        unit.cache.state = GameContext(unit)
         if (unit.movement.canMoveTo(currentTile)) unitToPlaceTile = currentTile
 
         // if it's not suitable, try to find another tile nearby
@@ -566,7 +613,7 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
             var potentialCandidates = getPassableNeighbours(currentTile)
             while (unitToPlaceTile == null && tryCount++ < 10) {
                 unitToPlaceTile = potentialCandidates
-                        .sortedByDescending { if (unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater) it.isLand else true } // Land units should prefer to go into land tiles
+                        .sortedByDescending { if (unit.baseUnit.isLandUnit && !unit.cache.canMoveOnWater) it.isLand else true } // Land units should prefer to go into land tiles
                         .firstOrNull { unit.movement.canMoveTo(it) }
                 if (unitToPlaceTile != null) continue
                 // if it's not found yet, let's check their neighbours
@@ -630,17 +677,18 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
      * @param newNation new nation to be set up
      */
     fun switchPlayersNation(player: Player, newNation: Nation) {
-        val newCiv = Civilization(newNation.name).apply { nation = newNation }
+        val newCiv = Civilization(newNation)
         tileList.forEach {
             for (unit in it.getUnits()) if (unit.owner == player.chosenCiv) {
-                unit.owner = newNation.name
+                unit.owner = newCiv.civID
                 unit.civ = newCiv
+                unit.setTransients(newCiv.gameInfo.ruleset)
             }
         }
         for (element in startingLocations.filter { it.nation != player.chosenCiv }) {
             startingLocations.remove(element)
-            if (startingLocations.none { it.nation == newNation.name && it.position == element.position })
-                startingLocations.add(StartingLocation(element.position, newNation.name))
+            if (startingLocations.none { it.nation == newCiv.civID && it.position == element.position })
+                startingLocations.add(StartingLocation(element.position, newCiv.civID))
         }
         setStartingLocationsTransients()
     }
@@ -688,7 +736,7 @@ class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     }
 
     /** Removes all starting positions for [position], rebuilding the transients */
-    fun removeStartingLocations(position: Vector2) {
+    fun removeStartingLocations(position: HexCoord) {
         startingLocations.removeAll { it.position == position }
         setStartingLocationsTransients()
     }
