@@ -1,12 +1,10 @@
 package com.unciv.logic.map.mapgenerator
 
+import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
-import com.unciv.logic.map.MapParameters
-import com.unciv.logic.map.MapShape
-import com.unciv.logic.map.MapType
-import com.unciv.logic.map.TileMap
+import com.unciv.logic.map.*
 import com.unciv.logic.map.mapgenerator.mapregions.MapRegions
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.Counter
@@ -18,7 +16,9 @@ import com.unciv.models.ruleset.tile.TerrainType
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.screens.mapeditorscreen.MapGeneratorSteps
-import com.unciv.ui.screens.mapeditorscreen.TileInfoNormalizer
+import com.unciv.logic.map.tile.TileNormalizer
+import com.unciv.models.ruleset.tile.TileImprovement
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.utils.debug
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.isActive
@@ -102,7 +102,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         else
             TileMap(mapSize.radius, ruleset, mapParameters.worldWrap)
 
-        mapParameters.createdWithVersion = UncivGame.VERSION.toNiceString()
+        mapParameters.createdWithVersion = UncivGame.VERSION.toSerializeString()
         map.mapParameters = mapParameters
 
         if (mapType == MapType.empty) {
@@ -165,20 +165,63 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
                 NaturalWonderGenerator(ruleset, randomness).spawnNaturalWonders(map)
             }
             // Fallback spread resources function - used when generating maps in map editor
-            runAndMeasure("spreadResources") {
-                spreadResources(map)
-            }
-        }
-        runAndMeasure("spreadAncientRuins") {
-            spreadAncientRuins(map)
+            runAndMeasure("spreadResources") { spreadResources(map) }
         }
+        runAndMeasure("spreadAncientRuins") { spreadAncientRuins(map) }
+        
+        mirror(map)
 
         // Map generation may generate incompatible terrain/feature combinations
         for (tile in map.values)
-            TileInfoNormalizer.normalizeToRuleset(tile, ruleset)
+            TileNormalizer.normalizeToRuleset(tile, ruleset)
 
         return map
     }
+    
+    private fun flipTopBottom(vector: Vector2): Vector2 = Vector2(-vector.y, -vector.x)
+    private fun flipTopBottom(vector: HexCoord): HexCoord = HexCoord.of(-vector.y, -vector.x)
+    private fun flipLeftRight(vector: Vector2): Vector2 = Vector2(vector.y, vector.x)
+    private fun flipLeftRight(vector: HexCoord): HexCoord = HexCoord.of(vector.y, vector.x)
+
+    private fun mirror(map: TileMap) {
+        fun getMirrorTile(tile: Tile, mirroringType: String): Tile? {
+            val mirrorTileVector = when (mirroringType) {
+                MirroringType.topbottom -> if (tile.getRow() <= 0) return null else flipTopBottom(tile.position)
+                MirroringType.leftright -> if (tile.getColumn() <= 0) return null else flipLeftRight(tile.position)
+                MirroringType.aroundCenterTile -> if (tile.getRow() <= 0) return null else flipLeftRight(flipTopBottom(tile.position))
+                MirroringType.fourway -> when {
+                    tile.getRow() < 0 && tile.getColumn() < 0 -> return null
+                    tile.getRow() < 0 && tile.getColumn() >= 0 -> flipLeftRight(tile.position)
+                    tile.getRow() >= 0 && tile.getColumn() < 0 -> flipTopBottom(tile.position)
+                    else -> flipLeftRight(flipTopBottom(tile.position))
+                }
+
+                else -> return null
+            }
+            return map.getIfTileExistsOrNull(mirrorTileVector.x.toInt(), mirrorTileVector.y.toInt())
+        }
+        
+        fun copyTile(tile: Tile, mirroringType: String) {
+            val mirrorTile = getMirrorTile(tile, mirroringType) ?: return
+            
+            tile.setBaseTerrain(mirrorTile.getBaseTerrain())
+            tile.naturalWonder = mirrorTile.naturalWonder
+            tile.setTerrainFeatures(mirrorTile.terrainFeatures)
+            tile.tileResource = mirrorTile.tileResource
+            tile.improvement = mirrorTile.improvement
+            
+            for (neighbor in tile.neighbors){
+                val neighborMirror = getMirrorTile(neighbor, mirroringType) ?: continue
+                if (neighborMirror !in mirrorTile.neighbors) continue // we landed on the edge here
+                tile.setConnectedByRiver(neighbor, mirrorTile.isConnectedByRiver(neighborMirror))
+            }
+        }
+
+        if (map.mapParameters.mirroring == MirroringType.none) return
+        for (tile in map.values) {
+            copyTile(tile, map.mapParameters.mirroring)
+        }
+    }
 
     fun generateSingleStep(map: TileMap, step: MapGeneratorSteps) {
         if (map.mapParameters.seed == 0L)
@@ -225,7 +268,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         fun convertTerrains(ruleset: Ruleset, tiles: Iterable<Tile>) {
             for (tile in tiles) {
                 val conversionUnique =
-                    tile.getBaseTerrain().getMatchingUniques(UniqueType.ChangesTerrain)
+                    tile.getBaseTerrain().getMatchingUniques(UniqueType.ChangesTerrain, GameContext(tile = tile))
                         .firstOrNull { tile.isAdjacentTo(it.params[1]) }
                         ?: continue
                 val terrain = ruleset.terrains[conversionUnique.params[0]] ?: continue
@@ -277,15 +320,16 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
             val maxLakeSize = ruleset.modOptions.constants.maxLakeSize
 
             while (waterTiles.isNotEmpty()) {
-                val initialWaterTile = waterTiles.removeFirst()
+                val initialWaterTile = waterTiles.removeAt(0)
                 tilesInArea += initialWaterTile
                 tilesToCheck += initialWaterTile
 
                 // Floodfill to cluster water tiles
                 while (tilesToCheck.isNotEmpty()) {
-                    val tileWeAreChecking = tilesToCheck.removeFirst()
-                    for (vector in tileWeAreChecking.neighbors
-                        .filter { !tilesInArea.contains(it) and waterTiles.contains(it) }) {
+                    val tileWeAreChecking = tilesToCheck.removeAt(0)
+                    for (vector in tileWeAreChecking.neighbors){
+                        if (tilesInArea.contains(vector)) continue
+                        if (!waterTiles.contains(vector)) continue
                         tilesInArea += vector
                         tilesToCheck += vector
                         waterTiles -= vector
@@ -309,22 +353,28 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
     }
 
     private fun spreadAncientRuins(map: TileMap) {
-        val ruinsEquivalents = ruleset.tileImprovements.filter { it.value.isAncientRuinsEquivalent() }
-        if (map.mapParameters.noRuins || ruinsEquivalents.isEmpty() )
-            return
-        val suitableTiles = map.values.filter { it.isLand && !it.isImpassible() }
+        val ruinsEquivalents = ruleset.tileImprovements.filter {
+            it.value.isAncientRuinsEquivalent()
+        }
+        if (map.mapParameters.noRuins || ruinsEquivalents.isEmpty()) return
+
+        fun isPlaceable(improvement: TileImprovement, tile: Tile) =
+            tile.improvementFunctions.canImprovementBeBuiltHere(improvement, gameContext = GameContext.IgnoreConditionals)
+
+        val suitableTiles = map.values.filter { it.isLand && !it.isImpassible()
+                && ruinsEquivalents.values.any { improvement -> isPlaceable(improvement, it) } }
         val locations = randomness.chooseSpreadOutLocations(
                 (suitableTiles.size * ruleset.modOptions.constants.ancientRuinCountMultiplier).roundToInt(),
                 suitableTiles,
                 map.mapParameters.mapSize.radius)
         for (tile in locations)
-            tile.changeImprovement(ruinsEquivalents.keys.random())
+            tile.improvement = ruinsEquivalents.values.filter { isPlaceable(it, tile) }.random().name
     }
 
     private fun spreadResources(tileMap: TileMap) {
         val mapRadius = tileMap.mapParameters.mapSize.radius
         for (tile in tileMap.values)
-            tile.resource = null
+            tile.tileResource = null
 
         spreadStrategicResources(tileMap, mapRadius)
         spreadResources(tileMap, mapRadius, ResourceType.Luxury)
@@ -384,7 +434,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
 
     /**
      * [MapParameters.tilesPerBiomeArea] to set biomes size
-     * [MapParameters.temperatureExtremeness] to favor very high and very low temperatures
+     * [MapParameters.temperatureintensity] to favor very high and very low temperatures
      * [MapParameters.temperatureShift] to shift temperature towards cold (negative) or hot (positive)
      */
     private fun applyHumidityAndTemperature(tileMap: TileMap) {
@@ -394,7 +444,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         tileMap.setTransients(ruleset)
 
         val scale = tileMap.mapParameters.tilesPerBiomeArea.toDouble()
-        val temperatureExtremeness = tileMap.mapParameters.temperatureExtremeness
+        val temperatureintensity = tileMap.mapParameters.temperatureintensity
         val temperatureShift = tileMap.mapParameters.temperatureShift
         val humidityShift = if (temperatureShift > 0) -temperatureShift / 2 else 0f
 
@@ -417,7 +467,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
             val humidityRandom = randomness.getPerlinNoise(tile, humiditySeed, scale = scale, nOctaves = 1)
             val humidity = ((humidityRandom + 1.0) / 2.0 + humidityShift).coerceIn(0.0..1.0)
 
-            val expectedTemperature = if (tileMap.mapParameters.shape === MapShape.flatEarth) {
+            val expectedTemperature = if (tileMap.mapParameters.shape == MapShape.flatEarth) {
                 // Flat Earth uses radius because North is center of map
                 val radius = getTileRadius(tile, tileMap)
                 val radiusTemperature = getTemperatureAtRadius(radius)
@@ -430,7 +480,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
 
             val randomTemperature = randomness.getPerlinNoise(tile, temperatureSeed, scale = scale, nOctaves = 1)
             var temperature = (5.0 * expectedTemperature + randomTemperature) / 6.0
-            temperature = abs(temperature).pow(1.0 - temperatureExtremeness) * temperature.sign
+            temperature = abs(temperature).pow(1.0 - temperatureintensity) * temperature.sign
             temperature = (temperature + temperatureShift).coerceIn(-1.0..1.0)
 
             // Old, static map generation rules - necessary for existing base ruleset mods to continue to function
@@ -463,8 +513,9 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
     }
 
     private fun getTileRadius(tile: Tile, tileMap: TileMap): Float {
-        val latitudeRatio = abs(tile.latitude) / tileMap.maxLatitude
-        val longitudeRatio = abs(tile.longitude) / tileMap.maxLongitude
+        // Numbers betwee 0.0-1.0
+        val latitudeRatio = abs(tile.latitude) / tileMap.maxLatitude.toFloat()
+        val longitudeRatio = abs(tile.longitude) / tileMap.maxLongitude.toFloat()
         return sqrt(latitudeRatio.pow(2) + longitudeRatio.pow(2))
     }
 
@@ -542,6 +593,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
                             tile.temperature!! < it.params[0].toDouble() || tile.temperature!! > it.params[1].toDouble()
                                     || tile.humidity!! < it.params[2].toDouble() || tile.humidity!! > it.params[3].toDouble()
                         }
+                            && NaturalWonderGenerator.fitsTerrainUniques(vegetationTerrain, tile)
                 }
                 if (possibleVegetation.isEmpty()) continue
                 val randomVegetation = possibleVegetation.random(randomness.RNG)
@@ -559,8 +611,11 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         }
         for (tile in tileMap.values.asSequence().filter { it.terrainFeatures.isEmpty() }) {
             if (randomness.RNG.nextDouble() <= tileMap.mapParameters.rareFeaturesRichness) {
-                val possibleFeatures = rareFeatures.filter { it.occursOn.contains(tile.baseTerrain)
-                        && (!tile.isHill() || it.occursOn.contains(Constants.hill)) }
+                val possibleFeatures = rareFeatures.filter {
+                    it.occursOn.contains(tile.baseTerrain)
+                            && (!tile.isHill() || it.occursOn.contains(Constants.hill))
+                            && NaturalWonderGenerator.fitsTerrainUniques(it, tile)
+                }
                 if (possibleFeatures.any())
                     tile.addTerrainFeature(possibleFeatures.random(randomness.RNG).name)
             }
@@ -568,7 +623,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
     }
 
     /**
-     * [MapParameters.temperatureExtremeness] as in [applyHumidityAndTemperature]
+     * [MapParameters.temperatureintensity] as in [applyHumidityAndTemperature]
      */
     private fun spawnIce(tileMap: TileMap) {
         val waterTerrain: Set<String> =
@@ -589,7 +644,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
                     0f, 1f))
             }.toList()
 
-        if (tileMap.mapParameters.shape === MapShape.flatEarth) {
+        if (tileMap.mapParameters.shape == MapShape.flatEarth) {
             spawnFlatEarthIceWalls(tileMap, iceEquivalents)
         }
 
@@ -604,13 +659,14 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
             val randomTemperature = randomness.getPerlinNoise(tile, temperatureSeed, scale = tileMap.mapParameters.tilesPerBiomeArea.toDouble(), nOctaves = 1)
             val latitudeTemperature = 1.0 - 2.0 * abs(tile.latitude) / tileMap.maxLatitude
             var temperature = ((latitudeTemperature + randomTemperature) / 2.0)
-            temperature = abs(temperature).pow(1.0 - tileMap.mapParameters.temperatureExtremeness) * temperature.sign
+            temperature = abs(temperature).pow(1.0 - tileMap.mapParameters.temperatureintensity) * temperature.sign
             temperature = (temperature + tileMap.mapParameters.temperatureShift).coerceIn(-1.0..1.0)
 
             val candidates = iceEquivalents
                 .filter {
-                    it.matches(temperature, 1.0) &&
-                    tile.lastTerrain.name in it.terrain.occursOn
+                    it.matches(temperature, 1.0)
+                            && tile.lastTerrain.name in it.terrain.occursOn
+                            && NaturalWonderGenerator.fitsTerrainUniques(it.terrain, tile)
                 }.map { it.terrain.name }
             when (candidates.size) {
                 1 -> tile.addTerrainFeature(candidates.first())
@@ -674,7 +730,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
 
         // Flat Earth needs a 1 tile wide perimeter of ice/mountain/snow and a 2 radius cluster of ice in the center.
         for (tile in tileMap.values) {
-            val isCenterTile = tile.latitude == 0f && tile.longitude == 0f
+            val isCenterTile = tile.latitude == 0 && tile.longitude == 0
             val isEdgeTile = tile.neighbors.count() < 6
 
             // Make center tiles ice or snow or mountain depending on availability
