@@ -1,8 +1,10 @@
 package com.unciv.logic.map.mapgenerator
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.TileMap
+import com.unciv.logic.map.mapgenerator.RiverGenerator.Companion.continueRiverOn
+import com.unciv.logic.map.mapgenerator.RiverGenerator.RiverCoordinate
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.unique.UniqueType
@@ -71,7 +73,7 @@ class RiverGenerator(
         // Recommendation: Draw a bunch of hexagons on paper before trying to understand this, it's super helpful!
 
         var riverCoordinate = RiverCoordinate(tileMap, initialPosition.position,
-                RiverCoordinate.BottomRightOrLeft.values().random(randomness.RNG))
+                RiverCoordinate.BottomRightOrLeft.entries.random(randomness.RNG))
 
         repeat(maxRiverLength) {     // Arbitrary max on river length, otherwise this will go in circles - rarely
             if (riverCoordinate.getAdjacentTiles().any { it.isWater }) return
@@ -97,7 +99,7 @@ class RiverGenerator(
      * such that always the north-most hex and one of the two clock directions 5 / 7 o'clock are used. */
     class RiverCoordinate(
         private val tileMap: TileMap,
-        private val position: Vector2,
+        private val position: HexCoord,
         private val bottomRightOrLeft: BottomRightOrLeft
     ) {
         enum class BottomRightOrLeft {
@@ -108,8 +110,8 @@ class RiverGenerator(
             BottomRight
         }
 
-        private val x = position.x.toInt()
-        private val y = position.y.toInt()
+        private val x = position.x
+        private val y = position.y
         // Depending on the tile instance, some of the following will never be used. Tested with lazies: ~2% slower
         private val myTile = tileMap[position]
         private val myTopLeft = tileMap.getIfTileExistsOrNull(x + 1, y)
@@ -207,7 +209,7 @@ class RiverGenerator(
         fun getNeighborTile(selectedTile: Tile): Tile? =
             selectedTile.tileMap.getClockPositionNeighborTile(selectedTile, clockPosition)
         companion object {
-            val names get() = values().map { it.name }
+            val names get() = entries.map { it.name }
         }
     }
 
