@@ -1,13 +1,13 @@
 package com.unciv.logic.map.mapgenerator.mapregions
 
 import com.unciv.Constants
-import com.unciv.logic.map.MapResources
 import com.unciv.logic.map.TileMap
+import com.unciv.logic.map.mapgenerator.resourceplacement.MapRegionResources
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TerrainType
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import kotlin.math.abs
@@ -29,8 +29,7 @@ object StartNormalizer {
             }
         }
 
-        if (tileMap.mapParameters.mapResources == MapResources.strategicBalance ||
-            tileMap.mapParameters.strategicBalance)
+        if (!isMinorCiv && tileMap.mapParameters.getStrategicBalance())
             placeStrategicBalanceResources(startTile, ruleset, tileData)
 
         normalizeProduction(startTile, isMinorCiv, ruleset, tileData)
@@ -160,11 +159,11 @@ object StartNormalizer {
                 val plot = grassTypePlots.random()
                 grassTypePlots.remove(plot)
 
-                if (plot.resource != null) continue
+                if (plot.tileResource != null) continue
 
                 val bonusToPlace = productionBonuses.filter { it.generatesNaturallyOn(plot) }.randomOrNull()
                 if (bonusToPlace != null) {
-                    plot.resource = bonusToPlace.name
+                    plot.tileResource = bonusToPlace
                     productionBonusesNeeded--
                 }
             }
@@ -217,8 +216,7 @@ object StartNormalizer {
                 else -> 0
             }
         }
-        if (tileMap.mapParameters.mapResources == MapResources.legendaryStart  ||
-            tileMap.mapParameters.legendaryStart)
+        if (tileMap.mapParameters.getLegendaryStart())
             bonusesNeeded += 2
 
         // Attempt to place one grassland at a plains-only spot (nor for minors)
@@ -271,7 +269,7 @@ object StartNormalizer {
             candidatePlots.remove(plot) // remove the plot as it has now been tried, whether successfully or not
             if (plot.getBaseTerrain().hasUnique(
                     UniqueType.BlocksResources,
-                    StateForConditionals(attackedTile = plot)
+                    GameContext(attackedTile = plot)
                 )
             )
                 continue // Don't put bonuses on snow hills
