@@ -2,7 +2,7 @@ package com.unciv.logic.map.mapunit
 
 import com.unciv.Constants
 import com.unciv.models.ruleset.tile.TerrainType
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 
@@ -53,21 +53,27 @@ class MapUnitCache(private val mapUnit: MapUnit) {
     val doubleMovementInTerrain = HashMap<String, DoubleMovement>()
 
     var canEnterIceTiles = false
+    var cannotEmbark = false
     var cannotEnterOceanTiles = false
     var canEnterForeignTerrain: Boolean = false
     var canEnterCityStates: Boolean = false
     var costToDisembark: Float? = null
     var costToEmbark: Float? = null
-    var paradropRange = 0
+
+    /** A hashmap where the key represents the tileFilter, and the value is how far away the tile could be */
+    val paradropDestinationTileFilters = mutableMapOf<String, Int>()
 
     var hasUniqueToBuildImprovements = false    // not canBuildImprovements to avoid confusion
     var hasUniqueToCreateWaterImprovements = false
 
     var hasStrengthBonusInRadiusUnique = false
     var hasCitadelPlacementUnique = false
+    
+    var state = GameContext.EmptyState
 
     fun updateUniques() {
 
+        state = GameContext(mapUnit)
         allTilesCosts1 = mapUnit.hasUnique(UniqueType.AllTilesCost1Move)
         canPassThroughImpassableTiles = mapUnit.hasUnique(UniqueType.CanPassImpassable)
         ignoresTerrainCost = mapUnit.hasUnique(UniqueType.IgnoresTerrainCost)
@@ -77,7 +83,8 @@ class MapUnitCache(private val mapUnit: MapUnit) {
         canMoveOnWater = mapUnit.hasUnique(UniqueType.CanMoveOnWater)
 
         doubleMovementInTerrain.clear()
-        for (unique in mapUnit.getMatchingUniques(UniqueType.DoubleMovementOnTerrain, stateForConditionals = StateForConditionals.IgnoreConditionals)) {
+        for (unique in mapUnit.getMatchingUniques(UniqueType.DoubleMovementOnTerrain,
+                gameContext = GameContext.IgnoreConditionals, true)) {
             val param = unique.params[0]
             val terrain = mapUnit.civ.gameInfo.ruleset.terrains[param]
             doubleMovementInTerrain[param] = DoubleMovement(unique = unique,
@@ -103,10 +110,8 @@ class MapUnitCache(private val mapUnit: MapUnit) {
 
         //todo: consider parameterizing [terrainFilter] in some of the following:
         canEnterIceTiles = mapUnit.hasUnique(UniqueType.CanEnterIceTiles)
-        cannotEnterOceanTiles = mapUnit.hasUnique(
-            UniqueType.CannotEnterOcean,
-            StateForConditionals(civInfo = mapUnit.civ, unit = mapUnit)
-        )
+        cannotEmbark = mapUnit.hasUnique(UniqueType.CannotEmbark)
+        cannotEnterOceanTiles = mapUnit.hasUnique(UniqueType.CannotEnterOcean, state)
 
         hasUniqueToBuildImprovements = mapUnit.hasUnique(UniqueType.BuildImprovements)
         hasUniqueToCreateWaterImprovements = mapUnit.hasUnique(UniqueType.CreateWaterImprovements)
@@ -116,10 +121,10 @@ class MapUnitCache(private val mapUnit: MapUnit) {
 
         canEnterCityStates = mapUnit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence)
 
-        hasStrengthBonusInRadiusUnique = mapUnit.hasUnique(UniqueType.StrengthBonusInRadius)
+        hasStrengthBonusInRadiusUnique = mapUnit.hasUnique(UniqueType.StrengthBonusInRadius, GameContext.IgnoreConditionals)
         hasCitadelPlacementUnique = mapUnit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
             .mapNotNull { mapUnit.civ.gameInfo.ruleset.tileImprovements[it.params[0]] }
-            .any { it.hasUnique(UniqueType.TakesOverAdjacentTiles) }
+            .any { it.hasUnique(UniqueType.OneTimeTakeOverTilesInRadius) }
     }
 
     companion object {
@@ -141,7 +146,10 @@ class MapUnitCache(private val mapUnit: MapUnit) {
             UniqueType.ForestsAndJunglesAreRoads,
             UniqueType.IgnoreHillMovementCost,
             // Movement algorithm avoids damage on route, meaning terrain damage requires caching
-            UniqueType.DamagesContainingUnits
+            UniqueType.DamagesContainingUnits,
+            UniqueType.LandUnitEmbarkation,
+            UniqueType.LandUnitsCrossTerrainAfterUnitGained,
+            UniqueType.EnemyUnitsSpendExtraMovement
             )
     }
 }
