@@ -1,11 +1,11 @@
 package com.unciv.logic.map.mapunit
 
 import com.unciv.models.ruleset.RejectionReasonType
-import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
+import yairm210.purity.annotations.Readonly
 import kotlin.math.pow
 
 class UnitUpgradeManager(val unit: MapUnit) {
@@ -17,6 +17,7 @@ class UnitUpgradeManager(val unit: MapUnit) {
      *  @param ignoreResources Ignore resource requirements (tech still counts)
      *          Used to display disabled Upgrade button
      */
+    @Readonly
     fun canUpgrade(
         unitToUpgradeTo: BaseUnit,
         ignoreRequirements: Boolean = false,
@@ -44,6 +45,7 @@ class UnitUpgradeManager(val unit: MapUnit) {
      */
     // Only one use from getUpgradeAction at the moment, so AI-specific rules omitted
     //todo Does the AI never buy upgrades???
+    @Readonly
     fun getCostOfUpgrade(unitToUpgradeTo: BaseUnit): Int {
         // Source rounds to int every step, we don't
         //TODO From the source, this should apply _Production_ modifiers (Temple of Artemis? GameSpeed! StartEra!), at the moment it doesn't
@@ -56,7 +58,7 @@ class UnitUpgradeManager(val unit: MapUnit) {
         // the UniqueType being allowed on a BaseUnit - we don't have a MapUnit in the loop.
         // Actually instantiating every intermediate to support such mods: todo
         var civModifier = 1f
-        val stateForConditionals = StateForConditionals(unit.civ, unit = unit)
+        val stateForConditionals = unit.cache.state
         for (unique in unit.civ.getMatchingUniques(UniqueType.UnitUpgradeCost, stateForConditionals))
             civModifier *= unique.params[0].toPercent()
 
@@ -84,26 +86,36 @@ class UnitUpgradeManager(val unit: MapUnit) {
      *  but then the lambda in UnitActionsUpgrade will complain and need to be forced back to Unit type.
      */
     fun performUpgrade(upgradedUnit: BaseUnit, isFree: Boolean, goldCostOfUpgrade: Int? = null) {
+        // When mashing the upgrade button, you can 'queue' 2 upgrade actions
+        //  If both are performed, what you get is the unit is doubled
+        //  This prevents this, since we lack another way to do so -_-'  
+        if (unit.isDestroyed) return  
+            
+        val wasEscorting = unit.isEscorting()
         unit.destroy(destroyTransportedUnit = false)
         val civ = unit.civ
         val position = unit.currentTile.position
-        val newUnit = civ.units.placeUnitNearTile(position, upgradedUnit)
+        val newUnit = civ.units.placeUnitNearTile(position, upgradedUnit, unit.id, copiedFrom = unit)
 
         /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
          * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
          */
         if (newUnit == null) {
-            val resurrectedUnit = civ.units.placeUnitNearTile(position, unit.baseUnit)!!
-            unit.copyStatisticsTo(resurrectedUnit)
+            civ.units.placeUnitNearTile(position, unit.baseUnit, copiedFrom = unit)!!
             return
         }
 
         // Managed to upgrade
         if (!isFree) civ.addGold(-(goldCostOfUpgrade ?: getCostOfUpgrade(upgradedUnit)))
-        unit.copyStatisticsTo(newUnit)
         newUnit.currentMovement = 0f
         // wake up if lost ability to fortify
         if (newUnit.isFortified() && !newUnit.canFortify(ignoreAlreadyFortified = true))
             newUnit.action = null
+        // wake up from Guarding if can't Withdraw
+        if (newUnit.isGuarding() && !newUnit.hasUnique(UniqueType.WithdrawsBeforeMeleeCombat))
+            newUnit.action = null
+        // Re-escort if it was escorting
+        if (newUnit.currentTile.position == position && wasEscorting)
+            newUnit.startEscorting()
     }
 }
