@@ -2,79 +2,100 @@
 
 package com.unciv.logic.map.mapunit.movement
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
+import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.logic.map.BFS
+import com.unciv.logic.map.HexCoord
+import com.unciv.logic.map.HexMath
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitActionType
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.UnitMovementMemoryType
+import com.unciv.utils.getOrPut
+import yairm210.purity.annotations.Cache
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
+import java.util.BitSet
 
 
 class UnitMovement(val unit: MapUnit) {
 
-    private val pathfindingCache = PathfindingCache(unit)
+    @Cache private val pathfindingCache = PathfindingCache(unit)
 
-    class ParentTileAndTotalDistance(val tile: Tile, val parentTile: Tile, val totalDistance: Float)
-
-    fun isUnknownTileWeShouldAssumeToBePassable(tile: Tile) = !unit.civ.hasExplored(tile)
+    class ParentTileAndTotalMovement(val tile: Tile, val parentTile: Tile, val totalMovement: Float)
 
+    @Readonly fun isUnknownTileWeShouldAssumeToBePassable(tile: Tile) = !unit.civ.hasExplored(tile)
+    
     /**
+     * Gets the tiles the unit could move to at [position] with [unitMovement].
      * Does not consider if tiles can actually be entered, use canMoveTo for that.
      * If a tile can be reached within the turn, but it cannot be passed through, the total distance to it is set to unitMovement
      */
-    fun getDistanceToTilesWithinTurn(
-        origin: Vector2,
+    @Readonly @Suppress("purity") // mutates passed parameter
+    fun getMovementToTilesAtPosition(
+        position: HexCoord,
         unitMovement: Float,
         considerZoneOfControl: Boolean = true,
-        tilesToIgnore: HashSet<Tile>? = null,
-        passThroughCache: HashMap<Tile, Boolean> = HashMap(),
-        movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap()
+        tilesToIgnoreBitset: BitSet? = null,
+        canPassThroughCache: ArrayList<Boolean?> = ArrayList(),
+        movementCostCache: HashMap<Int, Float> = HashMap(),
+        includeOtherEscortUnit: Boolean = true
     ): PathsToTilesWithinTurn {
-        val distanceToTiles = PathsToTilesWithinTurn()
+        @LocalState val distanceToTiles = PathsToTilesWithinTurn()
 
         val currentUnitTile = unit.currentTile
         // This is for performance, because this is called all the time
-        val unitTile = if (origin == currentUnitTile.position) currentUnitTile else currentUnitTile.tileMap[origin]
-        distanceToTiles[unitTile] = ParentTileAndTotalDistance(unitTile, unitTile, 0f)
+        val unitTile = if (position == currentUnitTile.position) currentUnitTile else currentUnitTile.tileMap[position]
+        distanceToTiles[unitTile] = ParentTileAndTotalMovement(unitTile, unitTile, 0f)
 
         // If I can't move my only option is to stay...
         if (unitMovement == 0f || unit.cache.cannotMove) return distanceToTiles
+        // If our escort can't move, ditto
+        if (includeOtherEscortUnit && unit.isEscorting()
+            && unit.getOtherEscortUnit()?.currentMovement == 0f) return distanceToTiles
 
         var tilesToCheck = listOf(unitTile)
-
+        
         while (tilesToCheck.isNotEmpty()) {
             val updatedTiles = ArrayList<Tile>()
             for (tileToCheck in tilesToCheck)
                 for (neighbor in tileToCheck.neighbors) {
-                    if (tilesToIgnore?.contains(neighbor) == true) continue // ignore this tile
+                    // ignore this tile
+                    if (tilesToIgnoreBitset != null && tilesToIgnoreBitset.get(neighbor.zeroBasedIndex)) continue // ignore this tile
                     var totalDistanceToTile: Float = when {
                         !neighbor.isExplored(unit.civ) ->
-                            distanceToTiles[tileToCheck]!!.totalDistance + 1f  // If we don't know then we just guess it to be 1.
-                        !passThroughCache.getOrPut(neighbor) { canPassThrough(neighbor) } -> unitMovement // Can't go here.
+                            distanceToTiles[tileToCheck]!!.totalMovement + 1f  // If we don't know then we just guess it to be 1.
+                        
+                        !canPassThroughCache.getOrPut(neighbor.zeroBasedIndex){
+                            canPassThrough(neighbor)
+                        } -> unitMovement // Can't go here.
                         // The reason that we don't just "return" is so that when calculating how to reach an enemy,
                         // You need to assume his tile is reachable, otherwise all movement algorithms on reaching enemy
                         // cities and units goes kaput.
                         else -> {
-                            val key = Pair(tileToCheck, neighbor)
-                            val movementCost =
-                                movementCostCache.getOrPut(key) {
-                                    MovementCost.getMovementCostBetweenAdjacentTiles(unit, tileToCheck, neighbor, considerZoneOfControl)
-                                }
-                            distanceToTiles[tileToCheck]!!.totalDistance + movementCost
+                            val key = HexMath.tilesAndNeighborUniqueIndex(tileToCheck, neighbor)
+                            val movementCost = movementCostCache.getOrPut(key) {
+                                MovementCost.getMovementCostBetweenAdjacentTilesEscort(unit, tileToCheck, neighbor, considerZoneOfControl, includeOtherEscortUnit)
+                            }
+                            distanceToTiles[tileToCheck]!!.totalMovement + movementCost
                         }
                     }
 
-                    if (!distanceToTiles.containsKey(neighbor) || distanceToTiles[neighbor]!!.totalDistance > totalDistanceToTile) { // this is the new best path
-                        if (totalDistanceToTile < unitMovement - Constants.minimumMovementEpsilon)  // We can still keep moving from here!
+                    val currentBestPath = distanceToTiles[neighbor]
+                    if (currentBestPath == null || currentBestPath.totalMovement > totalDistanceToTile) { // this is the new best path
+                        val usableMovement = if (includeOtherEscortUnit && unit.isEscorting())
+                            minOf(unitMovement, unit.getOtherEscortUnit()!!.currentMovement)
+                        else unitMovement
+
+                        if (totalDistanceToTile < usableMovement - Constants.minimumMovementEpsilon)  // We can still keep moving from here!
                             updatedTiles += neighbor
                         else
-                            totalDistanceToTile = unitMovement
+                            totalDistanceToTile = usableMovement
                         // In Civ V, you can always travel between adjacent tiles, even if you don't technically
                         // have enough movement points - it simply depletes what you have
 
-                        distanceToTiles[neighbor] = ParentTileAndTotalDistance(neighbor, tileToCheck, totalDistanceToTile)
+                        distanceToTiles[neighbor] = ParentTileAndTotalMovement(neighbor, tileToCheck, totalDistanceToTile)
                     }
                 }
 
@@ -88,14 +109,16 @@ class UnitMovement(val unit: MapUnit) {
      * Does not consider if the [destination] tile can actually be entered, use [canMoveTo] for that.
      * Returns an empty list if there's no way to get to the destination.
      */
+    @Readonly
     fun getShortestPath(destination: Tile, avoidDamagingTerrain: Boolean = false): List<Tile> {
         if (unit.cache.cannotMove) return listOf()
 
         // First try and find a path without damaging terrain
-        if (!avoidDamagingTerrain && unit.civ.passThroughImpassableUnlocked && unit.baseUnit.isLandUnit()) {
+        if (!avoidDamagingTerrain && unit.civ.passThroughImpassableUnlocked && unit.baseUnit.isLandUnit) {
             val damageFreePath = getShortestPath(destination, true)
             if (damageFreePath.isNotEmpty()) return damageFreePath
         }
+        
         if (destination.neighbors.none { isUnknownTileWeShouldAssumeToBePassable(it) || canPassThrough(it) }) {
             // edge case where this all of the tiles around the destination are
             // explored and known the unit can't pass through any of thoes tiles so we know a priori that no path exists
@@ -120,32 +143,38 @@ class UnitMovement(val unit: MapUnit) {
         var distance = 1
         val unitMaxMovement = unit.getMaxMovement().toFloat()
         val newTilesToCheck = ArrayList<Tile>()
-        val visitedTiles: HashSet<Tile> = hashSetOf(currentTile)
+        val visitedTilesBitset = BitSet(currentTile.tileMap.tileList.size)
+        visitedTilesBitset.set(currentTile.zeroBasedIndex)
         val civilization = unit.civ
 
-        val passThroughCache = HashMap<Tile, Boolean>()
-        val movementCostCache = HashMap<Pair<Tile, Tile>, Float>()
+        val passThroughCacheNew = ArrayList<Boolean?>(currentTile.tileMap.tileList.size)
+        val movementCostCache = HashMap<Int, Float>()
         val canMoveToCache = HashMap<Tile, Boolean>()
 
         while (true) {
             newTilesToCheck.clear()
+            fun isUnfriendlyCityState(tile:Tile): Boolean = tile.getOwner().let { it != null && it.isCityState
+                    && it.getDiplomacyManager(unit.civ)?.isRelationshipLevelLT(RelationshipLevel.Friend) == true }
 
-            var tilesByPreference = tilesToCheck.sortedBy { it.aerialDistanceTo(destination) }
-            // Avoid embarkation when possible
-            if (unit.type.isLandUnit()) tilesByPreference = tilesByPreference.sortedByDescending { it.isLand }
-
+            // When comparing booleans, we get false first, so we need to negate the isLand / isCityState checks
+            // By order of preference: 1. Land tiles 2. Aerial distance 3. Not city states
+            val comparison: Comparator<Tile> = if (unit.type.isLandUnit())
+                compareBy({!it.isLand}, {it.aerialDistanceTo(destination)}, ::isUnfriendlyCityState)
+            else compareBy({it.aerialDistanceTo(destination)}, ::isUnfriendlyCityState)
+            
+            val tilesByPreference = tilesToCheck.sortedWith(comparison)
 
             for (tileToCheck in tilesByPreference) {
                 val distanceToTilesThisTurn = if (distance == 1) {
-                    getDistanceToTiles(true, passThroughCache, movementCostCache) // check cache
+                    getDistanceToTiles(true, passThroughCacheNew, movementCostCache) // check cache
                 }
                 else {
-                    getDistanceToTilesWithinTurn(
+                    getMovementToTilesAtPosition(
                         tileToCheck.position,
                         unitMaxMovement,
                         false,
-                        visitedTiles,
-                        passThroughCache,
+                        visitedTilesBitset,
+                        passThroughCacheNew,
                         movementCostCache
                     )
                 }
@@ -168,15 +197,15 @@ class UnitMovement(val unit: MapUnit) {
                         pathfindingCache.setShortestPathCache(destination, path)
 
                         return path
-                    } else {
-                        if (movementTreeParents.containsKey(reachableTile)) continue // We cannot be faster than anything existing...
-                        if (!isUnknownTileWeShouldAssumeToBePassable(reachableTile) &&
-                            !canMoveToCache.getOrPut(reachableTile) { canMoveTo(reachableTile) })
-                        // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
-                            continue
-                        movementTreeParents[reachableTile] = tileToCheck
-                        newTilesToCheck.add(reachableTile)
                     }
+                    
+                    if (movementTreeParents.containsKey(reachableTile)) continue // We cannot be faster than anything existing...
+                    if (!isUnknownTileWeShouldAssumeToBePassable(reachableTile) &&
+                        !canMoveToCache.getOrPut(reachableTile) { canMoveTo(reachableTile) })
+                    // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
+                        continue
+                    movementTreeParents[reachableTile] = tileToCheck
+                    newTilesToCheck.add(reachableTile)
                 }
             }
 
@@ -187,11 +216,11 @@ class UnitMovement(val unit: MapUnit) {
             }
 
             // add newTilesToCheck to visitedTiles so we do not path over these tiles in a later iteration
-            visitedTiles.addAll(newTilesToCheck)
+            for (tile in newTilesToCheck) visitedTilesBitset.set(tile.zeroBasedIndex)
             // no need to check tiles that are surrounded by reachable tiles, only need to check the edgemost tiles.
             // Because anything we can reach from intermediate tiles, can be more easily reached by the edgemost tiles,
             // since we'll have to pass through an edgemost tile in order to reach the destination anyway
-            tilesToCheck = newTilesToCheck.filterNot { tile -> tile.neighbors.all { it in visitedTiles } }
+            tilesToCheck = newTilesToCheck.filterNot { tile -> tile.neighbors.all { visitedTilesBitset.get(it.zeroBasedIndex) } }
 
             distance++
         }
@@ -199,13 +228,14 @@ class UnitMovement(val unit: MapUnit) {
 
     class UnreachableDestinationException(msg: String) : Exception(msg)
 
+    @Readonly
     fun getTileToMoveToThisTurn(finalDestination: Tile): Tile {
 
         val currentTile = unit.getTile()
         if (currentTile == finalDestination) return currentTile
 
         // If we can fly, head there directly
-        if ((unit.baseUnit.movesLikeAirUnits() || unit.isPreparingParadrop()) && canMoveTo(finalDestination)) return finalDestination
+        if ((unit.baseUnit.movesLikeAirUnits || unit.isPreparingParadrop()) && canMoveTo(finalDestination)) return finalDestination
 
         val distanceToTiles = getDistanceToTiles()
 
@@ -228,7 +258,7 @@ class UnitMovement(val unit: MapUnit) {
             in destinationNeighbors -> currentTile // We're right nearby anyway, no need to move
             else -> destinationNeighbors
                 .filter { distanceToTiles.containsKey(it) && canMoveTo(it) }
-                .minByOrNull { distanceToTiles.getValue(it).totalDistance } // we can get a little closer
+                .minByOrNull { distanceToTiles.getValue(it).totalMovement } // we can get a little closer
                 ?: currentTile // We can't get closer...
         }
     }
@@ -237,13 +267,8 @@ class UnitMovement(val unit: MapUnit) {
      * @return The tile that we reached this turn
      */
     fun headTowards(destination: Tile): Tile {
-        val escortUnit = if (unit.isEscorting()) unit.getOtherEscortUnit() else null
-        val startTile = unit.getTile()
         val destinationTileThisTurn = getTileToMoveToThisTurn(destination)
         moveToTile(destinationTileThisTurn)
-        if (startTile != unit.getTile() && escortUnit != null) {
-            escortUnit.movement.headTowards(unit.getTile())
-        }
         return unit.currentTile
     }
 
@@ -251,22 +276,25 @@ class UnitMovement(val unit: MapUnit) {
      *  Also note that REACHABLE tiles are not necessarily tiles that the unit CAN ENTER
      *  @see canReachInCurrentTurn
      */
+    @Readonly
     fun canReach(destination: Tile) = canReachCommon(destination) {
         getShortestPath(it).any()
     }
 
     /** Cached and thus not as performance-heavy as [canReach] */
+    @Readonly
     fun canReachInCurrentTurn(destination: Tile) = canReachCommon(destination) {
         getDistanceToTiles().containsKey(it)
     }
 
-    private inline fun canReachCommon(destination: Tile, specificFunction: (Tile) -> Boolean) = when {
+    @Readonly
+    private inline fun canReachCommon(destination: Tile, @Readonly specificFunction: (Tile) -> Boolean) = when {
         unit.cache.cannotMove ->
             destination == unit.getTile()
-        unit.baseUnit.movesLikeAirUnits() ->
+        unit.baseUnit.movesLikeAirUnits ->
             unit.currentTile.aerialDistanceTo(destination) <= unit.getMaxMovementForAirUnits()
         unit.isPreparingParadrop() ->
-            unit.currentTile.aerialDistanceTo(destination) <= unit.cache.paradropRange && canParadropOn(destination)
+            canParadropOn(destination, unit.currentTile.aerialDistanceTo(destination))
         else ->
             specificFunction(destination)  // Note: Could pass destination as implicit closure from outer fun to lambda, but explicit is clearer
     }
@@ -275,14 +303,16 @@ class UnitMovement(val unit: MapUnit) {
      * @param includeOtherEscortUnit determines whether or not this method will also check its the other escort unit if it has one
      * Leave it as default unless you know what [getReachableTilesInCurrentTurn] does.
      */
+    @Readonly
     fun getReachableTilesInCurrentTurn(includeOtherEscortUnit: Boolean = true): Sequence<Tile> {
         return when {
             unit.cache.cannotMove -> sequenceOf(unit.getTile())
-            unit.baseUnit.movesLikeAirUnits() ->
+            unit.baseUnit.movesLikeAirUnits ->
                 unit.getTile().getTilesInDistanceRange(IntRange(1, unit.getMaxMovementForAirUnits()))
-            unit.isPreparingParadrop() ->
-                unit.getTile().getTilesInDistance(unit.cache.paradropRange)
-                    .filter { unit.movement.canParadropOn(it) }
+            unit.isPreparingParadrop() -> {
+                unit.getTile().getTilesInDistance(unit.cache.paradropDestinationTileFilters.maxOf { it.value } )
+                    .filter { unit.movement.canParadropOn(it, it.aerialDistanceTo(unit.getTile())) }
+            }
             includeOtherEscortUnit && unit.isEscorting() -> {
                     val otherUnitTiles = unit.getOtherEscortUnit()!!.movement.getReachableTilesInCurrentTurn(false).toSet()
                     unit.movement.getDistanceToTiles().filter { otherUnitTiles.contains(it.key) }.keys.asSequence()
@@ -292,11 +322,13 @@ class UnitMovement(val unit: MapUnit) {
     }
 
     /** Returns whether we can perform a swap move to the specified tile */
+    @Readonly
     fun canUnitSwapTo(destination: Tile): Boolean {
         return canReachInCurrentTurn(destination) && canUnitSwapToReachableTile(destination)
     }
 
     /** Returns the tiles to which we can perform a swap move */
+    @Readonly
     fun getUnitSwappableTiles(): Sequence<Tile> {
         return getReachableTilesInCurrentTurn().filter { canUnitSwapToReachableTile(it) }
     }
@@ -305,29 +337,39 @@ class UnitMovement(val unit: MapUnit) {
      * Returns whether we can perform a unit swap move to the specified tile, given that it is
      * reachable in the current turn
      */
-    private fun canUnitSwapToReachableTile(reachableTile: Tile): Boolean {
+    @Readonly
+    private fun canUnitSwapToReachableTile(reachableTile: Tile, checkEscorted: Boolean = true): Boolean {
         // Air units cannot swap
-        if (unit.baseUnit.movesLikeAirUnits()) return false
+        if (unit.baseUnit.movesLikeAirUnits) return false
         // We can't swap with ourself
         if (reachableTile == unit.getTile()) return false
         if (unit.cache.cannotMove) return false
-        // Check whether the tile contains a unit of the same type as us that we own and that can
-        // also reach our tile in its current turn.
-        val otherUnit = (
-            if (unit.isCivilian())
-                reachableTile.civilianUnit
-            else
-                reachableTile.militaryUnit
-            ) ?: return false
+
+        // Check whether the tile contains a unit of the same type as us that we own and that can also reach our tile in its current turn.
+        // When looking for escort formation swaps, however, the 'other' unit should be taken disregarding this unit's type.
+        // (So when we have a escorting Warrior+Worker pair, and want to move that pair to a tile with just a Worker or juse a Warrior,
+        // we can get the Swap button no matter which of the pair we selected first)
+        val escortedUnit = if (checkEscorted && unit.isEscorting()) unit.getOtherEscortUnit() else null // Only isEscorting has the actual flag check
+        fun MapUnit.getMatchInReachableTile() =
+            if (isCivilian()) reachableTile.civilianUnit
+            else reachableTile.militaryUnit
+        val otherUnit =
+            unit.getMatchInReachableTile()
+            ?: escortedUnit?.getMatchInReachableTile()
+            ?: return false
         val ourPosition = unit.getTile()
         if (otherUnit.owner != unit.owner
-            || otherUnit.cache.cannotMove  // redundant, line below would cover it too
+            || otherUnit.cache.cannotMove  // redundant but faster, line below would cover it too
             || !otherUnit.movement.canReachInCurrentTurn(ourPosition)) return false
 
-        if (!canMoveTo(reachableTile, canSwap = true)) return false
-        if (!otherUnit.movement.canMoveTo(ourPosition, canSwap = true)) return false
-        // All clear!
-        return true
+        if (!canMoveTo(reachableTile, allowSwap = true)) return false
+        if (!otherUnit.movement.canMoveTo(ourPosition, allowSwap = true)) return false
+
+        if (escortedUnit == null) return true // All clear!
+
+        // Check whether the escorted unit can move OR swap too
+        if (escortedUnit.movement.canMoveTo(reachableTile, allowSwap = false, includeOtherEscortUnit = false)) return true
+        return escortedUnit.movement.canUnitSwapToReachableTile(reachableTile, false)
     }
 
     /**
@@ -367,7 +409,7 @@ class UnitMovement(val unit: MapUnit) {
             unit.removeFromTile() // we "teleport" them away
             unit.putInTile(allowedTile)
             // Cancel sleep or fortification if forcibly displaced - for now, leave movement / auto / explore orders
-            if (unit.isSleeping() || unit.isFortified())
+            if (unit.isSleeping() || unit.isFortified() || unit.isGuarding())
                 unit.action = null
             unit.mostRecentMoveType = UnitMovementMemoryType.UnitTeleported
 
@@ -390,7 +432,7 @@ class UnitMovement(val unit: MapUnit) {
         // Reset closestEnemy chache
         val escortUnit = if (unit.isEscorting()) unit.getOtherEscortUnit()!! else null
 
-        if (unit.baseUnit.movesLikeAirUnits()) { // air units move differently from all other units
+        if (unit.baseUnit.movesLikeAirUnits) { // air units move differently from all other units
             if (unit.action != UnitActionType.Automate.value) unit.action = null
             unit.removeFromTile()
             unit.isTransported = false // it has left the carrier by own means
@@ -426,7 +468,7 @@ class UnitMovement(val unit: MapUnit) {
         unit.mostRecentMoveType = UnitMovementMemoryType.UnitMoved
         val pathToLastReachableTile = distanceToTiles.getPathToTile(lastReachableTile)
 
-        if (unit.isFortified() || unit.isSetUpForSiege() || unit.isSleeping())
+        if (unit.isFortified() || unit.isGuarding() || unit.isSetUpForSiege() || unit.isSleeping())
             unit.action = null // un-fortify/un-setup/un-sleep after moving
 
         // If this unit is a carrier, keep record of its air payload whereabouts.
@@ -519,7 +561,9 @@ class UnitMovement(val unit: MapUnit) {
      * Swaps this unit with the unit on the given tile
      * Precondition: this unit can swap-move to the given tile, as determined by canUnitSwapTo
      */
-    fun swapMoveToTile(destination: Tile) {
+    fun swapMoveToTile(destination: Tile, keepEscorting: Boolean = false) {
+        if (keepEscorting && unit.isEscorting()) return swapMoveEscortPair(destination)
+
         unit.stopEscorting()
         val otherUnit = (
             if (unit.isCivilian())
@@ -531,8 +575,8 @@ class UnitMovement(val unit: MapUnit) {
         val ourOldPosition = unit.getTile()
         val theirOldPosition = otherUnit.getTile()
 
-        val ourPayload = ourOldPosition.getUnits().filter { it.isTransported }.toList()
-        val theirPayload = theirOldPosition.getUnits().filter { it.isTransported }.toList()
+        val ourPayload = ourOldPosition.getUnits().filter { it.isTransported && unit.isTransportTypeOf(it) }.toList()
+        val theirPayload = theirOldPosition.getUnits().filter { it.isTransported && otherUnit.isTransportTypeOf(it) }.toList()
 
         // Swap the units
         // Step 1: Release the destination tile
@@ -564,6 +608,21 @@ class UnitMovement(val unit: MapUnit) {
         unit.mostRecentMoveType = UnitMovementMemoryType.UnitMoved
     }
 
+    private fun swapMoveEscortPair(destination: Tile) {
+        val escortedUnit = unit.getOtherEscortUnit() ?: return
+        val destinationWasEscortingUnit = destination.militaryUnit?.takeIf { it.isEscorting() }
+        unit.stopEscorting() // If a moveToTile comes first - it would try to keep the escort and fail
+        fun MapUnit.needsSwapTo() = (if (isCivilian()) destination.civilianUnit else destination.militaryUnit) != null
+        fun MapUnit.swapOrMoveToDestination() =
+            if (needsSwapTo()) movement.swapMoveToTile(destination)
+            else movement.moveToTile(destination)
+        unit.swapOrMoveToDestination()
+        escortedUnit.swapOrMoveToDestination()
+        unit.startEscorting()
+        destinationWasEscortingUnit?.startEscorting()
+    }
+
+    @Readonly
     private fun isCityCenterCannotEnter(tile: Tile) = tile.isCityCenter()
         && tile.getOwner() != unit.civ
         && !tile.getCity()!!.hasJustBeenConquered
@@ -574,51 +633,77 @@ class UnitMovement(val unit: MapUnit) {
      * @param includeOtherEscortUnit determines whether or not this method will also check if the other escort unit [canMoveTo] if it has one.
      * Leave it as default unless you know what [canMoveTo] does.
      */
-    fun canMoveTo(tile: Tile, assumeCanPassThrough: Boolean = false, canSwap: Boolean = false, includeOtherEscortUnit: Boolean = true): Boolean {
-        if (unit.baseUnit.movesLikeAirUnits())
-            return canAirUnitMoveTo(tile, unit)
+    @Readonly
+    fun canMoveTo(tile: Tile, assumeCanPassThrough: Boolean = false, allowSwap: Boolean = false, includeOtherEscortUnit: Boolean = true) = 
+        getCannotMoveToReason(tile, assumeCanPassThrough, allowSwap, includeOtherEscortUnit) == null
+    
+    enum class CannotMoveToReason{
+        CannotPassThrough,
+        CannotEnterCityCenter,
+        EscortCannotMove,
+        TileIsNotEmpty,
+        NoAirUnitTransport,
+    }
+    
+    @Readonly
+    fun getCannotMoveToReason(tile: Tile, assumeCanPassThrough: Boolean = false, allowSwap: Boolean = false, includeOtherEscortUnit: Boolean = true): CannotMoveToReason? {
+        if (unit.baseUnit.movesLikeAirUnits)
+            return getAirUnitCannotMoveToReason(tile, unit)
 
         if (!assumeCanPassThrough && !canPassThrough(tile))
-            return false
+            return CannotMoveToReason.CannotPassThrough
 
         // even if they'll let us pass through, we can't enter their city - unless we just captured it
         if (isCityCenterCannotEnter(tile))
-            return false
+            return CannotMoveToReason.CannotEnterCityCenter
 
         if (includeOtherEscortUnit && unit.isEscorting()
-            && !unit.getOtherEscortUnit()!!.movement.canMoveTo(tile, assumeCanPassThrough,canSwap, includeOtherEscortUnit = false))
-            return false
+            && !unit.getOtherEscortUnit()!!.movement.canMoveTo(tile, assumeCanPassThrough, allowSwap, includeOtherEscortUnit = false))
+            return CannotMoveToReason.EscortCannotMove
 
-        return if (unit.isCivilian())
-            (tile.civilianUnit == null || (canSwap && tile.civilianUnit!!.owner == unit.owner))
+        val tileIsEmpty = if (unit.isCivilian())
+            (tile.civilianUnit == null || (allowSwap && tile.civilianUnit!!.owner == unit.owner))
                 && (tile.militaryUnit == null || tile.militaryUnit!!.owner == unit.owner)
         else
         // can skip checking for airUnit since not a city
-            (tile.militaryUnit == null || (canSwap && tile.militaryUnit!!.owner == unit.owner))
+            (tile.militaryUnit == null || (allowSwap && tile.militaryUnit!!.owner == unit.owner))
                 && (tile.civilianUnit == null || tile.civilianUnit!!.owner == unit.owner || unit.civ.isAtWarWith(tile.civilianUnit!!.civ))
+        
+        if (!tileIsEmpty) return CannotMoveToReason.TileIsNotEmpty
+        
+        return null
     }
 
-    private fun canAirUnitMoveTo(tile: Tile, unit: MapUnit): Boolean {
+    @Readonly
+    private fun getAirUnitCannotMoveToReason(tile: Tile, unit: MapUnit): CannotMoveToReason? {
         // landing in the city
         if (tile.isCityCenter()) {
-            if (tile.airUnits.filter { !it.isTransported }.size < 6 && tile.getCity()?.civ == unit.civ)
-                return true // if city is free - no problem, get in
+            if (tile.airUnits.filter { !it.isTransported }.size < tile.getCity()!!.getMaxAirUnits() && tile.getCity()?.civ == unit.civ)
+                return null // if city is free - no problem, get in
         } // let's check whether it enters city on carrier now...
 
         if (tile.militaryUnit != null) {
             val unitAtDestination = tile.militaryUnit!!
-            return unitAtDestination.canTransport(unit)
+            if (unitAtDestination.canTransport(unit)) return null
         }
-        return false
+        return CannotMoveToReason.NoAirUnitTransport
     }
 
     // Can a paratrooper land at this tile?
-    private fun canParadropOn(destination: Tile): Boolean {
+    @Readonly
+    private fun canParadropOn(destination: Tile, distance: Int): Boolean {
         if (unit.cache.cannotMove) return false
-        // Can only move to land tiles within range that are visible and not impassible
+
+        // Can only move to tiles within range that are visible and not impassible
         // Based on some testing done in the base game
-        if (!destination.isLand || destination.isImpassible() || !unit.civ.viewableTiles.contains(destination)) return false
-        return true
+        if (destination.isImpassible() || !unit.civ.viewableTiles.contains(destination)) return false
+
+        // The destination is valid if any of the `tileFilters` match, and is within range
+        for ((tileFilter, distanceAllowed) in unit.cache.paradropDestinationTileFilters) {
+            if (distance <= distanceAllowed && destination.matchesFilter(tileFilter, unit.civ)) return true
+        }
+
+        return false
     }
 
     /**
@@ -632,6 +717,7 @@ class UnitMovement(val unit: MapUnit) {
      * @param includeOtherEscortUnit determines whether or not this method will also check if the other escort unit [canPassThrough] if it has one.
      * Leave it as default unless you know what [canPassThrough] does.
      */
+    @Readonly
     fun canPassThrough(tile: Tile, includeOtherEscortUnit: Boolean = true): Boolean {
         if (tile.isImpassible()) {
             // special exception - ice tiles are technically impassible, but some units can move through them anyway
@@ -642,7 +728,7 @@ class UnitMovement(val unit: MapUnit) {
                 return false
         }
         if (tile.isLand
-            && unit.baseUnit.isWaterUnit()
+            && unit.baseUnit.isWaterUnit
             && !tile.isCityCenter())
             return false
 
@@ -651,8 +737,9 @@ class UnitMovement(val unit: MapUnit) {
                 unit.civ.getMatchingUniques(UniqueType.UnitsMayEnterOcean)
                     .any { unit.matchesFilter(it.params[0]) }
         }
-        if (tile.isWater && unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater) {
+        if (tile.isWater && unit.baseUnit.isLandUnit && !unit.cache.canMoveOnWater) {
             if (!unit.civ.tech.unitsCanEmbark) return false
+            if (unit.cache.cannotEmbark) return false
             if (tile.isOcean && !unit.civ.tech.embarkedUnitsCanEnterOcean && !unitSpecificAllowOcean)
                 return false
         }
@@ -660,7 +747,7 @@ class UnitMovement(val unit: MapUnit) {
             if (!unitSpecificAllowOcean && unit.cache.cannotEnterOceanTiles) return false
         }
 
-        if (unit.cache.canEnterCityStates && tile.getOwner()?.isCityState() == true)
+        if (unit.cache.canEnterCityStates && tile.getOwner()?.isCityState == true)
             return true
         if (!unit.cache.canEnterForeignTerrain && !tile.canCivPassThrough(unit.civ)) return false
 
@@ -673,7 +760,7 @@ class UnitMovement(val unit: MapUnit) {
         if (firstUnit != null && unit.civ != firstUnit.civ) {
             // Allow movement through unguarded, at-war Civilian Unit. Capture on the way
             // But not for Embarked Units capturing on Water
-            if (!(unit.baseUnit.isLandUnit() && tile.isWater && !unit.cache.canMoveOnWater)
+            if (!(unit.baseUnit.isLandUnit && tile.isWater && !unit.cache.canMoveOnWater)
                 && firstUnit.isCivilian() && unit.civ.isAtWarWith(firstUnit.civ))
                 return true
             // Cannot enter hostile tile with any unit in there
@@ -690,35 +777,23 @@ class UnitMovement(val unit: MapUnit) {
      * @param includeOtherEscortUnit determines whether or not this method will also check if the other escort units [getDistanceToTiles] if it has one.
      * Leave it as default unless you know what [getDistanceToTiles] does.
      */
+    @Readonly
     fun getDistanceToTiles(
         considerZoneOfControl: Boolean = true,
-        passThroughCache: HashMap<Tile, Boolean> = HashMap(),
-        movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap(),
+        passThroughCacheNew: ArrayList<Boolean?> = ArrayList(unit.getTile().tileMap.tileList.size),
+        movementCostCache: HashMap<Int, Float> = HashMap(),
         includeOtherEscortUnit: Boolean = true
     ): PathsToTilesWithinTurn {
-        val cacheResults = pathfindingCache.getDistanceToTiles(considerZoneOfControl)
-        if (cacheResults != null) {
-            return cacheResults
-        }
-        val distanceToTiles = getDistanceToTilesWithinTurn(
+        val distanceToTiles = getMovementToTilesAtPosition(
             unit.currentTile.position,
             unit.currentMovement,
             considerZoneOfControl,
             null,
-            passThroughCache,
-            movementCostCache
+            passThroughCacheNew,
+            movementCostCache,
+            includeOtherEscortUnit
         )
 
-        if (includeOtherEscortUnit) {
-            // Only save to cache only if we are the original call and not the subsequent escort unit call
-            pathfindingCache.setDistanceToTiles(considerZoneOfControl, distanceToTiles)
-            if (unit.isEscorting()) {
-                // We should only be able to move to tiles that our escort can also move to
-                val escortDistanceToTiles = unit.getOtherEscortUnit()!!.movement
-                    .getDistanceToTiles(considerZoneOfControl, includeOtherEscortUnit = false)
-                distanceToTiles.keys.removeAll { !escortDistanceToTiles.containsKey(it) }
-            }
-        }
         return distanceToTiles
     }
 
@@ -793,14 +868,13 @@ class UnitMovement(val unit: MapUnit) {
 class PathfindingCache(private val unit: MapUnit) {
     private var shortestPathCache = listOf<Tile>()
     private var destination: Tile? = null
-    private val distanceToTilesCache = mutableMapOf<Boolean, PathsToTilesWithinTurn>()
     private var movement = -1f
     private var currentTile: Tile? = null
 
     /** Check if the caches are valid (only checking if the unit has moved or consumed movement points;
      * the isPlayerCivilization check is performed in the functions because we want isValid() == false
      * to have a specific behavior) */
-    private fun isValid(): Boolean = (movement == unit.currentMovement) && (unit.getTile() == currentTile)
+    @Readonly private fun isValid(): Boolean = (movement == unit.currentMovement) && (unit.getTile() == currentTile)
 
     fun getShortestPathCache(destination: Tile): List<Tile> {
         if (unit.civ.isHuman()) return listOf()
@@ -818,23 +892,7 @@ class PathfindingCache(private val unit: MapUnit) {
         }
     }
 
-    fun getDistanceToTiles(zoneOfControl: Boolean): PathsToTilesWithinTurn? {
-        if (unit.civ.isHuman()) return null
-        if (isValid())
-            return distanceToTilesCache[zoneOfControl]
-        return null
-    }
-
-    fun setDistanceToTiles(zoneOfControl: Boolean, paths: PathsToTilesWithinTurn) {
-        if (unit.civ.isHuman()) return
-        if (!isValid()) {
-            clear() // we want to reset the entire cache at this point
-        }
-        distanceToTilesCache[zoneOfControl] = paths
-    }
-
     fun clear() {
-        distanceToTilesCache.clear()
         movement = unit.currentMovement
         currentTile = unit.getTile()
         destination = null
@@ -842,7 +900,7 @@ class PathfindingCache(private val unit: MapUnit) {
     }
 }
 
-class PathsToTilesWithinTurn : LinkedHashMap<Tile, UnitMovement.ParentTileAndTotalDistance>() {
+class PathsToTilesWithinTurn : LinkedHashMap<Tile, UnitMovement.ParentTileAndTotalMovement>() {
     fun getPathToTile(tile: Tile): List<Tile> {
         if (!containsKey(tile))
             throw Exception("Can't reach this tile!")
