@@ -48,6 +48,22 @@ class ModConstants {
     var minimalCityDistance = 3
     var minimalCityDistanceOnDifferentContinents = 2
 
+    var baseCityBombardRange = 2
+    var cityWorkRange = 3
+    var cityExpandRange = 5
+    
+    // Number of air units that can be stationed in a city
+    var cityAirUnitCapacity = 6
+
+    // Modifies how much the gold value of a one-sided trade is applied to the gifts diplomatic modifier.
+    // Eg: One side offers a city, resource or gold for nothing in return.
+    var goldGiftMultiplier = 1f
+    // Modifies how much the gold value of a regular trade is applied to the gifts diplomatic modifier.
+    var goldGiftTradeMultiplier = .8f
+    // Modifies how quickly the GaveUsGifts dimplomacy modifier runs out. A higher value makes it run out quicker.
+    // Normally the gifts reduced by ~2.5% per turn depending on the diplomatic relations with the default value.
+    var goldGiftDegradationMultiplier = 1f
+
     // Constants used to calculate Unit Upgrade gold Cost (can only be modded all-or-nothing)
     // This is a data class for one reason only: The equality implementation enables Gdx Json to omit it when default (otherwise only the individual fields are omitted)
     data class UnitUpgradeCost(
@@ -67,8 +83,8 @@ class ModConstants {
     var naturalWonderCountAddedConstant = 0.1f
 
     // MapGenerator.spreadAncientRuins: number of ruins = suitable tile count * this
-    var ancientRuinCountMultiplier = 0.02f
-    // MapGenerator.spawnIce: spawn Ice where T < this, with T calculated from temperatureExtremeness, latitude and perlin noise.
+    var ancientRuinCountMultiplier = 0.025f
+    // MapGenerator.spawnIce: spawn Ice where T < this, with T calculated from temperatureintensity, latitude and perlin noise.
     var spawnIceBelowTemperature = -0.8f
     // MapGenerator.spawnLakesAndCoasts: Water bodies up to this tile count become Lakes
     var maxLakeSize = 10
@@ -85,7 +101,39 @@ class ModConstants {
     var pantheonBase = 10
     var pantheonGrowth = 5
 
-    var workboatAutomationSearchMaxTiles = 20
+    // AI behaviour
+    var workboatAutomationSearchMaxTiles = 37
+    var minimumCityLocationTileValue = 53f
+
+    // Civilization
+    var minimumWarDuration = 10
+    var baseTurnsUntilRevolt = 4
+    var cityStateElectionTurns = 15
+
+    // Economy
+    var maxGoldTradeOffer = 10000
+
+    // City State Tribute: maximum points from Force ranking towards reaching Tribute willingness threshold
+    var tributeGlobalModifier = 100 // 75 in BNW
+    var tributeLocalModifier = 100 // 125 in BNW
+
+    // Espionage
+    var maxSpyRank = 3
+    // How much of a skill bonus each rank gives.
+    // Rank 0 is 100, rank 1 is 130, and so on.
+    // Half as much for a coup.
+    var spyRankSkillPercentBonus = 30
+    // Rank 2 is +25% tech steal rate, rank 3 is +50%, and so on
+    var spyRankStealPercentBonus = 25
+    // Steal cost equal to 125% of the most expensive stealable tech
+    var spyTechStealCostModifier = 1.25f
+    
+    // Score value of things
+    var scoreFromPopulation = 3 // 4 in BNW
+    var scoreFromWonders = 40 // 25 in BNW
+
+    // UI: If set >= 0, ImprovementPicker will silently skip improvements whose tech requirement is more advanced than your current Era + this
+    var maxImprovementTechErasForward = -1
 
     fun merge(other: ModConstants) {
         for (field in this::class.java.declaredFields) {
