@@ -2,11 +2,13 @@ package com.unciv.models
 
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.unciv.Constants
+import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.translations.getPlaceholderParameters
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.images.ImageGetter
+import com.unciv.models.ruleset.unique.UniqueType
 
 
 /** Unit Actions - class - carries dynamic data and actual execution.
@@ -23,24 +25,55 @@ open class UnitAction(
     val title: String = type.value,
     val isCurrentAction: Boolean = false,
     val uncivSound: UncivSound = type.uncivSound,
+    val associatedUnique: Unique? = null,
     /** Action is Null if this unit *can* execute the action but *not right now* - it's embarked, out of moves, etc */
     val action: (() -> Unit)? = null
 ) {
-    fun getIcon(): Actor {
+    fun getIcon(size: Float = 20f): Actor {
         if (type.imageGetter != null)
             return type.imageGetter.invoke()
         return when (type) {
             UnitActionType.CreateImprovement -> {
-                ImageGetter.getImprovementPortrait(title.getPlaceholderParameters()[0])
+                ImageGetter.getImprovementPortrait(title.getPlaceholderParameters()[0], size)
             }
             UnitActionType.SpreadReligion -> {
                 val religionName = title.getPlaceholderParameters()[0]
                 ImageGetter.getReligionPortrait(
                     if (ImageGetter.religionIconExists(religionName)) religionName
-                    else "Pantheon", 20f
+                    else "Pantheon", size
                 )
             }
-            else -> ImageGetter.getUnitActionPortrait("Star")
+            UnitActionType.TriggerUnique -> {
+                when (associatedUnique?.type) {
+                    UniqueType.OneTimeEnterGoldenAge, UniqueType.OneTimeEnterGoldenAgeTurns -> ImageGetter.getUnitActionPortrait("StartGoldenAge", size)
+                    UniqueType.GainFreeBuildings, UniqueType.RemoveBuilding, UniqueType.OneTimeSellBuilding, UniqueType.OneTimeFreeUnit, UniqueType.FreeSpecificBuildings -> ImageGetter.getConstructionPortrait(associatedUnique.params[0], size)
+                    UniqueType.OneTimeAmountFreeUnits -> ImageGetter.getConstructionPortrait(associatedUnique.params[1], size)
+                    UniqueType.OneTimeFreePolicy, UniqueType.OneTimeAmountFreePolicies, UniqueType.OneTimeAdoptPolicyOrBelief, UniqueType.OneTimeRemovePolicy, UniqueType.OneTimeRemovePolicyRefund -> ImageGetter.getUnitActionPortrait("HurryPolicy", size)
+                    UniqueType.OneTimeRevealEntireMap, UniqueType.OneTimeRevealSpecificMapTiles, UniqueType.OneTimeRevealCrudeMap -> ImageGetter.getUnitActionPortrait("Explore", size)
+                    UniqueType.OneTimeConsumeResources, UniqueType.OneTimeProvideResources, UniqueType.OneTimeGainResource -> ImageGetter.getResourcePortrait(associatedUnique.params[1], size)
+                    UniqueType.OneTimeChangeTerrain -> ImageGetter.getUnitActionPortrait("Transform", size)
+                    UniqueType.OneTimeRemoveResourcesFromTile, UniqueType.OneTimeRemoveImprovementsFromTile -> ImageGetter.getUnitActionPortrait("Pillage", size)
+                    UniqueType.OneTimeGainPopulation, UniqueType.OneTimeGainPopulationRandomCity -> ImageGetter.getStatIcon("Population", size)
+                    UniqueType.OneTimeGainStat -> ImageGetter.getStatIcon(associatedUnique.params[1], size)
+                    UniqueType.OneTimeGainStatRange -> ImageGetter.getStatIcon(associatedUnique.params[2], size)
+                    UniqueType.OneTimeUnitHeal -> ImageGetter.getPromotionPortrait("Heal Instantly", size)
+                    UniqueType.OneTimeUnitGainXP, UniqueType.OneTimeSpiesLevelUp -> ImageGetter.getUnitActionPortrait("Promote", size)
+                    UniqueType.OneTimeUnitUpgrade, UniqueType.OneTimeUnitSpecialUpgrade -> ImageGetter.getUnitActionPortrait("Upgrade", size)
+                    UniqueType.UnitsGainPromotion, UniqueType.OneTimeUnitGainPromotion, UniqueType.OneTimeUnitRemovePromotion, UniqueType.OneTimeUnitGainStatus, UniqueType.OneTimeUnitLoseStatus -> ImageGetter.getPromotionPortrait(associatedUnique.params[1], size)
+                    UniqueType.OneTimeFreeBelief, UniqueType.OneTimeGainPantheon, UniqueType.OneTimeGainProphet -> ImageGetter.getUnitActionPortrait("EnhanceReligion", size)
+                    UniqueType.OneTimeUnitDamage -> ImageGetter.getUnitActionPortrait("Pillage", size)
+                    UniqueType.FreeStatBuildings -> ImageGetter.getUnitActionPortrait("HurryConstruction", size)
+                    UniqueType.TriggerEvent -> ImageGetter.getUniquePortrait(associatedUnique.params[0], size)
+                    UniqueType.OneTimeUnitGainMovement -> ImageGetter.getUnitActionPortrait("MoveTo", size)
+                    UniqueType.OneTimeUnitLoseMovement -> ImageGetter.getUnitActionPortrait("StopMove", size)
+                    UniqueType.OneTimeUnitDestroyed -> ImageGetter.getUnitActionPortrait("DisbandUnit", size)
+                    UniqueType.OneTimeFreeTechRuins, UniqueType.OneTimeAmountFreeTechs, UniqueType.OneTimeFreeTech -> ImageGetter.getUnitActionPortrait("HurryResearch", size)
+                    UniqueType.OneTimeGainTechPercent -> ImageGetter.getTechIconPortrait(associatedUnique.params[1], size)
+                    UniqueType.OneTimeDiscoverTech -> ImageGetter.getTechIconPortrait(associatedUnique.params[0], size)
+                    else -> ImageGetter.getUnitActionPortrait("Star", size)
+                }
+            }
+            else -> ImageGetter.getUnitActionPortrait("Star", size)
         }
     }
 
@@ -70,7 +103,7 @@ open class UnitAction(
 
 /** Specialized [UnitAction] for upgrades
  *
- *  Transports [unitToUpgradeTo] from [creation][com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade.getUpgradeAction]
+ *  Transports [unitToUpgradeTo] from [creation][com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade.getUpgradeActions]
  *  to [UI][com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsTable.update]
  */
 class UpgradeUnitAction(
@@ -81,18 +114,22 @@ class UpgradeUnitAction(
     action: (() -> Unit)?
 ) : UnitAction(UnitActionType.Upgrade, 120f, title, action = action)
 
-/** Unit Actions - generic enum with static properties
+/**
+ * Unit Actions - generic enum with static properties
  *
- * @param value         _default_ label to display, can be overridden in UnitAction instantiation
+ * Note for Creators of new UnitActions:
+ * - If your action uses a dynamic label overriding `UnitActionType.value`,
+ *   then make sure `value` is the required translation template (if it requires multiple templates, leave it empty or use any example template).
+ * - If you use `tr()` language, make sure you use **at most one** pair of `{}` curly braces.
+ *
+ * Reason: `TranslationTests.allUnitActionsHaveTemplate` will check the existence of a matching template.
+ *
+ * @param value         _default_ label to display, can be overridden in UnitAction instantiation. In that case, this must be the translation template or empty.
  * @param imageGetter   optional lambda to get an Icon - `null` if icon is dependent on outside factors and needs special handling
  * @param binding       keyboard binding - omitting it will look up the KeyboardBinding of the same name (recommended)
  * @param isSkippingToNextUnit if "Auto Unit Cycle" setting and this bit are on, this action will skip to the next unit
  * @param uncivSound    _default_ sound, can be overridden in UnitAction instantiation
- */
-
-// Note for Creators of new UnitActions: If your action uses a dynamic label overriding UnitActionType.value,
-// then you need to teach [com.unciv.testing.TranslationTests.allUnitActionsHaveTranslation] how to deal with it!
-
+*/
 enum class UnitActionType(
     val value: String,
     val imageGetter: (()-> Actor)?,
@@ -126,13 +163,15 @@ enum class UnitActionType(
         { ImageGetter.getUnitActionPortrait("Fortify") }, UncivSound.Fortify),
     FortifyUntilHealed("Fortify until healed",
         { ImageGetter.getUnitActionPortrait("FortifyUntilHealed") }, UncivSound.Fortify),
+    Guard("Guard",
+        { ImageGetter.getUnitActionPortrait("Guard") }, UncivSound.Fortify, defaultPage = 0),
     Explore("Explore",
         { ImageGetter.getUnitActionPortrait("Explore") }),
     StopExploration("Stop exploration",
         { ImageGetter.getUnitActionPortrait("Stop") }, false),
     Promote("Promote",
         { ImageGetter.getUnitActionPortrait("Promote") }, false, UncivSound.Promote),
-    Upgrade("Upgrade",
+    Upgrade("Upgrade to [unitType] ([goldCost] gold)",
         { ImageGetter.getUnitActionPortrait("Upgrade") }, UncivSound.Upgrade),
     Transform("Transform",
         { ImageGetter.getUnitActionPortrait("Transform") }, UncivSound.Upgrade),
@@ -165,8 +204,8 @@ enum class UnitActionType(
     FoundReligion("Found a Religion",
         { ImageGetter.getUnitActionPortrait("FoundReligion") }, UncivSound.Choir),
     TriggerUnique("Trigger unique",
-        { ImageGetter.getUnitActionPortrait("Star") }, false, UncivSound.Chimes),
-    SpreadReligion("Spread Religion",
+        null, false, UncivSound.Chimes),
+    SpreadReligion("Spread [religionName]",
         null, UncivSound.Choir),
     RemoveHeresy("Remove Heresy",
         { ImageGetter.getUnitActionPortrait("RemoveHeresy") }, UncivSound.Fire),
@@ -176,8 +215,8 @@ enum class UnitActionType(
         { ImageGetter.getUnitActionPortrait("DisbandUnit") }, false, defaultPage = 1),
     GiftUnit("Gift unit",
         { ImageGetter.getUnitActionPortrait("Present") }, UncivSound.Silent, defaultPage = 1),
-    Wait("Wait",
-        { ImageGetter.getUnitActionPortrait("Wait") }, UncivSound.Silent),
+    Skip("Skip turn",
+        { ImageGetter.getUnitActionPortrait("Skip") }, UncivSound.Silent, defaultPage = 0),
     ShowAdditionalActions("Show more",
         { ImageGetter.getUnitActionPortrait("ShowMore") }, false),
     HideAdditionalActions("Back",
@@ -194,6 +233,6 @@ enum class UnitActionType(
 
     val binding: KeyboardBinding =
             binding ?:
-            KeyboardBinding.values().firstOrNull { it.name == name } ?:
+            KeyboardBinding.entries.firstOrNull { it.name == name } ?:
             KeyboardBinding.None
 }
