@@ -4,16 +4,9 @@ import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.civilization.PlayerType
 import com.unciv.models.ruleset.Speed
 
-
-@Suppress("EnumEntryName")  // These merit unusual names
-enum class BaseRuleset(val fullName:String) {
-    Civ_V_Vanilla("Civ V - Vanilla"),
-    Civ_V_GnK("Civ V - Gods & Kings"),
-}
-
 class GameParameters : IsPartOfGameInfoSerialization { // Default values are the default new game
     var difficulty = "Prince"
-    var speed = Speed.DEFAULT
+    var speed: String = Speed.DEFAULT // Not an instance of class Speed
 
     var randomNumberOfPlayers = false
     var minNumberOfPlayers = 3
@@ -38,13 +31,22 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
     var nuclearWeaponsEnabled = true
     var espionageEnabled = false
     var noStartBias = false
+    var shufflePlayerOrder = false
 
     var victoryTypes: ArrayList<String> = arrayListOf()
     var startingEra = "Ancient era"
 
+    // Multiplayer parameters
     var isOnlineMultiplayer = false
     var multiplayerServerUrl: String? = null
     var anyoneCanSpectate = true
+    /** After this amount of minutes, anyone can choose to 'skip turn' of the current player to keep the game going */
+    var minutesUntilSkipTurn = 60 * 24
+    /** Initial players' timer to play before they can be forced to resign permanently*/
+    var minutesUntilForceResign = 3 * 24 * 60
+    /** Time a player recover on their timer before they can be forced to resign. Time isn't added if the player get their turn skipped*/
+    var minutesRecoveredPerTurn = 60 * 24
+
     var baseRuleset: String = BaseRuleset.Civ_V_GnK.fullName
     var mods = LinkedHashSet<String>()
 
@@ -74,6 +76,7 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
         parameters.nuclearWeaponsEnabled = nuclearWeaponsEnabled
         parameters.espionageEnabled = espionageEnabled
         parameters.noStartBias = noStartBias
+        parameters.shufflePlayerOrder = shufflePlayerOrder
         parameters.victoryTypes = ArrayList(victoryTypes)
         parameters.startingEra = startingEra
         parameters.isOnlineMultiplayer = isOnlineMultiplayer
