@@ -6,20 +6,27 @@ import com.badlogic.gdx.utils.Base64Coder
 import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.multiplayer.FriendList
+import com.unciv.logic.multiplayer.chat.ChatWebSocket
 import com.unciv.models.UncivSound
+import com.unciv.models.metadata.GameSettings.WindowState.Companion.minimumHeight
+import com.unciv.models.metadata.GameSettings.WindowState.Companion.minimumWidth
+import com.unciv.models.translations.tr
 import com.unciv.ui.components.fonts.FontFamilyData
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.KeyboardBindings
-import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
 import com.unciv.ui.screens.worldscreen.NotificationsScroll
 import com.unciv.utils.Display
 import com.unciv.utils.ScreenOrientation
+import java.awt.Rectangle
+import yairm210.purity.annotations.Readonly
 import java.text.Collator
+import java.text.NumberFormat
 import java.time.Duration
 import java.util.Locale
 import kotlin.reflect.KClass
 import kotlin.reflect.KMutableProperty0
 
+/** Settings that apply across all games, stored in GameSettings.json */
 class GameSettings {
 
     /** Allows panning the map by moving the pointer to the screen edges */
@@ -34,8 +41,11 @@ class GameSettings {
     var showSettlersSuggestedCityLocations: Boolean = true
 
     var checkForDueUnits: Boolean = true
+    var checkForDueUnitsCycles: Boolean = false
     var autoUnitCycle: Boolean = true
+    var smallUnitButton: Boolean = true
     var singleTapMove: Boolean = false
+    var longTapMove: Boolean = true
     var language: String = Constants.english
     @Transient
     var locale: Locale? = null
@@ -51,11 +61,12 @@ class GameSettings {
     var pauseBetweenTracks = 10
 
     var turnsBetweenAutosaves = 1
+    var maxAutosavesStored = 10
     var tileSet: String = Constants.defaultTileset
     var unitSet: String? = Constants.defaultUnitset
     var skin: String = Constants.defaultSkin
     var showTutorials: Boolean = true
-    var autoAssignCityProduction: Boolean = true
+    var autoAssignCityProduction: Boolean = false
 
     /** This set of construction names has two effects:
      *  * Matching constructions are no longer candidates for [autoAssignCityProduction]
@@ -84,6 +95,10 @@ class GameSettings {
     var useDemographics: Boolean = false
     var showZoomButtons: Boolean = false
     var forbidPopupClickBehindToClose: Boolean = false
+    var useCirclesToIndicateMovableTiles: Boolean = false
+    
+    var unitMovementButtonAnimation: Boolean = false
+    var unitActionsTableAnimation: Boolean = false
 
     var notificationsLogMaxTurns = 5
 
@@ -96,8 +111,8 @@ class GameSettings {
 
     var autoPlay = GameSettingsAutoPlay()
 
-    // This is a string not an enum so if tabs change it won't screw up the json serialization
-    var lastOverviewPage = EmpireOverviewCategories.Cities.name
+    /** Holds EmpireOverviewScreen per-page persistable states */
+    val overview = OverviewPersistableData()
 
     /** Orientation for mobile platforms */
     var displayOrientation = ScreenOrientation.Landscape
@@ -128,6 +143,9 @@ class GameSettings {
     /** Size of automatic display of UnitSet art in Civilopedia - 0 to disable */
     var pediaUnitArtSize = 0f
 
+    /** Cycle units by distance instead of queue */
+    var alternateUnitCycleOrder: Boolean = false
+
     /** Don't close developer console after a successful command */
     var keepConsoleOpen = false
     /** Persist the history of successful developer console commands */
@@ -146,6 +164,7 @@ class GameSettings {
     //region <Methods>
 
     fun save() {
+        if (Gdx.app == null) return // Simulation mode from ConsoleLauncher
         refreshWindowSize()
         UncivGame.Current.files.setGeneralSettings(this)
     }
@@ -164,14 +183,7 @@ class GameSettings {
     }
 
     fun updateLocaleFromLanguage() {
-        val bannedCharacters = listOf(' ', '_', '-', '(', ')') // Things not to have in enum names
-        val languageName = language.filterNot { it in bannedCharacters }
-        locale = try {
-            val code = LocaleCode.valueOf(languageName)
-            Locale(code.language, code.country)
-        } catch (_: Exception) {
-            Locale.getDefault()
-        }
+        locale = LocaleCode.getLocale(language)
     }
 
     fun getFontSize(): Int {
@@ -188,6 +200,11 @@ class GameSettings {
         return Collator.getInstance(getCurrentLocale())
     }
 
+    @Readonly
+    fun getCurrentNumberFormat(): NumberFormat {
+        return LocaleCode.getNumberFormatFromLanguage(language)
+    }
+
     //endregion
     //region <Nested classes>
 
@@ -204,7 +221,7 @@ class GameSettings {
      *  retrieving a valid position from our upstream libraries while the window is maximized or iconified has proven tricky so far.
      */
     data class WindowState(val width: Int = 900, val height: Int = 600) {
-        constructor(bounds: java.awt.Rectangle) : this(bounds.width, bounds.height)
+        constructor(bounds: Rectangle) : this(bounds.width, bounds.height)
 
         companion object {
             /** Our choice of minimum window width */
@@ -236,7 +253,7 @@ class GameSettings {
          *  @return `this` unchanged if it is within valid limits, otherwise a new WindowState that is.
          *  @see coerceIn
          */
-        fun coerceIn(maximumWindowBounds: java.awt.Rectangle) =
+        fun coerceIn(maximumWindowBounds: Rectangle) =
             coerceIn(maximumWindowBounds.width, maximumWindowBounds.height)
     }
 
@@ -245,84 +262,62 @@ class GameSettings {
         val virtualWidth: Float,
         val virtualHeight: Float
     ) {
+        Micro(630f,420f),
         Tiny(750f,500f),
         Small(900f,600f),
         Medium(1050f,700f),
         Large(1200f,800f),
-        Huge(1500f,1000f)
+        Huge(1500f,1000f),
+        FullHD(1920f, 1280f),
+        QuadHD(2560f, 1707f);
+        override fun toString() = name.tr() // Allow direct use in a SelectBox
     }
 
     enum class NationPickerListMode { Icons, List }
 
-    /** Map Unciv language key to Java locale, for the purpose of getting a Collator for sorting.
-     *  - Effect depends on the Java libraries and may not always conform to expectations.
-     *    If in doubt, debug and see what Locale instance you get and compare its properties with `Locale.getDefault()`.
-     *    (`Collator.getInstance(LocaleCode.*.run { Locale(language, country) }) to Collator.getInstance())`, drill to both `rules`, compare hashes - if equal and other properties equal, then Java doesn't know your Language))
-     *  @property name same as translation file name with ' ', '_', '-', '(', ')' removed
-     *  @property language ISO 639-1 code for the language
-     *  @property country ISO 3166 code for the nation this is predominantly spoken in
-     *  @property trueLanguage If set, used instead of language to trick Java into supplying a close-enough collator (a no-match would otherwise give us the default collator, not a collator for a partial match)
-     */
-    enum class LocaleCode(val language: String, val country: String, val trueLanguage: String? = null) {
-        Afrikaans("af", "ZA"),
-        Arabic("ar", "IQ"),
-        Belarusian("be", "BY"),
-        Bosnian("bs", "BA"),
-        BrazilianPortuguese("pt", "BR"),
-        Bulgarian("bg", "BG"),
-        Catalan("ca", "ES"),
-        Croatian("hr", "HR"),
-        Czech("cs", "CZ"),
-        Danish("da", "DK"),
-        Dutch("nl", "NL"),
-        English("en", "US"),
-        Estonian("et", "EE"),
-        Finnish("fi", "FI"),
-        French("fr", "FR"),
-        German("de", "DE"),
-        Greek("el", "GR"),
-        Hindi("hi", "IN"),
-        Hungarian("hu", "HU"),
-        Indonesian("in", "ID"),
-        Italian("it", "IT"),
-        Japanese("ja", "JP"),
-        Korean("ko", "KR"),
-        Latin("la", "IT"),
-        Latvian("lv", "LV"),
-        Lithuanian("lt", "LT"),
-        Malay("ms", "MY"),
-        Norwegian("no", "NO"),
-        NorwegianNynorsk("nn", "NO"),
-        PersianPinglishDIN("fa", "IR"), // These might just fall back to default
-        PersianPinglishUN("fa", "IR"),
-        Polish("pl", "PL"),
-        Portuguese("pt", "PT"),
-        Romanian("ro", "RO"),
-        Russian("ru", "RU"),
-        Rusyn("uk", "UA", "rus"), // No specific locale for rus exists, so use closest for collator
-        Serbian("sr", "RS"),
-        SimplifiedChinese("zh", "CN"),
-        Slovak("sk", "SK"),
-        Spanish("es", "ES"),
-        Swedish("sv", "SE"),
-        Thai("th", "TH"),
-        TraditionalChinese("zh", "TW"),
-        Turkish("tr", "TR"),
-        Ukrainian("uk", "UA"),
-        Vietnamese("vi", "VN"),
-        Zulu("zu", "ZA")
-    }
-
     //endregion
     //region Multiplayer-specific
 
     class GameSettingsMultiplayer {
-        var userId = ""
-        var passwords = mutableMapOf<String, String>()
+        private var userId = ""
+        fun getUserId() = userId
+        fun setUserId(value: String) {
+            if (userId.isNotEmpty() && userId != value) {
+                ChatWebSocket.restart(force = true)
+            }
+            userId = value
+        }
+
+        /**
+         * Never ever make it public. If you need a method make it.
+         * But do remember to call [ChatWebSocket.restart] with `force = true` whenever required.
+         */
+        private val passwords = mutableMapOf<String, String>()
+        @Readonly
+        fun getPassword(serverUrl: String) = passwords[serverUrl]
+        @Readonly
+        fun getCurrentServerPassword() = passwords[server]
+        fun setCurrentServerPassword(password: String) {
+            val oldPassword = passwords[server]
+            if (oldPassword != null && oldPassword != password) {
+                ChatWebSocket.restart(force = true)
+            }
+            passwords[server] = password
+        }
+
         @Suppress("unused")  // @GGuenni knows what he intended with this field
         var userName: String = ""
-        var server = Constants.uncivXyzServer
-        var friendList: MutableList<FriendList.Friend> = mutableListOf()
+
+        private var server = Constants.uncivXyzServer
+        fun getServer() = server
+        fun setServer(value: String) {
+            if (server != value) {
+                server = value
+                ChatWebSocket.restart(force = true)
+            }
+        }
+
+        val friendList: MutableList<FriendList.Friend> = mutableListOf()
         var turnCheckerEnabled = true
         var turnCheckerPersistentNotificationEnabled = true
         var turnCheckerDelay: Duration = Duration.ofMinutes(5)
@@ -354,30 +349,5 @@ class GameSettings {
         var autoPlayDiplomacy: Boolean = true
     }
 
-    @Suppress("SuspiciousCallableReferenceInLambda")  // By @Azzurite, safe as long as that warning below is followed
-    enum class GameSetting(
-        val kClass: KClass<*>,
-        private val propertyGetter: (GameSettings) -> KMutableProperty0<*>
-    ) {
-        //     Uncomment these once they are refactored to send events on change
-//     MULTIPLAYER_USER_ID(String::class, { it.multiplayer::userId }),
-//     MULTIPLAYER_SERVER(String::class, { it.multiplayer::server }),
-//     MULTIPLAYER_STATUSBUTTON_IN_SINGLEPLAYER(Boolean::class, { it.multiplayer::statusButtonInSinglePlayer }),
-//     MULTIPLAYER_TURN_CHECKER_ENABLED(Boolean::class, { it.multiplayer::turnCheckerEnabled }),
-//     MULTIPLAYER_TURN_CHECKER_PERSISTENT_NOTIFICATION_ENABLED(Boolean::class, { it.multiplayer::turnCheckerPersistentNotificationEnabled }),
-//     MULTIPLAYER_HIDE_DROPBOX_WARNING(Boolean::class, { it.multiplayer::hideDropboxWarning }),
-        MULTIPLAYER_TURN_CHECKER_DELAY(Duration::class, { it.multiplayer::turnCheckerDelay }),
-        MULTIPLAYER_CURRENT_GAME_REFRESH_DELAY(Duration::class, { it.multiplayer::currentGameRefreshDelay }),
-        MULTIPLAYER_ALL_GAME_REFRESH_DELAY(Duration::class, { it.multiplayer::allGameRefreshDelay }),
-        MULTIPLAYER_CURRENT_GAME_TURN_NOTIFICATION_SOUND(UncivSound::class, { it.multiplayer::currentGameTurnNotificationSound }),
-        MULTIPLAYER_OTHER_GAME_TURN_NOTIFICATION_SOUND(UncivSound::class, { it.multiplayer::otherGameTurnNotificationSound });
-
-        /** **Warning:** It is the obligation of the caller to select the same type [T] that the [kClass] of this property has */
-        fun <T> getProperty(settings: GameSettings): KMutableProperty0<T> {
-            @Suppress("UNCHECKED_CAST")
-            return propertyGetter(settings) as KMutableProperty0<T>
-        }
-    }
-
     //endregion
 }
