@@ -1,6 +1,5 @@
 package com.unciv.models.ruleset
 
-import com.unciv.Constants
 import com.unciv.logic.GameInfo
 import com.unciv.logic.MultiFilter
 import com.unciv.logic.city.City
@@ -9,19 +8,15 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileImprovement
-import com.unciv.models.ruleset.unique.Conditionals
-import com.unciv.models.ruleset.unique.LocalUniqueCache
-import com.unciv.models.ruleset.unique.StateForConditionals
-import com.unciv.models.ruleset.unique.Unique
-import com.unciv.models.ruleset.unique.UniqueParameterType
-import com.unciv.models.ruleset.unique.UniqueTarget
-import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unique.*
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
-import com.unciv.models.translations.fillPlaceholders
 import com.unciv.ui.components.extensions.getNeedMoreAmountString
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.objectdescriptions.BuildingDescriptions
+import yairm210.purity.annotations.Cache
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
 
 
 class Building : RulesetStatsObject(), INonPerpetualConstruction {
@@ -32,7 +27,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     var maintenance = 0
     private var percentStatBonus: Stats? = null
     var specialistSlots: Counter<String> = Counter()
-    fun newSpecialists(): Counter<String>  = specialistSlots
+    @Readonly fun newSpecialists(): Counter<String>  = specialistSlots
 
     var greatPersonPoints = Counter<String>()
 
@@ -40,7 +35,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     override var hurryCostModifier = 0
     var isWonder = false
     var isNationalWonder = false
-    fun isAnyWonder() = isWonder || isNationalWonder
+    @Readonly fun isAnyWonder() = isWonder || isNationalWonder
     var requiredBuilding: String? = null
 
     /** A strategic resource that will be consumed by this building */
@@ -48,7 +43,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
     /** This Building can only be built if one of these resources is nearby - it must be improved! */
     var requiredNearbyImprovedResources: List<String>? = null
-    var cityStrength = 0
+    var cityStrength = 0.0
     var cityHealth = 0
     var replaces: String? = null
     var uniqueTo: String? = null
@@ -65,11 +60,16 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     fun getDescription(city: City, showAdditionalInfo: Boolean) = BuildingDescriptions.getDescription(this, city, showAdditionalInfo)
     override fun getCivilopediaTextLines(ruleset: Ruleset) = BuildingDescriptions.getCivilopediaTextLines(this, ruleset)
 
-    override fun isHiddenBySettings(gameInfo: GameInfo): Boolean {
-        if (super<INonPerpetualConstruction>.isHiddenBySettings(gameInfo)) return true
+    override fun getSortGroup(ruleset: Ruleset): Int = ruleset.technologies[requiredTech]?.era(ruleset)?.eraNumber ?: 100
+    override fun getSubCategory(ruleset: Ruleset): String? = ruleset.technologies[requiredTech]?.era(ruleset)?.name ?: "Other"
+
+    override fun isUnavailableBySettings(gameInfo: GameInfo): Boolean {
+        if (super<INonPerpetualConstruction>.isUnavailableBySettings(gameInfo)) return true
         if (!gameInfo.gameParameters.nuclearWeaponsEnabled && hasUnique(UniqueType.EnablesNuclearWeapons)) return true
         return isHiddenByStartingEra(gameInfo)
     }
+    
+    @Readonly
     private fun isHiddenByStartingEra(gameInfo: GameInfo): Boolean {
         if (!isWonder) return false
         // do not rely on this.ruleset or unit tests break
@@ -77,41 +77,47 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
         return name in startingEra.startingObsoleteWonders
     }
 
+    @Readonly
     fun getStats(city: City,
                  /* By default, do not cache - if we're getting stats for only one building this isn't efficient.
                  * Only use a cache if it was sent to us from outside, which means we can use the results for other buildings.  */
                  localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Stats {
         // Calls the clone function of the NamedStats this class is derived from, not a clone function of this class
-        val stats = cloneStats()
+        @LocalState val stats = cloneStats()
+        
+        val conditionalState = city.state
 
         for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.StatsFromObject)) {
-            if (!matchesFilter(unique.params[1])) continue
+            if (!matchesFilter(unique.params[1], conditionalState)) continue
             stats.add(unique.stats)
         }
 
-        for (unique in getMatchingUniques(UniqueType.Stats, StateForConditionals(city.civ, city)))
+        for (unique in getMatchingUniques(UniqueType.Stats, conditionalState))
             stats.add(unique.stats)
 
         if (!isWonder)
             for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.StatsFromBuildings)) {
-                if (matchesFilter(unique.params[1]))
+                if (matchesFilter(unique.params[1], conditionalState))
                     stats.add(unique.stats)
             }
         return stats
     }
 
+    @Readonly
     fun getStatPercentageBonuses(city: City?, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Stats {
-        val stats = percentStatBonus?.clone() ?: Stats()
-        val civInfo = city?.civ ?: return stats  // initial stats
+        @LocalState val stats = percentStatBonus?.clone() ?: Stats()
+        if (city == null) return stats  // initial stats
 
-        for (unique in localUniqueCache.forCivGetMatchingUniques(civInfo, UniqueType.StatPercentFromObject)) {
-            if (matchesFilter(unique.params[2]))
+        val conditionalState = city.state
+
+        for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.StatPercentFromObject)) {
+            if (matchesFilter(unique.params[2], conditionalState))
                 stats.add(Stat.valueOf(unique.params[1]), unique.params[0].toFloat())
         }
 
-        for (unique in localUniqueCache.forCivGetMatchingUniques(civInfo, UniqueType.AllStatsPercentFromObject)) {
-            if (!matchesFilter(unique.params[1])) continue
-            for (stat in Stat.values()) {
+        for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.AllStatsPercentFromObject)) {
+            if (!matchesFilter(unique.params[1], conditionalState)) continue
+            for (stat in Stat.entries) {
                 stats.add(stat, unique.params[0].toFloat())
             }
         }
@@ -121,7 +127,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
     override fun getProductionCost(civInfo: Civilization, city: City?): Int {
         var productionCost = cost.toFloat()
-        val stateForConditionals = StateForConditionals(civInfo, city)
+        val stateForConditionals = city?.state ?: civInfo.state
 
         for (unique in getMatchingUniques(UniqueType.CostIncreasesWhenBuilt, stateForConditionals))
             productionCost += civInfo.civConstructions.builtItemsWithIncreasingCost[name] * unique.params[0].toInt()
@@ -132,7 +138,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
         for (unique in getMatchingUniques(UniqueType.CostPercentageChange, stateForConditionals))
             productionCost *= unique.params[0].toPercent()
 
-        if (civInfo.isCityState())
+        if (civInfo.isCityState)
             productionCost *= 1.5f
         else if (civInfo.isHuman()) {
             if (!isWonder)
@@ -151,29 +157,30 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
 
     override fun canBePurchasedWithStat(city: City?, stat: Stat): Boolean {
-        if (stat == Stat.Gold && isAnyWonder()) return false
-        if (city == null) return super.canBePurchasedWithStat(null, stat)
+        val purchaseReason = canBePurchasedWithStatReasons(null, stat)
+        if (purchaseReason != PurchaseReason.UniqueAllowed && stat == Stat.Gold && isAnyWonder()) return false
+        if (city == null) return purchaseReason.purchasable
 
-        val conditionalState = StateForConditionals(civInfo = city.civ, city = city)
+        val conditionalState = city.state
         return (
             city.getMatchingUniques(UniqueType.BuyBuildingsIncreasingCost, conditionalState)
                 .any {
                     it.params[2] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[3])
                 }
             || city.getMatchingUniques(UniqueType.BuyBuildingsByProductionCost, conditionalState)
-                .any { it.params[1] == stat.name && matchesFilter(it.params[0]) }
+                .any { it.params[1] == stat.name && matchesFilter(it.params[0], conditionalState) }
             || city.getMatchingUniques(UniqueType.BuyBuildingsWithStat, conditionalState)
                 .any {
                     it.params[1] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[2])
                 }
             || city.getMatchingUniques(UniqueType.BuyBuildingsForAmountStat, conditionalState)
                 .any {
                     it.params[2] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[3])
                 }
             || super.canBePurchasedWithStat(city, stat)
@@ -181,7 +188,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     }
 
     override fun getBaseBuyCost(city: City, stat: Stat): Float? {
-        val conditionalState = StateForConditionals(civInfo = city.civ, city = city)
+        val conditionalState = city.state
 
         return sequence {
             val baseCost = super.getBaseBuyCost(city, stat)
@@ -190,7 +197,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
             yieldAll(city.getMatchingUniques(UniqueType.BuyBuildingsIncreasingCost, conditionalState)
                 .filter {
                     it.params[2] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[3])
                 }.map {
                     getCostForConstructionsIncreasingInPrice(
@@ -201,13 +208,13 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
                 }
             )
             yieldAll(city.getMatchingUniques(UniqueType.BuyBuildingsByProductionCost, conditionalState)
-                .filter { it.params[1] == stat.name && matchesFilter(it.params[0]) }
+                .filter { it.params[1] == stat.name && matchesFilter(it.params[0], conditionalState) }
                 .map { (getProductionCost(city.civ, city) * it.params[2].toInt()).toFloat() }
             )
             if (city.getMatchingUniques(UniqueType.BuyBuildingsWithStat, conditionalState)
                 .any {
                     it.params[1] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[2])
                 }
             ) {
@@ -216,7 +223,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
             yieldAll(city.getMatchingUniques(UniqueType.BuyBuildingsForAmountStat, conditionalState)
                 .filter {
                     it.params[2] == stat.name
-                    && matchesFilter(it.params[0])
+                    && matchesFilter(it.params[0], conditionalState)
                     && city.matchesFilter(it.params[3])
                 }.map { it.params[1].toInt() * city.civ.gameInfo.speed.statCostModifiers[stat]!! }
             )
@@ -225,13 +232,14 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
     override fun getStatBuyCost(city: City, stat: Stat): Int? {
         var cost = getBaseBuyCost(city, stat)?.toDouble() ?: return null
+        val conditionalState = city.state
 
         for (unique in city.getMatchingUniques(UniqueType.BuyItemsDiscount))
             if (stat.name == unique.params[0])
                 cost *= unique.params[1].toPercent()
 
         for (unique in city.getMatchingUniques(UniqueType.BuyBuildingsDiscount)) {
-            if (stat.name == unique.params[0] && matchesFilter(unique.params[1]))
+            if (stat.name == unique.params[0] && matchesFilter(unique.params[1], conditionalState))
                 cost *= unique.params[2].toPercent()
         }
 
@@ -248,6 +256,10 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
         val rejectionReasons = getRejectionReasons(cityConstructions)
 
+        if (hasUnique(UniqueType.ShowsWhenUnbuilable, cityConstructions.city.state) &&
+            rejectionReasons.none { it.isNeverVisible() })
+            return true
+
         if (rejectionReasons.any { it.type == RejectionReasonType.RequiresBuildingInSomeCities }
                 && cityConstructions.city.civ.gameInfo.gameParameters.oneCityChallenge)
             return false // You will never be able to get more cities, this building is effectively disabled
@@ -258,14 +270,15 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     }
 
     override fun getRejectionReasons(cityConstructions: CityConstructions): Sequence<RejectionReason> = sequence {
-        val cityCenter = cityConstructions.city.getCenterTile()
-        val civ = cityConstructions.city.civ
-        val ruleSet = civ.gameInfo.ruleset
+        val city = cityConstructions.city
+        val cityCenter = city.getCenterTile()
+        val civ = city.civ
+        val stateForConditionals = city.state
 
         if (cityConstructions.isBuilt(name))
             yield(RejectionReasonType.AlreadyBuilt.toInstance())
 
-        if (isHiddenBySettings(civ.gameInfo)) {
+        if (isUnavailableBySettings(civ.gameInfo)) {
             // Repeat the starting era test isHiddenBySettings already did to change the RejectionReasonType
             if (isHiddenByStartingEra(civ.gameInfo))
                 yield(RejectionReasonType.WonderDisabledEra.toInstance())
@@ -278,7 +291,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
             // EXCEPT for [UniqueType.OnlyAvailable] and [UniqueType.CanOnlyBeBuiltInCertainCities]
             // since they trigger (reject) only if conditionals ARE NOT met
             if (unique.type != UniqueType.OnlyAvailable && unique.type != UniqueType.CanOnlyBeBuiltWhen &&
-                !unique.conditionalsApply(StateForConditionals(civ, cityConstructions.city))) continue
+                !unique.conditionalsApply(stateForConditionals)) continue
 
             when (unique.type) {
                 // for buildings that are created as side effects of other things, and not directly built,
@@ -296,7 +309,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
                     yield(RejectionReasonType.ShouldNotBeDisplayed.toInstance())
 
                 UniqueType.RequiresPopulation ->
-                    if (unique.params[0].toInt() > cityConstructions.city.population.population)
+                    if (unique.params[0].toInt() > city.population.population)
                         yield(RejectionReasonType.PopulationRequirement.toInstance(unique.text))
 
                 UniqueType.MustBeOn ->
@@ -308,7 +321,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
                         yield(RejectionReasonType.MustNotBeOnTile.toInstance(unique.text))
 
                 UniqueType.MustBeNextTo ->
-                    if (!cityCenter.isAdjacentTo(unique.params[0], civ))
+                    if (!cityCenter.isAdjacentTo(unique.params[0], civ) && !cityCenter.matchesFilter(unique.params[0], civ))
                         yield(RejectionReasonType.MustBeNextToTile.toInstance(unique.text))
 
                 UniqueType.MustNotBeNextTo ->
@@ -317,14 +330,10 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
 
                 UniqueType.MustHaveOwnedWithinTiles ->
                     if (cityCenter.getTilesInDistance(unique.params[1].toInt())
-                        .none { it.matchesFilter(unique.params[0], civ) && it.getOwner() == cityConstructions.city.civ }
+                        .none { it.matchesFilter(unique.params[0], civ) && it.getOwner() == civ }
                     )
                         yield(RejectionReasonType.MustOwnTile.toInstance(unique.text))
 
-                UniqueType.CanOnlyBeBuiltInCertainCities ->
-                    if (!cityConstructions.city.matchesFilter(unique.params[0]))
-                        yield(RejectionReasonType.CanOnlyBeBuiltInSpecificCities.toInstance(unique.text))
-
                 UniqueType.ObsoleteWith ->
                     if (civ.tech.isResearched(unique.params[0]))
                         yield(RejectionReasonType.Obsoleted.toInstance(unique.text))
@@ -338,52 +347,21 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
                     if (!civ.hasUnique(UniqueType.EnablesConstructionOfSpaceshipParts))
                         yield(RejectionReasonType.RequiresBuildingInSomeCity.toInstance("Apollo project not built!"))
                 }
-
-                UniqueType.RequiresBuildingInSomeCities -> {
-                    val buildingFilter = unique.params[0]
-                    val numberOfCitiesRequired = unique.params[1].toInt()
-                    val numberOfCitiesWithBuilding = civ.cities.count {
-                        it.cityConstructions.containsBuildingOrEquivalent(buildingFilter)
+                
+                UniqueType.CreatesOneImprovement -> {
+                    val improvement = ruleset.tileImprovements[unique.params[0]]
+                    if (improvement == null) {
+                        yield(RejectionReasonType.NoSuchImprovement.toInstance("Unknown improvement: ${unique.params[0]}"))
+                    } else if (city.getTiles().none { !it.improvementFunctions.canBuildImprovement(improvement, city.state) }) {
+                        yield(RejectionReasonType.NoTileCanContainImprovement.toInstance())
                     }
-                    if (numberOfCitiesWithBuilding < numberOfCitiesRequired) {
-                        val equivalentBuildingFilter = if (ruleSet.buildings.containsKey(buildingFilter))
-                            civ.getEquivalentBuilding(buildingFilter).name
-                        else buildingFilter
-                        yield(
-                                // replace with civ-specific building for user
-                                RejectionReasonType.RequiresBuildingInSomeCities.toInstance(
-                                    unique.text.fillPlaceholders(equivalentBuildingFilter, numberOfCitiesRequired.toString()) +
-                                            " ($numberOfCitiesWithBuilding/$numberOfCitiesRequired)"
-                                ) )
-                    }
-                }
-
-                UniqueType.RequiresBuildingInAllCities -> {
-                    val filter = unique.params[0]
-                    if (civ.gameInfo.ruleset.buildings.containsKey(filter)
-                            && civ.cities.any {
-                                !it.isPuppet && !it.cityConstructions.containsBuildingOrEquivalent(unique.params[0])
-                            }
-                    ) {
-                        yield(
-                                // replace with civ-specific building for user
-                                RejectionReasonType.RequiresBuildingInAllCities.toInstance(
-                                    "Requires a [${civ.getEquivalentBuilding(unique.params[0])}] in all cities"
-                                )
-                        )
-                    }
-                }
-
-                UniqueType.HiddenBeforeAmountPolicies -> {
-                    if (cityConstructions.city.civ.getCompletedPolicyBranchesCount() < unique.params[0].toInt())
-                        yield(RejectionReasonType.MorePolicyBranches.toInstance(unique.text))
                 }
 
                 else -> {}
             }
         }
 
-        if (uniqueTo != null && uniqueTo != civ.civName)
+        if (uniqueTo != null && !civ.matchesFilter(uniqueTo!!, stateForConditionals))
             yield(RejectionReasonType.UniqueToOtherNation.toInstance("Unique to $uniqueTo"))
 
         if (civ.cache.uniqueBuildings.any { it.replaces == name })
@@ -398,7 +376,7 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
             if (civ.cities.any { it != cityConstructions.city && it.cityConstructions.isBeingConstructedOrEnqueued(name) })
                 yield(RejectionReasonType.WonderBeingBuiltElsewhere.toInstance())
 
-            if (civ.isCityState())
+            if (civ.isCityState)
                 yield(RejectionReasonType.CityStateWonder.toInstance())
 
             if (cityConstructions.city.isPuppet)
@@ -421,23 +399,31 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
             yield(RejectionReasonType.RequiresBuildingInThisCity.toInstance("Requires a [${civ.getEquivalentBuilding(requiredBuilding!!)}] in this city"))
         }
 
-        for ((resourceName, requiredAmount) in getResourceRequirementsPerTurn(
-            StateForConditionals(cityConstructions.city.civ, cityConstructions.city))
-        ) {
+        for ((resourceName, requiredAmount) in getResourceRequirementsPerTurn(stateForConditionals)) {
             val availableAmount = cityConstructions.city.getAvailableResourceAmount(resourceName)
             if (availableAmount < requiredAmount) {
                 yield(RejectionReasonType.ConsumesResources.toInstance(resourceName.getNeedMoreAmountString(requiredAmount - availableAmount)))
             }
         }
+        
+        // If we've already paid the unit costs, we don't need to pay it again
+        if (cityConstructions.getWorkDone(name) == 0)
+            for ((resourceName, amount) in getStockpiledResourceRequirements(stateForConditionals)) {
+                val availableResources = cityConstructions.city.getAvailableResourceAmount(resourceName)
+                if (availableResources < amount)
+                    yield(RejectionReasonType.ConsumesResources.toInstance(resourceName.getNeedMoreAmountString(amount - availableResources)))
+            }
 
         if (requiredNearbyImprovedResources != null) {
             val containsResourceWithImprovement = cityConstructions.city.getWorkableTiles()
                 .any {
-                    it.resource != null
-                    && requiredNearbyImprovedResources!!.contains(it.resource!!)
-                    && it.getOwner() == civ
-                    && ((it.getUnpillagedImprovement() != null && it.tileResource.isImprovedBy(it.improvement!!)) || it.isCityCenter()
-                       || (it.getUnpillagedTileImprovement()?.isGreatImprovement() == true && it.tileResource.resourceType == ResourceType.Strategic)
+                    val tileResource = it.tileResource
+                    tileResource != null &&
+                        requiredNearbyImprovedResources!!.contains(tileResource.name) &&
+                        it.getOwner() == civ &&
+                        ((it.getUnpillagedImprovement() != null && tileResource.isImprovedBy(it.improvement!!)) ||
+                            it.isCityCenter() ||
+                            (it.getUnpillagedTileImprovement()?.isGreatImprovement() == true && tileResource.resourceType == ResourceType.Strategic)
                     )
                 }
             if (!containsResourceWithImprovement)
@@ -449,11 +435,12 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
      * Handles inverted conditional rejections and cumulative conditional reporting
      * See also [com.unciv.models.ruleset.unit.BaseUnit.notMetRejections]
      */
+    @Readonly
     private fun notMetRejections(unique: Unique, cityConstructions: CityConstructions, built: Boolean=false): Sequence<RejectionReason> = sequence {
         val civ = cityConstructions.city.civ
-        for (conditional in unique.conditionals) {
+        for (conditional in unique.modifiers) {
             // We yield a rejection only when conditionals are NOT met
-            if (Conditionals.conditionalApplies(unique, conditional, StateForConditionals(civ, cityConstructions.city)))
+            if (Conditionals.conditionalApplies(unique, conditional, cityConstructions.city.state))
                 continue
             when (conditional.type) {
                 UniqueType.ConditionalBuildingBuiltAmount -> {
@@ -493,87 +480,101 @@ class Building : RulesetStatsObject(), INonPerpetualConstruction {
     override fun isBuildable(cityConstructions: CityConstructions): Boolean =
             getRejectionReasons(cityConstructions).none()
 
-    override fun postBuildEvent(cityConstructions: CityConstructions, boughtWith: Stat?): Boolean {
+    fun construct(cityConstructions: CityConstructions) {
         val civInfo = cityConstructions.city.civ
 
         if (civInfo.gameInfo.spaceResources.contains(name)) {
             civInfo.victoryManager.currentsSpaceshipParts.add(name, 1)
-            return true
         }
 
         cityConstructions.addBuilding(this)
-        return true
     }
 
 
-    private val cachedMatchesFilterResult = HashMap<String, Boolean>()
+    @Cache private val cachedMatchesFilterResult = HashMap<String, Boolean>()
 
     /** Implements [UniqueParameterType.BuildingFilter] */
-    fun matchesFilter(filter: String): Boolean {
-        val cachedAnswer = cachedMatchesFilterResult[filter]
-        if (cachedAnswer != null) return cachedAnswer
-        val newAnswer = MultiFilter.multiFilter(filter, { matchesSingleFilter(it) })
-        cachedMatchesFilterResult[filter] = newAnswer
-        return newAnswer
-    }
+    @Readonly
+    fun matchesFilter(filter: String, state: GameContext? = null): Boolean =
+        MultiFilter.multiFilter(filter, {
+            cachedMatchesFilterResult.getOrPut(it) { matchesSingleFilter(it) } ||
+                state != null && hasTagUnique(it, state) ||
+                state == null && hasTagUnique(it)
+        })
 
+    @Readonly
     private fun matchesSingleFilter(filter: String): Boolean {
+        // all cases are constants for performance
         return when (filter) {
-            in Constants.all -> true
-            name -> true
+            "all", "All" -> true
             "Building", "Buildings" -> !isAnyWonder()
             "Wonder", "Wonders" -> isAnyWonder()
             "National Wonder", "National" -> isNationalWonder
             "World Wonder", "World" -> isWonder
-            replaces -> true
             else -> {
-                if (uniques.contains(filter)) return true
+                if (filter == name) return true
+                if (filter == replaces) return true
                 if (::ruleset.isInitialized) // False when loading ruleset and checking buildingsToRemove
                     for (requiredTech: String in requiredTechs())
-                        if (ruleset.technologies[requiredTech]?.matchesFilter(filter) == true) return true
+                        if (ruleset.technologies[requiredTech]?.matchesFilter(filter, multiFilter = false) == true) return true
                 val stat = Stat.safeValueOf(filter)
                 return (stat != null && isStatRelated(stat))
             }
         }
     }
 
-    fun isStatRelated(stat: Stat): Boolean {
-        if (get(stat) > 0) return true
-        if (getStatPercentageBonuses(null)[stat] > 0) return true
-        if (getMatchingUniques(UniqueType.Stats).any { it.stats[stat] > 0 }) return true
+    @Readonly
+    fun isStatRelated(stat: Stat, city: City? = null): Boolean {
+        if (city != null) {
+            if (getStats(city)[stat] > 0) return true
+            if (getStatPercentageBonuses(city)[stat] > 0) return true
+        }
+        else {
+            if (get(stat) > 0) return true
+            if (getMatchingUniques(UniqueType.Stats).any { it.stats[stat] > 0 }) return true
+            if (getStatPercentageBonuses(null)[stat] > 0) return true
+        }
         if (getMatchingUniques(UniqueType.StatsFromTiles).any { it.stats[stat] > 0 }) return true
         if (getMatchingUniques(UniqueType.StatsPerPopulation).any { it.stats[stat] > 0 }) return true
-        if (stat == Stat.Happiness && hasUnique(UniqueType.RemoveAnnexUnhappiness)) return true
+        if (stat == Stat.Happiness && hasUnique(UniqueType.RemovesAnnexUnhappiness)) return true
         return false
     }
 
-    private val _hasCreatesOneImprovementUnique by lazy {
-        hasUnique(UniqueType.CreatesOneImprovement)
+    // This can be 'by lazy' across all rulesets because it only checks building uniques
+    private val improvementToCreate: String? by lazy { 
+        val unique = getMatchingUniques(UniqueType.CreatesOneImprovement).firstOrNull()
+        if (unique == null) null
+        else unique.params[0]
     }
-    fun hasCreateOneImprovementUnique() = _hasCreatesOneImprovementUnique
+    
+    @Readonly fun hasCreateOneImprovementUnique() = improvementToCreate != null
 
-    private var _getImprovementToCreate: TileImprovement? = null
+    @Readonly
+    // Only the name can be cached across rulesets.
+    // The improvement itself CANNOT be cached because it's ruleset-dependent.
     private fun getImprovementToCreate(ruleset: Ruleset): TileImprovement? {
-        if (!hasCreateOneImprovementUnique()) return null
-        if (_getImprovementToCreate == null) {
-            val improvementUnique = getMatchingUniques(UniqueType.CreatesOneImprovement)
-                .firstOrNull() ?: return null
-            _getImprovementToCreate = ruleset.tileImprovements[improvementUnique.params[0]]
-        }
-        return _getImprovementToCreate
+        if (improvementToCreate == null) return null
+        return ruleset.tileImprovements[improvementToCreate]
     }
+    
+    @Readonly
     fun getImprovementToCreate(ruleset: Ruleset, civInfo: Civilization): TileImprovement? {
         val improvement = getImprovementToCreate(ruleset) ?: return null
         return civInfo.getEquivalentTileImprovement(improvement)
     }
 
-    fun isSellable() = !isAnyWonder() && !hasUnique(UniqueType.Unsellable)
+    @Readonly fun isSellable() = !isAnyWonder() && !hasUnique(UniqueType.Unsellable)
 
-    override fun getResourceRequirementsPerTurn(stateForConditionals: StateForConditionals?): Counter<String> {
+    @Readonly
+    override fun getResourceRequirementsPerTurn(state: GameContext?): Counter<String> {
+        val uniques = getMatchingUniques(UniqueType.ConsumesResources,
+            state ?: GameContext.EmptyState)
+        if (uniques.none() && requiredResource == null) return Counter.ZERO
+        
         val resourceRequirements = Counter<String>()
         if (requiredResource != null) resourceRequirements[requiredResource!!] = 1
-        for (unique in getMatchingUniques(UniqueType.ConsumesResources, stateForConditionals))
-            resourceRequirements[unique.params[1]] += unique.params[0].toInt()
+        for (unique in uniques)
+            resourceRequirements.add(unique.params[1], unique.params[0].toInt())
         return resourceRequirements
     }
 }
