@@ -5,25 +5,28 @@ import com.unciv.logic.city.CityConstructions
 import com.unciv.logic.civilization.Civilization
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.unique.IHasUniques
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.INamed
 import com.unciv.models.stats.Stat
+import com.unciv.models.stats.Stat.Companion.statsUsableToBuy
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.components.fonts.Fonts
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 import kotlin.math.pow
 import kotlin.math.roundToInt
 
 interface IConstruction : INamed {
-    fun isBuildable(cityConstructions: CityConstructions): Boolean
-    fun shouldBeDisplayed(cityConstructions: CityConstructions): Boolean
+    @Readonly fun isBuildable(cityConstructions: CityConstructions): Boolean
+    @Readonly fun shouldBeDisplayed(cityConstructions: CityConstructions): Boolean
+    
     /** Gets *per turn* resource requirements - does not include immediate costs for stockpiled resources.
-     * Uses [stateForConditionals] to determine which civ or city this is built for*/
-    fun getResourceRequirementsPerTurn(stateForConditionals: StateForConditionals? = null): Counter<String>
-    fun requiredResources(stateForConditionals: StateForConditionals? = null): Set<String>
-    /** We can't call this getMatchingUniques because then it would conflict with IHasUniques */
-    fun getMatchingUniquesNotConflicting(uniqueType: UniqueType) = sequenceOf<Unique>()
+     * Uses [state] to determine which civ or city this is built for*/
+    @Readonly fun getResourceRequirementsPerTurn(state: GameContext? = null): Counter<String>
+    @Readonly fun requiredResources(state: GameContext = GameContext.EmptyState): Set<String>
+    @Readonly fun getStockpiledResourceRequirements(state: GameContext): Counter<String>
 }
 
 interface INonPerpetualConstruction : IConstruction, INamed, IHasUniques {
@@ -33,54 +36,68 @@ interface INonPerpetualConstruction : IConstruction, INamed, IHasUniques {
     // https://yairm210.github.io/Unciv/Developers/Translations%2C-mods%2C-and-modding-freedom-in-Open-Source#filters
     var requiredTech: String?
 
-    override fun legacyRequiredTechs(): Sequence<String> = if (requiredTech == null) sequenceOf() else sequenceOf(requiredTech!!)
+    override fun legacyRequiredTechs(): Sequence<String> = if (requiredTech == null) emptySequence() else sequenceOf(requiredTech!!)
 
-    fun getProductionCost(civInfo: Civilization, city: City?): Int
-    fun getStatBuyCost(city: City, stat: Stat): Int?
-    fun getRejectionReasons(cityConstructions: CityConstructions): Sequence<RejectionReason>
-
-    /** Returns whether was successful - can fail for units if we can't place them */
-    fun postBuildEvent(cityConstructions: CityConstructions, boughtWith: Stat? = null): Boolean  // Yes I'm hilarious.
+    @Readonly fun getProductionCost(civInfo: Civilization, city: City?): Int
+    @Readonly fun getStatBuyCost(city: City, stat: Stat): Int?
+    @Readonly fun getRejectionReasons(cityConstructions: CityConstructions): Sequence<RejectionReason>
 
     /** Only checks if it has the unique to be bought with this stat, not whether it is purchasable at all */
-    fun canBePurchasedWithStat(city: City?, stat: Stat): Boolean {
-        val stateForConditionals = StateForConditionals(city?.civ, city)
-        if (stat == Stat.Production || stat == Stat.Happiness) return false
-        if (hasUnique(UniqueType.CannotBePurchased, stateForConditionals)) return false
+    @Readonly fun canBePurchasedWithStat(city: City?, stat: Stat): Boolean {
+        return canBePurchasedWithStatReasons(city, stat).purchasable
+    }
+
+    /** Only checks if it has the unique to be bought with this stat, not whether it is purchasable at all */
+    @Readonly
+    fun canBePurchasedWithStatReasons(city: City?, stat: Stat): PurchaseReason {
+        val gameContext = city?.state ?: GameContext.EmptyState
+        if (stat == Stat.Production || stat == Stat.Happiness) return PurchaseReason.Invalid
+        if (hasUnique(UniqueType.CannotBePurchased, gameContext)) return PurchaseReason.Unpurchasable
         // Can be purchased with [Stat] [cityFilter]
-        if (city != null && getMatchingUniques(UniqueType.CanBePurchasedWithStat, stateForConditionals)
-            .any { it.params[0] == stat.name && city.matchesFilter(it.params[1]) }
-        ) return true
+        if (getMatchingUniques(UniqueType.CanBePurchasedWithStat, GameContext.IgnoreConditionals)
+            .any {
+                it.params[0] == stat.name &&
+                    (city == null || (it.conditionalsApply(gameContext) && city.matchesFilter(it.params[1])))
+            }
+        ) return PurchaseReason.UniqueAllowed
         // Can be purchased for [amount] [Stat] [cityFilter]
-        if (city != null && getMatchingUniques(UniqueType.CanBePurchasedForAmountStat, stateForConditionals)
-            .any { it.params[1] == stat.name && city.matchesFilter(it.params[2]) }
-        ) return true
-        if (stat == Stat.Gold) return !hasUnique(UniqueType.Unbuildable, stateForConditionals)
-        return false
+        if (getMatchingUniques(UniqueType.CanBePurchasedForAmountStat, GameContext.IgnoreConditionals)
+            .any {
+                it.params[1] == stat.name &&
+                    (city == null || (it.conditionalsApply(gameContext) && city.matchesFilter(it.params[2])))
+            }
+        ) return PurchaseReason.UniqueAllowed
+        if (stat == Stat.Gold && !hasUnique(UniqueType.Unbuildable, gameContext)) return PurchaseReason.Allowed
+        return PurchaseReason.NotAllowed
     }
 
     /** Checks if the construction should be purchasable, not whether it can be bought with a stat at all */
+    @Readonly
     fun isPurchasable(cityConstructions: CityConstructions): Boolean {
         val rejectionReasons = getRejectionReasons(cityConstructions)
         return rejectionReasons.all { it.type == RejectionReasonType.Unbuildable }
     }
 
+    @Readonly
     fun canBePurchasedWithAnyStat(city: City): Boolean {
-        return Stat.values().any { canBePurchasedWithStat(city, it) }
+        return statsUsableToBuy.any { canBePurchasedWithStat(city, it) }
     }
 
+    @Readonly
     fun getCivilopediaGoldCost(): Int {
         // Same as getBaseGoldCost, but without game-specific modifiers
         return ((30.0 * cost.toFloat()).pow(0.75) * hurryCostModifier.toPercent() / 10).toInt() * 10
     }
 
+    @Readonly
     fun getBaseGoldCost(civInfo: Civilization, city: City?): Double {
         // https://forums.civfanatics.com/threads/rush-buying-formula.393892/
         return (30.0 * getProductionCost(civInfo, city)).pow(0.75) * hurryCostModifier.toPercent()
     }
 
+    @Readonly
     fun getBaseBuyCost(city: City, stat: Stat): Float? {
-        val conditionalState = StateForConditionals(civInfo = city.civ, city = city)
+        val conditionalState = city.state
 
         // Can be purchased for [amount] [Stat] [cityFilter]
         val lowestCostUnique = getMatchingUniques(UniqueType.CanBePurchasedForAmountStat, conditionalState)
@@ -97,32 +114,58 @@ interface INonPerpetualConstruction : IConstruction, INamed, IHasUniques {
         return null
     }
 
+    @Readonly
     fun getCostForConstructionsIncreasingInPrice(baseCost: Int, increaseCost: Int, previouslyBought: Int): Int {
         return (baseCost + increaseCost / 2f * ( previouslyBought * previouslyBought + previouslyBought )).toInt()
     }
 
-    override fun getMatchingUniquesNotConflicting(uniqueType: UniqueType): Sequence<Unique> =
-            getMatchingUniques(uniqueType)
+    /** We can't call this getMatchingUniques because then it would conflict with IHasUniques */
+    @Readonly
+    fun getMatchingUniquesNotConflicting(uniqueType: UniqueType, gameContext: GameContext): Sequence<Unique> =
+            getMatchingUniques(uniqueType, gameContext)
 
-    override fun requiredResources(stateForConditionals: StateForConditionals?): Set<String> {
-        return getResourceRequirementsPerTurn(stateForConditionals).keys +
-                getMatchingUniques(UniqueType.CostsResources, stateForConditionals).map { it.params[1] }
+    @Readonly
+    override fun requiredResources(state: GameContext): Set<String> {
+        return getResourceRequirementsPerTurn(state).keys +
+                getMatchingUniques(UniqueType.CostsResources, state).map { it.params[1] }
+    }
+    
+    @Readonly
+    override fun getStockpiledResourceRequirements(state: GameContext): Counter<String> {
+        val counter = Counter<String>()
+        for (unique in getMatchingUniquesNotConflicting(UniqueType.CostsResources, state)){
+            var amount = unique.params[0].toInt()
+            if (unique.isModifiedByGameSpeed()) amount = (amount * state.gameInfo!!.speed.modifier).toInt()
+            counter.add(unique.params[1], amount)
+        }
+        return counter
     }
 }
 
+enum class PurchaseReason(val purchasable: Boolean) {
+    Allowed(true),
+    Invalid(false),
+    Unpurchasable(false),
+    UniqueAllowed(true),
+    Other(false),
+    OtherAllowed(true),
+    NotAllowed(false)
+}
 
 
 class RejectionReason(val type: RejectionReasonType,
                       val errorMessage: String = type.errorMessage,
                       val shouldShow: Boolean = type.shouldShow) {
 
-    fun techPolicyEraWonderRequirements(): Boolean = type in techPolicyEraWonderRequirements
+    @Readonly fun techPolicyEraWonderRequirements(): Boolean = type in techPolicyEraWonderRequirements
 
-    fun hasAReasonToBeRemovedFromQueue(): Boolean = type in reasonsToDefinitivelyRemoveFromQueue
+    @Readonly fun hasAReasonToBeRemovedFromQueue(): Boolean = type in reasonsToDefinitivelyRemoveFromQueue
 
-    fun isImportantRejection(): Boolean = type in orderedImportantRejectionTypes
+    @Readonly fun isImportantRejection(): Boolean = type in orderedImportantRejectionTypes
 
-    fun isConstructionRejection(): Boolean = type in constructionRejectionReasonType
+    @Readonly fun isConstructionRejection(): Boolean = type in constructionRejectionReasonType
+
+    @Readonly fun isNeverVisible(): Boolean = type in neverVisible
 
     /** Returns the index of [orderedImportantRejectionTypes] with the smallest index having the
      * highest precedence */
@@ -130,43 +173,56 @@ class RejectionReason(val type: RejectionReasonType,
         return orderedImportantRejectionTypes.indexOf(type)
     }
 
-    // Used for constant variables in the functions above
-    private val techPolicyEraWonderRequirements = hashSetOf(
-        RejectionReasonType.Obsoleted,
-        RejectionReasonType.RequiresTech,
-        RejectionReasonType.RequiresPolicy,
-        RejectionReasonType.MorePolicyBranches,
-        RejectionReasonType.RequiresBuildingInSomeCity,
-    )
-    private val reasonsToDefinitivelyRemoveFromQueue = hashSetOf(
-        RejectionReasonType.Obsoleted,
-        RejectionReasonType.WonderAlreadyBuilt,
-        RejectionReasonType.NationalWonderAlreadyBuilt,
-        RejectionReasonType.CannotBeBuiltWith,
-        RejectionReasonType.MaxNumberBuildable,
-    )
-    private val orderedImportantRejectionTypes = listOf(
-        RejectionReasonType.ShouldNotBeDisplayed,
-        RejectionReasonType.WonderBeingBuiltElsewhere,
-        RejectionReasonType.RequiresBuildingInAllCities,
-        RejectionReasonType.RequiresBuildingInThisCity,
-        RejectionReasonType.RequiresBuildingInSomeCity,
-        RejectionReasonType.RequiresBuildingInSomeCities,
-        RejectionReasonType.CanOnlyBeBuiltInSpecificCities,
-        RejectionReasonType.CannotBeBuiltUnhappiness,
-        RejectionReasonType.PopulationRequirement,
-        RejectionReasonType.ConsumesResources,
-        RejectionReasonType.CanOnlyBePurchased,
-        RejectionReasonType.MaxNumberBuildable,
-        RejectionReasonType.NoPlaceToPutUnit,
-    )
-    // Exceptions. Used for units spawned/upgrade path, not built
-    private val constructionRejectionReasonType = listOf(
-        RejectionReasonType.Unbuildable,
-        RejectionReasonType.CannotBeBuiltUnhappiness,
-        RejectionReasonType.CannotBeBuilt,
-        RejectionReasonType.CanOnlyBeBuiltInSpecificCities,
-    )
+    companion object {
+        // Used for constant variables in the functions above
+        private val techPolicyEraWonderRequirements = hashSetOf(
+            RejectionReasonType.Obsoleted,
+            RejectionReasonType.RequiresTech,
+            RejectionReasonType.RequiresPolicy,
+            RejectionReasonType.MorePolicyBranches,
+            RejectionReasonType.RequiresBuildingInSomeCity,
+        )
+        private val reasonsToDefinitivelyRemoveFromQueue = hashSetOf(
+            RejectionReasonType.Obsoleted,
+            RejectionReasonType.WonderAlreadyBuilt,
+            RejectionReasonType.NationalWonderAlreadyBuilt,
+            RejectionReasonType.CannotBeBuiltWith,
+            RejectionReasonType.MaxNumberBuildable,
+        )
+        private val orderedImportantRejectionTypes = listOf(
+            RejectionReasonType.ShouldNotBeDisplayed,
+            RejectionReasonType.WonderBeingBuiltElsewhere,
+            RejectionReasonType.RequiresBuildingInAllCities,
+            RejectionReasonType.RequiresBuildingInThisCity,
+            RejectionReasonType.RequiresBuildingInSomeCity,
+            RejectionReasonType.RequiresBuildingInSomeCities,
+            RejectionReasonType.CanOnlyBeBuiltInSpecificCities,
+            RejectionReasonType.PopulationRequirement,
+            RejectionReasonType.ConsumesResources,
+            RejectionReasonType.CanOnlyBePurchased,
+            RejectionReasonType.MaxNumberBuildable,
+            RejectionReasonType.NoPlaceToPutUnit,
+        )
+        // Exceptions. Used for units spawned/upgrade path, not built
+        private val constructionRejectionReasonType = listOf(
+            RejectionReasonType.Unbuildable,
+            RejectionReasonType.CannotBeBuilt,
+            RejectionReasonType.CanOnlyBeBuiltInSpecificCities,
+        )
+        private val neverVisible = listOf(
+            RejectionReasonType.AlreadyBuilt,
+            RejectionReasonType.WonderAlreadyBuilt,
+            RejectionReasonType.NationalWonderAlreadyBuilt,
+            RejectionReasonType.DisabledBySetting,
+            RejectionReasonType.UniqueToOtherNation,
+            RejectionReasonType.ReplacedByOurUnique,
+            RejectionReasonType.Obsoleted,
+            RejectionReasonType.WonderBeingBuiltElsewhere,
+            RejectionReasonType.RequiresTech,
+            RejectionReasonType.NoSettlerForOneCityPlayers,
+            RejectionReasonType.WaterUnitsInCoastalCities,
+        )
+    }
 }
 
 
@@ -191,12 +247,10 @@ enum class RejectionReasonType(val shouldShow: Boolean, val errorMessage: String
     UniqueToOtherNation(false, "Unique to another nation"),
     ReplacedByOurUnique(false, "Our unique replaces this"),
     CannotBeBuilt(false, "Cannot be built by this nation"),
-    CannotBeBuiltUnhappiness(true, "Unhappiness"),
 
     Obsoleted(false, "Obsolete"),
     RequiresTech(false, "Required tech not researched"),
     RequiresPolicy(false, "Requires a specific policy!"),
-    UnlockedWithEra(false, "Unlocked when reaching a specific era"),
     MorePolicyBranches(false, "Hidden until more policy branches are fully adopted"),
 
     RequiresNearbyResource(false, "Requires a certain resource being exploited nearby"),
@@ -206,6 +260,9 @@ enum class RejectionReasonType(val shouldShow: Boolean, val errorMessage: String
     RequiresBuildingInAllCities(true, "Requires a specific building in all cities!"),
     RequiresBuildingInSomeCities(true, "Requires a specific building in more cities!"),
     RequiresBuildingInSomeCity(true, "Requires a specific building anywhere in your empire!"),
+    
+    NoSuchImprovement(false, "No such improvement exists in the ruleset"),
+    NoTileCanContainImprovement(false, "No tile can contain this improvement"),
 
     WonderAlreadyBuilt(false, "Wonder already built"),
     NationalWonderAlreadyBuilt(false, "National Wonder already built"),
@@ -221,6 +278,10 @@ enum class RejectionReasonType(val shouldShow: Boolean, val errorMessage: String
     NoSettlerForOneCityPlayers(false, "No settlers for city-states or one-city challengers"),
     NoPlaceToPutUnit(true, "No space to place this unit");
 
+    val defaultInstance by lazy { RejectionReason(this, errorMessage, shouldShow) }
+    @Pure fun toInstance() = defaultInstance
+
+    @Pure
     fun toInstance(errorMessage: String = this.errorMessage,
         shouldShow: Boolean = this.shouldShow): RejectionReason {
         return RejectionReason(this, errorMessage, shouldShow)
@@ -231,7 +292,8 @@ open class PerpetualConstruction(override var name: String, val description: Str
     IConstruction {
 
     override fun shouldBeDisplayed(cityConstructions: CityConstructions) = isBuildable(cityConstructions)
-    open fun getProductionTooltip(city: City, withIcon: Boolean = false) : String = ""
+    @Readonly open fun getProductionTooltip(city: City, withIcon: Boolean = false) : String = ""
+    override fun getStockpiledResourceRequirements(state: GameContext) = Counter.ZERO
 
     companion object {
         val science = PerpetualStatConversion(Stat.Science)
@@ -252,9 +314,9 @@ open class PerpetualConstruction(override var name: String, val description: Str
     override fun isBuildable(cityConstructions: CityConstructions): Boolean =
             throw Exception("Impossible!")
 
-    override fun getResourceRequirementsPerTurn(stateForConditionals: StateForConditionals?) = Counter.ZERO
+    override fun getResourceRequirementsPerTurn(state: GameContext?) = Counter.ZERO
 
-    override fun requiredResources(stateForConditionals: StateForConditionals?): Set<String> = emptySet()
+    override fun requiredResources(state: GameContext): Set<String> = emptySet()
 }
 
 open class PerpetualStatConversion(val stat: Stat) :
@@ -262,14 +324,14 @@ open class PerpetualStatConversion(val stat: Stat) :
 
     override fun getProductionTooltip(city: City, withIcon: Boolean) : String
             = "\r\n${(city.cityStats.currentCityStats.production / getConversionRate(city)).roundToInt()}${if (withIcon) stat.character else ""}/${Fonts.turn}"
-    fun getConversionRate(city: City) : Int = (1/city.cityStats.getStatConversionRate(stat)).roundToInt()
+    @Readonly fun getConversionRate(city: City) : Int = (1/city.cityStats.getStatConversionRate(stat)).roundToInt()
 
     override fun isBuildable(cityConstructions: CityConstructions): Boolean {
         val city = cityConstructions.city
         if (stat == Stat.Faith && !city.civ.gameInfo.isReligionEnabled())
             return false
 
-        val stateForConditionals = StateForConditionals(city.civ, city, tile = city.getCenterTile())
+        val stateForConditionals = city.state
         return city.civ.getMatchingUniques(UniqueType.EnablesCivWideStatProduction, stateForConditionals)
             .any { it.params[0] == stat.name }
     }
