@@ -6,33 +6,33 @@ import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueTarget
 
-object ModOptionsConstants {
-    const val diplomaticRelationshipsCannotChange = "Diplomatic relationships cannot change"
-    const val convertGoldToScience = "Can convert gold to science with sliders"
-    const val allowCityStatesSpawnUnits = "Allow City States to spawn with additional units"
-    const val tradeCivIntroductions = "Can trade civilization introductions for [] Gold"
-    const val disableReligion = "Disable religion"
-    const val allowRazeCapital = "Allow raze capital"
-    const val allowRazeHolyCity = "Allow raze holy city"
-}
-
 class ModOptions : IHasUniques {
-    override var name = "ModOptions"
-
+    //region Modder choices
     var isBaseRuleset = false
     var techsToRemove = HashSet<String>()
     var buildingsToRemove = HashSet<String>()
     var unitsToRemove = HashSet<String>()
     var nationsToRemove = HashSet<String>()
+    var policyBranchesToRemove = HashSet<String>()
+    var policiesToRemove = HashSet<String>()
+    var beliefsToRemove = HashSet<String>()
+    var religionsToRemove = HashSet<String>()
+    val constants = ModConstants()
+    var unitset: String? = null
+    var tileset: String? = null
+    //endregion
 
-
-    var lastUpdated = ""
+    //region Metadata, automatic
     var modUrl = ""
     var defaultBranch = "master"
     var author = ""
+    var lastUpdated = ""
     var modSize = 0
     var topics = mutableListOf<String>()
+    //endregion
 
+    //region IHasUniques
+    override var name = "ModOptions"
     override var uniques = ArrayList<String>()
 
     @delegate:Transient
@@ -41,6 +41,5 @@ class ModOptions : IHasUniques {
     override val uniqueMap: UniqueMap by lazy(::uniqueMapProvider)
 
     override fun getUniqueTarget() = UniqueTarget.ModOptions
-
-    val constants = ModConstants()
+    //endregion
 }
