@@ -1,10 +1,12 @@
 package com.unciv.models.ruleset
 
+import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
 import com.unciv.Constants
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
 import com.unciv.logic.BackwardCompatibility.updateDeprecations
+import com.unciv.logic.GameInfo
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.nation.CityStateType
@@ -17,21 +19,60 @@ import com.unciv.models.ruleset.tech.Technology
 import com.unciv.models.ruleset.tile.Terrain
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.tile.TileResource
-import com.unciv.models.ruleset.unique.IHasUniques
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.ruleset.unit.Promotion
+import com.unciv.models.ruleset.unit.UnitNameGroup
 import com.unciv.models.ruleset.unit.UnitType
 import com.unciv.models.ruleset.validation.RulesetValidator
 import com.unciv.models.ruleset.validation.UniqueValidator
+import com.unciv.models.stats.GameResource
 import com.unciv.models.stats.INamed
+import com.unciv.models.stats.Stat
+import com.unciv.models.stats.SubStat
 import com.unciv.models.translations.tr
 import com.unciv.ui.screens.civilopediascreen.ICivilopediaText
 import com.unciv.utils.Log
+import org.jetbrains.annotations.VisibleForTesting
+import yairm210.purity.annotations.Readonly
 import kotlin.collections.set
 
+enum class RulesetFile(
+    val filename: String,
+    @Readonly val getRulesetObjects: Ruleset.() -> Sequence<IRulesetObject> = { emptySequence() },
+    @Readonly val getUniques: Ruleset.() -> Sequence<Unique> = { getRulesetObjects().flatMap { it.uniqueObjects } }
+) {
+    Beliefs("Beliefs.json", { beliefs.values.asSequence() }),
+    Buildings("Buildings.json", { buildings.values.asSequence() }),
+    Eras("Eras.json", { eras.values.asSequence() }),
+    Religions("Religions.json"),
+    Nations("Nations.json", { nations.values.asSequence() }),
+    Policies("Policies.json", { policies.values.asSequence() }),
+    Techs("Techs.json", { technologies.values.asSequence() }),
+    Terrains("Terrains.json", { terrains.values.asSequence() }),
+    Tutorials("Tutorials.json", { tutorials.values.asSequence() }),
+    TileImprovements("TileImprovements.json", { tileImprovements.values.asSequence() }),
+    TileResources("TileResources.json", { tileResources.values.asSequence() }),
+    Specialists("Specialists.json"),
+    Units("Units.json", { units.values.asSequence() }),
+    UnitPromotions("UnitPromotions.json", { unitPromotions.values.asSequence() }),
+    UnitNameGroup("UnitNameGroups.json", { unitNameGroups.values.asSequence() }),
+    UnitTypes("UnitTypes.json", { unitTypes.values.asSequence() }),
+    VictoryTypes("VictoryTypes.json"),
+    CityStateTypes("CityStateTypes.json", getUniques =
+        { cityStateTypes.values.asSequence().flatMap { it.allyBonusUniqueMap.getAllUniques() + it.friendBonusUniqueMap.getAllUniques() } }),
+    Personalities("Personalities.json", { personalities.values.asSequence() }),
+    Events("Events.json", { events.values.asSequence() + events.values.flatMap { it.choices } }),
+    GlobalUniques("GlobalUniques.json", { sequenceOf(globalUniques) }),
+    ModOptions("ModOptions.json", getUniques = { modOptions.uniqueObjects.asSequence() }),
+    Speeds("Speeds.json", { speeds.values.asSequence() }),
+    Difficulties("Difficulties.json"),
+    Quests("Quests.json"),
+    Ruins("Ruins.json", { ruinRewards.values.asSequence() });
+}
+
 class Ruleset {
 
     /** If (and only if) this Ruleset is a mod, this will be the source folder.
@@ -59,7 +100,9 @@ class Ruleset {
     val difficulties = LinkedHashMap<String, Difficulty>()
     val eras = LinkedHashMap<String, Era>()
     val speeds = LinkedHashMap<String, Speed>()
-    var globalUniques = GlobalUniques()
+    /** Only [Ruleset.load], [GameInfo], [BaseUnit] and [RulesetValidator] should access this directly.
+     *  All other uses should call [GameInfo.getGlobalUniques] instead. */
+    internal var globalUniques = GlobalUniques()
     val nations = LinkedHashMap<String, Nation>()
     val policies = LinkedHashMap<String, Policy>()
     val policyBranches = LinkedHashMap<String, PolicyBranch>()
@@ -72,8 +115,10 @@ class Ruleset {
     val terrains = LinkedHashMap<String, Terrain>()
     val tileImprovements = LinkedHashMap<String, TileImprovement>()
     val tileResources = LinkedHashMap<String, TileResource>()
+    val tutorials = LinkedHashMap<String, Tutorial>()
     val units = LinkedHashMap<String, BaseUnit>()
     val unitPromotions = LinkedHashMap<String, Promotion>()
+    val unitNameGroups = LinkedHashMap<String, UnitNameGroup>()
     val unitTypes = LinkedHashMap<String, UnitType>()
     var victories = LinkedHashMap<String, Victory>()
     var cityStateTypes = LinkedHashMap<String, CityStateType>()
@@ -84,49 +129,72 @@ class Ruleset {
 
     //region cache fields
     val greatGeneralUnits by lazy {
-        units.values.filter { it.hasUnique(UniqueType.GreatPersonFromCombat, StateForConditionals.IgnoreConditionals) }
+        units.values.filter { it.hasUnique(UniqueType.GreatPersonFromCombat, GameContext.IgnoreConditionals) }
     }
 
     val tileRemovals by lazy { tileImprovements.values.filter { it.name.startsWith(Constants.remove) } }
+    val nonRoadTileRemovals by lazy { tileRemovals.filter { rulesetImprovement ->
+            RoadStatus.entries.toTypedArray().none { it.removeAction == rulesetImprovement.name } } }
 
     /** Contains all happiness levels that moving *from* them, to one *below* them, can change uniques that apply */
     val allHappinessLevelsThatAffectUniques by lazy {
         sequence {
-            for (rulesetObject in (units.values + terrains.values + buildings.values + technologies.values + eras.values
-                + beliefs.values + unitPromotions.values + tileResources.values + policies.values + tileImprovements.values + globalUniques))
-                for (unique in rulesetObject.uniqueObjects)
-                    for (conditional in unique.conditionals){
-                        if (conditional.type == UniqueType.ConditionalBelowHappiness) yield(conditional.params[0].toInt())
-                        if (conditional.type == UniqueType.ConditionalBetweenHappiness){
-                            yield(conditional.params[0].toInt())
-                            yield(conditional.params[1].toInt() + 1)
-                        }
-                        if (conditional.type == UniqueType.ConditionalHappy) yield(0)
+            for (unique in this@Ruleset.allUniques())
+                for (conditional in unique.modifiers){
+                    if (conditional.type == UniqueType.ConditionalWhenBelowAmountStatResource
+                        && conditional.params[1] == "Happiness") yield(conditional.params[0].toInt())
+                    if (conditional.type == UniqueType.ConditionalWhenAboveAmountStatResource
+                        && conditional.params[1] == "Happiness") yield(conditional.params[0].toInt())
+                    if (conditional.type == UniqueType.ConditionalWhenBetweenStatResource
+                        && conditional.params[2] == "Happiness"){
+                        yield(conditional.params[0].toInt())
+                        yield(conditional.params[1].toInt() + 1)
                     }
+                    if (conditional.type == UniqueType.ConditionalHappy) yield(0)
+                }
         }.toSet()
     }
 
-    val roadImprovement by lazy { RoadStatus.Road.improvement(this) }
-    val railroadImprovement by lazy { RoadStatus.Railroad.improvement(this) }
+    val roadImprovement: TileImprovement? by lazy { RoadStatus.Road.improvement(this) }
+    val railroadImprovement: TileImprovement? by lazy { RoadStatus.Railroad.improvement(this) }
     //endregion
 
     fun clone(): Ruleset {
         val newRuleset = Ruleset()
         newRuleset.add(this)
+        // Make sure the clone is recognizable - e.g. startNewGame fallback when a base mod was removed needs this
+        newRuleset.name = name
+        newRuleset.modOptions.isBaseRuleset = modOptions.isBaseRuleset
         return newRuleset
     }
 
-    private fun <T : INamed> createHashmap(items: Array<T>): LinkedHashMap<String, T> {
+    fun getGameResource(resourceName: String): GameResource? = Stat.safeValueOf(resourceName)
+        ?: SubStat.safeValueOf(resourceName)
+        ?: tileResources[resourceName]
+
+    private inline fun <reified T : INamed> createHashmap(items: Array<T>): LinkedHashMap<String, T> {
         val hashMap = LinkedHashMap<String, T>(items.size)
         for (item in items) {
-            hashMap[item.name] = item
-            (item as? IRulesetObject)?.originRuleset = name
+            val itemName = try { item.name }
+            catch (_: Exception) {
+                throw Exception("${T::class.simpleName} is missing a name!")
+            }
+
+            hashMap[itemName] = item
+            (item as? IRulesetObject)?.originRuleset = name // RULESET name
         }
         return hashMap
     }
 
     fun add(ruleset: Ruleset) {
         beliefs.putAll(ruleset.beliefs)
+        ruleset.modOptions.beliefsToRemove
+            .flatMap { beliefsToRemove ->
+                beliefs.filter { it.value.matchesFilter(beliefsToRemove) }.keys
+            }.toSet().forEach {
+                beliefs.remove(it)
+            }
+
         ruleset.modOptions.buildingsToRemove
             .flatMap { buildingToRemove ->
                 buildings.filter { it.value.matchesFilter(buildingToRemove) }.keys
@@ -137,10 +205,7 @@ class Ruleset {
         difficulties.putAll(ruleset.difficulties)
         eras.putAll(ruleset.eras)
         speeds.putAll(ruleset.speeds)
-        globalUniques = GlobalUniques().apply {
-            uniques.addAll(globalUniques.uniques)
-            uniques.addAll(ruleset.globalUniques.uniques)
-        }
+        globalUniques = GlobalUniques.combine(globalUniques, ruleset.globalUniques)
         ruleset.modOptions.nationsToRemove
             .flatMap { nationToRemove ->
                 nations.filter { it.value.matchesFilter(nationToRemove) }.keys
@@ -148,10 +213,38 @@ class Ruleset {
                 nations.remove(it)
             }
         nations.putAll(ruleset.nations)
+        ruleset.modOptions.policyBranchesToRemove
+            .flatMap { policyBranchToRemove ->
+                policyBranches.filter { it.value.matchesFilter(policyBranchToRemove) }.keys
+            }.toSet().forEach {
+                policyBranches.remove(it)
+            }
         policyBranches.putAll(ruleset.policyBranches)
         policies.putAll(ruleset.policies)
+
+        // Remove the policies
+        ruleset.modOptions.policiesToRemove
+            .flatMap { policyToRemove ->
+                policies.filter { it.value.matchesFilter(policyToRemove) }.keys
+            }.toSet().forEach {
+                policies.remove(it)
+            }
+
+        // Remove the policies if they exist in the policy branches too
+        for (policyToRemove in ruleset.modOptions.policiesToRemove) {
+            for (branch in policyBranches.values) {
+                branch.policies.removeAll { it.matchesFilter(policyToRemove) }
+            }
+        }
+
         quests.putAll(ruleset.quests)
+
+        // Remove associated Religions, including when they're favored by Nations
         religions.addAll(ruleset.religions)
+        religions.removeAll(ruleset.modOptions.religionsToRemove)
+        nations.filter { it.value.favoredReligion in ruleset.modOptions.religionsToRemove }
+            .forEach { it.value.favoredReligion = null }
+
         ruinRewards.putAll(ruleset.ruinRewards)
         specialists.putAll(ruleset.specialists)
         ruleset.modOptions.techsToRemove
@@ -165,12 +258,13 @@ class Ruleset {
         terrains.putAll(ruleset.terrains)
         tileImprovements.putAll(ruleset.tileImprovements)
         tileResources.putAll(ruleset.tileResources)
+        tutorials.putAll(ruleset.tutorials)
         unitTypes.putAll(ruleset.unitTypes)
         victories.putAll(ruleset.victories)
         cityStateTypes.putAll(ruleset.cityStateTypes)
         ruleset.modOptions.unitsToRemove
             .flatMap { unitToRemove ->
-                units.filter { it.apply { value.ruleset = this@Ruleset }.value.matchesFilter(unitToRemove) }.keys
+                units.filter { it.apply { value.setRuleset(this@Ruleset) }.value.matchesFilter(unitToRemove) }.keys
             }.toSet().forEach {
                 units.remove(it)
             }
@@ -181,6 +275,7 @@ class Ruleset {
         modOptions.constants.merge(ruleset.modOptions.constants)
 
         unitPromotions.putAll(ruleset.unitPromotions)
+        unitNameGroups.putAll(ruleset.unitNameGroups)
 
         mods += ruleset.mods
     }
@@ -205,7 +300,9 @@ class Ruleset {
         terrains.clear()
         tileImprovements.clear()
         tileResources.clear()
+        tutorials.clear()
         unitPromotions.clear()
+        unitNameGroups.clear()
         units.clear()
         unitTypes.clear()
         victories.clear()
@@ -214,38 +311,18 @@ class Ruleset {
         events.clear()
     }
 
-    fun allRulesetObjects(): Sequence<IRulesetObject> =
-            beliefs.values.asSequence() +
-            buildings.values.asSequence() +
-            //difficulties is only INamed
-            eras.values.asSequence() +
-            speeds.values.asSequence() +
-            sequenceOf(globalUniques) +
-            nations.values.asSequence() +
-            policies.values.asSequence() +
-            policyBranches.values.asSequence() +
-            // quests is only INamed
-            // religions is just Strings
-            ruinRewards.values.asSequence() +
-            // specialists is only NamedStats
-            technologies.values.asSequence() +
-            terrains.values.asSequence() +
-            tileImprovements.values.asSequence() +
-            tileResources.values.asSequence() +
-            unitPromotions.values.asSequence() +
-            units.values.asSequence() +
-            unitTypes.values.asSequence() +
-            personalities.values.asSequence()
-            // Victories is only INamed
-    fun allIHasUniques(): Sequence<IHasUniques> =
-            allRulesetObjects() + sequenceOf(modOptions)
-    fun allICivilopediaText(): Sequence<ICivilopediaText> =
-            allRulesetObjects() + events.values + events.values.flatMap { it.choices }
+    @Readonly
+    fun allRulesetObjects(): Sequence<IRulesetObject> = RulesetFile.entries.asSequence().flatMap { it.getRulesetObjects(this) }
+    @Readonly
+    fun allUniques(): Sequence<Unique> = RulesetFile.entries.asSequence().flatMap { it.getUniques(this) }
+    @Readonly fun allICivilopediaText(): Sequence<ICivilopediaText> = allRulesetObjects() + events.values.flatMap { it.choices }
 
     fun load(folderHandle: FileHandle) {
+        fun RulesetFile.file() = folderHandle.child(filename)
+
         // Note: Most files are loaded using createHashmap, which sets originRuleset automatically.
         // For other files containing IRulesetObject's we'll have to remember to do so manually - e.g. Tech.
-        val modOptionsFile = folderHandle.child("ModOptions.json")
+        val modOptionsFile = RulesetFile.ModOptions.file()
         if (modOptionsFile.exists()) {
             try {
                 modOptions = json().fromJsonFile(ModOptions::class.java, modOptionsFile)
@@ -255,7 +332,7 @@ class Ruleset {
             }
         }
 
-        val techFile = folderHandle.child("Techs.json")
+        val techFile = RulesetFile.Techs.file()
         if (techFile.exists()) {
             val techColumns = json().fromJsonFile(Array<TechColumn>::class.java, techFile)
             for (techColumn in techColumns) {
@@ -269,10 +346,10 @@ class Ruleset {
             }
         }
 
-        val buildingsFile = folderHandle.child("Buildings.json")
+        val buildingsFile = RulesetFile.Buildings.file()
         if (buildingsFile.exists()) buildings += createHashmap(json().fromJsonFile(Array<Building>::class.java, buildingsFile))
 
-        val terrainsFile = folderHandle.child("Terrains.json")
+        val terrainsFile = RulesetFile.Terrains.file()
         if (terrainsFile.exists()) {
             terrains += createHashmap(json().fromJsonFile(Array<Terrain>::class.java, terrainsFile))
             for (terrain in terrains.values) {
@@ -281,40 +358,43 @@ class Ruleset {
             }
         }
 
-        val resourcesFile = folderHandle.child("TileResources.json")
+        val resourcesFile = RulesetFile.TileResources.file()
         if (resourcesFile.exists()) tileResources += createHashmap(json().fromJsonFile(Array<TileResource>::class.java, resourcesFile))
 
-        val improvementsFile = folderHandle.child("TileImprovements.json")
+        val improvementsFile = RulesetFile.TileImprovements.file()
         if (improvementsFile.exists()) tileImprovements += createHashmap(json().fromJsonFile(Array<TileImprovement>::class.java, improvementsFile))
 
-        val erasFile = folderHandle.child("Eras.json")
+        val erasFile = RulesetFile.Eras.file()
         if (erasFile.exists()) eras += createHashmap(json().fromJsonFile(Array<Era>::class.java, erasFile))
         // While `eras.values.toList()` might seem more logical, eras.values is a MutableCollection and
         // therefore does not guarantee keeping the order of elements like a LinkedHashMap does.
         // Using map{} sidesteps this problem
         eras.map { it.value }.withIndex().forEach { it.value.eraNumber = it.index }
 
-        val speedsFile = folderHandle.child("Speeds.json")
+        val speedsFile = RulesetFile.Speeds.file()
         if (speedsFile.exists()) {
             speeds += createHashmap(json().fromJsonFile(Array<Speed>::class.java, speedsFile))
         }
 
-        val unitTypesFile = folderHandle.child("UnitTypes.json")
+        val unitTypesFile = RulesetFile.UnitTypes.file()
         if (unitTypesFile.exists()) unitTypes += createHashmap(json().fromJsonFile(Array<UnitType>::class.java, unitTypesFile))
 
-        val unitsFile = folderHandle.child("Units.json")
+        val unitsFile = RulesetFile.Units.file()
         if (unitsFile.exists()) units += createHashmap(json().fromJsonFile(Array<BaseUnit>::class.java, unitsFile))
 
-        val promotionsFile = folderHandle.child("UnitPromotions.json")
+        val promotionsFile = RulesetFile.UnitPromotions.file()
         if (promotionsFile.exists()) unitPromotions += createHashmap(json().fromJsonFile(Array<Promotion>::class.java, promotionsFile))
 
-        val questsFile = folderHandle.child("Quests.json")
+        val unitNameGroupsFile = RulesetFile.UnitNameGroup.file()
+        if (unitNameGroupsFile.exists()) unitNameGroups += createHashmap(json().fromJsonFile(Array<UnitNameGroup>::class.java, unitNameGroupsFile))
+
+        val questsFile = RulesetFile.Quests.file()
         if (questsFile.exists()) quests += createHashmap(json().fromJsonFile(Array<Quest>::class.java, questsFile))
 
-        val specialistsFile = folderHandle.child("Specialists.json")
+        val specialistsFile = RulesetFile.Specialists.file()
         if (specialistsFile.exists()) specialists += createHashmap(json().fromJsonFile(Array<Specialist>::class.java, specialistsFile))
 
-        val policiesFile = folderHandle.child("Policies.json")
+        val policiesFile = RulesetFile.Policies.file()
         if (policiesFile.exists()) {
             policyBranches += createHashmap(
                 json().fromJsonFile(Array<PolicyBranch>::class.java, policiesFile)
@@ -359,77 +439,86 @@ class Ruleset {
             }
         }
 
-        val beliefsFile = folderHandle.child("Beliefs.json")
+        val beliefsFile = RulesetFile.Beliefs.file()
         if (beliefsFile.exists())
             beliefs += createHashmap(json().fromJsonFile(Array<Belief>::class.java, beliefsFile))
 
-        val religionsFile = folderHandle.child("Religions.json")
+        val religionsFile = RulesetFile.Religions.file()
         if (religionsFile.exists())
             religions += json().fromJsonFile(Array<String>::class.java, religionsFile).toList()
 
-        val ruinRewardsFile = folderHandle.child("Ruins.json")
+        val ruinRewardsFile = RulesetFile.Ruins.file()
         if (ruinRewardsFile.exists())
             ruinRewards += createHashmap(json().fromJsonFile(Array<RuinReward>::class.java, ruinRewardsFile))
 
-        val nationsFile = folderHandle.child("Nations.json")
+        val nationsFile = RulesetFile.Nations.file()
         if (nationsFile.exists()) {
             nations += createHashmap(json().fromJsonFile(Array<Nation>::class.java, nationsFile))
             for (nation in nations.values) nation.setTransients()
         }
 
-        val difficultiesFile = folderHandle.child("Difficulties.json")
+        val difficultiesFile = RulesetFile.Difficulties.file()
         if (difficultiesFile.exists())
             difficulties += createHashmap(json().fromJsonFile(Array<Difficulty>::class.java, difficultiesFile))
 
-        val globalUniquesFile = folderHandle.child("GlobalUniques.json")
+        val globalUniquesFile = RulesetFile.GlobalUniques.file()
         if (globalUniquesFile.exists()) {
             globalUniques = json().fromJsonFile(GlobalUniques::class.java, globalUniquesFile)
             globalUniques.originRuleset = name
         }
 
-        val victoryTypesFile = folderHandle.child("VictoryTypes.json")
+        val victoryTypesFile = RulesetFile.VictoryTypes.file()
         if (victoryTypesFile.exists()) {
             victories += createHashmap(json().fromJsonFile(Array<Victory>::class.java, victoryTypesFile))
         }
 
-        val cityStateTypesFile = folderHandle.child("CityStateTypes.json")
+        val cityStateTypesFile = RulesetFile.CityStateTypes.file()
         if (cityStateTypesFile.exists()) {
             cityStateTypes += createHashmap(json().fromJsonFile(Array<CityStateType>::class.java, cityStateTypesFile))
         }
 
-        val personalitiesFile = folderHandle.child("Personalities.json")
+        val personalitiesFile = RulesetFile.Personalities.file()
         if (personalitiesFile.exists()) {
             personalities += createHashmap(json().fromJsonFile(Array<Personality>::class.java, personalitiesFile))
         }
 
-        val eventsFile = folderHandle.child("Events.json")
+        val eventsFile = RulesetFile.Events.file()
         if (eventsFile.exists()) {
             events += createHashmap(json().fromJsonFile(Array<Event>::class.java, eventsFile))
         }
 
-
+        // Tutorials exist per builtin ruleset or mod, but there's also a global file that's always loaded
+        // Note we can't rely on UncivGame.Current here, so we do the same thing getBuiltinRulesetFileHandle in RulesetCache does
+        val globalTutorialsFile = Gdx.files.internal("jsons").child(RulesetFile.Tutorials.filename)
+        if (globalTutorialsFile.exists())
+            tutorials += createHashmap(json().fromJsonFile(Array<Tutorial>::class.java, globalTutorialsFile))
+        val tutorialsFile = RulesetFile.Tutorials.file()
+        if (tutorialsFile.exists())
+            tutorials += createHashmap(json().fromJsonFile(Array<Tutorial>::class.java, tutorialsFile))
 
         // Add objects that might not be present in base ruleset mods, but are required
         if (modOptions.isBaseRuleset) {
+            val fallbackRuleset by lazy { RulesetCache.getVanillaRuleset() } // clone at most once
             // This one should be temporary
             if (unitTypes.isEmpty()) {
-                unitTypes.putAll(RulesetCache.getVanillaRuleset().unitTypes)
+                unitTypes.putAll(fallbackRuleset.unitTypes)
             }
 
             // These should be permanent
-            if (ruinRewards.isEmpty())
-                ruinRewards.putAll(RulesetCache.getVanillaRuleset().ruinRewards)
+            if (!ruinRewardsFile.exists())
+                ruinRewards.putAll(fallbackRuleset.ruinRewards)
 
-            if (globalUniques.uniques.isEmpty()) {
-                globalUniques = RulesetCache.getVanillaRuleset().globalUniques
+            if (!globalUniquesFile.exists()) {
+                globalUniques = fallbackRuleset.globalUniques
             }
             // If we have no victories, add all the default victories
-            if (victories.isEmpty()) victories.putAll(RulesetCache.getVanillaRuleset().victories)
+            if (victories.isEmpty()) victories.putAll(fallbackRuleset.victories)
 
-            if (speeds.isEmpty()) speeds.putAll(RulesetCache.getVanillaRuleset().speeds)
+            if (speeds.isEmpty()) speeds.putAll(fallbackRuleset.speeds)
+            if (difficulties.isEmpty()) difficulties.putAll(fallbackRuleset.difficulties)
 
             if (cityStateTypes.isEmpty())
-                for (cityStateType in RulesetCache.getVanillaRuleset().cityStateTypes.values)
+                for (cityStateType in fallbackRuleset.cityStateTypes.values)
                     cityStateTypes[cityStateType.name] = CityStateType().apply {
                         name = cityStateType.name
                         color = cityStateType.color
@@ -450,6 +539,8 @@ class Ruleset {
                             ).isEmpty()
                         })
                     }
+
+            updateResourceTransients()
         }
     }
 
@@ -458,24 +549,35 @@ class Ruleset {
      *  Alternatively, if you edit a tech column's building costs, you want it to affect all buildings in that column.
      *  This deals with that
      *  */
-    fun updateBuildingCosts() {
+    internal fun updateBuildingCosts() {
         for (building in buildings.values) {
-            if (building.cost == -1 && building.getMatchingUniques(UniqueType.Unbuildable).none { it.conditionals.isEmpty() }) {
-                val column = building.techColumn(this)
-                if (column != null) {
-                    building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
-                }
-            }
+            if (building.cost != -1) continue
+            if (building.getMatchingUniques(UniqueType.Unbuildable).any { it.modifiers.isEmpty() }) continue
+            val column = building.techColumn(this) ?: continue
+            building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
         }
     }
 
+    /** Introduced to support UniqueType.ImprovesResources: gives a resource the chance to scan improvements */
+    internal fun updateResourceTransients() {
+        for (resource in tileResources.values)
+            resource.setTransients(this)
+    }
+
+    @VisibleForTesting
+    /** For use by class TestGame. Use only before triggering the globalUniques.uniqueObjects lazy. */
+    fun addGlobalUniques(vararg uniques: String) {
+        globalUniques.uniques.addAll(uniques)
+    }
+
     /** Used for displaying a RuleSet's name */
     override fun toString() = when {
         name.isNotEmpty() -> name
         mods.size == 1 && RulesetCache[mods.first()]!!.modOptions.isBaseRuleset -> mods.first()
-        else -> "Combined RuleSet"
+        else -> "Combined RuleSet ($mods)"
     }
 
+    @Readonly
     fun getSummary(): String {
         val stringList = ArrayList<String>()
         if (modOptions.isBaseRuleset) stringList += "Base Ruleset"
@@ -490,5 +592,5 @@ class Ruleset {
         return stringList.joinToString { it.tr() }
     }
 
-    fun checkModLinks(tryFixUnknownUniques: Boolean = false) = RulesetValidator(this).getErrorList(tryFixUnknownUniques)
+    fun getErrorList(tryFixUnknownUniques: Boolean = false) = RulesetValidator.create(this, tryFixUnknownUniques).getErrorList()
 }
