@@ -1,14 +1,18 @@
 package com.unciv.models.ruleset
 
-import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.stats.Stat
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.objectdescriptions.uniquesToCivilopediaTextLines
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
+import yairm210.purity.annotations.Readonly
+import java.util.*
+import kotlin.collections.ArrayList
+import kotlin.collections.HashMap
 import kotlin.math.abs
 
-class Speed : RulesetObject(), IsPartOfGameInfoSerialization {
+class Speed : RulesetObject() {
     var modifier: Float = 1f
     var goldCostModifier: Float = modifier
     var productionCostModifier: Float = modifier
@@ -26,15 +30,45 @@ class Speed : RulesetObject(), IsPartOfGameInfoSerialization {
     var startYear: Float = -4000f
     var turns: ArrayList<HashMap<String, Float>> = ArrayList()
 
-    val yearsPerTurn: ArrayList<YearsPerTurn> by lazy {
-        ArrayList<YearsPerTurn>().apply {
-            turns.forEach { this.add(YearsPerTurn(it["yearsPerTurn"]!!, it["untilTurn"]!!.toInt())) }
+    // These could be private but for RulesetValidator checking it
+    data class YearsPerTurn(val yearInterval: Float, val untilTurn: Int) {
+        internal constructor(rawRow: HashMap<String, Float>) : this(rawRow["yearsPerTurn"]!!, rawRow["untilTurn"]!!.toInt())
+    }
+    val yearsPerTurn: ArrayList<YearsPerTurn> by lazy { turns.mapTo(ArrayList()) { YearsPerTurn(it) } }
+
+    /** End of defined turn range, used for starting Era's `startPercent` calculation */
+    @Readonly fun numTotalTurns(): Int = yearsPerTurn.last().untilTurn
+
+    /** Calculate a Year from a turn number.
+     *
+     *  Note that years can have fractional parts and the integer part of the year for two consecutive turns _can_ be equal,
+     *  but Unciv currently has no way to display that. This is left as Float to enable such display in the future,
+     *  maybe as months, or even the 18+1 'months' of the mayan Haab'.
+     *
+     *  @param turn The logical turn number, any offset from starting in an advanced Era already added in
+     */
+    @Readonly
+    fun turnToYear(turn: Int): Float {
+        var year = startYear
+        var intervalStartTurn = 0
+        val lastIntervalEndTurn = numTotalTurns()
+        for ((turnLength, intervalEndTurn) in yearsPerTurn) {
+            if (intervalStartTurn >= turn) break // ensure year isn't projected backwards for negative `turn`
+            if (turn <= intervalEndTurn || intervalEndTurn == lastIntervalEndTurn) {
+                // We can interpolate linearly within this interval and are done
+                year += (turn - intervalStartTurn) * turnLength
+                break
+            }
+            // Accumulate total length in years of this interval and move on to the following intervals.
+            year += (intervalEndTurn - intervalStartTurn) * turnLength
+            intervalStartTurn = intervalEndTurn
         }
+        return year
     }
 
-    val statCostModifiers: HashMap<Stat, Float> by lazy {
-        val map = HashMap<Stat, Float>()
-        for (stat in Stat.values()) {
+    val statCostModifiers: EnumMap<Stat, Float> by lazy {
+        val map = EnumMap<Stat, Float>(Stat::class.java)
+        for (stat in Stat.entries) {
             val modifier = when (stat) {
                 Stat.Production -> productionCostModifier
                 Stat.Gold -> goldCostModifier
@@ -54,6 +88,7 @@ class Speed : RulesetObject(), IsPartOfGameInfoSerialization {
         const val DEFAULTFORSIMULATION: String = "Standard"
     }
 
+    // Note: Speed uniques will be treated as part of GlobalUniques
     override fun getUniqueTarget(): UniqueTarget = UniqueTarget.Speed
 
     override fun makeLink(): String = "Speed/$name"
@@ -74,18 +109,7 @@ class Speed : RulesetObject(), IsPartOfGameInfoSerialization {
         yield(FormattedLine("Adjacent city religious pressure: [$religiousPressureAdjacentCity]${Fonts.faith}"))
         yield(FormattedLine("Peace deal duration: [$peaceDealDuration] turns${Fonts.turn}"))
         yield(FormattedLine("Start year: [" + ("{[${abs(startYear).toInt()}] " + (if (startYear < 0) "BC" else "AD") + "}]").tr()))
+        yieldAll(uniquesToCivilopediaTextLines())
     }.toList()
     override fun getSortGroup(ruleset: Ruleset): Int = (modifier * 1000).toInt()
-
-    fun numTotalTurns(): Int = yearsPerTurn.last().untilTurn
-}
-
-class YearsPerTurn {
-    var yearInterval: Float = 0f
-    var untilTurn: Int = 0
-
-    constructor(yearsPerTurn: Float, turnsPerIncrement: Int) {
-        this.yearInterval = yearsPerTurn
-        this.untilTurn = turnsPerIncrement
-    }
 }
