@@ -5,6 +5,7 @@ import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.stats.INamed
 import com.unciv.ui.components.extensions.colorFromRGB
+import yairm210.purity.annotations.Readonly
 
 class CityStateType: INamed {
     override var name = ""
@@ -14,9 +15,9 @@ class CityStateType: INamed {
     var allyBonusUniques = ArrayList<String>()
     val allyBonusUniqueMap by lazy { allyBonusUniques.toUniqueMap() }
     private fun ArrayList<String>.toUniqueMap() =
-        UniqueMap(asSequence().map { Unique(it, sourceObjectType = UniqueTarget.CityState) })
+        UniqueMap(asSequence().map { Unique(it, sourceObjectType = UniqueTarget.CityState, sourceObjectName = name) })
 
     var color: List<Int> = listOf(255,255,255)
     private val colorObject by lazy { colorFromRGB(color) }
-    fun getColor() = colorObject
+    @Readonly fun getColor() = colorObject
 }
