@@ -3,28 +3,43 @@ package com.unciv.models.ruleset.nation
 import com.badlogic.gdx.graphics.Color
 import com.unciv.Constants
 import com.unciv.logic.MultiFilter
+import com.unciv.models.ImmutableColor
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetObject
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.squareBraceRegex
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.extensions.colorFromRGB
+import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.objectdescriptions.BaseUnitDescriptions
 import com.unciv.ui.objectdescriptions.BuildingDescriptions
 import com.unciv.ui.objectdescriptions.ImprovementDescriptions
 import com.unciv.ui.objectdescriptions.uniquesToCivilopediaTextLines
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 import kotlin.math.pow
 
 class Nation : RulesetObject() {
     var leaderName = ""
-    fun getLeaderDisplayName() = if (isCityState || isSpectator) name
-        else "[$leaderName] of [$name]"
+
+    /**
+     * Retrieves a display name for the nation's leader, considering the provided title (untranslated).
+     *
+     * @param [title] Optional title to apply to the leader. For example: `[leaderName] the Great`
+     */
+    @Readonly fun getLeaderDisplayName(title: String = ""): String = when {
+        isCityState || isSpectator -> name
+        title.isEmpty() -> "[$leaderName] of [$name]"
+        else -> "[${title.fillPlaceholders(leaderName)}] of [$name]"
+    }
 
     val style = ""
-    fun getStyleOrCivName() = style.ifEmpty { name }
+    @Readonly fun getStyleOrCivName() = style.ifEmpty { name }
 
     var cityStateType: String? = null
     var preferredVictoryType: String = Constants.neutralVictoryType
@@ -37,6 +52,10 @@ class Nation : RulesetObject() {
     var attacked = ""
     /** Shown for AlertType.Defeated */
     var defeated = ""
+    /** Shown for MajorCivDiplomacyTable.getDenounceButton */
+    var denounced = ""
+    /** Shown for Declaration of Friendship */
+    var declaringFriendship = ""
     /** Shown for AlertType.FirstContact */
     var introduction = ""
     /** Shown in TradePopup when other Civs initiate trade with a player */
@@ -69,13 +88,13 @@ class Nation : RulesetObject() {
     override fun getUniqueTarget() = UniqueTarget.Nation
 
     @Transient
-    private lateinit var outerColorObject: Color
-    fun getOuterColor(): Color = outerColorObject
+    private var outerColorObject:ImmutableColor = ImmutableColor(Color.WHITE) // Not lateinit for unit tests
+    fun getOuterColor(): ImmutableColor = outerColorObject
 
     @Transient
-    private lateinit var innerColorObject: Color
+    private var innerColorObject: ImmutableColor = ImmutableColor(Color.BLACK) // Not lateinit for unit tests
 
-    fun getInnerColor(): Color = innerColorObject
+    fun getInnerColor(): ImmutableColor = innerColorObject
 
     val isCityState by lazy { cityStateType != null }
     val isMajorCiv by lazy { !isBarbarian && !isCityState && !isSpectator }
@@ -91,13 +110,15 @@ class Nation : RulesetObject() {
     var ignoreHillMovementCost = false
 
     fun setTransients() {
-        outerColorObject = colorFromRGB(outerColor)
+        fun safeColorFromRGB(rgb: List<Int>) = ImmutableColor(if (rgb.size >= 3) colorFromRGB(rgb) else Color.PURPLE)
 
-        innerColorObject = if (innerColor == null) Color.BLACK
-                           else colorFromRGB(innerColor!!)
+        outerColorObject = safeColorFromRGB(outerColor)
 
-        forestsAndJunglesAreRoads = uniqueMap.containsKey(UniqueType.ForestsAndJunglesAreRoads.placeholderText)
-        ignoreHillMovementCost = uniqueMap.containsKey(UniqueType.IgnoreHillMovementCost.placeholderText)
+        innerColorObject = if (innerColor == null) ImmutableColor(ImageGetter.CHARCOAL)
+                           else safeColorFromRGB(innerColor!!)
+
+        forestsAndJunglesAreRoads = uniqueMap.hasUnique(UniqueType.ForestsAndJunglesAreRoads)
+        ignoreHillMovementCost = uniqueMap.hasUnique(UniqueType.IgnoreHillMovementCost)
     }
 
 
@@ -107,6 +128,11 @@ class Nation : RulesetObject() {
         isBarbarian -> 9
         else -> 0
     }
+    override fun getSubCategory(ruleset: Ruleset): String? = when {
+        isCityState -> "City-States"
+        isBarbarian -> "Other"
+        else -> "Civilizations"
+    }
 
     override fun getCivilopediaTextLines(ruleset: Ruleset): List<FormattedLine> {
         val textList = ArrayList<FormattedLine>()
@@ -148,6 +174,7 @@ class Nation : RulesetObject() {
         return textList
     }
 
+    @Readonly
     private fun getCityStateInfo(ruleset: Ruleset): List<FormattedLine> {
         val textList = ArrayList<FormattedLine>()
 
@@ -162,7 +189,7 @@ class Nation : RulesetObject() {
             val bonuses = uniqueMap.getAllUniques().filterNot { it.isHiddenToUsers() }
             if (bonuses.none()) return
             textList += FormattedLine()
-            textList += FormattedLine("{$header:} ")
+            textList += FormattedLine("{$header} ")
             for (unique in bonuses) {
                 textList += FormattedLine(unique, indent = 1)
                 if (unique.type == UniqueType.CityStateUniqueLuxury) showResources = true
@@ -192,14 +219,15 @@ class Nation : RulesetObject() {
 
     private fun getUniqueBuildingsText(ruleset: Ruleset) = sequence {
         for (building in ruleset.buildings.values) {
-            if (building.uniqueTo != name) continue
+            if (building.uniqueTo == null) continue
+            if (!matchesFilter(building.uniqueTo!!)) continue
             if (building.isHiddenFromCivilopedia(ruleset)) continue
             yield(FormattedLine(separator = true))
             yield(FormattedLine("{${building.name}} -", link = building.makeLink()))
             if (building.replaces != null && ruleset.buildings.containsKey(building.replaces!!)) {
                 val originalBuilding = ruleset.buildings[building.replaces!!]!!
                 yield(FormattedLine("Replaces [${originalBuilding.name}]", link = originalBuilding.makeLink(), indent = 1))
-                yieldAll(BuildingDescriptions.getDifferences(ruleset, originalBuilding, building))
+                yieldAll(BuildingDescriptions.getDifferences(originalBuilding, building))
                 yield(FormattedLine())
             } else if (building.replaces != null) {
                 yield(FormattedLine("Replaces [${building.replaces}], which is not found in the ruleset!", indent = 1))
@@ -211,7 +239,8 @@ class Nation : RulesetObject() {
 
     private fun getUniqueUnitsText(ruleset: Ruleset) = sequence {
         for (unit in ruleset.units.values) {
-            if (unit.uniqueTo != name || unit.isHiddenFromCivilopedia(ruleset)) continue
+            if (unit.isHiddenFromCivilopedia(ruleset)) continue
+            if (unit.uniqueTo == null || !matchesFilter(unit.uniqueTo!!)) continue
             yield(FormattedLine(separator = true))
             yield(FormattedLine("{${unit.name}} -", link = "Unit/${unit.name}"))
             if (unit.replaces != null && ruleset.units.containsKey(unit.replaces!!)) {
@@ -237,7 +266,8 @@ class Nation : RulesetObject() {
 
     private fun getUniqueImprovementsText(ruleset: Ruleset) = sequence {
         for (improvement in ruleset.tileImprovements.values) {
-            if (improvement.uniqueTo != name || improvement.isHiddenFromCivilopedia(ruleset)) continue
+            if (improvement.isHiddenFromCivilopedia(ruleset)) continue
+            if (improvement.uniqueTo == null || !matchesFilter(improvement.uniqueTo!!)) continue
 
             yield(FormattedLine(separator = true))
             yield(FormattedLine(improvement.name, link = "Improvement/${improvement.name}"))
@@ -255,27 +285,36 @@ class Nation : RulesetObject() {
         }
     }
 
-    fun getContrastRatio() = getContrastRatio(getInnerColor(), getOuterColor())
-
-    fun matchesFilter(filter: String): Boolean {
-        return MultiFilter.multiFilter(filter, ::matchesSingleFilter)
+    @Readonly
+    fun matchesFilter(filter: String, state: GameContext? = null, multiFilter: Boolean = true): Boolean {
+        // Todo: Add 'multifilter=false' option to Multifilter itself to cut down on duplicate code
+        return if (multiFilter) MultiFilter.multiFilter(filter, {
+            matchesSingleFilter(filter) ||
+                state != null && hasTagUnique(it, state) ||
+                state == null && hasTagUnique(it)
+        })
+        else matchesSingleFilter(filter) ||
+            state != null && hasTagUnique(filter, state) ||
+            state == null && hasTagUnique(filter)
     }
 
+    @Readonly
     private fun matchesSingleFilter(filter: String): Boolean {
+        // All cases are compile-time constants, for performance
         return when (filter) {
-            in Constants.all -> true
-            name -> true
+            "All", "all" -> true
             "Major" -> isMajorCiv
-            // "CityState" to be deprecated, replaced by "City-States"
-            "CityState", Constants.cityStates -> isCityState
-            else -> uniques.contains(filter)
+            Constants.cityStates, "City-State" -> isCityState
+            else -> filter == name
         }
     }
 }
 
 
 /** All defined by https://www.w3.org/TR/WCAG20/#relativeluminancedef */
+@Pure
 fun getRelativeLuminance(color: Color): Double {
+    @Pure
     fun getRelativeChannelLuminance(channel: Float): Double =
             if (channel < 0.03928) channel / 12.92
             else ((channel + 0.055) / 1.055).pow(2.4)
