@@ -13,17 +13,21 @@ import com.unciv.models.ruleset.RejectionReason
 import com.unciv.models.ruleset.RejectionReasonType
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetObject
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.Conditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.Unique
+import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.models.ruleset.unique.Conditionals
 import com.unciv.models.stats.Stat
 import com.unciv.ui.components.extensions.getNeedMoreAmountString
 import com.unciv.ui.components.extensions.toPercent
-import com.unciv.ui.components.extensions.yieldIfNotNull
 import com.unciv.ui.objectdescriptions.BaseUnitDescriptions
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
+import com.unciv.utils.yieldIfNotNull
+import yairm210.purity.annotations.Cache
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Readonly
 import kotlin.math.pow
 
 // This is BaseUnit because Unit is already a base Kotlin class and to avoid mixing the two up
@@ -42,7 +46,8 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
     var interceptRange = 0
     var unitType: String = ""
 
-    val type by lazy { ruleset.unitTypes[unitType]!! }
+    val type by lazy { ruleset.unitTypes[unitType]
+        ?: throw Exception("Unit $name has unit type $unitType which is not present in ruleset!") }
     override var requiredTech: String? = null
     var requiredResource: String? = null
 
@@ -51,24 +56,38 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
     var replacementTextForUniques = ""
     var promotions = HashSet<String>()
     var obsoleteTech: String? = null
-    fun techsThatObsoleteThis(): Sequence<String> = if (obsoleteTech == null) sequenceOf() else sequenceOf(obsoleteTech!!)
-    fun techsAtWhichAutoUpgradeInProduction(): Sequence<String> = techsThatObsoleteThis()
-    fun techsAtWhichNoLongerAvailable(): Sequence<String> = techsThatObsoleteThis()
-    @Suppress("unused") // Keep the how-to around
-    fun isObsoletedBy(techName: String): Boolean = techsThatObsoleteThis().contains(techName)
+    
+    @Readonly fun techsThatObsoleteThis(): Sequence<String> = if (obsoleteTech == null) emptySequence() else sequenceOf(obsoleteTech!!)
+    @Readonly fun techsAtWhichAutoUpgradeInProduction(): Sequence<String> = techsThatObsoleteThis()
+    @Readonly fun techsAtWhichNoLongerAvailable(): Sequence<String> = techsThatObsoleteThis()
     var upgradesTo: String? = null
     var replaces: String? = null
     var uniqueTo: String? = null
     var attackSound: String? = null
 
-    @Transient
-    var cachedForceEvaluation: Int = -1
 
     @Transient
     val costFunctions = BaseUnitCost(this)
 
     lateinit var ruleset: Ruleset
+        private set
 
+    fun setRuleset(ruleset: Ruleset) {
+        this.ruleset = ruleset
+        val list = ArrayList(uniques)
+        list.addAll(ruleset.globalUniques.unitUniques)
+        list.addAll(type.uniques)
+        rulesetUniqueObjects = uniqueObjectsProvider(list)
+        rulesetUniqueMap = uniqueMapProvider(rulesetUniqueObjects) // Has global uniques by the unique objects already
+    }
+
+    @Transient
+    var rulesetUniqueObjects: List<Unique> = ArrayList()
+        private set
+
+    @Transient
+    var rulesetUniqueMap: UniqueMap = UniqueMap()
+        private set
 
     /** Generate short description as comma-separated string for Technology description "Units enabled" and GreatPersonPickerScreen */
     fun getShortDescription(uniqueExclusionFilter: Unique.() -> Boolean = {false}) = BaseUnitDescriptions.getShortDescription(this, uniqueExclusionFilter)
@@ -82,30 +101,38 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
     override fun getCivilopediaTextLines(ruleset: Ruleset): List<FormattedLine> =
             BaseUnitDescriptions.getCivilopediaTextLines(this, ruleset)
 
-    override fun isHiddenBySettings(gameInfo: GameInfo) =
-        super<INonPerpetualConstruction>.isHiddenBySettings(gameInfo) ||
+    override fun getSortGroup(ruleset: Ruleset): Int = ruleset.technologies[requiredTech]?.era(ruleset)?.eraNumber ?: 100
+    override fun getSubCategory(ruleset: Ruleset): String? = ruleset.technologies[requiredTech]?.era(ruleset)?.name ?: "Other"
+
+    @Readonly
+    override fun isUnavailableBySettings(gameInfo: GameInfo) =
+        super<INonPerpetualConstruction>.isUnavailableBySettings(gameInfo) ||
         (!gameInfo.gameParameters.nuclearWeaponsEnabled && isNuclearWeapon())
 
-    fun getUpgradeUnits(stateForConditionals: StateForConditionals? = null): Sequence<String> {
+    @Readonly
+    fun getUpgradeUnits(gameContext: GameContext = GameContext.EmptyState): Sequence<String> {
         return sequence {
             yieldIfNotNull(upgradesTo)
-            for (unique in getMatchingUniques(UniqueType.CanUpgrade, stateForConditionals))
+            for (unique in getMatchingUniques(UniqueType.CanUpgrade, gameContext))
                 yield(unique.params[0])
         }
     }
 
-    fun getRulesetUpgradeUnits(stateForConditionals: StateForConditionals? = null): Sequence<BaseUnit> {
+    @Readonly
+    fun getRulesetUpgradeUnits(gameContext: GameContext = GameContext.EmptyState): Sequence<BaseUnit> {
         return sequence {
-            for (unit in getUpgradeUnits(stateForConditionals))
+            for (unit in getUpgradeUnits(gameContext))
                 yieldIfNotNull(ruleset.units[unit])
         }
     }
 
-    fun getMapUnit(civInfo: Civilization): MapUnit {
-        val unit = MapUnit()
+    @Readonly @Suppress("purity") // technically DOES increase gameInfo.lastUnitId...
+    fun newMapUnit(civInfo: Civilization, unitId: Int? = null): MapUnit {
+        @LocalState val unit = MapUnit()
         unit.name = name
         unit.civ = civInfo
-        unit.owner = civInfo.civName
+        unit.owner = civInfo.civID
+        unit.id = unitId ?: ++civInfo.gameInfo.lastUnitId
 
         // must be after setting name & civInfo because it sets the baseUnit according to the name
         // and the civInfo is required for using `hasUnique` when determining its movement options
@@ -114,18 +141,38 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
         return unit
     }
 
+    @Readonly
+    override fun hasUnique(uniqueType: UniqueType, state: GameContext?): Boolean {
+        val gameContext = state ?: GameContext.EmptyState
+        return if (::ruleset.isInitialized) rulesetUniqueMap.hasUnique(uniqueType, gameContext)
+        else super<RulesetObject>.hasUnique(uniqueType, gameContext)
+    }
+
+    @Readonly
+    override fun hasTagUnique(uniqueTag: String, state: GameContext?): Boolean {
+        val gameContext = state ?: GameContext.EmptyState
+        return if (::ruleset.isInitialized) rulesetUniqueMap.hasUnique(uniqueTag, gameContext)
+        else super<RulesetObject>.hasTagUnique(uniqueTag, gameContext)
+    }
+
+    @Readonly
+    override fun hasTagUnique(uniqueTag: String): Boolean {
+        return if (::ruleset.isInitialized) rulesetUniqueMap.hasTagUnique(uniqueTag)
+        else super<RulesetObject>.hasTagUnique(uniqueTag)
+    }
+
     /** Allows unique functions (getMatchingUniques, hasUnique) to "see" uniques from the UnitType */
-    override fun getMatchingUniques(uniqueType: UniqueType, stateForConditionals: StateForConditionals?): Sequence<Unique> {
-        val ourUniques = super<RulesetObject>.getMatchingUniques(uniqueType, stateForConditionals)
-        if (! ::ruleset.isInitialized) { // Not sure if this will ever actually happen, but better safe than sorry
-            return ourUniques
-        }
-        val typeUniques = type.getMatchingUniques(uniqueType, stateForConditionals)
-        // Memory optimization - very rarely do we actually get uniques from both sources,
-        //   and sequence addition is expensive relative to the rare case that we'll actually need it
-        if (ourUniques.none()) return typeUniques
-        if (typeUniques.none()) return ourUniques
-        return ourUniques + type.getMatchingUniques(uniqueType, stateForConditionals)
+    @Readonly
+    override fun getMatchingUniques(uniqueType: UniqueType, state: GameContext): Sequence<Unique> {
+        return if (::ruleset.isInitialized) rulesetUniqueMap.getMatchingUniques(uniqueType, state)
+        else super<RulesetObject>.getMatchingUniques(uniqueType, state)
+    }
+
+    /** Allows unique functions (getMatchingUniques, hasUnique) to "see" uniques from the UnitType */
+    @Readonly
+    override fun getMatchingTagUniques(uniqueTag: String, state: GameContext): Sequence<Unique> {
+        return if (::ruleset.isInitialized) rulesetUniqueMap.getMatchingTagUniques(uniqueTag, state)
+        else super<RulesetObject>.getMatchingTagUniques(uniqueTag, state)
     }
 
     override fun getProductionCost(civInfo: Civilization, city: City?): Int  = costFunctions.getProductionCost(civInfo, city)
@@ -139,15 +186,6 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
         return super.canBePurchasedWithStat(city, stat)
     }
 
-    /** Whenever we call .hasUniques() or .getMatchingUniques(), we also want to return the uniques from the unit type
-     * All of the IHasUniques functions converge to getMatchingUniques, so overriding this one function gives us all of them */
-    override fun getMatchingUniques(uniqueTemplate: String, stateForConditionals: StateForConditionals?): Sequence<Unique> {
-        val baseUnitMatchingUniques = super<RulesetObject>.getMatchingUniques(uniqueTemplate, stateForConditionals)
-        return if (::ruleset.isInitialized) baseUnitMatchingUniques +
-                type.getMatchingUniques(uniqueTemplate, stateForConditionals)
-        else baseUnitMatchingUniques // for e.g. Mod Checker, we may check a BaseUnit's uniques without initializing ruleset
-    }
-
     override fun getBaseBuyCost(city: City, stat: Stat): Float? {
         return sequence {
             val baseCost = super.getBaseBuyCost(city, stat)
@@ -159,70 +197,70 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
 
     override fun getStatBuyCost(city: City, stat: Stat): Int? = costFunctions.getStatBuyCost(city, stat)
 
-    fun getDisbandGold(civInfo: Civilization) = getBaseGoldCost(civInfo, null).toInt() / 20
+    @Readonly fun getDisbandGold(civInfo: Civilization) = getBaseGoldCost(civInfo, null).toInt() / 20
 
     override fun shouldBeDisplayed(cityConstructions: CityConstructions): Boolean {
         val rejectionReasons = getRejectionReasons(cityConstructions)
 
+        if (hasUnique(UniqueType.ShowsWhenUnbuilable, cityConstructions.city.state) &&
+            rejectionReasons.none { it.isNeverVisible() })
+            return true
+
         if (rejectionReasons.none { !it.shouldShow }) return true
         if (canBePurchasedWithAnyStat(cityConstructions.city)
             && rejectionReasons.all { it.type == RejectionReasonType.Unbuildable }) return true
         return false
     }
 
-    override fun getRejectionReasons(cityConstructions: CityConstructions): Sequence<RejectionReason> = sequence {
-        if (isWaterUnit() && !cityConstructions.city.isCoastal())
-            yield(RejectionReasonType.WaterUnitsInCoastalCities.toInstance())
-        if (isAirUnit()) {
-            val fakeUnit = getMapUnit(cityConstructions.city.civ)
-            val canUnitEnterTile = fakeUnit.movement.canMoveTo(cityConstructions.city.getCenterTile())
-            if (!canUnitEnterTile)
-                yield(RejectionReasonType.NoPlaceToPutUnit.toInstance())
-        }
-        val civInfo = cityConstructions.city.civ
-
-        for (unique in getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals))
-            yieldAll(notMetRejections(unique, cityConstructions))
-
-        for (unique in getMatchingUniques(UniqueType.CanOnlyBeBuiltWhen, StateForConditionals.IgnoreConditionals))
-            yieldAll(notMetRejections(unique, cityConstructions, true))
-
-        for (unique in getMatchingUniques(UniqueType.Unavailable, StateForConditionals(civInfo, cityConstructions.city)))
-            yield(RejectionReasonType.ShouldNotBeDisplayed.toInstance())
-
-        for (unique in getMatchingUniques(UniqueType.RequiresPopulation))
-            if (unique.params[0].toInt() > cityConstructions.city.population.population)
-                yield(RejectionReasonType.PopulationRequirement.toInstance(unique.text))
-
-        yieldAll(getRejectionReasons(civInfo, cityConstructions.city))
-    }
+    override fun getRejectionReasons(cityConstructions: CityConstructions): Sequence<RejectionReason> =
+        getRejectionReasons(cityConstructions.city.civ, cityConstructions.city)
 
+    @Readonly
     fun getRejectionReasons(
         civ: Civilization,
         city: City? = null,
         additionalResources: Counter<String> = Counter.ZERO
     ): Sequence<RejectionReason> = sequence {
+
+        val stateForConditionals = city?.state ?: civ.state
+
+        if (city != null && isWaterUnit && !city.isNaval())
+            yield(RejectionReasonType.WaterUnitsInCoastalCities.toInstance())
+
+        for (unique in getMatchingUniques(UniqueType.OnlyAvailable, GameContext.IgnoreConditionals))
+            yieldAll(notMetRejections(unique, civ, city))
+
+        for (unique in getMatchingUniques(UniqueType.CanOnlyBeBuiltWhen, GameContext.IgnoreConditionals))
+            yieldAll(notMetRejections(unique, civ, city, true))
+
+        for (unique in getMatchingUniques(UniqueType.Unavailable, stateForConditionals))
+            yield(RejectionReasonType.ShouldNotBeDisplayed.toInstance())
+
+        if (city != null)
+            for (unique in getMatchingUniques(UniqueType.RequiresPopulation))
+                if (unique.params[0].toInt() > city.population.population)
+                    yield(RejectionReasonType.PopulationRequirement.toInstance(unique.getDisplayText()))
+
         for (requiredTech: String in requiredTechs())
             if (!civ.tech.isResearched(requiredTech))
                 yield(RejectionReasonType.RequiresTech.toInstance("$requiredTech not researched"))
+        
         for (obsoleteTech: String in techsAtWhichNoLongerAvailable())
             if (civ.tech.isResearched(obsoleteTech))
                 yield(RejectionReasonType.Obsoleted.toInstance("Obsolete by $obsoleteTech"))
 
-        if (uniqueTo != null && uniqueTo != civ.civName)
+        if (uniqueTo != null && !civ.matchesFilter(uniqueTo!!, stateForConditionals))
             yield(RejectionReasonType.UniqueToOtherNation.toInstance("Unique to $uniqueTo"))
         if (civ.cache.uniqueUnits.any { it.replaces == name })
             yield(RejectionReasonType.ReplacedByOurUnique.toInstance("Our unique unit replaces this"))
 
-        if (isHiddenBySettings(civ.gameInfo))
+        if (isUnavailableBySettings(civ.gameInfo))
             yield(RejectionReasonType.DisabledBySetting.toInstance())
 
-        val stateForConditionals = StateForConditionals(civ, city)
-
         if (hasUnique(UniqueType.Unbuildable, stateForConditionals))
             yield(RejectionReasonType.Unbuildable.toInstance())
 
-        if ((civ.isCityState() || civ.isOneCityChallenger()) && hasUnique(UniqueType.FoundCity, stateForConditionals))
+        if ((civ.isCityState || civ.isOneCityChallenger()) && hasUnique(UniqueType.FoundCity, GameContext.IgnoreConditionals))
             yield(RejectionReasonType.NoSettlerForOneCityPlayers.toInstance())
 
         if (getMatchingUniques(UniqueType.MaxNumberBuildable, stateForConditionals).any {
@@ -230,26 +268,40 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             })
             yield(RejectionReasonType.MaxNumberBuildable.toInstance())
 
-        if (!civ.isBarbarian()) { // Barbarians don't need resources
+        if (!civ.isBarbarian) { // Barbarians don't need resources
             val civResources = Counter(civ.getCivResourcesByName()) + additionalResources
-            for ((resource, requiredAmount) in getResourceRequirementsPerTurn(StateForConditionals(civ))) {
+            for ((resource, requiredAmount) in getResourceRequirementsPerTurn(stateForConditionals)) {
                 val availableAmount = civResources[resource]
                 if (availableAmount < requiredAmount) {
                     val message = resource.getNeedMoreAmountString(requiredAmount - availableAmount)
                     yield(RejectionReasonType.ConsumesResources.toInstance(message))
                 }
             }
+
+            // If we've already paid the unit costs, we don't need to pay it again
+            if (city == null || city.cityConstructions.getWorkDone(name) == 0)
+                for ((resourceName, amount) in getStockpiledResourceRequirements(stateForConditionals)) {
+                    val availableResources = city?.getAvailableResourceAmount(resourceName) ?: civ.getResourceAmount(resourceName)
+                    if (availableResources < amount)
+                        yield(RejectionReasonType.ConsumesResources.toInstance(resourceName.getNeedMoreAmountString(amount - availableResources)))
+                }
         }
 
         for (unique in civ.getMatchingUniques(UniqueType.CannotBuildUnits, stateForConditionals))
-            if (this@BaseUnit.matchesFilter(unique.params[0])) {
-                val hasHappinessCondition = unique.conditionals.any {
-                    it.type == UniqueType.ConditionalBelowHappiness || it.type == UniqueType.ConditionalBetweenHappiness
-                }
-                if (hasHappinessCondition)
-                    yield(RejectionReasonType.CannotBeBuiltUnhappiness.toInstance(unique.text))
-                else yield(RejectionReasonType.CannotBeBuilt.toInstance())
+            if (this@BaseUnit.matchesFilter(unique.params[0], stateForConditionals)) {
+                yield(RejectionReasonType.CannotBeBuilt.toInstance())
             }
+
+        if (city != null && isAirUnit() && !canUnitEnterTile(city)) {
+            // Not actually added to civ so doesn't require destroy
+            yield(RejectionReasonType.NoPlaceToPutUnit.toInstance())
+        }
+    }
+    
+    @Readonly @Suppress("purity") // Good suppression - we cheat by creating a unit
+    fun canUnitEnterTile(city: City): Boolean {
+        val fakeUnit = newMapUnit(city.civ, Constants.NO_ID)
+        return fakeUnit.movement.canMoveTo(city.getCenterTile())
     }
 
     /**
@@ -257,11 +309,11 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
      * Also custom handles [UniqueType.ConditionalBuildingBuiltAmount], and
      * [UniqueType.ConditionalBuildingBuiltAll]
      */
-    private fun notMetRejections(unique: Unique, cityConstructions: CityConstructions, built: Boolean=false): Sequence<RejectionReason> = sequence {
-        val civ = cityConstructions.city.civ
-        for (conditional in unique.conditionals) {
+    @Readonly
+    private fun notMetRejections(unique: Unique, civ: Civilization, city: City?, built: Boolean=false): Sequence<RejectionReason> = sequence {
+        for (conditional in unique.modifiers) {
             // We yield a rejection only when conditionals are NOT met
-            if (Conditionals.conditionalApplies(unique, conditional, StateForConditionals(civ, cityConstructions.city)))
+            if (Conditionals.conditionalApplies(unique, conditional, city?.state ?: civ.state))
                 continue
             when (conditional.type) {
                 UniqueType.ConditionalBuildingBuiltAmount -> {
@@ -290,7 +342,7 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
                 }
                 else -> {
                     if (built)
-                        yield(RejectionReasonType.CanOnlyBeBuiltInSpecificCities.toInstance(unique.text))
+                        yield(RejectionReasonType.CanOnlyBeBuiltInSpecificCities.toInstance(unique.getDisplayText()))
                     else
                         yield(RejectionReasonType.ShouldNotBeDisplayed.toInstance())
                 }
@@ -298,27 +350,28 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
         }
     }
 
-    fun isBuildable(civInfo: Civilization) = getRejectionReasons(civInfo).none()
+    @Readonly fun isBuildable(civInfo: Civilization) = getRejectionReasons(civInfo).none()
 
     override fun isBuildable(cityConstructions: CityConstructions): Boolean =
             getRejectionReasons(cityConstructions).none()
 
-    override fun postBuildEvent(cityConstructions: CityConstructions, boughtWith: Stat?): Boolean {
+    fun construct(cityConstructions: CityConstructions, boughtWith: Stat?): MapUnit? {
         val civInfo = cityConstructions.city.civ
         val unit = civInfo.units.addUnit(this, cityConstructions.city)
-            ?: return false  // couldn't place the unit, so there's actually no unit =(
+            ?: return null  // couldn't place the unit, so there's actually no unit =(
 
         //movement penalty
-        if (boughtWith != null && !civInfo.gameInfo.gameParameters.godMode && !unit.hasUnique(UniqueType.MoveImmediatelyOnceBought))
+        if (boughtWith != null && !civInfo.gameInfo.gameParameters.godMode && !unit.hasUnique(UniqueType.CanMoveImmediatelyOnceBought))
             unit.currentMovement = 0f
 
         addConstructionBonuses(unit, cityConstructions)
 
-        return true
+        return unit
     }
 
     // This returns the name of the unit this tech upgrades this unit to,
     // or null if there is no automatic upgrade at that tech.
+    @Readonly
     fun automaticallyUpgradedInProductionToUnitByTech(techName: String): String? {
         for (obsoleteTech: String in techsAtWhichAutoUpgradeInProduction())
             if (obsoleteTech == techName)
@@ -332,11 +385,8 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
         @Suppress("LocalVariableName")
         var XP = 0
 
-        for (unique in
-        cityConstructions.city.getMatchingUniques(UniqueType.UnitStartingExperience)
-            .filter { cityConstructions.city.matchesFilter(it.params[2]) }
-        ) {
-            if (unit.matchesFilter(unique.params[0]))
+        for (unique in cityConstructions.city.getMatchingUniques(UniqueType.UnitStartingExperience)) {
+            if (unit.matchesFilter(unique.params[0]) && cityConstructions.city.matchesFilter(unique.params[2]))
                 XP += unique.params[1].toInt()
         }
         unit.promotions.XP = XP
@@ -346,127 +396,131 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             val filter = unique.params[0]
             val promotion = unique.params.last()
 
-            if (unit.matchesFilter(filter)
-                || (
-                    filter == "relevant"
+            val isRelevantPromotion = filter == "relevant"
                     && civInfo.gameInfo.ruleset.unitPromotions.values
-                        .any {
-                            it.name == promotion
-                            && unit.type.name in it.unitTypes
-                        }
-                    )
-            ) {
+                .any { it.name == promotion && unit.type.name in it.unitTypes }
+            
+            if (isRelevantPromotion || unit.matchesFilter(filter)) {
                 unit.promotions.addPromotion(promotion, isFree = true)
             }
         }
     }
 
+    @Readonly
     fun getReplacedUnit(ruleset: Ruleset): BaseUnit {
         return if (replaces == null) this
         else ruleset.units[replaces!!]!!
     }
 
 
-    private val cachedMatchesFilterResult = HashMap<String, Boolean>()
+    @Cache private val cachedMatchesFilterResult = HashMap<String, Boolean>()
 
     /** Implements [UniqueParameterType.BaseUnitFilter][com.unciv.models.ruleset.unique.UniqueParameterType.BaseUnitFilter] */
-    fun matchesFilter(filter: String): Boolean {
-        val cachedAnswer = cachedMatchesFilterResult[filter]
-        if (cachedAnswer != null) return cachedAnswer
-        val newAnswer = MultiFilter.multiFilter(filter, { matchesSingleFilter(it) })
-        cachedMatchesFilterResult[filter] = newAnswer
-        return newAnswer
+    @Readonly
+    fun matchesFilter(filter: String, state: GameContext? = null, multiFilter: Boolean = true): Boolean {
+        return if (multiFilter) MultiFilter.multiFilter(filter, {
+            cachedMatchesFilterResult.getOrPut(it) { matchesSingleFilter(it, state) } ||
+                state != null && hasTagUnique(it, state) ||
+                state == null && hasTagUnique(it)
+        })
+        else cachedMatchesFilterResult.getOrPut(filter) { matchesSingleFilter(filter, state) } ||
+            state != null && hasTagUnique(filter, state) ||
+            state == null && hasTagUnique(filter)
     }
-
-    fun matchesSingleFilter(filter: String): Boolean {
+    
+    @Readonly
+    fun matchesSingleFilter(filter: String, state: GameContext? = null): Boolean {
+        // all cases are constants for performance
         return when (filter) {
-            unitType -> true
-            name -> true
-            replaces -> true
-            in Constants.all -> true
-
+            "all", "All" -> true
             "Melee" -> isMelee()
             "Ranged" -> isRanged()
             "Civilian" -> isCivilian()
-            "Military" -> isMilitary()
-            "Land" -> isLandUnit()
-            "Water" -> isWaterUnit()
+            "Military" -> isMilitary
+            "Land" -> isLandUnit
+            "Water" -> isWaterUnit
             "Air" -> isAirUnit()
-            "non-air" -> !movesLikeAirUnits()
+            "non-air" -> !movesLikeAirUnits
 
             "Nuclear Weapon" -> isNuclearWeapon()
-            "Great Person" -> isGreatPerson()
+            "Great Person" -> isGreatPerson
             "Religious" -> hasUnique(UniqueType.ReligiousUnit)
 
             else -> {
-                if (type.matchesFilter(filter)) return true
+                if (filter == unitType) return true
+                else if (filter == name) return true
+                else if (filter == replaces) return true
+                
                 for (requiredTech: String in requiredTechs())
-                    if (ruleset.technologies[requiredTech]?.matchesFilter(filter) == true) return true
+                    if (ruleset.technologies[requiredTech]?.matchesFilter(filter, state, false) == true) return true
                 if (
                 // Uniques using these kinds of filters should be deprecated and replaced with adjective-only parameters
                     filter.endsWith(" units")
                     // "military units" --> "Military", using invariant locale
-                    && matchesFilter(filter.removeSuffix(" units").lowercase().replaceFirstChar { it.uppercaseChar() })
+                    && matchesFilter(filter.removeSuffix(" units").lowercase().replaceFirstChar { it.uppercaseChar() },
+                        multiFilter = false)
                 ) return true
-                return uniqueMap.contains(filter)
+                return false
             }
         }
     }
 
     /** Determine whether this is a City-founding unit - abstract, **without any game context**.
      *  Use other methods for MapUnits or when there is a better StateForConditionals available. */
-    fun isCityFounder() = hasUnique(UniqueType.FoundCity, StateForConditionals.IgnoreConditionals)
+    @Readonly fun isCityFounder() = hasUnique(UniqueType.FoundCity, GameContext.IgnoreConditionals)
 
-    fun isGreatPerson() = getMatchingUniques(UniqueType.GreatPerson).any()
-    fun isGreatPersonOfType(type: String) = getMatchingUniques(UniqueType.GreatPerson).any { it.params[0] == type }
+    val isGreatPerson by lazy { getMatchingUniques(UniqueType.GreatPerson).any() }
+    @Readonly fun isGreatPersonOfType(type: String) = getMatchingUniques(UniqueType.GreatPerson).any { it.params[0] == type }
 
     /** Has a MapUnit implementation that does not ignore conditionals, which should be usually used */
-    private fun isNuclearWeapon() = hasUnique(UniqueType.NuclearWeapon, StateForConditionals.IgnoreConditionals)
+    @Readonly private fun isNuclearWeapon() = hasUnique(UniqueType.NuclearWeapon, GameContext.IgnoreConditionals)
 
-    fun movesLikeAirUnits() = type.getMovementType() == UnitMovementType.Air
+    val movesLikeAirUnits by lazy { type.getMovementType() == UnitMovementType.Air }
 
     /** Returns resource requirements from both uniques and requiredResource field */
-    override fun getResourceRequirementsPerTurn(stateForConditionals: StateForConditionals?): Counter<String> {
+    override fun getResourceRequirementsPerTurn(state: GameContext?): Counter<String> {
         val resourceRequirements = Counter<String>()
         if (requiredResource != null) resourceRequirements[requiredResource!!] = 1
-        for (unique in getMatchingUniques(UniqueType.ConsumesResources, stateForConditionals))
-            resourceRequirements[unique.params[1]] += unique.params[0].toInt()
+        for (unique in getMatchingUniques(UniqueType.ConsumesResources, state ?: GameContext.EmptyState))
+            resourceRequirements.add(unique.params[1], unique.params[0].toInt())
         return resourceRequirements
     }
 
 
-    fun isRanged() = rangedStrength > 0
-    fun isMelee() = !isRanged() && strength > 0
-    fun isMilitary() = isRanged() || isMelee()
-    fun isCivilian() = !isMilitary()
+    @Readonly fun isRanged() = rangedStrength > 0
+    @Readonly fun isMelee() = !isRanged() && strength > 0
+    val isMilitary by lazy { isRanged() || isMelee() }
+    @Readonly fun isCivilian() = !isMilitary
 
-    private val isLandUnitInternal by lazy { type.isLandUnit() }
-    fun isLandUnit() = isLandUnitInternal
-    fun isWaterUnit() = type.isWaterUnit()
-    fun isAirUnit() = type.isAirUnit()
+    val isLandUnit by lazy { type.isLandUnit() }
+    val isWaterUnit by lazy { type.isWaterUnit() }
+    @Readonly fun isAirUnit() = type.isAirUnit()
 
+    @Readonly
     fun isProbablySiegeUnit() = isRanged()
-            && getMatchingUniques(UniqueType.Strength, StateForConditionals.IgnoreConditionals)
-                .any { it.params[0].toInt() > 0
-                    && it.conditionals.any { conditional -> conditional.type == UniqueType.ConditionalVsCity }
-                }
+            && getMatchingUniques(UniqueType.Strength, GameContext.IgnoreConditionals)
+                .any { it.params[0].toInt() > 0 && it.hasModifier(UniqueType.ConditionalVsCity) }
 
+
+    @Transient @Cache
+    private var cachedForceEvaluation: Int = -1
+    
+    @Readonly
     fun getForceEvaluation(): Int {
-        if (cachedForceEvaluation < 0) evaluateForce()
+        if (cachedForceEvaluation < 0) 
+            cachedForceEvaluation = evaluateForce()
         return cachedForceEvaluation
     }
 
-    private fun evaluateForce() {
-        if (strength == 0 && rangedStrength == 0) {
-            cachedForceEvaluation = 0
-            return
-        }
+    @Readonly
+    private fun evaluateForce(): Int {
+        if (strength == 0 && rangedStrength == 0) return 0
 
         var power = strength.toFloat().pow(1.5f)
         var rangedPower = rangedStrength.toFloat().pow(1.45f)
 
         // Value ranged naval units less
-        if (isWaterUnit()) {
+        if (isWaterUnit) {
             rangedPower /= 2
         }
         if (rangedPower > 0)
@@ -482,26 +536,33 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             power += 4000
 
         // Uniques
-        val allUniques = uniqueObjects.asSequence() +
+        val allUniques = rulesetUniqueObjects.asSequence() +
             promotions.asSequence()
                 .mapNotNull { ruleset.unitPromotions[it] }
                 .flatMap { it.uniqueObjects }
+        
+        // When we have multiple conditional strength bonuses, only the highest one counts
+        // Otherwise we get massive overvaluation of units with many conflicting conditionals
+        var highestConditionalPowerBonus = 1f
 
         for (unique in allUniques) {
             when (unique.type) {
                 UniqueType.Strength -> {
                     if (unique.params[0].toInt() <= 0) continue
-                    if (unique.conditionals.any { it.type == UniqueType.ConditionalVsUnits }) { // Bonus vs some units - a quarter of the bonus
-                        power *= (unique.params[0].toInt() / 4f).toPercent()
+                    
+                    if (unique.hasModifier(UniqueType.ConditionalVsUnits)) { // Bonus vs some units - a quarter of the bonus
+                        highestConditionalPowerBonus = (unique.params[0].toInt() / 4f).toPercent()
                     } else if (
-                        unique.conditionals.any {
+                        unique.modifiers.any {
                             it.type == UniqueType.ConditionalVsCity // City Attack - half the bonus
                                 || it.type == UniqueType.ConditionalAttacking // Attack - half the bonus
                                 || it.type == UniqueType.ConditionalDefending // Defense - half the bonus
                                 || it.type == UniqueType.ConditionalFightingInTiles
                         } // Bonus in terrain or feature - half the bonus
                     ) {
-                        power *= (unique.params[0].toInt() / 2f).toPercent()
+                        highestConditionalPowerBonus = (unique.params[0].toInt() / 2f).toPercent()
+                    } else {
+                        highestConditionalPowerBonus = (unique.params[0].toInt()).toPercent() // Static bonus
                     }
                 }
                 UniqueType.StrengthNearCapital ->
@@ -509,15 +570,16 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
                         power *= (unique.params[0].toInt() / 4f).toPercent()  // Bonus decreasing with distance from capital - not worth much most of the map???
 
                 UniqueType.MayParadrop // Paradrop - 25% bonus
-                    -> power += power / 4
+                    -> power *= 1.25f
                 UniqueType.MustSetUp // Must set up - 20 % penalty
-                    -> power -= power / 5
+                    -> power /= 1.20f
                 UniqueType.AdditionalAttacks // Extra attacks - 20% bonus per extra attack
-                    -> power += (power * unique.params[0].toInt()) / 5
+                    -> power *= (unique.params[0].toInt() * 20f).toPercent()
                 else -> {}
             }
         }
+        power *= highestConditionalPowerBonus
 
-        cachedForceEvaluation = power.toInt()
+        return power.toInt()
     }
 }
