@@ -5,16 +5,23 @@ import com.unciv.models.ruleset.RulesetObject
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
+import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.objectdescriptions.uniquesToCivilopediaTextLines
 import com.unciv.ui.objectdescriptions.uniquesToDescription
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
 import com.unciv.ui.screens.pickerscreens.PromotionPickerScreen
+import yairm210.purity.annotations.Pure
 
 class Promotion : RulesetObject() {
     var prerequisites = listOf<String>()
 
     var unitTypes = listOf<String>() // The json parser wouldn't agree to deserialize this as a list of UnitTypes. =(
 
+    var innerColor: List<Int>? = null
+    val innerColorObject by lazy { if (innerColor == null) null else colorFromRGB(innerColor!!)}
+    var outerColor: List<Int>? = null
+    val outerColorObject by lazy { if (outerColor == null) null else colorFromRGB(outerColor!!)}
+
     /** Used as **column** hint in the current [PromotionPickerScreen]
      *  This is no longer a direct position, it is used to sort before an automatic distribution.
      *  -1 determines that the modder has not set a position */
@@ -148,6 +155,13 @@ class Promotion : RulesetObject() {
         return textList
     }
 
+    override fun getSubCategory(ruleset: Ruleset): String? = unitTypes.firstOrNull() ?: "Other"
+    override fun getSortGroup(ruleset: Ruleset): Int {
+        val unitTypeName = unitTypes.firstOrNull() ?: return 1000
+        if (!ruleset.unitTypes.contains(unitTypeName)) return 1000
+        return ruleset.unitTypes.keys.indexOf(unitTypeName)
+    }
+
     companion object {
         data class PromotionBaseNameAndLevel(
             val nameWithoutBrackets: String,
@@ -159,6 +173,7 @@ class Promotion : RulesetObject() {
          *  Used by Portrait (where it only has the string, the Promotion object is forgotten) and
          *  PromotionPickerScreen. Here to allow clear "Promotion.getBaseNameAndLevel" signature.
          */
+        @Pure
         fun getBaseNameAndLevel(promotionName: String): PromotionBaseNameAndLevel {
             val nameWithoutBrackets = promotionName.replace("[", "").replace("]", "")
             val level = when {
