@@ -4,14 +4,10 @@ import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetObject
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.ui.objectdescriptions.BaseUnitDescriptions.getUnitTypeCivilopediaTextLines
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 
 
-enum class UnitLayer { // The layer in which the unit moves
-    Civilian,
-    Military,
-    Air
-}
-
 enum class UnitMovementType { // The types of tiles the unit can by default enter
     Land, // Only land tiles except when certain techs are researched
     Water, // Only water tiles
@@ -19,9 +15,8 @@ enum class UnitMovementType { // The types of tiles the unit can by default ente
 }
 
 class UnitType() : RulesetObject() {
-    private var movementType: String? = null
+    internal var movementType: String? = null
     private val unitMovementType: UnitMovementType? by lazy { if (movementType == null) null else UnitMovementType.valueOf(movementType!!) }
-
     override fun getUniqueTarget() = UniqueTarget.UnitType
     override fun makeLink() = "UnitType/$name"
 
@@ -30,35 +25,38 @@ class UnitType() : RulesetObject() {
         this.movementType = domain
     }
 
-    fun getMovementType() = unitMovementType
+    @Readonly fun getMovementType() = unitMovementType
 
-    fun isLandUnit() = unitMovementType == UnitMovementType.Land
-    fun isWaterUnit() = unitMovementType == UnitMovementType.Water
-    fun isAirUnit() = unitMovementType == UnitMovementType.Air
+    @Pure fun isLandUnit() = unitMovementType == UnitMovementType.Land
+    @Pure fun isWaterUnit() = unitMovementType == UnitMovementType.Water
+    @Pure fun isAirUnit() = unitMovementType == UnitMovementType.Air
 
     /** Implements [UniqueParameterType.UnitTypeFilter][com.unciv.models.ruleset.unique.UniqueParameterType.UnitTypeFilter] */
+    @Readonly
     fun matchesFilter(filter: String): Boolean {
         return when (filter) {
             "Land" -> isLandUnit()
             "Water" -> isWaterUnit()
             "Air" -> isAirUnit()
-            in uniqueMap -> true
-            else -> false
+            else -> hasTagUnique(filter)
         }
     }
 
     override fun getCivilopediaTextLines(ruleset: Ruleset) = getUnitTypeCivilopediaTextLines(ruleset)
-    override fun getSortGroup(ruleset: Ruleset): Int {
-        return if (name.startsWith("Domain: ")) 1 else 2
-    }
 
-    fun isUsed(ruleset: Ruleset) = ruleset.units.values.any { it.unitType == name }
+    /**
+     * Sort by the Domain, while keeping the Domain at the top of the sub-category.
+     */
+    override fun getSortGroup(ruleset: Ruleset): Int = (unitMovementType?.ordinal ?: 100) * 2 + (if (name.startsWith("Domain: ")) 0 else 1)
+    override fun getSubCategory(ruleset: Ruleset): String? = unitMovementType?.name ?: "Other"
+
+    @Readonly fun isUsed(ruleset: Ruleset) = ruleset.units.values.any { it.unitType == name }
 
     companion object {
         val City = UnitType("City", "Land")
 
         fun getCivilopediaIterator(ruleset: Ruleset): Collection<UnitType> {
-            return UnitMovementType.values().map {
+            return UnitMovementType.entries.map {
                 // Create virtual UnitTypes to describe the movement domains - Civilopedia only.
                 // It is important that the name includes the [] _everywhere_
                 // (here, CivilopediaImageGetters, links, etc.) so translation comes as cheap as possible.
