@@ -1,7 +1,7 @@
 package com.unciv.models.stats
 
 import com.unciv.models.translations.tr
-import kotlin.reflect.KMutableProperty0
+import yairm210.purity.annotations.*
 
 /**
  * A container for the seven basic ["currencies"][Stat] in Unciv,
@@ -11,6 +11,7 @@ import kotlin.reflect.KMutableProperty0
  *
  * Also possible: `<Stats>`.[values].sum() and similar aggregates over a Sequence<Float>.
  */
+@InternalState
 open class Stats(
     var production: Float = 0f,
     var food: Float = 0f,
@@ -21,31 +22,37 @@ open class Stats(
     var faith: Float = 0f
 ): Iterable<Stats.StatValuePair> {
 
-    // This is what facilitates indexed access by [Stat] or add(Stat,Float)
-    // without additional memory allocation or expensive conditionals
-    private fun statToProperty(stat: Stat): KMutableProperty0<Float> {
+    /** Indexed read of a value for a given [Stat], e.g. `this.gold == this[Stat.Gold]` */
+    @Readonly
+    operator fun get(stat: Stat): Float {
         return when(stat) {
-            Stat.Production -> ::production
-            Stat.Food -> ::food
-            Stat.Gold -> ::gold
-            Stat.Science -> ::science
-            Stat.Culture -> ::culture
-            Stat.Happiness -> ::happiness
-            Stat.Faith -> ::faith
+            Stat.Production -> production
+            Stat.Food -> food
+            Stat.Gold -> gold
+            Stat.Science -> science
+            Stat.Culture -> culture
+            Stat.Happiness -> happiness
+            Stat.Faith -> faith
         }
     }
-
-    /** Indexed read of a value for a given [Stat], e.g. `this.gold == this[Stat.Gold]` */
-    operator fun get(stat: Stat): Float {
-        return statToProperty(stat).get()
-    }
     /** Indexed write of a value for a given [Stat], e.g. `this.gold += 1f` is equivalent to `this[Stat.Gold] += 1f` */
-    operator fun set(stat: Stat, value: Float) = statToProperty(stat).set(value)
+    operator fun set(stat: Stat, value: Float) {
+        when(stat) {
+            Stat.Production -> production = value
+            Stat.Food -> food = value
+            Stat.Gold -> gold = value
+            Stat.Science -> science = value
+            Stat.Culture -> culture = value
+            Stat.Happiness -> happiness = value
+            Stat.Faith -> faith = value
+        }
+    }
 
     /** Compares two instances. Not callable via `==`. */
     // This is an overload, not an override conforming to the kotlin conventions of `equals(Any?)`,
     // so do not rely on it to be called for the `==` operator! A tad more efficient, though.
     @Suppress("CovariantEquals", "WrongEqualsTypeParameter")    // historical reasons to keep this function signature
+    @Readonly
     fun equals(otherStats: Stats): Boolean {
         return production == otherStats.production
                 && food == otherStats.food
@@ -58,9 +65,10 @@ open class Stats(
 
     /** **Non-Mutating function**
      * @return a new instance containing the same values as `this` */
-    fun clone() = Stats(production, food, gold, science, culture, happiness, faith)
+    @Readonly fun clone() = Stats(production, food, gold, science, culture, happiness, faith)
 
     /** @return `true` if all values are zero */
+    @Readonly
     fun isEmpty() = (
             production == 0f
             && food == 0f
@@ -81,7 +89,7 @@ open class Stats(
         faith = 0f
     }
 
-    /** **Mutating function**
+    /** **Mutating function** (but does **not** mutate [other])
      * Adds each value of another [Stats] instance to this one in place
      * @return this for chaining */
     fun add(other: Stats): Stats {
@@ -113,10 +121,11 @@ open class Stats(
 
     /** **Non-Mutating function**
      * @return a new [Stats] instance with the result of multiplying each value of this instance by [number] as a new instance */
-    operator fun times(number: Int) = times(number.toFloat())
+    @Readonly operator fun times(number: Int) = times(number.toFloat())
 
     /** **Non-Mutating function**
      * @return a new [Stats] instance with the result of multiplying each value of this instance by [number] as a new instance */
+    @Readonly
     operator fun times(number: Float) = Stats(
         production * number,
         food * number,
@@ -141,27 +150,28 @@ open class Stats(
 
     /** **Non-Mutating function**
      * @return a new [Stats] instance */
-    operator fun div(number: Float) = times(1/number)
+    @Readonly operator fun div(number: Float) = times(1/number)
 
     /** **Mutating function**
      * Apply weighting for Production Ranking */
     fun applyRankingWeights() {
         food *= 14
-        production *= 12
-        gold *= 8 // 3 gold worth about 2 production
-        science *= 7
-        culture *= 6
+        production *= 12.01f // tie break Production vs gold
+        gold *= 6 // 2 gold worth about 1 production
+        science *= 9.01f // 4 Science better than 3 Production
+        culture *= 8
         happiness *= 10 // base
-        faith *= 5
+        faith *= 7
     }
 
     /** ***Not*** only a debug helper. It returns a string representing the content, already _translated_.
      *
      * Example output: `+1 Production, -1 Food`.
      */
+    @Readonly
     override fun toString(): String {
         return this.joinToString {
-            (if (it.value > 0) "+" else "") + it.value.toInt() + " " + it.key.toString().tr()
+            (if (it.value > 0) "+" else "") + it.value.toInt().tr() + " " + it.key.toString().tr()
         }
     }
 
@@ -173,22 +183,24 @@ open class Stats(
     // For display in diplomacy window
     fun toStringWithDecimals(): String {
         return this.joinToString {
-            (if (it.value > 0) "+" else "") + it.value.toString().removeSuffix(".0") + " " + it.key.toString().tr()
+            (if (it.value > 0) "+" else "") + it.value.tr().removeSuffix(".0") + " " + it.key.toString().tr()
         }
     }
 
     // function that removes the icon from the Stats object since the circular icons all appear the same
     // delete this and replace above instances with toString() once the text-coloring-affecting-font-icons bug is fixed (e.g., in notification text)
+    @Readonly
     fun toStringWithoutIcons(): String {
         return this.joinToString {
-            it.value.toInt().toString() + " " + it.key.name.tr().substring(startIndex = 1)
+            it.value.toInt().tr() + " " + it.key.name.tr().substring(startIndex = 1)
         }
     }
 
     /** Return a string of just +/- value and Stat symbol*/
-    fun toStringOnlyIcons(): String {
+    @Readonly
+    fun toStringOnlyIcons(addPlusSign: Boolean = true): String {
         return this.joinToString {
-            (if (it.value > 0) "+" else "") + it.value.toInt() + " " + it.key.character
+            (if (addPlusSign && it.value > 0) "+" else "") + it.value.toInt() + " " + it.key.character
         }
     }
 
@@ -198,6 +210,7 @@ open class Stats(
     /** Enables iteration over the non-zero [Stat]/value [pairs][StatValuePair].
      * Explicit use unnecessary - [Stats] is [iterable][Iterable] directly.
      * @see iterator */
+    @Readonly
     fun asSequence() = sequence {
         if (production != 0f) yield(StatValuePair(Stat.Production, production))
         if (food != 0f) yield(StatValuePair(Stat.Food, food))
@@ -227,7 +240,7 @@ open class Stats(
 
 
     companion object {
-        private val allStatNames = Stat.values().joinToString("|") { it.name }
+        private val allStatNames = Stat.entries.joinToString("|") { it.name }
         private val statRegexPattern = "([+-])(\\d+) ($allStatNames)"
         private val statRegex = Regex(statRegexPattern)
         private val entireStringRegexPattern = Regex("$statRegexPattern(, $statRegexPattern)*")
@@ -240,6 +253,7 @@ open class Stats(
          * - Order is not important.
          * @see [parse]
          */
+        @Pure
         fun isStats(string: String): Boolean {
             if (string.isEmpty() || string[0] !in "+-") return false // very quick negative check before the heavy Regex
             return entireStringRegexPattern.matches(string)
@@ -252,13 +266,15 @@ open class Stats(
          * - Order is not important.
          * @see [isStats]
          */
+        @Pure
         fun parse(string: String): Stats {
             val toReturn = Stats()
             val statsWithBonuses = string.split(", ")
-            for(statWithBonuses in statsWithBonuses) {
+            statsWithBonuses.forEach { statWithBonuses ->
                 val match = statRegex.matchEntire(statWithBonuses)!!
-                val statName = match.groupValues[3]
-                val statAmount = match.groupValues[2].toFloat() * (if (match.groupValues[1] == "-") -1 else 1)
+                @Immutable val groupValues = match.groupValues
+                val statName = groupValues[3]
+                val statAmount = groupValues[2].toFloat() * (if (groupValues[1] == "-") -1 else 1)
                 toReturn.add(Stat.valueOf(statName), statAmount)
             }
             return toReturn
@@ -269,6 +285,7 @@ open class Stats(
     }
 }
 
+@InternalState
 class StatMap : LinkedHashMap<String,Stats>() {
     fun add(source: String, stats: Stats) {
         // We always clone to avoid touching the mutable stats of uniques
