@@ -2,12 +2,14 @@ package com.unciv.models.translations
 
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
+import com.unciv.UncivGame
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
+import com.unciv.logic.civilization.diplomacy.Demand
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
 import com.unciv.models.SpyAction
 import com.unciv.models.metadata.BaseRuleset
-import com.unciv.models.metadata.GameSettings.LocaleCode
+import com.unciv.models.metadata.LocaleCode
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.Event
@@ -28,12 +30,14 @@ import com.unciv.models.ruleset.tech.TechColumn
 import com.unciv.models.ruleset.tile.Terrain
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.tile.TileResource
+import com.unciv.models.ruleset.unique.Countables
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueFlag
 import com.unciv.models.ruleset.unique.UniqueParameterType
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
+import com.unciv.models.ruleset.unit.UnitNameGroup
 import com.unciv.models.ruleset.unit.Promotion
 import com.unciv.models.ruleset.unit.UnitType
 import com.unciv.ui.components.input.KeyboardBinding
@@ -42,11 +46,13 @@ import com.unciv.utils.debug
 import java.io.File
 import java.lang.reflect.Field
 import java.lang.reflect.Modifier
+import org.jetbrains.annotations.VisibleForTesting
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 
 object TranslationFileWriter {
 
     private const val specialNewLineCode = "# This is an empty line "
-    const val templateFileLocation = "jsons/translations/template.properties"
     private const val languageFileLocation = "jsons/translations/%s.properties"
     private const val shortDescriptionKey = "Fastlane_short_description"
     private const val shortDescriptionFile = "short_description.txt"
@@ -72,7 +78,7 @@ object TranslationFileWriter {
 
             // See #5168 for some background on this
             for ((modName, modTranslations) in translations.modsWithTranslations) {
-                val modFolder = Gdx.files.local("mods").child(modName)
+                val modFolder = UncivGame.Current.files.getModFolder(modName)
                 val modPercentages = generateTranslationFiles(modTranslations, modFolder, translations)
                 writeLanguagePercentages(modPercentages, modFolder)  // unused by the game but maybe helpful for the mod developer
             }
@@ -86,7 +92,7 @@ object TranslationFileWriter {
 
     private fun getFileHandle(modFolder: FileHandle?, fileLocation: String) =
             if (modFolder != null) modFolder.child(fileLocation)
-            else Gdx.files.local(fileLocation)
+            else UncivGame.Current.files.getLocalFile(fileLocation)
 
     /**
      * Writes new language files per Mod or for BaseRuleset - only each language that exists in [translations].
@@ -103,12 +109,12 @@ object TranslationFileWriter {
         val linesToTranslate = mutableListOf<String>()
 
         if (modFolder == null) { // base game
-            val templateFile = getFileHandle(null, templateFileLocation) // read the template
-            if (templateFile.exists())
-                linesToTranslate.addAll(templateFile.reader(TranslationFileReader.charset).readLines())
+            TranslationFileReader.readTemplates {
+                linesToTranslate.addAll(it)
+            }
 
             linesToTranslate += "\n\n#################### Lines from Unique Types #######################\n"
-            for (uniqueType in UniqueType.values()) {
+            for (uniqueType in UniqueType.entries) {
                 val deprecationAnnotation = uniqueType.getDeprecationAnnotation()
                 if (deprecationAnnotation != null) continue
                 if (uniqueType.flags.contains(UniqueFlag.HiddenToUsers)) continue
@@ -116,43 +122,53 @@ object TranslationFileWriter {
                 linesToTranslate += "${uniqueType.getTranslatable()} = "
             }
 
-            for (uniqueParameterType in UniqueParameterType.values()) {
+            for (uniqueParameterType in UniqueParameterType.entries) {
                 val strings = uniqueParameterType.getTranslationWriterStringsForOutput()
                 if (strings.isEmpty()) continue
                 linesToTranslate += "\n######### ${uniqueParameterType.displayName} ###########\n"
                 linesToTranslate.addAll(strings.map { "$it = " })
             }
 
-            for (uniqueTarget in UniqueTarget.values())
+            for (uniqueTarget in UniqueTarget.entries)
                 linesToTranslate += "$uniqueTarget = "
 
+            linesToTranslate += "\n\n#################### Lines from Countables #######################\n"
+            for (countable in Countables.entries)
+                if (countable.text.isNotEmpty())
+                    linesToTranslate += "${countable.text} = "
+
             linesToTranslate += "\n\n#################### Lines from spy actions #######################\n"
-            for (spyAction in SpyAction.values()) {
+            for (spyAction in SpyAction.entries)
                 linesToTranslate += "${spyAction.displayString} = "
-            }
 
             linesToTranslate += "\n\n#################### Lines from diplomatic modifiers #######################\n"
-            for (diplomaticModifier in DiplomaticModifiers.values())
+            for (diplomaticModifier in DiplomaticModifiers.entries)
                 linesToTranslate += "${diplomaticModifier.text} = "
 
-            linesToTranslate += "\n\n#################### Lines from key bindings #######################\n"
-            for (category in KeyboardBinding.Category.values()) {
-                linesToTranslate += "${category.label} = "
-            }
-            for (binding in KeyboardBinding.values()) {
-                linesToTranslate += "${binding.label} = "
+            linesToTranslate += "\n\n#################### Lines from demands #######################\n"
+            for (demand in Demand.entries) {
+                linesToTranslate += "\n### ${demand.name} \n"
+                val uiTexts = listOf(demand.demandText, demand.acceptDemandText, demand.refuseDemandText,
+                    demand.violationNoticedText, demand.agreedToDemandText, demand.refusedDemandText,
+                    demand.wePromisedText, demand.theyPromisedText)
+                for (text in uiTexts)
+                    linesToTranslate += "$text = "
             }
 
-            for (baseRuleset in BaseRuleset.values()) {
+            linesToTranslate += "\n\n#################### Lines from key bindings #######################\n"
+            for (bindingLabel in KeyboardBinding.getTranslationEntries())
+                linesToTranslate += "$bindingLabel = "
+
+            for (baseRuleset in BaseRuleset.entries) {
                 val generatedStringsFromBaseRuleset =
-                        GenerateStringsFromJSONs(Gdx.files.local("jsons/${baseRuleset.fullName}"))
+                        GenerateStringsFromJSONs(UncivGame.Current.files.getLocalFile("jsons/${baseRuleset.fullName}"))
                 for (entry in generatedStringsFromBaseRuleset)
                     fileNameToGeneratedStrings[entry.key + " from " + baseRuleset.fullName] = entry.value
             }
 
-            // Tutorials reside one level above the base rulesets - if they were per-ruleset the following lines would be unnecessary
-            val tutorialStrings = GenerateStringsFromJSONs(Gdx.files.local("jsons")) { it.name == "Tutorials.json" }
-            fileNameToGeneratedStrings["Tutorials"] = tutorialStrings.values.first()
+            // Global Tutorials reside one level above the base rulesets - if we had only per-ruleset tutorials the following lines would be unnecessary
+            val tutorialStrings = GenerateStringsFromJSONs(UncivGame.Current.files.getLocalFile("jsons")) { it.name == "Tutorials.json" }
+            fileNameToGeneratedStrings["Global Tutorials"] = tutorialStrings.values.first()
         } else {
             fileNameToGeneratedStrings.putAll(GenerateStringsFromJSONs(modFolder.child("jsons")))
         }
@@ -185,30 +201,41 @@ object TranslationFileWriter {
                     continue
                 }
 
-                val translationKey = line.split(" = ")[0].replace("\\n", "\n")
-                val hashMapKey =
-                    if (translationKey == Translations.englishConditionalOrderingString)
-                        Translations.englishConditionalOrderingString
-                    else translationKey
+                val lineParts = line.split(" = ")
+                val translationKey = lineParts[0].replace("\\n", "\n")
+                val hashMapKey = translationKey
                         .replace(pointyBraceRegex, "")
                         .replace(squareBraceRegex, "[]")
 
                 if (existingTranslationKeys.contains(hashMapKey)) continue // don't add it twice
                 existingTranslationKeys.add(hashMapKey)
 
-                // count translatable lines only once
-                if (languageIndex == 0) countOfTranslatableLines++
+                fun incrementCountOfTranslatableLines() {
+                    // count translatable lines only once
+                    if (languageIndex == 0) countOfTranslatableLines++
+                }
 
+                val isPretranslatable = lineParts[1].isNotEmpty()
                 val existingTranslation = translations[hashMapKey]
                 var translationValue = if (existingTranslation != null && language in existingTranslation) {
+                    // String has translation - copy it
+                    incrementCountOfTranslatableLines()
                     translationsOfThisLanguage++
                     existingTranslation[language]!!
                 } else if (baseTranslations?.get(hashMapKey)?.containsKey(language) == true) {
                     // String is used in the mod but also exists in base - ignore
                     continue
+                } else if (isPretranslatable) {
+                    // We could omit the following two lines so pre-translatables would not count towards completion percentages at all
+                    incrementCountOfTranslatableLines()
+                    translationsOfThisLanguage++
+                    lineParts[1]
                 } else {
                     // String is not translated either here or in base
-                    stringBuilder.appendLine(" # Requires translation!")
+                    if (translationKey != Translations.conditionalOrderingKey) {
+                        incrementCountOfTranslatableLines()
+                        stringBuilder.appendLine(" # Requires translation!")
+                    }
                     ""
                 }
 
@@ -249,20 +276,22 @@ object TranslationFileWriter {
         return countOfTranslatedLines
     }
 
+    @Pure
     private fun StringBuilder.appendTranslation(key: String, value: String) {
         appendLine(key.replace("\n", "\\n") +
                 " = " + value.replace("\n", "\\n"))
     }
 
     private fun writeLanguagePercentages(percentages: HashMap<String, Int>, modFolder: FileHandle? = null) {
+        val comment = "# Automatically generated - ${if (modFolder == null) "do not edit manually" else "for your info only"}\n\n"
         val output = percentages.asSequence()
             .sortedBy { it.key }
-            .joinToString("\n", postfix = "\n") { "${it.key} = ${it.value}" }
+            .joinToString("\n", prefix = comment, postfix = "\n") { "${it.key} = ${it.value}" }
         getFileHandle(modFolder, TranslationFileReader.percentagesFileLocation)
             .writeString(output, false)
     }
 
-    // used for unit test only
+    @VisibleForTesting
     fun getGeneratedStringsSize(): Int {
         return GenerateStringsFromJSONs(Gdx.files.local("jsons/Civ V - Vanilla")).values.sumOf {
             // exclude empty lines
@@ -318,18 +347,22 @@ object TranslationFileWriter {
                 val javaClass = getJavaClassByName(filename)
                     ?: continue // unknown JSON, let's skip it
 
-                val array = json().fromJsonFile(javaClass, jsonFile.path())
+                val data = json().fromJsonFile(javaClass, jsonFile.path())
 
                 resultStrings = mutableSetOf()
                 this[filename] = resultStrings
 
                 @Suppress("RemoveRedundantQualifierName")  // to clarify this is the kotlin way
-                if (array is kotlin.Array<*>)
-                    for (element in array) {
+                if (data is kotlin.Array<*>) {
+                    for (element in data) {
                         serializeElement(element!!) // let's serialize the strings recursively
                         // This is a small hack to insert multiple /n into the set, which can't contain identical lines
                         resultStrings.add("$specialNewLineCode ${uniqueIndexOfNewLine++}")
                     }
+                } else {
+                    serializeElement(data)
+                    resultStrings.add("$specialNewLineCode ${uniqueIndexOfNewLine++}")
+                }
             }
             val displayName = if (jsonsFolder.name() != "jsons") jsonsFolder.name()
                 else jsonsFolder.parent().name().ifEmpty { "Tutorials" }  // Show mod name - or special case
@@ -349,12 +382,17 @@ object TranslationFileWriter {
             resultStrings.add("$string = ")
         }
 
+        fun submitPretranslatableString(string: String) {
+            if (string.isEmpty()) return
+            resultStrings.add("$string = $string")
+        }
+
         fun submitString(string: String, unique: Unique) {
             if (unique.isHiddenToUsers())
                 return // We don't need to translate this at all, not user-visible
 
             val stringToTranslate = string.removeConditionals()
-            for (conditional in unique.conditionals) {
+            for (conditional in unique.modifiers) {
                 submitString(conditional.text, conditional)
             }
 
@@ -422,22 +460,29 @@ object TranslationFileWriter {
                 if (element is Tutorial && field.name == "name"
                         && UniqueType.HiddenFromCivilopedia.placeholderText in element.uniques)
                     continue
+                val isPreTranslatable = isFieldPreTranslatable(element.javaClass, field)
+                fun submitCollectionItem(item: Any?) {
+                    if (item === null) return
+                    if (item !is String) serializeElement(item)
+                    else if (isPreTranslatable) submitPretranslatableString(item)
+                    else submitString(item)
+                }
                 // this field can contain sub-objects, let's serialize them as well
                 @Suppress("RemoveRedundantQualifierName")  // to clarify List does _not_ inherit from anything in java.util
                 when {
                     // Promotion names are not uniques but since we did the "[unitName] ability"
                     // they need the "parameters" treatment too
                     // Same for victory milestones
-                    (field.name == "uniques" || field.name == "promotions" || field.name == "milestones")
+                    (field.name in fieldsToProcessParameters)
                             && (fieldValue is java.util.AbstractCollection<*>) ->
                         for (item in fieldValue)
                             if (item is String) submitString(item, Unique(item)) else serializeElement(item!!)
                     fieldValue is java.util.AbstractCollection<*> ->
                         for (item in fieldValue)
-                            if (item is String) submitString(item) else serializeElement(item!!)
+                            submitCollectionItem(item)
                     fieldValue is kotlin.collections.List<*> ->
                         for (item in fieldValue)
-                            if (item is String) submitString(item) else serializeElement(item!!)
+                            submitCollectionItem(item)
                     element is Promotion && field.name == "name" ->  // see above
                         submitString(fieldValue.toString(), Unique(fieldValue.toString()))
                     else -> submitString(fieldValue.toString())
@@ -464,6 +509,21 @@ object TranslationFileWriter {
                 "RuinReward.uniques", "TerrainType.name",
                 "CityStateType.friendBonusUniques", "CityStateType.allyBonusUniques",
                 "Era.citySound",
+                "keyShortcut",
+                "originRuleset"
+            )
+
+            /** Fields that are collections of simple strings that will be offered pre-translated.
+             *  Matching rules as in [untranslatableFieldSet].
+             *
+             *  Note This is determined in [GenerateStringsFromJSONs], then communicated to [generateTranslationFiles]
+             *  by emitting e.g. 'Paris = Paris' instead of 'Paris = '. [generateTranslationFiles] then detects that and ensures
+             *  it's written out like that and not commented with " # Requires translation!".
+             */
+            private val preTranslatableFieldSet = setOf(
+                "Nation.cities",
+                "Nation.spyNames",
+                "UnitNameGroup.unitNames"
             )
 
             /** Specifies Enums where the name property _is_ translatable, by Class name */
@@ -476,6 +536,11 @@ object TranslationFileWriter {
                 UniqueParameterType.Comment
             )
 
+            private val fieldsToProcessParameters = setOf(
+                "uniques", "promotions", "milestones",
+            )
+
+            @Readonly
             private fun isFieldTypeRelevant(type: Class<*>) =
                     type == String::class.java ||
                     type == java.util.ArrayList::class.java ||
@@ -496,6 +561,14 @@ object TranslationFileWriter {
                         (clazz.componentType?.simpleName ?: clazz.simpleName) + "." + field.name !in untranslatableFieldSet
             }
 
+            /** Checks whether a field's content should be marked as "pre-translatable", meaning if it's not yet translated for a language,
+             *  then `thing = thing` gets written instead of `# requires translation`...
+             *  ONLY applies to collections of simple strings which can't contain {} placeholders.
+             */
+            private fun isFieldPreTranslatable(clazz: Class<*>, field: Field) =
+                field.name in preTranslatableFieldSet ||
+                (clazz.componentType?.simpleName ?: clazz.simpleName) + "." + field.name in preTranslatableFieldSet
+
             private fun getJavaClassByName(name: String): Class<Any>? {
                 return when (name) {
                     "Beliefs" -> emptyArray<Belief>().javaClass
@@ -505,6 +578,7 @@ object TranslationFileWriter {
                     "Eras" -> emptyArray<Era>().javaClass
                     "Events" -> emptyArray<Event>().javaClass
                     "GlobalUniques" -> GlobalUniques().javaClass
+                    "UnitNameGroups" -> emptyArray<UnitNameGroup>().javaClass
                     "Nations" -> emptyArray<Nation>().javaClass
                     "Policies" -> emptyArray<PolicyBranch>().javaClass
                     "Quests" -> emptyArray<Quest>().javaClass
@@ -556,8 +630,7 @@ object TranslationFileWriter {
                 !endWithNewline && translated.endsWith('\n') -> translated.removeSuffix("\n")
                 else -> translated
             }
-            val localeCode = LocaleCode.valueOf(language.replace("_",""))
-            val path = fastlanePath + (localeCode.trueLanguage ?: localeCode.language)
+            val path = fastlanePath + LocaleCode.fastlaneFolder(language)
             File(path).mkdirs()
             File(path + File.separator + fileName).writeText(fileContent)
         }
