@@ -3,14 +3,21 @@ package com.unciv.models.translations
 import com.badlogic.gdx.Gdx
 import com.unciv.Constants
 import com.unciv.UncivGame
+import com.unciv.models.metadata.LocaleCode
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
+import com.unciv.ui.components.fonts.DiacriticSupport
 import com.unciv.ui.components.fonts.FontRulesetIcons
 import com.unciv.utils.Log
 import com.unciv.utils.debug
 import java.util.Locale
+import org.jetbrains.annotations.VisibleForTesting
+import yairm210.purity.annotations.Immutable
+import yairm210.purity.annotations.LocalState
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 
 /**
  *  This collection holds all translations for the game.
@@ -41,7 +48,6 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
     // used by tr() whenever GameInfo not initialized (allowing new game screen to use mod translations)
     var translationActiveMods = LinkedHashSet<String>()
 
-
     /**
      * Searches for the translation entry of a given [text] for a given [language].
      * This includes translations provided by mods from [activeMods]
@@ -52,6 +58,7 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
      *
      * @return the translation entry or null when not available
      */
+    @Readonly
     fun get(text: String, language: String, activeMods: HashSet<String>? = null): TranslationEntry? {
         if (activeMods != null)
             for (activeMod in activeMods) {
@@ -65,12 +72,22 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
     }
 
     /**
+     * Returns a specific translation, or the original [text] if no translation for [language] exists
      * @see get
      */
+    @Readonly
     fun getText(text: String, language: String, activeMods: HashSet<String>? = null): String {
         return get(text, language, activeMods)?.get(language) ?: text
     }
 
+    /**
+     * Returns a specific translation, or [default] if no translation for [language] exists
+     */
+    @Readonly
+    fun getText(text: String, language: String, activeMods: HashSet<String>? = null, default: String): String {
+        return get(text, language, activeMods)?.get(language) ?: default
+    }
+
     /** Get all languages present in `this`, used for [TranslationFileWriter] and `TranslationTests` */
     fun getLanguages() = linkedSetOf<String>().apply {
             for (entry in values)
@@ -81,7 +98,7 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
     /** This reads all translations for a specific language, including _all_ installed mods.
      *  Vanilla translations go into `this` instance, mod translations into [modsWithTranslations].
      */
-    private fun tryReadTranslationForLanguage(language: String) {
+    private fun tryReadTranslationForLanguage(language: String, noDiacritics: Boolean = false) {
         val translationStart = System.currentTimeMillis()
 
         val translationFileName = "jsons/translations/$language.properties"
@@ -106,34 +123,40 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
                     modsWithTranslations[modFolder.name()] = translationsForMod
                 }
                 try {
-                    translationsForMod.createTranslations(language, TranslationFileReader.read(modTranslationFile))
+                    translationsForMod.createTranslations(language, TranslationFileReader.read(modTranslationFile), noDiacritics)
                 } catch (ex: Exception) {
                     Log.error("Exception reading translations for ${modFolder.name()} $language", ex)
                 }
             }
         }
 
-        createTranslations(language, languageTranslations)
+        createTranslations(language, languageTranslations, noDiacritics)
 
         debug("Loading translation file for %s - %sms", language, System.currentTimeMillis() - translationStart)
     }
 
-    private fun createTranslations(language: String, languageTranslations: HashMap<String,String>) {
-        for (translation in languageTranslations) {
-            val hashKey = if (translation.key.contains('[') && !translation.key.contains('<'))
-                translation.key.getPlaceholderText()
-            else translation.key
+    @VisibleForTesting
+    fun createTranslations(language: String, languageTranslations: HashMap<String, String>, noDiacritics: Boolean = false) {
+        val diacriticSupport = if (noDiacritics) null
+            else DiacriticSupport(languageTranslations).takeIf { it.isEnabled() }
+        for ((key, value) in languageTranslations) {
+            val hashKey = if (key.contains('[') && !key.contains('<'))
+                key.getPlaceholderText()
+            else key
             var entry = this[hashKey]
             if (entry == null) {
-                entry = TranslationEntry(translation.key)
+                entry = TranslationEntry(key)
                 this[hashKey] = entry
             }
-            entry[language] = translation.value
+            entry[language] = diacriticSupport?.remapDiacritics(value) ?: value
         }
     }
 
+
     fun tryReadTranslationForCurrentLanguage() {
+        DiacriticSupport.reset()
         tryReadTranslationForLanguage(UncivGame.Current.settings.language)
+        DiacriticSupport.freeTranslationData()
     }
 
     /** Get a list of supported languages for [readAllLanguagesTranslation] */
@@ -161,12 +184,19 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
 
         languages.remove("template")
         languages.remove("completionPercentages")
-        languages.remove("Thai") // Spacing looks horrible, so disable until we figure out what to do with it
 
         return languages.filter { Gdx.files.internal("jsons/translations/$it.properties").exists() }
     }
 
-    /** Ensure _all_ languages are loaded, used by [TranslationFileWriter] and `TranslationTests` */
+    /** Ensure _all_ languages are loaded, used by [TranslationFileWriter] and `TranslationTests` only.
+     *
+     *  #### Notes:
+     *  -  Expects to run on a newly created instance.
+     *  -  Loads the translations with no diacritic mapping, so what we read will be what we write
+     *     (otherwise we would write out the fake alphabet-conversions, a one-way destructive mistake).
+     *  -  Relies on usage by TFW and tests only, if the result is ever meant to support translations that are actually displayed, a refactor will be needed.
+     *  -  Does not clear the fake alphabet possibly present in DiacriticSupport, but will not use it either.
+     */
     fun readAllLanguagesTranslation() {
         // Apparently you can't iterate over the files in a directory when running out of a .jar...
         // https://www.badlogicgames.com/forum/viewtopic.php?f=11&t=27250
@@ -175,7 +205,7 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
         val translationStart = System.currentTimeMillis()
 
         for (language in getLanguagesWithTranslationFile()) {
-            tryReadTranslationForLanguage(language)
+            tryReadTranslationForLanguage(language, noDiacritics = true)
         }
 
         debug("Loading translation files - %sms", System.currentTimeMillis() - translationStart)
@@ -189,40 +219,41 @@ class Translations : LinkedHashMap<String, TranslationEntry>() {
         debug("Loading percent complete of languages - %sms", System.currentTimeMillis() - startTime)
     }
 
-    fun getConditionalOrder(language: String): String {
-        return getText(englishConditionalOrderingString, language, null)
-    }
+    @Readonly
+    fun getConditionalOrder(language: String) =
+        getText(conditionalOrderingKey, language, null, defaultConditionalOrderingString)
 
-    fun placeConditionalsAfterUnique(language: String): Boolean {
-        if (get(conditionalUniqueOrderString, language, null)?.get(language) == "before")
-            return false
-        return true
-    }
+    @Readonly
+    fun placeConditionalsAfterUnique(language: String) =
+        getText(conditionalPlacementKey, language, null, "") != "before"
 
     /** Returns the equivalent of a space in the given language
      * Defaults to a space if no translation is provided
      */
-    fun getSpaceEquivalent(language: String): String {
-        val translation = getText("\" \"", language, null)
-        return translation.substring(1, translation.length-1)
-    }
+    @Readonly
+    fun getSpaceEquivalent(language: String) =
+        getText("\" \"", language, null).removeSurrounding("\"")
 
-    fun shouldCapitalize(language: String): Boolean {
-        return get(shouldCapitalizeString, language, null)?.get(language)?.toBoolean() ?: true
-    }
+    @Readonly
+    fun shouldCapitalize(language: String) =
+        getText(shouldCapitalizeKey, language, null, "true").toBoolean()
 
-    fun triggerNotificationEffectBeforeCause(language: String): Boolean{
-        return get(effectBeforeCause, language, null)?.get(language)?.toBoolean() ?: true
-    }
+    @Readonly
+    fun triggerNotificationEffectBeforeCause(language: String) =
+        getText(effectBeforeCauseKey, language, null, "true").toBoolean()
 
     companion object {
-        // Whenever this string is changed, it should also be changed in the translation files!
-        // It is mostly used as the template for translating the order of conditionals
-        const val englishConditionalOrderingString =
-            "<with a garrison> <for [mapUnitFilter] units> <above [amount] HP> <below [amount] HP> <vs cities> <vs [mapUnitFilter] units> <when fighting in [tileFilter] tiles> <when attacking> <when defending> <if this city has at least [amount] specialists> <when at war> <when not at war> <while the empire is happy> <during a Golden Age> <during the [era]> <starting from the [era]> <before the [era]> <with [techOrPolicy]> <without [techOrPolicy]>"
-        const val conditionalUniqueOrderString = "ConditionalsPlacement"
-        const val shouldCapitalizeString = "StartWithCapitalLetter"
-        const val effectBeforeCause = "EffectBeforeCause"
+        @VisibleForTesting
+        const val conditionalOrderingKey = "ConditionalsOrder"
+        @VisibleForTesting
+        const val defaultConditionalOrderingString =
+            "<with a garrison> <for [mapUnitFilter] units> <when above [amount] HP> <when below [amount] HP> <vs cities> <vs [mapUnitFilter] units> <when fighting in [tileFilter] tiles> <when attacking> <when defending> <if this city has at least [amount] specialists> <when at war> <when not at war> <while the empire is happy> <during a Golden Age> <during the [era]> <starting from the [era]> <before the [era]> <with [techOrPolicy]> <without [techOrPolicy]>"
+        @VisibleForTesting
+        const val conditionalPlacementKey = "ConditionalsPlacement"
+        @VisibleForTesting
+        const val shouldCapitalizeKey = "StartWithCapitalLetter"
+        @VisibleForTesting
+        const val effectBeforeCauseKey = "EffectBeforeCause"
     }
 }
 
@@ -244,6 +275,9 @@ val curlyBraceRegex = Regex("""\{([^}]*)\}""")
 @Suppress("RegExpRedundantEscape") // Some Android versions need ]}) escaped
 val pointyBraceRegex = Regex("""\<([^>]*)\>""")
 
+// Used to match continous digits 0, 12, 1232 etc
+@Suppress("RegExpRedundantEscape") // Some Android versions need ]}) escaped
+val digitsRegex = Regex("""\d""")
 
 object TranslationActiveModsCache {
     private var cachedHash = Int.MIN_VALUE
@@ -294,138 +328,153 @@ object TranslationActiveModsCache {
  *                  - phrases between curly braces are translated individually
  *                  Additionally, they may contain conditionals between '<' and '>'
  *  @param      hideIcons disables auto-inserting icons for ruleset objects (but not Stats)
+ *  @param      hideStats disables auto-inserting icons for Stats (but not rulset objects)
  *  @return     The translated string
  *                  defaults to the input string if no translation is available,
  *                  but with placeholder or sentence brackets removed.
  */
-fun String.tr(hideIcons: Boolean = false): String {
+@Readonly
+fun String.tr(hideIcons: Boolean = false, hideStats: Boolean = false): String {
     val language: String = UncivGame.Current.settings.language
 
     // '<' and '>' checks for quick 'no' answer, regex to ensure that no one accidentally put '><' and ruined things
-    if (contains('<') && contains('>') && pointyBraceRegex.containsMatchIn(this)) { // Conditionals!
-        /**
-         * So conditionals can contain placeholders, such as <vs [unitFilter] units>, which themselves
-         * can contain multiple filters, such as <vs [{Military} {Water}] units>.
-         * Moreover, we can have any amount of conditionals in any order, and translations
-         * can reorder these conditionals in any way they like, even putting them in front
-         * of the rest of the translatable string.
-         * All of this nesting makes it quite difficult to translate, and is the reason we check
-         * for these first.
-         *
-         * The plan: First translate each of the conditionals on its own, and then combine them
-         * together into the final fully translated string.
-         */
-
-        var translatedBaseUnique = this.removeConditionals().tr(hideIcons)
-
-        val conditionals = this.getConditionals().map { it.placeholderText }
-        val conditionsWithTranslation: LinkedHashMap<String, String> = linkedMapOf()
-
-        for (conditional in this.getConditionals())
-            conditionsWithTranslation[conditional.placeholderText] = conditional.text.tr(hideIcons)
-
-        val translatedConditionals: MutableList<String> = mutableListOf()
-
-        // Somewhere, we asked the translators to reorder all possible conditionals in a way that
-        // makes sense in their language. We get this ordering, and than extract each of the
-        // translated conditionals, removing the <> surrounding them, and removing param values
-        // where it exists.
-        val conditionalOrdering = UncivGame.Current.translations.getConditionalOrder(language)
-        for (placedConditional in pointyBraceRegex.findAll(conditionalOrdering)
-            .map { it.value.substring(1, it.value.length - 1).getPlaceholderText() }) {
-            if (placedConditional in conditionals) {
-                translatedConditionals.add(conditionsWithTranslation[placedConditional]!!)
-                conditionsWithTranslation.remove(placedConditional)
-            }
-        }
-
-        // If the translated string that should contain all conditionals doesn't contain
-        // a few conditionals used here, just add the translations of these to the end.
-        // We do test for this, but just in case.
-        translatedConditionals.addAll(conditionsWithTranslation.values)
-
-        // After that, add the translation of the base unique either before or after these conditionals
-        if (UncivGame.Current.translations.placeConditionalsAfterUnique(language)) {
-            translatedConditionals.add(0, translatedBaseUnique)
-        } else {
-            if (UncivGame.Current.translations.shouldCapitalize(language) && translatedBaseUnique[0].isUpperCase())
-                translatedBaseUnique = translatedBaseUnique.replaceFirstChar { it.lowercase() }
-            translatedConditionals.add(translatedBaseUnique)
-        }
-
-        var fullyTranslatedString = translatedConditionals.joinToString(
-            UncivGame.Current.translations.getSpaceEquivalent(language)
-        )
-        if (UncivGame.Current.translations.shouldCapitalize(language))
-            fullyTranslatedString = fullyTranslatedString.replaceFirstChar { it.uppercase() }
-        return fullyTranslatedString
+    if (contains('<') && contains('>') && pointyBraceRegex.containsMatchIn(this)) {
+        return translateConditionals(hideIcons, language)
     }
 
     // curly and square brackets can be nested inside of each other so find the leftmost curly/square
     // bracket then process that first
     val indexSquare = this.indexOf('[')
     val indexCurly = this.indexOf('{')
-    val processSquare = indexSquare >= 0 && (indexCurly < 0 || indexSquare < indexCurly)
-    val processCurly =  indexCurly >= 0 && (indexSquare < 0 || indexCurly < indexSquare)
 
-    // There might still be optimization potential here!
-    if (processSquare) { // Placeholders!
-        /**
-         * I'm SURE there's an easier way to do this but I can't think of it =\
-         * So what's all this then?
-         * Well, not all languages are like English. So say I want to say "work on Library has completed in Akkad",
-         * but in a completely different language like Japanese or German,
-         * It could come out "Akkad hast die worken onner Library gerfinishen" or whatever,
-         * basically, the order of the words in the sentence is not guaranteed.
-         * So to translate this, I give a sentence like "work on [building] has completed in [city]"
-         * and the german can put those placeholders where he wants, so  "[city] hast die worken onner [building] gerfinishen"
-         * The string on which we call tr() will look like "work on [library] has completed in [Akkad]"
-         * We will find the german placeholder text, and replace the placeholders with what was filled in the text we got!
-         */
+    val squareBracketsEncounteredFirst = indexSquare >= 0 && indexCurly !in 0..indexSquare
+    val curlyBracketsEncounteredFirst =  indexCurly >= 0 && indexSquare !in 0..indexCurly
 
-        // Convert "work on [building] has completed in [city]" to "work on [] has completed in []"
-        val translationStringWithSquareBracketsOnly = this.getPlaceholderText()
-        // That is now the key into the translation HashMap!
-        val translationEntry = UncivGame.Current.translations
-            .get(translationStringWithSquareBracketsOnly, language, TranslationActiveModsCache.activeMods)
+    if (squareBracketsEncounteredFirst)
+        return translatePlaceholders(language, hideIcons)
 
-        var languageSpecificPlaceholder: String
-        val originalEntry: String
-        if (translationEntry == null || !translationEntry.containsKey(language)) {
-            // Translation placeholder doesn't exist for this language, default to English
-            languageSpecificPlaceholder = this
-            originalEntry = this
-        } else {
-            languageSpecificPlaceholder = translationEntry[language]!!
-            originalEntry = translationEntry.entry
-        }
-
-        // Take the terms in the message, WITHOUT square brackets
-        val termsInMessage = this.getPlaceholderParameters()
-        // Take the terms from the placeholder
-        val termsInTranslationPlaceholder = originalEntry.getPlaceholderParameters()
-        if (termsInMessage.size != termsInTranslationPlaceholder.size)
-            throw Exception("Message $this has a different number of terms than the placeholder $translationEntry!")
-
-        for (i in termsInMessage.indices) {
-            languageSpecificPlaceholder = languageSpecificPlaceholder.replace(
-                "[${termsInTranslationPlaceholder[i]}]", // re-add square brackets to placeholder terms
-                termsInMessage[i].tr(hideIcons)
-            )
-        }
-        return languageSpecificPlaceholder      // every component is already translated
-    }
-
-    if (processCurly) { // Translating partial sentences
+    if (curlyBracketsEncounteredFirst) // Translating partial sentences
         return curlyBraceRegex.replace(this) { it.groups[1]!!.value.tr(hideIcons) }
+
+    return translateIndividualWord(language, hideIcons, hideStats)
+}
+
+
+@Readonly
+private fun String.translateConditionals(hideIcons: Boolean, language: String): String {
+    /**
+     * So conditionals can contain placeholders, such as <vs [unitFilter] units>, which themselves
+     * can contain multiple filters, such as <vs [{Military} {Water}] units>.
+     * Moreover, we can have any amount of conditionals in any order, and translations
+     * can reorder these conditionals in any way they like, even putting them in front
+     * of the rest of the translatable string.
+     * All of this nesting makes it quite difficult to translate, and is the reason we check
+     * for these first.
+     *
+     * The plan: First translate each of the conditionals on its own, and then combine them
+     * together into the final fully translated string.
+     */
+
+    // Get all special parameters stored in the translation files
+    val conditionalOrdering = UncivGame.Current.translations.getConditionalOrder(language)
+    val conditionalsAfterUnique = UncivGame.Current.translations.placeConditionalsAfterUnique(language)
+    val shouldCapitalize = UncivGame.Current.translations.shouldCapitalize(language)
+    val spaceEquivalent = UncivGame.Current.translations.getSpaceEquivalent(language)
+
+    // Somewhere, we asked the translators to reorder all possible conditionals in a way that
+    // makes sense in their language. We get this ordering, and map their placeholderText to their ordinal.
+    val sortPriorities: Map<String, Int> = pointyBraceRegex
+        .findAll(conditionalOrdering)
+        .map { it.groupValues[1].getPlaceholderText() }
+        .withIndex()
+        .associate { (index, key) -> key to index }
+
+    // Now sort the input conditionals by looking their sort priority up (sorting to the end if none defined).
+    // Remember this sort operator is stable - entries with identical priority keep their original ordering,
+    // so if the conditional order definition is empty, all conditionals keep their place.
+    fun sortPriority(unique: Unique) = sortPriorities.getOrDefault(unique.placeholderText, Int.MAX_VALUE)
+    val translatedConditionals = this.getModifiersSequence()
+        .sortedBy(::sortPriority)
+        .map { it.text.tr(hideIcons) }
+
+    // After that, add the translation of the base unique either before or after these conditionals
+    val translatedBaseUnique = this.removeConditionals().tr(hideIcons)
+    val translatedComponents = sequence {
+        if (conditionalsAfterUnique) yield(translatedBaseUnique)
+        yieldAll(translatedConditionals)
+        if (!conditionalsAfterUnique)
+            if (shouldCapitalize) yield(translatedBaseUnique.replaceFirstChar { it.lowercase() })
+            else yield(translatedBaseUnique)
     }
 
+    var fullyTranslatedString = translatedComponents.joinToString(spaceEquivalent)
+    if (shouldCapitalize)
+        fullyTranslatedString = fullyTranslatedString.replaceFirstChar { it.uppercase() }
+    return fullyTranslatedString
+}
+
+@Readonly
+private fun String.translatePlaceholders(language: String, hideIcons: Boolean): String {
+    /**
+     * I'm SURE there's an easier way to do this but I can't think of it =\
+     * So what's all this then?
+     * Well, not all languages are like English. So say I want to say "work on Library has completed in Akkad",
+     * but in a completely different language like Japanese or German,
+     * It could come out "Akkad hast die worken onner Library gerfinishen" or whatever,
+     * basically, the order of the words in the sentence is not guaranteed.
+     * So to translate this, I give a sentence like "work on [building] has completed in [city]"
+     * and the german can put those placeholders where he wants, so  "[city] hast die worken onner [building] gerfinishen"
+     * The string on which we call tr() will look like "work on [library] has completed in [Akkad]"
+     * We will find the german placeholder text, and replace the placeholders with what was filled in the text we got!
+     */
+
+    // Convert "work on [building] has completed in [city]" to "work on [] has completed in []"
+    val translationStringWithSquareBracketsOnly = this.getPlaceholderText()
+    // That is now the key into the translation HashMap!
+    val translationEntry = UncivGame.Current.translations
+        .get(translationStringWithSquareBracketsOnly, language, TranslationActiveModsCache.activeMods)
+
+    var languageSpecificPlaceholder: String
+    val originalEntry: String
+    if (translationEntry == null || !translationEntry.containsKey(language)) {
+        // Translation placeholder doesn't exist for this language, default to English
+        languageSpecificPlaceholder = this
+        originalEntry = this
+    } else {
+        languageSpecificPlaceholder = translationEntry[language]!!
+        originalEntry = translationEntry.entry
+    }
+
+    // Take the terms in the message, WITHOUT square brackets
+    val termsInMessage = this.getPlaceholderParameters()
+    // Take the terms from the placeholder
+    val termsInTranslationPlaceholder = originalEntry.getPlaceholderParameters()
+    if (termsInMessage.size != termsInTranslationPlaceholder.size)
+        throw Exception("Message $this has a different number of terms than the placeholder $translationEntry!")
+
+    for (i in termsInMessage.indices) {
+        languageSpecificPlaceholder = languageSpecificPlaceholder.replace(
+            "[${termsInTranslationPlaceholder[i]}]", // re-add square brackets to placeholder terms
+            termsInMessage[i].tr(hideIcons)
+        )
+    }
+    return languageSpecificPlaceholder      // every component is already translated
+}
+
+
+/** No brackets of any kind, just a single word */
+@Readonly
+private fun String.translateIndividualWord(language: String, hideIcons: Boolean, hideStats: Boolean): String {
     if (Stats.isStats(this)) return Stats.parse(this).toString()
 
-    val translation = UncivGame.Current.translations.getText(this, language, TranslationActiveModsCache.activeMods)
+    val translation = UncivGame.Current.translations.getText(
+        this, language, TranslationActiveModsCache.activeMods
+    ).replace(digitsRegex) {
+        it.value.toLong().tr(language)
+    }
 
     val stat = Stat.safeValueOf(this)
-    if (stat != null) return stat.character + translation
+    if (!hideStats && stat != null) return stat.character + translation
 
     if (!hideIcons && FontRulesetIcons.rulesetObjectNameToChar.containsKey(this))
         return FontRulesetIcons.rulesetObjectNameToChar[this]!! + translation
@@ -439,14 +488,16 @@ fun String.tr(hideIcons: Boolean = false): String {
  * For example, a string like 'The city of [New [York]]' will return ['New [York]'],
  * allowing us to have nested translations!
  */
+@Pure
 fun String.getPlaceholderParameters(): List<String> {
     if (!this.contains('[')) return emptyList()
 
     val stringToParse = this.removeConditionals()
+
     val parameters = ArrayList<String>()
     var depthOfBraces = 0
     var startOfCurrentParameter = -1
-    for (i in stringToParse.indices) {
+    stringToParse.indices.forEach { i ->
         if (stringToParse[i] == '[') {
             if (depthOfBraces == 0) startOfCurrentParameter = i+1
             depthOfBraces++
@@ -459,41 +510,54 @@ fun String.getPlaceholderParameters(): List<String> {
     return parameters
 }
 
+@Pure
 fun String.getPlaceholderText(): String {
     var stringToReturn = this.removeConditionals()
-    val placeholderParameters = stringToReturn.getPlaceholderParameters()
-    for (placeholderParameter in placeholderParameters)
-        stringToReturn = stringToReturn.replace("[$placeholderParameter]", "[]")
+    @LocalState val placeholderParameters = stringToReturn.getPlaceholderParameters()
+    placeholderParameters.forEach { placeholderParameter ->
+        stringToReturn = stringToReturn.replaceFirst("[$placeholderParameter]", "[]")
+    }
     return stringToReturn
 }
 
+@Pure
 fun String.equalsPlaceholderText(str: String): Boolean {
+    if (isEmpty()) return str.isEmpty()
+    if (str.isEmpty()) return false // Empty strings have no .first()
     if (first() != str.first()) return false // for quick negative return 95% of the time
     return this.getPlaceholderText() == str
 }
 
+@Pure
 fun String.hasPlaceholderParameters(): Boolean {
     if (!this.contains('[')) return false
     return squareBraceRegex.containsMatchIn(this.removeConditionals())
 }
 
 /** Substitutes placeholders with [strings], respecting order of appearance. */
+@Pure
 fun String.fillPlaceholders(vararg strings: String): String {
-    val keys = this.getPlaceholderParameters()
+    @Immutable val keys = this.getPlaceholderParameters()
     if (keys.size > strings.size)
         throw Exception("String $this has a different number of placeholders ${keys.joinToString()} (${keys.size}) than the substitutive strings ${strings.joinToString()} (${strings.size})!")
 
     var filledString = this.replace(squareBraceRegex, "[]")
-    for (i in keys.indices)
+    keys.indices.forEach { i ->
         filledString = filledString.replaceFirst("[]", "[${strings[i]}]")
+    }
     return filledString
 }
 
-fun String.getConditionals(): List<Unique> {
-    if (!this.contains('<')) return emptyList()
-    return pointyBraceRegex.findAll(this).map { Unique(it.groups[1]!!.value) }.toList()
+@Pure
+private fun String.getModifiersSequence(): Sequence<Unique> {
+    if (!this.contains('<')) return emptySequence()
+    @Immutable val matchResults = pointyBraceRegex.findAll(this)
+    return matchResults.map { Unique(it.groups[1]!!.value) }
 }
+@Readonly
+fun String.getModifiers() = getModifiersSequence().toList()
 
+@Pure
 fun String.removeConditionals(): String {
     if (!this.contains('<')) return this // no need to regex search
     return this
@@ -507,3 +571,25 @@ fun String.removeConditionals(): String {
         .replace("  ", " ")
         .trim()
 }
+
+/** Formats number according to current language
+ *
+ *  Note: The inverse operation is UncivGame.Current.settings.getCurrentNumberFormat().parse(string), handled in the [UncivTextField.Numeric][com.unciv.ui.components.widgets.UncivTextField.Numeric] widget.
+ *
+ *  @return locale-dependent String representation of receiver, may contain formatting like thousands separators
+ */
+@Readonly
+fun Number.tr(): String {
+    return UncivGame.Current.settings.getCurrentNumberFormat().format(this)
+}
+
+/** Formats number according to a specific [language]
+ *
+ *  Note: The inverse operation is `LocaleCode.getNumberFormatFromLanguage(language).parse(string)`.
+ *
+ *  @return locale-dependent String representation of receiver, may contain formatting like thousands separators
+ */
+@Readonly
+fun Number.tr(language: String): String {
+    return LocaleCode.getNumberFormatFromLanguage(language).format(this)
+}
