@@ -20,17 +20,21 @@ import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.ImageButton
 import com.badlogic.gdx.scenes.scene2d.ui.Label
 import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane
+import com.badlogic.gdx.scenes.scene2d.ui.SelectBox
+import com.badlogic.gdx.scenes.scene2d.ui.Stack
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton.TextButtonStyle
 import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup
 import com.badlogic.gdx.utils.Align
+import com.badlogic.gdx.utils.Array
 import com.unciv.Constants
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.DEL
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.toString
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.valueOf
 import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.components.input.ActorAttachments
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
@@ -38,6 +42,8 @@ import com.unciv.ui.components.input.onChange
 import com.unciv.ui.images.IconCircleGroup
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
+import yairm210.purity.annotations.Pure
+import yairm210.purity.annotations.Readonly
 
 /**
  * Collection of extension functions mostly for libGdx widgets
@@ -48,9 +54,12 @@ private class RestorableTextButtonStyle(
     val restoreStyle: ButtonStyle
 ) : TextButtonStyle(baseStyle)
 
+//todo ButtonStyle *does* have a `disabled` Drawable, and Button ignores touches in disabled state anyway - all this is a wrong approach
 /** Disable a [Button] by setting its [touchable][Button.touchable] and [style][Button.style] properties. */
 fun Button.disable() {
     touchable = Touchable.disabled
+    /** We want disabled buttons to "swallow" the click so that things behind aren't activated, so we don't change touchable
+       The action won't be activated due to [ActorAttachments.activate] checking the isDisabled property */
     isDisabled = true
     val oldStyle = style
     if (oldStyle is RestorableTextButtonStyle) return
@@ -59,12 +68,12 @@ fun Button.disable() {
 }
 /** Enable a [Button] by setting its [touchable][Button.touchable] and [style][Button.style] properties. */
 fun Button.enable() {
+    touchable = Touchable.enabled
     val oldStyle = style
     if (oldStyle is RestorableTextButtonStyle) {
         style = oldStyle.restoreStyle
     }
     isDisabled = false
-    touchable = Touchable.enabled
 }
 /** Enable or disable a [Button] by setting its [touchable][Button.touchable] and [style][Button.style] properties,
  *  or returns the corresponding state.
@@ -85,16 +94,34 @@ fun colorFromHex(hexColor: Int): Color {
 }
 
 /** Create a new [Color] instance from [r]/[g]/[b] given as Integers in the range 0..255 */
-fun colorFromRGB(r: Int, g: Int, b: Int) = Color(r / 255f, g / 255f, b / 255f, 1f)
+@Pure fun colorFromRGB(r: Int, g: Int, b: Int) = Color(r / 255f, g / 255f, b / 255f, 1f)
+
 /** Create a new [Color] instance from r/g/b given as Integers in the range 0..255 in the form of a 3-element List [rgb] */
-fun colorFromRGB(rgb: List<Int>) = colorFromRGB(rgb[0], rgb[1], rgb[2])
-/** Linearly interpolates between this [Color] and [BLACK][Color.BLACK] by [t] which is in the range [[0,1]].
+@Readonly fun colorFromRGB(rgb: List<Int>) = colorFromRGB(rgb[0], rgb[1], rgb[2])
+
+/** Linearly interpolates between this [Color] and [BLACK][ImageGetter.CHARCOAL] by [t] which is in the range [[0,1]].
  * The result is returned as a new instance. */
 fun Color.darken(t: Float): Color = Color(this).lerp(Color.BLACK, t)
-/** Linearly interpolates between this [Color] and [WHITE][Color.WHITE] by [t] which is in the range [[0,1]].
- * The result is returned as a new instance. */
-fun Color.brighten(t: Float): Color = Color(this).lerp(Color.WHITE, t)
 
+/** Linearly interpolates between this [Color] and [WHITE][Color.WHITE] by [t] which is in the range [[0,1]],
+ * preserving color ratio in RGB. The result is returned as a new instance. */
+fun Color.brighten(t: Float): Color = Color(this).let {
+    val lightness = maxOf(r, g, b)
+    val targetRatio = (lightness + t * (1 - lightness)) / lightness
+    return it.mul(targetRatio)
+}
+
+/** Ensures that the `lightness` value of the given color
+ * in `HSL` scale is at least [minLightness].
+ * Creates a copy.
+ */
+fun Color.coerceLightnessAtLeast(minLightness: Float): Color {
+    /** see [Color.toHsv] implementation to understand this */
+    val lightness = maxOf(r, g, b)
+    return if (lightness < minLightness) {
+        this.cpy().mul(minLightness / lightness)
+    } else this
+}
 
 fun Actor.centerX(parent: Actor) { x = parent.width / 2 - width / 2 }
 fun Actor.centerY(parent: Actor) { y = parent.height / 2 - height / 2 }
@@ -114,7 +141,7 @@ fun Actor.surroundWithCircle(
     return IconCircleGroup(size, this, resizeActor, color, circleImageLocation)
 }
 
-fun Actor.surroundWithThinCircle(color: Color=Color.BLACK): IconCircleGroup = surroundWithCircle(width+2f, false, color)
+fun Actor.surroundWithThinCircle(color: Color=ImageGetter.CHARCOAL): IconCircleGroup = surroundWithCircle(width+2f, false, color)
 
 
 fun Actor.addBorder(size: Float, color: Color, expandCell: Boolean = false): Table {
@@ -203,7 +230,7 @@ private fun getSeparatorImage(color: Color) = Image(ImageGetter.getWhiteDotDrawa
  * Create a horizontal separator as an empty Container with a colored background.
  * @param colSpan Optionally override [colspan][Cell.colspan] which defaults to the current column count.
  */
-fun Table.addSeparator(color: Color = Color.WHITE, colSpan: Int = 0, height: Float = 2f): Cell<Image> {
+fun Table.addSeparator(color: Color = BaseScreen.skin.getColor("color"), colSpan: Int = 0, height: Float = 1f): Cell<Image> {
     if (!cells.isEmpty && !cells.last().isEndRow) row()
     val separator = getSeparatorImage(color)
     val cell = add(separator)
@@ -222,17 +249,32 @@ fun Table.addSeparatorVertical(color: Color = Color.WHITE, width: Float = 2f): C
     return add(getSeparatorImage(color)).width(width).fillY()
 }
 
+/**
+ * When using Tables as touchables, much like buttons,
+ * this function renders them disabled: faded and not touchable.
+ */
+fun Table.setEnabled(enabled: Boolean) {
+    color.a = if (enabled) 1f else 0.5f
+    touchable = if (enabled) Touchable.enabled else Touchable.disabled 
+}
+
 /** Alternative to [Table].[add][Table] that returns the Table instead of the new Cell to allow a different way of chaining */
 fun <T : Actor> Table.addCell(actor: T): Table {
     add(actor)
     return this
 }
 
-/** Shortcut for [Cell].[pad][com.badlogic.gdx.scenes.scene2d.ui.Cell.pad] with top=bottom and left=right */
+/** Shortcut for [Cell].[pad][Cell.pad] with top=bottom and left=right */
 fun <T : Actor> Cell<T>.pad(vertical: Float, horizontal: Float): Cell<T> {
     return pad(vertical, horizontal, vertical, horizontal)
 }
 
+fun <T> SelectBox<T>.setItems(newItems: Collection<T>){
+    val array = Array<T>()
+    newItems.forEach { array.add(it) }
+    items = array
+}
+
 /** Sets both the width and height to [size] */
 fun Image.setSize(size: Float) {
     setSize(size, size)
@@ -278,10 +320,27 @@ fun getCloseButton(
     return closeButton
 }
 
+/**
+ * Adds a white-circled (x) close button to [parent], positioned to the top right,
+ * slightly shifted outwards.
+ */
+fun addRoundCloseButton(
+    parent: Group,
+    action: () -> Unit
+): Group {
+    val size = 30f
+    val button = getCloseButton(size, size-15f, Color.CLEAR, Color.RED, action = action)
+        .surroundWithCircle(size, false, BaseScreen.clearColor)
+        .surroundWithCircle(size+4f, false, Color.WHITE)
+    parent.addActor(button)
+    button.setPosition(parent.width - button.width*3/4, parent.height - button.height*3/4)
+    return button
+}
+
 /** Translate a [String] and make a [Label] widget from it */
 fun String.toLabel() = Label(this.tr(), BaseScreen.skin)
 /** Make a [Label] widget containing this [Int] as text */
-fun Int.toLabel() = this.toString().toLabel()
+fun Int.toLabel() = this.tr().toLabel()
 
 /** Translate a [String] and make a [Label] widget from it with a specified font color and size */
 fun String.toLabel(fontColor: Color = Color.WHITE,
@@ -296,7 +355,8 @@ fun String.toLabel(fontColor: Color = Color.WHITE,
         labelStyle.fontColor = fontColor
         if (fontSize != Constants.defaultFontSize) labelStyle.font = Fonts.font
     }
-    return Label(this.tr(hideIcons), labelStyle).apply {
+    val translatedText = this.tr(hideIcons) 
+    return Label(translatedText, labelStyle).apply {
         setFontScale(fontSize / Fonts.ORIGINAL_FONT_SIZE)
         setAlignment(alignment)
     }
@@ -314,12 +374,13 @@ fun String.toCheckBox(startsOutChecked: Boolean = false, changeAction: ((Boolean
         }
         // Add a little distance between the icon and the text. 0 looks glued together,
         // 5 is about half an uppercase letter, and 1 about the width of the vertical line in "P".
-        imageCell.padRight(1f)
+        imageCell.padRight(Constants.defaultFontSize / 2.0f)
     }
 
 /** Sets the [font color][Label.LabelStyle.fontColor] on a [Label] and returns it to allow chaining */
 fun Label.setFontColor(color: Color): Label {
-    style = Label.LabelStyle(style).apply { fontColor=color }
+    if (color != style.fontColor)
+        style = Label.LabelStyle(style).apply { fontColor = color }
     return this
 }
 
@@ -328,7 +389,7 @@ fun Label.setFontSize(size: Int): Label {
     style = Label.LabelStyle(style)
     style.font = Fonts.font
     @Suppress("UsePropertyAccessSyntax") setStyle(style)
-    setFontScale(size/ Fonts.ORIGINAL_FONT_SIZE)
+    setFontScale(size / Fonts.ORIGINAL_FONT_SIZE)
     return this
 }
 
@@ -403,6 +464,7 @@ object GdxKeyCodeFixes {
 
 fun Input.isShiftKeyPressed() = isKeyPressed(Input.Keys.SHIFT_LEFT) || isKeyPressed(Input.Keys.SHIFT_RIGHT)
 fun Input.isControlKeyPressed() = isKeyPressed(Input.Keys.CONTROL_LEFT) || isKeyPressed(Input.Keys.CONTROL_RIGHT)
+fun Input.isAltKeyPressed() = isKeyPressed(Input.Keys.ALT_LEFT) || isKeyPressed(Input.Keys.ALT_RIGHT)
 fun Input.areSecretKeysPressed() = isKeyPressed(Input.Keys.SHIFT_RIGHT) &&
         (isKeyPressed(Input.Keys.CONTROL_RIGHT) || isKeyPressed(Input.Keys.ALT_RIGHT))
 
@@ -455,3 +517,9 @@ fun TextureData.getReadonlyPixmap(): Pixmap {
     field.isAccessible = true
     return field.get(this) as Pixmap
 }
+
+fun <T: Actor>Stack.addInTable(actor: T): Cell<T> {
+    val table = Table()
+    add(table)
+    return table.add(actor).grow()
+}
