@@ -33,10 +33,10 @@ import kotlin.math.ceil
  */
 object Fonts {
 
-    /** All text is originally rendered in 50px (set in AndroidLauncher and DesktopLauncher), and then scaled to fit the size of the text we need now.
+    /** All text is originally rendered in one size, and then scaled to fit the size of the text we need now.
      * This has several advantages: It means we only render each character once (good for both runtime and RAM),
      * AND it means that our 'custom' emojis only need to be once size (50px) and they'll be rescaled for what's needed. */
-    const val ORIGINAL_FONT_SIZE = 50f
+    const val ORIGINAL_FONT_SIZE = 100f
     const val DEFAULT_FONT_FAMILY = ""
 
     lateinit var fontImplementation: FontImplementation
@@ -58,6 +58,22 @@ object Fonts {
             .sortedWith(compareBy(UncivGame.Current.settings.getCollatorFromLocale()) { it.localName })
     }
 
+    /**
+     * Helper for v-centering the text of Icon – Label -type components:
+     *
+     * Normal vertical centering uses the entire font height. In reality,
+     * it is customary to align the centre from the baseline to the ascent
+     * with the centre of the other element.  This function estimates the
+     * correct amount to shift the text element.
+     */
+    fun getDescenderHeight(fontSize: Int): Float {
+        val ratio = fontImplementation.getMetrics().run {
+            descent / height }
+        // For whatever reason, undershooting the adjustment slightly
+        // causes rounding to work better
+        return ratio * fontSize.toFloat() + 2.25f
+    }
+
     /**
      * Turn a TextureRegion into a Pixmap.
      *
@@ -98,11 +114,13 @@ object Fonts {
         return pixmap
     }
 
+    //region Symbols added to font from atlas textures
     const val turn = '⏳'               // U+23F3 'hourglass'
     const val strength = '†'            // U+2020 'dagger'
     const val rangedStrength = '‡'      // U+2021 'double dagger'
     const val movement = '➡'            // U+27A1 'black rightwards arrow'
     const val range = '…'               // U+2026 'horizontal ellipsis'
+    const val health = '♡'              // U+2661 'white heart suit'
     const val production = '⚙'          // U+2699 'gear'
     const val gold = '¤'                // U+00A4 'currency sign'
     const val food = '⁂'                // U+2042 'asterism' (to avoid 🍏 U+1F34F 'green apple' needing 2 symbols in utf-16 and 4 in utf-8)
@@ -122,7 +140,22 @@ object Fonts {
     const val greatScientist = '⚛'      // U+269B 'atom'
     const val death = '☠'               // U+2620 'skull and crossbones'
     const val automate = '⛏'            // U+26CF 'pick'
-    const val infinity = '∞'            // U+221E - not in `allSymbols`, taken as-is from system font
+
+    //region Symbols that can be optionally added to the font from atlas textures
+    // (a mod can override these, otherwise the font supplies the glyph)
+    const val infinity = '∞'            // U+221E
+    const val clock = '⌚'               // U+231A 'watch'
+    const val star = '✯'                // U+272F 'pinwheel star'
+    const val status = '◉'              // U+25C9 'fisheye'
+    // The following two are used for sort visualization.
+    // They may disappear (show as placeholder box) on Linux if you clean out asian fonts.
+    // Alternatives: "↑" U+2191, "↓" U+2193 - much wider and weird spacing in some fonts (e.g. Verdana).
+    // These are possibly the highest codepoints in use in Unciv -
+    // Taken into account when limiting FontRulesetIcons codepoints (it respects the private area ending at U+F8FF)
+    const val sortUpArrow = '￪'         // U+FFEA 'half wide upward arrow'
+    const val sortDownArrow = '￬'       // U+FFEC 'half wide downward arrow'
+    const val rightArrow = '→'          // U+2192, e.g. Battle table or event-based tutorials
+    //endregion
 
     val allSymbols = mapOf(
         turn to "EmojiIcons/Turn",
@@ -144,6 +177,15 @@ object Fonts {
         greatScientist to "EmojiIcons/Great Scientist",
         death to "EmojiIcons/Death",
         automate to "EmojiIcons/Automate",
+        infinity to "EmojiIcons/Infinity",
+        clock to "EmojiIcons/SortedByTime",
+        star to "EmojiIcons/Star",
+        status to "EmojiIcons/SortedByStatus",
+        sortUpArrow to "EmojiIcons/SortedAscending",
+        sortDownArrow to "EmojiIcons/SortedDescending",
+        rightArrow to "EmojiIcons/RightArrow",
         *MayaCalendar.allSymbols
     )
+    //endregion
+
 }
