@@ -2,10 +2,14 @@ package com.unciv.ui.components.tilegroups
 
 import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.map.NeighborDirection
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.RoadStatus
+import com.unciv.models.metadata.GameSettings
+import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.tilesets.TileSetCache
 import com.unciv.models.tilesets.TileSetConfig
+import com.unciv.ui.components.tilegroups.layers.EdgeTileImage
 import com.unciv.ui.images.ImageAttempter
 import com.unciv.ui.images.ImageGetter
 
@@ -30,6 +34,13 @@ class TileSetStrings(
     unitSet: String? = UncivGame.Current.settings.unitSet,
     fallbackDepth: Int = 1
 ) {
+
+    constructor(ruleset: Ruleset, settings: GameSettings) : this(
+        ruleset.modOptions.tileset ?: settings.tileSet,
+        ruleset.modOptions.unitset ?: settings.unitSet
+    )
+
+
     /** Separator used to mark variants, e.g. nation style or era specific */
     val tag = "-"
 
@@ -43,12 +54,12 @@ class TileSetStrings(
 
     // These need to be by lazy since the orFallback expects a tileset, which it may not get.
     val hexagon: String by lazy { orFallback { tileSetLocation + "Hexagon"} }
-    val hexagonList by lazy { listOf(hexagon) }
+    val hexagonList by lazy { listOf(hexagon) } // since it's orFallback, needs to be lazy
     val crosshatchHexagon by lazy { orFallback { tileSetLocation + "CrosshatchHexagon" } }
     val unexploredTile by lazy { orFallback { tileSetLocation + "UnexploredTile" } }
     val crosshair by lazy { orFallback { getString(tileSetLocation, "Crosshair") } }
     val highlight by lazy { orFallback { getString(tileSetLocation, "Highlight") } }
-    val roadsMap = RoadStatus.values()
+    val roadsMap = RoadStatus.entries
         .filterNot { it == RoadStatus.None }
         .associateWith { tileSetLocation + it.name }
     val naturalWonder = tileSetLocation + "Tiles/NaturalWonder"
@@ -57,6 +68,26 @@ class TileSetStrings(
     val bottomRightRiver by lazy { orFallback { tilesLocation + "River-BottomRight"} }
     val bottomRiver by lazy { orFallback { tilesLocation + "River-Bottom"} }
     val bottomLeftRiver  by lazy { orFallback { tilesLocation + "River-BottomLeft"} }
+    
+    val edgeImagesByPosition = ImageGetter.getAllImageNames()
+        .filter { it.startsWith(tileSetLocation +"Edges/") }
+        .mapNotNull {
+            val split = it.split('/').last() // without folder
+                .split("-")
+            // Comprised of 3 parts: origin tilefilter, destination tilefilter, 
+            //   and edge type: Bottom, BottomLeft or BottomRight
+            if (split.size != 4) return@mapNotNull null
+
+            // split[0] is name and is unused
+            val originTileFilter = split[1]
+            val destinationTileFilter = split[2]
+            val neighborDirection = split[3]
+            val neighborDirectionEnumValue = NeighborDirection.entries
+                .firstOrNull { it.name == neighborDirection } ?: return@mapNotNull null
+
+            EdgeTileImage(it, originTileFilter, destinationTileFilter, neighborDirectionEnumValue)
+        }
+        .groupBy { it.edgeType }
 
     val unitsLocation = unitSetLocation + "Units/"
 
