@@ -5,7 +5,8 @@ import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.utils.Align
 import com.unciv.logic.civilization.Civilization
-import com.unciv.ui.components.tilegroups.CityButton
+import com.unciv.models.ruleset.unique.LocalUniqueCache
+import com.unciv.ui.components.tilegroups.citybutton.CityButton
 import com.unciv.ui.components.tilegroups.TileGroup
 import com.unciv.ui.components.tilegroups.WorldTileGroup
 import com.unciv.utils.DebugUtils
@@ -20,18 +21,18 @@ class TileLayerCityButton(tileGroup: TileGroup, size: Float) : TileLayer(tileGro
     }
 
     override fun act(delta: Float) {
-        if (tileGroup.tile.isCityCenter())
+        if (tile.isCityCenter())
             super.act(delta)
     }
 
     override fun hit(x: Float, y: Float, touchable: Boolean): Actor? {
-        if (tileGroup.tile.isCityCenter())
+        if (tile.isCityCenter())
             return super.hit(x, y, touchable)
         return null
     }
 
     override fun draw(batch: Batch?, parentAlpha: Float) {
-        if (tileGroup.tile.isCityCenter())
+        if (tile.isCityCenter())
             super.draw(batch, parentAlpha)
     }
 
@@ -43,12 +44,10 @@ class TileLayerCityButton(tileGroup: TileGroup, size: Float) : TileLayer(tileGro
         cityButton?.moveButtonDown()
     }
 
-    override fun doUpdate(viewingCiv: Civilization?) {
+    override fun doUpdate(viewingCiv: Civilization?, localUniqueCache: LocalUniqueCache) {
+        if (tileGroup !is WorldTileGroup) return
 
-        if (tileGroup !is WorldTileGroup)
-            return
-
-        val city = tile().getCity()
+        val city = tile.getCity()
 
         // There used to be a city here but it was razed
         if (city == null && cityButton != null) {
@@ -56,21 +55,16 @@ class TileLayerCityButton(tileGroup: TileGroup, size: Float) : TileLayer(tileGro
             cityButton = null
         }
 
-        if (viewingCiv == null)
-            return
-
-        val tileIsViewable = isViewable(viewingCiv)
-        val shouldShow = DebugUtils.VISIBLE_MAP
+        if (viewingCiv == null) return
+        if (city == null || !tileGroup.tile.isCityCenter()) return
 
         // Create (if not yet) and update city button
-        if (city != null && tileGroup.tile.isCityCenter()) {
-            if (cityButton == null) {
-                cityButton = CityButton(city, tileGroup)
-                addActor(cityButton)
-            }
-
-            cityButton!!.update(shouldShow || tileIsViewable)
+        if (cityButton == null) {
+            cityButton = CityButton(city, tileGroup)
+            addActor(cityButton)
         }
+
+        cityButton!!.update(DebugUtils.VISIBLE_MAP || isViewable(viewingCiv))
     }
 
 }
