@@ -1,6 +1,7 @@
 package com.unciv.ui.components.tilegroups.layers
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.g2d.Batch
 import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.ui.Image
@@ -9,11 +10,14 @@ import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.HexMath
 import com.unciv.logic.map.tile.Tile
+import com.unciv.logic.map.toHexCoord
 import com.unciv.models.ruleset.unique.LocalUniqueCache
+import com.unciv.models.translations.tr
 import com.unciv.ui.components.MapArrowType
 import com.unciv.ui.components.MiscArrowTypes
 import com.unciv.ui.components.TintedMapArrow
 import com.unciv.ui.components.UnitMovementMemoryType
+import com.unciv.ui.components.extensions.brighten
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.centerX
 import com.unciv.ui.components.extensions.toLabel
@@ -44,15 +48,178 @@ private class MapArrow(val targetTile: Tile, val arrowType: MapArrowType, val st
     }
 }
 
+class TileLayerYield(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, size){
+    private val yields = YieldGroup().apply {
+        // Unlike resource or improvement this is created and added only once,
+        // It's the contents that get updated
+        isVisible = false
+        setOrigin(Align.center)
+        setScale(0.7f)
+        y = tileGroup.height * 0.25f - height / 2
+        // Adding YieldGroup to miscLayerGroup
+        this@TileLayerYield.addActor(this)
+    }
+    
+    override fun doUpdate(viewingCiv: Civilization?, localUniqueCache: LocalUniqueCache) {
+        val showTileYields = if (tileGroup is WorldTileGroup) UncivGame.Current.settings.showTileYields else true
+        updateYieldIcon(viewingCiv, showTileYields, localUniqueCache)
+    }
+
+    // JN updating display of tile yields
+    private fun updateYieldIcon(
+        viewingCiv: Civilization?,
+        show: Boolean,
+        localUniqueCache: LocalUniqueCache
+    ) {
+        val effectiveVisible = show &&
+                !tileGroup.isForMapEditorIcon &&  // don't have a map to calc yields
+                !(viewingCiv == null && tileGroup.isForceVisible) // main menu background
+
+        // Hiding yield icons (in order to update)
+        yields.isVisible = false
+        if (effectiveVisible) yields.run {
+            // Update YieldGroup Icon
+            if (tileGroup is CityTileGroup)
+                setStats(tile.stats.getTileStats(tileGroup.city, viewingCiv, localUniqueCache))
+            else
+                setStats(tile.stats.getTileStats(viewingCiv, localUniqueCache))
+            toFront()
+            centerX(tileGroup)
+            isVisible = true
+        }
+    }
+
+    fun setYieldVisible(isVisible: Boolean) {
+        yields.isVisible = isVisible
+        this.isVisible = isVisible // don't try rendering the layer if there's nothing in it
+    }
+    
+    fun dimYields(dim: Boolean) { yields.color.a = if (dim) 0.5f else 1f }
+
+    fun reset(localUniqueCache: LocalUniqueCache) {
+        updateYieldIcon(null, false, localUniqueCache)
+    }
+
+    override fun act(delta: Float) {}
+    override fun hit(x: Float, y: Float, touchable: Boolean): Actor? = null
+    override fun draw(batch: Batch?, parentAlpha: Float) = super.draw(batch, parentAlpha)
+}
+
+
+class TileLayerResource(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, size){
+
+    private var resourceName: String? = null
+    private var resourceAmount: Int = -1
+    private var resourceIcon: Actor? = null
+
+    private fun updateResourceIcon(viewingCiv: Civilization?, show: Boolean) {
+        // This could change on any turn, since resources need certain techs to reveal them
+        val effectiveVisible = when {
+            tileGroup.isForceVisible -> show
+            show && viewingCiv == null -> true
+            show && viewingCiv?.canSeeResource(tile.tileResource) == true -> true
+            else -> false
+        }
+
+        // If resource has changed (e.g. tech researched) - force new icon next time it's needed
+        if (resourceName != tile.resource || resourceAmount != tile.resourceAmount) {
+            resourceName = tile.resource
+            resourceAmount = tile.resourceAmount
+            resourceIcon?.remove()
+            resourceIcon = null
+        }
+
+        // Get a fresh Icon if and only if necessary
+        if (resourceName != null && effectiveVisible && resourceIcon == null) {
+            val icon = ImageGetter.getResourcePortrait(resourceName!!, 20f, resourceAmount)
+            icon.center(tileGroup)
+            icon.x -= 22 // left
+            icon.y += 10 // top
+            addActor(icon)
+            resourceIcon = icon
+        }
+
+        resourceIcon?.isVisible = effectiveVisible
+
+
+        if (resourceIcon!=null){
+            val isViewable = viewingCiv == null || isViewable(viewingCiv)
+            dimResource(!isViewable)
+        }
+    }
+
+    fun reset() {
+        updateResourceIcon(null, false)
+    }
+
+    fun dimResource(dim: Boolean) { resourceIcon?.color?.a = if (dim) 0.5f else 1f }
+    
+    override fun doUpdate(viewingCiv: Civilization?, localUniqueCache: LocalUniqueCache) {
+        val showResourcesAndImprovements = if (tileGroup is WorldTileGroup)
+            UncivGame.Current.settings.showResourcesAndImprovements else true
+
+        updateResourceIcon(viewingCiv, showResourcesAndImprovements)
+    }
+
+    override fun determineVisibility() {
+        isVisible = resourceIcon?.isVisible == true
+    }
+}
+
+class TileLayerImprovement(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, size){
+    private var improvementPlusPillagedID: String? = null
+    var improvementIcon: Actor? = null
+        private set  // Getter public for BattleTable to display as City Combatant
+
+    
+    override fun doUpdate(viewingCiv: Civilization?, localUniqueCache: LocalUniqueCache) {
+        val showResourcesAndImprovements = if (tileGroup is WorldTileGroup)
+            UncivGame.Current.settings.showResourcesAndImprovements else true
+
+        updateImprovementIcon(viewingCiv, showResourcesAndImprovements)
+    }
+
+    fun dimImprovement(dim: Boolean) { improvementIcon?.color?.a = if (dim) 0.5f else 1f }
+
+    private fun updateImprovementIcon(viewingCiv: Civilization?, show: Boolean) {
+        // If improvement has changed, force new icon next time it is needed
+        val improvementToShow = tile.getShownImprovement(viewingCiv)
+        val newImprovementPlusPillagedID = if (improvementToShow==null) null
+        else if (tile.improvementIsPillaged) "$improvementToShow-Pillaged"
+        else improvementToShow
+
+        if (improvementPlusPillagedID != newImprovementPlusPillagedID) {
+            improvementPlusPillagedID = newImprovementPlusPillagedID
+            improvementIcon?.remove()
+            improvementIcon = null
+        }
+
+        // Get new icon when needed
+        if (improvementPlusPillagedID != null && show && improvementIcon == null) {
+            val icon = ImageGetter.getImprovementPortrait(improvementToShow!!, isPillaged = tile.improvementIsPillaged)
+            icon.center(tileGroup)
+            icon.x -= 22 // left
+            icon.y -= 12 // bottom
+            addActor(icon)
+            improvementIcon = icon
+        }
+
+        improvementIcon?.isVisible = show
+    }
+
+    override fun determineVisibility() {
+        isVisible = improvementIcon?.isVisible == true
+    }
+
+    fun reset() {
+        updateImprovementIcon(null, false)
+    }
+}
+
 class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, size) {
 
     // For different unit views, we want to effectively "ignore" the terrain and color it by special view
-    private val terrainOverlay = ImageGetter.getImage(strings().hexagon ).setHexagonSize()
-
-    init {
-        terrainOverlay.isVisible = false
-        addActor(terrainOverlay)
-    }
+    private var terrainOverlay: Image? = ImageGetter.getImage(strings.hexagon).setHexagonSize()
 
     override fun act(delta: Float) {}
     override fun hit(x: Float, y: Float, touchable: Boolean): Actor? {
@@ -65,16 +232,7 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
         }
     }
 
-    private val yields = YieldGroup().apply {
-        // Unlike resource or improvement this is created and added only once,
-        // It's the contents that get updated
-        isVisible = false
-        setOrigin(Align.center)
-        setScale(0.7f)
-        y = tileGroup.height * 0.25f - height / 2
-        // Adding YieldGroup to miscLayerGroup
-        this@TileLayerMisc.addActor(this)
-    }
+    override fun draw(batch: Batch?, parentAlpha: Float) = super.draw(batch, parentAlpha)
 
     /** Array list of all arrows to draw from this tile on the next update. */
     private val arrowsToDraw = ArrayList<MapArrow>()
@@ -82,16 +240,8 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
 
     private var hexOutlineIcon: Actor? = null
 
-    private var resourceName: String? = null
-    private var resourceAmount: Int = -1
-    private var resourceIcon: Actor? = null
-
     private var workedIcon: Actor? = null
 
-    private var improvementPlusPillagedID: String? = null
-    var improvementIcon: Actor? = null
-        private set  // Getter public for BattleTable to display as City Combatant
-
     private val startingLocationIcons = mutableListOf<Actor>()
 
     private fun clearArrows() {
@@ -107,12 +257,12 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
 
         for (arrowToAdd in arrowsToDraw) {
             val targetTile = arrowToAdd.targetTile
-            var targetPos = Vector2(targetTile.position)
-            if (tile().tileMap.mapParameters.worldWrap)
-                targetPos = HexMath.getUnwrappedNearestTo(targetPos,
-                    tile().position, tile().tileMap.maxLongitude)
-            val targetRelative = HexMath.hex2WorldCoords(targetPos)
-                .sub(HexMath.hex2WorldCoords(tile().position))
+            var targetPos = Vector2(targetTile.position.toVector2())
+            if (tile.tileMap.mapParameters.worldWrap)
+                targetPos = HexMath.getUnwrappedNearestTo(targetPos.toHexCoord(),
+                    tile.position, tile.tileMap.maxLongitude)
+            val targetRelative = HexMath.hex2WorldCoords(targetPos.toHexCoord())
+                .sub(HexMath.hex2WorldCoords(tile.position))
 
             val targetDistance = sqrt(targetRelative.x.pow(2) + targetRelative.y.pow(2))
             val targetAngle = atan2(targetRelative.y, targetRelative.x)
@@ -137,68 +287,6 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
         }
     }
 
-    private fun updateImprovementIcon(viewingCiv: Civilization?, show: Boolean) {
-        // If improvement has changed, force new icon next time it is needed
-        val improvementToShow = tile().getShownImprovement(viewingCiv)
-        val newImprovementPlusPillagedID = if (improvementToShow==null) null
-        else if (tile().improvementIsPillaged) "$improvementToShow-Pillaged"
-        else improvementToShow
-
-        if (improvementPlusPillagedID != newImprovementPlusPillagedID) {
-            improvementPlusPillagedID = newImprovementPlusPillagedID
-            improvementIcon?.remove()
-            improvementIcon = null
-        }
-
-        // Get new icon when needed
-        if (improvementPlusPillagedID != null && show && improvementIcon == null) {
-            val icon = ImageGetter.getImprovementPortrait(improvementToShow!!, dim = false, isPillaged = tile().improvementIsPillaged)
-            icon.center(tileGroup)
-            icon.x -= 22 // left
-            icon.y -= 12 // bottom
-            addActor(icon)
-            improvementIcon = icon
-        }
-
-        improvementIcon?.isVisible = show
-    }
-
-    private fun updateResourceIcon(viewingCiv: Civilization?, show: Boolean) {
-        // This could change on any turn, since resources need certain techs to reveal them
-        val effectiveVisible = when {
-            tileGroup.isForceVisible -> show
-            show && viewingCiv == null -> true
-            show && tile().hasViewableResource(viewingCiv!!) -> true
-            else -> false
-        }
-
-        // If resource has changed (e.g. tech researched) - force new icon next time it's needed
-        if (resourceName != tile().resource || resourceAmount != tile().resourceAmount) {
-            resourceName = tile().resource
-            resourceAmount = tile().resourceAmount
-            resourceIcon?.remove()
-            resourceIcon = null
-        }
-
-        // Get a fresh Icon if and only if necessary
-        if (resourceName != null && effectiveVisible && resourceIcon == null) {
-            val icon = ImageGetter.getResourcePortrait(resourceName!!, 20f, resourceAmount)
-            icon.center(tileGroup)
-            icon.x -= 22 // left
-            icon.y += 10 // top
-            addActor(icon)
-            resourceIcon = icon
-        }
-
-        resourceIcon?.isVisible = effectiveVisible
-
-
-        if (resourceIcon!=null){
-            val isViewable = viewingCiv == null || isViewable(viewingCiv)
-            dimResource(!isViewable)
-        }
-    }
-
     private fun updateStartingLocationIcon(show: Boolean) {
         // The starting location icons are visible in map editor only, but this method is abused for the
         // "Show coordinates on tiles" debug option as well. Calling code made sure this is only called
@@ -213,8 +301,8 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
             return
 
         if (DebugUtils.SHOW_TILE_COORDS) {
-            val label = this.tile().position.toPrettyString()
-            startingLocationIcons.add(label.toLabel(Color.BLACK.cpy().apply { a = 0.7f }, 14).apply {
+            val label = this.tile.position.toVector2().toPrettyString()
+            startingLocationIcons.add(label.toLabel(ImageGetter.CHARCOAL.cpy().apply { a = 0.7f }, 14).apply {
                 tileGroup.layerMisc.addActor(this)
                 setOrigin(Align.center)
                 center(tileGroup)
@@ -228,7 +316,7 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
             })
         }
 
-        val tilemap = tile().tileMap
+        val tilemap = tile.tileMap
 
         if (tilemap.startingLocationsByNation.isEmpty())
             return
@@ -237,7 +325,7 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
         // Sorted so major get precedence and to make the display deterministic, otherwise you could get
         // different stacking order of the same nations in the same editing session
         val nations = tilemap.startingLocationsByNation.asSequence()
-            .filter { tile() in it.value }
+            .filter { tile in it.value }
             .filter { it.key in tilemap.ruleset!!.nations } // Ignore missing nations
             .map { it.key to tilemap.ruleset!!.nations[it.key]!! }
             .sortedWith(compareBy({ it.second.isCityState }, { it.first }))
@@ -265,13 +353,13 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
         // Add a Label with the total count for this tile
         if (nations.size > 3) {
             // Tons of locations for this tile - display number in red, behind the top three
-            startingLocationIcons.add(nations.size.toString().toLabel(Color.BLACK.cpy().apply { a = 0.7f }, 14).apply {
+            startingLocationIcons.add(nations.size.tr().toLabel(ImageGetter.CHARCOAL.cpy().apply { a = 0.7f }, 14).apply {
                 tileGroup.layerMisc.addActor(this)
                 setOrigin(Align.center)
                 center(tileGroup)
                 moveBy(14.4f, -9f)
             })
-            startingLocationIcons.add(nations.size.toString().toLabel(Color.FIREBRICK, 14).apply {
+            startingLocationIcons.add(nations.size.tr().toLabel(Color.FIREBRICK, 14).apply {
                 tileGroup.layerMisc.addActor(this)
                 setOrigin(Align.center)
                 center(tileGroup)
@@ -280,29 +368,6 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
         }
     }
 
-    // JN updating display of tile yields
-    private fun updateYieldIcon(
-        viewingCiv: Civilization?,
-        show: Boolean,
-        localUniqueCache: LocalUniqueCache
-    ) {
-        val effectiveVisible = show &&
-                !tileGroup.isForMapEditorIcon &&  // don't have a map to calc yields
-                !(viewingCiv == null && tileGroup.isForceVisible) // main menu background
-
-        // Hiding yield icons (in order to update)
-        yields.isVisible = false
-        if (effectiveVisible) yields.run {
-            // Update YieldGroup Icon
-            if (tileGroup is CityTileGroup)
-                setStats(tile().stats.getTileStats(tileGroup.city, viewingCiv, localUniqueCache))
-            else
-                setStats(tile().stats.getTileStats(viewingCiv, localUniqueCache))
-            toFront()
-            centerX(tileGroup)
-            isVisible = true
-        }
-    }
 
     fun removeWorkedIcon() {
         workedIcon?.remove()
@@ -332,77 +397,54 @@ class TileLayerMisc(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, si
     }
 
     /** Activates a colored semitransparent overlay. [color] is cloned, brightened by 0.3f and an alpha of 0.4f applied. */
-    fun overlayTerrain(color: Color) {
-        terrainOverlay.color = color.cpy().lerp(Color.WHITE, 0.3f).apply { a = 0.4f }
-        terrainOverlay.isVisible = true
-        determineVisibility()
-    }
+    fun overlayTerrain(color: Color) = overlayTerrainInner(color.brighten(0.3f).apply { a = 0.4f })
 
     /** Activates a colored semitransparent overlay. [color] is cloned and [alpha] applied. No brightening unlike the overload without explicit alpha! */
-    fun overlayTerrain(color: Color, alpha: Float) {
-        terrainOverlay.color = color.cpy().apply { a = alpha }
-        terrainOverlay.isVisible = true
+    fun overlayTerrain(color: Color, alpha: Float) = overlayTerrainInner(color.cpy().apply { a = alpha })
+    
+    private fun overlayTerrainInner(color: Color) {
+        if (terrainOverlay == null){
+            terrainOverlay = ImageGetter.getImage(strings.hexagon).setHexagonSize()
+            addActor(terrainOverlay)
+        }
+        terrainOverlay?.color = color
         determineVisibility()
     }
 
     fun hideTerrainOverlay() {
-        terrainOverlay.isVisible = false
+        terrainOverlay?.remove()
+        terrainOverlay = null
         determineVisibility()
     }
 
 
     fun addArrow(targetTile: Tile, type: MapArrowType) {
-        if (targetTile.position != tile().position)
-            arrowsToDraw.add(MapArrow(targetTile, type, strings()))
+        if (targetTile.position != tile.position)
+            arrowsToDraw.add(MapArrow(targetTile, type, strings))
     }
 
     fun resetArrows() {
         arrowsToDraw.clear()
     }
 
-    fun dimImprovement(dim: Boolean) { improvementIcon?.color?.a = if (dim) 0.5f else 1f }
-    fun dimResource(dim: Boolean) { resourceIcon?.color?.a = if (dim) 0.5f else 1f }
-    fun dimYields(dim: Boolean) { yields.color.a = if (dim) 0.5f else 1f }
     fun dimPopulation(dim: Boolean) { workedIcon?.color?.a = if (dim) 0.4f else 1f }
 
-    fun setYieldVisible(isVisible: Boolean) {
-        yields.isVisible = isVisible
-        determineVisibility()
-    }
 
     override fun doUpdate(viewingCiv: Civilization?, localUniqueCache: LocalUniqueCache) {
-
-        var showResourcesAndImprovements = true
-        var showTileYields = true
-
-        if (tileGroup is WorldTileGroup) {
-            showResourcesAndImprovements = UncivGame.Current.settings.showResourcesAndImprovements
-            showTileYields = UncivGame.Current.settings.showTileYields
-        }
-
-        updateImprovementIcon(viewingCiv, showResourcesAndImprovements)
-        updateYieldIcon(viewingCiv, showTileYields, localUniqueCache)
-        updateResourceIcon(viewingCiv, showResourcesAndImprovements)
         if (tileGroup !is WorldTileGroup || DebugUtils.SHOW_TILE_COORDS)
             updateStartingLocationIcon(true)
         updateArrows()
     }
 
     override fun determineVisibility() {
-        isVisible = yields.isVisible
-                || resourceIcon?.isVisible == true
-                || improvementIcon?.isVisible == true
-                || workedIcon != null
+        isVisible = workedIcon != null
                 || hexOutlineIcon != null
                 || arrows.isNotEmpty()
                 || startingLocationIcons.isNotEmpty()
-                || terrainOverlay.isVisible
+                || terrainOverlay != null
     }
 
-    fun reset(localUniqueCache: LocalUniqueCache) {
-        updateImprovementIcon(null, false)
-        updateYieldIcon(null, false, localUniqueCache)
-        updateResourceIcon(null, false)
+    fun reset() {
         updateStartingLocationIcon(false)
         clearArrows()
     }
