@@ -1,23 +1,27 @@
 package com.unciv.ui.components.tilegroups.layers
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.g2d.Batch
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.map.NeighborDirection
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.unique.LocalUniqueCache
-import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.components.tilegroups.TileGroup
+import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
+import yairm210.purity.annotations.Readonly
 import kotlin.random.Random
 
 class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup, size) {
 
     override fun act(delta: Float) {}
     override fun hit(x: Float, y: Float, touchable: Boolean): Actor? = null
+    override fun draw(batch: Batch?, parentAlpha: Float) = super.draw(batch, parentAlpha)
 
-    private val tileBaseImages: ArrayList<Image> = ArrayList()
+    val tileBaseImages: ArrayList<Image> = ArrayList()
     private var tileImageIdentifiers = listOf<String>()
     private var bottomRightRiverImage: Image? = null
     private var bottomRiverImage: Image? = null
@@ -25,23 +29,23 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
 
     private fun getTerrainImageLocations(terrainSequence: Sequence<String>): List<String> {
         val allTerrains = terrainSequence.joinToString("+")
-        if (strings().tileSetConfig.ruleVariants.containsKey(allTerrains))
-            return strings().tileSetConfig.ruleVariants[allTerrains]!!.map { strings().getTile(it) }
-        val allTerrainTile = strings().getTile(allTerrains)
+        if (strings.tileSetConfig.ruleVariants.containsKey(allTerrains))
+            return strings.tileSetConfig.ruleVariants[allTerrains]!!.map { strings.getTile(it) }
+        val allTerrainTile = strings.getTile(allTerrains)
         return if (ImageGetter.imageExists(allTerrainTile)) listOf(allTerrainTile)
-        else terrainSequence.map { strings().orFallback { getTile(it) } }.toList()
+        else terrainSequence.map { strings.orFallback { getTile(it) } }.toList()
     }
 
     private fun getImprovementAndResourceImages(resourceAndImprovementSequence: Sequence<String>): List<String> {
-        val altogether = resourceAndImprovementSequence.joinToString("+").let { strings().getTile(it) }
+        val altogether = resourceAndImprovementSequence.joinToString("+").let { strings.getTile(it) }
         return if (ImageGetter.imageExists(altogether)) listOf(altogether)
-        else resourceAndImprovementSequence.map { strings().orFallback { getTile(it) } }.toList()
+        else resourceAndImprovementSequence.map { strings.orFallback { getTile(it) } }.toList()
     }
 
     private fun usePillagedImprovementImage(tile: Tile, viewingCiv: Civilization?): Boolean {
         if (!tile.improvementIsPillaged || !UncivGame.Current.settings.showPixelImprovements) return false
         val shownImprovement = tile.getShownImprovement(viewingCiv) ?: return false
-        return ImageGetter.imageExists(strings().getTile("$shownImprovement-Pillaged"))
+        return ImageGetter.imageExists(strings.getTile("$shownImprovement-Pillaged"))
     }
 
     private fun getTileBaseImageLocations(viewingCiv: Civilization?): List<String> {
@@ -49,11 +53,11 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
         val isForceVisible = tileGroup.isForceVisible
 
         if (viewingCiv == null && !isForceVisible)
-            return strings().hexagonList
+            return strings.hexagonList
 
-        val baseHexagon = if (strings().tileSetConfig.useColorAsBaseTerrain)
-            listOf(strings().hexagon)
-        else listOf()
+        val baseHexagon: ArrayList<String> = if (strings.tileSetConfig.useColorAsBaseTerrain)
+            ArrayList<String>().apply { add(strings.hexagon) }
+        else ArrayList()
 
         val tile = tileGroup.tile
 
@@ -61,7 +65,7 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
         val shouldShowImprovement = shownImprovement != null && UncivGame.Current.settings.showPixelImprovements
 
         val shouldShowResource = UncivGame.Current.settings.showPixelImprovements && tile.resource != null &&
-                (isForceVisible || viewingCiv == null || tile.hasViewableResource(viewingCiv))
+                (isForceVisible || viewingCiv == null || viewingCiv.canSeeResource(tile.tileResource))
 
         val resourceAndImprovementSequence = sequence {
             if (shouldShowResource)  yield(tile.resource!!)
@@ -75,20 +79,81 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
         val terrainImages = if (tile.naturalWonder != null)
             sequenceOf(tile.baseTerrain, tile.naturalWonder!!)
         else  sequenceOf(tile.baseTerrain) + tile.terrainFeatures.asSequence()
+        val edgeImages = getEdgeTileLocations()
         val allTogether = (terrainImages + resourceAndImprovementSequence).joinToString("+")
-        val allTogetherLocation = strings().getTile(allTogether)
+        val allTogetherLocation = strings.getTile(allTogether)
 
+        // If the tilesetconfig *explicitly* lists the terrains+improvements etc, we can't know where in that list to place the edges
+        //   So we default to placing them over everything else.
+        // If there is no explicit list, then we can know to place them between the terrain and the improvement
         return when {
-            strings().tileSetConfig.ruleVariants[allTogether] != null -> baseHexagon + strings().tileSetConfig.ruleVariants[allTogether]!!.map { strings().getTile(it) }
-            ImageGetter.imageExists(allTogetherLocation) -> baseHexagon + allTogetherLocation
-            tile.naturalWonder != null -> getNaturalWonderBackupImage(baseHexagon)
-            else -> baseHexagon + getTerrainImageLocations(terrainImages) + getImprovementAndResourceImages(resourceAndImprovementSequence)
+            strings.tileSetConfig.ruleVariants[allTogether] != null -> baseHexagon.apply { 
+                addAll(strings.tileSetConfig.ruleVariants[allTogether]!!.map { strings.getTile(it) })
+                addAll(edgeImages)
+            } 
+            ImageGetter.imageExists(allTogetherLocation) -> baseHexagon.apply { add(allTogetherLocation); addAll(edgeImages) }
+            tile.naturalWonder != null -> getNaturalWonderBackupImage(baseHexagon) + edgeImages
+            else -> baseHexagon.apply { 
+                addAll(getTerrainImageLocations(terrainImages))
+                addAll(edgeImages)
+                addAll(getImprovementAndResourceImages(resourceAndImprovementSequence))
+            }
         }
     }
+    
+    private class NeighborEdgeData(val neighbor: Tile, val direction: NeighborDirection?) {
+        var ourTerrains: Set<String> = emptySet()
+        var neighborTerrains: Set<String> = emptySet()
+        var edgeFiles: Sequence<String> = emptySequence()
+    }
+    
+    private val neighborEdgeDataList: Sequence<NeighborEdgeData> = if (!tile.isTilemapInitialized()) emptySequence()
+        else tile.neighbors.map {
+            val clockPosition = tile.tileMap.getNeighborTileClockPosition(tile, it)
+            val direction = NeighborDirection.byClockPosition[clockPosition]
+            NeighborEdgeData(it, direction)
+        }.toList().asSequence()
+    
+    
+    private fun getEdgeTileLocations(): Sequence<String> {
+        if (!tile.isTilemapInitialized()) // fake tile 
+            return emptySequence()
+        return neighborEdgeDataList
+            .flatMap { getMatchingEdges(it) }
+    }
+
+    private fun getMatchingEdges(neighborEdgeData: NeighborEdgeData): Sequence<String>{
+        if (neighborEdgeData.ourTerrains == tile.cachedTerrainData.terrainNameSet
+            && neighborEdgeData.neighborTerrains == neighborEdgeData.neighbor.cachedTerrainData.terrainNameSet)
+                return neighborEdgeData.edgeFiles
+        
+        if (neighborEdgeData.direction == null) return emptySequence()
+        val possibleEdgeImages = strings.edgeImagesByPosition[neighborEdgeData.direction] ?: return emptySequence()
+        
+        // Required for performance - full matchesFilter is too expensive for something that needs to run every update()
+        @Readonly
+        fun matchesFilterMinimal(originTile: Tile, filter: String): Boolean {
+            if (originTile.cachedTerrainData.terrainNameSet.contains(filter)) return true
+            if (originTile.getBaseTerrain().type.name == filter) return true
+            return false
+        }
+
+        val cachedSequence = possibleEdgeImages.filter {
+            if (!matchesFilterMinimal(tile, it.originTileFilter)) return@filter false
+            if (!matchesFilterMinimal(neighborEdgeData.neighbor, it.destinationTileFilter)) return@filter false
+            return@filter true
+        }.map { it.fileName }.asSequence()
+        
+        neighborEdgeData.ourTerrains = tile.cachedTerrainData.terrainNameSet
+        neighborEdgeData.neighborTerrains = neighborEdgeData.neighbor.cachedTerrainData.terrainNameSet
+        neighborEdgeData.edgeFiles = cachedSequence
+        
+        return cachedSequence
+    }
 
     private fun updateTileImage(viewingCiv: Civilization?) {
         val tileBaseImageLocations = getTileBaseImageLocations(viewingCiv)
-
+        
         if (tileBaseImageLocations.size == tileImageIdentifiers.size) {
             if (tileBaseImageLocations.withIndex().all { (i, imageLocation) -> tileImageIdentifiers[i] == imageLocation })
                 return // All image identifiers are the same as the current ones, no need to change anything
@@ -104,7 +169,7 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
 
             val locationToCheck =
                     if (tileGroup.tile.owningCity != null)
-                        strings().getOwnedTileImageLocation(baseLocation, tileGroup.tile.getOwner()!!)
+                        strings.getOwnedTileImageLocation(baseLocation, tileGroup.tile.getOwner()!!)
                     else baseLocation
 
             val existingImages = ArrayList<String>()
@@ -119,12 +184,13 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
             val finalLocation = existingImages.random(
                 Random(tileGroup.tile.position.hashCode() + locationToCheck.hashCode()))
             val image = ImageGetter.getImage(finalLocation)
+            image.name = finalLocation // for debug mode reveal
 
             tileBaseImages.add(image)
             addActor(image)
 
-            if (strings().tileSetConfig.tileScales.isNotEmpty()) {
-                val scale = strings().tileSetConfig.tileScales[baseLocation.takeLastWhile { it != '/' }]
+            if (strings.tileSetConfig.tileScales.isNotEmpty()) {
+                val scale = strings.tileSetConfig.tileScales[baseLocation.takeLastWhile { it != '/' }]
                 image.setHexagonSize(scale)
             } else {
                 image.setHexagonSize()
@@ -138,21 +204,21 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
         val colorPillagedTile = isViewable && tile.isPillaged() && !usePillagedImprovementImage(tile, viewingCiv)
 
         val baseTerrainColor = when {
-            colorPillagedTile && strings().tileSetConfig.useColorAsBaseTerrain -> tile.getBaseTerrain()
+            colorPillagedTile && strings.tileSetConfig.useColorAsBaseTerrain -> tile.getBaseTerrain()
                 .getColor().lerp(Color.BROWN, 0.6f)
             colorPillagedTile -> Color.WHITE.cpy().lerp(Color.BROWN, 0.6f)
-            strings().tileSetConfig.useColorAsBaseTerrain && !isViewable -> tile.getBaseTerrain()
-                .getColor().lerp(strings().tileSetConfig.fogOfWarColor, 0.6f)
-            strings().tileSetConfig.useColorAsBaseTerrain -> tile.getBaseTerrain()
+            strings.tileSetConfig.useColorAsBaseTerrain && !isViewable -> tile.getBaseTerrain()
+                .getColor().lerp(strings.tileSetConfig.fogOfWarColor, 0.6f)
+            strings.tileSetConfig.useColorAsBaseTerrain -> tile.getBaseTerrain()
                 .getColor()
-            !isViewable -> Color.WHITE.cpy().lerp(strings().tileSetConfig.fogOfWarColor, 0.6f)
+            !isViewable -> Color.WHITE.cpy().lerp(strings.tileSetConfig.fogOfWarColor, 0.6f)
             else -> Color.WHITE.cpy()
         }
 
         val color = when {
             colorPillagedTile -> Color.WHITE.cpy().lerp(Color.RED.cpy(), 0.5f)
             !isViewable -> Color.WHITE.cpy()
-                .lerp(strings().tileSetConfig.fogOfWarColor, 0.6f)
+                .lerp(strings.tileSetConfig.fogOfWarColor, 0.6f)
             else -> Color.WHITE.cpy()
         }
 
@@ -161,9 +227,9 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
     }
 
     private fun updateRivers(displayBottomRight: Boolean, displayBottom: Boolean, displayBottomLeft: Boolean) {
-        bottomRightRiverImage = updateRiver(bottomRightRiverImage,displayBottomRight, strings().bottomRightRiver)
-        bottomRiverImage = updateRiver(bottomRiverImage, displayBottom, strings().bottomRiver)
-        bottomLeftRiverImage = updateRiver(bottomLeftRiverImage, displayBottomLeft, strings().bottomLeftRiver)
+        bottomRightRiverImage = updateRiver(bottomRightRiverImage,displayBottomRight, strings.bottomRightRiver)
+        bottomRiverImage = updateRiver(bottomRiverImage, displayBottom, strings.bottomRiver)
+        bottomLeftRiverImage = updateRiver(bottomLeftRiverImage, displayBottomLeft, strings.bottomLeftRiver)
     }
 
     private fun updateRiver(currentImage: Image?, shouldDisplay: Boolean, imageName: String): Image? {
@@ -206,8 +272,10 @@ class TileLayerTerrain(tileGroup: TileGroup, size: Float) : TileLayer(tileGroup,
         updateRivers(displayBottomRight = false, displayBottom = false, displayBottomLeft = false)
     }
 
-    private fun getNaturalWonderBackupImage(baseHexagon: List<String>): List<String> =
-            if (strings().tileSetConfig.useSummaryImages) baseHexagon + strings().naturalWonder
-            else baseHexagon + strings().orFallback{ getTile(tileGroup.tile.naturalWonder!!) }
+    private fun getNaturalWonderBackupImage(baseHexagon: ArrayList<String>): ArrayList<String> =
+            if (strings.tileSetConfig.useSummaryImages) baseHexagon.apply { add(strings.naturalWonder) }
+            else baseHexagon.apply { add(strings.orFallback{ getTile(tileGroup.tile.naturalWonder!!) }) }
 
 }
+
+data class EdgeTileImage(val fileName:String, val originTileFilter: String, val destinationTileFilter: String, val edgeType: NeighborDirection)
