@@ -5,9 +5,7 @@ import com.badlogic.gdx.files.FileHandle
 import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.graphics.Texture
 import com.badlogic.gdx.graphics.Texture.TextureFilter
-import com.badlogic.gdx.graphics.g2d.NinePatch
-import com.badlogic.gdx.graphics.g2d.TextureAtlas
-import com.badlogic.gdx.graphics.g2d.TextureRegion
+import com.badlogic.gdx.graphics.g2d.*
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Group
 import com.badlogic.gdx.scenes.scene2d.ui.Image
@@ -22,18 +20,11 @@ import com.unciv.json.json
 import com.unciv.models.ruleset.PerpetualConstruction
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
+import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.skins.SkinCache
 import com.unciv.models.tilesets.TileSetCache
-import com.unciv.ui.components.extensions.center
-import com.unciv.ui.components.extensions.centerX
-import com.unciv.ui.components.extensions.centerY
-import com.unciv.ui.components.extensions.setFontColor
-import com.unciv.ui.components.extensions.setFontSize
-import com.unciv.ui.components.extensions.setSize
-import com.unciv.ui.components.extensions.surroundWithCircle
-import com.unciv.ui.components.extensions.surroundWithThinCircle
-import com.unciv.ui.components.extensions.toGroup
-import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.NonTransformGroup
+import com.unciv.ui.components.extensions.*
 import com.unciv.ui.components.fonts.FontRulesetIcons
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.utils.debug
@@ -45,6 +36,7 @@ import kotlin.math.sqrt
 object ImageGetter {
     const val whiteDotLocation = "OtherIcons/whiteDot"
     const val circleLocation = "OtherIcons/Circle"
+    val CHARCOAL = Color(0x111111FF)
 
     // We use texture atlases to minimize texture swapping - see https://yairm210.medium.com/the-libgdx-performance-guide-1d068a84e181
     lateinit var atlas: TextureAtlas
@@ -56,6 +48,8 @@ object ImageGetter {
     private val textureRegionDrawables = HashMap<String, TextureRegionDrawable>()
     private val ninePatchDrawables = HashMap<String, NinePatchDrawable>()
 
+    fun getSpecificAtlas(name: String): TextureAtlas? = atlases[name]
+
     fun resetAtlases() {
         atlases.values.forEach { it.dispose() }
         atlases.clear()
@@ -64,7 +58,10 @@ object ImageGetter {
     fun reloadImages() = setNewRuleset(ruleset)
 
     /** Required every time the ruleset changes, in order to load mod-specific images */
-    fun setNewRuleset(ruleset: Ruleset) {
+    fun setNewRuleset(ruleset: Ruleset, ignoreIfModsAreEqual: Boolean = false) {
+        if (ignoreIfModsAreEqual && ruleset.mods == ImageGetter.ruleset.mods)
+            return
+            
         ImageGetter.ruleset = ruleset
         textureRegionDrawables.clear()
 
@@ -74,7 +71,7 @@ object ImageGetter {
         // These are from the mods
         val visualMods = UncivGame.Current.settings.visualMods + ruleset.mods
         for (mod in visualMods) {
-            loadModAtlases(mod, Gdx.files.local("mods/$mod"))
+            loadModAtlases(mod, UncivGame.Current.files.getModFolder(mod))
         }
 
         TileSetCache.assembleTileSetConfigs(ruleset.mods)
@@ -85,7 +82,7 @@ object ImageGetter {
     }
 
     /** Loads all atlas/texture files from a folder, as controlled by an Atlases.json */
-    fun loadModAtlases(mod: String, folder: FileHandle) {
+    private fun loadModAtlases(mod: String, folder: FileHandle) {
         // See #4993 - you can't .list() on a jar file, so the ImagePacker leaves us the list of actual atlases.
         val controlFile = folder.child("Atlases.json")
         val fileNames = (if (controlFile.exists()) json().fromJson(Array<String>::class.java, controlFile)
@@ -108,6 +105,8 @@ object ImageGetter {
             }
             for (region in tempAtlas.regions) {
                 if (region.name.startsWith("Skins")) {
+                    // TODO: give user a mod warning that the image names has to be [name].9.png
+                    //      if this throws an exception
                     val ninePatch = tempAtlas.createPatch(region.name)
                     ninePatchDrawables[region.name] = NinePatchDrawable(ninePatch)
                 } else {
@@ -181,8 +180,14 @@ object ImageGetter {
      *  @return `null` if no match found.
      */
     fun findExternalImage(name: String): FileHandle? {
-        val folders = ruleset.mods.asSequence().map { Gdx.files.local("mods/$it/ExtraImages") } +
-            sequenceOf(Gdx.files.internal("ExtraImages"))
+        val folders = try { // For CI mod checker, we can't access "local" files
+            // since Gdx files are not set up
+            ruleset.mods.asSequence().map { UncivGame.Current.files.getLocalFile("mods/$it/ExtraImages") } +
+                    sequenceOf(Gdx.files.internal("ExtraImages"))
+        } catch (e: Exception) {
+            debug("Error loading mods: $e")
+            sequenceOf()
+        }
         val extensions = sequenceOf("", ".png", ".jpg")
         return folders.flatMap { folder ->
             extensions.map { folder.child(name + it) }
@@ -204,13 +209,11 @@ object ImageGetter {
     fun getExternalImage(fileName: String) =
         getExternalImage(Gdx.files.internal("ExtraImages/$fileName"))
 
-    fun getImage(fileName: String?): Image {
-        return ImageWithCustomSize(getDrawable(fileName))
-    }
+    fun getImage(fileName: String?, tintColor: Color? = null): Image = 
+        ImageWithCustomSize(getDrawable(fileName)).apply { color = tintColor ?: Color.WHITE }
 
-    fun getDrawable(fileName: String?): TextureRegionDrawable {
-        return textureRegionDrawables[fileName] ?: textureRegionDrawables[whiteDotLocation]!!
-    }
+    fun getDrawable(fileName: String?): TextureRegionDrawable =
+        textureRegionDrawables[fileName] ?: textureRegionDrawables[whiteDotLocation]!!
 
     fun getDrawableOrNull(fileName: String?): TextureRegionDrawable? {
         if (fileName == null)
@@ -231,30 +234,23 @@ object ImageGetter {
     }
 
     fun imageExists(fileName: String) = textureRegionDrawables.containsKey(fileName)
-    fun techIconExists(techName: String) = imageExists("TechIcons/$techName")
-    fun unitIconExists(unitName: String) = imageExists("UnitIcons/$unitName")
     fun ninePatchImageExists(fileName: String) = ninePatchDrawables.containsKey(fileName)
 
-    fun getStatIcon(statName: String): Image {
-        return getImage("StatIcons/$statName")
-                .apply { setSize(20f, 20f) }
-    }
+    fun getStatIcon(statName: String, size: Float = 20f): Image = getImage("StatIcons/$statName")
+            .apply { setSize(size, size) }
 
     fun wonderImageExists(wonderName: String) = imageExists("WonderImages/$wonderName")
     fun getWonderImage(wonderName: String) = getImage("WonderImages/$wonderName")
 
     fun getNationIcon(nation: String) = getImage("NationIcons/$nation")
-    fun getNationPortrait(nation: Nation, size: Float): Portrait {
-        return PortraitNation(nation.name, size)
-    }
+    fun getNationPortrait(nation: Nation, size: Float): Portrait = PortraitNation(nation.name, size)
 
-    fun getRandomNationPortrait(size: Float): Portrait {
-        return PortraitNation(Constants.random, size)
-    }
+    fun getRandomNationPortrait(size: Float): Portrait = PortraitNation(Constants.random, size)
 
-    fun getUnitIcon(unitName: String, color: Color = Color.BLACK): Image {
-        return getImage("UnitIcons/$unitName").apply { this.color = color }
-    }
+    fun getUnitIcon(unit: BaseUnit, color: Color = CHARCOAL): Image =
+        if (imageExists("UnitIcons/${unit.name}"))
+            getImage("UnitIcons/${unit.name}").apply { this.color = color }
+        else getImage("UnitTypeIcons/${unit.type}").apply { this.color = color }
 
     fun getConstructionPortrait(construction: String, size: Float): Group {
         if (ruleset.buildings.containsKey(construction)) {
@@ -268,29 +264,19 @@ object ImageGetter {
         return getStatIcon(construction).surroundWithCircle(size).surroundWithThinCircle()
     }
 
-    fun getUniquePortrait(uniqueName: String, size: Float): Group {
-        return PortraitUnique(uniqueName, size)
-    }
+    fun getUniquePortrait(uniqueName: String, size: Float): Group = PortraitUnique(uniqueName, size)
 
-    fun getPromotionPortrait(promotionName: String, size: Float = 30f): Group {
-        return PortraitPromotion(promotionName, size)
-    }
+    fun getPromotionPortrait(promotionName: String, size: Float = 30f): Group = PortraitPromotion(promotionName, size)
 
-    fun getResourcePortrait(resourceName: String, size: Float, amount: Int = 0): Group {
-        return PortraitResource(resourceName, size, amount)
-    }
+    fun getResourcePortrait(resourceName: String, size: Float, amount: Int= 0): Group =
+        PortraitResource(resourceName, size, amount)
 
-    fun getTechIconPortrait(techName: String, circleSize: Float): Group {
-        return PortraitTech(techName, circleSize)
-    }
+    fun getTechIconPortrait(techName: String, circleSize: Float): Group = PortraitTech(techName, circleSize)
 
-    fun getImprovementPortrait(improvementName: String, size: Float = 20f, dim: Boolean = false, isPillaged: Boolean = false): Portrait {
-        return PortraitImprovement(improvementName, size, dim, isPillaged)
-    }
+    fun getImprovementPortrait(improvementName: String, size: Float = 20f, isPillaged: Boolean = false): Portrait =
+        PortraitImprovement(improvementName, size, false, isPillaged)
 
-    fun getUnitActionPortrait(actionName: String, size: Float = 20f): Portrait {
-        return PortraitUnitAction(actionName, size)
-    }
+    fun getUnitActionPortrait(actionName: String, size: Float = 20f): Portrait = PortraitUnitAction(actionName, size)
 
     fun getReligionIcon(iconName: String): Image { return getImage("ReligionIcons/$iconName") }
     fun getReligionPortrait(iconName: String, size: Float): Portrait {
@@ -304,6 +290,20 @@ object ImageGetter {
 
     fun religionIconExists(iconName: String) = imageExists("ReligionIcons/$iconName")
 
+    /**
+     * Gets the icon associated with the given Victory Type.
+     *
+     * @return The icon, or the Score icon as a fallback.
+     */
+    fun getVictoryTypeIcon(victoryType: String, size: Float, tint: Color = CHARCOAL): Image =
+        if (imageExists("VictoryTypeIcons/$victoryType"))
+            getImage("VictoryTypeIcons/$victoryType").apply { setSize(size, size) }
+        else
+            getImage("OtherIcons/Score").apply {
+                setSize(size, size)
+                this.color = tint
+            }
+
     fun getCircleDrawable() = getDrawable(circleLocation)
     fun getCircle() = getImage(circleLocation)
     fun getCircle(color: Color = Color.WHITE, size: Float? = null) = getCircle().apply {
@@ -320,8 +320,7 @@ object ImageGetter {
         return redCross
     }
 
-    fun getCrossedImage(image: Actor, iconSize: Float) = Group().apply {
-            isTransform = false
+    fun getCrossedImage(image: Actor, iconSize: Float) = NonTransformGroup().apply {
             setSize(iconSize, iconSize)
             image.center(this)
             addActor(image)
@@ -351,7 +350,7 @@ object ImageGetter {
                 .setProgress(progressColor, percentComplete, padding = progressPadding)
     }
 
-    class ProgressBar(width: Float, height: Float, val vertical: Boolean = true) : Group() {
+    class ProgressBar(width: Float, height: Float, val vertical: Boolean = true) : NonTransformGroup() {
 
         var primaryPercentage: Float = 0f
         var secondaryPercentage: Float = 0f
@@ -363,9 +362,10 @@ object ImageGetter {
 
         init {
             setSize(width, height)
-            isTransform = false
         }
 
+        override fun draw(batch: Batch?, parentAlpha: Float) = super.draw(batch, parentAlpha)
+
         fun setLabel(color: Color, text: String, fontSize: Int = Constants.defaultFontSize) : ProgressBar {
             label = text.toLabel()
             label?.setAlignment(Align.center)
@@ -439,12 +439,12 @@ object ImageGetter {
         }
         healthBar.add(healthPartOfBar).size(healthBarSize * healthPercent, height)
 
-        val emptyPartOfBar = getDot(Color.BLACK)
+        val emptyPartOfBar = getDot(CHARCOAL)
         healthBar.add(emptyPartOfBar).size(healthBarSize * (1 - healthPercent), height)
 
         healthBar.pad(1f)
         healthBar.pack()
-        healthBar.background = BaseScreen.skinStrings.getUiBackground("General/HealthBar", tintColor = Color.BLACK)
+        healthBar.background = BaseScreen.skinStrings.getUiBackground("General/HealthBar", tintColor = CHARCOAL)
         return healthBar
     }
 
@@ -479,6 +479,8 @@ object ImageGetter {
         specialist.color = color
         return specialist
     }
+    
+    fun getAllImageNames() = textureRegionDrawables.keys
 
     fun getAvailableSkins() = ninePatchDrawables.keys.asSequence().map { it.split("/")[1] }.distinct()
 
@@ -493,6 +495,8 @@ object ImageGetter {
         .filter { it.startsWith("TileSets") && !it.contains("/Units/") }
         .map { it.split("/")[1] }.distinct()
 
-    fun getAvailableUnitsets() = textureRegionDrawables.keys.asSequence().filter { it.contains("/Units/") }
-        .map { it.split("/")[1] }.distinct()
+    fun getAvailableUnitsets() = textureRegionDrawables.keys.asSequence()
+        .filter { it.startsWith("TileSets") && it.contains("/Units/") }
+        .map { it.split("/")[1] }
+        .distinct()
 }
