@@ -4,13 +4,14 @@ import com.unciv.logic.city.City
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.Ruleset
-import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.extensions.getConsumesAmountString
+import com.unciv.ui.components.extensions.getCostsAmountString
+import com.unciv.ui.components.extensions.toStringSigned
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
 
@@ -27,12 +28,12 @@ object BuildingDescriptions {
             infoList += "+${value.toInt()}% ${key.name.tr()}"
 
         if (requiredNearbyImprovedResources != null)
-            infoList += "Requires worked [" + requiredNearbyImprovedResources!!.joinToString("/") { it.tr() } + "] near city"
+            infoList += "Requires improved [" + requiredNearbyImprovedResources!!.joinToString("/") { it.tr() } + "] near city"
         if (uniques.isNotEmpty()) {
             if (replacementTextForUniques.isNotEmpty()) infoList += replacementTextForUniques
             else infoList += getUniquesStringsWithoutDisablers(uniqueInclusionFilter)
         }
-        if (cityStrength != 0) infoList += "{City strength} +$cityStrength"
+        if (cityStrength != 0.0) infoList += "{City strength} +$cityStrength"
         if (cityHealth != 0) infoList += "{City health} +$cityHealth"
         val separator = if (multiline) "\n" else "; "
         return infoList.joinToString(separator) { it.tr() }
@@ -48,25 +49,33 @@ object BuildingDescriptions {
         if (isWonder) translatedLines += "Wonder".tr()
         if (isNationalWonder) translatedLines += "National Wonder".tr()
         if (!isFree) {
-            for ((resourceName, amount) in getResourceRequirementsPerTurn(StateForConditionals(city.civ, city))) {
-                val available = city.getAvailableResourceAmount(resourceName)
+            // Consumes
+            for ((resourceName, amount) in getResourceRequirementsPerTurn(city.state)) {
+                val available = if (showAdditionalInfo) city.getAvailableResourceAmount(resourceName) else -1
                 val resource = city.getRuleset().tileResources[resourceName] ?: continue
-                val consumesString = resourceName.getConsumesAmountString(amount, resource.isStockpiled())
+                translatedLines += resourceName.getConsumesAmountString(amount, resource.isStockpiled, available).tr()
+            }
 
-                translatedLines += if (showAdditionalInfo) "$consumesString ({[$available] available})".tr()
-                else consumesString.tr()
+            // Costs
+            for ((resourceName, amount) in getStockpiledResourceRequirements(city.state)) {
+                val resource = city.getRuleset().tileResources[resourceName] ?: continue
+                val available = if (showAdditionalInfo) city.getAvailableResourceAmount(resourceName) else -1
+                translatedLines += resourceName.getCostsAmountString(amount, available).tr()
             }
         }
 
         if (uniques.isNotEmpty()) {
             if (replacementTextForUniques.isNotEmpty()) translatedLines += replacementTextForUniques.tr()
-            else translatedLines += getUniquesStringsWithoutDisablers().map { it.tr() }
+            else translatedLines += getUniquesStringsWithoutDisablers {
+                it.type != UniqueType.ConsumesResources &&
+                it.type != UniqueType.CostsResources
+            }.map { it.tr() }
         }
         if (!stats.isEmpty())
             translatedLines += stats.toString()
 
         for ((stat, value) in getStatPercentageBonuses(city))
-            if (value != 0f) translatedLines += "+${value.toInt()}% {${stat.name}}".tr()
+            if (value != 0f) translatedLines += "${value.toInt().toStringSigned()}% {${stat.name}}".tr()
 
         for ((greatPersonName, value) in greatPersonPoints)
             translatedLines += "+$value " + "[$greatPersonName] points".tr()
@@ -75,9 +84,9 @@ object BuildingDescriptions {
             translatedLines += "+$amount " + "[$specialistName] slots".tr()
 
         if (requiredNearbyImprovedResources != null)
-            translatedLines += "Requires worked [${requiredNearbyImprovedResources!!.joinToString("/") { it.tr() }}] near city".tr()
+            translatedLines += "Requires improved [${requiredNearbyImprovedResources!!.joinToString("/") { it.tr() }}] near city".tr()
 
-        if (cityStrength != 0) translatedLines += "{City strength} +$cityStrength".tr()
+        if (cityStrength != 0.0) translatedLines += "{City strength} +$cityStrength".tr()
         if (cityHealth != 0) translatedLines += "{City health} +$cityHealth".tr()
         if (maintenance != 0 && !isFree) translatedLines += "{Maintenance cost}: $maintenance {Gold}".tr()
         if (showAdditionalInfo) additionalDescription(building, city, translatedLines)
@@ -87,15 +96,9 @@ object BuildingDescriptions {
     fun additionalDescription (building: Building, city: City, lines: ArrayList<String>) {
         // Inefficient in theory. In practice, buildings seem to have only a small handful of uniques.
         for (unique in building.uniqueObjects) {
-            if (unique.type == UniqueType.RequiresBuildingInAllCities) {
-                missingCityText(unique.params[0], city, "non-[Puppeted]", lines)
-            }
-
-            else if (unique.type == UniqueType.OnlyAvailable || unique.type == UniqueType.CanOnlyBeBuiltWhen)
-                for (conditional in unique.conditionals) {
-                    if (conditional.type == UniqueType.ConditionalBuildingBuiltAll) {
-                        missingCityText(conditional.params[0], city, conditional.params[1], lines)
-                    }
+            if (unique.type == UniqueType.OnlyAvailable || unique.type == UniqueType.CanOnlyBeBuiltWhen)
+                for (conditional in unique.getModifiers(UniqueType.ConditionalBuildingBuiltAll)) {
+                    missingCityText(conditional.params[0], city, conditional.params[1], lines)
                 }
         }
     }
@@ -122,16 +125,18 @@ object BuildingDescriptions {
      * @param replacementBuilding The "uniqueTo" Building
      */
     fun getDifferences(
-        ruleset: Ruleset, originalBuilding: Building, replacementBuilding: Building
+        originalBuilding: Building, replacementBuilding: Building
     ): Sequence<FormattedLine> = sequence {
-        for ((key, value) in replacementBuilding)
-            if (value != originalBuilding[key])
-                yield(FormattedLine( key.name.tr() + " " +"[${value.toInt()}] vs [${originalBuilding[key].toInt()}]".tr(), indent=1))
+
+        for (stat in Stat.entries) // Do not iterate on object since that excludes zero values
+            if (replacementBuilding[stat] != originalBuilding[stat])
+                yield(FormattedLine( stat.name.tr() + " " +"[${replacementBuilding[stat].toInt()}] vs [${originalBuilding[stat].toInt()}]".tr(), indent=1))
 
         val originalStatBonus = originalBuilding.getStatPercentageBonuses(null)
-        for ((key, value) in replacementBuilding.getStatPercentageBonuses(null))
-            if (value != originalStatBonus[key])
-                yield(FormattedLine("[${value.toInt()}]% ".tr() + key.name.tr() + " vs [${originalStatBonus[key].toInt()}]% ".tr() + key.name.tr(), indent = 1))
+        val replacementStatBonus = replacementBuilding.getStatPercentageBonuses(null)
+        for (stat in Stat.entries)
+            if (replacementStatBonus[stat] != originalStatBonus[stat])
+                yield(FormattedLine("[${replacementStatBonus[stat].toInt()}]% [${stat.name}] vs [${originalStatBonus[stat].toInt()}]% [${stat.name}]", indent=1))
 
         if (replacementBuilding.maintenance != originalBuilding.maintenance)
             yield(FormattedLine("{Maintenance} ".tr() + "[${replacementBuilding.maintenance}] vs [${originalBuilding.maintenance}]".tr(), indent=1))
@@ -147,7 +152,7 @@ object BuildingDescriptions {
         } else {
             val newAbilityPredicate: (Unique)->Boolean = { it.text in originalBuilding.uniques || it.isHiddenToUsers() }
             for (unique in replacementBuilding.uniqueObjects.filterNot(newAbilityPredicate))
-                yield(FormattedLine(unique.text, indent=1))  // FormattedLine(unique) would look worse - no indent and autolinking could distract
+                yield(FormattedLine(unique.getDisplayText(), indent=1))  // FormattedLine(unique) would look worse - no indent and autolinking could distract
         }
 
         val lostAbilityPredicate: (Unique)->Boolean = { it.text in replacementBuilding.uniques || it.isHiddenToUsers() }
@@ -158,7 +163,7 @@ object BuildingDescriptions {
     }
 
     fun getCivilopediaTextLines(building: Building, ruleset: Ruleset): List<FormattedLine> = building.run {
-        fun Float.formatSignedInt() = (if (this > 0f) "+" else "") + this.toInt().toString()
+        fun Float.formatSignedInt() = (if (this > 0f) "+" else "") + this.toInt().tr()
 
         val textList = ArrayList<FormattedLine>()
 
@@ -186,15 +191,19 @@ object BuildingDescriptions {
         if (requiredTech != null)
             textList += FormattedLine("Required tech: [$requiredTech]",
                 link="Technology/$requiredTech")
-        if (requiredBuilding != null)
-            textList += FormattedLine("Requires [$requiredBuilding] to be built in the city",
-                link="Building/$requiredBuilding")
+        if (requiredBuilding != null) {
+            val linkType = if (ruleset.buildings[requiredBuilding]?.isWonder == true) "Wonder" else "Building"
+            textList += FormattedLine(
+                "Requires [$requiredBuilding] to be built in the city",
+                link="$linkType/$requiredBuilding"
+            )
+        }
 
         if (requiredResource != null) {
             textList += FormattedLine()
             val resource = ruleset.tileResources[requiredResource]
             textList += FormattedLine(
-                requiredResource!!.getConsumesAmountString(1, resource!!.isStockpiled()),
+                requiredResource!!.getConsumesAmountString(1, resource!!.isStockpiled),
                 link="Resources/$requiredResource", color="#F42" )
         }
 
@@ -235,14 +244,14 @@ object BuildingDescriptions {
 
         if (requiredNearbyImprovedResources != null) {
             textList += FormattedLine()
-            textList += FormattedLine("Requires at least one of the following resources worked near the city:")
+            textList += FormattedLine("Requires at least one of the following resources improved near the city:")
             requiredNearbyImprovedResources!!.forEach {
                 textList += FormattedLine(it, indent = 1, link = "Resource/$it")
             }
         }
 
-        if (cityStrength != 0 || cityHealth != 0 || maintenance != 0) textList += FormattedLine()
-        if (cityStrength != 0) textList +=  FormattedLine("{City strength} +$cityStrength")
+        if (cityStrength != 0.0 || cityHealth != 0 || maintenance != 0) textList += FormattedLine()
+        if (cityStrength != 0.0) textList +=  FormattedLine("{City strength} +$cityStrength")
         if (cityHealth != 0) textList +=  FormattedLine("{City health} +$cityHealth")
         if (maintenance != 0) textList +=  FormattedLine("{Maintenance cost}: $maintenance {Gold}")
 
@@ -273,7 +282,7 @@ object BuildingDescriptions {
                 if (!tileBonusHashmap.containsKey(stats)) tileBonusHashmap[stats] = ArrayList()
                 tileBonusHashmap[stats]!!.add(unique.params[1])
             }
-            else -> yield(unique.text)
+            else -> yield(unique.getDisplayText())
         }
         for ((key, value) in tileBonusHashmap)
             yield( "[stats] from [tileFilter] tiles in this city"
