@@ -1,17 +1,13 @@
 package com.unciv.ui.screens.basescreen
 
-import com.badlogic.gdx.Gdx
-import com.badlogic.gdx.files.FileHandle
 import com.unciv.Constants
 import com.unciv.UncivGame
-import com.unciv.json.fromJsonFile
-import com.unciv.json.json
 import com.unciv.models.TutorialTrigger
-import com.unciv.models.ruleset.Tutorial
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.Popup
 import com.unciv.ui.screens.civilopediascreen.ICivilopediaText
+import yairm210.purity.annotations.Readonly
 
 
 class TutorialController(screen: BaseScreen) {
@@ -20,30 +16,8 @@ class TutorialController(screen: BaseScreen) {
     private var isTutorialShowing = false
     var allTutorialsShowedCallback: (() -> Unit)? = null
     private val tutorialRender = TutorialRender(screen)
-
-    private val tutorials: LinkedHashMap<String, Tutorial> = loadTutorialsFromJson()
-
-    companion object {
-        // static to allow use from TutorialTranslationTests
-        fun loadTutorialsFromJson(includeMods: Boolean = true): LinkedHashMap<String, Tutorial> {
-            val result = linkedMapOf<String, Tutorial>()
-            for (file in tutorialFiles(includeMods)) {
-                json().fromJsonFile(Array<Tutorial>::class.java, file)
-                    .associateByTo(result) { it.name }
-            }
-            return result
-        }
-
-        private fun tutorialFiles(includeMods: Boolean) = sequence<FileHandle> {
-            yield(Gdx.files.internal("jsons/Tutorials.json"))
-            if (!includeMods) return@sequence
-            val mods = UncivGame.Current.gameInfo?.ruleset?.mods
-                ?: return@sequence
-            val files = mods.asSequence()
-                .map { Gdx.files.local("mods/$it/jsons/Tutorials.json") }
-            yieldAll(files.filter { it.exists() })
-        }
-    }
+    //TODO needs more testing whether this holds in all cases
+    private val tutorials by screen.getCivilopediaRuleset()::tutorials
 
     fun showTutorial(tutorial: TutorialTrigger) {
         tutorialQueue.add(tutorial)
@@ -76,18 +50,11 @@ class TutorialController(screen: BaseScreen) {
         }
     }
 
+    @Readonly
     private fun getTutorial(tutorial: TutorialTrigger): List<String> {
         val name = tutorial.value.replace('_', ' ').trimStart()
         return tutorials[name]?.steps ?: emptyList()
     }
-
-    /** Get all Tutorials to be displayed in the Civilopedia */
-    fun getCivilopediaTutorials(): Collection<ICivilopediaText> {
-        // Todo This is essentially an 'un-private' kludge and the accessor
-        //      in CivilopediaCategories desperately needs independence from TutorialController:
-        //      Move storage to RuleSet someday?
-        return tutorials.values
-    }
 }
 
 
@@ -105,8 +72,9 @@ class TutorialRender(private val screen: BaseScreen) {
         val tutorialPopup = Popup(screen)
         tutorialPopup.name = Constants.tutorialPopupNamePrefix + tutorialName
 
-        if (Gdx.files.internal("ExtraImages/$tutorialName").exists()) {
-            tutorialPopup.add(ImageGetter.getExternalImage(tutorialName)).row()
+        val externalImage = ImageGetter.findExternalImage(tutorialName)
+        if (externalImage != null) {
+            tutorialPopup.add(ImageGetter.getExternalImage(externalImage)).row()
         }
 
         tutorialPopup.addGoodSizedLabel(texts[0]).row()
