@@ -1,18 +1,18 @@
 package com.unciv.ui.screens.cityscreen
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Group
+import com.badlogic.gdx.scenes.scene2d.InputEvent
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Cell
 import com.badlogic.gdx.scenes.scene2d.ui.Table
-import com.badlogic.gdx.scenes.scene2d.ui.TextButton
+import com.badlogic.gdx.scenes.scene2d.utils.ClickListener
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.GUI
 import com.unciv.logic.city.City
 import com.unciv.logic.city.CityConstructions
-import com.unciv.logic.map.tile.Tile
-import com.unciv.models.Religion
 import com.unciv.models.UncivSound
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.IConstruction
@@ -20,7 +20,6 @@ import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.models.ruleset.PerpetualConstruction
 import com.unciv.models.ruleset.RejectionReason
 import com.unciv.models.ruleset.RejectionReasonType
-import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
@@ -32,10 +31,9 @@ import com.unciv.ui.components.extensions.addCell
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.brighten
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.getConsumesAmountString
-import com.unciv.ui.components.extensions.isEnabled
 import com.unciv.ui.components.extensions.packIfNeeded
+import com.unciv.ui.components.extensions.setEnabled
 import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
@@ -49,8 +47,6 @@ import com.unciv.ui.components.widgets.ColorMarkupLabel
 import com.unciv.ui.components.widgets.ExpanderTab
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.CityScreenConstructionMenu
-import com.unciv.ui.popups.Popup
-import com.unciv.ui.popups.closeAllPopups
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.utils.Concurrency
 import com.unciv.utils.launchOnGLThread
@@ -66,19 +62,20 @@ private class ConstructionButtonDTO(
 
 /**
  * Manager to hold and coordinate two widgets for the city screen left side:
- * - Construction queue with the enqueue / buy buttons.
+ * - Construction queue with the buy button.
  *   The queue is scrollable, limited to one third of the stage height.
  * - Available constructions display, scrolling, grouped with expanders and therefore of dynamic height.
  */
 class CityConstructionsTable(private val cityScreen: CityScreen) {
     /* -1 = Nothing, >= 0 queue entry (0 = current construction) */
-    private var selectedQueueEntry = -1 // None
-    private var preferredBuyStat = Stat.Gold  // Used for keyboard buy
+    var selectedQueueEntry = -1 // None
 
     private val upperTable = Table(BaseScreen.skin)
     private val constructionsQueueScrollPane: ScrollPane
     private val constructionsQueueTable = Table()
-    private val buyButtonsTable = Table()
+    private val queueExpander: ExpanderTab
+    private val buttonsTable = Table()
+    private val buyButtonFactory = BuyButtonFactory(cityScreen)
 
     private val lowerTable = Table()
     private val availableConstructionsScrollPane: ScrollPane
@@ -86,8 +83,11 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
     private val lowerTableScrollCell: Cell<ScrollPane>
 
     private val pad = 10f
+    private val miniQueueHeight = 54f
     private val posFromEdge = CityScreen.posFromEdge
     private val stageHeight = cityScreen.stage.height
+    
+    private val highlightColor = Color.GREEN.darken(0.4f)
 
     /** Gets or sets visibility of [both widgets][CityConstructionsTable] */
     var isVisible: Boolean
@@ -102,20 +102,36 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         constructionsQueueScrollPane.setOverscroll(false, false)
         constructionsQueueTable.background = BaseScreen.skinStrings.getUiBackground(
             "CityScreen/CityConstructionTable/ConstructionsQueueTable",
-            tintColor = Color.BLACK
+            tintColor = ImageGetter.CHARCOAL
+        )
+        queueExpander = ExpanderTab(
+            "Construction queue", 
+            onChange = { cityScreen.update() },
+            defaultPad = 0f,
+            // keep lowerTable at fixed position
+            startsOutOpened = false,
+            expanderHeight = miniQueueHeight
         )
 
         upperTable.defaults().left().top()
-        upperTable.add(constructionsQueueScrollPane)
-            .maxHeight(stageHeight / 3 - 10f)
+        
+        // The construction queue is collapsed by default, so there's more vertical space for the
+        // available-constructions-table. When the user decides to expand the
+        // queue, we can use the majority of available vertical space (3/4 of stage height)!
+        // Landscape on Android benefits most from this UX/UI optimization.
+        // Effectively, the available-construction-table takes almost all available vertical space
+        // by default, and by expancing the construction queue, the user changes precedence to the
+        // construction queue.
+        upperTable.add(constructionsQueueScrollPane).padBottom(pad).maxHeight(stageHeight*3/4).row()
+        upperTable.add(buttonsTable)
+            .height("".toTextButton().height) // constant height in order to not let the lowerTable jump
             .padBottom(pad).row()
-        upperTable.add(buyButtonsTable).padBottom(pad).row()
-
+        
         availableConstructionsScrollPane = ScrollPane(availableConstructionsTable.addBorder(2f, Color.WHITE))
         availableConstructionsScrollPane.setOverscroll(false, false)
         availableConstructionsTable.background = BaseScreen.skinStrings.getUiBackground(
             "CityScreen/CityConstructionTable/AvailableConstructionsTable",
-            tintColor = Color.BLACK
+            tintColor = ImageGetter.CHARCOAL
         )
         lowerTableScrollCell = lowerTable.add(availableConstructionsScrollPane).bottom()
         lowerTable.row()
@@ -145,18 +161,36 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         upperTable.pack()
         // Need to reposition when height changes as setPosition's alignment does not persist, it's just a readability shortcut to calculate bottomLeft
         upperTable.setPosition(posFromEdge, stageHeight - posFromEdge, Align.topLeft)
-        lowerTableScrollCell.maxHeight(stageHeight - upperTable.height - 2 * posFromEdge)
+        lowerTableScrollCell.maxHeight(
+            (stageHeight - upperTable.height - 2 * posFromEdge).coerceAtLeast(20f)
+        )
     }
 
     private fun updateButtons(construction: IConstruction?) {
-        buyButtonsTable.clear()
         if (!cityScreen.canChangeState) return
         /** [UniqueType.MayBuyConstructionsInPuppets] support - we need a buy button for civs that could buy items in puppets */
         if (cityScreen.city.isPuppet && !cityScreen.city.getMatchingUniques(UniqueType.MayBuyConstructionsInPuppets).any()) return
-        buyButtonsTable.add(getQueueButton(construction)).padRight(5f)
-        if (construction != null && construction !is PerpetualConstruction)
-            for (button in getBuyButtons(construction as INonPerpetualConstruction))
-                buyButtonsTable.add(button).padRight(5f)
+        buttonsTable.clear()
+        
+        for (button in buyButtonFactory.getBuyButtons(construction)) {
+            buttonsTable.add(button).width(120f).padRight(10f)
+        }
+        // priority buttons and remove button
+        val queue = cityScreen.city.cityConstructions.constructionQueue
+        if (selectedQueueEntry in 0..<queue.size && queue.size > 1) {
+            val constructionName = queue[selectedQueueEntry]
+
+            val raiseButton = getRaisePriorityButton(selectedQueueEntry, constructionName, cityScreen.city)
+            raiseButton.setEnabled(cityScreen.canCityBeChanged() && selectedQueueEntry > 0)
+            buttonsTable.add(raiseButton).padRight(5f)
+
+            val lowerButton = getLowerPriorityButton(selectedQueueEntry, constructionName, cityScreen.city)
+            lowerButton.setEnabled(selectedQueueEntry != queue.lastIndex && cityScreen.canCityBeChanged())
+            buttonsTable.add(lowerButton).padRight(5f)
+            
+            if (cityScreen.canCityBeChanged() && !queueExpander.isOpen && selectedQueueEntry in 1..4)
+                buttonsTable.add(getRemoveFromQueueButton(selectedQueueEntry, cityScreen.city)).padLeft(10f)
+        }
     }
 
     private fun updateConstructionQueue() {
@@ -165,7 +199,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
 
         val city = cityScreen.city
         val cityConstructions = city.cityConstructions
-        val currentConstruction = cityConstructions.currentConstructionFromQueue
+        val currentConstruction = cityConstructions. currentConstructionName()
         val queue = cityConstructions.constructionQueue
 
         constructionsQueueTable.defaults().pad(0f)
@@ -177,29 +211,72 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             constructionsQueueTable.add(getQueueEntry(0, currentConstruction))
                     .expandX().fillX().row()
         else
-            constructionsQueueTable.add("Pick a construction".toLabel()).pad(2f).row()
+            constructionsQueueTable.add("Pick a construction".toLabel()).height(50f).pad(2f).row()
 
-        constructionsQueueTable.addSeparator()
-
-        if (queue.size > 1) {
-            constructionsQueueTable.add(getHeader("Construction queue")).fillX()
-            constructionsQueueTable.addSeparator()
-            queue.forEachIndexed { i, constructionName ->
-                // The first entry is already displayed as "Current construction"
-                if (i != 0) {
-                    constructionsQueueTable.add(getQueueEntry(i, constructionName))
-                        .expandX().fillX().row()
-                    if (i != queue.size - 1)
-                        constructionsQueueTable.addSeparator()
+        // always show queue expander, even when empty, in order to keep lowerTable at constant position
+        queueExpander.innerTable.clear()
+        queueExpander.headerContent.clear()
+        queueExpander.isHeaderIconVisible = queue.size >= 2
+        queueExpander.header.pad(0f)
+        queueExpander.setText("Construction queue".tr())
+        queue.forEachIndexed { i, constructionName ->
+            // The first entry is already displayed as "Current construction"
+            if (i != 0) {
+                queueExpander.innerTable.add(getQueueEntry(i, constructionName))
+                    .expandX().fillX().row()
+                if (i != queue.lastIndex) {
+                    queueExpander.innerTable.addSeparator()
                 }
             }
         }
+        if (!queueExpander.isOpen) {
+            updateQueuePreview(queue)
+        }
+        constructionsQueueTable.add(queueExpander).fillX().pad(2f)
 
         constructionsQueueScrollPane.layout()
         constructionsQueueScrollPane.scrollY = queueScrollY
         constructionsQueueScrollPane.updateVisualScroll()
     }
 
+    private fun updateQueuePreview(queue: MutableList<String>) {
+        queueExpander.header.pad(-5f, 0f, -5f, 0f)
+        val title = when(queue.size) {
+            in 0..1 -> "Queue empty".tr()
+            in 2..4 -> "Queue".tr()
+            else -> ""
+        }
+        queueExpander.setText(title)
+        queue.forEachIndexed { i, constructionName ->
+            if (i in 1..3) {
+                val color = if (selectedQueueEntry == i) highlightColor else BaseScreen.skinStrings.skinConfig.baseColor
+                val image = ImageGetter.getConstructionPortrait(constructionName, 40f).surroundWithCircle(miniQueueHeight, false, color)
+                image.addListener(object: ClickListener() {
+                    // Calling event.stop() to prevent click propagation to the parent,
+                    // the expander header.
+                    // We are using touchDown and touchUp here, because event.stop()
+                    // won't work on the click() callback alone. This is because a click consists
+                    // of both a touch down and touch up.
+                    override fun touchDown(event: InputEvent, x: Float, y: Float, pointer: Int, button: Int): Boolean {
+                        event.stop()
+                        return super.touchDown(event, x, y, pointer, button)
+                    }
+                    override fun touchUp(event: InputEvent, x: Float, y: Float, pointer: Int, button: Int) {
+                        cityScreen.selectConstruction(constructionName)
+                        selectedQueueEntry = i
+                        cityScreen.update()
+                        event.stop()
+                        super.touchUp(event, x, y, pointer, button)
+                    }
+                })
+                queueExpander.headerContent.add(image)
+            }
+            if (i == 4) {
+                queueExpander.headerContent.add("(+{${queue.size - i}})".toLabel())
+            }
+        }
+    }
+
     private fun getConstructionButtonDTOs(): ArrayList<ConstructionButtonDTO> {
         val constructionButtonDTOList = ArrayList<ConstructionButtonDTO>()
 
@@ -215,8 +292,8 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             val useStoredProduction = entry is Building || !cityConstructions.isBeingConstructedOrEnqueued(entry.name)
             val buttonText = cityConstructions.getTurnsToConstructionString(entry, useStoredProduction).trim()
             val resourcesRequired = if (entry is BaseUnit)
-                entry.getResourceRequirementsPerTurn(StateForConditionals(city.civ))
-                else entry.getResourceRequirementsPerTurn(StateForConditionals(city.civ, city))
+                entry.getResourceRequirementsPerTurn(city.civ.state)
+                else entry.getResourceRequirementsPerTurn(city.state)
             val mostImportantRejection =
                     entry.getRejectionReasons(cityConstructions)
                         .filter { it.isImportantRejection() }
@@ -277,7 +354,8 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
                             && dto.rejectionReason?.type == RejectionReasonType.RequiresBuildingInThisCity
                             && constructionButtonDTOList.any {
                                 (it.construction is Building) && (it.construction.name == dto.construction.requiredBuilding
-                                        || it.construction.replaces == dto.construction.requiredBuilding || it.construction.hasUnique(dto.construction.requiredBuilding!!))
+                                        || it.construction.replaces == dto.construction.requiredBuilding
+                                        || it.construction.hasTagUnique(dto.construction.requiredBuilding!!, cityScreen.city.state))
                             })
                         continue
 
@@ -337,11 +415,11 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
                 else cityConstructions.getTurnsToConstructionString(construction, isFirstConstructionOfItsKind)
 
         val constructionResource = if (construction is BaseUnit)
-                construction.getResourceRequirementsPerTurn(StateForConditionals(city.civ, city))
-            else construction.getResourceRequirementsPerTurn(StateForConditionals(city.civ))
+                construction.getResourceRequirementsPerTurn(city.civ.state)
+            else construction.getResourceRequirementsPerTurn(city.state)
         for ((resourceName, amount) in constructionResource) {
             val resource = cityConstructions.city.getRuleset().tileResources[resourceName] ?: continue
-            text += "\n" + resourceName.getConsumesAmountString(amount, resource.isStockpiled()).tr()
+            text += "\n" + resourceName.getConsumesAmountString(amount, resource.isStockpiled).tr()
         }
 
         table.defaults().pad(2f).minWidth(40f)
@@ -350,49 +428,59 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         table.add(ImageGetter.getConstructionPortrait(constructionName, 40f)).padRight(10f)
         table.add(text.toLabel()).expandX().fillX().left()
 
-        if (constructionQueueIndex > 0 && cityScreen.canCityBeChanged())
-            table.add(getRaisePriorityButton(constructionQueueIndex, constructionName, city)).right()
-        else table.add().right()
-        if (constructionQueueIndex != cityConstructions.constructionQueue.lastIndex && cityScreen.canCityBeChanged())
-            table.add(getLowerPriorityButton(constructionQueueIndex, constructionName, city)).right()
-        else table.add().right()
+        if (queueExpander.isOpen) {
+            if (constructionQueueIndex > 0 && cityScreen.canCityBeChanged()){
+                table.add(getRaisePriorityButton(constructionQueueIndex, constructionName, city)).right()}
+            else table.add().right()
+            if (constructionQueueIndex != cityConstructions.constructionQueue.lastIndex && cityScreen.canCityBeChanged())
+                table.add(getLowerPriorityButton(constructionQueueIndex, constructionName, city)).right()
+            else table.add().right()
+        }
 
         if (cityScreen.canCityBeChanged()) table.add(getRemoveFromQueueButton(constructionQueueIndex, city)).right()
         else table.add().right()
 
         table.touchable = Touchable.enabled
 
-        fun selectQueueEntry(onBeforeUpdate: () -> Unit) {
-            cityScreen.selectConstruction(constructionName)
-            selectedQueueEntry = constructionQueueIndex
-            onBeforeUpdate()
-            cityScreen.update()  // Not before CityScreenConstructionMenu or table will have no parent to get stage coords
-            ensureQueueEntryVisible()
+        table.onClick { selectQueueEntry(constructionQueueIndex) }
+        if (cityScreen.canCityBeChanged()) {
+            table.onRightClick {
+                selectQueueEntry(constructionQueueIndex) {
+                    CityScreenConstructionMenu(cityScreen.stage, table, cityScreen.city, construction) {
+                        cityScreen.city.reassignPopulation()
+                        cityScreen.update()
+                    }
+                }
+            }
         }
 
-        table.onClick { selectQueueEntry {} }
-        if (cityScreen.canCityBeChanged())
-            table.onRightClick { selectQueueEntry {
-                CityScreenConstructionMenu(cityScreen.stage, table, cityScreen.city, construction) {
-                    cityScreen.city.reassignPopulation()
-                    cityScreen.update()
-                }
-            } }
-
         return table
     }
 
+    private fun selectQueueEntry(constructionQueueIndex: Int, onBeforeUpdate: () -> Unit = {}) {
+        if (constructionQueueIndex in 0..<cityScreen.city.cityConstructions.constructionQueue.size) {
+            cityScreen.selectConstructionFromQueue(constructionQueueIndex)
+            selectedQueueEntry = constructionQueueIndex
+        } else {
+            cityScreen.clearSelection()
+            selectedQueueEntry = -1
+        }    
+        onBeforeUpdate()
+        cityScreen.update()  // Not before CityScreenConstructionMenu or table will have no parent to get stage coords
+        ensureQueueEntryVisible()
+    }
+
     private fun highlightQueueEntry(queueEntry: Table, highlight: Boolean) {
         queueEntry.background =
             if (highlight)
                 BaseScreen.skinStrings.getUiBackground(
                     "CityScreen/CityConstructionTable/QueueEntrySelected",
-                    tintColor = Color.GREEN.darken(0.5f)
+                    tintColor = highlightColor
                 )
             else
                 BaseScreen.skinStrings.getUiBackground(
                     "CityScreen/CityConstructionTable/QueueEntry",
-                    tintColor = Color.BLACK
+                    tintColor = ImageGetter.CHARCOAL
                 )
     }
 
@@ -426,7 +514,14 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         val resourceTable = Table().apply { isTransform = false }
 
         val textColor = if (constructionButtonDTO.rejectionReason == null) Color.WHITE else Color.RED
-        constructionTable.add(construction.name.toLabel(fontColor = textColor, hideIcons = true).apply { wrap=true })
+
+        val statIcons = if (construction is Building)
+            " " + Stat.entries.filter { construction.isStatRelated(it, cityScreen.city) }.map { it.character }.joinToString("")
+        else ""
+        
+        val constructionNameText = "${construction.name.tr(hideIcons = true)}$statIcons"
+
+        constructionTable.add(constructionNameText.toLabel(fontColor = textColor, hideIcons = true).apply { wrap=true })
             .width(cityScreen.stage.width/5).expandX().left().row()
 
         resourceTable.add(constructionButtonDTO.buttonText.toLabel()).expandX().left()
@@ -434,15 +529,16 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             for ((resource, amount) in constructionButtonDTO.resourcesRequired) {
                 val color = if (constructionButtonDTO.rejectionReason?.type == RejectionReasonType.ConsumesResources)
                     Color.RED else Color.WHITE
-                resourceTable.add(amount.toString().toLabel(fontColor = color)).expandX().left().padLeft(5f)
+                resourceTable.add(amount.tr().toLabel(fontColor = color)).expandX().left().padLeft(5f)
                 resourceTable.add(ImageGetter.getResourcePortrait(resource, 15f)).padBottom(1f)
             }
         }
-        for (unique in constructionButtonDTO.construction.getMatchingUniquesNotConflicting(UniqueType.CostsResources)) {
+        for ((resourceName, amount) in constructionButtonDTO.construction
+            .getStockpiledResourceRequirements(cityScreen.city.state)) {
             val color = if (constructionButtonDTO.rejectionReason?.type == RejectionReasonType.ConsumesResources)
                 Color.RED else Color.WHITE
-            resourceTable.add(ColorMarkupLabel(unique.params[0], color)).expandX().left().padLeft(5f)
-            resourceTable.add(ImageGetter.getResourcePortrait(unique.params[1], 15f)).padBottom(1f)
+            resourceTable.add(ColorMarkupLabel(amount.toString(), color)).expandX().left().padLeft(5f)
+            resourceTable.add(ImageGetter.getResourcePortrait(resourceName, 15f)).padBottom(1f)
         }
         constructionTable.add(resourceTable).expandX().left()
 
@@ -450,7 +546,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
 
         if (!cannotAddConstructionToQueue(construction, cityScreen.city, cityScreen.city.cityConstructions)) {
             val addToQueueButton = ImageGetter.getImage("OtherIcons/New")
-                .apply { color = Color.BLACK }.surroundWithCircle(40f)
+                .apply { color = ImageGetter.CHARCOAL }.surroundWithCircle(40f)
             addToQueueButton.onClick(UncivSound.Silent) {
                 // Since the pickConstructionButton.onClick adds the construction if it's selected,
                 // this effectively adds the construction even if it's unselected
@@ -510,7 +606,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             // Lazy because possibly not needed (highlight true, clearOthers false) and slightly costly
             BaseScreen.skinStrings.getUiBackground(
                 "CityScreen/CityConstructionTable/PickConstructionButton",
-                tintColor = Color.BLACK
+                tintColor = ImageGetter.CHARCOAL
             )
         }
 
@@ -518,7 +614,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             if (highlight)
                 BaseScreen.skinStrings.getUiBackground(
                     "CityScreen/CityConstructionTable/PickConstructionButtonSelected",
-                    tintColor = Color.GREEN.darken(0.5f)
+                    tintColor = highlightColor
                 )
             else unselected
 
@@ -551,38 +647,6 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
                 || city.isPuppet
     }
 
-    private fun getQueueButton(construction: IConstruction?): TextButton {
-        val city = cityScreen.city
-        val cityConstructions = city.cityConstructions
-        val button: TextButton
-
-        if (isSelectedQueueEntry()) {
-            button = "Remove from queue".toTextButton()
-            button.onActivation(binding = KeyboardBinding.AddConstruction) {
-                cityConstructions.removeFromQueue(selectedQueueEntry, false)
-                cityScreen.clearSelection()
-                selectedQueueEntry = -1
-                cityScreen.city.reassignPopulation()
-                cityScreen.update()
-            }
-            if (city.isPuppet)
-                button.disable()
-        } else {
-            button = "Add to queue".toTextButton()
-            if (construction == null
-                    || cannotAddConstructionToQueue(construction, city, cityConstructions)) {
-                button.disable()
-            } else {
-                button.onActivation(binding = KeyboardBinding.AddConstruction, sound = UncivSound.Silent) {
-                    addConstructionToQueue(construction, cityConstructions)
-                }
-            }
-        }
-
-        button.labelCell.pad(5f)
-        return button
-    }
-
     private fun addConstructionToQueue(construction: IConstruction, cityConstructions: CityConstructions) {
         // Some evil person decided to double tap real fast - #4977
         if (cannotAddConstructionToQueue(construction, cityScreen.city, cityConstructions))
@@ -616,167 +680,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             else -> UncivSound.Click
         }
     }
-
-    private fun getBuyButtons(construction: INonPerpetualConstruction?): List<TextButton> {
-        return Stat.statsUsableToBuy.mapNotNull { getBuyButton(construction, it) }
-    }
-
-    private fun getBuyButton(construction: INonPerpetualConstruction?, stat: Stat = Stat.Gold): TextButton? {
-        if (stat !in Stat.statsUsableToBuy || construction == null)
-            return null
-
-        val city = cityScreen.city
-        val button = "".toTextButton()
-
-        if (!isConstructionPurchaseShown(construction, stat)) {
-            // This can't ever be bought with the given currency.
-            // We want one disabled "buy" button without a price for "priceless" buildings such as wonders
-            // We don't want such a button when the construction can be bought using a different currency
-            if (stat != Stat.Gold || construction.canBePurchasedWithAnyStat(city))
-                return null
-            button.setText("Buy".tr())
-            button.disable()
-        } else {
-            val constructionBuyCost = construction.getStatBuyCost(city, stat)!!
-            button.setText("Buy".tr() + " " + constructionBuyCost + stat.character)
-
-            button.onActivation(binding = KeyboardBinding.BuyConstruction) {
-                button.disable()
-                buyButtonOnClick(construction, stat)
-            }
-            button.isEnabled = isConstructionPurchaseAllowed(construction, stat, constructionBuyCost)
-            preferredBuyStat = stat  // Not very intelligent, but the least common currency "wins"
-        }
-
-        button.labelCell.pad(5f)
-
-        return button
-    }
-
-    private fun buyButtonOnClick(construction: INonPerpetualConstruction, stat: Stat = preferredBuyStat) {
-        if (construction !is Building || !construction.hasCreateOneImprovementUnique())
-            return askToBuyConstruction(construction, stat)
-        if (selectedQueueEntry < 0)
-            return cityScreen.startPickTileForCreatesOneImprovement(construction, stat, true)
-        // Buying a UniqueType.CreatesOneImprovement building from queue must pass down
-        // the already selected tile, otherwise a new one is chosen from Automation code.
-        val improvement = construction.getImprovementToCreate(
-            cityScreen.city.getRuleset(), cityScreen.city.civ)!!
-        val tileForImprovement = cityScreen.city.cityConstructions.getTileForImprovement(improvement.name)
-        askToBuyConstruction(construction, stat, tileForImprovement)
-    }
-
-    /** Ask whether user wants to buy [construction] for [stat].
-     *
-     * Used from onClick and keyboard dispatch, thus only minimal parameters are passed,
-     * and it needs to do all checks and the sound as appropriate.
-     */
-    fun askToBuyConstruction(
-        construction: INonPerpetualConstruction,
-        stat: Stat = preferredBuyStat,
-        tile: Tile? = null
-    ) {
-        if (!isConstructionPurchaseShown(construction, stat)) return
-        val city = cityScreen.city
-        val constructionStatBuyCost = construction.getStatBuyCost(city, stat)!!
-        if (!isConstructionPurchaseAllowed(construction, stat, constructionStatBuyCost)) return
-
-        cityScreen.closeAllPopups()
-        ConfirmBuyPopup(construction, stat,constructionStatBuyCost, tile)
-    }
-
-    private inner class ConfirmBuyPopup(
-        construction: INonPerpetualConstruction,
-        stat: Stat,
-        constructionStatBuyCost: Int,
-        tile: Tile?
-    ) : Popup(cityScreen.stage) {
-        init {
-            val city = cityScreen.city
-            val balance = city.getStatReserve(stat)
-            val majorityReligion = city.religion.getMajorityReligion()
-            val yourReligion = city.civ.religionManager.religion
-            val isBuyingWithFaithForForeignReligion = construction.hasUnique(UniqueType.ReligiousUnit) && majorityReligion != yourReligion
-
-            addGoodSizedLabel("Currently you have [$balance] [${stat.name}].").padBottom(10f).row()
-            if (isBuyingWithFaithForForeignReligion) {
-                // Earlier tests should forbid this Popup unless both religions are non-null, but to be safe:
-                fun Religion?.getName() = this?.getReligionDisplayName() ?: Constants.unknownCityName
-                addGoodSizedLabel("You are buying a religious unit in a city that doesn't follow the religion you founded ([${yourReligion.getName()}]). " +
-                    "This means that the unit is tied to that foreign religion ([${majorityReligion.getName()}]) and will be less useful.").row()
-                addGoodSizedLabel("Are you really sure you want to purchase this unit?", Constants.headingFontSize).run {
-                    actor.color = Color.FIREBRICK
-                    padBottom(10f)
-                    row()
-                }
-            }
-            addGoodSizedLabel("Would you like to purchase [${construction.name}] for [$constructionStatBuyCost] [${stat.character}]?").row()
-
-            addCloseButton(Constants.cancel, KeyboardBinding.Cancel) { cityScreen.update() }
-            val confirmStyle = BaseScreen.skin.get("positive", TextButton.TextButtonStyle::class.java)
-            addOKButton("Purchase", KeyboardBinding.Confirm, confirmStyle) {
-                purchaseConstruction(construction, stat, tile)
-            }
-            equalizeLastTwoButtonWidths()
-            open(true)
-        }
-    }
-
-    /** This tests whether the buy button should be _shown_ */
-    private fun isConstructionPurchaseShown(construction: INonPerpetualConstruction, stat: Stat): Boolean {
-        val city = cityScreen.city
-        return construction.canBePurchasedWithStat(city, stat)
-    }
-
-    /** This tests whether the buy button should be _enabled_ */
-    private fun isConstructionPurchaseAllowed(construction: INonPerpetualConstruction, stat: Stat, constructionBuyCost: Int): Boolean {
-        val city = cityScreen.city
-        return when {
-            city.isPuppet && !city.getMatchingUniques(UniqueType.MayBuyConstructionsInPuppets).any() -> false
-            !cityScreen.canChangeState -> false
-            city.isInResistance() -> false
-            !construction.isPurchasable(city.cityConstructions) -> false    // checks via 'rejection reason'
-            construction is BaseUnit && !city.canPlaceNewUnit(construction) -> false
-            city.civ.gameInfo.gameParameters.godMode -> true
-            constructionBuyCost == 0 -> true
-            else -> city.getStatReserve(stat) >= constructionBuyCost
-        }
-}
-
-    /** Called only by askToBuyConstruction's Yes answer - not to be confused with [CityConstructions.purchaseConstruction]
-     * @param tile supports [UniqueType.CreatesOneImprovement]
-     */
-    private fun purchaseConstruction(
-        construction: INonPerpetualConstruction,
-        stat: Stat = Stat.Gold,
-        tile: Tile? = null
-    ) {
-        SoundPlayer.play(stat.purchaseSound)
-        val city = cityScreen.city
-        if (!city.cityConstructions.purchaseConstruction(construction, selectedQueueEntry, false, stat, tile)) {
-            Popup(cityScreen).apply {
-                add("No space available to place [${construction.name}] near [${city.name}]".tr()).row()
-                addCloseButton()
-                open()
-            }
-            return
-        }
-        if (isSelectedQueueEntry() || cityScreen.selectedConstruction?.isBuildable(city.cityConstructions) != true) {
-            selectedQueueEntry = -1
-            cityScreen.clearSelection()
-
-            // Allow buying next queued or auto-assigned construction right away
-            city.cityConstructions.chooseNextConstruction()
-            if (city.cityConstructions.currentConstructionFromQueue.isNotEmpty()) {
-                val newConstruction = city.cityConstructions.getCurrentConstruction()
-                if (newConstruction is INonPerpetualConstruction)
-                    cityScreen.selectConstruction(newConstruction)
-            }
-        }
-        cityScreen.city.reassignPopulation()
-        cityScreen.update()
-    }
-
+    
     private fun getMovePriorityButton(
         arrowDirection: Int,
         binding: KeyboardBinding,
@@ -785,7 +689,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         movePriority: (Int) -> Int
     ): Table {
         val button = Table()
-        button.add(ImageGetter.getArrowImage(arrowDirection).apply { color = Color.BLACK }.surroundWithCircle(40f))
+        button.add(ImageGetter.getArrowImage(arrowDirection).apply { color = ImageGetter.CHARCOAL }.surroundWithCircle(40f))
         button.touchable = Touchable.enabled
         // Don't bind the queue reordering keys here - those should affect only the selected entry, not all of them
         button.onActivation {
@@ -821,7 +725,9 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
             city.cityConstructions.removeFromQueue(constructionQueueIndex, false)
             cityScreen.clearSelection()
             cityScreen.city.reassignPopulation()
-            cityScreen.update()
+            // Select next entry in list if available.
+            // If the last one was deleted, select the new last one.
+            selectQueueEntry(constructionQueueIndex.coerceAtMost(city.cityConstructions.constructionQueue.lastIndex)) { }
         }
         return tab
     }
@@ -839,19 +745,25 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
     }
 
     private fun ensureQueueEntryVisible() {
-        // Ensure the selected queue entry stays visible, and if moved to the "current" top slot, that the header is visible too
-        // This uses knowledge about how we build constructionsQueueTable without re-evaluating that stuff:
-        // Every odd row is a separator, cells have no padding, and there's one header on top and another between selectedQueueEntries 0 and 1
-        val button = constructionsQueueTable.cells[if (selectedQueueEntry == 0) 2 else 2 * selectedQueueEntry + 4].actor
-        val buttonOrHeader = if (selectedQueueEntry == 0) constructionsQueueTable.cells[0].actor else button
-        // The 4f includes the two separators on top/bottom of the entry/header (the y offset we'd need cancels out with constructionsQueueTable.y being 2f as well):
-        val height = buttonOrHeader.y + buttonOrHeader.height - button.y + 4f
-        // Alternatively, scrollTo(..., true, true) would keep the selection as centered as possible:
-        constructionsQueueScrollPane.scrollTo(2f, button.y, button.width, height)
+        getSelectedQueueButton()?.let {
+            constructionsQueueScrollPane.scrollTo(2f, it.y, it.width, it.height, true, true)
+        }
+    }
+
+    private fun getSelectedQueueButton(): Actor? {
+        if (selectedQueueEntry == 0) {
+            return constructionsQueueTable.cells[0].actor
+        }
+        if (selectedQueueEntry > 0 && selectedQueueEntry < cityScreen.city.cityConstructions.constructionQueue.size) {
+            // *2 because it's always the entry and a separator
+            return queueExpander.innerTable.cells[selectedQueueEntry * 2 - 2].actor
+        }
+        return null
     }
 
     private fun resizeAvailableConstructionsScrollPane() {
-        availableConstructionsScrollPane.height = min(availableConstructionsTable.prefHeight, lowerTableScrollCell.maxHeight)
+        availableConstructionsScrollPane.height = 
+            min(availableConstructionsTable.prefHeight, lowerTableScrollCell.maxHeight)
         lowerTable.pack()
     }
 
@@ -860,7 +772,7 @@ class CityConstructionsTable(private val cityScreen: CityScreen) {
         list: ArrayList<Table>,
         prefWidth: Float,
         toggleKey: KeyboardBinding,
-        startsOutOpened: Boolean = true
+        startsOutOpened: Boolean = !cityScreen.isCrampedPortrait()
     ) {
         if (list.isEmpty()) return
 
