@@ -21,6 +21,7 @@ import com.unciv.models.stats.Stat
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.CityAmbiencePlayer
 import com.unciv.ui.audio.SoundPlayer
+import com.unciv.ui.components.ParticleEffectMapFireworks
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.packIfNeeded
@@ -43,6 +44,7 @@ import com.unciv.ui.popups.closeAllPopups
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.worldscreen.WorldScreen
+import kotlin.math.max
 
 class CityScreen(
     internal val city: City,
@@ -131,25 +133,33 @@ class CityScreen(
     var pickTileData: PickTileForImprovementData? = null
     /** A [Building] with [UniqueType.CreatesOneImprovement] has been selected _in the queue_: show the tile it will place the improvement on */
     private var selectedQueueEntryTargetTile: Tile? = null
+    var selectedQueueEntry
+        get() = constructionsTable.selectedQueueEntry
+        set(value) { constructionsTable.selectedQueueEntry = value }
     /** Cached city.expansion.chooseNewTileToOwn() */
     // val should be OK as buying tiles is what changes this, and that would re-create the whole CityScreen
     private val nextTileToOwn = city.expansion.chooseNewTileToOwn()
 
     private var cityAmbiencePlayer: CityAmbiencePlayer?  = ambiencePlayer ?: CityAmbiencePlayer(city)
 
+    /** Particle effects for WLTK day decoration */
+    private val isWLTKday = city.isWeLoveTheKingDayActive()
+    private val fireworks: ParticleEffectMapFireworks?
+    internal var pauseFireworks = false
+
     init {
-        if (city.isWeLoveTheKingDayActive() && UncivGame.Current.settings.citySoundsVolume > 0) {
+        if (isWLTKday && UncivGame.Current.settings.citySoundsVolume > 0) {
             SoundPlayer.play(UncivSound("WLTK"))
         }
+        fireworks = if (isWLTKday) ParticleEffectMapFireworks.create(game, mapScrollPane) else null
 
         UncivGame.Current.settings.addCompletedTutorialTask("Enter city screen")
 
         addTiles()
 
-        //stage.setDebugTableUnderMouse(true)
-        stage.addActor(cityStatsTable)
         // If we are spying then we shoulden't be able to see their construction screen.
         constructionsTable.addActorsToStage()
+        stage.addActor(cityStatsTable)
         stage.addActor(selectedConstructionTable)
         stage.addActor(tileTable)
         stage.addActor(cityPickerTable)  // add late so it's top in Z-order and doesn't get covered in cramped portrait
@@ -158,6 +168,13 @@ class CityScreen(
 
         globalShortcuts.add(KeyboardBinding.PreviousCity) { page(-1) }
         globalShortcuts.add(KeyboardBinding.NextCity) { page(1) }
+
+        if (isPortrait()) mapScrollPane.apply {
+            // center scrolling so city center sits more to the bottom right
+            scrollX = (maxX - constructionsTable.getLowerWidth() - posFromEdge) / 2
+            scrollY = (maxY - cityStatsTable.packIfNeeded().height - posFromEdge + cityPickerTable.top) / 2
+            updateVisualScroll()
+        }
     }
 
     override fun getCivilopediaRuleset() = selectedCiv.gameInfo.ruleset
@@ -173,12 +190,6 @@ class CityScreen(
 
         // Rest of screen: Map of surroundings
         updateTileGroups()
-        if (isPortrait()) mapScrollPane.apply {
-            // center scrolling so city center sits more to the bottom right
-            scrollX = (maxX - constructionsTable.getLowerWidth() - posFromEdge) / 2
-            scrollY = (maxY - cityStatsTable.packIfNeeded().height - posFromEdge + cityPickerTable.top) / 2
-            updateVisualScroll()
-        }
     }
 
     internal fun updateWithoutConstructionAndMap() {
@@ -190,7 +201,7 @@ class CityScreen(
 
         // In portrait mode only: calculate already occupied horizontal space
         val rightMargin = when {
-            !isPortrait() -> 0f
+            !isPortrait() || isCrampedPortrait() -> 0f
             selectedTile != null -> tileTable.packIfNeeded().width
             selectedConstruction != null -> selectedConstructionTable.packIfNeeded().width
             else -> posFromEdge
@@ -207,16 +218,25 @@ class CityScreen(
         cityPickerTable.setPosition(centeredX, exitCityButton.top + 10f, Align.bottom)
 
         // Top right of screen: Stats / Specialists
-        var statsHeight = stage.height - posFromEdge * 2
-        if (selectedTile != null)
-            statsHeight -= tileTable.height + 10f
-        if (selectedConstruction != null)
-            statsHeight -= selectedConstructionTable.height + 10f
-        cityStatsTable.update(statsHeight)
-        cityStatsTable.setPosition(stage.width - posFromEdge, stage.height - posFromEdge, Align.topRight)
+        updateCityStats()
 
         // Top center: Annex/Raze button
         updateAnnexAndRazeCityButton()
+
+    }
+
+    private fun updateCityStats() {
+        var statsHeight = stage.height - posFromEdge * 2
+        if (selectedTile != null)
+            statsHeight -= tileTable.top + 10f
+        if (selectedConstruction != null)
+            statsHeight -= selectedConstructionTable.top + 10f
+        cityStatsTable.update(statsHeight)
+        cityStatsTable.setPosition(
+            stage.width - posFromEdge,
+            stage.height - posFromEdge,
+            Align.topRight
+        )
     }
 
     fun canCityBeChanged(): Boolean {
@@ -244,16 +264,18 @@ class CityScreen(
             val improvementToPlace = pickTileData!!.improvement
             return when {
                 tile.isMarkedForCreatesOneImprovement() -> Color.BROWN to 0.7f
-                !tile.improvementFunctions.canBuildImprovement(improvementToPlace, city.civ) -> Color.RED to 0.4f
+                !tile.improvementFunctions.canBuildImprovement(improvementToPlace, city.state) -> Color.RED to 0.4f
                 isExistingImprovementValuable(tile) -> Color.ORANGE to 0.5f
                 tile.improvement != null -> Color.YELLOW to 0.6f
                 tile.turnsToImprovement > 0 -> Color.YELLOW to 0.6f
                 else -> Color.GREEN to 0.5f
             }
         }
+
         for (tileGroup in tileGroups) {
-            tileGroup.update()
+            tileGroup.update(selectedCiv)
             tileGroup.layerMisc.removeHexOutline()
+            if (isSpying) continue // the rest is only for own cities
 
             if (tileGroup.tileState == CityTileState.BLOCKADED)
                 displayTutorial(TutorialTrigger.CityTileBlockade)
@@ -268,6 +290,9 @@ class CityScreen(
                     getPickImprovementColor(tileGroup.tile).run {
                         tileGroup.layerMisc.addHexOutline(first.cpy().apply { this.a = second }) }
             }
+
+            if (fireworks != null && tileGroup.tile.position == city.location)
+                fireworks.setActorBounds(tileGroup)
         }
     }
 
@@ -277,7 +302,7 @@ class CityScreen(
         fun addWltkIcon(name: String, apply: Image.()->Unit = {}) =
             razeCityButtonHolder.add(ImageGetter.getImage(name).apply(apply)).size(wltkIconSize)
 
-        if (city.isWeLoveTheKingDayActive()) {
+        if (isWLTKday && fireworks == null) {
             addWltkIcon("OtherIcons/WLTK LR") { color = Color.GOLD }
             addWltkIcon("OtherIcons/WLTK 1") { color = Color.FIREBRICK }.padRight(10f)
         }
@@ -309,7 +334,7 @@ class CityScreen(
             razeCityButtonHolder.add(stopRazingCityButton) //.colspan(cityPickerTable.columns)
         }
 
-        if (city.isWeLoveTheKingDayActive()) {
+        if (isWLTKday && fireworks == null) {
             addWltkIcon("OtherIcons/WLTK 2") { color = Color.FIREBRICK }.padLeft(10f)
             addWltkIcon("OtherIcons/WLTK LR") {
                 color = Color.GOLD
@@ -319,17 +344,30 @@ class CityScreen(
         }
 
         razeCityButtonHolder.pack()
-        val centerX = if (!isPortrait()) stage.width / 2
-            else constructionsTable.getUpperWidth().let { it + (stage.width - cityStatsTable.width - it) / 2 }
-        razeCityButtonHolder.setPosition(centerX, stage.height - 20f, Align.top)
+        if(isCrampedPortrait()) {
+            // cramped portrait: move raze button down to city picker
+            val centerX = cityPickerTable.x + cityPickerTable.width / 2 - razeCityButtonHolder.width / 2
+            razeCityButtonHolder.setPosition(centerX, cityPickerTable.y + cityPickerTable.height + 10)
+            // and also re-position the tooltips, which would otherwise be covered
+            tileTable.setPosition(stage.width - posFromEdge, razeCityButtonHolder.top + 10f, Align.bottomRight)
+            selectedConstructionTable.setPosition(stage.width - posFromEdge, razeCityButtonHolder.top + 10f, Align.bottomRight)
+            updateCityStats() // limit city stats height according to the tooltips
+        } else {
+            val centerX = if (isPortrait())
+                constructionsTable.getUpperWidth().let { it + (stage.width - cityStatsTable.width - it) / 2 }
+            else
+                stage.width / 2
+            razeCityButtonHolder.setPosition(centerX, stage.height - 20f, Align.top)
+        }
         stage.addActor(razeCityButtonHolder)
     }
 
     private fun addTiles() {
-        val tileSetStrings = TileSetStrings()
-        val cityTileGroups = city.getCenterTile().getTilesInDistance(5)
+        val viewRange = max(city.getExpandRange(), city.getWorkRange())
+        val tileSetStrings = TileSetStrings(city.civ.gameInfo.ruleset, game.settings)
+        val cityTileGroups = city.getCenterTile().getTilesInDistance(viewRange)
                 .filter { selectedCiv.hasExplored(it) }
-                .map { CityTileGroup(city, it, tileSetStrings) }
+                .map { CityTileGroup(city, it, tileSetStrings, false) }
 
         for (tileGroup in cityTileGroups) {
             tileGroup.onClick { tileGroupOnClick(tileGroup, city) }
@@ -342,8 +380,8 @@ class CityScreen(
         for (tileGroup in tileGroups) {
             val xDifference = city.getCenterTile().position.x - tileGroup.tile.position.x
             val yDifference = city.getCenterTile().position.y - tileGroup.tile.position.y
-            //if difference is bigger than 5 the tileGroup we are looking for is on the other side of the map
-            if (xDifference > 5 || xDifference < -5 || yDifference > 5 || yDifference < -5) {
+            //if difference is bigger than the expansion range the tileGroup we are looking for is on the other side of the map
+            if (xDifference > viewRange || xDifference < -viewRange || yDifference > viewRange || yDifference < -viewRange) {
                 //so we want to unwrap its position
                 tilesToUnwrap.add(tileGroup)
             }
@@ -439,9 +477,11 @@ class CityScreen(
             val pickTileData = this.pickTileData!!
             this.pickTileData = null
             val improvement = pickTileData.improvement
-            if (tileInfo.improvementFunctions.canBuildImprovement(improvement, city.civ)) {
+            if (tileInfo.improvementFunctions.canBuildImprovement(improvement, city.state)
+                && !tileInfo.isMarkedForCreatesOneImprovement()) {
+                
                 if (pickTileData.isBuying) {
-                    constructionsTable.askToBuyConstruction(pickTileData.building, pickTileData.buyStat, tileInfo)
+                    BuyButtonFactory(this).askToBuyConstruction(pickTileData.building, pickTileData.buyStat, tileInfo)
                 } else {
                     // This way to store where the improvement a CreatesOneImprovement Building will create goes
                     // might get a bit fragile if several buildings constructing the same improvement type
@@ -463,6 +503,9 @@ class CityScreen(
     internal fun hasFreeBuilding(building: Building) =
         city.civ.civConstructions.hasFreeBuilding(city, building)
 
+    fun selectConstructionFromQueue(index: Int) {
+        selectConstruction(city.cityConstructions.constructionQueue[index])
+    }
     fun selectConstruction(name: String) {
         selectConstruction(city.cityConstructions.getConstruction(name))
     }
@@ -501,7 +544,7 @@ class CityScreen(
     fun exit() {
         val newScreen = game.popScreen()
         if (newScreen is WorldScreen) {
-            newScreen.mapHolder.setCenterPosition(city.location, immediately = true)
+            newScreen.mapHolder.setCenterPosition(city.location.toHexCoord(), immediately = true)
             newScreen.bottomUnitTable.selectUnit()
         }
     }
@@ -521,6 +564,7 @@ class CityScreen(
         val indexOfCity = viewableCities.indexOf(city)
         val indexOfNextCity = (indexOfCity + delta + numCities) % numCities
         val newCityScreen = CityScreen(viewableCities[indexOfNextCity], ambiencePlayer = passOnCityAmbiencePlayer())
+        newCityScreen.mapScrollPane.zoom(mapScrollPane.scaleX) // Retain zoom
         newCityScreen.update()
         game.replaceCurrentScreen(newCityScreen)
     }
@@ -531,6 +575,13 @@ class CityScreen(
 
     override fun dispose() {
         cityAmbiencePlayer?.dispose()
+        fireworks?.dispose()
         super.dispose()
     }
+
+    override fun render(delta: Float) {
+        super.render(delta)
+        if (pauseFireworks) return
+        fireworks?.render(stage, delta)
+    }
 }
