@@ -7,18 +7,17 @@ import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Button
 import com.badlogic.gdx.scenes.scene2d.ui.SplitPane
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.IHasUniques
-import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.INamed
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.getCloseButton
 import com.unciv.ui.components.extensions.toImageButton
 import com.unciv.ui.components.extensions.toLabel
-import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
@@ -41,21 +40,32 @@ class CivilopediaScreen(
 ) : BaseScreen(), RecreateOnResize {
 
     /** Container collecting data per Civilopedia entry
-     * @param name From [Ruleset] object [INamed.name]
-     * @param image Icon for button
-     * @param flavour [ICivilopediaText]
-     * @param y Y coordinate for scrolling to
-     * @param height Cell height
+     * @property name From [Ruleset] object [INamed.name]
+     * @property image Icon for button
+     * @property flavour Original [ICivilopediaText] reference allowing to render its [ICivilopediaText.civilopediaText]
+     * @property y Y coordinate for scrolling to
+     * @property height Cell height for scrolling to
+     * @property sortBy Optional, enables overriding alphabetical order. Ususally supplied by [ICivilopediaText.getSortGroup]
+     * @property subCategory Optional, enablesg grouping with sub-category labels. Ususally supplied by [ICivilopediaText.getSubCategory]
      */
     private class CivilopediaEntry (
         val name: String,
         val image: Actor? = null,
         val flavour: ICivilopediaText? = null,
-        val y: Float = 0f,              // coordinates of button cell used to scroll to entry
+        val y: Float = 0f,
         val height: Float = 0f,
-        val sortBy: Int = 0             // optional, enabling overriding alphabetical order
+        val sortBy: Int = 0,
+        val subCategory: String? = null
     ) {
-        fun withCoordinates(y: Float, height: Float) = CivilopediaEntry(name, image, flavour, y, height, sortBy)
+        constructor(ruleset: Ruleset, item: ICivilopediaText, category: CivilopediaCategories, imageSize: Float) : this(
+            (item as INamed).name,
+            category.getImage?.invoke(item.getIconName(), imageSize),
+            flavour = item,
+            sortBy = item.getSortGroup(ruleset),
+            subCategory = item.getSubCategory(ruleset)
+        )
+
+        fun withCoordinates(y: Float, height: Float) = CivilopediaEntry(name, image, flavour, y, height, sortBy, subCategory)
     }
 
     private val categoryToEntries = LinkedHashMap<CivilopediaCategories, Collection<CivilopediaEntry>>()
@@ -73,7 +83,7 @@ class CivilopediaScreen(
     private var currentEntry: String = ""
     private val currentEntryPerCategory = HashMap<CivilopediaCategories, String>()
 
-    private val searchPopup by lazy { CivilopediaSearchPopup(this, tutorialController) {
+    private val searchPopup by lazy { CivilopediaSearchPopup(this) {
         selectLink(it)
     } }
 
@@ -118,16 +128,19 @@ class CivilopediaScreen(
         buttonTableScroll.scrollX = buttonInfo.x + (buttonInfo.width - buttonTableScroll.width) / 2
 
         if (category !in categoryToEntries) return        // defense, allowing buggy panes to remain empty while others work
-        var entries = categoryToEntries[category]!!
-        if (category != CivilopediaCategories.Difficulty) // this is the only case where we need them in order
-            // Alphabetical order of localized names, using system default locale
-            entries = entries.sortedWith(
-                compareBy<CivilopediaEntry>{ it.sortBy }
-                    .thenBy (UncivGame.Current.settings.getCollatorFromLocale()) {
-                        // In order for the extra icons on Happiness and Faith to not affect sort order
-                        it.name.tr(true).replace(Fonts.happiness.toString(),"").replace(Fonts.faith.toString(),"") })
+        val entries = categoryToEntries[category]!!
+            // Sort by [CivilopediaEntry.sortBy], then alphabetical order of localized names, using system default locale
+            // Categories that should not sort alphabetically, e.g. Difficulties, will override `getSortGroup`.
+            .sortedWith(
+                compareBy<CivilopediaEntry> { it.sortBy }
+                .thenBy (UncivGame.Current.settings.getCollatorFromLocale()) {
+                    // In order for the extra icons on Happiness and Faith to not affect sort order
+                    it.name.tr(hideIcons = true, hideStats = true)
+                }
+            )
 
         var currentY = -1f
+        var currentSubCategory: String? = null
 
         for (entry in entries) {
             val entryButton = Table().apply {
@@ -143,10 +156,17 @@ class CivilopediaScreen(
                 else
                     entryButton.add(entry.image).padLeft(10f)
             entryButton.left().add(entry.name
-                .toLabel(Color.WHITE, 25, hideIcons=true)).pad(10f)
+                .toLabel(Color.WHITE, 25, hideIcons = true)).pad(10f)
             entryButton.onClick { selectEntry(entry) }
             entryButton.name = entry.name               // make button findable
-            val cell = entrySelectTable.add(entryButton).height(75f).expandX().fillX()
+
+            if (currentSubCategory != entry.subCategory) {
+                if (entry.subCategory != null)
+                    entrySelectTable.add(SubCategoryTable(entry.subCategory)).fillX().row()
+                currentSubCategory = entry.subCategory
+            }
+
+            val cell = entrySelectTable.add(entryButton).height(75f).growX()
             entrySelectTable.row()
             if (currentY < 0f) currentY = cell.padTop
             entryIndex[entry.name] = entry.withCoordinates(currentY, cell.prefHeight)
@@ -155,8 +175,13 @@ class CivilopediaScreen(
 
         entrySelectScroll.layout()          // necessary for positioning in selectRow to work
 
+        // Select the first entry if an entry hasn't already been selected
         val entry = currentEntryPerCategory[category]
-        if (entry != null) selectEntry(entry)
+        if (entry != null) {
+            selectEntry(entry)
+        } else if (entries.isNotEmpty()) {
+            selectEntry(entries.first().name, true)
+        }
     }
 
     /** Select a specified entry within the current category. Unknown strings are ignored!
@@ -199,30 +224,18 @@ class CivilopediaScreen(
     init {
         val imageSize = 50f
 
-        val religionEnabled = showReligionInCivilopedia(ruleset)
-        val victoryTypes = game.gameInfo?.gameParameters?.victoryTypes ?: ruleset.victories.keys
+        val religionEnabled = showReligionInCivilopedia(ruleset) // To filter the Belief Category only
 
-        fun shouldBeDisplayed(obj: IHasUniques): Boolean {
-            return when {
-                obj.hasUnique(UniqueType.HiddenFromCivilopedia) -> false
-                (!religionEnabled && obj.hasUnique(UniqueType.HiddenWithoutReligion)) -> false
-                obj.getMatchingUniques(UniqueType.HiddenWithoutVictoryType).any { !victoryTypes.contains(it.params[0]) } -> false
-                else -> true
-            }
-        }
+        // do not confuse with IConstruction.shouldBeDisplayed - that one tests all prerequisites for building
+        fun shouldBeDisplayed(obj: ICivilopediaText) =
+            obj !is IHasUniques || !obj.isHiddenFromCivilopedia(game.gameInfo, ruleset)
 
-        for (loopCategory in CivilopediaCategories.values()) {
-            if (loopCategory.hide) continue
+        for (loopCategory in CivilopediaCategories.entries) {
             if (!religionEnabled && loopCategory == CivilopediaCategories.Belief) continue
             categoryToEntries[loopCategory] =
-                loopCategory.getCategoryIterator(ruleset, tutorialController)
-                    .filter { (it as? IHasUniques)?.let { obj -> shouldBeDisplayed(obj) } ?: true }
-                    .map { CivilopediaEntry(
-                        (it as INamed).name,
-                        loopCategory.getImage?.invoke(it.getIconName(), imageSize),
-                        flavour = it,
-                        sortBy = it.getSortGroup(ruleset)
-                    ) }
+                loopCategory.getCategoryIterator(ruleset, game.gameInfo)
+                    .filter(::shouldBeDisplayed)
+                    .map { CivilopediaEntry(ruleset, it, loopCategory, imageSize) }
         }
 
         val buttonTable = Table()
@@ -268,7 +281,7 @@ class CivilopediaScreen(
         entrySelectTable.top()
         entrySelectScroll.setOverscroll(false, false)
         val descriptionTable = Table()
-        descriptionTable.add(flavourTable).row()
+        descriptionTable.add(flavourTable).padTop(7f).padBottom(5f).row()  // 2f of that 7f is used up by Portrait painting e.g. a Nation's outer border *outside its bounds*
         val entrySplitPane = SplitPane(entrySelectScroll, ScrollPane(descriptionTable), false, skin)
         entrySplitPane.splitAmount = 0.3f
         entryTable.addActor(entrySplitPane)
@@ -318,13 +331,16 @@ class CivilopediaScreen(
     override fun recreate(): BaseScreen = CivilopediaScreen(ruleset, currentCategory, currentEntry)
 
     companion object {
-        /** Test whether to show Religion-specific items, does not require a game to be running */
-        // Here we decide whether to show Religion in Civilopedia from Main Menu (no gameInfo loaded)
-        fun showReligionInCivilopedia(ruleset: Ruleset? = null) = when {
-            UncivGame.isCurrentInitialized() && UncivGame.Current.gameInfo != null ->
-                UncivGame.Current.gameInfo!!.isReligionEnabled()
-            ruleset != null -> ruleset.beliefs.isNotEmpty()
-            else -> true
+        /** Test whether to show Religion-specific items, does not require a game to be running
+         *  - Do not make public - use IHasUniques.isHiddenFromCivilopedia if possible!
+         */
+        private fun showReligionInCivilopedia(ruleset: Ruleset? = null): Boolean {
+            val gameInfo = UncivGame.getGameInfoOrNull()
+            return when {
+                gameInfo != null -> gameInfo.isReligionEnabled()
+                ruleset != null -> ruleset.beliefs.isNotEmpty()
+                else -> true
+            }
         }
     }
 }
