@@ -6,11 +6,14 @@ import com.badlogic.gdx.scenes.scene2d.ui.Cell
 import com.badlogic.gdx.scenes.scene2d.ui.SelectBox
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
 import com.badlogic.gdx.utils.Align
+import com.unciv.UncivGame
+import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
+import com.unciv.models.ruleset.unique.IHasUniques
 import com.unciv.models.stats.INamed
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.UncivTextField
+import com.unciv.ui.components.widgets.UncivTextField
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.enable
 import com.unciv.ui.components.extensions.toLabel
@@ -30,11 +33,10 @@ import com.badlogic.gdx.utils.Array as GdxArray
 
 class CivilopediaSearchPopup(
     private val pediaScreen: CivilopediaScreen,
-    private val tutorialController: TutorialController,
     private val linkAction: (String) -> Unit
 ) : Popup(pediaScreen) {
     private var ruleset = pediaScreen.ruleset
-    private val searchText = UncivTextField.create("")   // Always focused, "hint" never seen
+    private val searchText = UncivTextField("")   // Always focused, "hint" never seen
     private val modSelect = ModSelectBox()
     private lateinit var resultExpander: ExpanderTab
     private val resultCell: Cell<Actor?>
@@ -80,7 +82,7 @@ class CivilopediaSearchPopup(
                 val regex = Regex(text, RegexOption.IGNORE_CASE)
                 checkLine = { regex.containsMatchIn(it) }
             } catch (ex: Exception) {
-                ToastPopup("Invalid regular expression", pediaScreen, 4000).open(true)
+                ToastPopup("Invalid regular expression", pediaScreen, 4000)
                 searchButton.enable()
                 return
             }
@@ -111,20 +113,22 @@ class CivilopediaSearchPopup(
     }
 
     private fun CoroutineScope.searchLoop() {
-        for (category in CivilopediaCategories.values()) {
+        val gameInfo = UncivGame.getGameInfoOrNull()
+        for (category in CivilopediaCategories.entries) {
             if (!isActive) break
             if (!ruleset.modOptions.isBaseRuleset && category == CivilopediaCategories.Tutorial)
                 continue  // Search tutorials only when the mod filter is a base ruleset
-            for (entry in category.getCategoryIterator(ruleset, tutorialController)) {
+            for (entry in category.getCategoryIterator(ruleset, gameInfo)) {
                 if (!isActive) break
                 if (entry !is INamed) continue
                 if (!ruleset.modOptions.isBaseRuleset) {
                     val sort = entry.getSortGroup(ruleset)
                     if (category == CivilopediaCategories.UnitType && sort < 2)
                         continue  // Search "Domain:" entries only when the mod filter is a base ruleset
-                    if (category == CivilopediaCategories.Belief && sort == 0)
-                        continue  // Search "Religions" from `getCivilopediaReligionEntry` only when the mod filter is a base ruleset
+                    if (category == CivilopediaCategories.Belief && sort == BeliefType.None.ordinal)
+                        continue  // Search "Beliefs" and "Religions" from `getCivilopediaBeliefsEntry`/`getCivilopediaReligionEntry` only when the mod filter is a base ruleset
                 }
+                if (entry is IHasUniques && entry.isHiddenFromCivilopedia(gameInfo, ruleset)) continue
                 searchEntry(entry)
             }
         }
