@@ -1,53 +1,54 @@
 package com.unciv.ui.screens.devconsole
-
 import com.unciv.Constants
 import com.unciv.logic.city.City
-import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.LocationAction
 import com.unciv.logic.civilization.Notification
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.map.mapgenerator.RiverGenerator
 import com.unciv.logic.map.mapgenerator.RiverGenerator.RiverDirections
-import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.tile.Terrain
 import com.unciv.models.ruleset.tile.TerrainType
 
-class ConsoleTileCommands: ConsoleCommandNode {
-    // Note: We *don't* call `TileInfoNormalizer.normalizeToRuleset(selectedTile, console.gameInfo.ruleset)`
+internal class ConsoleTileCommands: ConsoleCommandNode {
+    // Note: We *don't* call `TileNormalizer.normalizeToRuleset(selectedTile, console.gameInfo.ruleset)`
     // - we want the console to allow invalid tile configurations.
 
     override val subcommands = hashMapOf<String, ConsoleCommand>(
 
         "checkfilter" to ConsoleAction("tile checkfilter <tileFilter>") { console, params ->
             val selectedTile = console.getSelectedTile()
-            DevConsoleResponse.hint(selectedTile.matchesFilter(params[0]).toString())
+            DevConsoleResponse.hint(selectedTile.matchesFilter(params[0].originalUnquoted()).toString())
         },
 
         "setimprovement" to ConsoleAction("tile setimprovement <improvementName> [civName]") { console, params ->
             val selectedTile = console.getSelectedTile()
-            val improvement = console.gameInfo.ruleset.tileImprovements.values.findCliInput(params[0])
-                ?: throw ConsoleErrorException("Unknown improvement")
-            var civ: Civilization? = null
-            if (params.size == 2) {
-                civ = console.getCivByName(params[1])
-            }
-            selectedTile.improvementFunctions.changeImprovement(improvement.name, civ)
+            val improvement = params[0].find(console.gameInfo.ruleset.tileImprovements.values)
+            val civ = params.getOrNull(1)?.let { console.getCivByName(it) }
+            selectedTile.improvementFunctions.setImprovement(improvement.name, civ)
             selectedTile.getCity()?.reassignPopulation()
             DevConsoleResponse.OK
         },
 
         "removeimprovement" to ConsoleAction("tile removeimprovement") { console, _ ->
             val selectedTile = console.getSelectedTile()
-            selectedTile.improvementFunctions.changeImprovement(null)
+            selectedTile.improvementFunctions.setImprovement(null)
             selectedTile.getCity()?.reassignPopulation()
             DevConsoleResponse.OK
         },
+        
+        "setpillaged" to ConsoleAction("tile setpillaged <boolean>"){ console, params ->
+            val selectedTile = console.getSelectedTile()
+            val setPillaged = params[0].toBoolean()
+            if (setPillaged) selectedTile.setPillaged()
+            else selectedTile.setRepaired()
+            DevConsoleResponse.OK
+        },
 
         "removeroad" to ConsoleAction("tile removeroad") { console, _ ->
             val selectedTile = console.getSelectedTile()
-            selectedTile.roadStatus = RoadStatus.None
+            selectedTile.removeRoad()
             //todo this covers many cases but not all - do we really need to loop over all civs?
             selectedTile.getOwner()?.cache?.updateCitiesConnectedToCapital()
             DevConsoleResponse.OK
@@ -78,8 +79,7 @@ class ConsoleTileCommands: ConsoleCommandNode {
 
         "setterrain" to ConsoleAction("tile setterrain <terrainName>") { console, params ->
             val selectedTile = console.getSelectedTile()
-            val terrain = console.gameInfo.ruleset.terrains.values.findCliInput(params[0])
-                ?: throw ConsoleErrorException("Unknown terrain")
+            val terrain = params[0].find(console.gameInfo.ruleset.terrains.values)
             if (terrain.type == TerrainType.NaturalWonder)
                 setNaturalWonder(selectedTile, terrain)
             else
@@ -88,9 +88,8 @@ class ConsoleTileCommands: ConsoleCommandNode {
 
         "setresource" to ConsoleAction("tile setresource <resourceName>") { console, params ->
             val selectedTile = console.getSelectedTile()
-            val resource = console.gameInfo.ruleset.tileResources.values.findCliInput(params[0])
-                ?: throw ConsoleErrorException("Unknown resource")
-            selectedTile.resource = resource.name
+            val resource = params[0].find(console.gameInfo.ruleset.tileResources.values)
+            selectedTile.tileResource = resource
             selectedTile.setTerrainTransients()
             selectedTile.getCity()?.reassignPopulation()
             DevConsoleResponse.OK
@@ -98,7 +97,7 @@ class ConsoleTileCommands: ConsoleCommandNode {
 
         "removeresource" to ConsoleAction("tile removeresource") { console, _ ->
             val selectedTile = console.getSelectedTile()
-            selectedTile.resource = null
+            selectedTile.tileResource = null
             selectedTile.setTerrainTransients()
             selectedTile.getCity()?.reassignPopulation()
             DevConsoleResponse.OK
@@ -110,19 +109,7 @@ class ConsoleTileCommands: ConsoleCommandNode {
         "setowner" to ConsoleAction("tile setowner [civName|cityName]") { console, params ->
             val selectedTile = console.getSelectedTile()
             val oldOwner = selectedTile.getCity()
-            val newOwner: City? =
-                if (params.isEmpty() || params[0].isEmpty()) null
-                else {
-                    val param = params[0].toCliInput()
-                    // Look for a city name to assign the Tile to
-                    console.gameInfo.civilizations
-                        .flatMap { civ -> civ.cities }
-                        .firstOrNull { it.name.toCliInput() == param }
-                    // If the user didn't specify a City, they must have given us a Civilization instead -
-                    // copy of TileImprovementFunctions.takeOverTilesAround.fallbackNearestCity
-                    ?: console.getCivByName(params[0]) // throws if no match
-                        .cities.minByOrNull { it.getCenterTile().aerialDistanceTo(selectedTile) + (if (it.isBeingRazed) 5 else 0) }
-                }
+            val newOwner = getOwnerCity(console, params, selectedTile)
             // for simplicity, treat assign to civ without cities same as un-assign
             oldOwner?.expansion?.relinquishOwnership(selectedTile) // redundant if new owner is not null, but simpler for un-assign
             newOwner?.expansion?.takeOwnership(selectedTile)
@@ -132,17 +119,25 @@ class ConsoleTileCommands: ConsoleCommandNode {
         "find" to ConsoleAction("tile find <tileFilter>") { console, params ->
             val filter = params[0]
             val locations = console.gameInfo.tileMap.tileList
-                .filter { it.matchesFilter(filter) }
+                .filter { it.matchesFilter(filter.originalUnquoted()) }  // filters are case sensitive
                 .map { it.position }
             if (locations.isEmpty()) DevConsoleResponse.hint("None found")
             else {
-                val notification = Notification("tile find [$filter]", arrayOf(NotificationIcon.Spy),
-                    LocationAction(locations).asIterable(), NotificationCategory.General)
+                val notification = Notification("tile find ${filter.toStringAsPlaceholder()}: ${locations.size} matches", arrayOf(NotificationIcon.Spy),
+                    locations.map { LocationAction(it) }, NotificationCategory.General)
                 console.screen.notificationsScroll.oneTimeNotification = notification
                 notification.execute(console.screen)
                 DevConsoleResponse.OK
             }
         },
+
+        "spawn-barbarian-encampment" to ConsoleAction("tile spawn-barbarian-encampment") { console, _ ->
+            val selectedTile = console.getSelectedTile()
+            console.gameInfo.barbarians.createNewCamp(selectedTile)
+
+            DevConsoleResponse.OK
+        },
+
     )
 
     private fun setBaseTerrain(tile: Tile, terrain: Terrain): DevConsoleResponse {
@@ -162,7 +157,7 @@ class ConsoleTileCommands: ConsoleCommandNode {
         setBaseTerrain(tile, wonder.turnsInto ?: tile.baseTerrain)
         for (civ in tile.tileMap.gameInfo.civilizations) {
             if (wonder.name in civ.naturalWonders) continue
-            if (civ.isDefeated() || civ.isBarbarian() || civ.isSpectator()) continue
+            if (civ.isDefeated() || civ.isBarbarian || civ.isSpectator()) continue
             if (!civ.hasExplored(tile)) continue
             civ.cache.discoverNaturalWonders()
             civ.updateStatsForNextTurn()
@@ -170,20 +165,19 @@ class ConsoleTileCommands: ConsoleCommandNode {
         return DevConsoleResponse.OK
     }
 
-    private fun getTerrainFeature(console: DevConsolePopup, param: String) =
+    private fun getTerrainFeature(console: DevConsolePopup, param: CliInput) = param.find(
         console.gameInfo.ruleset.terrains.values.asSequence()
-        .filter { it.type == TerrainType.TerrainFeature }.findCliInput(param)
-        ?: throw ConsoleErrorException("Unknown feature")
+            .filter { it.type == TerrainType.TerrainFeature }
+    )
 
     private class ConsoleRiverAction(format: String, newValue: Boolean) : ConsoleAction(
         format,
         action = { console, params -> action(console, params, newValue) }
     ) {
         companion object {
-            private fun action(console: DevConsolePopup, params: List<String>, newValue: Boolean): DevConsoleResponse {
+            private fun action(console: DevConsolePopup, params: List<CliInput>, newValue: Boolean): DevConsoleResponse {
                 val selectedTile = console.getSelectedTile()
-                val direction = findCliInput<RiverDirections>(params[0])
-                    ?: throw ConsoleErrorException("Unknown direction - use " + RiverDirections.names.joinToString())
+                val direction = params[0].enumValue<RiverDirections>()
                 val otherTile = direction.getNeighborTile(selectedTile)
                     ?: throw ConsoleErrorException("tile has no neighbor to the " + direction.name)
                 if (!otherTile.isLand)
@@ -193,4 +187,16 @@ class ConsoleTileCommands: ConsoleCommandNode {
             }
         }
     }
+
+    private fun getOwnerCity(console: DevConsolePopup, params: List<CliInput>, selectedTile: Tile): City? {
+        val param = params.getOrNull(0) ?: return null
+        if (param.isEmpty()) return null
+        // Look for a city by name to assign the Tile to
+        val namedCity = param.findOrNull(console.gameInfo.civilizations.flatMap { civ -> civ.cities })
+        if (namedCity != null) return namedCity
+        // If the user didn't specify a City, they must have given us a Civilization instead
+        val namedCiv = console.getCivByNameOrNull(param)
+            ?: throw ConsoleErrorException("$param is neither a city nor a civilization")
+        return namedCiv.cities.minByOrNull { it.getCenterTile().aerialDistanceTo(selectedTile) + (if (it.isBeingRazed) 5 else 0) }
+    }
 }
