@@ -3,8 +3,8 @@ package com.unciv.ui.screens.devconsole
 
 import com.badlogic.gdx.graphics.Color
 
-@Suppress("DataClassPrivateConstructor") // abuser need to find copy() first
-data class DevConsoleResponse private constructor (
+@ConsistentCopyVisibility  // See https://youtrack.jetbrains.com/issue/KT-11914
+internal data class DevConsoleResponse private constructor (
     val color: Color,
     val message: String? = null,
     val isOK: Boolean = false
