@@ -5,9 +5,11 @@ import com.badlogic.gdx.scenes.scene2d.ui.SplitPane
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
 import com.badlogic.gdx.utils.Align
+import com.unciv.Constants
 import com.unciv.GUI
 import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomacyManager
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
@@ -26,7 +28,7 @@ import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
-import com.unciv.ui.components.tilegroups.InfluenceTable
+import com.unciv.ui.components.tilegroups.citybutton.InfluenceTable
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
@@ -139,31 +141,35 @@ class DiplomacyScreen(
 
             val civIndicator = ImageGetter.getNationPortrait(civ.nation, nationIconSize)
 
-            val relationLevel = civ.getDiplomacyManager(viewingCiv).relationshipLevel()
-            val relationshipIcon = if (civ.isCityState() && relationLevel == RelationshipLevel.Ally)
+            val diplomacy = civ.getDiplomacyManager(viewingCiv)!!
+            val relationLevel = diplomacy.relationshipLevel()
+            val relationshipIcon = if (civ.isCityState && relationLevel == RelationshipLevel.Ally)
                 ImageGetter.getImage("OtherIcons/Star")
                     .surroundWithCircle(size = 30f, color = relationLevel.color).apply {
                         actor.color = Color.GOLD
                     }
             else
                 ImageGetter.getCircle(
-                    color = if (viewingCiv.isAtWarWith(civ)) Color.RED else relationLevel.color,
+                    color = if (civ.isHuman() && viewingCiv.isHuman()) getHumanRelationshipColor(diplomacy)
+                    else if (diplomacy.diplomaticStatus == DiplomaticStatus.DefensivePact) Color.PURPLE
+                    else if (civ.isAtWarWith(viewingCiv)) Color.RED
+                    else relationLevel.color,
                     size = 30f
                 )
             civIndicator.addActor(relationshipIcon)
 
-            if (civ.isCityState()) {
+            if (civ.isCityState) {
                 val innerColor = civ.gameInfo.ruleset.nations[civ.civName]!!.getInnerColor()
                 val typeIcon = ImageGetter.getImage("CityStateIcons/"+civ.cityStateType.name)
                     .surroundWithCircle(size = 35f, color = innerColor).apply {
-                        actor.color = Color.BLACK
+                        actor.color = ImageGetter.CHARCOAL
                     }
                 civIndicator.addActor(typeIcon)
                 typeIcon.y = floor(civIndicator.height - typeIcon.height)
                 typeIcon.x = floor(civIndicator.width - typeIcon.width)
             }
 
-            if (civ.isCityState() && civ.questManager.haveQuestsFor(viewingCiv)) {
+            if (civ.isCityState && civ.questManager.haveQuestsFor(viewingCiv)) {
                 val questIcon = ImageGetter.getImage("OtherIcons/Quest")
                     .surroundWithCircle(size = 30f, color = Color.GOLDENROD)
                 civIndicator.addActor(questIcon)
@@ -205,7 +211,7 @@ class DiplomacyScreen(
         UncivGame.Current.musicController.chooseTrack(otherCiv.civName,
             MusicMood.peaceOrWar(viewingCiv.isAtWarWith(otherCiv)),MusicTrackChooserFlags.setSelectNation)
         rightSideTable.add(ScrollPane(
-            if (otherCiv.isCityState()) CityStateDiplomacyTable(this).getCityStateDiplomacyTable(otherCiv)
+            if (otherCiv.isCityState) CityStateDiplomacyTable(this).getCityStateDiplomacyTable(otherCiv)
             else MajorCivDiplomacyTable(this).getMajorCivDiplomacyTable(otherCiv)
         )).height(stage.height)
     }
@@ -219,11 +225,59 @@ class DiplomacyScreen(
         return tradeTable
     }
 
+    /**
+     * Helper function for updateLeftSideTable() and getHumanRelationshipTable() (human-human relationships only)
+     * @param otherCivDiplomacyManager Other human player [DiplomacyManager]
+     * @return Relationship color between two human players
+     */
+    private fun getHumanRelationshipColor(otherCivDiplomacyManager: DiplomacyManager): Color {
+        // should ensure colors align with equivalent human-AI relationship colors (RelationshipLevel ?)
+        // as of writing, RelationshipLevel colors are not appropriate IMO, so using hardcoded values for now
+        return if (otherCivDiplomacyManager.diplomaticStatus == DiplomaticStatus.DefensivePact)
+            Color.CYAN
+        else if (otherCivDiplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship))
+            Color.GREEN
+        else if (otherCivDiplomacyManager.diplomaticStatus == DiplomaticStatus.War)
+            Color.RED
+        else
+            RelationshipLevel.Neutral.color
+    }
+
+    /**
+     * Human-human relationships only. See also: getHumanRelationshipColor()
+     * @param otherCivDiplomacyManager Other human player [DiplomacyManager]
+     * @return Relationship text (e.g. "Friend")
+     */
+    private fun getHumanRelationshipText(otherCivDiplomacyManager: DiplomacyManager): String {
+        return if (otherCivDiplomacyManager.diplomaticStatus == DiplomaticStatus.DefensivePact)
+            Constants.defensivePact
+        else if (otherCivDiplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship))
+            RelationshipLevel.Friend.name
+        else if (otherCivDiplomacyManager.diplomaticStatus == DiplomaticStatus.War)
+            RelationshipLevel.Enemy.name
+        else
+            RelationshipLevel.Neutral.name
+    }
+
+    /**
+     * @param otherCivDiplomacyManager Other human player [DiplomacyManager]
+     * @return Relationship [Table] for human vs human player only
+     */
+    internal fun getHumanRelationshipTable(otherCivDiplomacyManager: DiplomacyManager): Table {
+        val relationshipTable = Table()
+        val relationshipColor: Color = getHumanRelationshipColor(otherCivDiplomacyManager)
+        val relationshipText: String = getHumanRelationshipText(otherCivDiplomacyManager)
+
+        relationshipTable.add("{Our relationship}: ".toLabel())
+        relationshipTable.add(relationshipText.toLabel(relationshipColor)).row()
+        return relationshipTable
+    }
+
     internal fun getRelationshipTable(otherCivDiplomacyManager: DiplomacyManager): Table {
         val relationshipTable = Table()
 
         val opinionOfUs =
-            if (otherCivDiplomacyManager.civInfo.isCityState()) otherCivDiplomacyManager.getInfluence().toInt()
+            if (otherCivDiplomacyManager.civInfo.isCityState) otherCivDiplomacyManager.getInfluence().toInt()
             else otherCivDiplomacyManager.opinionOfOtherCiv().toInt()
 
         relationshipTable.add("{Our relationship}: ".toLabel())
@@ -238,7 +292,7 @@ class DiplomacyScreen(
         }
 
         relationshipTable.add(relationshipText.toLabel(relationshipColor)).row()
-        if (otherCivDiplomacyManager.civInfo.isCityState())
+        if (otherCivDiplomacyManager.civInfo.isCityState)
             relationshipTable.add(
                 InfluenceTable(
                     otherCivDiplomacyManager.getInfluence(),
@@ -257,7 +311,7 @@ class DiplomacyScreen(
         val turnsToPeaceTreaty = diplomacyManager.turnsToPeaceTreaty()
         if (turnsToPeaceTreaty > 0) {
             declareWarButton.disable()
-            declareWarButton.setText(declareWarButton.text.toString() + " ($turnsToPeaceTreaty${Fonts.turn})")
+            declareWarButton.setText(declareWarButton.text.toString() + " (${turnsToPeaceTreaty.tr()}${Fonts.turn})")
         }
         declareWarButton.onClick {
             ConfirmPopup(this, getDeclareWarButtonText(otherCiv), "Declare war") {
@@ -278,33 +332,26 @@ class DiplomacyScreen(
         messageLines += "Declare war on [${otherCiv.civName}]?"
         // Tell the player who all will join the other side from defensive pacts
         val otherCivDefensivePactList = otherCiv.diplomacy.values.filter {
-            otherCivDiploManager -> otherCivDiploManager.otherCiv() != viewingCiv
+            otherCivDiploManager -> otherCivDiploManager.otherCiv != viewingCiv
             && otherCivDiploManager.diplomaticStatus == DiplomaticStatus.DefensivePact
-            && !otherCivDiploManager.otherCiv().isAtWarWith(viewingCiv) }
-            .map { it.otherCiv() }.toMutableList()
-        // Go through and find all of the defensive pact chains and add them to the list
-        var listIndex = 0
-        while (listIndex < otherCivDefensivePactList.size) {
-            messageLines += if (viewingCiv.knows(otherCivDefensivePactList[listIndex]))
-                "[${otherCivDefensivePactList[listIndex].civName}] will also join them in the war"
-            else "An unknown civilization will also join them in the war"
+            && !otherCivDiploManager.otherCiv.isAtWarWith(viewingCiv) }
+            .map { it.otherCiv }
 
-            // Add their defensive pact allies
-            otherCivDefensivePactList.addAll(otherCivDefensivePactList[listIndex].diplomacy.values
-                .filter { diploChain -> diploChain.diplomaticStatus == DiplomaticStatus.DefensivePact
-                    && !otherCivDefensivePactList.contains(diploChain.otherCiv())
-                    && diploChain.otherCiv() != viewingCiv && diploChain.otherCiv() != otherCiv
-                    && !diploChain.otherCiv().isAtWarWith(viewingCiv) }
-                .map { it.otherCiv() })
-            listIndex++
+        // Defensive pact chains are not allowed now
+        for (civ in otherCivDefensivePactList) {
+            messageLines += if (viewingCiv.knows(civ)) {
+                "[${civ.civName}] will also join them in the war"
+            } else {
+                "[An unknown civilization] will also join them in the war"
+            }
         }
 
         // Tell the player that their defensive pacts will be canceled.
         for (civDiploManager in viewingCiv.diplomacy.values) {
-            if (civDiploManager.otherCiv() != otherCiv
+            if (civDiploManager.otherCiv != otherCiv
                 && civDiploManager.diplomaticStatus == DiplomaticStatus.DefensivePact
-                && !otherCivDefensivePactList.contains(civDiploManager.otherCiv())) {
-                messageLines += "This will cancel your defensive pact with [${civDiploManager.otherCivName}]"
+                && !otherCivDefensivePactList.contains(civDiploManager.otherCiv)) {
+                messageLines += "This will cancel your defensive pact with [${civDiploManager.otherCiv.civName}]"
             }
         }
         return messageLines.joinToString("\n") { "{$it}" }
@@ -338,7 +385,7 @@ class DiplomacyScreen(
         val goToOnMapButton = "Go to on map".toTextButton()
         goToOnMapButton.onClick {
             val worldScreen = UncivGame.Current.resetToWorldScreen()
-            worldScreen.mapHolder.setCenterPosition(civilization.getCapital()!!.location, selectUnit = false)
+            worldScreen.mapHolder.setCenterPosition(civilization.getCapital()!!.location.toHexCoord(), selectUnit = false)
         }
         return goToOnMapButton
     }
