@@ -1,5 +1,7 @@
 ﻿package com.unciv.ui.screens.mainmenuscreen
 
+import com.badlogic.gdx.Gdx
+import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.ui.Stack
@@ -10,11 +12,11 @@ import com.unciv.GUI
 import com.unciv.UncivGame
 import com.unciv.logic.GameInfo
 import com.unciv.logic.GameStarter
+import com.unciv.logic.HolidayDates
 import com.unciv.logic.UncivShowableException
 import com.unciv.logic.map.MapParameters
 import com.unciv.logic.map.MapShape
 import com.unciv.logic.map.MapSize
-import com.unciv.logic.map.MapSizeNew
 import com.unciv.logic.map.MapType
 import com.unciv.logic.map.mapgenerator.MapGenerator
 import com.unciv.models.metadata.BaseRuleset
@@ -26,23 +28,27 @@ import com.unciv.ui.audio.SoundPlayer
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.surroundWithCircle
+import com.unciv.ui.components.extensions.surroundWithThinCircle
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.input.onLongPress
 import com.unciv.ui.components.tilegroups.TileGroupMap
 import com.unciv.ui.components.widgets.AutoScrollPane
 import com.unciv.ui.images.ImageGetter
+import com.unciv.ui.images.padTopDescent
 import com.unciv.ui.popups.Popup
 import com.unciv.ui.popups.ToastPopup
 import com.unciv.ui.popups.closeAllPopups
 import com.unciv.ui.popups.hasOpenPopups
+import com.unciv.ui.popups.options.aboutTab
 import com.unciv.ui.popups.popups
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
-import com.unciv.ui.screens.civilopediascreen.CivilopediaScreen
 import com.unciv.ui.screens.mainmenuscreen.EasterEggRulesets.modifyForEasterEgg
 import com.unciv.ui.screens.mapeditorscreen.EditorMapHolder
 import com.unciv.ui.screens.mapeditorscreen.MapEditorScreen
@@ -57,7 +63,8 @@ import com.unciv.ui.screens.worldscreen.mainmenu.WorldScreenMenuPopup
 import com.unciv.utils.Concurrency
 import com.unciv.utils.launchOnGLThread
 import kotlinx.coroutines.Job
-import kotlin.math.min
+import yairm210.purity.annotations.Pure
+import kotlin.math.ceil
 
 
 class MainMenuScreen: BaseScreen(), RecreateOnResize {
@@ -74,6 +81,10 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
         const val mapFadeTime = 1.3f
         const val mapFirstFadeTime = 0.3f
         const val mapReplaceDelay = 20f
+        /** Inner size of the Civilopedia+Discord+Github buttons (effective size adds 2f for the thin circle) */
+        const val buttonsSize = 60f
+        /** Distance of the Civilopedia and Discord+Github buttons from the stage edges */
+        const val buttonsPosFromEdge = 30f
     }
 
     /** Create one **Main Menu Button** including onClick/key binding
@@ -96,6 +107,7 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
         )
         table.add(ImageGetter.getImage(icon)).size(50f).padRight(20f)
         table.add(text.toLabel(fontSize = 30, alignment = Align.left)).expand().left().minWidth(200f)
+            .padTopDescent()
 
         table.touchable = Touchable.enabled
         table.onActivation(binding = binding) {
@@ -118,9 +130,12 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
         // If we were in a mod, some of the resource images for the background map we're creating
         // will not exist unless we reset the ruleset and images
         val baseRuleset = RulesetCache.getVanillaRuleset()
-        ImageGetter.ruleset = baseRuleset
+        ImageGetter.setNewRuleset(baseRuleset)
 
         if (game.settings.enableEasterEggs) {
+            val holiday = HolidayDates.getHolidayByDate()
+            if (holiday != null)
+                EasterEggFloatingArt(stage, holiday.name)
             val easterEggMod = EasterEggRulesets.getTodayEasterEggRuleset()
             if (easterEggMod != null)
                 easterEggRuleset = RulesetCache.getComplexRuleset(baseRuleset, listOf(easterEggMod))
@@ -166,12 +181,6 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
             { game.pushScreen(ModManagementScreen()) }
         column2.add(modsTable).row()
 
-        if (game.files.getScenarioFiles().any()){
-            val scenarioTable = getMenuButton("Scenarios", "OtherIcons/Scenarios", KeyboardBinding.Scenarios)
-            { game.pushScreen(ScenarioScreen()) }
-            column2.add(scenarioTable).row()
-        }
-
         val optionsTable = getMenuButton("Options", "OtherIcons/Options", KeyboardBinding.MainMenuOptions)
             { openOptionsPopup() }
         optionsTable.onLongPress { openOptionsPopup(withDebug = true) }
@@ -196,50 +205,80 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
             game.popScreen()
         }
 
-        val helpButton = "?".toLabel(fontSize = 48)
+        val civilopediaButton = "?".toLabel(fontSize = 48)
             .apply { setAlignment(Align.center) }
-            .surroundWithCircle(60f, color = skinStrings.skinConfig.baseColor)
-            .apply { actor.y -= 2.5f } // compensate font baseline (empirical)
-            .surroundWithCircle(64f, resizeActor = false)
-        helpButton.touchable = Touchable.enabled
+            .surroundWithCircle(buttonsSize, color = skinStrings.skinConfig.baseColor)
+            .apply { actor.y -= Fonts.getDescenderHeight(48) / 2 } // compensate font baseline
+            .surroundWithThinCircle(Color.WHITE)
+        civilopediaButton.touchable = Touchable.enabled
         // Passing the binding directly to onActivation gives you a size 26 tooltip...
-        helpButton.onActivation { openCivilopedia() }
-        helpButton.keyShortcuts.add(KeyboardBinding.Civilopedia)
-        helpButton.addTooltip(KeyboardBinding.Civilopedia, 30f)
-        helpButton.setPosition(30f, 30f)
-        stage.addActor(helpButton)
+        civilopediaButton.onActivation { openCivilopedia() }
+        civilopediaButton.keyShortcuts.add(KeyboardBinding.Civilopedia)
+        civilopediaButton.addTooltip(KeyboardBinding.Civilopedia, 30f)
+        civilopediaButton.setPosition(buttonsPosFromEdge, buttonsPosFromEdge)
+        stage.addActor(civilopediaButton)
+
+        val rightSideButtons = Table().apply { defaults().space(10f) }
+        val discordButton = ImageGetter.getImage("OtherIcons/Discord")
+            .surroundWithCircle(buttonsSize, color = skinStrings.skinConfig.baseColor)
+            .surroundWithThinCircle(Color.WHITE)
+            .onActivation { Gdx.net.openURI("https://discord.gg/bjrB4Xw") }
+        rightSideButtons.add(discordButton)
+
+        val githubButton = ImageGetter.getImage("OtherIcons/Github")
+            .surroundWithCircle(buttonsSize, color = skinStrings.skinConfig.baseColor)
+            .surroundWithThinCircle(Color.WHITE)
+            .onActivation { Gdx.net.openURI(Constants.uncivRepoURL) }
+        rightSideButtons.add(githubButton)
+
+        rightSideButtons.pack()
+        rightSideButtons.setPosition(stage.width - buttonsPosFromEdge, buttonsPosFromEdge, Align.bottomRight)
+        stage.addActor(rightSideButtons)
+
+        val versionLabel = "{Version} ${UncivGame.VERSION.text}".toLabel()
+        versionLabel.setAlignment(Align.center)
+        val versionTable = Table()
+        versionTable.background = skinStrings.getUiBackground("MainMenuScreen/Version",
+            skinStrings.roundedEdgeRectangleShape, Color.DARK_GRAY.cpy().apply { a = 0.7f })
+        versionTable.add(versionLabel)
+        versionTable.pack()
+        versionTable.setPosition(stage.width / 2, 10f, Align.bottom)
+        versionTable.touchable = Touchable.enabled
+        versionTable.onClick {
+            val popup = Popup(stage)
+            popup.add(aboutTab()).row()
+            popup.addCloseButton()
+            popup.open()
+        }
+        stage.addActor(versionTable)
     }
 
     private fun startBackgroundMapGeneration() {
         stopBackgroundMapGeneration()  // shouldn't be necessary as resize re-instantiates this class
         backgroundMapGenerationJob = Concurrency.run("ShowMapBackground") {
-            var scale = 1f
-            var mapWidth = stage.width / TileGroupMap.groupHorizontalAdvance
-            var mapHeight = stage.height / TileGroupMap.groupSize
-            if (mapWidth * mapHeight > 3000f) {  // 3000 as max estimated number of tiles is arbitrary (we had typically 721 before)
-                scale = mapWidth * mapHeight / 3000f
-                mapWidth /= scale
-                mapHeight /= scale
-                scale = min(scale, 20f)
-            }
+            // MapSize.Small has easily enough tiles to fill the entire background - unless the user sized their window to some extreme aspect ratio
+            val mapWidth = stage.width / TileGroupMap.groupHorizontalAdvance
+            val mapHeight = stage.height / TileGroupMap.groupSize
+            @Pure fun Float.scaleCoord(scale: Float) = ceil(this * scale).toInt().coerceAtLeast(6)
+            // These scale values are chosen so that a common 4:3 screen minus taskbar gives the same as MapSize.Small
+            val backgroundMapSize = MapSize(mapWidth.scaleCoord(.77f), mapHeight.scaleCoord(1f))
 
             val newMap = MapGenerator(backgroundMapRuleset, this)
                 .generateMap(MapParameters().apply {
                     shape = MapShape.rectangular
-                    mapSize = MapSizeNew(MapSize.Small)
+                    mapSize = backgroundMapSize
                     type = MapType.pangaea
-                    temperatureExtremeness = .7f
+                    temperatureintensity = .7f
                     waterThreshold = -0.1f // mainly land, gets about 30% water
                     modifyForEasterEgg()
                 })
 
             launchOnGLThread { // for GL context
-                ImageGetter.setNewRuleset(backgroundMapRuleset)
+                ImageGetter.setNewRuleset(backgroundMapRuleset, ignoreIfModsAreEqual = true)
                 val mapHolder = EditorMapHolder(
                     this@MainMenuScreen,
                     newMap
                 ) {}
-                mapHolder.setScale(scale)
                 mapHolder.color = mapHolder.color.cpy()
                 mapHolder.color.a = 0f
                 backgroundStack.add(mapHolder)
@@ -285,7 +324,6 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
             } else {
                 GUI.resetToWorldScreen()
                 GUI.getWorldScreen().popups.filterIsInstance<WorldScreenMenuPopup>().forEach(Popup::close)
-                ImageGetter.ruleset = game.gameInfo!!.ruleset
             }
         } else {
             QuickSave.autoLoadGame(this)
@@ -357,6 +395,10 @@ class MainMenuScreen: BaseScreen(), RecreateOnResize {
         return MainMenuScreen()
     }
 
+    override fun resume() {
+        startBackgroundMapGeneration()
+    }
+
     // We contain a map...
     override fun getShortcutDispatcherVetoer() = KeyShortcutDispatcherVeto.createTileGroupMapDispatcherVetoer()
 }
