@@ -11,10 +11,10 @@ import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable
 import com.unciv.UncivGame
+import com.unciv.logic.files.MapSaver
 import com.unciv.logic.map.MapParameters
 import com.unciv.logic.map.MapShape
 import com.unciv.logic.map.MapSize
-import com.unciv.logic.map.MapSizeNew
 import com.unciv.logic.map.TileMap
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.metadata.BaseRuleset
@@ -22,7 +22,7 @@ import com.unciv.models.metadata.GameParameters
 import com.unciv.models.metadata.GameSetupInfo
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
-import com.unciv.ui.components.UncivTextField
+import com.unciv.ui.components.widgets.UncivTextField
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
 import com.unciv.ui.components.input.KeyboardPanningListener
@@ -41,6 +41,8 @@ import com.unciv.utils.Dispatcher
 import com.unciv.utils.Log
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.Job
+import kotlinx.coroutines.delay
+import kotlinx.coroutines.isActive
 
 
 //todo normalize properly
@@ -86,7 +88,7 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
     val tabs: MapEditorMainTabs
     var tileClickHandler: ((tile: Tile)->Unit)? = null
     private var zoomController: ZoomButtonPair? = null
-    val descriptionTextField = UncivTextField.create("Enter a description for the users of this map")
+    val descriptionTextField = UncivTextField("Enter a description for the users of this map")
 
     private val highlightedTileGroups = mutableListOf<TileGroup>()
 
@@ -97,7 +99,7 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
         if (map == null) {
             ruleset = RulesetCache[BaseRuleset.Civ_V_GnK.fullName]!!
             tileMap = TileMap(MapSize.Tiny.radius, ruleset, false).apply {
-                mapParameters.mapSize = MapSizeNew(MapSize.Tiny)
+                mapParameters.mapSize = MapSize.Tiny
             }
         } else {
             ruleset = map.ruleset ?: RulesetCache.getComplexRuleset(map.mapParameters)
@@ -119,6 +121,30 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
         globalShortcuts.add(KeyCharAndCode.ctrl('n')) { selectGeneratePage(0) }
         globalShortcuts.add(KeyCharAndCode.ctrl('g')) { selectGeneratePage(1) }
         globalShortcuts.add(KeyCharAndCode.BACK) { closeEditor() }
+
+        startMapAutosaveLoop()
+    }
+
+    private fun startMapAutosaveLoop() {
+        startBackgroundJob("MapEditorAutosaveLoop", true) {
+            while (true) {
+                delay(10_000L) // every 10 seconds
+
+                if (!isActive) return@startBackgroundJob
+                if (!isDirty) continue
+
+                try {
+                    val mapToSave = getMapCloneForSave()
+                    if (!isActive) return@startBackgroundJob
+                    mapToSave.assignContinents(TileMap.AssignContinentsMode.Reassign)
+                    if (!isActive) return@startBackgroundJob
+                    MapSaver.saveMap("Autosave", mapToSave)
+                    isDirty = false
+                } catch (ex: Exception) {
+                    Log.error("Failed to autosave map", ex)
+                }
+            }
+        }
     }
 
     override fun getCivilopediaRuleset() = ruleset
@@ -186,10 +212,8 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
         val newHolder = EditorMapHolder(this, tileMap) {
             tileClickHandler?.invoke(it)
         }
-        for (oldPanningListener in stage.root.listeners.filterIsInstance<KeyboardPanningListener>())
-            stage.removeListener(oldPanningListener)  // otherwise they accumulate
         newHolder.mapPanningSpeed = UncivGame.Current.settings.mapPanningSpeed
-        stage.addListener(KeyboardPanningListener(newHolder, allowWASD = false))
+        enableKeyboardPanningListener(newHolder, true)
         if (Gdx.app.type == Application.ApplicationType.Desktop)
             newHolder.isAutoScrollEnabled = UncivGame.Current.settings.mapAutoScroll
 
@@ -212,6 +236,15 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
         return newHolder
     }
 
+    internal fun enableKeyboardPanningListener(holder: EditorMapHolder? = null, enable: Boolean) {
+        for (oldPanningListener in stage.root.listeners.filterIsInstance<KeyboardPanningListener>()) {
+            stage.removeListener(oldPanningListener)  // otherwise they accumulate
+            oldPanningListener.dispose()
+        }
+        if (!enable) return
+        stage.addListener(KeyboardPanningListener(holder ?: mapHolder, allowWASD = false))
+    }
+
     // We contain a map...
     override fun getShortcutDispatcherVetoer() = KeyShortcutDispatcherVeto.createTileGroupMapDispatcherVetoer()
 
@@ -302,10 +335,11 @@ class MapEditorScreen(map: TileMap? = null) : BaseScreen(), RecreateOnResize {
     }
 
     private fun cancelJobs() {
-        for (job in jobs)
+        for (job in jobs.toList())
             job.cancel()
         jobs.clear()
     }
+    
 
     //region Overlay Image
 
