@@ -1,7 +1,6 @@
 package com.unciv.ui.screens.mapeditorscreen.tabs
 
 import com.badlogic.gdx.graphics.Color
-import com.badlogic.gdx.scenes.scene2d.Group
 import com.badlogic.gdx.scenes.scene2d.ui.ButtonGroup
 import com.badlogic.gdx.scenes.scene2d.ui.CheckBox
 import com.badlogic.gdx.scenes.scene2d.ui.Table
@@ -12,16 +11,13 @@ import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
-import com.unciv.models.ruleset.tile.ResourceType
-import com.unciv.models.ruleset.tile.Terrain
-import com.unciv.models.ruleset.tile.TerrainType
-import com.unciv.models.ruleset.tile.TileImprovement
-import com.unciv.models.ruleset.tile.TileResource
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.tile.*
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.MusicMood
 import com.unciv.ui.audio.MusicTrackChooserFlags
+import com.unciv.ui.components.NonTransformGroup
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.input.onClick
@@ -61,7 +57,7 @@ class MapEditorEditTerrainTab(
 
     private fun allTerrains() = ruleset.terrains.values.asSequence()
         .filter { it.type.isBaseTerrain }
-        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, StateForConditionals.IgnoreConditionals) }
+        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, GameContext.IgnoreConditionals) }
     private fun getTerrains() = allTerrains()
         .map { FormattedLine(it.name, it.name, "Terrain/${it.name}", size = 32) }
         .asIterable()
@@ -103,7 +99,7 @@ class MapEditorEditFeaturesTab(
 
     private fun allowedFeatures() = ruleset.terrains.values.asSequence()
         .filter { it.type == TerrainType.TerrainFeature }
-        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, StateForConditionals.IgnoreConditionals) }
+        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, GameContext.IgnoreConditionals) }
     private fun getFeatures() = allowedFeatures()
         .map { FormattedLine(it.name, it.name, "Terrain/${it.name}", size = 32) }
         .asIterable()
@@ -137,7 +133,7 @@ class MapEditorEditWondersTab(
 
     private fun allowedWonders() = ruleset.terrains.values.asSequence()
         .filter { it.type == TerrainType.NaturalWonder }
-        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, StateForConditionals.IgnoreConditionals) }
+        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, GameContext.IgnoreConditionals) }
     private fun getWonders() = allowedWonders()
         .map { FormattedLine(it.name, it.name, "Terrain/${it.name}", size = 32) }
         .asIterable()
@@ -162,7 +158,7 @@ class MapEditorEditResourcesTab(
         val eraser = FormattedLine("Remove resource", icon = eraserIcon, size = 32, iconCrossed = true)
         add(eraser.render(0f).apply { onClick {
             editTab.setBrush("Remove resource", eraserIcon, pediaLink = "", isRemove = true) { tile ->
-                tile.resource = null
+                tile.tileResource = null
                 tile.resourceAmount = 0
             }
         } }).padBottom(0f).row()
@@ -173,7 +169,7 @@ class MapEditorEditResourcesTab(
         ) { resourceName ->
             val resource = ruleset.tileResources[resourceName]!!
             editTab.setBrush(resourceName, resource.makeLink()) {
-                if (it.resource == resourceName && resource.resourceType == ResourceType.Strategic)
+                if (it.tileResource == resource && resource.resourceType == ResourceType.Strategic)
                     it.resourceAmount = (it.resourceAmount + 1).coerceAtMost(42)
                 else
                     it.setTileResource(resource, rng = editTab.randomness.RNG)
@@ -183,7 +179,7 @@ class MapEditorEditResourcesTab(
 
     private fun allowedResources() = ruleset.tileResources.values.asSequence()
         .filterNot { it.hasUnique(UniqueType.CityStateOnlyResource) }
-        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, StateForConditionals.IgnoreConditionals) }
+        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, GameContext.IgnoreConditionals) }
     private fun getResources(): Iterable<FormattedLine> = sequence {
         var lastGroup = ResourceType.Bonus
         for (resource in allowedResources()) {
@@ -225,20 +221,20 @@ class MapEditorEditImprovementsTab(
             getImprovements(),
             iconDisplay = FormattedLine.IconDisplay.NoLink
         ) {
-            val road = RoadStatus.values().firstOrNull { r -> r.name == it }
+            val road = RoadStatus.entries.firstOrNull { r -> r.name == it }
             if (road != null)
-                editTab.setBrush(BrushHandlerType.Road, it, "Improvement/$it") { tile ->
-                    tile.roadStatus = if (tile.roadStatus == road) RoadStatus.None else road
+                editTab.setBrush(it, "Improvement/$it", handlerType = BrushHandlerType.Road) { tile ->
+                    tile.setRoadStatus(if (tile.roadStatus == road) RoadStatus.None else road, null)
                 }
             else
                 editTab.setBrush(it, "Improvement/$it") { tile ->
-                    tile.changeImprovement(it)
+                    tile.setImprovement(it)
                 }
         }).padTop(0f).row()
     }
 
     private fun allowedImprovements() = ruleset.tileImprovements.values.asSequence()
-        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, StateForConditionals.IgnoreConditionals) }
+        .filterNot { it.hasUnique(UniqueType.ExcludedFromMapEditor, GameContext.IgnoreConditionals) }
     private fun getImprovements(): Iterable<FormattedLine> = sequence {
         var lastGroup = 0
         for (improvement in allowedImprovements()) {
@@ -256,7 +252,7 @@ class MapEditorEditImprovementsTab(
 
     companion object {
         private fun TileImprovement.group() = when {
-            RoadStatus.values().any { it.name == name } -> 2
+            RoadStatus.entries.any { it.name == name } -> 2
             "Great Improvement" in uniques -> 3
             uniqueTo != null -> 4
             "Unpillagable" in uniques -> 5
@@ -286,7 +282,7 @@ class MapEditorEditStartsTab(
         val eraserIcon = "Nation/${firstNation.name}"
         val eraser = FormattedLine("Remove starting locations", icon = eraserIcon, size = 24, iconCrossed = true)
         add(eraser.render(0f).apply { onClick {
-            editTab.setBrush(BrushHandlerType.Direct, "Remove", eraserIcon, pediaLink = "", isRemove = true) { tile ->
+            editTab.setBrush("Remove", eraserIcon, handlerType = BrushHandlerType.Direct, pediaLink = "", isRemove = true) { tile ->
                 tile.tileMap.removeStartingLocations(tile.position)
             }
         } }).padBottom(0f).row()
@@ -304,9 +300,9 @@ class MapEditorEditStartsTab(
                 val icon = "Nation/$it"
                 val pediaLink = if (it == Constants.spectator) "" else icon
                 val isMajorCiv = ruleset.nations[it]?.isMajorCiv ?: false
-                val selectedUsage = if (isMajorCiv) TileMap.StartingLocation.Usage.values()[usageOptionGroup.checkedIndex]
+                val selectedUsage = if (isMajorCiv) TileMap.StartingLocation.Usage.entries[usageOptionGroup.checkedIndex]
                     else TileMap.StartingLocation.Usage.Normal
-                editTab.setBrush(BrushHandlerType.Direct, it.spectatorToAnyCiv(), icon, pediaLink) { tile ->
+                editTab.setBrush(it.spectatorToAnyCiv(), icon, BrushHandlerType.Direct, pediaLink) { tile ->
                     // toggle the starting location here, note this allows
                     // both multiple locations per nation and multiple nations per tile
                     if (!tile.tileMap.addStartingLocation(it, tile, selectedUsage))
@@ -332,7 +328,7 @@ class MapEditorEditStartsTab(
         table.defaults().pad(5f)
         table.add("Use for new game \"Select players\" button:".toLabel()).colspan(3).row()
         val defaultUsage = TileMap.StartingLocation.Usage.default
-        for (usage in TileMap.StartingLocation.Usage.values()) {
+        for (usage in TileMap.StartingLocation.Usage.entries) {
             val checkBox = CheckBox(usage.label.tr(), skin)
             table.add(checkBox)
             usageOptionGroup.add(checkBox)
@@ -372,8 +368,8 @@ class MapEditorEditRiversTab(
                     tile.hasBottomRightRiver = false
                     tile.hasBottomRiver = false
                     // User probably expects all six edges to be cleared
-                    val x = tile.position.x.toInt()
-                    val y = tile.position.y.toInt()
+                    val x = tile.position.x
+                    val y = tile.position.y
                     tile.tileMap.getIfTileExistsOrNull(x, y + 1)?.hasBottomLeftRiver = false
                     tile.tileMap.getIfTileExistsOrNull(x + 1, y)?.hasBottomRightRiver = false
                     tile.tileMap.getIfTileExistsOrNull(x + 1, y + 1)?.hasBottomRiver = false
@@ -447,7 +443,7 @@ class MapEditorEditRiversTab(
     private fun Tile.makeTileGroup(): TileGroup {
         ruleset = this@MapEditorEditRiversTab.ruleset
         setTerrainTransients()
-        return TileGroup(this, TileSetStrings(), iconSize * 36f/54f).apply {
+        return TileGroup(this, TileSetStrings(ruleset, UncivGame.Current.settings), iconSize * 36f/54f).apply {
             isForceVisible = true
             isForMapEditorIcon = true
             update()
@@ -471,9 +467,8 @@ class MapEditorEditRiversTab(
         }.makeTileGroup()
     private fun getRemoveRiverIcon() =
         ImageGetter.getCrossedImage(getTileGroupWithRivers(RiverEdge.All), iconSize)
-    private fun getRiverIcon(edge: RiverEdge) = Group().apply {
+    private fun getRiverIcon(edge: RiverEdge) = NonTransformGroup().apply {
         // wrap same as getRemoveRiverIcon so the icons align the same (using getTileGroupWithRivers directly works but looks ugly - reason unknown to me)
-        isTransform = false
         setSize(iconSize, iconSize)
         val tileGroup = getTileGroupWithRivers(edge)
         tileGroup.center(this)
