@@ -26,7 +26,7 @@ import com.unciv.ui.popups.ToastPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
 import com.unciv.ui.screens.mapeditorscreen.MapEditorScreen
-import com.unciv.ui.screens.mapeditorscreen.TileInfoNormalizer
+import com.unciv.logic.map.tile.TileNormalizer
 import com.unciv.ui.screens.mapeditorscreen.tabs.MapEditorOptionsTab.TileMatchFuzziness
 import com.unciv.utils.Log
 
@@ -107,10 +107,10 @@ class MapEditorEditTab(
             minimumWidth = subTabsWidth,
             maximumWidth = subTabsWidth,
             headerPadding = 5f,
-            capacity = AllEditSubTabs.values().size
+            capacity = AllEditSubTabs.entries.size
         )
 
-        for (page in AllEditSubTabs.values()) {
+        for (page in AllEditSubTabs.entries) {
             // Empty tabs with placeholders, filled when activated()
             subTabs.addPage(page.caption, Group(), ImageGetter.getImage(page.icon), 20f,
                 shortcutKey = KeyCharAndCode(page.key), disabled = true)
@@ -150,41 +150,34 @@ class MapEditorEditTab(
     }
 
     // "Normal" setBrush overload, using named RulesetObject icon
-    fun setBrush(name: String, icon: String, pediaLink: String = icon, isRemove: Boolean = false, applyAction: (Tile)->Unit) {
-        brushHandlerType = BrushHandlerType.Tile
+    fun setBrush(name: String, icon: String, handlerType: BrushHandlerType = BrushHandlerType.Tile,
+                 pediaLink: String = icon, isRemove: Boolean = false,
+                 applyAction: (Tile)->Unit) {
+        brushHandlerType = handlerType
         val brushActor = FormattedLine(name, icon = icon, iconCrossed = isRemove).render(0f)
         linkCivilopedia(brushActor, pediaLink)
         brushCell.setActor(brushActor)
         brushAction = applyAction
     }
+    
     // Helper overload for brushes using icons not existing as RulesetObject
-    private fun setBrush(name: String, icon: Actor, pediaLink: String, applyAction: (Tile)->Unit) {
-        brushHandlerType = BrushHandlerType.Tile
-        val line = Table().apply {
+    fun setBrush(handlerType: BrushHandlerType, name: String, icon: Actor, pediaLink: String, applyAction: (Tile)->Unit) {
+        brushHandlerType = handlerType
+        val brushActor = Table().apply {
             add(icon).padRight(10f)
             add(name.toLabel())
         }
-        linkCivilopedia(line, pediaLink)
-        brushCell.setActor(line)
+        linkCivilopedia(brushActor, pediaLink)
+        brushCell.setActor(brushActor)
         brushAction = applyAction
     }
-    // This overload is used by Roads and Starting locations
-    fun setBrush(handlerType: BrushHandlerType, name: String, icon: String, pediaLink: String = icon, isRemove: Boolean = false, applyAction: (Tile)->Unit) {
-        setBrush(name, icon, pediaLink, isRemove, applyAction)
-        brushHandlerType = handlerType
-    }
-    // This overload is used by Rivers
-    fun setBrush(handlerType: BrushHandlerType, name: String, icon: Actor, pediaLink: String, applyAction: (Tile)->Unit) {
-        setBrush(name, icon, pediaLink, applyAction)
-        brushHandlerType = handlerType
-    }
 
     override fun activated(index: Int, caption: String, pager: TabbedPager) {
         if (editorScreen.editTabsNeedRefresh) {
             // ruleset has changed
             ruleset = editorScreen.ruleset
             ImageGetter.setNewRuleset(ruleset)
-            for (page in AllEditSubTabs.values()) {
+            for (page in AllEditSubTabs.entries) {
                 val tab = page.instantiate(this, ruleset)
                 subTabs.replacePage(page.caption, tab)
                 subTabs.setPageDisabled(page.caption, (tab as IMapEditorEditSubTabs).isDisabled())
@@ -255,44 +248,31 @@ class MapEditorEditTab(
             if (brushSize == -1) {
                 val bfs = BFS(tile) { it.isSimilarEnough(tile) }
                 bfs.stepToEnd()
-                bfs.getReachedTiles().asSequence()
+                bfs.getReachedTiles().toSet()
             } else {
-                tile.getTilesInDistance(brushSize - 1)
+                tile.getTilesInDistance(brushSize - 1).toSet()
             }
+        
         for (tileToPaint in tiles) {
             when (brushHandlerType) {
                 BrushHandlerType.Direct -> directPaintTile(tileToPaint)
+                BrushHandlerType.River -> directPaintTile(tileToPaint)
                 BrushHandlerType.Tile -> paintTile(tileToPaint)
-                BrushHandlerType.Road -> roadPaintTile(tileToPaint)
-                BrushHandlerType.River -> riverPaintTile(tileToPaint)
+                BrushHandlerType.Road -> paintTile(tileToPaint)
                 else -> {} // other cases can't reach here
             }
         }
+        
+        // Adjacent tiles could have images changed as well, due to rivers/edge tiles/roads
+        val tilesToUpdate = tiles.flatMap { it.neighbors + it }.toSet()
+        for (tileToUpdate in tilesToUpdate) editorScreen.updateTile(tileToUpdate)
     }
 
     /** Used for starting locations - no temp tile as brushAction needs to access tile.tileMap */
     private fun directPaintTile(tile: Tile) {
         brushAction(tile)
         editorScreen.isDirty = true
-        editorScreen.updateAndHighlight(tile)
-    }
-
-    /** Used for rivers - same as [directPaintTile] but may need to update 10,12 and 2 o'clock neighbor tiles too
-     *
-     *  Note: Unlike [paintRiverFromTo] this does **not** call [MapGenerator.convertTerrains] to allow more freedom.
-     */
-    private fun riverPaintTile(tile: Tile) {
-        directPaintTile(tile)
-        for (neighbor in tile.neighbors) {
-            if (neighbor.position.x > tile.position.x || neighbor.position.y > tile.position.y)
-                editorScreen.updateTile(neighbor)
-        }
-    }
-
-    // Used for roads - same as paintTile but all neighbors need TileGroup.update too
-    private fun roadPaintTile(tile: Tile) {
-        if (!paintTile(tile)) return
-        for (neighbor in tile.neighbors) editorScreen.updateTile(neighbor)
+        editorScreen.highlightTile(tile)
     }
 
     /** apply brush to a single tile */
@@ -312,7 +292,7 @@ class MapEditorEditTab(
 
         brushAction(tile)
         tile.setTerrainTransients()
-        TileInfoNormalizer.normalizeToRuleset(tile, ruleset)
+        TileNormalizer.normalizeToRuleset(tile, ruleset)
         if (!paintedTile.isSimilarEnough(tile)) {
             // revert tile to original state
             tile.applyFrom(savedTile)
@@ -322,7 +302,7 @@ class MapEditorEditTab(
         if (tile.naturalWonder != savedTile.naturalWonder)
             editorScreen.naturalWondersNeedRefresh = true
         editorScreen.isDirty = true
-        editorScreen.updateAndHighlight(tile)
+        editorScreen.highlightTile(tile)
         return true
     }
 
@@ -345,7 +325,7 @@ class MapEditorEditTab(
         // 90% copy w/o position, improvement times or transients. Add units once Unit paint is in.
         baseTerrain = other.baseTerrain
         setTerrainFeatures(other.terrainFeatures)
-        resource = other.resource
+        tileResource = other.tileResource
         improvement = other.improvement
         naturalWonder = other.naturalWonder
         roadStatus = other.roadStatus
@@ -357,7 +337,7 @@ class MapEditorEditTab(
 
     companion object {
         private fun getBrushTip(value: Float, abbreviate: Boolean = false) = when {
-            value <= 5f -> value.toInt().toString()
+            value <= 5f -> value.toInt().tr()
             abbreviate -> "Floodfill_Abbreviation"
             else -> "Floodfill"
         }
