@@ -4,6 +4,10 @@ import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.graphics.Texture
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.badlogic.gdx.scenes.scene2d.ui.TextButton
+import com.badlogic.gdx.utils.GdxRuntimeException
+import com.unciv.UncivGame
+import com.unciv.logic.github.Github
 import com.unciv.logic.github.GithubAPI
 import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.Ruleset
@@ -15,8 +19,12 @@ import com.unciv.ui.components.extensions.toCheckBox
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.input.onClick
-import com.unciv.logic.github.Github
+import com.unciv.ui.components.input.onRightClick
+import com.unciv.ui.popups.ToastPopup
+import com.unciv.ui.screens.modmanager.ModManagementScreen.Companion.cleanModName
 import com.unciv.utils.Concurrency
+import com.unciv.utils.Log
+import java.io.IOException
 import kotlin.math.max
 
 internal class ModInfoAndActionPane : Table() {
@@ -24,6 +32,7 @@ internal class ModInfoAndActionPane : Table() {
     private val imageHolder = Table()
     private val sizeLabel = "".toLabel()
     private var isBuiltin = false
+    private var currentRepoName = ""
 
     /** controls "Permanent audiovisual mod" checkbox existence */
     private var enableVisualCheckBox = false
@@ -39,6 +48,7 @@ internal class ModInfoAndActionPane : Table() {
         isBuiltin = false
         enableVisualCheckBox = false
         update(
+            isLocal = false,
             repo.name, repo.html_url, repo.default_branch,
             repo.pushed_at, repo.owner.login, repo.size,
             repo.owner.avatar_url
@@ -51,31 +61,34 @@ internal class ModInfoAndActionPane : Table() {
     fun update(mod: Ruleset) {
         val modName = mod.name
         val modOptions = mod.modOptions  // The ModOptions as enriched by us with GitHub metadata when originally downloaded
-        isBuiltin = modOptions.modUrl.isEmpty() && BaseRuleset.values().any { it.fullName == modName }
+        isBuiltin = modOptions.modUrl.isEmpty() && BaseRuleset.entries.any { it.fullName == modName }
         enableVisualCheckBox = ModCompatibility.isAudioVisualMod(mod)
         update(
+            isLocal = true,
             modName, modOptions.modUrl, modOptions.defaultBranch,
             modOptions.lastUpdated, modOptions.author, modOptions.modSize
         )
     }
 
     private fun update(
-           modName: String,
-           repoUrl: String,
-           defaultBranch: String,
-           updatedAt: String,
-           author: String,
-           modSize: Int,
-           avatarUrl: String? = null
+        isLocal: Boolean,
+        modName: String,
+        repoUrl: String,
+        defaultBranch: String,
+        updatedAt: String,
+        author: String,
+        modSize: Int,
+        avatarUrl: String? = null
     ) {
         // Display metadata
         clear()
+        currentRepoName = modName
 
         imageHolder.clear()
         when {
-            isBuiltin -> addUncivLogo()
-            repoUrl.isEmpty() -> addLocalPreviewImage(modName)
-            else -> addPreviewImage(repoUrl, defaultBranch, avatarUrl)
+            isBuiltin -> addUncivLogo(modName)
+            isLocal -> addLocalPreviewImage(modName)
+            else -> addPreviewImage(modName, repoUrl, defaultBranch, avatarUrl)
         }
         add(imageHolder).row()
 
@@ -87,9 +100,15 @@ internal class ModInfoAndActionPane : Table() {
 
         // offer link to open the repo itself in a browser
         if (repoUrl.isNotEmpty()) {
-            add("Open Github page".toTextButton().onClick {
+            val githubButton = "Open Github page".toTextButton()
+            githubButton.onClick {
                 Gdx.net.openURI(repoUrl)
-            }).row()
+            }
+            githubButton.onRightClick {
+                Gdx.app.clipboard.contents = repoUrl
+                ToastPopup("Link copied to clipboard", stage)
+            }
+            add(githubButton).row()
         }
 
         // display "updated" date
@@ -114,27 +133,24 @@ internal class ModInfoAndActionPane : Table() {
             add("Permanent audiovisual mod".toCheckBox(startsOutChecked, changeAction)).row()
     }
 
-    fun addUpdateModButton(modInfo: ModUIData, doDownload: () -> Unit) {
-        if (!modInfo.hasUpdate) return
-        val updateModTextbutton = "Update [${modInfo.name}]".toTextButton()
-        updateModTextbutton.onClick {
-            updateModTextbutton.setText("Downloading...".tr())
-            doDownload()
-        }
+    fun addUpdateModButton(modInfo: ModUIData): TextButton? {
+        if (!modInfo.hasUpdate) return null
+        val updateModTextbutton = "Update [${cleanModName(modInfo.name)}]".toTextButton()
         add(updateModTextbutton).row()
+        return updateModTextbutton
     }
 
-    private fun addPreviewImage(repoUrl: String, defaultBranch: String, avatarUrl: String?) {
+    private fun addPreviewImage(modName: String, repoUrl: String, defaultBranch: String, avatarUrl: String?) {
         if (!repoUrl.startsWith("http")) return // invalid url
 
         if (repoUrlToPreviewImage.containsKey(repoUrl)) {
             val texture = repoUrlToPreviewImage[repoUrl]
-            if (texture != null) setTextureAsPreview(texture)
+            if (texture != null) setTextureAsPreview(texture, modName)
             return
         }
 
         Concurrency.run {
-            val imagePixmap = Github.tryGetPreviewImage(repoUrl, defaultBranch, avatarUrl)
+            val imagePixmap = Github.getPreviewImageOrNull(repoUrl, defaultBranch, avatarUrl)
 
             if (imagePixmap == null) {
                 repoUrlToPreviewImage[repoUrl] = null
@@ -144,26 +160,34 @@ internal class ModInfoAndActionPane : Table() {
                 val texture = Texture(imagePixmap)
                 imagePixmap.dispose()
                 repoUrlToPreviewImage[repoUrl] = texture
-                setTextureAsPreview(texture)
+                setTextureAsPreview(texture, modName)
             }
         }
     }
 
     private fun addLocalPreviewImage(modName: String) {
         // No concurrency, order of magnitude 20ms
-        val modFolder = Gdx.files.local("mods/$modName")
+        val modFolder = UncivGame.Current.files.getModFolder(modName)
         val previewFile = modFolder.child("preview.jpg").takeIf { it.exists() }
             ?: modFolder.child("preview.png").takeIf { it.exists() }
             ?: return
-        setTextureAsPreview(Texture(previewFile))
+        try {
+            setTextureAsPreview(Texture(previewFile), modName)
+        } catch (ex: Throwable) {
+            val cause = if (ex is GdxRuntimeException) ex.cause else ex
+            Log.debug("Could not load local preview file %s: %s", previewFile.path(), cause)
+            if (cause is IOException) previewFile.delete() // File content invalid and not loadable as pixmap gives this
+        }
     }
 
-    private fun addUncivLogo() {
-        setTextureAsPreview(Texture(Gdx.files.internal("ExtraImages/banner.png")))
+    private fun addUncivLogo(modName: String) {
+        setTextureAsPreview(Texture(Gdx.files.internal("ExtraImages/banner.png")), modName)
     }
 
-    private fun setTextureAsPreview(texture: Texture) {
-        val cell = imageHolder.add(Image(texture))
+    private fun setTextureAsPreview(texture: Texture, modName: String) {
+        val image = Image(texture)
+        if (modName != currentRepoName) return // user has selected another mod in the meantime
+        val cell = imageHolder.add(image)
         val largestImageSize = max(texture.width, texture.height)
         if (largestImageSize > ModManagementScreen.maxAllowedPreviewImageSize) {
             val resizeRatio = ModManagementScreen.maxAllowedPreviewImageSize / largestImageSize
