@@ -18,12 +18,12 @@ import com.unciv.logic.map.TileMap
 import com.unciv.models.metadata.Player
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.nation.Nation
+import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.SmallButtonStyle
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.enable
 import com.unciv.ui.components.extensions.pad
-import com.unciv.ui.components.extensions.toGdxArray
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.input.onActivation
@@ -32,6 +32,7 @@ import com.unciv.ui.components.widgets.LoadingImage
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.victoryscreen.LoadMapPreview
 import com.unciv.utils.Concurrency
+import com.unciv.utils.toGdxArray
 import kotlinx.coroutines.Job
 import kotlinx.coroutines.flow.asFlow
 import kotlinx.coroutines.flow.catch
@@ -209,13 +210,22 @@ class MapFileSelectTable(
             .toGdxArray()
         fun getPreselect(): MapWrapper? {
             if (mapFiles.isEmpty) return null
-            val recent = mapFiles.asSequence()
-                .filter { it.fileHandle.isRecentlyModified() }
-                .maxByOrNull { it.fileHandle.lastModified() }
-            val oldestTimestamp = mapFiles.minOfOrNull { it.fileHandle.lastModified() } ?: 0L
-            // Do not use most recent if all maps in the category have the same time within a tenth of a second (like a mod unzip does)
-            if (recent != null && (recent.fileHandle.lastModified() - oldestTimestamp) > 100 || mapFiles.size == 1)
-                return recent
+            val modOptionPreselect = RulesetCache[selectedRuleset]
+                ?.modOptions?.getMatchingUniques(UniqueType.ModMapPreselection)
+                ?.firstOrNull()?.params?.get(0)
+            if (modOptionPreselect != null) {
+                val preselectFile = mapFiles.firstOrNull { it.fileHandle.name() == modOptionPreselect }
+                if (preselectFile != null)
+                    return preselectFile
+            } else {
+                val recent = mapFiles.asSequence()
+                    .filter { it.fileHandle.isRecentlyModified() }
+                    .maxByOrNull { it.fileHandle.lastModified() }
+                val oldestTimestamp = mapFiles.minOfOrNull { it.fileHandle.lastModified() } ?: 0L
+                // Do not use most recent if all maps in the category have the same time within a tenth of a second (like a mod unzip does)
+                if (recent != null && (recent.fileHandle.lastModified() - oldestTimestamp) > 100 || mapFiles.size == 1)
+                    return recent
+            }
             val named = mapFiles.firstOrNull { it.fileHandle.name() == preselectedName }
             if (named != null)
                 return named
@@ -362,11 +372,11 @@ class MapFileSelectTable(
         val players = newGameScreen.playerPickerTable.gameParameters.players
         players.clear()
         mapNations.asSequence()
-            .map { it.name to it.name.tr(hideIcons = true) } // Sort by translation but keep untranslated name
+            .map { it to it.name.tr(hideIcons = true) } // Sort by translation but keep untranslated name
             .sortedWith(
-                compareBy<Pair<String, String>>{ it.first != mapHumanPick }
+                compareBy<Pair<Nation, String>>{ it.first.name != mapHumanPick }
                 .thenBy(collator) { it.second }
-            ).map { Player(it.first, if (it.first == mapHumanPick) PlayerType.Human else PlayerType.AI) }
+            ).map { Player(it.first, if (it.first.name == mapHumanPick) PlayerType.Human else PlayerType.AI) }
             .toCollection(players)
         newGameScreen.playerPickerTable.update()
     }
