@@ -7,13 +7,14 @@ import com.unciv.ui.components.input.onChange
 import com.unciv.ui.components.widgets.TranslatedSelectBox
 import com.unciv.ui.screens.basescreen.BaseScreen
 
-class MapOptionsTable(private val newGameScreen: NewGameScreen, isReset: Boolean = true) : Table() {
+class MapOptionsTable(private val newGameScreen: NewGameScreen) : Table() {
 
     private val mapParameters = newGameScreen.gameSetupInfo.mapParameters
     private var mapTypeSpecificTable = Table()
     internal val generatedMapOptionsTable = MapParametersTable(newGameScreen, mapParameters, MapGeneratedMainType.generated)
     private val randomMapOptionsTable = MapParametersTable(newGameScreen, mapParameters, MapGeneratedMainType.randomGenerated)
     private val savedMapOptionsTable = MapFileSelectTable(newGameScreen, mapParameters)
+    private val scenarioOptionsTable = ScenarioSelectTable(newGameScreen)
     internal val mapTypeSelectBox: TranslatedSelectBox
 
     init {
@@ -23,8 +24,9 @@ class MapOptionsTable(private val newGameScreen: NewGameScreen, isReset: Boolean
 
         val mapTypes = arrayListOf(MapGeneratedMainType.generated, MapGeneratedMainType.randomGenerated)
         if (savedMapOptionsTable.isNotEmpty()) mapTypes.add(MapGeneratedMainType.custom)
+        if (newGameScreen.game.files.getScenarioFiles().any()) mapTypes.add(MapGeneratedMainType.scenario)
 
-        mapTypeSelectBox = TranslatedSelectBox(mapTypes, MapGeneratedMainType.generated, BaseScreen.skin)
+        mapTypeSelectBox = TranslatedSelectBox(mapTypes, MapGeneratedMainType.generated)
 
         fun updateOnMapTypeChange() {
             mapTypeSpecificTable.clear()
@@ -46,6 +48,12 @@ class MapOptionsTable(private val newGameScreen: NewGameScreen, isReset: Boolean
                     mapTypeSpecificTable.add(randomMapOptionsTable)
                     newGameScreen.unlockTables()
                 }
+                MapGeneratedMainType.scenario -> {
+                    mapParameters.name = ""
+                    mapTypeSpecificTable.add(scenarioOptionsTable)
+                    scenarioOptionsTable.selectScenario()
+                    newGameScreen.lockTables()
+                }
             }
             newGameScreen.gameSetupInfo.gameParameters.godMode = false
             newGameScreen.updateTables()
@@ -63,5 +71,10 @@ class MapOptionsTable(private val newGameScreen: NewGameScreen, isReset: Boolean
         add(mapTypeSpecificTable).row()
     }
 
+    internal fun getSelectedScenario(): ScenarioSelectTable.ScenarioData? {
+        if (mapTypeSelectBox.selected.value != MapGeneratedMainType.scenario) return null
+        return scenarioOptionsTable.selectedScenario
+    }
+
     internal fun cancelBackgroundJobs() = savedMapOptionsTable.cancelBackgroundJobs()
 }
