@@ -3,29 +3,20 @@ package com.unciv.ui.screens.newgamescreen
 import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.scenes.scene2d.ui.CheckBox
 import com.badlogic.gdx.scenes.scene2d.ui.Table
-import com.badlogic.gdx.scenes.scene2d.ui.TextField
-import com.badlogic.gdx.scenes.scene2d.ui.TextField.TextFieldFilter.DigitsOnlyFilter
 import com.badlogic.gdx.utils.Align
 import com.unciv.UncivGame
-import com.unciv.logic.map.MapGeneratedMainType
-import com.unciv.logic.map.MapParameters
-import com.unciv.logic.map.MapResources
-import com.unciv.logic.map.MapShape
-import com.unciv.logic.map.MapSize
-import com.unciv.logic.map.MapSizeNew
-import com.unciv.logic.map.MapType
-import com.unciv.ui.components.UncivTextField
-import com.unciv.ui.components.extensions.pad
-import com.unciv.ui.components.extensions.toCheckBox
-import com.unciv.ui.components.extensions.toLabel
-import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.logic.map.*
+import com.unciv.logic.map.mapgenerator.MapGenerator
+import com.unciv.logic.map.mapgenerator.MapResourceSetting
+import com.unciv.models.metadata.GameParameters
+import com.unciv.models.ruleset.RulesetCache
+import com.unciv.ui.components.extensions.*
 import com.unciv.ui.components.input.onChange
 import com.unciv.ui.components.input.onClick
-import com.unciv.ui.components.widgets.ExpanderTab
-import com.unciv.ui.components.widgets.TranslatedSelectBox
-import com.unciv.ui.components.widgets.UncivSlider
-import com.unciv.ui.components.widgets.WrappableLabel
+import com.unciv.ui.components.widgets.*
 import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.screens.victoryscreen.LoadMapPreview
+import com.unciv.utils.Concurrency
 
 /** Table for editing [mapParameters]
  *
@@ -40,15 +31,15 @@ class MapParametersTable(
     private val forMapEditor: Boolean = false,
     private val sizeChangedCallback: (()->Unit)? = null
 ) : Table() {
-    // These are accessed fom outside the class to read _and_ write values,
+    // These are accessed from outside the class to read _and_ write values,
     // namely from MapOptionsTable, NewMapScreen and NewGameScreen
     lateinit var mapTypeSelectBox: TranslatedSelectBox
-    lateinit var customMapSizeRadius: TextField
-    lateinit var customMapWidth: TextField
-    lateinit var customMapHeight: TextField
+    lateinit var customMapSizeRadius: UncivTextField.Integer
+    lateinit var customMapWidth: UncivTextField.Integer
+    lateinit var customMapHeight: UncivTextField.Integer
 
     private lateinit var worldSizeSelectBox: TranslatedSelectBox
-    private var customWorldSizeTable = Table ()
+    private var customWorldSizeTable = Table()
     private var hexagonalSizeTable = Table()
     private var rectangularSizeTable = Table()
     lateinit var resourceSelectBox: TranslatedSelectBox
@@ -57,13 +48,16 @@ class MapParametersTable(
     private lateinit var worldWrapCheckbox: CheckBox
     private lateinit var legendaryStartCheckbox: CheckBox
     private lateinit var strategicBalanceCheckbox: CheckBox
-    private lateinit var seedTextField: TextField
+    private lateinit var seedTextField: UncivTextField.Numeric
 
     private lateinit var mapShapesOptionsValues: HashSet<String>
     private lateinit var mapTypesOptionsValues: HashSet<String>
     private lateinit var mapSizesOptionsValues: HashSet<String>
     private lateinit var mapResourcesOptionsValues: HashSet<String>
 
+    private val maxMapSize = ((previousScreen as? NewGameScreen)?.getColumnWidth() ?: 200f) - 10f // There is 5px padding each side
+    private val mapTypeExample = Table()
+
     // Keep references (in the key) and settings value getters (in the value) of the 'advanced' sliders
     // in a HashMap for reuse later - in the reset to defaults button. Better here as field than as closure.
     // A HashMap indexed on a Widget is problematic, as it does not define its own hashCode and equals
@@ -96,11 +90,12 @@ class MapParametersTable(
         addResourceSelectBox()
         addWrappedCheckBoxes()
         addAdvancedSettings()
+        generateExampleMap()
     }
 
     fun reseed() {
         mapParameters.reseed()
-        seedTextField.text = mapParameters.seed.toString()
+        seedTextField.value = mapParameters.seed
     }
 
     private fun addMapShapeSelectBox() {
@@ -122,10 +117,11 @@ class MapParametersTable(
             add(optionsTable).colspan(2).grow().row()
         } else {
             val mapShapeSelectBox =
-                    TranslatedSelectBox(mapShapes, mapParameters.shape, skin)
+                    TranslatedSelectBox(mapShapes, mapParameters.shape)
             mapShapeSelectBox.onChange {
                 mapParameters.shape = mapShapeSelectBox.selected.value
                 updateWorldSizeTable()
+                generateExampleMap()
             }
 
             add ("{Map Shape}:".toLabel()).left()
@@ -133,6 +129,26 @@ class MapParametersTable(
         }
     }
 
+    private fun generateExampleMap(){
+        val ruleset = if (previousScreen is NewGameScreen) previousScreen.ruleset else RulesetCache.getVanillaRuleset()
+        Concurrency.run("Generate example map") {
+            val mapParametersForExample = if (forMapEditor) mapParameters else mapParameters.clone().apply { seed = 0 }
+            val exampleMap = MapGenerator(ruleset).generateMap(mapParametersForExample, GameParameters(), emptyList())
+            Concurrency.runOnGLThread {
+                mapTypeExample.clear()
+                val mapPreview = LoadMapPreview(exampleMap, maxMapSize, maxMapSize)
+                if (!forMapEditor){
+                    val label = "Example map".toLabel()
+                    label.centerX(mapPreview)
+                    label.y = mapPreview.height - label.height - 10f
+                    mapPreview.addActor(label)
+                }
+                mapTypeExample.add(mapPreview)
+                pack()
+            }
+        }
+    }
+
     private fun addMapTypeSelectBox() {
         // MapType is not an enum so we can't simply enumerate. //todo: make it so!
         val mapTypes = listOfNotNull(
@@ -161,7 +177,8 @@ class MapParametersTable(
             }
             add(optionsTable).colspan(2).grow().row()
         } else {
-            mapTypeSelectBox = TranslatedSelectBox(mapTypes, mapParameters.type, skin)
+            mapTypeSelectBox = TranslatedSelectBox(mapTypes, mapParameters.type)
+
 
             mapTypeSelectBox.onChange {
                 mapParameters.type = mapTypeSelectBox.selected.value
@@ -169,29 +186,32 @@ class MapParametersTable(
                 // If the map won't be generated, these options are irrelevant and are hidden
                 noRuinsCheckbox.isVisible = mapParameters.type != MapType.empty
                 noNaturalWondersCheckbox.isVisible = mapParameters.type != MapType.empty
+
+                generateExampleMap()
             }
 
             add("{Map Generation Type}:".toLabel()).left()
             add(mapTypeSelectBox).fillX().row()
+            add(mapTypeExample).colspan(2).grow().row()
         }
     }
 
     private fun addWorldSizeTable() {
         if (mapGeneratedMainType == MapGeneratedMainType.randomGenerated) {
-            val mapSizes = MapSize.values().map { it.name }
+            val mapSizes = MapSize.names()
             mapSizesOptionsValues = mapSizes.toHashSet()
             val optionsTable = MultiCheckboxTable("{Enabled World Sizes}", "NewGameWorldSizes", mapSizesOptionsValues) {
                 if (mapSizesOptionsValues.isEmpty()) {
-                    mapParameters.mapSize = MapSizeNew(mapSizes.random())
+                    mapParameters.mapSize = MapSize(mapSizes.random())
                 } else {
-                    mapParameters.mapSize = MapSizeNew(mapSizesOptionsValues.random())
+                    mapParameters.mapSize = MapSize(mapSizesOptionsValues.random())
                 }
             }
             add(optionsTable).colspan(2).grow().row()
         } else {
-            val mapSizes = MapSize.values().map { it.name } + listOf(MapSize.custom)
-            worldSizeSelectBox = TranslatedSelectBox(mapSizes, mapParameters.mapSize.name, skin)
-            worldSizeSelectBox.onChange { updateWorldSizeTable() }
+            val mapSizes = MapSize.names() + listOf(MapSize.custom)
+            worldSizeSelectBox = TranslatedSelectBox(mapSizes, mapParameters.mapSize.name)
+            worldSizeSelectBox.onChange { updateWorldSizeTable(); generateExampleMap() }
 
             addHexagonalSizeTable()
             addRectangularSizeTable()
@@ -205,12 +225,10 @@ class MapParametersTable(
     }
 
     private fun addHexagonalSizeTable() {
-        val defaultRadius = mapParameters.mapSize.radius.toString()
-        customMapSizeRadius = UncivTextField.create("Radius", defaultRadius).apply {
-            textFieldFilter = DigitsOnlyFilter()
-        }
+        val defaultRadius = mapParameters.mapSize.radius
+        customMapSizeRadius = UncivTextField.Integer("Radius", defaultRadius)
         customMapSizeRadius.onChange {
-            mapParameters.mapSize = MapSizeNew(customMapSizeRadius.text.toIntOrNull() ?: 0 )
+            mapParameters.mapSize = MapSize(customMapSizeRadius.intValue ?: 0 )
         }
         hexagonalSizeTable.add("{Radius}:".toLabel()).grow().left()
         hexagonalSizeTable.add(customMapSizeRadius).right().row()
@@ -219,21 +237,16 @@ class MapParametersTable(
     }
 
     private fun addRectangularSizeTable() {
-        val defaultWidth = mapParameters.mapSize.width.toString()
-        customMapWidth = UncivTextField.create("Width", defaultWidth).apply {
-            textFieldFilter = DigitsOnlyFilter()
-        }
-
-        val defaultHeight = mapParameters.mapSize.height.toString()
-        customMapHeight = UncivTextField.create("Height", defaultHeight).apply {
-            textFieldFilter = DigitsOnlyFilter()
-        }
+        val defaultWidth = mapParameters.mapSize.width
+        customMapWidth = UncivTextField.Integer("Width", defaultWidth)
+        val defaultHeight = mapParameters.mapSize.height
+        customMapHeight = UncivTextField.Integer("Height", defaultHeight)
 
         customMapWidth.onChange {
-            mapParameters.mapSize = MapSizeNew(customMapWidth.text.toIntOrNull() ?: 0, customMapHeight.text.toIntOrNull() ?: 0)
+            mapParameters.mapSize = MapSize(customMapWidth.intValue ?: 0, customMapHeight.intValue ?: 0)
         }
         customMapHeight.onChange {
-            mapParameters.mapSize = MapSizeNew(customMapWidth.text.toIntOrNull() ?: 0, customMapHeight.text.toIntOrNull() ?: 0)
+            mapParameters.mapSize = MapSize(customMapWidth.intValue ?: 0, customMapHeight.intValue ?: 0)
         }
 
         rectangularSizeTable.defaults().pad(5f)
@@ -253,23 +266,13 @@ class MapParametersTable(
         else if (mapParameters.shape == MapShape.rectangular && worldSizeSelectBox.selected.value == MapSize.custom)
             customWorldSizeTable.add(rectangularSizeTable).grow().row()
         else
-            mapParameters.mapSize = MapSizeNew(worldSizeSelectBox.selected.value)
+            mapParameters.mapSize = MapSize(worldSizeSelectBox.selected.value)
 
         sizeChangedCallback?.invoke()
     }
 
     private fun addResourceSelectBox() {
-        val mapResources = if (forMapEditor) listOf(
-            MapResources.sparse,
-            MapResources.default,
-            MapResources.abundant,
-        ) else listOf(
-            MapResources.sparse,
-            MapResources.default,
-            MapResources.abundant,
-            MapResources.strategicBalance,
-            MapResources.legendaryStart
-        )
+        val mapResources = MapResourceSetting.activeLabels()
 
         if (mapGeneratedMainType == MapGeneratedMainType.randomGenerated) {
             mapResourcesOptionsValues = mapResources.toHashSet()
@@ -282,7 +285,7 @@ class MapParametersTable(
             }
             add(optionsTable).colspan(2).grow().row()
         } else {
-            resourceSelectBox = TranslatedSelectBox(mapResources, mapParameters.mapResources, skin)
+            resourceSelectBox = TranslatedSelectBox(mapResources, mapParameters.mapResources)
 
             resourceSelectBox.onChange {
                 mapParameters.mapResources = resourceSelectBox.selected.value
@@ -362,44 +365,35 @@ class MapParametersTable(
     }
 
     private fun addAdvancedSettings() {
-        val expander = ExpanderTab("Advanced Settings", startsOutOpened = false) {
+        val expander = ExpanderTab("Advanced Settings", startsOutOpened = false, defaultPad = 0f) {
             addAdvancedControls(it)
         }
-        add(expander).pad(10f).padTop(10f).colspan(2).growX().row()
+        add(expander).padTop(10f).colspan(2).growX().row()
     }
 
     private fun addAdvancedControls(table: Table) {
-        table.defaults().pad(5f)
+        table.defaults().pad(2f).padTop(10f)
 
-        seedTextField = UncivTextField.create("RNG Seed", mapParameters.seed.toString())
-        seedTextField.textFieldFilter = DigitsOnlyFilter()
+        seedTextField = UncivTextField.Numeric("RNG Seed", mapParameters.seed, integerOnly = true)
 
-        // If the field is empty, fallback seed value to 0
         seedTextField.onChange {
-            mapParameters.seed = try {
-                seedTextField.text.toLong()
-            } catch (_: Exception) {
-                0L
-            }
+            mapParameters.seed = seedTextField.value?.toLong() ?: 0L
         }
 
         table.add("RNG Seed".toLabel()).left()
         table.add(seedTextField).fillX().padBottom(10f).row()
 
-        fun addSlider(text: String, getValue:()->Float, min: Float, max: Float, onChange: (value: Float)->Unit): UncivSlider {
-            val slider = UncivSlider(min, max, (max - min) / 20, onChange = onChange, initial = getValue())
+        fun addSlider(text: String, getValue:()->Float, min: Float, max: Float, step: Float, onChange: (value: Float)->Unit): UncivSlider {
+            val slider = UncivSlider(min, max, step, onChange = {onChange(it); generateExampleMap()}, initial = getValue())
             table.add(text.toLabel()).left()
             table.add(slider).fillX().row()
             advancedSliders[slider] = getValue
             return slider
         }
 
-        fun addSlider(text: String, getValue:()->Float, min: Float, max: Float, step: Float, onChange: (value: Float)->Unit): UncivSlider {
-            val slider = UncivSlider(min, max, step, onChange = onChange, initial = getValue())
-            table.add(text.toLabel()).left()
-            table.add(slider).fillX().row()
-            advancedSliders[slider] = getValue
-            return slider
+        fun addSlider(text: String, getValue:()->Float, min: Float, max: Float, onChange: (value: Float)->Unit): UncivSlider {
+            val step = (max - min) / 20
+            return addSlider(text, getValue, min, max, step, onChange)
         }
 
         fun addTextButton(text: String, shouldAddToTable: Boolean = false, action: ((Boolean) -> Unit)) {
@@ -423,8 +417,8 @@ class MapParametersTable(
         addSlider("Map Elevation", {mapParameters.elevationExponent}, 0.6f, 0.8f)
         { mapParameters.elevationExponent = it }
 
-        addSlider("Temperature extremeness", {mapParameters.temperatureExtremeness}, 0.4f, 0.8f)
-        { mapParameters.temperatureExtremeness = it }
+        addSlider("Temperature intensity", {mapParameters.temperatureintensity}, 0.4f, 0.8f)
+        { mapParameters.temperatureintensity = it }
 
         addSlider("Temperature shift", {mapParameters.temperatureShift}, -0.4f, 0.4f, 0.1f)
         { mapParameters.temperatureShift = it }
@@ -451,7 +445,7 @@ class MapParametersTable(
 
         addTextButton("Reset to defaults", true) {
             mapParameters.resetAdvancedSettings()
-            seedTextField.text = mapParameters.seed.toString()
+            seedTextField.value = mapParameters.seed
             for (entry in advancedSliders)
                 entry.key.value = entry.value()
         }
