@@ -9,32 +9,36 @@ import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.IdChecker
 import com.unciv.logic.civilization.PlayerType
+import com.unciv.logic.civilization.PlayerType.AI
+import com.unciv.logic.civilization.PlayerType.Human
 import com.unciv.logic.multiplayer.FriendList
 import com.unciv.models.metadata.GameParameters
 import com.unciv.models.metadata.GameSetupInfo
 import com.unciv.models.metadata.Player
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
+import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.input.KeyCharAndCode
-import com.unciv.ui.components.UncivTextField
-import com.unciv.ui.components.widgets.WrappableLabel
 import com.unciv.ui.components.extensions.darken
 import com.unciv.ui.components.extensions.isEnabled
+import com.unciv.ui.components.extensions.setFontColor
+import com.unciv.ui.components.extensions.surroundWithCircle
+import com.unciv.ui.components.extensions.toCheckBox
+import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
-import com.unciv.ui.components.extensions.setFontColor
-import com.unciv.ui.components.extensions.surroundWithCircle
-import com.unciv.ui.components.extensions.toLabel
-import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.widgets.UncivTextField
+import com.unciv.ui.components.widgets.WrappableLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.Popup
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.multiplayerscreens.FriendPickerList
 import com.unciv.ui.screens.pickerscreens.PickerPane
 import com.unciv.ui.screens.pickerscreens.PickerScreen
-import java.util.UUID
+import com.unciv.utils.isUUID
 import com.unciv.ui.components.widgets.AutoScrollPane as ScrollPane
 
 /**
@@ -68,6 +72,8 @@ class PlayerPickerTable(
             player.playerId = "" // This is to stop people from getting other users' IDs and cheating with them in multiplayer games
 
         top()
+        gameParameters.shufflePlayerOrder = false
+        add("Shuffle Civ Order at Start".toCheckBox(false) { gameParameters.shufflePlayerOrder = it }).padTop(5f).padBottom(5f).row()
         add(ScrollPane(playerListTable).apply { setOverscroll(false, false) }).width(civBlocksWidth)
         update()
         background = BaseScreen.skinStrings.getUiBackground("NewGameScreen/PlayerPickerTable", tintColor = BaseScreen.skinStrings.skinConfig.clearColor)
@@ -83,7 +89,8 @@ class PlayerPickerTable(
         val gameBasics = previousScreen.ruleset // the mod picking changes this ruleset
 
         reassignRemovedModReferences()
-        val newRulesetPlayableCivs = previousScreen.ruleset.nations.count { it.key != Constants.barbarians }
+        val newRulesetPlayableCivs = previousScreen.ruleset.nations
+            .count { it.key != Constants.barbarians && !it.value.hasUnique(UniqueType.WillNotBeChosenForNewGames) }
         if (gameParameters.players.size > newRulesetPlayableCivs)
             gameParameters.players = ArrayList(gameParameters.players.subList(0, newRulesetPlayableCivs))
         if (desiredCiv.isNotEmpty()) assignDesiredCiv(desiredCiv)
@@ -100,16 +107,16 @@ class PlayerPickerTable(
         }
 
         if (!locked && gameParameters.players.size < gameBasics.nations.values.count { it.isMajorCiv }) {
-            val addPlayerButton = "+".toLabel(Color.BLACK, 30)
+            val addPlayerButton = "+".toLabel(ImageGetter.CHARCOAL, 30)
                 .apply { this.setAlignment(Align.center) }
                 .surroundWithCircle(50f)
                 .onClick {
                     // no random mode - add first not spectator civ if still available
                     val player = if (noRandom || isRandomNumberOfPlayers) {
                         val availableCiv = getAvailablePlayerCivs().firstOrNull()
-                        if (availableCiv != null) Player(availableCiv.name)
+                        if (availableCiv != null) Player(availableCiv)
                         // Spectators can only be Humans
-                        else Player(Constants.spectator, PlayerType.Human)
+                        else Player(Constants.spectator, PlayerType.Human).apply { setNationTransient(gameBasics) }
                     } else Player()  // normal: add random AI
                     gameParameters.players.add(player)
                     update()
@@ -126,7 +133,7 @@ class PlayerPickerTable(
     fun updateRandomNumberLabel() {
         randomNumberLabel?.run {
             val playerRange = if (gameParameters.minNumberOfPlayers == gameParameters.maxNumberOfPlayers) {
-                gameParameters.minNumberOfPlayers.toString()
+                gameParameters.minNumberOfPlayers.tr()
             } else {
                 "${gameParameters.minNumberOfPlayers} - ${gameParameters.maxNumberOfPlayers}"
             }
@@ -148,7 +155,9 @@ class PlayerPickerTable(
      */
     private fun reassignRemovedModReferences() {
         for (player in gameParameters.players) {
-            if (!previousScreen.ruleset.nations.containsKey(player.chosenCiv) || previousScreen.ruleset.nations[player.chosenCiv]!!.isCityState)
+            if (!previousScreen.ruleset.nations.containsKey(player.chosenCiv)
+                || previousScreen.ruleset.nations[player.chosenCiv]!!.isCityState
+                || previousScreen.ruleset.nations[player.chosenCiv]!!.hasUnique(UniqueType.WillNotBeChosenForNewGames))
                 player.chosenCiv = Constants.random
         }
     }
@@ -161,7 +170,9 @@ class PlayerPickerTable(
         // No auto-select if desiredCiv already used
         if (gameParameters.players.any { it.chosenCiv == desiredCiv }) return
         // Do auto-select, silently no-op if no suitable slot (human with 'random' choice)
-        gameParameters.players.firstOrNull { it.chosenCiv == Constants.random && it.playerType == PlayerType.Human }?.chosenCiv = desiredCiv
+        val player = gameParameters.players.firstOrNull { it.chosenCiv == Constants.random && it.playerType == Human }
+        player?.chosenCiv = desiredCiv
+        player?.setNationTransient(previousScreen.ruleset)
     }
 
     /**
@@ -206,12 +217,12 @@ class PlayerPickerTable(
             updatePlayerTypeButtonEnabled()
         }
         playerTypeTextButton.onClick {
-            player.playerType = player.playerType.toggle()
+            player.playerType = if (player.playerType == AI) Human else AI
             update()
         }
 
         if (!locked) {
-            playerTable.add("-".toLabel(Color.BLACK, 30, Align.center)
+            playerTable.add("-".toLabel(ImageGetter.CHARCOAL, 30, Align.center)
                 .surroundWithCircle(40f)
                 .onClick {
                     gameParameters.players.remove(player)
@@ -229,24 +240,29 @@ class PlayerPickerTable(
     private fun Table.addPlayerTableMultiplayerControls(player: Player) {
         row()
 
-        val playerIdTextField = UncivTextField.create("Please input Player ID!", player.playerId)
+        val playerIdTextField = UncivTextField("Please input Player ID!", player.playerId)
         add(playerIdTextField).colspan(2).fillX().pad(5f)
         val errorLabel = "✘".toLabel(Color.RED)
         add(errorLabel).pad(5f).row()
 
         fun onPlayerIdTextUpdated() {
-            try {
-                UUID.fromString(IdChecker.checkAndReturnPlayerUuid(playerIdTextField.text))
+            if (IdChecker.checkAndReturnPlayerUuid(playerIdTextField.text)?.isUUID() ?: false) {
                 player.playerId = playerIdTextField.text.trim()
-                errorLabel.apply { setText("✔");setFontColor(Color.GREEN) }
-            } catch (_: Exception) {
-                errorLabel.apply { setText("✘");setFontColor(Color.RED) }
+                errorLabel.apply {
+                    setText("✔") // U+2714 heavy checkmark
+                    setFontColor(Color.GREEN)
+                }
+            } else {
+                errorLabel.apply {
+                    setText("✘") // U+2718 heavy ballot x
+                    setFontColor(Color.RED)
+                }
             }
         }
         onPlayerIdTextUpdated()
         playerIdTextField.addListener { onPlayerIdTextUpdated(); true }
 
-        val currentUserId = UncivGame.Current.settings.multiplayer.userId
+        val currentUserId = UncivGame.Current.settings.multiplayer.getUserId()
         val setCurrentUserButton = "Set current user".toTextButton()
         setCurrentUserButton.onClick {
             playerIdTextField.text = currentUserId
@@ -323,6 +339,7 @@ class PlayerPickerTable(
     internal fun getAvailablePlayerCivs(dontSkipNation: String? = null) =
         previousScreen.ruleset.nations.values.asSequence()
             .filter { it.isMajorCiv }
+            .filterNot { it.hasUnique(UniqueType.WillNotBeChosenForNewGames) }
             .filter { it.name == dontSkipNation || gameParameters.players.none { player -> player.chosenCiv == it.name } }
 
     /**
