@@ -11,7 +11,6 @@ import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories.EmpireOverviewTabState
-import com.unciv.ui.screens.overviewscreen.EmpireOverviewTab.EmpireOverviewTabPersistableData
 
 class EmpireOverviewScreen(
     private var viewingPlayer: Civilization,
@@ -25,38 +24,30 @@ class EmpireOverviewScreen(
     private val tabbedPager: TabbedPager
     private val pageObjects = HashMap<EmpireOverviewCategories, EmpireOverviewTab>()
 
-    companion object {
-        // This is what keeps per-tab states between overview invocations
-        var persistState: Map<EmpireOverviewCategories, EmpireOverviewTabPersistableData>? = null
-
-        private fun updatePersistState(pageObjects: HashMap<EmpireOverviewCategories, EmpireOverviewTab>) {
-            persistState = pageObjects.mapValues { it.value.persistableData }.filterNot { it.value.isEmpty() }
-        }
-    }
+    internal val persistState by game.settings::overview
 
     override fun dispose() {
         tabbedPager.selectPage(-1)
-        updatePersistState(pageObjects)
         super.dispose()
     }
 
     override fun getCivilopediaRuleset() = viewingPlayer.gameInfo.ruleset
 
     init {
-        val selectCategory = defaultCategory ?: EmpireOverviewCategories.values().firstOrNull { it.name == game.settings.lastOverviewPage }
+        val selectCategory = defaultCategory ?: persistState.last
         val iconSize = Constants.defaultFontSize.toFloat()
 
         tabbedPager = TabbedPager(
             stage.width, stage.width,
             centerAreaHeight, centerAreaHeight,
             separatorColor = Color.WHITE,
-            capacity = EmpireOverviewCategories.values().size)
+            capacity = EmpireOverviewCategories.entries.size)
 
-        for (category in EmpireOverviewCategories.values()) {
+        for (category in EmpireOverviewCategories.entries) {
             val tabState = category.testState(viewingPlayer)
             if (tabState == EmpireOverviewTabState.Hidden) continue
             val icon = if (category.iconName.isEmpty()) null else ImageGetter.getImage(category.iconName)
-            val pageObject = category.createTab(viewingPlayer, this, persistState?.get(category))
+            val pageObject = category.createTab(viewingPlayer, this, persistState[category])
             pageObject.pad(10f, 0f, 10f, 0f)
             pageObjects[category] = pageObject
             val index = tabbedPager.addPage(
@@ -72,22 +63,18 @@ class EmpireOverviewScreen(
                 select(pageObject, selection)
             }
         }
+        persistState.update(pageObjects)
 
         val closeButton = getCloseButton { game.popScreen() }
         tabbedPager.decorateHeader(closeButton)
 
         tabbedPager.setFillParent(true)
         stage.addActor(tabbedPager)
-
-//         closeButton.setPosition(stage.width - 10f, stage.height - 10f, Align.topRight)
-//         stage.addActor(closeButton)
    }
 
     override fun recreate(): BaseScreen {
         tabbedPager.selectPage(-1)  // trigger deselect on _old_ instance so the tabs can persist their stuff
-        updatePersistState(pageObjects)
-        return EmpireOverviewScreen(viewingPlayer,
-            EmpireOverviewCategories.values().firstOrNull { it.name == game.settings.lastOverviewPage })
+        return EmpireOverviewScreen(viewingPlayer, persistState.last)
     }
 
     fun resizePage(tab: EmpireOverviewTab) {
@@ -120,7 +107,7 @@ class EmpireOverviewScreen(
         // This is called by UncivGame.popScreen - e.g. after City Tab opened a City and the user closes that CityScreen...
         // Notify the current tab via its IPageExtensions.activated entry point so it can refresh if needed
         val index = tabbedPager.activePage
-        val category = EmpireOverviewCategories.values().getOrNull(index - 1) ?: return
+        val category = EmpireOverviewCategories.entries.getOrNull(index) ?: return
         pageObjects[category]?.activated(index, "", tabbedPager) // Fake caption marks this as popScreen-triggered
     }
 }
