@@ -29,6 +29,7 @@ import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.screens.pickerscreens.PickerScreen
 import com.unciv.ui.screens.worldscreen.WorldScreen
+import yairm210.purity.annotations.Pure
 
 /** Screen used for moving spies between cities */
 class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldScreen) : PickerScreen(true) {
@@ -85,19 +86,19 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
             spySelectionTable.add(spy.name.toLabel())
             spySelectionTable.add(spy.rank.toLabel())
             spySelectionTable.add(spy.getLocationName().toLabel())
-            val actionString = if (spy.action.showTurns && spy.turnsRemainingForAction != -1) 
+            val actionString = if (spy.action.showTurns)
                 "[${spy.action.displayString}] ${spy.turnsRemainingForAction}${Fonts.turn}"
-                else spy.action.displayString
+            else spy.action.displayString
             spySelectionTable.add(actionString.toLabel())
 
             val moveSpyButton = "Move".toTextButton()
             moveSpyButton.onClick {
                 onSpyClicked(moveSpyButton, spy)
             }
-            moveSpyButton.onRightClick { 
+            moveSpyButton.onRightClick {
                 onSpyRightClicked(spy)
             }
-            if (!worldScreen.canChangeState || !spy.isAlive()) {
+            if (!worldScreen.canChangeState || !spy.isAlive() || civInfo.isDefeated()) {
                 // Spectators aren't allowed to move the spies of the Civs they are viewing
                 moveSpyButton.disable()
             }
@@ -129,7 +130,7 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
                 compareBy<City> {
                     it.civ != civInfo
                 }.thenBy {
-                    it.civ.isCityState()
+                    it.civ.isCityState
                 }.thenBy(collator) {
                     it.civ.civName.tr(hideIcons = true)
                 }.thenBy(collator) {
@@ -147,13 +148,13 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
         val label = city.name.toLabel(hideIcons = true)
         label.onClick {
             worldScreen.game.popScreen() // If a detour to this screen (i.e. not directly from worldScreen) is made possible, use resetToWorldScreen instead
-            worldScreen.mapHolder.setCenterPosition(city.location)
+            worldScreen.mapHolder.setCenterPosition(city.location.toHexCoord())
         }
         citySelectionTable.add(label).fill()
         citySelectionTable.add(getSpyIcons(manager.getSpiesInCity(city)))
 
-        val spy = civInfo.espionageManager.getSpyAssignedToCity(city) 
-        if (city.civ.isCityState() && spy != null && spy.canDoCoup()) {
+        val spy = civInfo.espionageManager.getSpyAssignedToCity(city)
+        if (city.civ.isCityState && spy != null && spy.canDoCoup()) {
             val coupButton = CoupButton(city, spy.action == SpyAction.Coup)
             citySelectionTable.add(coupButton)
         } else {
@@ -164,25 +165,37 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
     }
 
     private fun getSpyIcon(spy: Spy) = Table().apply {
-        add (ImageGetter.getImage("OtherIcons/Spy_White").apply {
+        add(ImageGetter.getImage("OtherIcons/Spy_White").apply {
             color = Color.WHITE
         }).size(30f)
-        val color = when(spy.rank) {
+        @Pure
+        fun getColor(rank: Int): Color = when (rank) {
             1 -> Color.BROWN
             2 -> Color.LIGHT_GRAY
-            3 -> Color.GOLD
-            else -> return@apply
+            else -> Color.GOLD
         }
+
+        // If we have 10 or more ranks, display them with a bigger star
+        if (spy.rank >= 10) {
+            val star = ImageGetter.getImage("OtherIcons/Star")
+            star.color = getColor(spy.rank / 10)
+            add(star).size(20f).pad(3f)
+        }
+
+        val color = getColor(spy.rank)
         val starTable = Table()
-        repeat(spy.rank) {
+        // Create a grid of up to 9 stars
+        repeat(spy.rank % 10) {
             val star = ImageGetter.getImage("OtherIcons/Star")
             star.color = color
-            starTable.add(star).size(8f).pad(1f).row()
+            starTable.add(star).size(8f).pad(1f)
+            if (it % 3 == 2)
+                starTable.row()
         }
         add(starTable).center().padLeft(-4f)
 
         // Spectators aren't allowed to move the spies of the Civs they are viewing
-        if (worldScreen.canChangeState && spy.isAlive()) {
+        if (worldScreen.canChangeState && spy.isAlive() && !civInfo.isDefeated()) {
             onClick {
                 onSpyClicked(moveSpyButtons[spy]!!, spy)
             }
@@ -199,12 +212,13 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
     }
 
     private abstract inner class SpyCityActionButton : Button(SmallButtonStyle()) {
-        open fun setDirection(align: Int) { }
+        open fun setDirection(align: Int) {}
     }
 
     // city == null is interpreted as 'spy hideout'
     private inner class MoveToCityButton(city: City?) : SpyCityActionButton() {
         val arrow = ImageGetter.getArrowImage(Align.left)
+
         init {
             arrow.setSize(24f)
             add(arrow).size(24f)
@@ -240,7 +254,7 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
                 button.setDirection(Align.right)
             } else {
                 button.isVisible = city == null // hideout
-                        || !city.espionage.hasSpyOf(civInfo)
+                    || !city.espionage.hasSpyOf(civInfo)
                 button.setDirection(Align.left)
             }
         }
@@ -251,7 +265,7 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
         worldScreen.game.popScreen()
         worldScreen.shouldUpdate = true
     }
-    
+
     private fun resetSelection() {
         selectedSpy = null
         selectedSpyButton?.label?.setText("Move".tr())
@@ -262,6 +276,7 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
 
     private inner class CoupButton(city: City, isCurrentAction: Boolean) : SpyCityActionButton() {
         val fist = ImageGetter.getStatIcon("Resistance")
+
         init {
             fist.setSize(24f)
             add(fist).size(24f)
@@ -271,10 +286,10 @@ class EspionageOverviewScreen(val civInfo: Civilization, val worldScreen: WorldS
             onClick {
                 val spy = selectedSpy!!
                 if (!isCurrentAction) {
-                    ConfirmPopup(this@EspionageOverviewScreen, 
-                            "Do you want to stage a coup in [${city.civ.civName}] with a " +
-                                    "[${(selectedSpy!!.getCoupChanceOfSuccess(false) * 100f).toInt()}]% " +
-                                    "chance of success?", "Stage Coup") {
+                    ConfirmPopup(this@EspionageOverviewScreen,
+                        "Do you want to stage a coup in [${city.civ.civName}] with a " +
+                            "[${(selectedSpy!!.getCoupChanceOfSuccess(false) * 100f).toInt()}]% " +
+                            "chance of success?", "Stage Coup") {
                         spy.setAction(SpyAction.Coup, 1)
                         fist.color = Color.DARK_GRAY
                         update()
