@@ -1,43 +1,31 @@
 package com.unciv.ui.screens.overviewscreen
 
 import com.badlogic.gdx.graphics.Color
-import com.badlogic.gdx.math.Vector2
-import com.badlogic.gdx.scenes.scene2d.Actor
-import com.badlogic.gdx.scenes.scene2d.Group
-import com.badlogic.gdx.scenes.scene2d.Stage
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
-import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
-import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
-import com.unciv.logic.map.HexMath
 import com.unciv.models.ruleset.Policy.PolicyBranchType
-import com.unciv.models.ruleset.nation.getContrastRatio
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
+import com.unciv.models.translations.tr
 import com.unciv.ui.components.extensions.addBorder
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.addSeparatorVertical
-import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.equalizeColumns
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.fonts.Fonts
-import com.unciv.ui.components.input.ClickableCircle
-import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.widgets.AutoScrollPane
 import com.unciv.ui.components.widgets.ColorMarkupLabel
-import com.unciv.ui.components.widgets.ShadowedLabel
 import com.unciv.ui.images.ImageGetter
-import com.unciv.ui.popups.AnimatedMenuPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
+import yairm210.purity.annotations.Readonly
 import kotlin.math.roundToInt
 
 class GlobalPoliticsOverviewTable(
@@ -60,13 +48,13 @@ class GlobalPoliticsOverviewTable(
         defaults().pad(5f)
         background = BaseScreen.skinStrings.getUiBackground(
             "OverviewScreen/DiplomacyOverviewTab/CivTable",
-            tintColor = Color.BLACK
+            tintColor = ImageGetter.CHARCOAL
         )
     }
 
     // Reusable sequences for the Civilizations to display
-    private var undefeatedCivs = sequenceOf<Civilization>()
-    private var defeatedCivs = sequenceOf<Civilization>()
+    private var undefeatedCivs = listOf<Civilization>()
+    private var defeatedCivs = listOf<Civilization>()
 
     private var relevantCivsCount = "?"  // includes unknown civs if player allowed to know
     private var showDiplomacyGroup = false
@@ -93,7 +81,7 @@ class GlobalPoliticsOverviewTable(
     /** Clears fixedContent and adds the header cells.
      *  Needs to stay matched to [createGlobalPoliticsTable].
      *
-     *  9 Columns: 5 info, 4 separators. First gets an empty header for contend below = civ image
+     *  9 Columns: 5 info, 4 separators. The first column gets an empty header, the content below is the civ image
      */
     private fun createGlobalPoliticsHeader() = fixedContent.run {
         val diagramButton = "Show diagram".toTextButton().onClick(::updateDiagram)
@@ -178,7 +166,7 @@ class GlobalPoliticsOverviewTable(
         val allWorldWonders = wonderInfo.collectInfo(viewingPlayer)
 
         for (wonder in allWorldWonders) {
-            if (wonder.civ?.civName == civ.civName) {
+            if (wonder.civ == civ) {
                 val wonderName = wonder.name.toLabel()
                 if (wonder.location != null) {
                     wonderName.onClick {
@@ -193,8 +181,9 @@ class GlobalPoliticsOverviewTable(
         return wonderTable
     }
 
+    @Readonly
     private fun getCivName(otherciv: Civilization): String {
-        if (viewingPlayer.knows(otherciv) || otherciv.civName == viewingPlayer.civName) {
+        if (viewingPlayer.knows(otherciv) || otherciv == viewingPlayer) {
             return otherciv.civName
         }
         return "an unknown civilization"
@@ -203,7 +192,7 @@ class GlobalPoliticsOverviewTable(
     private fun getPoliticsOfCivTable(civ: Civilization): Table {
         val politicsTable = Table(skin)
 
-        if (!viewingPlayer.knows(civ) && civ.civName != viewingPlayer.civName)
+        if (!viewingPlayer.knows(civ) && civ != viewingPlayer)
             return politicsTable
 
         if (civ.isDefeated()) {
@@ -222,14 +211,14 @@ class GlobalPoliticsOverviewTable(
 
         // defensive pacts and declaration of friendships
         for (otherCiv in civ.getKnownCivs()) {
-            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DefensivePact) == true) {
+            if (civ.getDiplomacyManager(otherCiv)?.hasFlag(DiplomacyFlags.DefensivePact) == true) {
                 val friendText = ColorMarkupLabel("Defensive pact with [${getCivName(otherCiv)}]", Color.CYAN)
-                val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DefensivePact)} ${Fonts.turn})".toLabel()
+                val turnsLeftText = " (${civ.getDiplomacyManager(otherCiv)?.getFlag(DiplomacyFlags.DefensivePact)} ${Fonts.turn})".toLabel()
                 politicsTable.add(friendText)
                 politicsTable.add(turnsLeftText).row()
-            } else if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
+            } else if (civ.getDiplomacyManager(otherCiv)?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
                 val friendText = ColorMarkupLabel("Friends with [${getCivName(otherCiv)}]", Color.GREEN)
-                val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DeclarationOfFriendship)} ${Fonts.turn})".toLabel()
+                val turnsLeftText = " (${civ.getDiplomacyManager(otherCiv)?.getFlag(DiplomacyFlags.DeclarationOfFriendship)} ${Fonts.turn})".toLabel()
                 politicsTable.add(friendText)
                 politicsTable.add(turnsLeftText).row()
             }
@@ -238,9 +227,9 @@ class GlobalPoliticsOverviewTable(
 
         // denounced civs
         for (otherCiv in civ.getKnownCivs()) {
-            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.Denunciation) == true) {
+            if (civ.getDiplomacyManager(otherCiv)?.hasFlag(DiplomacyFlags.Denunciation) == true) {
                 val denouncedText = ColorMarkupLabel("Denounced [${getCivName(otherCiv)}]", Color.RED)
-                val turnsLeftText = "(${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.Denunciation)} ${Fonts.turn})".toLabel()
+                val turnsLeftText = "(${civ.getDiplomacyManager(otherCiv)?.getFlag(DiplomacyFlags.Denunciation)} ${Fonts.turn})".toLabel()
                 politicsTable.add(denouncedText)
                 politicsTable.add(turnsLeftText).row()
             }
@@ -249,7 +238,7 @@ class GlobalPoliticsOverviewTable(
 
         //allied CS
         for (cityState in gameInfo.getAliveCityStates()) {
-            if (cityState.diplomacy[civ.civName]?.isRelationshipLevelEQ(RelationshipLevel.Ally) == true) {
+            if (cityState.getDiplomacyManager(civ)?.isRelationshipLevelEQ(RelationshipLevel.Ally) == true) {
                 val alliedText = ColorMarkupLabel("Allied with [${getCivName(cityState)}]", Color.CYAN)
                 politicsTable.add(alliedText).row()
             }
@@ -264,7 +253,10 @@ class GlobalPoliticsOverviewTable(
     // Refresh content and determine landscape/portrait layout
     private fun updateDiagram() {
         persistableData.showDiagram = true
-        val politicsButton = "Show global politics".toTextButton().onClick(::updatePoliticsTable)
+        val politicsButton = "Show global politics".toTextButton().onClick {
+            updatePoliticsTable()
+            overviewScreen.resizePage(this)  // Or else the header stays curernt size, which with any non-empty diagram is most of the client area
+        }
 
         val toggleCityStatesButton: TextButton = Constants.cityStates.toTextButton().apply {
             onClick {
@@ -282,16 +274,16 @@ class GlobalPoliticsOverviewTable(
             add(civTableScroll.addBorder(2f, Color.WHITE)).pad(10f)
         }
 
-        val hideCivsCount = viewingPlayer.hideCivCount() ||
+        val hideCivsCount = viewingPlayer.shouldHideCivCount() ||
             persistableData.includeCityStates && viewingPlayer.hideCityStateCount()
         relevantCivsCount = if (hideCivsCount) "?"
             else gameInfo.civilizations.count {
-                !it.isSpectator() && !it.isBarbarian() && (persistableData.includeCityStates || !it.isCityState())
-            }.toString()
-        undefeatedCivs = sequenceOf(viewingPlayer) +
+                !it.isSpectator() && !it.isBarbarian && (persistableData.includeCityStates || !it.isCityState)
+            }.tr()
+        undefeatedCivs = listOf(viewingPlayer) +
                 viewingPlayer.diplomacyFunctions.getKnownCivsSorted(persistableData.includeCityStates)
         defeatedCivs = viewingPlayer.diplomacyFunctions.getKnownCivsSorted(persistableData.includeCityStates, true)
-            .filter { it.isDefeated() }
+            .filter { it.isDefeated() }.toList()
 
         clear()
         fixedContent.clear()
@@ -306,7 +298,7 @@ class GlobalPoliticsOverviewTable(
         if (showDiplomacyGroup) {
             val diplomacySize = (overviewScreen.stage.width - (if (portraitMode) 0f else civTable.minWidth))
                 .coerceAtMost(overviewScreen.centerAreaHeight)
-            val diplomacyGroup = DiplomacyGroup(undefeatedCivs, diplomacySize)
+            val diplomacyGroup = GlobalPoliticsDiagramGroup(undefeatedCivs, diplomacySize)
             table.add(diplomacyGroup).top()
         }
 
@@ -329,10 +321,11 @@ class GlobalPoliticsOverviewTable(
         civTableScroll.setScrollingDisabled(portraitMode, portraitMode)
     }
 
-    /** Same as [Civilization.hideCivCount] but for City-States instead of Major Civs */
+    /** Same as [Civilization.shouldHideCivCount] but for City-States instead of Major Civs */
+    @Readonly
     private fun Civilization.hideCityStateCount(): Boolean {
         if (!gameInfo.gameParameters.randomNumberOfCityStates) return false
-        val knownCivs = 1 + getKnownCivs().count { it.isCityState() }
+        val knownCivs = 1 + getKnownCivs().count { it.isCityState }
         if (knownCivs >= gameInfo.gameParameters.maxNumberOfCityStates) return false
         if (hasUnique(UniqueType.OneTimeRevealEntireMap)) return false
         return true
@@ -363,13 +356,13 @@ class GlobalPoliticsOverviewTable(
         add("Our Civilization:".toLabel()).colspan(columns).left().padLeft(10f).padTop(10f).row()
         add(getCivMiniTable(viewingPlayer)).left()
         val scoreText = if (viewingPlayer.isDefeated()) Fonts.death.toString()
-            else viewingPlayer.calculateTotalScore().toInt().toString()
+            else viewingPlayer.calculateTotalScore().toInt().tr()
         add(scoreText.toLabel()).left().row()
         val turnsTillNextDiplomaticVote = viewingPlayer.getTurnsTillNextDiplomaticVote() ?: return
         add("Turns until the next\ndiplomacy victory vote: [$turnsTillNextDiplomaticVote]".toLabel()).colspan(columns).row()
     }
 
-    private fun Table.addCivsCategory(columns: Int, aliveOrDefeated: String, civs: Sequence<Civilization>) {
+    private fun Table.addCivsCategory(columns: Int, aliveOrDefeated: String, civs: List<Civilization>) {
         addSeparator()
         val count = civs.count()
         add("Known and $aliveOrDefeated ([$count])".toLabel())
@@ -389,10 +382,10 @@ class GlobalPoliticsOverviewTable(
         }
 
         for (civ in civs) {
-            if (lastCivWasMajor && civ.isCityState())
+            if (lastCivWasMajor && civ.isCityState)
                 advanceCols(columns)
             add(getCivMiniTable(civ)).left()
-            if (civ.isCityState()) {
+            if (civ.isCityState) {
                 advanceCols(1)
             } else {
                 add(civ.calculateTotalScore().toInt().toLabel()).left()
@@ -401,153 +394,5 @@ class GlobalPoliticsOverviewTable(
         }
     }
 
-    /** This is the 'spider net'-like polygon showing one line per civ-civ relation
-     *  @param undefeatedCivs Civs to display - note the viewing player is always included, so it's possible the name is off and there's a dead civ included.
-     *  @param freeSize Width and height this [Group] sizes itself to
-     */
-    private class DiplomacyGroup(
-        undefeatedCivs: Sequence<Civilization>,
-        freeSize: Float
-    ): Group() {
-        private fun onCivClicked(civLines: HashMap<String, MutableSet<Actor>>, name: String) {
-            // ignore the clicks on "dead" civilizations, and remember the selected one
-            val selectedLines = civLines[name] ?: return
-
-            // let's check whether lines of all civs are visible (except selected one)
-            var atLeastOneLineVisible = false
-            var allAreLinesInvisible = true
-            for (lines in civLines.values) {
-                // skip the civilization selected by user, and civilizations with no lines
-                if (lines == selectedLines || lines.isEmpty()) continue
-
-                val visibility = lines.first().isVisible
-                atLeastOneLineVisible = atLeastOneLineVisible || visibility
-                allAreLinesInvisible = allAreLinesInvisible && visibility
-
-                // check whether both visible and invisible lines are present
-                if (atLeastOneLineVisible && !allAreLinesInvisible) {
-                    // invert visibility of the selected civ's lines
-                    selectedLines.forEach { it.isVisible = !it.isVisible }
-                    return
-                }
-            }
-
-            if (selectedLines.first().isVisible) {
-                // invert visibility of all lines except selected one
-                civLines.filter { it.key != name }
-                    .forEach { it.value.forEach { line -> line.isVisible = !line.isVisible } }
-            } else {
-                // it happens only when all are visible except selected one
-                // invert visibility of the selected civ's lines
-                selectedLines.forEach { it.isVisible = !it.isVisible }
-            }
-        }
-
-        init {
-            setSize(freeSize, freeSize)
-
-            // An Image Actor does not respect alpha for its hit area, it's always square, but we want a clickable _circle_
-            // Radius to show legend should be no larger than freeSize / 2.25f - 15f (see below), let's make it a little smaller
-            val clickableArea = ClickableCircle(freeSize / 1.25f - 25f)
-            clickableArea.onActivation {
-                DiagramLegendPopup(stage, this)
-            }
-            clickableArea.center(this)
-            addActor(clickableArea)
-
-            val civGroups = HashMap<String, Actor>()
-            val civLines = HashMap<String, MutableSet<Actor>>()
-            val civCount = undefeatedCivs.count()
-
-            for ((i, civ) in undefeatedCivs.withIndex()) {
-                val civGroup = ImageGetter.getNationPortrait(civ.nation, 30f)
-
-                val vector = HexMath.getVectorForAngle(2 * Math.PI.toFloat() * i / civCount)
-                civGroup.center(this)
-                civGroup.moveBy(vector.x * freeSize / 2.25f, vector.y * freeSize / 2.25f)
-                civGroup.touchable = Touchable.enabled
-                civGroup.onClick {
-                    onCivClicked(civLines, civ.civName)
-                }
-                civGroup.addTooltip(civ.civName, tipAlign = Align.bottomLeft)
-
-                civGroups[civ.civName] = civGroup
-                addActor(civGroup)
-            }
-
-            for (civ in undefeatedCivs) {
-                if (civ.isDefeated()) continue // if you're dead, icon but no lines (One more turn mode after losing)
-                for (diplomacy in civ.diplomacy.values) {
-                    val otherCiv = diplomacy.otherCiv()
-                    if (otherCiv !in undefeatedCivs || otherCiv.isDefeated()) continue
-                    val civGroup = civGroups[civ.civName]!!
-                    val otherCivGroup = civGroups[diplomacy.otherCivName]!!
-
-                    val statusLine = ImageGetter.getLine(
-                        startX = civGroup.x + civGroup.width / 2,
-                        startY = civGroup.y + civGroup.height / 2,
-                        endX = otherCivGroup.x + otherCivGroup.width / 2,
-                        endY = otherCivGroup.y + otherCivGroup.height / 2,
-                        width = 2f
-                    )
-
-                    statusLine.color = if (diplomacy.diplomaticStatus == DiplomaticStatus.War) Color.RED
-                    else if (diplomacy.diplomaticStatus == DiplomaticStatus.DefensivePact
-                        || (diplomacy.civInfo.isCityState() && diplomacy.civInfo.getAllyCiv() == diplomacy.otherCivName)
-                        || (otherCiv.isCityState() && otherCiv.getAllyCiv() == diplomacy.civInfo.civName)
-                    ) Color.CYAN
-                    else diplomacy.relationshipLevel().color
-
-                    if (!civLines.containsKey(civ.civName)) civLines[civ.civName] = mutableSetOf()
-                    civLines[civ.civName]!!.add(statusLine)
-
-                    addActorAt(0, statusLine)
-                }
-            }
-
-        }
-    }
-
-    private class DiagramLegendPopup(stage: Stage, diagram: Actor) : AnimatedMenuPopup(stage, diagram.getCenterInStageCoordinates()) {
-        init {
-            touchable = Touchable.enabled
-            onActivation { close() }
-        }
-
-        companion object {
-            private fun Actor.getCenterInStageCoordinates(): Vector2 = localToStageCoordinates(Vector2(width / 2, height / 2))
-
-            const val lineWidth = 3f  // a little thicker than the actual diagram
-            const val lowContrastWidth = 4f
-            const val lineLength = 120f
-        }
-
-        override fun createContentTable(): Table {
-            val legend = Table()
-            legend.background = ImageGetter.getDrawable("OtherIcons/Politics-diagram-bg")
-            legend.add(ShadowedLabel("Diagram line colors", Constants.headingFontSize)).colspan(2).row()
-            //todo Rethink hardcoding together with the statusLine.color one in DiplomacyGroup
-            legend.addLegendRow("War", Color.RED)
-            for (level in RelationshipLevel.values()) {
-                val lineColor = if (level == RelationshipLevel.Ally) Color.CYAN else level.color
-                legend.addLegendRow(level.name, lineColor)
-            }
-            legend.addLegendRow(Constants.defensivePact, Color.CYAN)
-            return super.createContentTable()!!.apply {
-                add(legend).grow()
-            }
-        }
-
-        fun Table.addLegendRow(text: String, color: Color) {
-            // empiric hack to equalize the "visual impact" a little. Afraid is worst at contrast 1.4, Enemy has 9.8
-            val contrast = getContrastRatio(Color.DARK_GRAY, color).toFloat()
-            val width = lineWidth + (lowContrastWidth - lineWidth) / contrast.coerceAtLeast(1f)
-            val line = ImageGetter.getLine(0f, width / 2, lineLength, width / 2, width)
-            line.color = color
-            add(line).size(lineLength, width).padTop(5f)
-            add(ShadowedLabel(text)).padLeft(5f).padTop(10f).row()
-        }
-    }
-
     //endregion
 }
