@@ -8,10 +8,11 @@ import com.unciv.UncivGame
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.tile.Tile
-import com.unciv.logic.trade.TradeType
+import com.unciv.logic.trade.TradeOfferType
 import com.unciv.models.ruleset.tile.ResourceSupplyList
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileResource
+import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.addSeparator
@@ -56,13 +57,17 @@ class ResourcesOverviewTab(
 
     private val resourceDrilldown: ResourceSupplyList = viewingPlayer.detailedCivResources
     private val extraDrilldown: ResourceSupplyList = getExtraDrilldown()
-    private val drilldownSequence = resourceDrilldown.asSequence() + extraDrilldown.asSequence()
+    private val allResources = resourceDrilldown.asSequence() + extraDrilldown  // Do not materialize into another ResourceSupplyList, we have intentional zero amounts
 
     // Order of source ResourceSupplyList: by tiles, enumerating the map in that spiral pattern
     // UI should not surprise player, thus we need a deterministic and guessable order
-    private val resources: List<TileResource> = drilldownSequence
+    private val resources: List<TileResource> = allResources
         .map { it.resource }
-        .filter { it.resourceType != ResourceType.Bonus }
+        .filter {
+            it.resourceType != ResourceType.Bonus &&
+            !it.hasUnique(UniqueType.NotShownOnWorldScreen, viewingPlayer.state) &&
+            !it.isCityWide // These are Civ-wide resources, so don't show the city-wide ones.
+        }
         .distinct()
         .sortedWith(
             compareBy<TileResource> { it.resourceType }
@@ -77,22 +82,22 @@ class ResourcesOverviewTab(
     private fun ResourceSupplyList.getLabel(resource: TileResource, origin: String): Label? {
         fun isAlliedAndUnimproved(tile: Tile): Boolean {
             val owner = tile.getOwner() ?: return false
-            if (owner != viewingPlayer && !(owner.isCityState() && owner.getAllyCiv() == viewingPlayer.civName)) return false
+            if (owner != viewingPlayer && !(owner.isCityState && owner.allyCiv == viewingPlayer)) return false
             return tile.countAsUnimproved()
         }
         val amount = get(resource, origin)?.amount ?: return null
-        val label = if (resource.isStockpiled() && amount > 0) "+$amount".toLabel()
+        val label = if (resource.isStockpiled && amount > 0) "+$amount".toLabel()
             else amount.toLabel()
         if (origin == ExtraInfoOrigin.Unimproved.name)
             label.onClick { overviewScreen.showOneTimeNotification(
-                gameInfo.getExploredResourcesNotification(viewingPlayer, resource.name, filter = ::isAlliedAndUnimproved)
+                gameInfo.getExploredResourcesNotification(viewingPlayer, resource, filter = ::isAlliedAndUnimproved)
             ) }
         return label
     }
 
     private fun ResourceSupplyList.getTotalLabel(resource: TileResource): Label {
         val total = filter { it.resource == resource }.sumOf { it.amount }
-        return if (resource.isStockpiled() && total > 0) "+$total".toLabel()
+        return if (resource.isStockpiled && total > 0) "+$total".toLabel()
         else total.toLabel()
     }
 
@@ -122,13 +127,14 @@ class ResourcesOverviewTab(
         TradeOffer("Trade offer","Trade offer", "Resources we're offering in trades")
         ;
         companion object {
-            fun safeValueOf(name: String) = values().firstOrNull { it.name == name }
+            fun safeValueOf(name: String) = entries.firstOrNull { it.name == name }
         }
     }
     private val fixedContent = Table()
 
     init {
         defaults().pad(defaultPad)
+        top()
         fixedContent.defaults().pad(defaultPad)
 
         turnImageH.onClick {
@@ -236,14 +242,17 @@ class ResourcesOverviewTab(
             row()
         }
 
+        if (rows == 0) return // can happen when opening overview on turn 0 before founding a city
         equalizeColumns(fixedContent, this)
         overviewScreen.resizePage(this)  // Without the height is miscalculated - shouldn't be
     }
 
-    private fun Tile.countAsUnimproved(): Boolean = resource != null &&
-            tileResource.resourceType != ResourceType.Bonus &&
-            hasViewableResource(viewingPlayer) &&
+    private fun Tile.countAsUnimproved(): Boolean {
+        val resource = tileResource
+        return viewingPlayer.canSeeResource(resource) &&
+            resource.resourceType != ResourceType.Bonus &&
             !providesResources(viewingPlayer)
+    }
 
     private fun getExtraDrilldown(): ResourceSupplyList {
         val newResourceSupplyList = ResourceSupplyList(keepZeroAmounts = true)
@@ -251,7 +260,7 @@ class ResourcesOverviewTab(
         fun City.addUnimproved() {
             for (tile in getTiles())
                 if (tile.countAsUnimproved())
-                    newResourceSupplyList.add(tile.tileResource, ExtraInfoOrigin.Unimproved.name)
+                    newResourceSupplyList.add(tile.tileResource!!, ExtraInfoOrigin.Unimproved.name)
         }
 
         // Show resources relevant to WTLK day and/or needing improvement
@@ -269,12 +278,12 @@ class ResourcesOverviewTab(
 
         for (otherCiv in viewingPlayer.getKnownCivs()) {
             // Show resources received through trade
-            for (trade in otherCiv.tradeRequests.filter { it.requestingCiv == viewingPlayer.civName })
-                for (offer in trade.trade.theirOffers.filter { it.type == TradeType.Strategic_Resource || it.type == TradeType.Luxury_Resource })
+            for (trade in otherCiv.tradeRequests.filter { it.requestingCiv == viewingPlayer.civID })
+                for (offer in trade.trade.theirOffers.filter { it.type == TradeOfferType.Strategic_Resource || it.type == TradeOfferType.Luxury_Resource })
                     newResourceSupplyList.add(gameInfo.ruleset.tileResources[offer.name]!!, ExtraInfoOrigin.TradeOffer.name, offer.amount)
 
             // Show resources your city-state allies have left unimproved
-            if (!otherCiv.isCityState() || otherCiv.getAllyCiv() != viewingPlayer.civName) continue
+            if (!otherCiv.isCityState || otherCiv.allyCiv != viewingPlayer) continue
             for (city in otherCiv.cities)
                 city.addUnimproved()
         }
@@ -282,7 +291,7 @@ class ResourcesOverviewTab(
         /** Show unlocked **strategic** resources even if you have no access at all */
         for (resource in viewingPlayer.gameInfo.ruleset.tileResources.values) {
             if (resource.resourceType != ResourceType.Strategic) continue
-            if (resource.revealedBy == null || viewingPlayer.tech.isResearched(resource.revealedBy!!))
+            if (viewingPlayer.canSeeResource(resource))
                 newResourceSupplyList.add(resource, "No source", 0)
         }
 
