@@ -135,10 +135,12 @@ class StatsOverviewTab(
         init {
             defaults().pad(5f)
             uniques = sequenceOf(
-                    UniqueType.ConditionalBetweenHappiness,
-                    UniqueType.ConditionalBelowHappiness
+                    UniqueType.ConditionalWhenAboveAmountStatResource,
+                    UniqueType.ConditionalWhenBetweenStatResource,
+                    UniqueType.ConditionalWhenBelowAmountStatResource,
                 ).flatMap { conditionalType ->
-                    viewingPlayer.getTriggeredUniques(conditionalType)
+                    viewingPlayer.getMatchingUniques(conditionalType)
+                        .filter { it.params.last() == "Happiness" }
                         .sortedBy { it.type } // otherwise order might change as a HashMap is involved
                 }.filterNot { it.isHiddenToUsers() }
                 .toSet()
@@ -225,7 +227,7 @@ class StatsOverviewTab(
     private fun updateScoreTable() = scoreTable.apply {
         clear()
         val scoreHeader = Table()
-        val scoreIcon = ImageGetter.getImage("CityStateIcons/Cultured")
+        val scoreIcon = ImageGetter.getImage("OtherIcons/Score")
         scoreIcon.color = Color.FIREBRICK
         scoreHeader.add(scoreIcon).padRight(1f).size(Constants.headingFontSize.toFloat())
         scoreHeader.add("Score".toLabel(fontSize = Constants.headingFontSize))
