@@ -15,22 +15,29 @@ import com.unciv.models.ruleset.unit.Promotion
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.SoundPlayer
 import com.unciv.ui.components.extensions.isEnabled
+import com.unciv.ui.components.extensions.toCheckBox
 import com.unciv.ui.components.extensions.toTextButton
-import com.unciv.ui.components.input.KeyboardBinding
-import com.unciv.ui.components.input.keyShortcuts
-import com.unciv.ui.components.input.onActivation
-import com.unciv.ui.components.input.onClick
-import com.unciv.ui.components.input.onDoubleClick
+import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.components.input.*
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import kotlin.math.abs
 
-class PromotionPickerScreen(
+class PromotionPickerScreen private constructor(
     val unit: MapUnit,
-    private val closeOnPick: Boolean = true,
-    private val onChange: (() -> Unit)? = null
+    private val closeOnPick: Boolean,
+    private val originalName: String?,
+    private val onChange: (() -> Unit)?
 ) : PickerScreen(), RecreateOnResize {
+    /** Show promotions organized by depencencies, allow picking new ones, allow unit rename
+     *  @param unit The MapUnit to work with
+     *  @param closeOnPick Should picking a new promotion close the screen?
+     *  @param onChange Optional callback called when a promotion is picked or during close if the name was changed
+     */
+    constructor(unit: MapUnit, closeOnPick: Boolean = true, onChange: (() -> Unit)? = null)
+        : this(unit, closeOnPick, unit.instanceName, onChange)
+
     // Style stuff
     private val colors = skin[PromotionScreenColors::class.java]
     private val promotedLabelStyle = Label.LabelStyle(skin[Label.LabelStyle::class.java]).apply {
@@ -49,19 +56,28 @@ class PromotionPickerScreen(
     private val canChangeState = GUI.isAllowedChangeState()
     private val canPromoteNow = canChangeState &&
             unit.promotions.canBePromoted() &&
-            unit.currentMovement > 0 && unit.attacksThisTurn == 0
+            unit.hasMovement() && unit.attacksThisTurn == 0
 
     // Logic
     private val tree = PromotionTree(unit)
 
+    // This if we should save the unit promotion or not.
+    private var saveUnitPromotion = false
 
     init {
-        setDefaultCloseAction()
+        closeButton.onActivation {
+            if (unit.instanceName != originalName)
+                onChange?.invoke()
+            game.popScreen()
+        }
+        closeButton.keyShortcuts.add(KeyCharAndCode.BACK)
 
         if (canPromoteNow) {
             rightSideButton.setText("Pick promotion".tr())
             rightSideButton.onClick(UncivSound.Silent) {
                 acceptPromotion(selectedPromotion)
+                
+                checkSaveUnitPromotion()
             }
         } else {
             rightSideButton.isVisible = false
@@ -174,11 +190,37 @@ class PromotionPickerScreen(
             // Each root tree should start from a completely empty row.
             row += 1
         }
-
-        topTable.add(promotionsTable)
+        topTable.add(promotionsTable).row()
+        saveUnitPromotionForCity()
+        
+        if (unit.statusMap.isNotEmpty()) addStatuses()
         addConnectingLines(emptySet())
     }
 
+    // adds the checkBoxs to choice to save unit promotion.
+    private fun saveUnitPromotionForCity() {
+        /* if you are not in a city tile then don't show up 
+         then player should not be able to save promotion in enermy tiles/puppet citys 
+         even their own because you can't build any unit there.
+        */ 
+        val currentCity = unit.currentTile.getCity() ?: return
+        if (currentCity.civ != unit.civ) return
+        if (currentCity.isPuppet) return
+        val checkBoxSaveUnitPromotion = "Default promotions for [${unit.baseUnit.name}]".toCheckBox(saveUnitPromotion) {saveUnitPromotion = it}
+        topTable.add(checkBoxSaveUnitPromotion).left().padTop(10f)
+    }
+    
+    // going to re-use this bit of code 2 time so turn it into a funtion
+    private fun checkSaveUnitPromotion() {
+        if (!saveUnitPromotion)  return
+        val unitCurrentCity = unit.currentTile.getCity()
+        if (unitCurrentCity != null) {
+            // If you are clicked the save baseUnit promotion, you want the next baseUnit to have the same promotion.
+            unitCurrentCity.unitShouldUseSavedPromotion[unit.baseUnit.name] = true
+            unitCurrentCity.unitToPromotions[unit.baseUnit.name] = unit.promotions
+        }
+    }
+    
     private fun getButton(tree: PromotionTree, node: PromotionTree.PromotionNode) : PromotionButton {
         val isPickable = canPromoteNow &&
             (!node.pathIsAmbiguous || node.distanceToAdopted == 1) &&
@@ -206,10 +248,23 @@ class PromotionPickerScreen(
         if (isPickable)
             button.onDoubleClick(UncivSound.Silent) {
                 acceptPromotion(button)
+                checkSaveUnitPromotion()
             }
 
         return button
     }
+    
+    private fun addStatuses() {
+        val statusTable = Table().apply { defaults().pad(5f) }
+        for (status in unit.statusMap.values.sortedBy { it.turnsLeft }) {
+            val statusButton = "{${status.name}}: ${status.turnsLeft}${Fonts.turn}".toTextButton()
+            val description = "{${status.name}}: ${status.turnsLeft}${Fonts.turn}\n".tr() +
+                    unit.civ.gameInfo.ruleset.unitPromotions[status.name]?.getDescription(emptySet())
+            statusButton.onClick { descriptionLabel.setText(description) }
+            statusTable.add(statusButton).left().row()
+        }
+        topTable.add(statusTable).row()
+    }
 
     private fun addConnectingLines(path: Set<Promotion>) {
         promotionsTable.pack()
@@ -337,9 +392,8 @@ class PromotionPickerScreen(
     }
 
     override fun recreate() = recreate(closeOnPick)
-
     fun recreate(closeOnPick: Boolean): BaseScreen {
-        val newScreen = PromotionPickerScreen(unit, closeOnPick, onChange)
+        val newScreen = PromotionPickerScreen(unit, closeOnPick, originalName, onChange)
         newScreen.setScrollY(scrollPane.scrollY)
         return newScreen
     }
