@@ -2,24 +2,21 @@ package com.unciv.ui.screens.savescreens
 
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
-import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
-import com.badlogic.gdx.utils.SerializationException
 import com.unciv.Constants
 import com.unciv.logic.MissingModsException
+import com.unciv.logic.MissingNationException
 import com.unciv.logic.UncivShowableException
 import com.unciv.logic.files.PlatformSaverLoader
 import com.unciv.logic.files.UncivFiles
-import com.unciv.logic.github.Github
-import com.unciv.logic.github.Github.folderNameToRepoName
+import com.unciv.logic.github.GithubAPI
+import com.unciv.logic.github.GithubAPI.downloadAndExtract
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.enable
-import com.unciv.ui.components.extensions.isEnabled
-import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.keyShortcuts
@@ -31,52 +28,20 @@ import com.unciv.ui.popups.ToastPopup
 import com.unciv.utils.Concurrency
 import com.unciv.utils.Log
 import com.unciv.utils.launchOnGLThread
-import java.io.FileNotFoundException
+import kotlinx.coroutines.CoroutineScope
 
 class LoadGameScreen : LoadOrSaveScreen() {
     private val copySavedGameToClipboardButton = getCopyExistingSaveToClipboardButton()
-    private val errorLabel = "".toLabel(Color.RED)
-    private val loadMissingModsButton = getLoadMissingModsButton()
-    private var missingModsToLoad: Iterable<String> = emptyList()
-
-    companion object {
+    
+    /** Inheriting here again exposes [getLoadExceptionMessage] and [loadMissingMods] to
+     *  other clients (WorldScreen, QuickSave, Multiplayer) without needing to rewrite many imports 
+     */
+    companion object : Helpers {
         private const val loadGame = "Load game"
         private const val loadFromCustomLocation = "Load from custom location"
         private const val loadFromClipboard = "Load copied data"
         private const val copyExistingSaveToClipboard = "Copy saved game to clipboard"
-        private const val downloadMissingMods = "Download missing mods"
-
-        /** Gets a translated exception message to show to the user.
-         * @return The first returned value is the message, the second is signifying if the user can likely fix this problem. */
-        fun getLoadExceptionMessage(ex: Throwable, primaryText: String = "Could not load game!"): Pair<String, Boolean> {
-            val errorText = StringBuilder(primaryText.tr())
-
-            val isUserFixable: Boolean
-            errorText.appendLine()
-            when (ex) {
-                is UncivShowableException -> {
-                    errorText.append(ex.localizedMessage)
-                    isUserFixable = true
-                }
-                is SerializationException -> {
-                    errorText.append("The file data seems to be corrupted.".tr())
-                    isUserFixable = false
-                }
-                is FileNotFoundException -> {
-                    isUserFixable = if (ex.cause?.message?.contains("Permission denied") == true) {
-                        errorText.append("You do not have sufficient permissions to access the file.".tr())
-                        true
-                    } else {
-                        false
-                    }
-                }
-                else -> {
-                    errorText.append("Unhandled problem, [${ex::class.simpleName} ${ex.stackTraceToString()}]".tr())
-                    isUserFixable = false
-                }
-            }
-            return Pair(errorText.toString(), isUserFixable)
-        }
+        internal const val downloadMissingMods = "Download missing mods"
     }
 
     init {
@@ -85,7 +50,7 @@ class LoadGameScreen : LoadOrSaveScreen() {
 
         setDefaultCloseAction()
         rightSideTable.initRightSideTable()
-        rightSideButton.onActivation { onLoadGame() }
+        rightSideButton.onActivation { onLoadGame(selectedSave) }
         rightSideButton.keyShortcuts.add(KeyCharAndCode.RETURN)
         rightSideButton.isVisible = false
         pickerPane.bottomTable.background = skinStrings.getUiBackground("LoadGameScreen/BottomTable", tintColor = skinStrings.skinConfig.clearColor)
@@ -106,27 +71,26 @@ class LoadGameScreen : LoadOrSaveScreen() {
         rightSideButton.enable()
     }
 
-    override fun doubleClickAction() {
-        onLoadGame()
+    override fun doubleClickAction(saveGameFile: FileHandle) {
+        onLoadGame(saveGameFile)
     }
 
     private fun Table.initRightSideTable() {
         add(getLoadFromClipboardButton()).row()
         addLoadFromCustomLocationButton()
-        add(errorLabel).width(stage.width / 2).row()
-        add(loadMissingModsButton).row()
+        add(errorLabel).width(stage.width / 2).center().row()
         add(deleteSaveButton).row()
         add(copySavedGameToClipboardButton).row()
         add(showAutosavesCheckbox).row()
     }
 
-    private fun onLoadGame() {
-        if (selectedSave == null) return
+    private fun onLoadGame(saveGameFile: FileHandle?) {
+        if (saveGameFile == null) return
         val loadingPopup = LoadingPopup(this)
         Concurrency.run(loadGame) {
             try {
                 // This is what can lead to ANRs - reading the file and setting the transients, that's why this is in another thread
-                val loadedGame = game.files.loadGameFromFile(selectedSave!!)
+                val loadedGame = game.files.loadGameFromFile(saveGameFile)
                 game.loadGame(loadedGame, callFromLoadScreen = true)
             } catch (notAPlayer: UncivShowableException) {
                 launchOnGLThread {
@@ -176,7 +140,7 @@ class LoadGameScreen : LoadOrSaveScreen() {
             errorLabel.isVisible = false
             loadFromCustomLocationButton.setText(Constants.loading.tr())
             loadFromCustomLocationButton.disable()
-            Concurrency.run(Companion.loadFromCustomLocation) {
+            Concurrency.run(loadFromCustomLocation) {
                 game.files.loadGameFromCustomLocation(
                     {
                         Concurrency.run { game.loadGame(it, callFromLoadScreen = true) }
@@ -196,15 +160,9 @@ class LoadGameScreen : LoadOrSaveScreen() {
     private fun getCopyExistingSaveToClipboardButton(): TextButton {
         val copyButton = copyExistingSaveToClipboard.toTextButton()
         copyButton.onActivation {
-            if (selectedSave == null) return@onActivation
+            val file = selectedSave ?: return@onActivation
             Concurrency.run(copyExistingSaveToClipboard) {
-                try {
-                    val gameText = selectedSave!!.readString()
-                    Gdx.app.clipboard.contents = if (gameText[0] == '{') Gzip.zip(gameText) else gameText
-                } catch (ex: Throwable) {
-                    ex.printStackTrace()
-                    ToastPopup("Could not save game to clipboard!", this@LoadGameScreen)
-                }
+                copySaveToClipboard(file)
             }
         }
         copyButton.disable()
@@ -214,78 +172,113 @@ class LoadGameScreen : LoadOrSaveScreen() {
         return copyButton
     }
 
-    private fun getLoadMissingModsButton(): TextButton {
-        val button = downloadMissingMods.toTextButton()
-        button.onClick {
-            loadMissingMods()
+    private fun CoroutineScope.copySaveToClipboard(file: FileHandle) {
+        val gameText = try {
+            file.readString()
+        } catch (ex: Throwable) {
+            val (errorText, isUserFixable) = getLoadExceptionMessage(ex, saveToClipboardErrorMessage)
+            if (!isUserFixable)
+                Log.error(saveToClipboardErrorMessage, ex)
+            launchOnGLThread {
+                ToastPopup(errorText, this@LoadGameScreen)
+            }
+            return
+        }
+        try {
+            Gdx.app.clipboard.contents = if (gameText[0] == '{') Gzip.zip(gameText) else gameText
+            launchOnGLThread {
+                ToastPopup("'[${file.name()}]' copied to clipboard!", this@LoadGameScreen)
+            }
+        } catch (ex: Throwable) {
+            Log.error(saveToClipboardErrorMessage, ex)
+            launchOnGLThread {
+                ToastPopup(saveToClipboardErrorMessage, this@LoadGameScreen)
+            }
         }
-        button.isVisible = false
-        return button
     }
 
     private fun handleLoadGameException(ex: Exception, primaryText: String = "Could not load game!") {
-        Log.error("Error while loading game", ex)
-        val (errorText, isUserFixable) = getLoadExceptionMessage(ex, primaryText)
+        val isUserFixable = handleException(ex, primaryText)
+        if (!isUserFixable) {
+            val cantLoadGamePopup = Popup(this@LoadGameScreen)
+            cantLoadGamePopup.addGoodSizedLabel("It looks like your saved game can't be loaded!").row()
+            cantLoadGamePopup.addGoodSizedLabel("If you could copy your game data (\"Copy saved game to clipboard\" - ").row()
+            cantLoadGamePopup.addGoodSizedLabel("  paste into an email to yairm210@hotmail.com)").row()
+            cantLoadGamePopup.addGoodSizedLabel("I could maybe help you figure out what went wrong, since this isn't supposed to happen!").row()
+            cantLoadGamePopup.addCloseButton()
+            cantLoadGamePopup.open()
+        }
 
-        Concurrency.runOnGLThread {
-            if (!isUserFixable) {
-                val cantLoadGamePopup = Popup(this@LoadGameScreen)
-                cantLoadGamePopup.addGoodSizedLabel("It looks like your saved game can't be loaded!").row()
-                cantLoadGamePopup.addGoodSizedLabel("If you could copy your game data (\"Copy saved game to clipboard\" - ").row()
-                cantLoadGamePopup.addGoodSizedLabel("  paste into an email to yairm210@hotmail.com)").row()
-                cantLoadGamePopup.addGoodSizedLabel("I could maybe help you figure out what went wrong, since this isn't supposed to happen!").row()
-                cantLoadGamePopup.addCloseButton()
-                cantLoadGamePopup.open()
+        if (ex is MissingModsException) {
+            loadMissingModsAsync(ex.missingMods)
+        }
+        if (ex is MissingNationException){
+            redownloadUnupdatedMods(ex.modNames)
+        }
+    }
+
+    /** If any nation is missing from a saved game, chances are that one of the mods needs to be redownloaded
+     * Since we don't know which one, we check all mods that
+     * A. Have at least one nation
+     * B. Are outdated
+     * */
+    private fun redownloadUnupdatedMods(modNames: LinkedHashSet<String>) {
+        descriptionLabel.setText("Downloading unupdated mods...")
+        Concurrency.runOnNonDaemonThreadPool("redownloadUnupdatedMods") { 
+            val modsToCheck = modNames.mapNotNull { RulesetCache[it] }
+                .filter { it.nations.any() && it.modOptions.modUrl.isNotEmpty()  }
+            
+            val reposToUpdate = modsToCheck
+                .mapNotNull { 
+                    val repo = GithubAPI.Repo.parseUrl(it.modOptions.modUrl) ?: return@mapNotNull null
+                    if (it.modOptions.lastUpdated == repo.pushed_at) return@mapNotNull null
+                    repo
+                }
+            
+            Concurrency.runOnGLThread {
+                ToastPopup("Updating mods: $reposToUpdate", this@LoadGameScreen)
+                descriptionLabel.setText("Downloading unupdated mods - 0/${reposToUpdate.size}")
             }
 
-            errorLabel.setText(errorText)
-            errorLabel.isVisible = true
-            if (ex is MissingModsException) {
-                loadMissingModsButton.isVisible = true
-                missingModsToLoad = ex.missingMods
+            for ((index, repo) in reposToUpdate.withIndex()) {
+                repo.downloadAndExtract()
+                Concurrency.runOnGLThread {
+                    ToastPopup("Downloaded ${repo.name}", this@LoadGameScreen)
+                    descriptionLabel.setText("Downloading unupdated mods - ${index+1}/${reposToUpdate.size}")
+                }
             }
         }
     }
 
-    private fun loadMissingMods() {
-        loadMissingModsButton.isEnabled = false
+    private fun loadMissingModsAsync(missingMods: Iterable<String>) {
         descriptionLabel.setText(Constants.loading.tr())
         Concurrency.runOnNonDaemonThreadPool(downloadMissingMods) {
             try {
-                for (rawName in missingModsToLoad) {
-                    val modName = rawName.folderNameToRepoName().lowercase()
-                    val repos = Github.tryGetGithubReposWithTopic(10, 1, modName)
-                        ?: throw UncivShowableException("Could not download mod list.")
-                    val repo = repos.items.firstOrNull { it.name.lowercase() == modName }
-                        ?: throw UncivShowableException("Could not find a mod named \"[$modName]\".")
-                    val modFolder = Github.downloadAndExtract(
-                        repo,
-                        Gdx.files.local("mods")
-                    )
-                        ?: throw Exception("Unexpected 404 error") // downloadAndExtract returns null for 404 errors and the like -> display something!
-                    Github.rewriteModOptions(repo, modFolder)
-                    val labelText = descriptionLabel.text // Surprise - a StringBuilder
-                    labelText.appendLine()
-                    labelText.append("[${repo.name}] Downloaded!".tr())
-                    launchOnGLThread { descriptionLabel.setText(labelText) }
-                }
-                launchOnGLThread {
-                    RulesetCache.loadRulesets()
-                    missingModsToLoad = emptyList()
-                    loadMissingModsButton.isVisible = false
-                    errorLabel.isVisible = false
-                    rightSideTable.pack()
-                    ToastPopup("Missing mods are downloaded successfully.", this@LoadGameScreen)
-                }
+                loadMissingMods(missingMods,
+                    onModDownloaded = {
+                        val labelText = descriptionLabel.text // A Gdx CharArray that has StringBuilder-like methods
+                        labelText.appendLine()
+                        labelText.append("[$it] Downloaded!".tr())
+                        launchOnGLThread { descriptionLabel.setText(labelText) }
+                    },
+                    onCompleted = {
+                        launchOnGLThread {
+                            RulesetCache.loadRulesets()
+                            errorLabel.isVisible = false
+                            rightSideTable.pack()
+                            ToastPopup("Missing mods are downloaded successfully.", this@LoadGameScreen)
+                        }
+                    }
+                )
             } catch (ex: Exception) {
-                handleLoadGameException(ex, "Could not load the missing mods!")
+                launchOnGLThread {
+                    handleLoadGameException(ex, "Could not load the missing mods!")
+                }
             } finally {
                 launchOnGLThread {
-                    loadMissingModsButton.isEnabled = true
                     descriptionLabel.setText("")
                 }
             }
         }
     }
-
 }
