@@ -35,7 +35,7 @@ object QuickSave {
     }
 
     fun load(screen: WorldScreen) {
-        UncivGame.Current.settings.autoPlay.stopAutoPlay()
+        screen.autoPlay.stopAutoPlay()
         val files = UncivGame.Current.files
         val toast = ToastPopup("Quickloading...", screen)
         Concurrency.run("QuickLoadGame") {
@@ -58,7 +58,6 @@ object QuickSave {
     }
 
     fun autoLoadGame(screen: MainMenuScreen) {
-        UncivGame.Current.settings.autoPlay.stopAutoPlay()
         val loadingPopup = LoadingPopup(screen)
         Concurrency.run("autoLoadGame") {
             // Load game from file to class on separate thread to avoid ANR...
@@ -90,7 +89,7 @@ object QuickSave {
 
             if (savedGame.gameParameters.isOnlineMultiplayer) {
                 try {
-                    screen.game.onlineMultiplayer.loadGame(savedGame)
+                    screen.game.onlineMultiplayer.downloadGame(savedGame)
                 } catch (_: OutOfMemoryError) {
                     outOfMemory()
                 } catch (notAPlayer: UncivShowableException) {
