@@ -25,7 +25,7 @@ class VictoryScreenCharts(
     private var selectedCiv = worldScreen.selectedCiv
     private val viewingCiv = worldScreen.viewingCiv
 
-    private val rankingTypeSelect = TranslatedSelectBox(RankingType.values().map { it.label }, rankingType.name, skin)
+    private val rankingTypeSelect = TranslatedSelectBox(RankingType.entries.map { it.label }, rankingType.name)
     private val civButtonsTable = Table()
     private val civButtonsScroll = AutoScrollPane(civButtonsTable)
     private val controlsColumn = Table()
@@ -35,11 +35,12 @@ class VictoryScreenCharts(
     }
 
     private var lineChart = LineChart(viewingCiv)
-    // if it is negative - no zoom, if positive - zoom at turn X
+
+    /** if it is negative - no zoom, if positive - zoom at turn X */
     private var zoomAtX : IntRange? = null
 
     init {
-        civButtonsScroll.setScrollingDisabled(true, false)
+        civButtonsScroll.setScrollingDisabled(x = true, y = false)
         civButtonsTable.defaults().space(20f).fillX()
         controlsColumn.defaults().space(20f).fillX()
         controlsColumn.add(rankingTypeSelect).right().row()
@@ -47,7 +48,7 @@ class VictoryScreenCharts(
         defaults().fill().pad(20f)
         add(controlsColumn)
         updateControls()
-        add(lineChart).growX().top().padLeft(0f)
+        add(lineChart).growX().expandY().top().padLeft(0f)
 
         val onChartClick = OnClickListener(function = { _ , x, _ ->
             zoomAtX = if (zoomAtX == null) lineChart.getTurnAt(x) else null
@@ -56,7 +57,7 @@ class VictoryScreenCharts(
         lineChart.addListener(onChartClick)
 
         rankingTypeSelect.onChange {
-            rankingType = RankingType.values()
+            rankingType = RankingType.entries
                 .firstOrNull { it.label == rankingTypeSelect.selected.value }
                 ?: RankingType.Score
             update()
@@ -73,8 +74,10 @@ class VictoryScreenCharts(
         val sortedCivs = gameInfo.civilizations.asSequence()
             .filter { it.isMajorCiv() }
             .map { VictoryScreen.CivWithStat(it, rankingType) }
-            .sortedBy { it.civ.civName }
-            .sortedByDescending { if(it.civ.isDefeated()) Int.MIN_VALUE else it.value }
+            .sortedWith(
+                compareByDescending<VictoryScreen.CivWithStat> { if (it.civ.isDefeated()) Int.MIN_VALUE else it.value }
+                .thenBy { it.civ.civName }
+            )
         for (civEntry in sortedCivs) {
             if (civEntry.civ != selectedCiv) civButtonsTable.add()
             else civButtonsTable.add(markerIcon).size(24f).right()
@@ -97,7 +100,7 @@ class VictoryScreenCharts(
     }
 
     private fun getLineChartData(rankingType: RankingType): List<DataPoint<Int>> {
-        return gameInfo.civilizations.asSequence()
+        val dataPoints = gameInfo.civilizations.asSequence()
             .filter { it.isMajorCiv() }
             .flatMap { civ ->
                 civ.statsHistory
@@ -109,7 +112,20 @@ class VictoryScreenCharts(
             .mapValues { group -> group.value.toMap() }
             .flatMap { turn ->
                 turn.value.map { (civ, value) -> DataPoint(turn.key, value, civ) }
-            }
+            }.toMutableList()
+
+        if (dataPoints.isEmpty()) // e.g. spectator on first turn, before any civ has had a chance to have a turn
+            return emptyList()
+
+        // Historical data does not include data for current turn for civs which haven't got their turn yet,
+        // so we append missing stat for current turn to the data for each such civ
+        val pointsByCiv = dataPoints.sortedBy { it.x }.groupBy { it.civ }
+        val actualTurn = dataPoints.maxOf { it.x }
+        for (civ in pointsByCiv.keys.filterNot { it.isDefeated() })
+            if (pointsByCiv[civ]!!.last().x != actualTurn)
+                dataPoints += DataPoint(actualTurn, civ.getStatForRanking(rankingType), civ)
+
+        return dataPoints
     }
 
     override fun activated(index: Int, caption: String, pager: TabbedPager) {
