@@ -6,7 +6,6 @@ import com.badlogic.gdx.math.Interpolation
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.actions.TemporalAction
-import com.badlogic.gdx.scenes.scene2d.ui.Container
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.Stack
 import com.badlogic.gdx.scenes.scene2d.ui.Table
@@ -19,10 +18,9 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.models.ruleset.MilestoneType
 import com.unciv.models.ruleset.Victory
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.widgets.TabbedPager
-import com.unciv.ui.components.extensions.isNarrowerThan4to3
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.widgets.TabbedPager
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.images.ImageWithCustomSize  // Kdoc, not used
 import com.unciv.ui.screens.basescreen.BaseScreen
@@ -74,7 +72,6 @@ class VictoryScreenIllustrations(
     }
 
     private val game = worldScreen.gameInfo
-    private val maxLabelWidth = parent.stage.run { width * (if (isNarrowerThan4to3()) 0.9f else 0.7f) }
     private val victories = game.getEnabledVictories().values
         .filter { it.hasIllustrations() }
         .sortedBy { it.name.tr(hideIcons = true) }
@@ -182,17 +179,20 @@ class VictoryScreenIllustrations(
 
             for (actor in actors) {
                 actor.color.a = 0f
-                // actor.width, actor.height are empirically equal to the image's pixel dimensions
-                // at this moment, before actor has a parent, but I don't trust that happenstance.
-                val pixelArea = Rectangle(((actor as Image).drawable as TextureRegionDrawable).region)
                 // Scale max image dimensions into holder space minus padding preserving aspect ratio
                 val imageArea = Rectangle(this.width - 30f, maxHeight - 30f)
-                if (pixelArea.width * imageArea.height > imageArea.width * pixelArea.height)
-                    imageArea.height = imageArea.width * pixelArea.height / pixelArea.width
-                else
-                    imageArea.width = imageArea.height * pixelArea.width / pixelArea.height
+                if (actor is ImageWithFixedPrefSize)  {
+                    val pixelArea = Rectangle((actor.drawable as TextureRegionDrawable).region)
+                    // Determine image aspect ratio, asuming square pixels:
+                    // (Image actor.width, actor.height are empirically equal to the image's pixel dimensions
+                    // at this moment, before actor has a parent, but I don't trust that happenstance.)
+                    if (pixelArea.width * imageArea.height > imageArea.width * pixelArea.height)
+                        imageArea.height = imageArea.width * pixelArea.height / pixelArea.width
+                    else
+                        imageArea.width = imageArea.height * pixelArea.width / pixelArea.height
+                    actor.setPrefSize(imageArea.width, imageArea.height)
+                }
                 actor.setSize(imageArea.width, imageArea.height)
-                if (actor is ImageWithFixedPrefSize) actor.setPrefSize(imageArea.width, imageArea.height)
                 add(actor)
             }
         }
@@ -228,14 +228,14 @@ class VictoryScreenIllustrations(
                     civ.victoryManager.currentsSpaceshipParts.sumValues()
                 }
                 MilestoneType.DestroyAllPlayers -> {
-                    total += if (selectedCiv.hideCivCount()) game.gameParameters.maxNumberOfPlayers
+                    total += if (selectedCiv.shouldHideCivCount()) game.gameParameters.maxNumberOfPlayers
                         else game.civilizations.count { it.isMajorCiv() }
                     game.civilizations.count {
                         it != civ && it.isMajorCiv() && civ.knows(it) && it.isDefeated()
                     }
                 }
                 MilestoneType.CaptureAllCapitals -> {
-                    total += if (selectedCiv.hideCivCount()) game.gameParameters.maxNumberOfPlayers
+                    total += if (selectedCiv.shouldHideCivCount()) game.gameParameters.maxNumberOfPlayers
                         else game.getCities().count { it.isOriginalCapital }
                     civ.cities.count { it.isOriginalCapital }
                 }
@@ -243,6 +243,11 @@ class VictoryScreenIllustrations(
                     total += milestone.params[0].toInt()
                     civ.policies.completedBranches.size
                 }
+                MilestoneType.MoreCountableThanEachPlayer -> {
+                    val relevantCivs = civ.gameInfo.civilizations.filter { milestone.getMoreCountableThanOtherCivRelevant(civ, it) }
+                    total += if (selectedCiv.shouldHideCivCount()) game.gameParameters.maxNumberOfPlayers - 1 else relevantCivs.size
+                    relevantCivs.count { milestone.getMoreCountableThanOtherCivPercent(civ, it) > 100f }
+                }
                 MilestoneType.WorldReligion -> {
                     total += game.civilizations.count { it.isMajorCiv() && it.isAlive() }
                     val religion = civ.religionManager.religion?.takeUnless { it.isPantheon() }
@@ -263,12 +268,13 @@ class VictoryScreenIllustrations(
             }
             points += milestonePoints
         }
+        if (total == 0) return 0  // no milestones, no points - e.g. game just started, there are no capitals
         return points * 100 / total
     }
 
     private fun getImages(victory: Victory): List<Actor> {
         game.victoryData?.run {
-            if (victory.name == victoryType && selectedCiv.civName == winningCiv) {
+            if (victory.name == victoryType && selectedCiv.civID == winningCiv) {
                 val image = getImageOrNull(victory, "Won")
                 return getWonOrLostStack(image, victory.victoryString, Color.GOLD)
             }
@@ -293,14 +299,8 @@ class VictoryScreenIllustrations(
     }
 
     private fun getWonOrLostStack(image: Image?, text: String?, color: Color): List<Actor> {
-        val container = if (text == null) null
-        else {
-            val label = text.toLabel(color, 50, Align.bottom)
-            label.wrap = true
-            label.width = maxLabelWidth
-            Container(label).apply { bottom() }
-        }
-        return listOfNotNull(image, container)
+        val label = text?.toLabel(color, 24, Align.bottom)?.apply { wrap = true }
+        return listOfNotNull(image, label)
     }
 
     /** Variant of [ImageWithCustomSize] that avoids certain problems.
