@@ -15,31 +15,27 @@ import com.unciv.logic.civilization.LocationAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.civilization.PopupAlert
-import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
-import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
-import com.unciv.logic.civilization.diplomacy.RelationshipLevel
-import com.unciv.models.ruleset.EventChoice
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.logic.civilization.diplomacy.*
+import com.unciv.logic.map.HexCoord
+import com.unciv.logic.map.mapunit.MapUnit
+import com.unciv.logic.map.toHexCoord
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.MusicMood
 import com.unciv.ui.audio.MusicTrackChooserFlags
-import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.pad
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
-import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.Popup
-import com.unciv.ui.screens.civilopediascreen.FormattedLine
-import com.unciv.ui.screens.civilopediascreen.MarkupRenderer
 import com.unciv.ui.screens.diplomacyscreen.LeaderIntroTable
 import com.unciv.ui.screens.victoryscreen.VictoryScreen
+import yairm210.purity.annotations.Readonly
 import java.util.EnumSet
 
 /**
@@ -74,8 +70,8 @@ class AlertPopup(
     private val viewingCiv get() = worldScreen.viewingCiv
     private val stageWidth get() = worldScreen.stage.width
     private val stageHeight get() = worldScreen.stage.height
-    private fun getCiv(civName: String) = gameInfo.getCivilization(civName)
-    private fun getCity(cityId: String) = gameInfo.getCities().first { it.id == cityId }
+    @Readonly private fun getCiv(civName: String) = gameInfo.getCivilization(civName)
+    @Readonly private fun getCity(cityId: String) = gameInfo.getCities().first { it.id == cityId }
     //endregion
 
     // This redirects all addCloseButton uses with only text and no action to accept the space key
@@ -83,7 +79,7 @@ class AlertPopup(
         addCloseButton(text, KeyboardBinding.NextTurnAlternate, null)
 
     init {
-        var skipThisAlert = false
+        var shouldOpen = true
 
         // This makes the buttons fill up available width. See comments in #9559.
         // To implement a middle ground, I would either simply replace growX() with minWidth(240f) or so,
@@ -92,66 +88,103 @@ class AlertPopup(
         bottomTable.defaults().growX()
 
         when (popupAlert.type) {
-            AlertType.WarDeclaration -> addWarDeclaration()
-            AlertType.Defeated -> addDefeated()
-            AlertType.FirstContact -> addFirstContact()
+            // Cities
             AlertType.CityConquered -> addCityConquered()
             AlertType.CityTraded -> addCityTraded()
-            AlertType.BorderConflict -> addBorderConflict()
-            AlertType.DemandToStopSettlingCitiesNear -> addDemandToStopSettlingCitiesNear()
-            AlertType.CitySettledNearOtherCivDespiteOurPromise -> addCitySettledNearOtherCivDespiteOurPromise()
+            AlertType.DiplomaticMarriage -> addDiplomaticMarriage()
+            // Demands and diplomacy
+            AlertType.FirstContact -> addFirstContact()
+            AlertType.WarDeclaration -> shouldOpen = addWarDeclaration()
+            AlertType.BorderConflict -> shouldOpen = addBorderConflict()
+            AlertType.TilesStolen -> shouldOpen = addTilesStolen()
+            
+            // demands
+            AlertType.DemandToStopSettlingCitiesNear -> shouldOpen = addDemand(Demand.DoNotSettleNearUs)
+            AlertType.CitySettledNearOtherCivDespiteOurPromise -> shouldOpen = addDemandViolationNoticed(Demand.DoNotSettleNearUs)
+            AlertType.DemandToStopSpreadingReligion -> shouldOpen = addDemand(Demand.DoNotSpreadReligion)
+            AlertType.ReligionSpreadDespiteOurPromise -> shouldOpen = addDemandViolationNoticed(Demand.DoNotSpreadReligion)
+            AlertType.DemandToStopSpyingOnUs -> shouldOpen = addDemand(Demand.DontSpyOnUs)
+            AlertType.SpyingOnUsDespiteOurPromise -> shouldOpen = addDemand(Demand.DontSpyOnUs)
+            
+            
+            AlertType.DeclarationOfFriendship -> shouldOpen = addDeclarationOfFriendship()
+            AlertType.BulliedProtectedMinor, AlertType.AttackedProtectedMinor, AlertType.AttackedAllyMinor -> 
+                shouldOpen = addBulliedOrAttackedProtectedOrAlliedMinor()
+            AlertType.Defeated -> addDefeated()
+            // We did stuff
             AlertType.WonderBuilt -> addWonderBuilt()
             AlertType.TechResearched -> addTechResearched()
             AlertType.GoldenAge -> addGoldenAge()
-            AlertType.DeclarationOfFriendship -> addDeclarationOfFriendship()
             AlertType.StartIntro -> addStartIntro()
-            AlertType.DiplomaticMarriage -> addDiplomaticMarriage()
-            AlertType.BulliedProtectedMinor, AlertType.AttackedProtectedMinor -> addBulliedOrAttackedProtectedMinor()
-            AlertType.RecapturedCivilian -> skipThisAlert = addRecapturedCivilian()
+            AlertType.RecapturedCivilian -> shouldOpen = addRecapturedCivilian()
             AlertType.GameHasBeenWon -> addGameHasBeenWon()
-            AlertType.Event -> skipThisAlert = !addEvent()
+            AlertType.Event -> shouldOpen = addEvent()
         }
-        if (!skipThisAlert) open()
+        if (shouldOpen) open()
+        else viewingCiv.popupAlerts.remove(popupAlert)
     }
 
     //region AlertType handlers
 
-    private fun addBorderConflict() {
+    private fun addBorderConflict(): Boolean {
         val civInfo = getCiv(popupAlert.value)
+        if (civInfo.isDefeated()) return false
         addLeaderName(civInfo)
         addGoodSizedLabel("Remove your troops in our border immediately!")
         addCloseButton("Sorry.", KeyboardBinding.Confirm)
         addCloseButton("Never!", KeyboardBinding.Cancel)
+        return true
+    }
+    
+    private fun addTilesStolen(): Boolean {
+        val civInfo = getCiv(popupAlert.value)
+        if (civInfo.isDefeated()) return false
+        addLeaderName(civInfo)
+        addGoodSizedLabel("Those lands were not yours to take. This has not gone unnoticed.")
+        addCloseButton()
+        return true
     }
 
-    private fun addBulliedOrAttackedProtectedMinor() {
+    private fun addBulliedOrAttackedProtectedOrAlliedMinor(): Boolean {
         val involvedCivs = popupAlert.value.split('@')
         val bullyOrAttacker = getCiv(involvedCivs[0])
+        if (bullyOrAttacker.isDefeated()) return false
         val cityState = getCiv(involvedCivs[1])
         val player = viewingCiv
         addLeaderName(bullyOrAttacker)
 
-        val isAtLeastNeutral = bullyOrAttacker.getDiplomacyManager(player).isRelationshipLevelGE(RelationshipLevel.Neutral)
+        val isAtLeastNeutral = bullyOrAttacker.getDiplomacyManager(player)!!.isRelationshipLevelGE(RelationshipLevel.Neutral)
         val text = when {
             popupAlert.type == AlertType.BulliedProtectedMinor && isAtLeastNeutral ->  // Nice message
                 "I've been informed that my armies have taken tribute from [${cityState.civName}], a city-state under your protection.\nI assure you, this was quite unintentional, and I hope that this does not serve to drive us apart."
             popupAlert.type == AlertType.BulliedProtectedMinor ->  // Nasty message
                 "We asked [${cityState.civName}] for a tribute recently and they gave in.\nYou promised to protect them from such things, but we both know you cannot back that up."
             isAtLeastNeutral ->  // Nice message
-                "It's come to my attention that I may have attacked [${cityState.civName}], a city-state under your protection.\nWhile it was not my goal to be at odds with your empire, this was deemed a necessary course of action."
+                "It's come to my attention that I may have attacked [${cityState.civName}].\nWhile it was not my goal to be at odds with your empire, this was deemed a necessary course of action."
             else ->  // Nasty message
                 "I thought you might like to know that I've launched an invasion of one of your little pet states.\nThe lands of [${cityState.civName}] will make a fine addition to my own."
         }
         addGoodSizedLabel(text).row()
+        
+        if (!player.isAtWarWith(bullyOrAttacker)) {
+            addCloseButton("THIS MEANS WAR!", KeyboardBinding.Confirm) {
+            player.getDiplomacyManager(bullyOrAttacker)!!.sideWithCityState()
+            val warReason = if (popupAlert.type == AlertType.AttackedAllyMinor) WarType.AlliedCityStateWar else WarType.ProtectedCityStateWar
+            player.getDiplomacyManager(bullyOrAttacker)!!.declareWar(DeclareWarReason(warReason, cityState))
+            cityState.getDiplomacyManager(player)!!.influence += 20f // You went to war for us!!
+        }.row()}
 
         addCloseButton("You'll pay for this!", KeyboardBinding.Confirm) {
-            player.getDiplomacyManager(bullyOrAttacker).sideWithCityState()
+            player.getDiplomacyManager(bullyOrAttacker)!!.sideWithCityState()
         }.row()
+
         addCloseButton("Very well.", KeyboardBinding.Cancel) {
             player.addNotification("You have broken your Pledge to Protect [${cityState.civName}]!",
                 cityState.cityStateFunctions.getNotificationActions(), NotificationCategory.Diplomacy, cityState.civName)
             cityState.cityStateFunctions.removeProtectorCiv(player, forced = true)
         }.row()
+        
+        return true
     }
 
     private fun addCityConquered() {
@@ -159,9 +192,9 @@ class AlertPopup(
         addQuestionAboutTheCity(city.name)
         val conqueringCiv = gameInfo.getCurrentPlayerCivilization()
 
-        if (city.foundingCiv != ""
-                && city.civ.civName != city.foundingCiv // can't liberate if the city actually belongs to those guys
-                && conqueringCiv.civName != city.foundingCiv) { // or belongs originally to us
+        if (city.foundingCivObject != null
+                && city.civ != city.foundingCivObject // can't liberate if the city actually belongs to those guys
+                && conqueringCiv != city.foundingCivObject) { // or belongs originally to us
             addLiberateOption(city, conqueringCiv)
             addSeparator()
         }
@@ -187,11 +220,13 @@ class AlertPopup(
         }
     }
 
-    private fun addCitySettledNearOtherCivDespiteOurPromise() {
+    private fun addDemandViolationNoticed(demand: Demand): Boolean {
         val otherciv = getCiv(popupAlert.value)
+        if (otherciv.isDefeated()) return false
         addLeaderName(otherciv)
-        addGoodSizedLabel("We noticed your new city near our borders, despite your promise. This will have....implications.").row()
+        addGoodSizedLabel(demand.violationNoticedText).row()
         addCloseButton("Very well.")
+        return true
     }
 
     private fun addCityTraded() {
@@ -199,24 +234,30 @@ class AlertPopup(
         addQuestionAboutTheCity(city.name)
         val conqueringCiv = gameInfo.getCurrentPlayerCivilization()
 
-        if (!conqueringCiv.isAtWarWith(getCiv(city.foundingCiv))) {
+        if (!conqueringCiv.isAtWarWith(city.foundingCivObject!!)) {
             addLiberateOption(city, conqueringCiv)
             addSeparator()
         }
         addCloseButton("Keep it").row()
     }
 
-    private fun addDeclarationOfFriendship() {
+    private fun addDeclarationOfFriendship(): Boolean {
         val otherciv = getCiv(popupAlert.value)
-        val playerDiploManager = viewingCiv.getDiplomacyManager(otherciv)
+        if (otherciv.isDefeated()) return false
+        val playerDiploManager = viewingCiv.getDiplomacyManager(otherciv)!!
         addLeaderName(otherciv)
-        addGoodSizedLabel("My friend, shall we declare our friendship to the world?").row()
+        addGoodSizedLabel(
+                if (otherciv.nation.declaringFriendship.isNotEmpty()) otherciv.nation.declaringFriendship else "My friend, shall we declare our friendship to the world?"
+        ).row()
+        addCloseButton("Declare Friendship ([30] turns)", KeyboardBinding.Confirm) {
+            playerDiploManager.signDeclarationOfFriendship()
+        }.row()
         addCloseButton("We are not interested.", KeyboardBinding.Cancel) {
             playerDiploManager.otherCivDiplomacy().setFlag(DiplomacyFlags.DeclinedDeclarationOfFriendship, 20)
         }.row()
-        addCloseButton("Declare Friendship ([30] turns)", KeyboardBinding.Confirm) {
-            playerDiploManager.signDeclarationOfFriendship()
-        }
+        val music = UncivGame.Current.musicController
+        music.playVoice("${otherciv.nation.name}.declaringFriendship")
+        return true
     }
 
     private fun addDefeated() {
@@ -228,19 +269,23 @@ class AlertPopup(
         music.playVoice("${civInfo.civName}.defeated")
     }
 
-    private fun addDemandToStopSettlingCitiesNear() {
+    private fun addDemand(demand: Demand): Boolean {
         val otherciv = getCiv(popupAlert.value)
-        val playerDiploManager = viewingCiv.getDiplomacyManager(otherciv)
+        if (otherciv.isDefeated()) return false
+        
+        val playerDiploManager = viewingCiv.getDiplomacyManager(otherciv)!!
         addLeaderName(otherciv)
-        addGoodSizedLabel("Please don't settle new cities near us.").row()
-        addCloseButton("Very well, we shall look for new lands to settle.", KeyboardBinding.Confirm) {
-            playerDiploManager.agreeNotToSettleNear()
+        addGoodSizedLabel(demand.demandText).row()
+        addCloseButton(demand.acceptDemandText, KeyboardBinding.Confirm) {
+            playerDiploManager.agreeToDemand(demand)
         }.row()
-        addCloseButton("We shall do as we please.", KeyboardBinding.Cancel) {
-            playerDiploManager.refuseDemandNotToSettleNear()
+        addCloseButton(demand.refuseDemandText, KeyboardBinding.Cancel) {
+            playerDiploManager.refuseDemand(demand)
         }
+        return true
     }
 
+
     private fun addDiplomaticMarriage() {
         val city = getCity(popupAlert.value)
         addGoodSizedLabel(city.name.tr() + ": " + "What would you like to do with the city?".tr(), Constants.headingFontSize) // Add name because there might be several cities
@@ -269,7 +314,7 @@ class AlertPopup(
         addLeaderName(civInfo)
         music.chooseTrack(civInfo.civName, MusicMood.themeOrPeace, MusicTrackChooserFlags.setSpecific)
         music.playVoice("${civInfo.civName}.introduction")
-        if (civInfo.isCityState()) {
+        if (civInfo.isCityState) {
             addGoodSizedLabel("We have encountered the City-State of [${nation.name}]!").row()
             addCloseButton("Excellent!")
         } else {
@@ -280,7 +325,7 @@ class AlertPopup(
 
     private fun addGameHasBeenWon() {
         val victoryData = gameInfo.victoryData!!
-        addGoodSizedLabel("[${victoryData.winningCiv}] has won a [${victoryData.victoryType}] Victory!").row()
+        addGoodSizedLabel("[${victoryData.winningCivObject.civName}] has won a [${victoryData.victoryType}] Victory!").row()
         addButton("Victory status") { close(); worldScreen.game.pushScreen(VictoryScreen(worldScreen)) }.row()
         addCloseButton()
     }
@@ -293,13 +338,14 @@ class AlertPopup(
         music.chooseTrack(viewingCiv.civName, MusicMood.Golden, MusicTrackChooserFlags.setSpecific)
     }
 
-    /** @return true to skip opening this Popup, as we're running in the initialization phase before the Popup is open */
+    /** @return false to skip opening this Popup, as we're running in the initialization phase before the Popup is open */
     private fun addRecapturedCivilian(): Boolean {
-        val position = Vector2().fromString(popupAlert.value)
+        val position = HexCoord.fromString(popupAlert.value)
         val tile = gameInfo.tileMap[position]
         val capturedUnit = tile.civilianUnit  // This has got to be it
-            ?: return true // the unit disappeared somehow? maybe a modded action?
-        val originalOwner = getCiv(capturedUnit.originalOwner!!)
+            ?: return false // the unit disappeared somehow? maybe a modded action?
+        val originalOwner = capturedUnit.originalOwningCiv!!
+        if (originalOwner.isDefeated()) return false
         val captor = viewingCiv
 
         addGoodSizedLabel("Return [${capturedUnit.name}] to [${originalOwner.civName}]?")
@@ -312,25 +358,25 @@ class AlertPopup(
             // Return it to original owner
             val unitName = capturedUnit.baseUnit.name
             capturedUnit.destroy()
-            val closestCity =
-                    originalOwner.cities.minByOrNull { it.getCenterTile().aerialDistanceTo(tile) }
+            val closestCity = originalOwner.cities.minByOrNull { it.getCenterTile().aerialDistanceTo(tile) }
+
             if (closestCity != null) {
                 // Attempt to place the unit near their nearest city
-                originalOwner.units.placeUnitNearTile(closestCity.location, unitName)
+                originalOwner.units.placeUnitNearTile(closestCity.location.toHexCoord(), unitName)
             }
 
-            if (originalOwner.isCityState()) {
-                originalOwner.getDiplomacyManager(captor).addInfluence(45f)
+            if (originalOwner.isCityState) {
+                originalOwner.getDiplomacyManagerOrMeet(captor).addInfluence(45f)
             } else if (originalOwner.isMajorCiv()) {
                 // No extra bonus from doing it several times
-                originalOwner.getDiplomacyManager(captor)
+                originalOwner.getDiplomacyManagerOrMeet(captor)
                     .setModifier(DiplomaticModifiers.ReturnedCapturedUnits, 20f)
             }
             val notificationSequence = sequence {
                 yield(LocationAction(tile.position))
                 if (closestCity != null)
                     yield(LocationAction(closestCity.location))
-                yield(DiplomacyAction(captor.civName))
+                yield(DiplomacyAction(captor))
                 yield(CivilopediaAction("Tutorial/Barbarians"))
             }
             originalOwner.addNotification("Your captured [${unitName}] has been returned by [${captor.civName}]", notificationSequence, NotificationCategory.Diplomacy, NotificationIcon.Trade, unitName, captor.civName)
@@ -339,7 +385,7 @@ class AlertPopup(
             // Take it for ourselves
             BattleUnitCapture.captureOrConvertToWorker(capturedUnit, captor)
         }
-        return false
+        return true
     }
 
     private fun addStartIntro() {
@@ -348,6 +394,11 @@ class AlertPopup(
         addGoodSizedLabel(civInfo.nation.startIntroPart1).row()
         addGoodSizedLabel(civInfo.nation.startIntroPart2).row()
         addCloseButton("Let's begin!")
+
+        // Since there's introduction text, play the startIntroPart1 voice hook with the nation's theme.
+        val music = UncivGame.Current.musicController
+        music.chooseTrack(civInfo.nation.name, MusicMood.themeOrPeace, MusicTrackChooserFlags.setSpecific)
+        music.playVoice("${civInfo.nation.name}.startIntroPart1")
     }
 
     private fun addTechResearched() {
@@ -364,8 +415,10 @@ class AlertPopup(
         music.chooseTrack(tech.name, MusicMood.Researched, MusicTrackChooserFlags.setSpecific)
     }
 
-    private fun addWarDeclaration() {
+    private fun addWarDeclaration(): Boolean {
         val civInfo = getCiv(popupAlert.value)
+        // technically they already declared war, but if they're dead it'll be strange that they talk to us
+        if (civInfo.isDefeated()) return false
         addLeaderName(civInfo)
         addGoodSizedLabel(civInfo.nation.declaringWar).row()
         bottomTable.defaults().pad(0f, 5f)
@@ -373,6 +426,7 @@ class AlertPopup(
         addCloseButton("Very well.")
         music.chooseTrack(civInfo.civName, MusicMood.War, MusicTrackChooserFlags.setSpecific)
         music.playVoice("${civInfo.civName}.declaringWar")
+        return true
     }
 
     private fun addWonderBuilt() {
@@ -397,9 +451,15 @@ class AlertPopup(
         }
 
         val centerTable = Table()
-        centerTable.add(wonder.quote.toLabel().apply { wrap = true }).width(stageWidth / 3).pad(10f)
-        centerTable.add(wonder.getShortDescription()
-            .toLabel().apply { wrap = true }).width(stageWidth / 3).pad(10f)
+        val centerTableColumnWidth = stageWidth / if (wonder.quote.isEmpty()) 2 else 3
+        if (wonder.quote.isNotEmpty()) {
+            centerTable.add(wonder.quote.toLabel().apply { wrap = true })
+                .width(centerTableColumnWidth)
+                .pad(10f)
+        }
+        centerTable.add(wonder.getShortDescription().toLabel().apply { wrap = true })
+            .width(centerTableColumnWidth)
+            .pad(10f)
         add(centerTable).row()
         addCloseButton()
         music.chooseTrack(wonder.name, MusicMood.Wonder, MusicTrackChooserFlags.setSpecific)
@@ -461,12 +521,12 @@ class AlertPopup(
         add(button).row()
         addGoodSizedLabel("Puppeted cities do not increase your tech or policy cost.").row()
         addGoodSizedLabel("You have no control over the the production of puppeted cities.").row()
-        addGoodSizedLabel("Puppeted cities also generate 25% less Gold and Science.").row()
+        addGoodSizedLabel("Puppeted cities also generate 25% less Science and Culture.").row()
         if (mayAnnex) addGoodSizedLabel("A puppeted city can be annexed at any time.").row()
     }
 
     private fun addLiberateOption(city: City, conqueringCiv: Civilization) {
-        val button = "Liberate (city returns to [originalOwner])".fillPlaceholders(city.foundingCiv).toTextButton()
+        val button = "Liberate (city returns to [originalOwner])".fillPlaceholders(city.foundingCivObject!!.civName).toTextButton()
         button.onActivation {
             city.liberateCity(conqueringCiv)
             close()
@@ -506,19 +566,23 @@ class AlertPopup(
 
     /** Returns if event was triggered correctly */
     private fun addEvent(): Boolean {
-        val event = gameInfo.ruleset.events[popupAlert.value] ?: return false
-
-        val stateForConditionals = StateForConditionals(gameInfo.currentPlayerCiv)
-        val choices = event.getMatchingChoices(stateForConditionals)
-            ?: return false
-
-        if (event.text.isNotEmpty())
-            addGoodSizedLabel(event.text)
-        if (event.civilopediaText.isNotEmpty()) {
-            add(event.renderCivilopediaText(stageWidth * 0.5f, ::openCivilopedia)).row()
+        // The event string is in the format "eventName" + (Constants.stringSplitCharacter + "unitId=1234")?
+        // We explicitly specify that this is a unitId, to enable us to add other context info in the future - for example city id
+        val splitString = popupAlert.value.split(Constants.stringSplitCharacter)
+        val eventName = splitString[0]
+        var unit: MapUnit? = null
+        for (i in 1 until splitString.size) {
+            if (splitString[i].startsWith("unitId=")){
+                val unitId = splitString[i].substringAfter("unitId=").toInt()
+                unit = viewingCiv.units.getUnitById(unitId)
+            }
         }
-
-        for (choice in choices) addChoice(choice)
+        
+        
+        val event = gameInfo.ruleset.events[eventName] ?: return false
+        val render = RenderEvent(event, worldScreen, unit) { close() }
+        if (!render.isValid) return false
+        add(render).pad(0f).row()
         return true
     }
 
@@ -529,30 +593,4 @@ class AlertPopup(
         worldScreen.shouldUpdate = true
         super.close()
     }
-
-    private fun addChoice(choice: EventChoice) {
-        addSeparator()
-
-        val button = choice.text.toTextButton()
-        button.onActivation {
-            close()
-            choice.triggerChoice(gameInfo.currentPlayerCiv)
-        }
-        val key = KeyCharAndCode.parse(choice.keyShortcut)
-        if (key != KeyCharAndCode.UNKNOWN) {
-            button.keyShortcuts.add(key)
-            button.addTooltip(key)
-        }
-        add(button).row()
-
-        val lines = (
-                choice.civilopediaText.asSequence()
-                + choice.triggeredUniqueObjects.asSequence()
-                    .filterNot { it.isHiddenToUsers() }
-                    .map { FormattedLine(it) }
-            ).asIterable()
-        add(MarkupRenderer.render(lines, stageWidth * 0.5f, linkAction = ::openCivilopedia)).row()
-    }
-
-    private fun openCivilopedia(link: String) = worldScreen.openCivilopedia(link)
 }
