@@ -1,40 +1,50 @@
 package com.unciv.ui.screens.worldscreen
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.math.MathUtils
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.actions.Actions
+import com.badlogic.gdx.scenes.scene2d.actions.TemporalAction
 import com.badlogic.gdx.scenes.scene2d.ui.Cell
 import com.badlogic.gdx.scenes.scene2d.ui.Container
 import com.badlogic.gdx.scenes.scene2d.ui.Image
+import com.badlogic.gdx.scenes.scene2d.ui.Label
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable
 import com.badlogic.gdx.utils.Align
 import com.unciv.GUI
 import com.unciv.logic.civilization.Notification
 import com.unciv.logic.civilization.NotificationCategory
-import com.unciv.ui.components.widgets.AutoScrollPane as ScrollPane
-import com.unciv.ui.components.widgets.ColorMarkupLabel
-import com.unciv.ui.components.widgets.WrappableLabel
-import com.unciv.ui.components.input.onClick
+import com.unciv.models.translations.tr
+import com.unciv.ui.components.MayaCalendar.toMayaNumerals
 import com.unciv.ui.components.extensions.packIfNeeded
 import com.unciv.ui.components.extensions.surroundWithCircle
+import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.widgets.ColorMarkupLabel
+import com.unciv.ui.components.widgets.WrappableLabel
 import com.unciv.ui.images.IconCircleGroup
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.screens.basescreen.BaseScreen.Companion.skinStrings
+import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
+import yairm210.purity.annotations.Pure
+import com.unciv.ui.components.widgets.AutoScrollPane as ScrollPane
 
 /*TODO
  *  Un-hiding the notifications when new ones arrive is a little pointless due to Categories:
  *      * try to scroll new into view? Very complicated as the "old" state is only "available" in the Widgets
  *      * Don't unless a new user option disables categories, then scroll to top?
- *  Idea: Blink or tint the button when new notifications while Hidden
- *  Idea: The little "1" on the bell - remove and draw actual count
  */
 
 class NotificationsScroll(
     private val worldScreen: WorldScreen
 ) : ScrollPane(null) {
-    enum class UserSetting(val static: Boolean = false) { Disabled(true), Hidden, Visible, Permanent(true) }
+    enum class UserSetting(val static: Boolean = false) {
+        Disabled(true), Hidden, Visible, Permanent(true);
+        companion object { @Pure fun default() = Visible }
+    }
 
     private companion object {
         /** Scale the entire ScrollPane by this factor */
@@ -65,8 +75,15 @@ class NotificationsScroll(
         const val restoreButtonSize = 42f
         /** Distance of restore button to TileInfoTable and screen edge */
         const val restoreButtonPad = 12f
+        /** The outer size of the number+circle within the restore button */
+        const val restoreButtonNumbersSize = 0.5f * restoreButtonSize
+        /** Font size for the "count" number label overlaid on the restore button */
+        const val restoreButtonNumberFontSize = 13
+        /** The x/y coords of the center of the number+circle within the restore button */
+        const val restoreButtonNumbersCenter = restoreButtonSize - restoreButtonNumbersSize / 2
         /** Background tint for [oneTimeNotification] */
-        private val oneTimeNotificationColor = Color.valueOf("fceea8")
+        private val oneTimeNotificationColor get() =
+            skinStrings.getUIColor("NotificationScroll/OneTimeNotificationColor", Color.valueOf("#fceea8"))
     }
 
     //region private fields
@@ -81,11 +98,11 @@ class NotificationsScroll(
     /** For category header decoration lines */
     private val minCategoryLineWidth = worldScreen.stage.width * 0.075f
     /** Show restoreButton when less than this much of the pane is left */
-    private val scrolledAwayEpsilon = minCategoryLineWidth
+    private val scrolledAwayEpsilon = maxEntryWidth * 0.3f // yes should include inverseScaleFactor
 
     private val restoreButton = RestoreButton()
 
-    private var userSetting = UserSetting.Visible
+    private lateinit var userSetting: UserSetting
     private var userSettingChanged = false
     private var enlargeHighlight = false
 
@@ -144,7 +161,14 @@ class NotificationsScroll(
         coveredNotificationsTop: Float,
         coveredNotificationsBottom: Float
     ) {
-        if (getUserSettingCheckDisabled()) return
+        getUserSetting()
+        if (userSetting == UserSetting.Disabled) {
+            restoreButton.setPosition(coveredNotificationsBottom)
+            applyUserSettingChange()
+            restoreButton.updateCount(notifications.size)
+            return
+        }
+
         enlargeHighlight = GUI.getSettings().enlargeSelectedNotification
 
         // Remember scroll position _relative to topRight_
@@ -168,16 +192,21 @@ class NotificationsScroll(
         scrollY = if (selectedCell == null) {
             previousScrollY
         } else selectedCell!!.let {
-            val actualBottom = (it.actorY + notificationsTable.y) * scaleFactor
-            val actualTop = (it.actorY + it.actorHeight + notificationsTable.y) * scaleFactor
+            /** Note: This now works for [ListItem]'s wrapped 0 or 1 times. If the hierarchy of the
+                selection *relative to* [notificationsTable] changes again, this will need patching.
+            */
+            val trueActorY = it.actorY + (if (it.table == notificationsTable) 0f else it.table.y)
+            val actualBottom = (trueActorY + notificationsTable.y) * scaleFactor
+            val actualTop = (trueActorY + it.actorHeight + notificationsTable.y) * scaleFactor
             val fullyVisible = actualBottom >= coveredNotificationsBottom && actualTop <= stage.height - coveredNotificationsTop
             val centeredBottom = (stage.height - coveredNotificationsTop + coveredNotificationsBottom - it.actorHeight * scaleFactor) / 2
-            val centeredScrollY = centeredBottom * inverseScaleFactor - it.actorY + maxY
+            val centeredScrollY = centeredBottom * inverseScaleFactor - trueActorY + maxY
             if (fullyVisible) previousScrollY else centeredScrollY
         }
         updateVisualScroll()
 
         applyUserSettingChange()
+
         if (!userSetting.static) {
             restoreButton.unblock()
             if (contentChanged && !userSettingChanged && isHidden)
@@ -186,10 +215,7 @@ class NotificationsScroll(
 
         // Do the positioning here since WorldScreen may also call update when just its geometry changed
         setPosition(stage.width - width * scaleFactor, 0f)
-        restoreButton.setPosition(
-            stage.width - restoreButtonPad,
-            coveredNotificationsBottom + restoreButtonPad,
-            Align.bottomRight)
+        restoreButton.setPosition(coveredNotificationsBottom)
     }
 
     private fun updateContent(
@@ -223,6 +249,9 @@ class NotificationsScroll(
         if (notificationsHash == newHash && highlightNotification == null) return false
         notificationsHash = newHash
 
+        // Inform the Bell Button about the count - ignoring oneTimeNotification
+        restoreButton.updateCount(notifications.size)
+
         // Rebuild the notifications list
         notificationsTable.clear()
         notificationsTable.pack()  // forget last width!
@@ -241,19 +270,41 @@ class NotificationsScroll(
             .groupBy { it.category }
             .toSortedMap()  // This sorts by Category ordinal, so far intentional - the order of the grouped lists are unaffected
         for ((category, categoryNotifications) in orderedNotifications) {
-            if (category == NotificationCategory.General)
-                notificationsTable.add().padTop(categoryTopPad).row()  // Make sure category w/o header gets same spacing
+            val header: CategoryHeader?
+            if (category == NotificationCategory.General) {
+                notificationsTable.add().padTop(categoryTopPad)
+                    .row()  // Make sure category w/o header gets same spacing
+                header = null
+            }
             else {
-                val header = CategoryHeader(category, backgroundDrawable)
+                header = CategoryHeader(category, backgroundDrawable)
                 categoryHeaders.add(header)
                 notificationsTable.add(header).right().row()
             }
-            for (notification in categoryNotifications) {
-                val item = ListItem(notification, backgroundDrawable)
-                itemWidths.add(item.itemWidth)
-                val itemCell = notificationsTable.add(item)
-                if (notification == highlightNotification) selectedCell = itemCell
-                itemCell.right().row()
+
+            val notificationCategoryTable = Table()
+
+            fun fillNotificationCategoryTable() {
+                for (notification in categoryNotifications) {
+                    val item = ListItem(notification, backgroundDrawable)
+                    itemWidths.add(item.itemWidth)
+                    val itemCell = notificationCategoryTable.add(item)
+                    if (notification == highlightNotification) selectedCell = itemCell
+                    itemCell.right().row()
+                }
+            }
+
+            fillNotificationCategoryTable()
+            notificationsTable.add(notificationCategoryTable).right().row()
+
+            header?.onClick {
+                if (notificationCategoryTable.hasChildren()) {
+                    notificationCategoryTable.clear()
+                    notificationCategoryTable.pack()
+                } else {
+                    fillNotificationCategoryTable()
+                    notificationCategoryTable.pack()
+                }
             }
         }
 
@@ -286,7 +337,7 @@ class NotificationsScroll(
                 .minHeight(2f).width(minCategoryLineWidth)
             add(Table().apply {
                 background = backgroundDrawable
-                val label = ColorMarkupLabel(category.name, Color.BLACK, fontSize = fontSize)
+                val label = ColorMarkupLabel(category.name, ImageGetter.CHARCOAL, fontSize = fontSize)
                 add(label)
                 captionWidth = prefWidth  // of this wrapper including background rims
                 captionWidth
@@ -333,7 +384,7 @@ class NotificationsScroll(
             }
 
             val maxLabelWidth = maxEntryWidth - (itemIconSize + 5f) * notification.icons.size - 10f
-            val label = WrappableLabel(notification.text, maxLabelWidth, Color.BLACK, labelFontSize, hideIcons = true)
+            val label = WrappableLabel(notification.text, maxLabelWidth, ImageGetter.CHARCOAL, labelFontSize, hideIcons = true)
             label.setAlignment(Align.center)
             if (label.prefWidth > maxLabelWidth * scaleFactor) {  // can't explain why the comparison needs scaleFactor
                 label.wrap = true
@@ -387,21 +438,58 @@ class NotificationsScroll(
         private var blockCheck = true
         private var blockAct = true
         private var active = false
+        private var shownCount = -1
+        private val countLabel: Label
+        private val bellIcon = ImageGetter.getImage("OtherIcons/Notifications")
+        private val labelInnerCircle: Image
+        private val labelOuterCircle: Image
 
         init {
-            actor = ImageGetter.getImage("OtherIcons/Notifications")
+            actor = bellIcon
                 .surroundWithCircle(restoreButtonSize * 0.9f, color = BaseScreen.skinStrings.skinConfig.baseColor)
                 .surroundWithCircle(restoreButtonSize, resizeActor = false)
             size(restoreButtonSize)
+
+            countLabel = "".toLabel(ImageGetter.CHARCOAL, restoreButtonNumberFontSize, Align.center)
+            // not using surroundWithCircle for the count, as the centering will break if positioned within another IconCircleGroup (why?)
+            labelInnerCircle = ImageGetter.getCircle(Color.WHITE, restoreButtonNumbersSize * 0.9f)
+            labelInnerCircle.centerAtNumberPosition()
+            labelOuterCircle = ImageGetter.getCircle(ImageGetter.CHARCOAL, restoreButtonNumbersSize)
+            labelOuterCircle.centerAtNumberPosition()
+            actor.addActor(labelOuterCircle)
+            actor.addActor(labelInnerCircle)
+            actor.addActor(countLabel)
+
             color = color.cpy()  // So we don't mutate a skin element while fading
             color.a = 0f  // for first fade-in
-            onClick {
-                scrollX = maxX
-                hide()
-            }
+            onClick(::clicked)
             pack()  // `this` needs to adopt the size of `actor`, won't happen automatically (surprisingly)
         }
 
+        /** Set RestoreButton position - relative to the screen as parent is stage.root */
+        fun setPosition(coveredNotificationsBottom: Float) =
+            setPosition(
+                this@NotificationsScroll.stage.width - restoreButtonPad,
+                coveredNotificationsBottom + restoreButtonPad,
+                Align.bottomRight
+            )
+
+        private fun Actor.centerAtNumberPosition() = setPosition(restoreButtonNumbersCenter, restoreButtonNumbersCenter, Align.center)
+
+        fun updateCount(count: Int) {
+            if (count == shownCount) return
+            shownCount = count
+            val countText = when {
+                count >= 100 -> "+"
+                worldScreen.selectedCiv.isLongCountDisplay() -> count.toMayaNumerals()
+                else -> count.tr()
+            }
+            countLabel.setText(countText) // should we use Maya numerals for the Maya?
+            countLabel.pack()
+            countLabel.centerAtNumberPosition()
+            if (active) flash()
+        }
+
         fun block() {
             blockCheck = true
             blockAct = true
@@ -420,6 +508,7 @@ class NotificationsScroll(
                 worldScreen.stage.root.addActorAfter(this@NotificationsScroll, this)
             blockAct = false
             addAction(Actions.fadeIn(1f))
+            if (userSetting == UserSetting.Disabled) flash()
         }
 
         fun hide() {
@@ -440,38 +529,58 @@ class NotificationsScroll(
 
         fun checkScrollX(scrollX: Float) {
             if (blockCheck) return
-            if (active && scrollX >= scrolledAwayEpsilon * 2)
+            if (active && scrollX >= scrolledAwayEpsilon * 1.2f)
                 hide()
             if (!active && scrollX <= scrolledAwayEpsilon)
                 show()
         }
 
+        fun clicked() {
+            if (userSetting == UserSetting.Disabled) {
+                worldScreen.openEmpireOverview(EmpireOverviewCategories.Notifications)
+            } else {
+                scrollX = maxX
+                hide()
+            }
+        }
+
+        fun flash() {
+            addAction(ButtonFlashAction(bellIcon))
+        }
+
         override fun act(delta: Float) {
             // Actions are blocked while update() is rebuilding the UI elements - to be safe from unexpected state changes
             if (!blockAct) super.act(delta)
         }
     }
 
-    private fun getUserSettingCheckDisabled(): Boolean {
+    private class ButtonFlashAction(private val icon: Actor) : TemporalAction(4f) {
+        val tempColor = Color()
+        override fun update(percent: Float) {
+            val shift = (1f - MathUtils.cos(12f * percent * MathUtils.PI)) / 2f
+            tempColor.set(Color.WHITE).lerp(Color.RED, shift)
+            icon.color.set(tempColor)
+        }
+    }
+
+    private fun getUserSetting() {
         val settingString = GUI.getSettings().notificationScroll
-        val setting = UserSetting.values().firstOrNull { it.name == settingString }
-            ?: UserSetting.Visible
-        userSettingChanged = false
-        if (setting == userSetting)
-            return setting == UserSetting.Disabled
+        val setting = UserSetting.entries.firstOrNull { it.name == settingString }
+            ?: UserSetting.default()
+        if (::userSetting.isInitialized && setting == userSetting) {
+            userSettingChanged = false
+            return
+        }
 
         userSetting = setting
         userSettingChanged = true
-        if (setting != UserSetting.Disabled) return false
 
         notificationsTable.clear()
         notificationsHash = 0
         scrollX = 0f
         updateVisualScroll()
-        setScrollingDisabled(false, false)
+        setScrollingDisabled(x = false, y = false)
         isVisible = false
-        restoreButton.hide()
-        return true
     }
 
     private fun applyUserSettingChange() {
@@ -497,7 +606,10 @@ class NotificationsScroll(
             UserSetting.Permanent -> {
                 restoreButton.hide()
             }
-            else -> return
+            UserSetting.Disabled -> {
+                restoreButton.show()
+                return // Leave the NotificationsScroll invisible
+            }
         }
         isVisible = true
     }
