@@ -3,7 +3,6 @@ package com.unciv.ui.screens.worldscreen
 import com.badlogic.gdx.Application
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.Input
-import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
@@ -14,6 +13,7 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.PlayerType
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.event.EventBus
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.MapVisualization
 import com.unciv.logic.multiplayer.MultiplayerGameUpdated
 import com.unciv.logic.multiplayer.storage.FileStorageRateLimitReached
@@ -21,11 +21,11 @@ import com.unciv.logic.multiplayer.storage.MultiplayerAuthException
 import com.unciv.logic.trade.TradeEvaluation
 import com.unciv.models.TutorialTrigger
 import com.unciv.models.metadata.GameSetupInfo
+import com.unciv.models.ruleset.Event
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.centerX
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
 import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.input.KeyboardPanningListener
@@ -50,24 +50,30 @@ import com.unciv.ui.screens.savescreens.SaveGameScreen
 import com.unciv.ui.screens.victoryscreen.VictoryScreen
 import com.unciv.ui.screens.worldscreen.bottombar.BattleTable
 import com.unciv.ui.screens.worldscreen.bottombar.TileInfoTable
+import com.unciv.ui.screens.worldscreen.chat.ChatButton
 import com.unciv.ui.screens.worldscreen.mainmenu.WorldScreenMusicPopup
 import com.unciv.ui.screens.worldscreen.minimap.MinimapHolder
 import com.unciv.ui.screens.worldscreen.status.AutoPlayStatusButton
 import com.unciv.ui.screens.worldscreen.status.MultiplayerStatusButton
 import com.unciv.ui.screens.worldscreen.status.NextTurnButton
 import com.unciv.ui.screens.worldscreen.status.NextTurnProgress
+import com.unciv.ui.screens.worldscreen.status.SmallUnitButton
 import com.unciv.ui.screens.worldscreen.status.StatusButtons
 import com.unciv.ui.screens.worldscreen.topbar.WorldScreenTopBar
 import com.unciv.ui.screens.worldscreen.unit.AutoPlay
 import com.unciv.ui.screens.worldscreen.unit.UnitTable
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsTable
+import com.unciv.ui.screens.worldscreen.worldmap.WorldMapHolder
+import com.unciv.ui.screens.worldscreen.worldmap.WorldMapTileUpdater.updateTiles
 import com.unciv.utils.Concurrency
 import com.unciv.utils.debug
 import com.unciv.utils.launchOnGLThread
 import com.unciv.utils.launchOnThreadPool
 import com.unciv.utils.withGLContext
+import kotlinx.coroutines.CompletableDeferred
 import kotlinx.coroutines.Job
 import kotlinx.coroutines.coroutineScope
+import yairm210.purity.annotations.Readonly
 import java.util.Timer
 import kotlin.concurrent.timer
 
@@ -85,7 +91,7 @@ class WorldScreen(
     val viewingCiv: Civilization,
     restoreState: RestoreState? = null
 ) : BaseScreen() {
-    /** When set, causes the screen to update in the next [render][BaseScreen.render] event */
+    /** When set, causes the screen to update in the next [render][render] event */
     var shouldUpdate = false
 
     /** Indicates it's the player's ([viewingCiv]) turn */
@@ -109,6 +115,7 @@ class WorldScreen(
     // Floating Widgets going counter-clockwise
     internal val topBar = WorldScreenTopBar(this)
     internal val techPolicyAndDiplomacy = TechPolicyDiplomacyButtons(this)
+    internal val chatButton = ChatButton(this)
     private val unitActionsTable = UnitActionsTable(this)
     /** Bottom left widget holding information about a selected unit or city */
     internal val bottomUnitTable = UnitTable(this)
@@ -119,9 +126,11 @@ class WorldScreen(
     internal val notificationsScroll = NotificationsScroll(this)
     internal val nextTurnButton = NextTurnButton(this)
     private val statusButtons = StatusButtons(nextTurnButton)
+    internal val smallUnitButton = SmallUnitButton(this, statusButtons)
     private val tutorialTaskTable = Table().apply {
         background = skinStrings.getUiBackground("WorldScreen/TutorialTaskTable", tintColor = skinStrings.skinConfig.baseColor.darken(0.5f))
     }
+    private var tutorialTaskTableHash = 0
 
     private var nextTurnUpdateJob: Job? = null
 
@@ -148,38 +157,35 @@ class WorldScreen(
         stage.addActor(mapHolder)
         stage.scrollFocus = mapHolder
         stage.addActor(notificationsScroll)  // very low in z-order, so we're free to let it extend _below_ tile info and minimap if we want
-        stage.addActor(minimapWrapper)
+        stage.addActor(tutorialTaskTable)    // behind topBar!
         stage.addActor(topBar)
         stage.addActor(statusButtons)
         stage.addActor(techPolicyAndDiplomacy)
-        stage.addActor(tutorialTaskTable)
+        stage.addActor(chatButton)
 
         stage.addActor(zoomController)
         zoomController.isVisible = UncivGame.Current.settings.showZoomButtons
 
         stage.addActor(bottomUnitTable)
+        stage.addActor(unitActionsTable)
         stage.addActor(bottomTileInfoTable)
+        stage.addActor(minimapWrapper)
         battleTable.width = stage.width / 3
         battleTable.x = stage.width / 3
         stage.addActor(battleTable)
 
-        stage.addActor(unitActionsTable)
-
-        val tileToCenterOn: Vector2 =
+        val tileToCenterOn: HexCoord =
                 when {
-                    viewingCiv.getCapital() != null -> viewingCiv.getCapital()!!.location
+                    viewingCiv.getCapital() != null -> viewingCiv.getCapital()!!.location.toHexCoord()
                     viewingCiv.units.getCivUnits().any() -> viewingCiv.units.getCivUnits().first().getTile().position
-                    else -> Vector2.Zero
+                    else -> HexCoord.Zero
                 }
 
         mapHolder.isAutoScrollEnabled = Gdx.app.type == Application.ApplicationType.Desktop && game.settings.mapAutoScroll
         mapHolder.mapPanningSpeed = game.settings.mapPanningSpeed
 
         // Don't select unit and change selectedCiv when centering as spectator
-        if (viewingCiv.isSpectator())
-            mapHolder.setCenterPosition(tileToCenterOn, immediately = true, selectUnit = false)
-        else
-            mapHolder.setCenterPosition(tileToCenterOn, immediately = true, selectUnit = true)
+        mapHolder.setCenterPosition(tileToCenterOn, immediately = true, selectUnit = !viewingCiv.isSpectator())
 
         tutorialController.allTutorialsShowedCallback = { shouldUpdate = true }
 
@@ -221,15 +227,18 @@ class WorldScreen(
     // Handle disabling and re-enabling WASD listener while Options are open
     override fun openOptionsPopup(startingPage: Int, withDebug: Boolean, onClose: () -> Unit) {
         val oldListener = stage.root.listeners.filterIsInstance<KeyboardPanningListener>().firstOrNull()
-        if (oldListener != null) stage.removeListener(oldListener)
+        if (oldListener != null) {
+            stage.removeListener(oldListener)
+            oldListener.dispose()
+        }
         super.openOptionsPopup(startingPage, withDebug) {
             addKeyboardListener()
             onClose()
         }
     }
 
-    fun openEmpireOverview(category: EmpireOverviewCategories? = null) {
-        game.pushScreen(EmpireOverviewScreen(selectedCiv, category))
+    fun openEmpireOverview(category: EmpireOverviewCategories? = null, selection: String = "") {
+        game.pushScreen(EmpireOverviewScreen(selectedCiv, category, selection))
     }
 
     fun openNewGameScreen() {
@@ -262,7 +271,7 @@ class WorldScreen(
         globalShortcuts.add(KeyboardBinding.QuickLoad) { QuickSave.load(this) }
         globalShortcuts.add(KeyboardBinding.ViewCapitalCity) {
             val capital = gameInfo.getCurrentPlayerCivilization().getCapital()
-            if (capital != null && !mapHolder.setCenterPosition(capital.location))
+            if (capital != null && !mapHolder.setCenterPosition(capital.location.toHexCoord()))
                 game.pushScreen(CityScreen(capital))
         }
         globalShortcuts.add(KeyboardBinding.Options) { // Game Options
@@ -278,18 +287,20 @@ class WorldScreen(
         globalShortcuts.add(Input.Keys.NUMPAD_ADD) { this.mapHolder.zoomIn() }    //   '+' Zoom
         globalShortcuts.add(Input.Keys.NUMPAD_SUBTRACT) { this.mapHolder.zoomOut() }    //   '-' Zoom
         globalShortcuts.add(KeyboardBinding.ToggleUI) { toggleUI() }
-        globalShortcuts.add(KeyboardBinding.ToggleResourceDisplay) { minimapWrapper.resourceImageButton.toggle() }
         globalShortcuts.add(KeyboardBinding.ToggleYieldDisplay) { minimapWrapper.yieldImageButton.toggle() }
         globalShortcuts.add(KeyboardBinding.ToggleWorkedTilesDisplay) { minimapWrapper.populationImageButton.toggle() }
         globalShortcuts.add(KeyboardBinding.ToggleMovementDisplay) { minimapWrapper.movementsImageButton.toggle() }
+        globalShortcuts.add(KeyboardBinding.ToggleResourceDisplay) { minimapWrapper.resourceImageButton.toggle() }
+        globalShortcuts.add(KeyboardBinding.ToggleImprovementDisplay) { minimapWrapper.improvementsImageButton.toggle() }
 
         globalShortcuts.add(KeyboardBinding.DeveloperConsole, action = ::openDeveloperConsole)
     }
 
+    @Readonly
     fun openDeveloperConsole() {
-        // No cheating unless you're by yourself
-        if (gameInfo.civilizations.count { it.isHuman() } > 1) return
-        val consolePopup = DevConsolePopup(this)
+        // No cheating unless you're by yourself, ignoring a possible spectator
+        if (gameInfo.civilizations.count { it.isHuman() && !it.isSpectator() } > 1) return
+        DevConsolePopup(this)
     }
 
     private fun toggleUI() {
@@ -325,10 +336,10 @@ class WorldScreen(
         try {
             debug("loadLatestMultiplayerState current game: gameId: %s, turn: %s, curCiv: %s",
                 gameInfo.gameId, gameInfo.turns, gameInfo.currentPlayer)
-            val latestGame = game.onlineMultiplayer.downloadGame(gameInfo.gameId)
+            val latestGame = game.onlineMultiplayer.multiplayerServer.downloadGame(gameInfo.gameId)
             debug("loadLatestMultiplayerState downloaded game: gameId: %s, turn: %s, curCiv: %s",
                 latestGame.gameId, latestGame.turns, latestGame.currentPlayer)
-            if (viewingCiv.civName == latestGame.currentPlayer || viewingCiv.civName == Constants.spectator) {
+            if (viewingCiv.civID == latestGame.currentPlayer || viewingCiv.civID == Constants.spectator) {
                 game.notifyTurnStarted()
             }
             launchOnGLThread {
@@ -371,7 +382,7 @@ class WorldScreen(
             else bottomTileInfoTable.selectedCiv = viewingCiv
             bottomTileInfoTable.updateTileTable(mapHolder.selectedTile)
             bottomTileInfoTable.x = stage.width - bottomTileInfoTable.width
-            bottomTileInfoTable.y = if (game.settings.showMinimap) minimapWrapper.height else 0f
+            bottomTileInfoTable.y = if (game.settings.showMinimap) minimapWrapper.height + 5f else 0f
 
             battleTable.update()
 
@@ -381,7 +392,7 @@ class WorldScreen(
         mapHolder.resetArrows()
         if (UncivGame.Current.settings.showUnitMovements) {
             val allUnits = gameInfo.civilizations.asSequence().flatMap { it.units.getCivUnits() }
-            val allAttacks = allUnits.map { unit -> unit.attacksSinceTurnStart.asSequence().map { attacked -> Triple(unit.civ, unit.getTile().position, attacked) } }.flatten() +
+            val allAttacks = allUnits.map { unit -> unit.attacksSinceTurnStart.asSequence().map { attacked -> Triple(unit.civ, unit.getTile().position, attacked.toHexCoord()) } }.flatten() +
                 gameInfo.civilizations.asSequence().flatMap { civInfo -> civInfo.attacksSinceTurnStart.asSequence().map { Triple(civInfo, it.source, it.target) } }
             mapHolder.updateMovementOverlay(
                 allUnits.filter(mapVisualization::isUnitPastVisible),
@@ -401,6 +412,8 @@ class WorldScreen(
         else mapHolder.updateTiles(viewingCiv)
 
         topBar.update(selectedCiv)
+        if (tutorialTaskTable.isVisible)
+            tutorialTaskTable.y = topBar.getYForTutorialTask() - tutorialTaskTable.height
 
         if (techPolicyAndDiplomacy.update())
             displayTutorial(TutorialTrigger.OtherCivEncountered)
@@ -441,8 +454,8 @@ class WorldScreen(
         updateGameplayButtons()
 
         val coveredNotificationsTop = stage.height - statusButtons.y
-        val coveredNotificationsBottom = bottomTileInfoTable.height +
-                (if (game.settings.showMinimap) minimapWrapper.height else 0f)
+        val coveredNotificationsBottom = (bottomTileInfoTable.height + bottomTileInfoTable.y)
+//                (if (game.settings.showMinimap) minimapWrapper.height else 0f)
         notificationsScroll.update(viewingCiv.notifications, coveredNotificationsTop, coveredNotificationsBottom)
 
         val posZoomFromRight = if (game.settings.showMinimap) minimapWrapper.width
@@ -450,50 +463,16 @@ class WorldScreen(
         zoomController.setPosition(stage.width - posZoomFromRight - 10f, 10f, Align.bottomRight)
     }
 
-    private fun getCurrentTutorialTask(): String {
-        val completedTasks = game.settings.tutorialTasksCompleted
-        if (!completedTasks.contains("Move unit"))
-            return "Move a unit!\nClick on a unit > Click on a destination > Click the arrow popup"
-        if (!completedTasks.contains("Found city"))
-            return "Found a city!\nSelect the Settler (flag unit) > Click on 'Found city' (bottom-left corner)"
-        if (!completedTasks.contains("Enter city screen"))
-            return "Enter the city screen!\nClick the city button twice"
-        if (!completedTasks.contains("Pick technology"))
-            return "Pick a technology to research!\nClick on the tech button (greenish, top left) > " +
-                    "\n select technology > click 'Research' (bottom right)"
-        if (!completedTasks.contains("Pick construction"))
-            return "Pick a construction!\nEnter city screen > Click on a unit or building (bottom left side) >" +
-                    " \n click 'add to queue'"
-        if (!completedTasks.contains("Pass a turn"))
-            return "Pass a turn!\nCycle through units with 'Next unit' > Click 'Next turn'"
-        if (!completedTasks.contains("Reassign worked tiles"))
-            return "Reassign worked tiles!\nEnter city screen > click the assigned (green) tile to unassign > " +
-                    "\n click an unassigned tile to assign population"
-        if (!completedTasks.contains("Meet another civilization"))
-            return "Meet another civilization!\nExplore the map until you encounter another civilization!"
-        if (!completedTasks.contains("Open the options table"))
-            return "Open the options table!\nClick the menu button (top left) > click 'Options'"
-        if (!completedTasks.contains("Construct an improvement"))
-            return "Construct an improvement!\nConstruct a Worker unit > Move to a Plains or Grassland tile > " +
-                    "\n Click 'Construct improvement' (above the unit table, bottom left)" +
-                    "\n > Choose the farm > \n Leave the worker there until it's finished"
-        if (!completedTasks.contains("Create a trade route")
-                && viewingCiv.cache.citiesConnectedToCapitalToMediums.any { it.key.civ == viewingCiv })
-            game.settings.addCompletedTutorialTask("Create a trade route")
-        if (viewingCiv.cities.size > 1 && !completedTasks.contains("Create a trade route"))
-            return "Create a trade route!\nConstruct roads between your capital and another city" +
-                    "\nOr, automate your worker and let him get to that eventually"
-        if (viewingCiv.isAtWar() && !completedTasks.contains("Conquer a city"))
-            return "Conquer a city!\nBring an enemy city down to low health > " +
-                    "\nEnter the city with a melee unit"
-        if (viewingCiv.units.getCivUnits().any { it.baseUnit.movesLikeAirUnits() } && !completedTasks.contains("Move an air unit"))
-            return "Move an air unit!\nSelect an air unit > select another city within range > " +
-                    "\nMove the unit to the other city"
-        if (!completedTasks.contains("See your stats breakdown"))
-            return "See your stats breakdown!\nEnter the Overview screen (top right corner) >" +
-                    "\nClick on 'Stats'"
-
-        return ""
+    private fun getCurrentTutorialTask(): Event? {
+        if (!game.settings.tutorialTasksCompleted.contains("Create a trade route")) {
+            if (viewingCiv.cache.citiesConnectedToCapitalToMediums.any { it.key.civ == viewingCiv })
+                game.settings.addCompletedTutorialTask("Create a trade route")
+        }
+        val stateForConditionals = viewingCiv.state
+        return gameInfo.ruleset.events.values.firstOrNull {
+            it.presentation == Event.Presentation.Floating &&
+                it.isAvailable(stateForConditionals)
+        }
     }
 
     private fun displayTutorialsOnUpdate() {
@@ -503,7 +482,7 @@ class WorldScreen(
         displayTutorial(TutorialTrigger.EnemyCityNeedsConqueringWithMeleeUnit) {
             viewingCiv.diplomacy.values.asSequence()
                     .filter { it.diplomaticStatus == DiplomaticStatus.War }
-                    .map { it.otherCiv() } // we're now lazily enumerating over CivilizationInfo's we're at war with
+                    .map { it.otherCiv } // we're now lazily enumerating over CivilizationInfo's we're at war with
                     .flatMap { it.cities.asSequence() } // ... all *their* cities
                     .filter { it.health == 1 } // ... those ripe for conquering
                     .flatMap { it.getCenterTile().getTilesInDistance(2) }
@@ -524,27 +503,38 @@ class WorldScreen(
     }
 
     private fun displayTutorialTaskOnUpdate() {
-        tutorialTaskTable.clear()
-        val tutorialTask = getCurrentTutorialTask()
-        if (tutorialTask == "" || !game.settings.showTutorials || viewingCiv.isDefeated()) {
+        fun setInvisible() {
             tutorialTaskTable.isVisible = false
-            return
+            tutorialTaskTable.clear()
+            tutorialTaskTableHash = 0
         }
+        if (!game.settings.showTutorials || viewingCiv.isDefeated()) return setInvisible()
+        val tutorialTask = getCurrentTutorialTask() ?: return setInvisible()
 
-        tutorialTaskTable.isVisible = true
         if (!UncivGame.Current.isTutorialTaskCollapsed) {
-            tutorialTaskTable.add(tutorialTask.toLabel()
-                .apply { setAlignment(Align.center) }).pad(10f)
+            val hash = tutorialTask.hashCode()  // Default implementation is OK - we see the same instance or not
+            if (hash != tutorialTaskTableHash) {
+                val renderEvent = RenderEvent(tutorialTask, this) {
+                    shouldUpdate = true
+                }
+                if (!renderEvent.isValid) return setInvisible()
+                tutorialTaskTable.clear()
+                tutorialTaskTable.add(renderEvent).pad(10f)
+                tutorialTaskTableHash = hash
+            }
         } else {
-            tutorialTaskTable.add(ImageGetter.getImage("CityStateIcons/Cultured").apply { setSize(30f,30f) }).pad(5f)
+            tutorialTaskTable.clear()
+            tutorialTaskTable.add(ImageGetter.getImage("OtherIcons/HiddenTutorialTask").apply { setSize(30f,30f) }).pad(5f)
+            tutorialTaskTableHash = 0
         }
         tutorialTaskTable.pack()
         tutorialTaskTable.centerX(stage)
-        tutorialTaskTable.y = topBar.y - tutorialTaskTable.height
+        tutorialTaskTable.y = topBar.getYForTutorialTask() - tutorialTaskTable.height
         tutorialTaskTable.onClick {
             UncivGame.Current.isTutorialTaskCollapsed = !UncivGame.Current.isTutorialTaskCollapsed
             displayTutorialTaskOnUpdate()
         }
+        tutorialTaskTable.isVisible = true
     }
 
     private fun updateSelectedCiv() {
@@ -565,14 +555,16 @@ class WorldScreen(
         val scrollX = mapHolder.scrollX
         val scrollY = mapHolder.scrollY
     }
+    
+    @Readonly
     fun getRestoreState(): RestoreState {
-        return RestoreState(mapHolder, selectedCiv.civName, viewingCiv.civName, fogOfWar)
+        return RestoreState(mapHolder, selectedCiv.civID, viewingCiv.civID, fogOfWar)
     }
 
     private fun restore(restoreState: RestoreState) {
 
         // This is not the case if you have a multiplayer game where you play as 2 civs
-        if (viewingCiv.civName == restoreState.viewingCivName) {
+        if (viewingCiv.civID == restoreState.viewingCivName) {
             mapHolder.zoom(restoreState.zoom)
             mapHolder.scrollX = restoreState.scrollX
             mapHolder.scrollY = restoreState.scrollY
@@ -599,20 +591,37 @@ class WorldScreen(
 
             progressBar.increment()
 
-            gameInfoClone.nextTurn(progressBar)
+            gameInfoClone.nextTurn(progressBar, true)
 
             if (originalGameInfo.gameParameters.isOnlineMultiplayer) {
+                // outer try-catch for non-auth exceptions
                 try {
-                    game.onlineMultiplayer.updateGame(gameInfoClone)
-                }catch (ex: Exception) {
-                    when (ex) {
-                        is MultiplayerAuthException -> {
+                    // keep retrying if upload fails AND reauthentication succeeds
+                    var retryUpload: Boolean
+                    do {
+                        try {
+                            game.onlineMultiplayer.updateGame(gameInfoClone)
+                            // upload succeeded
+                            retryUpload = false
+                        } catch (_: MultiplayerAuthException) {
+                            // true only if authentication succeeds (the popup permits retries)
+                            // false only if user closes the auth popup or the popup init crashes
+                            val authResult = CompletableDeferred<Boolean>()
                             launchOnGLThread {
-                                AuthPopup(this@WorldScreen) {
-                                        success -> if (success) nextTurn()
-                                }.open(true)
+                                try {
+                                    AuthPopup(this@WorldScreen, authResult::complete).open(true)
+                                } catch (ex: Exception) {
+                                    // GL thread crashed during AuthPopup init, let's wrap up
+                                    authResult.complete(false)
+                                    // ensure exception is passed to crash handler
+                                    throw ex
+                                }
                             }
+                            retryUpload = authResult.await()
                         }
+                    } while (retryUpload)
+                } catch (ex: Exception) { // non-auth exceptions
+                    when (ex) {
                         is FileStorageRateLimitReached -> {
                             val message = "Server limit reached! Please wait for [${ex.limitRemainingSeconds}] seconds"
                             launchOnGLThread {
@@ -623,10 +632,13 @@ class WorldScreen(
                             }
                         }
                         else -> {
-                            val message = "Could not upload game!"
+                            val message = "Could not upload game! Reason: [${ex.message ?: "Unknown"}]"
                             launchOnGLThread {
                                 val cantUploadNewGamePopup = Popup(this@WorldScreen)
                                 cantUploadNewGamePopup.addGoodSizedLabel(message).row()
+                                cantUploadNewGamePopup.addButton("Copy to clipboard") {
+                                    Gdx.app.clipboard.contents = ex.stackTraceToString()
+                                }
                                 cantUploadNewGamePopup.addCloseButton()
                                 cantUploadNewGamePopup.open()
                             }
@@ -637,7 +649,6 @@ class WorldScreen(
                     this@WorldScreen.shouldUpdate = true
                     return@runOnNonDaemonThreadPool
                 }
-
             }
 
             if (game.gameInfo != originalGameInfo) // while this was turning we loaded another game
@@ -657,9 +668,9 @@ class WorldScreen(
         }
     }
 
-    fun switchToNextUnit() {
+    fun switchToNextUnit(resetDue: Boolean = true) {
         // Try to select something new if we already have the next pending unit selected.
-        if (bottomUnitTable.selectedUnit != null)
+        if (bottomUnitTable.selectedUnit != null && resetDue)
             bottomUnitTable.selectedUnit!!.due = false
         val nextDueUnit = viewingCiv.units.cycleThroughDueUnits(bottomUnitTable.selectedUnit)
         if (nextDueUnit != null) {
@@ -676,7 +687,8 @@ class WorldScreen(
         }
         shouldUpdate = true
     }
-
+    
+    @Readonly
     internal fun isNextTurnUpdateRunning(): Boolean {
         val job = nextTurnUpdateJob
         return job != null && job.isActive
@@ -688,15 +700,15 @@ class WorldScreen(
         updateAutoPlayStatusButton()
         updateMultiplayerStatusButton()
 
-        statusButtons.wrap(false)
-        statusButtons.pack()
+        statusButtons.update(false)
         val maxWidth = stage.width - techPolicyAndDiplomacy.width - 25f
         if(statusButtons.width > maxWidth) {
-            statusButtons.width = maxWidth
-            statusButtons.wrap()
-            statusButtons.pack()
+            statusButtons.update(true)
         }
         statusButtons.setPosition(stage.width - statusButtons.width - 10f, topBar.y - statusButtons.height - 10f)
+
+        // Update chat button position to always be below techPolicyAndDiplomacy
+        chatButton.updatePosition()
     }
 
     private fun updateAutoPlayStatusButton() {
@@ -714,7 +726,8 @@ class WorldScreen(
     private fun updateMultiplayerStatusButton() {
         if (gameInfo.gameParameters.isOnlineMultiplayer || game.settings.multiplayer.statusButtonInSinglePlayer) {
             if (statusButtons.multiplayerStatusButton != null) return
-            statusButtons.multiplayerStatusButton = MultiplayerStatusButton(this, game.onlineMultiplayer.getGameByGameId(gameInfo.gameId))
+            statusButtons.multiplayerStatusButton = MultiplayerStatusButton(this,
+                game.onlineMultiplayer.multiplayerFiles.getGameByGameId(gameInfo.gameId))
         } else {
             if (statusButtons.multiplayerStatusButton == null) return
             statusButtons.multiplayerStatusButton = null
@@ -753,10 +766,10 @@ class WorldScreen(
 
 
     private fun showTutorialsOnNextTurn() {
-        if (!game.settings.showTutorials) return
+        if (!game.settings.showTutorials || autoPlay.isAutoPlaying()) return
         displayTutorial(TutorialTrigger.SlowStart)
         displayTutorial(TutorialTrigger.CityExpansion) { viewingCiv.cities.any { it.expansion.tilesClaimed() > 0 } }
-        displayTutorial(TutorialTrigger.BarbarianEncountered) { viewingCiv.viewableTiles.any { it.getUnits().any { unit -> unit.civ.isBarbarian() } } }
+        displayTutorial(TutorialTrigger.BarbarianEncountered) { viewingCiv.viewableTiles.any { it.getUnits().any { unit -> unit.civ.isBarbarian } } }
         displayTutorial(TutorialTrigger.RoadsAndRailroads) { viewingCiv.cities.size > 2 }
         displayTutorial(TutorialTrigger.Happiness) { viewingCiv.getHappiness() < 5 }
         displayTutorial(TutorialTrigger.Unhappiness) { viewingCiv.getHappiness() < 0 }
@@ -770,8 +783,6 @@ class WorldScreen(
             viewingCiv.getKnownCivs().filter { viewingCiv.isAtWarWith(it) }
                     .flatMap { it.cities.asSequence() }.any { viewingCiv.hasExplored(it.getCenterTile()) }
         }
-        displayTutorial(TutorialTrigger.ApolloProgram) { viewingCiv.hasUnique(UniqueType.EnablesConstructionOfSpaceshipParts) }
-        displayTutorial(TutorialTrigger.SiegeUnits) { viewingCiv.units.getCivUnits().any { it.baseUnit.isProbablySiegeUnit() } }
         displayTutorial(TutorialTrigger.Embarking) { viewingCiv.hasUnique(UniqueType.LandUnitEmbarkation) }
         displayTutorial(TutorialTrigger.NaturalWonders) { viewingCiv.naturalWonders.size > 0 }
         displayTutorial(TutorialTrigger.WeLoveTheKingDay) { viewingCiv.cities.any { it.demandedResource != "" } }
@@ -782,19 +793,17 @@ class WorldScreen(
         // Deselect Unit
         if (bottomUnitTable.selectedUnit != null) {
             bottomUnitTable.selectUnit()
-            bottomUnitTable.isVisible = false
             shouldUpdate = true
             return
         }
 
         // Deselect city
         if (bottomUnitTable.selectedCity != null) {
-            bottomUnitTable.selectedCity = null
-            bottomUnitTable.isVisible = false
+            bottomUnitTable.selectUnit()
             shouldUpdate = true
             return
         }
-        
+
         if (bottomUnitTable.selectedSpy != null) {
             bottomUnitTable.selectSpy(null)
             shouldUpdate = true
