@@ -1,9 +1,12 @@
 package com.unciv.ui.screens.worldscreen.bottombar
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.math.Interpolation
 import com.badlogic.gdx.scenes.scene2d.Touchable
+import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.ui.Label
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.badlogic.gdx.utils.Align
 import com.unciv.logic.battle.AirInterception
 import com.unciv.logic.battle.AttackableTile
 import com.unciv.logic.battle.Battle
@@ -19,20 +22,23 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.SoundPlayer
 import com.unciv.ui.components.extensions.addBorderAllowOpacity
+import com.unciv.ui.components.extensions.addRoundCloseButton
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.disable
+import com.unciv.ui.components.extensions.setSize
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.onClick
-import com.unciv.ui.components.widgets.UnitGroup
+import com.unciv.ui.components.widgets.UnitIconGroup
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.worldscreen.UndoHandler.Companion.clearUndoCheckpoints
 import com.unciv.ui.screens.worldscreen.WorldScreen
-import com.unciv.ui.screens.worldscreen.bottombar.BattleTableHelpers.battleAnimation
+import com.unciv.ui.screens.worldscreen.bottombar.BattleTableHelpers.battleAnimationDeferred
 import com.unciv.ui.screens.worldscreen.bottombar.BattleTableHelpers.getHealthBar
 import com.unciv.utils.DebugUtils
+import yairm210.purity.annotations.Readonly
 import kotlin.math.max
 import kotlin.math.roundToInt
 
@@ -84,9 +90,15 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
 
         isVisible = true
         pack()
-        addBorderAllowOpacity(1f, Color.WHITE)
+
+        addBorderAllowOpacity(2f, Color.WHITE)
+        addRoundCloseButton(this) {
+            isVisible = false
+        }
+
     }
 
+    @Readonly
     private fun tryGetAttacker(): ICombatant? {
         val unitTable = worldScreen.bottomUnitTable
         return if (unitTable.selectedUnit != null
@@ -98,11 +110,13 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
         else null // no attacker
     }
 
+    @Readonly
     private fun tryGetDefender(): ICombatant? {
         val selectedTile = worldScreen.mapHolder.selectedTile ?: return null // no selected tile
         return tryGetDefenderAtTile(selectedTile, false)
     }
 
+    @Readonly
     private fun tryGetDefenderAtTile(selectedTile: Tile, includeFriendly: Boolean): ICombatant? {
         val attackerCiv = worldScreen.viewingCiv
         val defender: ICombatant? = Battle.getMapCombatantOfTile(selectedTile)
@@ -123,7 +137,7 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
     }
 
     private fun getIcon(combatant: ICombatant) =
-        if (combatant is MapUnitCombatant) UnitGroup(combatant.unit,25f)
+        if (combatant is MapUnitCombatant) UnitIconGroup(combatant.unit,25f)
         else ImageGetter.getNationPortrait(combatant.getCivInfo().nation, 25f)
 
     private val quarterScreen = worldScreen.stage.width / 4
@@ -164,8 +178,8 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
             if (attacker.isRanged() && defender.isRanged() && !defender.isCity() && !(defender is MapUnitCombatant && defender.unit.isEmbarked()))
                 Fonts.rangedStrength
             else Fonts.strength // use strength icon if attacker is melee, defender is melee, defender is a city, or defender is embarked
-        add(attacker.getAttackingStrength().toString() + attackIcon)
-        add(defender.getDefendingStrength(attacker.isRanged()).toString() + defenceIcon).row()
+        add(attacker.getAttackingStrength(defender).tr() + attackIcon)
+        add(defender.getDefendingStrength(attacker).tr() + defenceIcon).row()
 
         val attackerModifiers =
                 BattleDamage.getAttackModifiers(attacker, defender, tileToAttackFrom).map {
@@ -188,14 +202,15 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
             addSeparator()
             val attackerStrength = BattleDamage.getAttackingStrength(attacker, defender, tileToAttackFrom).roundToInt()
             val defenderStrength = BattleDamage.getDefendingStrength(attacker, defender, tileToAttackFrom).roundToInt()
-            add(attackerStrength.toString() + attackIcon)
-            add(defenderStrength.toString() + attackIcon).row()
+            add(attackerStrength.tr() + attackIcon)
+            add(defenderStrength.tr() + attackIcon).row()
         }
 
         // from Battle.addXp(), check for can't gain more XP from Barbarians
         val maxXPFromBarbarians = attacker.getCivInfo().gameInfo.ruleset.modOptions.constants.maxXPfromBarbarians
         if (attacker is MapUnitCombatant && attacker.unit.promotions.totalXpProduced() >= maxXPFromBarbarians
-                && defender.getCivInfo().isBarbarian()) {
+                && defender.getCivInfo().isBarbarian
+        ) {
             add("Cannot gain more XP from Barbarians".toLabel(fontSize = 16).apply { wrap = true }).width(quarterScreen)
             row()
         }
@@ -225,7 +240,7 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
             val maxRemainingLifeDefender = max(defenderHealth-minDamageToDefender, 0)
 
             add(getHealthBar(attacker.getMaxHealth(), attacker.getHealth(), maxRemainingLifeAttacker, minRemainingLifeAttacker))
-            add(getHealthBar(defender.getMaxHealth(), defender.getHealth(), maxRemainingLifeDefender, minRemainingLifeDefender)).row()
+            add(getHealthBar(defender.getMaxHealth(), defender.getHealth(), maxRemainingLifeDefender, minRemainingLifeDefender, true)).row()
 
             fun avg(vararg values: Int) = values.average().roundToInt()
             // Don't use original damage estimates - they're raw, before clamping to 0..max
@@ -302,10 +317,12 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
         //Gdx.graphics.requestRendering()  // Use this if immediate rendering is required
 
         if (!canStillAttack) return
-        SoundPlayer.play(attacker.getAttackSound())
+        if (!SoundPlayer.play(UncivSound(attacker.getName())))
+            SoundPlayer.play(attacker.getAttackSound())
+
         val (damageToDefender, damageToAttacker) = Battle.attackOrNuke(attacker, attackableTile)
 
-        worldScreen.battleAnimation(attacker, damageToAttacker, defender, damageToDefender)
+        worldScreen.battleAnimationDeferred(attacker, damageToAttacker, defender, damageToDefender)
         if (!attacker.canAttack()) hide()
     }
 
@@ -338,15 +355,34 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
 
         val attackButton = "NUKE".toTextButton().apply { color = Color.RED }
 
-        val canReach = attacker.unit.currentTile.getTilesInDistance(attacker.unit.getRange()).contains(targetTile)
-
-        if (!worldScreen.isPlayersTurn || !attacker.canAttack() || !canReach || !canNuke) {
+        if (!worldScreen.isPlayersTurn || !attacker.canAttack() || !canNuke) {
             attackButton.disable()
             attackButton.label.color = Color.GRAY
         }
         else {
             attackButton.onClick(attacker.getAttackSound()) {
                 Nuke.NUKE(attacker, targetTile)
+
+                val nukeCircle = ImageGetter.getCircle()
+                nukeCircle.setSize(10f)
+                nukeCircle.setOrigin(Align.center)
+                nukeCircle.addAction(Actions.sequence(
+                    Actions.fadeOut(0f),
+                    Actions.parallel(
+                        Actions.fadeIn(1f, Interpolation.pow2In),
+                        Actions.scaleTo(200f, 200f, 1f, Interpolation.linear),
+                    ),
+                    Actions.delay(1f),
+                    Actions.fadeOut(1f, Interpolation.pow2Out),
+                    Actions.removeActor()
+                    )
+                )
+                val targetTileGroup = worldScreen.mapHolder.tileGroups[targetTile]!!
+                nukeCircle.x = targetTileGroup.x
+                nukeCircle.y = targetTileGroup.y
+                targetTileGroup.parent.addActor(nukeCircle)
+
+
                 worldScreen.mapHolder.removeUnitActionOverlay() // the overlay was one of attacking
                 worldScreen.shouldUpdate = true
             }
@@ -377,7 +413,7 @@ class BattleTable(val worldScreen: WorldScreen) : Table() {
         addSeparator().pad(0f)
 
         val attackIcon = Fonts.rangedStrength
-        add(attacker.getAttackingStrength().toString() + attackIcon)
+        add(attacker.getAttackingStrength().tr() + attackIcon)
         add("???$attackIcon").row()
 
         val attackerModifiers =
