@@ -13,6 +13,11 @@ import com.unciv.models.translations.tr
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.popups.hasOpenPopups
 import com.unciv.ui.screens.pickerscreens.PromotionPickerScreen
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions.getActionDefaultPage
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions.getPagingActions
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions.getUnitActions
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions.invokeUnitAction
+import yairm210.purity.annotations.Readonly
 
 /**
  *  Manages creation of [UnitAction] instances.
@@ -74,6 +79,7 @@ object UnitActions {
         UnitActionType.Paradrop to UnitActionsFromUniques::getParadropActions,
         UnitActionType.AirSweep to UnitActionsFromUniques::getAirSweepActions,
         UnitActionType.SetUp to UnitActionsFromUniques::getSetupActions,
+        UnitActionType.Guard to UnitActionsFromUniques::getGuardActions,
         UnitActionType.FoundCity to UnitActionsFromUniques::getFoundCityActions,
         UnitActionType.ConstructImprovement to UnitActionsFromUniques::getBuildingImprovementsActions,
         UnitActionType.ConnectRoad to UnitActionsFromUniques::getConnectRoadActions,
@@ -94,6 +100,7 @@ object UnitActions {
     )
 
     /** Gets the preferred "page" to display a [UnitAction] of type [unitActionType] on, possibly dynamic depending on the state or situation [unit] is in. */
+    @Readonly
     fun getActionDefaultPage(unit: MapUnit, unitActionType: UnitActionType) =
         actionTypeToPageGetter[unitActionType]?.invoke(unit) ?: unitActionType.defaultPage
 
@@ -150,7 +157,7 @@ object UnitActions {
 
         addExplorationActions(unit)
 
-        addWaitAction(unit)
+        addSkipAction(unit)
 
         // From here we have actions defaulting to the second page
         if (unit.isMoving()) {
@@ -165,16 +172,16 @@ object UnitActions {
 
     private suspend fun SequenceScope<UnitAction>.addEscortAction(unit: MapUnit) {
         // Air units cannot escort
-        if (unit.baseUnit.movesLikeAirUnits()) return
+        if (unit.baseUnit.movesLikeAirUnits) return
 
         val worldScreen = GUI.getWorldScreen()
         val selectedUnits = worldScreen.bottomUnitTable.selectedUnits
         if (selectedUnits.size == 2) {
-            // We can still create a formation in the case that we have two units selected 
+            // We can still create a formation in the case that we have two units selected
             // and they are on the same tile. We still have to manualy confirm they are on the same tile here.
             val tile = selectedUnits.first().getTile()
             if (selectedUnits.last().getTile() != tile) return
-            if (selectedUnits.any { it.baseUnit.movesLikeAirUnits() }) return
+            if (selectedUnits.any { it.baseUnit.movesLikeAirUnits }) return
         } else if (selectedUnits.size != 1) {
             return
         }
@@ -195,10 +202,10 @@ object UnitActions {
                 }))
         }
     }
-    
+
     private suspend fun SequenceScope<UnitAction>.addSwapAction(unit: MapUnit) {
         // Air units cannot swap
-        if (unit.baseUnit.movesLikeAirUnits()) return
+        if (unit.baseUnit.movesLikeAirUnits) return
         // Disable unit swapping if multiple units are selected. It would make little sense.
         // In principle, the unit swapping mode /will/ function with multiselect: it will simply
         // only consider the first selected unit, and ignore the other selections. However, it does
@@ -238,7 +245,7 @@ object UnitActions {
                             worldScreen.switchToNextUnit()
                     }.open()
                 }
-            }.takeIf { unit.currentMovement > 0 }
+            }.takeIf { unit.hasMovement() }
         ))
     }
 
@@ -249,16 +256,16 @@ object UnitActions {
             useFrequency = 150f, // We want to show the player that they can promote
             action = {
                 UncivGame.Current.pushScreen(PromotionPickerScreen(unit))
-            }.takeIf { unit.currentMovement > 0 && unit.attacksThisTurn == 0 }
+            }.takeIf { unit.hasMovement() && unit.attacksThisTurn == 0 }
         ))
     }
 
     private suspend fun SequenceScope<UnitAction>.addExplorationActions(unit: MapUnit) {
-        if (unit.baseUnit.movesLikeAirUnits()) return
+        if (unit.baseUnit.movesLikeAirUnits) return
         if (unit.isExploring()) return
         yield(UnitAction(UnitActionType.Explore, 5f) {
             unit.action = UnitActionType.Explore.value
-            if (unit.currentMovement > 0) UnitAutomation.automatedExplore(unit)
+            if (unit.hasMovement()) UnitAutomation.automatedExplore(unit)
         })
     }
 
@@ -275,7 +282,7 @@ object UnitActions {
             return
         }
 
-        if (!unit.canFortify() || unit.currentMovement == 0f) return
+        if (!unit.canFortify() || !unit.hasMovement()) return
 
         yield(UnitAction(UnitActionType.Fortify,
             action = { unit.fortify() }.takeIf { !unit.isFortified() || unit.isFortifyingUntilHealed() },
@@ -291,7 +298,7 @@ object UnitActions {
     }
 
     private suspend fun SequenceScope<UnitAction>.addSleepActions(unit: MapUnit, tile: Tile) {
-        if (unit.isFortified() || unit.canFortify() || unit.currentMovement == 0f) return
+        if (unit.isFortified() || unit.canFortify() || unit.isGuarding() || !unit.hasMovement()) return
         if (tile.hasImprovementInProgress() && unit.canBuildImprovement(tile.getTileImprovementInProgress()!!)) return
 
         yield(UnitAction(UnitActionType.Sleep,
@@ -312,7 +319,7 @@ object UnitActions {
         // We need to be in another civs territory.
         if (recipient == null || recipient.isCurrentPlayer()) return@sequence
 
-        if (recipient.isCityState()) {
+        if (recipient.isCityState) {
             if (recipient.isAtWarWith(unit.civ)) return@sequence // No gifts to enemy CS
             // City States only take military units (and units specifically allowed by uniques)
             if (!unit.isMilitary()
@@ -329,29 +336,29 @@ object UnitActions {
         // Transported units can't be gifted
         if (unit.isTransported) return@sequence
 
-        if (unit.currentMovement <= 0) {
+        if (!unit.hasMovement()) {
             yield(UnitAction(UnitActionType.GiftUnit, 1f, action = null))
             return@sequence
         }
 
         val giftAction = {
-            if (recipient.isCityState()) {
+            if (recipient.isCityState) {
                 for (unique in unit.getMatchingUniques(
                     UniqueType.GainInfluenceWithUnitGiftToCityState,
                     checkCivInfoUniques = true
                 )) {
                     if (unit.matchesFilter(unique.params[1])) {
-                        recipient.getDiplomacyManager(unit.civ)
+                        recipient.getDiplomacyManager(unit.civ)!!
                             .addInfluence(unique.params[0].toFloat() - 5f)
                         break
                     }
                 }
 
-                recipient.getDiplomacyManager(unit.civ).addInfluence(5f)
-            } else recipient.getDiplomacyManager(unit.civ)
+                recipient.getDiplomacyManager(unit.civ)!!.addInfluence(5f)
+            } else recipient.getDiplomacyManager(unit.civ)!!
                 .addModifier(DiplomaticModifiers.GaveUsUnits, 5f)
 
-            if (recipient.isCityState() && unit.isGreatPerson())
+            if (recipient.isCityState && unit.isGreatPerson())
                 unit.destroy()  // City states don't get GPs
             else
                 unit.gift(recipient)
@@ -366,22 +373,25 @@ object UnitActions {
             isCurrentAction = unit.isAutomated(),
             useFrequency = 25f,
             action = {
-                // Temporary, for compatibility - we want games serialized *moving through old versions* to come out the other end with units still automated
-                unit.action = UnitActionType.Automate.value
                 unit.automated = true
                 UnitAutomation.automateUnitMoves(unit)
-            }.takeIf { unit.currentMovement > 0 }
+            }.takeIf { unit.hasMovement() }
         ))
     }
 
-    private suspend fun SequenceScope<UnitAction>.addWaitAction(unit: MapUnit) {
+    // Skip one turn: marks a unit as due=false and doesn't cycle back in the queue
+    private suspend fun SequenceScope<UnitAction>.addSkipAction(unit: MapUnit) {
         yield(UnitAction(
-            type = UnitActionType.Wait,
-            useFrequency = 65f, // Preferably have this on the first page
+            type = UnitActionType.Skip,
+            useFrequency = 0f, // Last on first page (defaultPage=0)
             action = {
-                unit.due = false
-                GUI.getWorldScreen().switchToNextUnit()
-            }
+                unit.due = !unit.due
+                // If it's on, skips to next unit due to worldScreen.switchToNextUnit() in activateAction
+                // We don't want to switch twice since then we skip units :)
+                if (!unit.due && !UncivGame.Current.settings.autoUnitCycle)
+                    GUI.getWorldScreen().switchToNextUnit()
+            }.takeIf { unit.hasMovement() },
+            isCurrentAction = !unit.due
         ))
     }
 
