@@ -6,14 +6,16 @@ import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.PlayerType
 import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
+import com.unciv.logic.civilization.managers.ImprovementFunctions
 import com.unciv.logic.map.mapunit.MapUnit
+import com.unciv.logic.map.tile.ImprovementBuildingProblem
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.Counter
 import com.unciv.models.UncivSound
 import com.unciv.models.UnitAction
 import com.unciv.models.UnitActionType
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
@@ -24,6 +26,7 @@ import com.unciv.models.translations.tr
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.screens.pickerscreens.ImprovementPickerScreen
+import yairm210.purity.annotations.Readonly
 
 @Suppress("UNUSED_PARAMETER") // These methods are used as references in UnitActions.actionTypeToFunctions and need identical signature
 object UnitActionsFromUniques {
@@ -39,27 +42,35 @@ object UnitActionsFromUniques {
      * (no movement left, too close to another city).
      */
     internal fun getFoundCityAction(unit: MapUnit, tile: Tile): UnitAction? {
-        val unique = UnitActionModifiers.getUsableUnitActionUniques(unit, UniqueType.FoundCity)
-            .firstOrNull() ?: return null
+        // FoundPuppetCity is to found a puppet city for modding.
+        val unique = UnitActionModifiers.getUsableUnitActionUniques(unit,
+            UniqueType.FoundCity).firstOrNull() ?: 
+            UnitActionModifiers.getUsableUnitActionUniques(unit,
+            UniqueType.FoundPuppetCity).firstOrNull() ?: return null
 
         if (tile.isWater || tile.isImpassible()) return null
         // Spain should still be able to build Conquistadors in a one city challenge - but can't settle them
         if (unit.civ.isOneCityChallenger() && unit.civ.hasEverOwnedOriginalCapital) return null
 
-        if (unit.currentMovement <= 0 || !tile.canBeSettled())
+        if (!unit.hasMovement() || !tile.canBeSettled(unit.civ))
             return UnitAction(UnitActionType.FoundCity, 80f, action = null)
 
-        val hasActionModifiers = unique.conditionals.any { it.type?.targetTypes?.contains(
+        val hasActionModifiers = unique.modifiers.any { it.type?.targetTypes?.contains(
             UniqueTarget.UnitActionModifier
         ) == true }
         val foundAction = {
             if (unit.civ.playerType != PlayerType.AI)
                 UncivGame.Current.settings.addCompletedTutorialTask("Found city")
-            unit.civ.addCity(tile.position, unit)
+            // Get the city to be able to change it into puppet, for modding.
+            val city = unit.civ.addCity(tile.position, unit)
 
             if (hasActionModifiers) UnitActionModifiers.activateSideEffects(unit, unique)
             else unit.destroy()
             GUI.setUpdateWorldOnNextRender() // Set manually, since this could be triggered from the ConfirmPopup and not from the UnitActionsTable
+            // If unit has FoundPuppetCity make it into a puppet city.
+            if (unique.type == UniqueType.FoundPuppetCity) {
+                city.isPuppet = true
+            }
         }
 
         if (unit.civ.playerType == PlayerType.AI)
@@ -78,6 +89,7 @@ object UnitActionsFromUniques {
             useFrequency = 80f,
             title = title,
             uncivSound = UncivSound.Chimes,
+            associatedUnique = unique,
             action = {
                 // check if we would be breaking a promise
                 val leadersPromisedNotToSettleNear = getLeadersWePromisedNotToSettleNear(unit.civ, tile)
@@ -103,10 +115,11 @@ object UnitActionsFromUniques {
      * @param tile The tile where the new city would go
      * @return null if no promises broken, else a String listing the leader(s) we would p* off.
      */
+    @Readonly
     private fun getLeadersWePromisedNotToSettleNear(civInfo: Civilization, tile: Tile): String? {
         val leadersWePromisedNotToSettleNear = HashSet<String>()
         for (otherCiv in civInfo.getKnownCivs().filter { it.isMajorCiv() && !civInfo.isAtWarWith(it) }) {
-            val diplomacyManager = otherCiv.getDiplomacyManager(civInfo)
+            val diplomacyManager = otherCiv.getDiplomacyManager(civInfo)!!
             if (diplomacyManager.hasFlag(DiplomacyFlags.AgreedToNotSettleNearUs)) {
                 val citiesWithin6Tiles = otherCiv.cities
                     .filter { it.getCenterTile().aerialDistanceTo(tile) <= 6 }
@@ -126,15 +139,28 @@ object UnitActionsFromUniques {
             action = {
                 unit.action = UnitActionType.SetUp.value
                 unit.useMovementPoints(1f)
-            }.takeIf { unit.currentMovement > 0 && !isSetUp })
+            }.takeIf { unit.hasMovement() && !isSetUp })
         )
     }
 
     internal fun getParadropActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
-        val paradropUniques =
-            unit.getMatchingUniques(UniqueType.MayParadrop)
-        if (!paradropUniques.any() || unit.isEmbarked()) return emptySequence()
-        unit.cache.paradropRange = paradropUniques.maxOfOrNull { it.params[0] }!!.toInt()
+        unit.cache.paradropDestinationTileFilters.clear()
+
+        // Retrieve all parardrop uniques, considering the state of the unit
+        val paradropUniques = unit.getMatchingUniques(UniqueType.MayParadrop, unit.cache.state)
+
+        // Construct the list of possible destination tile filters, keeping the largest distance
+        for (unique in paradropUniques) {
+            val tileFilter = unique.params[0]
+            val distance = unique.params[1].toInt()
+            val existingDistance = unit.cache.paradropDestinationTileFilters[tileFilter]
+            if (existingDistance == null || distance > existingDistance) {
+                unit.cache.paradropDestinationTileFilters[tileFilter] = distance
+            }
+        }
+
+        if (unit.cache.paradropDestinationTileFilters.isEmpty()) return emptySequence()
+
         return sequenceOf(UnitAction(UnitActionType.Paradrop,
             isCurrentAction = unit.isPreparingParadrop(),
             useFrequency = 60f, // While it is important to see, it isn't nessesary used a lot
@@ -142,9 +168,7 @@ object UnitActionsFromUniques {
                 if (unit.isPreparingParadrop()) unit.action = null
                 else unit.action = UnitActionType.Paradrop.value
             }.takeIf {
-                !unit.hasUnitMovedThisTurn() &&
-                        tile.isFriendlyTerritory(unit.civ) &&
-                        !tile.isWater
+                !unit.hasUnitMovedThisTurn()
             })
         )
     }
@@ -165,56 +189,87 @@ object UnitActionsFromUniques {
         ))
     }
 
+    // Instead of Withdrawing, stand your ground!
+    // Different than Fortify
+    internal fun getGuardActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
+        if (!unit.hasUnique(UniqueType.WithdrawsBeforeMeleeCombat)) return emptySequence()
+        
+        if (unit.isGuarding()) {
+            val title = if (unit.canFortify()) "${"Guarding".tr()} ${unit.getFortificationTurns() * 20}%" else "Guarding".tr()
+            return sequenceOf(UnitAction(UnitActionType.Guard,
+                useFrequency = 0f,
+                isCurrentAction = true,
+                title = title
+            ))
+        }
+        
+        if (!unit.hasMovement()) return emptySequence()
+        
+        return sequenceOf(UnitAction(UnitActionType.Guard,
+            useFrequency = 0f,
+            action = {
+                unit.action = UnitActionType.Guard.value
+            }.takeIf { !unit.isGuarding() })
+        )
+    }
+
     internal fun getTriggerUniqueActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getUniques()) {
             // not a unit action
-            if (unique.conditionals.none { it.type?.targetTypes?.contains(UniqueTarget.UnitActionModifier) == true }) continue
+            if (unique.modifiers.none { it.type?.targetTypes?.contains(UniqueTarget.UnitActionModifier) == true }) continue
             // extends an existing unit action
-            if (unique.conditionals.any { it.type == UniqueType.UnitActionExtraLimitedTimes }) continue
+            if (unique.hasModifier(UniqueType.UnitActionExtraLimitedTimes)) continue
             if (!unique.isTriggerable) continue
-            if (!unique.conditionalsApply(StateForConditionals(civInfo = unit.civ, unit = unit, tile = unit.currentTile))) continue
+            if (!unique.conditionalsApply(unit.cache.state)) continue
             if (!UnitActionModifiers.canUse(unit, unique)) continue
 
             val baseTitle = when (unique.type) {
                 UniqueType.OneTimeEnterGoldenAgeTurns -> {
                     unique.placeholderText.fillPlaceholders(
                         unit.civ.goldenAges.calculateGoldenAgeLength(
-                            unique.params[0].toInt()).toString())
+                            unique.params[0].toInt()).tr())
                     }
-                UniqueType.OneTimeGainStatSpeed -> {
-                    val stat = unique.params[1]
-                    val modifier = unit.civ.gameInfo.speed.statCostModifiers[Stat.safeValueOf(stat)]
-                        ?: unit.civ.gameInfo.speed.modifier
-                    UniqueType.OneTimeGainStat.placeholderText.fillPlaceholders(
-                        (unique.params[0].toInt() * modifier).toInt().toString(), stat
-                    )
+                UniqueType.OneTimeGainStat -> {
+                    if (unique.hasModifier(UniqueType.ModifiedByGameSpeed)) {
+                        val stat = unique.params[1]
+                        val modifier = unit.civ.gameInfo.speed.statCostModifiers[Stat.safeValueOf(stat)]
+                            ?: unit.civ.gameInfo.speed.modifier
+                        UniqueType.OneTimeGainStat.placeholderText.fillPlaceholders(
+                            (unique.params[0].toInt() * modifier).toInt().tr(), stat
+                        )
+                    }
+                    else unique.text.removeConditionals()
                 }
                 UniqueType.OneTimeGainStatRange -> {
                     val stat = unique.params[2]
                     val modifier = unit.civ.gameInfo.speed.statCostModifiers[Stat.safeValueOf(stat)]
                         ?: unit.civ.gameInfo.speed.modifier
                     unique.placeholderText.fillPlaceholders(
-                        (unique.params[0].toInt() * modifier).toInt().toString(),
-                        (unique.params[1].toInt() * modifier).toInt().toString(),
+                        (unique.params[0].toInt() * modifier).toInt().tr(),
+                        (unique.params[1].toInt() * modifier).toInt().tr(),
                         stat
                     )
                 }
+                UniqueType.TriggerEvent -> unique.params[0]
                 else -> unique.text.removeConditionals()
             }
             val title = UnitActionModifiers.actionTextWithSideEffects(baseTitle, unique, unit)
 
             val unitAction = fun (): (()->Unit)? {
-                if (unit.currentMovement == 0f) return null
+                if (!unit.hasMovement()) return null
                 val triggerFunction = UniqueTriggerActivation.getTriggerFunction(unique, unit.civ, unit = unit, tile = unit.currentTile)
                     ?: return null
                 return { // This is the *action* that will be triggered!
-                    triggerFunction.invoke()
+                    repeat(unique.getUniqueMultiplier(unit.cache.state)) {
+                        triggerFunction.invoke()
+                    }
                     UnitActionModifiers.activateSideEffects(unit, unique)
                 }
             }()
 
             yield(
                 UnitAction(UnitActionType.TriggerUnique, 80f, title,
+                    associatedUnique = unique,
                     action = unitAction.takeIf {
                         UnitActionModifiers.canActivateSideEffects(unit, unique)
                     })
@@ -246,17 +301,17 @@ object UnitActionsFromUniques {
     }
 
     private fun getWaterImprovementAction(unit: MapUnit, tile: Tile): UnitAction? {
-        if (!tile.isWater || !unit.hasUnique(UniqueType.CreateWaterImprovements) || tile.resource == null) return null
+        if (!tile.isWater || !unit.hasUnique(UniqueType.CreateWaterImprovements)) return null
 
-        val improvementName = tile.tileResource.getImprovingImprovement(tile, unit.civ) ?: return null
+        val improvementName = tile.tileResource?.getImprovingImprovement(tile, unit.cache.state) ?: return null
         val improvement = tile.ruleset.tileImprovements[improvementName] ?: return null
-        if (!tile.improvementFunctions.canBuildImprovement(improvement, unit.civ)) return null
+        if (!tile.improvementFunctions.canBuildImprovement(improvement, unit.cache.state)) return null
 
         return UnitAction(UnitActionType.CreateImprovement, 82f, "Create [$improvementName]",
             action = {
-                tile.changeImprovement(improvementName, unit.civ, unit)
+                tile.setImprovement(improvementName, unit.civ, unit)
                 unit.destroy()  // Modders may wish for a nondestructive way, but that should be another Unique
-            }.takeIf { unit.currentMovement > 0 })
+            }.takeIf { unit.hasMovement() })
     }
 
     // Not internal: Used in SpecificUnitAutomation
@@ -264,15 +319,16 @@ object UnitActionsFromUniques {
         val uniquesToCheck = UnitActionModifiers.getUsableUnitActionUniques(unit, UniqueType.ConstructImprovementInstantly)
 
         val civResources = unit.civ.getCivResourcesByName()
+        val gameContext = GameContext(civInfo = unit.civ, unit = unit, tile = tile)
 
         for (unique in uniquesToCheck) {
             val improvementFilter = unique.params[0]
-            val improvements = tile.ruleset.tileImprovements.values.filter { it.matchesFilter(improvementFilter) }
+            val improvements = tile.ruleset.tileImprovements.values.filter { it.matchesFilter(improvementFilter, gameContext) }
 
             for (improvement in improvements) {
                 // Try to skip Improvements we can never build
                 // (getImprovementBuildingProblems catches those so the button is always disabled, but it nevertheless looks nicer)
-                if (tile.improvementFunctions.getImprovementBuildingProblems(improvement, unit.civ).any { it.permanent })
+                if (tile.improvementFunctions.getImprovementBuildingProblems(improvement, gameContext).any { it.permanent })
                     continue
 
                 val resourcesAvailable = improvement.getMatchingUniques(UniqueType.ConsumesResources).none { improvementUnique ->
@@ -285,17 +341,18 @@ object UnitActionsFromUniques {
                         unique,
                         unit
                     ),
+                    associatedUnique = unique,
                     action = {
                         val unitTile = unit.getTile()
-                        unitTile.changeImprovement(improvement.name, unit.civ, unit)
+                        unitTile.setImprovement(improvement.name, unit.civ, unit)
 
                         unit.civ.cache.updateViewableTiles() // to update 'last seen improvement'
 
                         UnitActionModifiers.activateSideEffects(unit, unique)
                     }.takeIf {
                         resourcesAvailable
-                            && unit.currentMovement > 0f
-                            && tile.improvementFunctions.canBuildImprovement(improvement, unit.civ)
+                            && unit.hasMovement()
+                            && tile.improvementFunctions.canBuildImprovement(improvement, unit.cache.state)
                             // Next test is to prevent interfering with UniqueType.CreatesOneImprovement -
                             // not pretty, but users *can* remove the building from the city queue an thus clear this:
                             && !tile.isMarkedForCreatesOneImprovement()
@@ -336,15 +393,14 @@ object UnitActionsFromUniques {
     internal fun getTransformActions(unit: MapUnit, tile: Tile) = sequence {
         val unitTile = unit.getTile()
         val civInfo = unit.civ
-        val stateForConditionals =
-            StateForConditionals(unit = unit, civInfo = civInfo, tile = unitTile)
+        val stateForConditionals = unit.cache.state
 
         for (unique in unit.getMatchingUniques(UniqueType.CanTransform, stateForConditionals)) {
             val unitToTransformTo = civInfo.getEquivalentUnit(unique.params[0])
 
             // Respect OnlyAvailable criteria
             if (unitToTransformTo.getMatchingUniques(
-                    UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals
+                    UniqueType.OnlyAvailable, GameContext.IgnoreConditionals
                 ).any { !it.conditionalsApply(stateForConditionals) }
             ) continue
 
@@ -353,7 +409,7 @@ object UnitActionsFromUniques {
             val resourceRequirementsDelta = Counter<String>()
             for ((resource, amount) in unit.getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, -amount)
-            for ((resource, amount) in unitToTransformTo.getResourceRequirementsPerTurn(StateForConditionals(unit.civ, unit = unit)))
+            for ((resource, amount) in unitToTransformTo.getResourceRequirementsPerTurn(unit.cache.state))
                 resourceRequirementsDelta.add(resource, amount)
             val newResourceRequirementsString = resourceRequirementsDelta.entries
                 .filter { it.value > 0 }
@@ -366,21 +422,21 @@ object UnitActionsFromUniques {
 
             yield(UnitAction(UnitActionType.Transform, 70f,
                 title = title,
+                associatedUnique = unique,
                 action = {
                     val oldMovement = unit.currentMovement
                     unit.destroy()
                     val newUnit =
-                        civInfo.units.placeUnitNearTile(unitTile.position, unitToTransformTo)
+                        civInfo.units.placeUnitNearTile(unitTile.position, unitToTransformTo, unit.id, copiedFrom = unit)
 
                     /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
                      * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
                      */
                     if (newUnit == null) {
                         val resurrectedUnit =
-                            civInfo.units.placeUnitNearTile(unitTile.position, unit.baseUnit)!!
-                        unit.copyStatisticsTo(resurrectedUnit)
+                            civInfo.units.placeUnitNearTile(unitTile.position, unit.baseUnit, unit.id, copiedFrom = unit)!!
+                        
                     } else { // Managed to upgrade
-                        unit.copyStatisticsTo(newUnit)
                         // have to handle movement manually because we killed the old unit
                         // a .destroy() unit has 0 movement
                         // and a new one may have less Max Movement
@@ -401,13 +457,13 @@ object UnitActionsFromUniques {
     internal fun getBuildingImprovementsActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
         if (!unit.cache.hasUniqueToBuildImprovements) return emptySequence()
 
-        val couldConstruct = unit.currentMovement > 0
+        val couldConstruct = unit.hasMovement()
             && !tile.isCityCenter()
             && unit.civ.gameInfo.ruleset.tileImprovements.values.any {
             ImprovementPickerScreen.canReport(
                 tile.improvementFunctions.getImprovementBuildingProblems(
                     it,
-                    unit.civ
+                    unit.cache.state
                 ).toSet()
             )
                 && unit.canBuildImprovement(it)
@@ -424,6 +480,7 @@ object UnitActionsFromUniques {
         ))
     }
 
+    @Readonly
     internal fun getRepairTurns(unit: MapUnit): Int {
         val tile = unit.currentTile
         if (!tile.isPillaged()) return 0
@@ -436,7 +493,8 @@ object UnitActionsFromUniques {
         return repairTurns.coerceAtMost(turnsToBuild)
     }
 
-    internal fun getRepairActions(unit: MapUnit, tile: Tile) = sequenceOf(getRepairAction(unit)).filterNotNull()
+    internal fun getRepairActions(unit: MapUnit, tile: Tile) =
+        sequenceOf(getRepairAction(unit)).filterNotNull()
 
     // Public - used in WorkerAutomation
     fun getRepairAction(unit: MapUnit) : UnitAction? {
@@ -447,17 +505,20 @@ object UnitActionsFromUniques {
         if (tile.isCityCenter()) return null
         if (!tile.isPillaged()) return null
 
-        val couldConstruct = unit.currentMovement > 0
+        val couldConstruct = unit.hasMovement()
             && !tile.isCityCenter() && tile.improvementInProgress != Constants.repair
             && !tile.isEnemyTerritory(unit.civ)
+                // Are there any other improvement building problems that should block repair?
+            && ImprovementFunctions.getImprovementBuildingProblems(unit.currentTile.getImprovementToRepair()!!, GameContext(civInfo = unit.civ, unit = unit, tile = tile))
+                .none { it == ImprovementBuildingProblem.OutsideBorders }
 
         val turnsToBuild = getRepairTurns(unit)
 
         return UnitAction(UnitActionType.Repair, 90f,
             title = "${UnitActionType.Repair} [${unit.currentTile.getImprovementToRepair()!!.name}] - [${turnsToBuild}${Fonts.turn}]",
             action = {
-                tile.turnsToImprovement = getRepairTurns(unit)
-                tile.improvementInProgress = Constants.repair
+                tile.queueImprovement(Constants.repair, turnsToBuild)
+                unit.action = null
             }.takeIf { couldConstruct }
         )
     }
