@@ -1,18 +1,18 @@
 package com.unciv.ui.screens.worldscreen.unit.actions
 
 import com.unciv.GUI
-import com.unciv.UncivGame
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitAction
 import com.unciv.models.UnitActionType
-import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
+import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.popups.ConfirmPopup
+import yairm210.purity.annotations.Readonly
 import kotlin.random.Random
 
 object UnitActionsPillage {
@@ -20,7 +20,7 @@ object UnitActionsPillage {
     internal fun getPillageActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
         val pillageAction = getPillageAction(unit, tile)
             ?: return emptySequence()
-        if (pillageAction.action == null || unit.civ.isAI() || (unit.civ.isHuman() && UncivGame.Current.settings.autoPlay.isAutoPlaying()))
+        if (pillageAction.action == null || unit.civ.isAIOrAutoPlaying())
             return sequenceOf(pillageAction)
         else return sequenceOf(UnitAction(UnitActionType.Pillage, 65f, pillageAction.title) {
             val pillageText = "Are you sure you want to pillage this [${tile.getImprovementToPillageName()!!}]?"
@@ -63,9 +63,19 @@ object UnitActionsPillage {
                 val freePillage = unit.hasUnique(UniqueType.NoMovementToPillage, checkCivInfoUniques = true)
                 if (!freePillage) unit.useMovementPoints(1f)
 
-                if (pillagingImprovement)  // only Improvements heal HP
-                    unit.healBy(25)
-            }.takeIf { unit.currentMovement > 0 && canPillage(unit, tile) }
+                if (pillagingImprovement) { // only Improvements heal HP
+                    var healAmount = 25f
+                    for (unique in unit.getMatchingUniques(UniqueType.PercentHealthFromPillaging, checkCivInfoUniques = true)) {
+                        healAmount *= unique.params[0].toPercent()
+                    }
+                    unit.healBy(healAmount.toInt())
+                }
+                
+                if (tile.getImprovementToPillage()?.hasUnique(UniqueType.DestroyedWhenPillaged) == true) {
+                    tile.removeImprovement()
+                }
+                    
+            }.takeIf { unit.hasMovement() && canPillage(unit, tile) }
         )
     }
 
@@ -74,17 +84,33 @@ object UnitActionsPillage {
         val improvement = tile.getImprovementToPillage()!!
 
         // Accumulate the loot
-        val pillageYield = Stats()
-        val stateForConditionals = StateForConditionals(unit=unit)
+        var pillageYield = Stats()
+        val stateForConditionals = unit.cache.state
+        val random = Random(unit.civ.gameInfo.turns * unit.getTile().position.hashCode().toLong())
         for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldRandom, stateForConditionals)) {
             for ((stat, value) in unique.stats) {
+                var yieldsToAdd = Stats()
                 // Unique text says "approximately [X]", so we add 0..X twice - think an RPG's 2d12
-                val looted = Random.nextInt((value + 1).toInt()) + Random.nextInt((value + 1).toInt())
-                pillageYield.add(stat, looted.toFloat())
+                yieldsToAdd.add(stat, (random.nextInt((value + 1).toInt()) + random.nextInt((value + 1).toInt()).toFloat()))
+                if (unique.isModifiedByGameSpeed())
+                    yieldsToAdd *= unit.civ.gameInfo.speed.modifier
+                if (unique.isModifiedByGameProgress())
+                    yieldsToAdd *= unique.getGameProgressModifier(unit.civ)
+                pillageYield.add(yieldsToAdd)
             }
         }
         for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldFixed, stateForConditionals)) {
-            pillageYield.add(unique.stats)
+            var yieldsToAdd = unique.stats
+            if (unique.isModifiedByGameSpeed())
+                yieldsToAdd *= unit.civ.gameInfo.speed.modifier
+            if (unique.isModifiedByGameProgress())
+                yieldsToAdd *= unique.getGameProgressModifier(unit.civ)
+            pillageYield.add(yieldsToAdd)
+        }
+
+        //Multiply according to uniques
+        for (unique in unit.getMatchingUniques(UniqueType.PercentYieldFromPillaging, checkCivInfoUniques = true)) {
+            pillageYield *= unique.params[0].toPercent()
         }
 
         // Please no notification when there's no loot
@@ -115,10 +141,11 @@ object UnitActionsPillage {
     }
 
     // Public - used in UnitAutomation
+    @Readonly
     fun canPillage(unit: MapUnit, tile: Tile): Boolean {
         if (unit.isTransported) return false
         if (!tile.canPillageTile()) return false
-        if (unit.hasUnique(UniqueType.CannotPillage)) return false
+        if (unit.hasUnique(UniqueType.CannotPillage, checkCivInfoUniques = true)) return false
         val tileOwner = tile.getOwner()
         // Can't pillage friendly tiles, just like you can't attack them - it's an 'act of war' thing
         return tileOwner == null || unit.civ.isAtWarWith(tileOwner)
