@@ -1,7 +1,9 @@
 package com.unciv.ui.screens.worldscreen.unit.actions
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.math.Interpolation
 import com.badlogic.gdx.scenes.scene2d.Touchable
+import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.ui.Button
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.unciv.UncivGame
@@ -9,6 +11,7 @@ import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.models.UnitAction
 import com.unciv.models.UnitActionType
 import com.unciv.models.UpgradeUnitAction
+import com.unciv.ui.components.extensions.brighten
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
@@ -16,6 +19,7 @@ import com.unciv.ui.components.input.onRightClick
 import com.unciv.ui.images.IconTextButton
 import com.unciv.ui.popups.UnitUpgradeMenu
 import com.unciv.ui.screens.worldscreen.WorldScreen
+import yairm210.purity.annotations.Readonly
 
 class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
     /** Distribute UnitActions on "pages" */
@@ -23,6 +27,7 @@ class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
     private var buttonsPerPage = Int.MAX_VALUE
     private var numPages = 2
     private var shownForUnitHash = 0
+    private var animationNotFinished = false
 
     companion object {
         /** Maximum for how many pages there can be. ([minButtonsPerPage]-1)*[maxAllowedPages]
@@ -53,17 +58,22 @@ class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
 
     fun update(unit: MapUnit?) {
         val newUnitHash = unit?.hashCode() ?: 0
+        var unitChanged = false
         if (shownForUnitHash != newUnitHash) {
+            unitChanged = true
             currentPage = 0
             shownForUnitHash = newUnitHash
         }
 
         clear()
+        keyShortcuts.clear()
         if (unit == null) return
         if (!worldScreen.canChangeState) return // No actions when it's not your turn or spectator!
 
         numPages = 0
         val pageActionBuckets = Array<ArrayDeque<UnitAction>>(maxAllowedPages) { ArrayDeque() }
+        
+        @Readonly
         fun freeSlotsOnPage(page: Int) = buttonsPerPage -
             pageActionBuckets[page].size -
             (if (numPages > 1) 1 else 0) // room for the navigation buttons
@@ -123,10 +133,22 @@ class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
             add(previousPageButton)
         if (currentPage < numPages - 1)
             add(nextPageButton)
+        
+        // Animate buttons
+        if (UncivGame.Current.settings.unitActionsTableAnimation) {
+            if (unitChanged || animationNotFinished) {
+                val buttons = children.filterIsInstance<Button>()
+                var delay = 0f
+                for (button in buttons) {
+                    animateButton(button, delay)
+                    delay += 0.05f
+                }
+            }
+        }
+
         pack()
 
         // Bind all currently invisible actions to their keys
-        keyShortcuts.clear()
         for (page in pageActionBuckets.indices) {
             if (page == currentPage) continue // these are already done
             for (unitAction in pageActionBuckets[page]) {
@@ -153,9 +175,10 @@ class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
 
         val fontColor = if (unitAction.isCurrentAction) Color.YELLOW else Color.WHITE
         val actionButton = IconTextButton(unitAction.title, icon, fontColor = fontColor)
+        actionButton.labelCell.padTop(0f) // aligned with icon 
 
         if (unitAction.type == UnitActionType.Promote && unitAction.action != null)
-            actionButton.color = Color.GREEN.cpy().lerp(Color.WHITE, 0.5f)
+            actionButton.color = Color.GREEN.brighten(0.5f)
 
         actionButton.pack()
 
@@ -178,7 +201,22 @@ class UnitActionsTable(val worldScreen: WorldScreen) : Table() {
         // overlay, since the user definitely wants to interact with the new unit.
         worldScreen.mapHolder.removeUnitActionOverlay()
         if (!UncivGame.Current.settings.autoUnitCycle) return
-        if (unit.isDestroyed || unitAction.type.isSkippingToNextUnit && (unit.isMoving() && unit.currentMovement == 0f || !unit.isMoving()))
+        if (unit.isDestroyed || 
+            unitAction.type.isSkippingToNextUnit && (!unit.isMoving() || !unit.hasMovement()))
             worldScreen.switchToNextUnit()
+        else worldScreen.bottomUnitTable.shouldUpdate = true
+    }
+
+    private fun animateButton(button: Button, delay: Float) {
+        button.color.a = 0f
+        button.addAction(Actions.sequence(
+            Actions.run { animationNotFinished = true },
+            Actions.run {
+                button.addAction(Actions.moveBy(-150f, 0f))
+                button.addAction(Actions.alpha(1f, 0.15f))
+                button.addAction(Actions.delay(delay, Actions.moveBy(150f, 0f, 0.15f, Interpolation.smooth)))
+            },
+            Actions.run { animationNotFinished = false }
+        ))
     }
 }
