@@ -2,9 +2,10 @@ package com.unciv.app.desktop
 
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.backends.lwjgl3.Lwjgl3ApplicationConfiguration
+import com.sun.jna.platform.win32.Kernel32Util
 import com.unciv.UncivGame
 
-class DesktopGame(config: Lwjgl3ApplicationConfiguration) : UncivGame() {
+class DesktopGame(config: Lwjgl3ApplicationConfiguration, override var customDataDirectory: String?) : UncivGame() {
 
     private var discordUpdater = DiscordUpdater()
     private val windowListener = UncivWindowListener()
@@ -49,4 +50,13 @@ class DesktopGame(config: Lwjgl3ApplicationConfiguration) : UncivGame() {
         super.dispose()
     }
 
+    override fun getSystemErrorMessage(errorCode: Int): String? {
+        return try {
+            if (System.getProperty("os.name")?.contains("Windows") == true)
+                Kernel32Util.formatMessage(errorCode)
+            else null
+        } catch (_: Throwable) {
+            null
+        }
+    }
 }
