@@ -41,7 +41,7 @@ internal class WorkerAutomationTest {
 
         val currentTile = testGame.tileMap[1,1] // owned by city
         currentTile.improvement = "Farm" // Set existing improvement
-        currentTile.resource = "Iron" // This tile also has a resource needs to be enabled by a building a Mine
+        currentTile.setTileResource("Iron") // This tile also has a resource needs to be enabled by a building a Mine
 
         val mapUnit = testGame.addUnit("Worker", civInfo, currentTile)
 
@@ -50,7 +50,8 @@ internal class WorkerAutomationTest {
 
         // Assert
         assertEquals("Worker should have replaced already existing improvement 'Farm' with 'Mine' to enable 'Iron' resource",
-            "Mine", currentTile.improvementInProgress)
+            "Mine", currentTile.improvementInProgress
+        )
         assertTrue(currentTile.turnsToImprovement > 0)
     }
 
@@ -66,7 +67,7 @@ internal class WorkerAutomationTest {
         val currentTile = testGame.tileMap[1,1]
         currentTile.addTerrainFeature("Hill")
         currentTile.addTerrainFeature("Forest")
-        currentTile.resource = "Citrus"
+        currentTile.setTileResource("Citrus")
         currentTile.resourceAmount = 1
 
         val mapUnit = testGame.addUnit("Worker", civInfo, currentTile)
@@ -74,7 +75,8 @@ internal class WorkerAutomationTest {
         workerAutomation.automateWorkerAction(mapUnit, hashSetOf())
 
         assertEquals("Worker should begun removing the forest to clear a luxury resource but didn't",
-            "Remove Forest", currentTile.improvementInProgress)
+            "Remove Forest", currentTile.improvementInProgress
+        )
         assertTrue(currentTile.turnsToImprovement > 0)
     }
 
@@ -85,11 +87,14 @@ internal class WorkerAutomationTest {
             civInfo.tech.techsResearched.add(testGame.ruleset.tileImprovements[improvement]!!.techRequired!!)
         }
 
-        testGame.addCity(civInfo, testGame.tileMap[0,0])
-
         val currentTile = testGame.tileMap[1,1]
+        val city = testGame.addCity(civInfo, testGame.tileMap[0,0])
+        // Currently worked tile is prioritized for worker actions
+        city.workedTiles.clear()
+        city.workedTiles.add(currentTile.position)
+
         currentTile.baseTerrain = Constants.grassland
-        currentTile.resource = "Iron"
+        currentTile.setTileResource("Iron")
 
         val mapUnit = testGame.addUnit("Worker", civInfo, currentTile)
 
@@ -98,7 +103,8 @@ internal class WorkerAutomationTest {
 
         // Assert
         assertEquals("Worker should be buliding a farm under 'Iron' resource because it can't see it and has nothing else to do",
-            "Farm", currentTile.improvementInProgress)
+            "Farm", currentTile.improvementInProgress
+        )
         assertTrue(currentTile.turnsToImprovement > 0)
     }
     
@@ -117,14 +123,15 @@ internal class WorkerAutomationTest {
         val currentTile = testGame.tileMap[1,1]
         currentTile.baseTerrain = Constants.grassland
         currentTile.addTerrainFeature(Constants.hill)
-        currentTile.resource = "Gold Ore"
+        currentTile.setTileResource("Gold Ore")
 
         val mapUnit = testGame.addUnit("Worker", civInfo, currentTile)
 
         workerAutomation.automateWorkerAction(mapUnit, hashSetOf())
 
         assertEquals("Worker should be buliding a mine on the Gold Ore luxury resource",
-            "Mine", currentTile.improvementInProgress)
+            "Mine", currentTile.improvementInProgress
+        )
         assertTrue(currentTile.turnsToImprovement > 0)
     }
 
@@ -329,8 +336,8 @@ internal class WorkerAutomationTest {
         testGame.addCity(civInfo, testGame.tileMap[0,0])
 
         val currentTile = testGame.tileMap[1,1] // owned by city
-        currentTile.resource = "Iron" // This tile also has a resource needs to be enabled by a building a Mine
-        currentTile.changeImprovement("Mine")
+        currentTile.setTileResource("Iron") // This tile also has a resource needs to be enabled by a building a Mine
+        currentTile.setImprovement("Mine")
         currentTile.setPillaged()
 
         val mapUnit = testGame.addUnit("Worker", civInfo, currentTile)
@@ -340,7 +347,8 @@ internal class WorkerAutomationTest {
 
         // Assert
         assertEquals("Worker should try to repair the mine",
-            "Repair", currentTile.improvementInProgress)
+            "Repair", currentTile.improvementInProgress
+        )
         assertTrue(currentTile.turnsToImprovement > 0)
     }
 
