@@ -1,8 +1,8 @@
 package com.unciv.logic.battle
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.map.HexCoord
 import com.unciv.testing.GdxTestRunner
 import com.unciv.testing.TestGame
 import org.junit.Assert.assertEquals
@@ -34,15 +34,15 @@ class TargetHelperTest {
     @Test
     fun `should retrieve city bombardable tiles`() {
         // given
-        val attackerCity = testGame.addCity(attackerCiv, testGame.getTile(Vector2.Zero))
+        val attackerCity = testGame.addCity(attackerCiv, testGame.getTile(HexCoord.Zero))
 
-        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(Vector2.Zero)) // own unit in own city, not bombardable
-        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(Vector2(1f, 0f))) // own unit in own city range, not bombardable
-        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(Vector2(3f, 3f))) // own unit outside city range, not bombardable
-        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(Vector2(1f, 1f))) // enemy unit inside city range, bombardable
-        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(Vector2(-3f, -3f))) // enemy unit outside city range, not bombardable
+        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(HexCoord.Zero)) // own unit in own city, not bombardable
+        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(1,0)) // own unit in own city range, not bombardable
+        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(3,3)) // own unit outside city range, not bombardable
+        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(1,1)) // enemy unit inside city range, bombardable
+        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(-3,-3)) // enemy unit outside city range, not bombardable
 
-        val tile = testGame.setTileTerrain(Vector2(-2f, -1f), Constants.coast)
+        val tile = testGame.setTileTerrain(HexCoord(-2, -1), Constants.coast)
         testGame.addTileToCity(attackerCity, tile)
         testGame.addUnit("Submarine", defenderCiv, tile) // enemy unit inside city range, invisible, not bombardable
 
@@ -51,14 +51,14 @@ class TargetHelperTest {
 
         // then
         assertEquals(1, bombardableTiles.toList().size)
-        assertTrue(bombardableTiles.contains(testGame.getTile(Vector2(1f, 1f))))
+        assertTrue(bombardableTiles.contains(testGame.getTile(1,1)))
     }
 
     @Test
     fun `should get attackable tile when melee next to enemy unit`() {
         // given
-        val attackFromTile = testGame.getTile(Vector2.Zero)
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackFromTile = testGame.getTile(HexCoord.Zero)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Warrior", attackerCiv, attackFromTile)
         attackerUnit.currentMovement = defaultWarriorMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -77,8 +77,8 @@ class TargetHelperTest {
     @Test
     fun `should get no attackable tiles when unit next to allied unit`() {
         // given
-        val attackFromTile = testGame.getTile(Vector2.Zero)
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackFromTile = testGame.getTile(HexCoord.Zero)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Warrior", attackerCiv, attackFromTile)
         attackerUnit.currentMovement = defaultWarriorMovePoints
         testGame.addUnit("Warrior", attackerCiv, attackedTile)
@@ -93,8 +93,8 @@ class TargetHelperTest {
     @Test
     fun `should get attackable tile when melee enemy reachable`() {
         // given
-        val attackFromTile = testGame.getTile(Vector2.Zero)
-        val attackedTile = testGame.getTile(Vector2(2f, 0f))  // move one tile -> melee next to each other -> attack
+        val attackFromTile = testGame.getTile(HexCoord.Zero)
+        val attackedTile = testGame.getTile(2,0)  // move one tile -> melee next to each other -> attack
         val attackerUnit = testGame.addUnit("Warrior", attackerCiv, attackFromTile)
         attackerUnit.currentMovement = defaultWarriorMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -105,7 +105,7 @@ class TargetHelperTest {
         // then
         assertEquals(1, attackableEnemies.toList().size)
         val attackableEnemy = attackableEnemies[0]
-        assertEquals(testGame.getTile(Vector2(1f, 0f)), attackableEnemy.tileToAttackFrom) // the attacker unit has moved...
+        assertEquals(testGame.getTile(1,0), attackableEnemy.tileToAttackFrom) // the attacker unit has moved...
         assertEquals(attackedTile, attackableEnemy.tileToAttack)
         assertEquals(defaultWarriorMovePoints - 1, attackableEnemy.movementLeftAfterMovingToAttackTile) // ...and thus when attacking has one fewer move point
     }
@@ -113,8 +113,8 @@ class TargetHelperTest {
     @Test
     fun `should get no attackable tiles when melee enemy out of reach`() {
         // given
-        val attackFromTile = testGame.getTile(Vector2.Zero)
-        val attackedTile = testGame.getTile(Vector2(3f, 0f))  // out of reach
+        val attackFromTile = testGame.getTile(HexCoord.Zero)
+        val attackedTile = testGame.getTile(3,0)  // out of reach
         val attackerUnit = testGame.addUnit("Warrior", attackerCiv, attackFromTile)
         attackerUnit.currentMovement = defaultWarriorMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -129,8 +129,8 @@ class TargetHelperTest {
     @Test
     fun `should get attackable tiles when ranged next to melee enemy unit with ZOC`() {
         // given
-        val attackerTile = testGame.getTile(Vector2.Zero)
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackerTile = testGame.getTile(HexCoord.Zero)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Archer", attackerCiv, attackerTile)
         attackerUnit.currentMovement = defaultArcherMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -163,15 +163,15 @@ class TargetHelperTest {
         assertEquals(attackedTile, attackableEnemyWithoutMoving.tileToAttack)
         assertEquals(defaultArcherMovePoints, attackableEnemyWithoutMoving.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingDown = attackableEnemies[1]
-        assertEquals(testGame.getTile(Vector2(-1f, -1f)), attackableEnemyMovingDown.tileToAttackFrom)
+        assertEquals(testGame.getTile(-1,-1), attackableEnemyMovingDown.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingDown.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingDown.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingUpRigth = attackableEnemies[2]
-        assertEquals(testGame.getTile(Vector2(0f, 1f)), attackableEnemyMovingUpRigth.tileToAttackFrom)
+        assertEquals(testGame.getTile(0,1), attackableEnemyMovingUpRigth.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingUpRigth.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingUpRigth.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingDownLeft = attackableEnemies[3]
-        assertEquals(testGame.getTile(Vector2(-1f, 0f)), attackableEnemyMovingDownLeft.tileToAttackFrom)
+        assertEquals(testGame.getTile(-1,0), attackableEnemyMovingDownLeft.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingDownLeft.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingDownLeft.movementLeftAfterMovingToAttackTile)
     }
@@ -179,8 +179,8 @@ class TargetHelperTest {
     @Test
     fun `should get attackable tiles when ranged already in range to enemy unit`() {
         // given
-        val attackerTile = testGame.getTile(Vector2(-1f, 0f))
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackerTile = testGame.getTile(-1,0)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Archer", attackerCiv, attackerTile)
         attackerUnit.currentMovement = defaultArcherMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -213,15 +213,15 @@ class TargetHelperTest {
         assertEquals(attackedTile, attackableEnemyWithoutMoving.tileToAttack)
         assertEquals(defaultArcherMovePoints, attackableEnemyWithoutMoving.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingUp = attackableEnemies[1]
-        assertEquals(testGame.getTile(Vector2(0f, 1f)), attackableEnemyMovingUp.tileToAttackFrom)
+        assertEquals(testGame.getTile(0,1), attackableEnemyMovingUp.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingUp.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingUp.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingDownLeft = attackableEnemies[2]
-        assertEquals(testGame.getTile(Vector2(-1f, -1f)), attackableEnemyMovingDownLeft.tileToAttackFrom)
+        assertEquals(testGame.getTile(-1,-1), attackableEnemyMovingDownLeft.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingDownLeft.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingDownLeft.movementLeftAfterMovingToAttackTile)
         val attackableEnemyMovingUpLeft = attackableEnemies[3]
-        assertEquals(testGame.getTile(Vector2.Zero), attackableEnemyMovingUpLeft.tileToAttackFrom)
+        assertEquals(testGame.getTile(HexCoord.Zero), attackableEnemyMovingUpLeft.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingUpLeft.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingUpLeft.movementLeftAfterMovingToAttackTile)
     }
@@ -229,10 +229,10 @@ class TargetHelperTest {
     @Test
     fun `should get attackable tiles when ranged movable in range to enemy unit`() {
         // given
-        val attackerTile = testGame.getTile(Vector2(-2f, 0f))
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackerTile = testGame.getTile(-2,0)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Archer", attackerCiv, attackerTile)
-        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(Vector2(0f, 1f))) // otherwise the archer cannot see the enemy warrior
+        testGame.addUnit("Warrior", attackerCiv, testGame.getTile(0,1)) // otherwise the archer cannot see the enemy warrior
         attackerUnit.currentMovement = defaultArcherMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
 
@@ -260,7 +260,7 @@ class TargetHelperTest {
         // then
         assertEquals(1, attackableEnemies.toList().size)
         val attackableEnemyMovingThenFiring = attackableEnemies[0]
-        assertEquals(testGame.getTile(Vector2(-1f, 0f)), attackableEnemyMovingThenFiring.tileToAttackFrom)
+        assertEquals(testGame.getTile(-1,0), attackableEnemyMovingThenFiring.tileToAttackFrom)
         assertEquals(attackedTile, attackableEnemyMovingThenFiring.tileToAttack)
         assertEquals(defaultArcherMovePoints - 1, attackableEnemyMovingThenFiring.movementLeftAfterMovingToAttackTile)
     }
@@ -268,8 +268,8 @@ class TargetHelperTest {
     @Test
     fun `should get no attackable tiles when enemy unit is out of sight`() {
         // given
-        val attackerTile = testGame.getTile(Vector2(-2f, 0f))
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackerTile = testGame.getTile(-2,0)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Archer", attackerCiv, attackerTile) // two title radius sight -> cannot see enemy warrior
         attackerUnit.currentMovement = defaultArcherMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -302,12 +302,12 @@ class TargetHelperTest {
     @Test
     fun `should get no attackable tiles when terrain obstacle`() {
         // given
-        testGame.setTileFeatures(Vector2.Zero, Constants.hill)
-        testGame.setTileFeatures(Vector2(1f, 1f), Constants.hill)
-        testGame.setTileFeatures(Vector2(0f, -1f), Constants.hill)
+        testGame.setTileFeatures(HexCoord.Zero, Constants.hill)
+        testGame.setTileFeatures(HexCoord(1, 1), Constants.hill)
+        testGame.setTileFeatures(HexCoord(0, -1), Constants.hill)
 
-        val attackerTile = testGame.getTile(Vector2(-1f, 0f))
-        val attackedTile = testGame.getTile(Vector2(1f, 0f))
+        val attackerTile = testGame.getTile(-1,0)
+        val attackedTile = testGame.getTile(1,0)
         val attackerUnit = testGame.addUnit("Archer", attackerCiv, attackerTile)
         attackerUnit.currentMovement = defaultArcherMovePoints
         testGame.addUnit("Warrior", defenderCiv, attackedTile)
@@ -340,9 +340,9 @@ class TargetHelperTest {
     @Test
     fun `should get no attackable tiles when has cannot attack unique`() {
         // given
-        val attackerUnit = testGame.addDefaultMeleeUnitWithUniques(attackerCiv, testGame.getTile(Vector2.Zero), "Cannot attack")
+        val attackerUnit = testGame.addDefaultMeleeUnitWithUniques(attackerCiv, testGame.getTile(HexCoord.Zero), "Cannot attack")
         attackerUnit.currentMovement = 2f
-        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(Vector2.X))
+        testGame.addUnit("Warrior", defenderCiv, testGame.getTile(1,0))
 
         // when
         val attackableEnemies = TargetHelper.getAttackableEnemies(attackerUnit, attackerUnit.movement.getDistanceToTiles())
