@@ -1,15 +1,13 @@
 package com.unciv.logic.city.managers
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.logic.city.City
 import com.unciv.logic.city.CityFocus
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.map.HexCoord
 import com.unciv.testing.GdxTestRunner
 import com.unciv.testing.TestGame
-import org.junit.Assert.assertEquals
-import org.junit.Assert.assertFalse
-import org.junit.Assert.assertTrue
+import org.junit.Assert.*
 import org.junit.Before
 import org.junit.Test
 import org.junit.runner.RunWith
@@ -26,13 +24,13 @@ class CityPopulationManagerTest {
     fun setUp() {
         testGame.makeHexagonalMap(3)
         civ = testGame.addCiv()
-        city = testGame.addCity(civ, testGame.getTile(Vector2.Zero), initialPopulation = 1)
+        city = testGame.addCity(civ, testGame.getTile(HexCoord.Zero), initialPopulation = 1)
     }
 
     @Test
     fun `should increase food requirements for next pop`() {
         // given
-        val biggerCity = testGame.addCity(civ, testGame.getTile(Vector2.X), initialPopulation = 2)
+        val biggerCity = testGame.addCity(civ, testGame.getTile(HexCoord(1,0)), initialPopulation = 2)
 
         // when
         val smallerCityFoodRequirements = city.population.getFoodToNextPopulation()
@@ -45,13 +43,11 @@ class CityPopulationManagerTest {
     @Test
     fun `should change food requirements for different gamespeeds`() {
         // given
-        val quickSpeedGame = TestGame()
-        quickSpeedGame.gameInfo.speed = quickSpeedGame.ruleset.speeds["Quick"]!!
-        val quickSpeedCity = quickSpeedGame.addCity(quickSpeedGame.addCiv(), quickSpeedGame.getTile(Vector2.Zero), initialPopulation = 1)
+        val quickSpeedGame = TestGame().apply { setSpeed("Quick") }
+        val quickSpeedCity = quickSpeedGame.addCity(quickSpeedGame.addCiv(), quickSpeedGame.getTile(HexCoord.Zero), initialPopulation = 1)
 
-        val epicSpeedGame = TestGame()
-        epicSpeedGame.gameInfo.speed = epicSpeedGame.ruleset.speeds["Epic"]!!
-        val epicSpeedCity = epicSpeedGame.addCity(epicSpeedGame.addCiv(), epicSpeedGame.getTile(Vector2.Zero), initialPopulation = 1)
+        val epicSpeedGame = TestGame().apply { setSpeed("Epic") }
+        val epicSpeedCity = epicSpeedGame.addCity(epicSpeedGame.addCiv(), epicSpeedGame.getTile(HexCoord.Zero), initialPopulation = 1)
 
         // when
         val quickFoodRequirements = quickSpeedCity.population.getFoodToNextPopulation()
@@ -67,7 +63,7 @@ class CityPopulationManagerTest {
     fun `should increase food requirements for city states`() {
         // given
         val cityState = testGame.addCiv(cityStateType = "Militaristic")
-        val cityStateCity = testGame.addCity(cityState, testGame.getTile(Vector2.X), initialPopulation = 1)
+        val cityStateCity = testGame.addCity(cityState, testGame.getTile(HexCoord(1,0)), initialPopulation = 1)
 
         // when
         val cityFoodRequirements = city.population.getFoodToNextPopulation()
@@ -80,10 +76,8 @@ class CityPopulationManagerTest {
     @Test
     fun `should increase food requirements for AI on easier difficulties`() {
         // given
-        val easierDifficultyGame = TestGame()
-        easierDifficultyGame.gameInfo.difficulty = "Chieftain"
-        easierDifficultyGame.gameInfo.difficultyObject = easierDifficultyGame.ruleset.difficulties["Chieftain"]!!
-        val easierCity = easierDifficultyGame.addCity(easierDifficultyGame.addCiv(), easierDifficultyGame.getTile(Vector2.Zero), initialPopulation = 1)
+        val easierDifficultyGame = TestGame().apply { setDifficulty("Chieftain") }
+        val easierCity = easierDifficultyGame.addCity(easierDifficultyGame.addCiv(), easierDifficultyGame.getTile(HexCoord.Zero), initialPopulation = 1)
 
         // when
         val cityFoodRequirements = city.population.getFoodToNextPopulation()
@@ -96,10 +90,8 @@ class CityPopulationManagerTest {
     @Test
     fun `should decrease food requirements for AI on higher difficulties`() {
         // given
-        val harderDifficultyGame = TestGame()
-        harderDifficultyGame.gameInfo.difficulty = "Deity"
-        harderDifficultyGame.gameInfo.difficultyObject = harderDifficultyGame.ruleset.difficulties["Deity"]!!
-        val harderCity = harderDifficultyGame.addCity(harderDifficultyGame.addCiv(), harderDifficultyGame.getTile(Vector2.Zero), initialPopulation = 1)
+        val harderDifficultyGame = TestGame().apply { setDifficulty("Deity") }
+        val harderCity = harderDifficultyGame.addCity(harderDifficultyGame.addCiv(), harderDifficultyGame.getTile(HexCoord.Zero), initialPopulation = 1)
 
         // when
         val cityFoodRequirements = city.population.getFoodToNextPopulation()
@@ -206,9 +198,9 @@ class CityPopulationManagerTest {
     fun `should automatically assign new pop to best job`() {
         // given
         city.workedTiles.clear()
-        city.workedTiles.add(Vector2(-1f, 0f))
-        city.lockedTiles.add(Vector2(-1f, 0f)) // force the first pop to work on a specific tile to avoid being reassigned
-        val goodTile = testGame.setTileTerrain(Vector2.X, Constants.grassland)
+        city.workedTiles.add(HexCoord(-1,0))
+        city.lockedTiles.add(HexCoord(-1,0)) // force the first pop to work on a specific tile to avoid being reassigned
+        val goodTile = testGame.setTileTerrain(HexCoord(1,0), Constants.grassland)
         goodTile.improvement = "Farm"
 
         assertFalse(city.workedTiles.contains(goodTile.position))
@@ -226,15 +218,15 @@ class CityPopulationManagerTest {
     @Test
     fun `should automatically assign new pop to best job according to city focus`() {
         // given
-        city.cityAIFocus = CityFocus.GoldFocus
+        city.setCityFocus(CityFocus.GoldFocus)
         city.workedTiles.clear()
-        city.workedTiles.add(Vector2(-1f, 0f))
-        city.lockedTiles.add(Vector2(-1f, 0f)) // force the first pop to work on a specific tile to avoid being reassigned
-        val goodFoodTile = testGame.setTileTerrain(Vector2.X, Constants.grassland)
+        city.workedTiles.add(HexCoord(-1,0))
+        city.lockedTiles.add(HexCoord(-1,0)) // force the first pop to work on a specific tile to avoid being reassigned
+        val goodFoodTile = testGame.setTileTerrain(HexCoord(1,0), Constants.grassland)
         goodFoodTile.improvement = "Farm"
         assertFalse(city.workedTiles.contains(goodFoodTile.position))
 
-        val goodGoldTile = testGame.setTileTerrain(Vector2.Y, Constants.grassland)
+        val goodGoldTile = testGame.setTileTerrain(HexCoord(0,1), Constants.grassland)
         val goldImprovement = testGame.createTileImprovement("[+5 Gold]")
         goodGoldTile.improvement = goldImprovement.name
         assertFalse(city.workedTiles.contains(goodGoldTile.position))
@@ -253,10 +245,10 @@ class CityPopulationManagerTest {
     @Test
     fun `should automatically assign new pop to best job with specialists`() {
         // given
-        city.cityAIFocus = CityFocus.GoldFocus
+        city.setCityFocus(CityFocus.GoldFocus)
         city.workedTiles.clear()
-        city.workedTiles.add(Vector2(-1f, 0f))
-        city.lockedTiles.add(Vector2(-1f, 0f)) // force the first pop to work on a specific tile to avoid being reassigned
+        city.workedTiles.add(HexCoord(-1,0))
+        city.lockedTiles.add(HexCoord(-1,0)) // force the first pop to work on a specific tile to avoid being reassigned
         val specialistBuilding = testGame.createBuilding()
         specialistBuilding.specialistSlots.add("Merchant", 1)
         city.cityConstructions.addBuilding(specialistBuilding)
