@@ -31,7 +31,7 @@ class CityMovingTests {
             civInfo.tech.addTechnology(tech)
 
         civInfo.diplomacyFunctions.makeCivilizationsMeet(enemy)
-        civInfo.getDiplomacyManager(enemy).declareWar()
+        civInfo.getDiplomacyManager(enemy)!!.declareWar()
     }
 
     @Test
@@ -115,13 +115,13 @@ class CityMovingTests {
     fun moveTheirCityToUsWhenTheyHaveResources() {
         val theirCapital = testGame.addCity(enemy, testGame.tileMap[2,0])
 
-        theirCapital.getCenterTile().resource = "Salt"
+        theirCapital.getCenterTile().setTileResource("Salt")
         theirCapital.getCenterTile().resourceAmount = 1
 
         val resourceTile = testGame.tileMap[0,1]
-        resourceTile.resource = "Iron"
+        resourceTile.setTileResource("Iron")
         resourceTile.resourceAmount = 3
-        resourceTile.changeImprovement("Mine")
+        resourceTile.setImprovement("Mine")
         theirCapital.expansion.takeOwnership(resourceTile)
 
         theirCapital.moveToCiv(civInfo)
