@@ -5,7 +5,7 @@ import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.tile.TerrainType
-import com.unciv.models.ruleset.unique.StateForConditionals
+import com.unciv.models.ruleset.unique.GameContext
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stats
 import com.unciv.testing.GdxTestRunner
@@ -44,22 +44,25 @@ class TileImprovementConstructionTests {
             // If this improvement requires additional conditions to be true,
             // its too complex to handle all of them, so just skip it and hope its fine
             // I would like some comments on whether this approach is fine or if it's better if I handle every single unique here as well
-            if (improvement.hasUnique(UniqueType.CanOnlyBeBuiltOnTile, StateForConditionals.IgnoreConditionals)) continue
-            if (improvement.hasUnique(UniqueType.Unbuildable, StateForConditionals.IgnoreConditionals)) continue
+            if (improvement.hasUnique(UniqueType.CanOnlyBeBuiltOnTile, GameContext.IgnoreConditionals)) continue
+            if (improvement.hasUnique(UniqueType.Unbuildable, GameContext.IgnoreConditionals)) continue
 
             val tile = tileMap[1,1]
             tile.baseTerrain = terrain
-            tile.resource = null
-            if (improvement.hasUnique(UniqueType.CanOnlyImproveResource, StateForConditionals.IgnoreConditionals)) {
-                tile.resource = testGame.ruleset.tileResources.values.firstOrNull { it.isImprovedBy(improvement.name) }?.name ?: continue
+            tile.tileResource = null
+            if (improvement.hasUnique(UniqueType.CanOnlyImproveResource, GameContext.IgnoreConditionals)) {
+                tile.tileResource = testGame.ruleset.tileResources.values.firstOrNull { it.isImprovedBy(improvement.name) } ?: continue
             }
             tile.setTransients()
 
             if (improvement.uniqueTo != null) {
-                civInfo.setNameForUnitTests(improvement.uniqueTo!!)
+                civInfo = testGame.addCiv(improvement.uniqueTo!!)
+                for (tech in testGame.ruleset.technologies.values)
+                    civInfo.tech.addTechnology(tech.name)
+                city.civ = civInfo
             }
 
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertTrue(improvement.name, canBeBuilt)
         }
     }
@@ -69,15 +72,15 @@ class TileImprovementConstructionTests {
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             val tile = tileMap[1,1]
-            tile.resource = testGame.ruleset.tileResources.values
-                .firstOrNull { it.isImprovedBy(improvement.name) }?.name
-            if (tile.resource == null) continue
+            tile.tileResource = testGame.ruleset.tileResources.values
+                .firstOrNull { it.isImprovedBy(improvement.name) }
+            if (tile.tileResource == null) continue
             // If this improvement requires additional conditions to be true,
             // its too complex to handle all of them, so just skip it and hope its fine
-            if (improvement.hasUnique(UniqueType.CanOnlyBeBuiltOnTile, StateForConditionals.IgnoreConditionals)) continue
+            if (improvement.hasUnique(UniqueType.CanOnlyBeBuiltOnTile, GameContext.IgnoreConditionals)) continue
 
             tile.setTransients()
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertTrue(improvement.name, canBeBuilt)
         }
     }
@@ -92,8 +95,13 @@ class TileImprovementConstructionTests {
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Can only be built on [Coastal] tiles")) continue
-            civInfo.setNameForUnitTests(improvement.uniqueTo ?: "OtherCiv")
-            val canBeBuilt = coastalTile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            if (improvement.uniqueTo != null) {
+                civInfo = testGame.addCiv(improvement.uniqueTo!!)
+                for (tech in testGame.ruleset.technologies.values)
+                    civInfo.tech.addTechnology(tech.name)
+                city.civ = civInfo
+            }
+            val canBeBuilt = coastalTile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertTrue(improvement.name, canBeBuilt)
         }
     }
@@ -105,7 +113,7 @@ class TileImprovementConstructionTests {
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Can only be built on [Coastal] tiles")) continue
             civInfo.setNameForUnitTests(improvement.uniqueTo ?: "OtherCiv")
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertFalse(improvement.name, canBeBuilt)
         }
     }
@@ -116,7 +124,7 @@ class TileImprovementConstructionTests {
             if (improvement.uniqueTo == null) continue
             civInfo.setNameForUnitTests("OtherCiv")
             val tile = tileMap[1,1]
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertFalse(improvement.name, canBeBuilt)
         }
     }
@@ -132,9 +140,9 @@ class TileImprovementConstructionTests {
             } ?: continue
             val tile = tileMap[1,1]
             tile.baseTerrain = "Plains"
-            tile.resource = wrongResource.name
+            tile.tileResource = wrongResource
             tile.setTransients()
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertFalse(improvement.name, canBeBuilt)
         }
     }
@@ -147,7 +155,7 @@ class TileImprovementConstructionTests {
         tile.addTerrainFeature("Forest")
         Assert.assertEquals(tile.terrainFeatures, listOf("Hill", "Forest"))
 
-        tile.changeImprovement("Landmark")
+        tile.setImprovement("Landmark")
         Assert.assertEquals(tile.terrainFeatures, listOf("Hill"))
     }
 
@@ -155,21 +163,21 @@ class TileImprovementConstructionTests {
     fun citadelTakesOverAdjacentTiles() {
         val tile = tileMap[1,1]
         Assert.assertFalse(tile.neighbors.all { it.owningCity == city })
-        tile.changeImprovement("Citadel", civInfo)
+        tile.setImprovement("Citadel", civInfo)
         Assert.assertTrue(tile.neighbors.all { it.owningCity == city })
     }
 
     @Test
     fun terraceFarmCanNOTBeBuiltOnBonus() {
         val tile = tileMap[1,1]
-        tile.resource = "Sheep"
+        tile.setTileResource("Sheep")
         tile.setTransients()
         tile.addTerrainFeature("Hill")
         civInfo.setNameForUnitTests("Inca")
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Cannot be built on [Bonus resource] tiles")) continue
-            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
+            val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state)
             Assert.assertFalse(improvement.name, canBeBuilt)
         }
     }
@@ -177,7 +185,7 @@ class TileImprovementConstructionTests {
     @Test
     fun buildingRoadBuildsARoad() {
         val tile = tileMap[1,1]
-        tile.improvementFunctions.changeImprovement("Road")
+        tile.improvementFunctions.setImprovement("Road")
         assert(tile.roadStatus == RoadStatus.Road)
     }
 
@@ -185,7 +193,7 @@ class TileImprovementConstructionTests {
     fun removingRoadRemovesRoad() {
         val tile = tileMap[1,1]
         tile.roadStatus = RoadStatus.Road
-        tile.improvementFunctions.changeImprovement("Remove Road")
+        tile.improvementFunctions.setImprovement("Remove Road")
         assert(tile.roadStatus == RoadStatus.None)
     }
 
@@ -193,9 +201,9 @@ class TileImprovementConstructionTests {
     fun removingForestRemovesForestAndLumbermill() {
         val tile = tileMap[1,1]
         tile.addTerrainFeature("Forest")
-        tile.improvementFunctions.changeImprovement("Lumber mill")
+        tile.improvementFunctions.setImprovement("Lumber mill")
         assert(tile.getTileImprovement()!!.name == "Lumber mill")
-        tile.improvementFunctions.changeImprovement("Remove Forest")
+        tile.improvementFunctions.setImprovement("Remove Forest")
         assert(tile.terrainFeatures.isEmpty())
         assert(tile.improvement == null) // Lumber mill can ONLY be on Forest, and is therefore removed
     }
@@ -204,11 +212,11 @@ class TileImprovementConstructionTests {
     fun removingForestRemovesForestButNotCamp() {
         val tile = tileMap[1,1]
         tile.addTerrainFeature("Forest")
-        tile.resource = "Deer"
+        tile.setTileResource("Deer")
         tile.baseTerrain = "Plains"
-        tile.improvementFunctions.changeImprovement("Camp")
+        tile.improvementFunctions.setImprovement("Camp")
         assert(tile.getTileImprovement()!!.name == "Camp")
-        tile.improvementFunctions.changeImprovement("Remove Forest")
+        tile.improvementFunctions.setImprovement("Remove Forest")
         assert(tile.terrainFeatures.isEmpty())
         assert(tile.improvement == "Camp") // Camp can be both on Forest AND on Plains, so not removed
     }
@@ -221,14 +229,14 @@ class TileImprovementConstructionTests {
 
         val improvement = testGame.createTileImprovement()
         Assert.assertFalse("Forest doesn't allow building unless allowed",
-            tile.improvementFunctions.canBuildImprovement(improvement, civInfo))
+            tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state))
 
 
         val allowedImprovement = testGame.createTileImprovement()
         allowedImprovement.terrainsCanBeBuiltOn += "Forest"
         Assert.assertTrue("Forest should allow building when allowed",
-            tile.improvementFunctions.canBuildImprovement(allowedImprovement, civInfo))
-        tile.changeImprovement(allowedImprovement.name)
+            tile.improvementFunctions.canBuildImprovement(allowedImprovement, civInfo.state))
+        tile.setImprovement(allowedImprovement.name)
         Assert.assertTrue(tile.improvement == allowedImprovement.name)
         Assert.assertTrue("Forest should not be removed with this improvement", tile.terrainFeatures.contains("Forest"))
     }
@@ -240,8 +248,8 @@ class TileImprovementConstructionTests {
         tile.addTerrainFeature("Forest")
 
         val improvement = testGame.createTileImprovement("Does not need removal of [Forest]")
-        Assert.assertTrue(tile.improvementFunctions.canBuildImprovement(improvement, civInfo))
-        tile.changeImprovement(improvement.name)
+        Assert.assertTrue(tile.improvementFunctions.canBuildImprovement(improvement, civInfo.state))
+        tile.setImprovement(improvement.name)
         Assert.assertTrue(tile.improvement == improvement.name)
         Assert.assertTrue("Forest should not be removed with this improvement", tile.terrainFeatures.contains("Forest"))
     }
@@ -253,7 +261,7 @@ class TileImprovementConstructionTests {
         tile.addTerrainFeature("Forest")
 
         val lumberMill = testGame.ruleset.tileImprovements["Lumber mill"]!!
-        tile.improvementFunctions.changeImprovement(lumberMill.name)
+        tile.improvementFunctions.setImprovement(lumberMill.name)
         assert(tile.getTileImprovement() == lumberMill)
 
         // 1f 1p from forest, 2p from lumber mill since all techs are researched
