@@ -1,8 +1,8 @@
 //  Taken from https://github.com/TomGrill/gdx-testing
 package com.unciv.uniques
 
-import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
+import com.unciv.logic.map.HexCoord
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.stats.Stats
@@ -30,7 +30,7 @@ class GlobalUniquesTests {
     @Test
     fun statsOnBuilding() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
         val building = game.createBuilding("[+1 Food]")
 
         city.cityConstructions.addBuilding(building)
@@ -57,7 +57,7 @@ class GlobalUniquesTests {
     @Test
     fun statsPerCity() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo,  game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo,  game.getTile(HexCoord.Zero), true)
         val building = game.createBuilding("[+1 Production] [in this city]")
 
         city.cityConstructions.addBuilding(building)
@@ -68,7 +68,7 @@ class GlobalUniquesTests {
     @Test
     fun statsPerSpecialist() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] from every specialist [in this city]")
         val specialistName = game.createSpecialist()
         building.specialistSlots.add(specialistName, 2)
@@ -82,7 +82,7 @@ class GlobalUniquesTests {
     @Test
     fun statsPerPopulation() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 4)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 4)
         val building = game.createBuilding("[+3 Gold] per [2] population [in this city]")
 
         city.cityConstructions.addBuilding(building)
@@ -93,7 +93,7 @@ class GlobalUniquesTests {
     @Test
     fun statsPerXPopulation() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] <in cities with at least [3] [Population]>")
 
         city.cityConstructions.addBuilding(building)
@@ -108,7 +108,7 @@ class GlobalUniquesTests {
     @Test
     fun statsFromCitiesOnSpecificTiles() {
         val civInfo = game.addCiv()
-        val tile = game.setTileTerrain(Vector2.Zero, Constants.desert)
+        val tile = game.setTileTerrain(HexCoord.Zero, Constants.desert)
         val city = game.addCity(civInfo, tile, true)
         val building = game.createBuilding("[+3 Gold] in cities on [${Constants.desert}] tiles")
         city.cityConstructions.addBuilding(building)
@@ -125,27 +125,40 @@ class GlobalUniquesTests {
     fun statsFromTiles() {
         game.makeHexagonalMap(2)
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles [in all cities]")
         city.cityConstructions.addBuilding(building)
 
-        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
     }
 
+    @Test
+    fun statsFromTilesMultifilter() {
+        game.makeHexagonalMap(2)
+        val civInfo = game.addCiv()
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
+        val building = game.createBuilding("[+4 Gold] from [{${Constants.grassland}} {Farm}] tiles [in all cities]")
+        city.cityConstructions.addBuilding(building)
+
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
+        tile2.setImprovement("Farm")
+        Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
+    }
+    
     @Test
     fun statsFromTilesWithout() {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles without [${Constants.forest}] [in this city]")
         city.cityConstructions.addBuilding(building)
 
-        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
         game.addTileToCity(city, tile2)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
 
-        val tile3 = game.setTileTerrainAndFeatures(Vector2(0f, 2f), Constants.grassland, Constants.forest)
+        val tile3 = game.setTileTerrainAndFeatures(HexCoord(0,2), Constants.grassland, Constants.forest)
         game.addTileToCity(city, tile3)
         Assert.assertFalse(tile3.stats.getTileStats(city, civInfo).gold == 4f)
     }
@@ -154,7 +167,7 @@ class GlobalUniquesTests {
     fun statsFromObject() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 2)
         val specialist = game.createSpecialist()
         val building = game.createBuilding("[+3 Faith] from every [${specialist}]")
 
@@ -168,7 +181,7 @@ class GlobalUniquesTests {
         val building2 = game.createBuilding("[+3 Faith] from every [${Constants.grassland}]")
         city.cityConstructions.addBuilding(building2)
 
-        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 3f)
 
         city.cityConstructions.removeBuilding(building2)
@@ -187,14 +200,14 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv("[+30 Science] from each Trade Route")
         civInfo.tech.addTechnology("The Wheel") // Required to form trade routes
-        val tile1 = game.getTile(Vector2.Zero)
-        val tile2 = game.getTile(Vector2(0f, 2f))
+        val tile1 = game.getTile(HexCoord.Zero)
+        val tile2 = game.getTile(HexCoord(0,2))
         tile1.roadStatus = RoadStatus.Road
         tile2.roadStatus = RoadStatus.Road
         @Suppress("UNUSED_VARIABLE")
         val city1 = game.addCity(civInfo, tile1)
         val city2 = game.addCity(civInfo, tile2)
-        val inBetweenTile = game.getTile(Vector2(0f, 1f))
+        val inBetweenTile = game.getTile(HexCoord(0,1))
         inBetweenTile.roadStatus = RoadStatus.Road
         civInfo.cache.updateCitiesConnectedToCapital()
         city2.cityStats.update()
@@ -215,14 +228,35 @@ class GlobalUniquesTests {
         Assert.assertTrue(civInfo.stats.getStatMapForNextTurn()["Policies"]!!.science == 30f)
     }
 
+    @Test
+    fun uniqueTypeOneTimeAdoptPolicyOrBelief() {
+        val civ = game.addCiv()
+        val tile = game.getTile(HexCoord.Zero)
+        val city = game.addCity(civ, tile, true)
+
+        // Policy
+        Assert.assertFalse(civ.policies.isAdopted("Legalism"))
+        city.cityConstructions.addBuilding(game.createBuilding("Adopt [Legalism]"))
+        Assert.assertTrue(civ.policies.isAdopted("Legalism"))
+
+        // Belief
+        Assert.assertFalse(civ.religionManager.religion?.hasBelief("Dance of the Aurora") ?: false)
+        city.cityConstructions.addBuilding(game.createBuilding("Adopt [Dance of the Aurora]")) // Expected to not work, since we require a Religion
+        Assert.assertFalse(civ.religionManager.religion?.hasBelief("Dance of the Aurora") ?: false)
+        game.addReligion(civ) // With the Religion, we should be able to adopt the belief
+        Assert.assertFalse(civ.religionManager.religion?.hasBelief("Dance of the Aurora") ?: false)
+        city.cityConstructions.addBuilding(game.createBuilding("Adopt [Dance of the Aurora]"))
+        Assert.assertTrue(civ.religionManager.religion?.hasBelief("Dance of the Aurora") ?: false)
+    }
+
     @Test
     fun statsFromGlobalCitiesFollowingReligion() {
         val civ1 = game.addCiv()
         val religion = game.addReligion(civ1)
         val belief = game.createBelief(BeliefType.Founder, "[+30 Science] for each global city following this religion")
-        religion.founderBeliefs.add(belief.name)
+        religion.addBeliefs(listOf(belief))
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 1) // Need someone to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000)
 
@@ -238,9 +272,9 @@ class GlobalUniquesTests {
         val civ1 = game.addCiv()
         val religion = game.addReligion(civ1)
         val belief = game.createBelief(BeliefType.Founder, "[+42 Happiness] for each global city following this religion")
-        religion.founderBeliefs.add(belief.name)
+        religion.addBeliefs(listOf(belief))
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 1) // Need someone to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000)
 
@@ -256,9 +290,9 @@ class GlobalUniquesTests {
         val civ1 = game.addCiv()
         val religion = game.addReligion(civ1)
         val belief = game.createBelief(BeliefType.Founder, "[+30 Science] from every [3] global followers [in all cities]")
-        religion.founderBeliefs.add(belief.name)
+        religion.addBeliefs(listOf(belief))
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 9) // Need people to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000000000) // To completely overwhelm the default atheism in a city
 
@@ -274,7 +308,7 @@ class GlobalUniquesTests {
     @Test
     fun statPercentBonus() {
         val civ = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]", "[+200]% [Science]")
         city.cityConstructions.addBuilding(building)
@@ -286,7 +320,7 @@ class GlobalUniquesTests {
     @Test
     fun statPercentBonusCities() {
         val civ = game.addCiv("[+200]% [Science] [in all cities]")
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]")
         city.cityConstructions.addBuilding(building)
@@ -304,13 +338,13 @@ class GlobalUniquesTests {
                 "[+200]% [Faith] from every [${emptyBuilding.name}]",
                 "[+200]% [Faith] from every [Farm]",
             )
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
         city.cityConstructions.addBuilding(faithBuilding)
 
-        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
-        tile2.changeImprovement("Farm")
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
+        tile2.setImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
         city.cityConstructions.addBuilding(emptyBuilding)
@@ -328,13 +362,13 @@ class GlobalUniquesTests {
                 "[+200]% Yield from every [${emptyBuilding.name}]",
                 "[+200]% Yield from every [Farm]",
             )
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
         city.cityConstructions.addBuilding(faithBuilding)
 
-        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
-        tile2.changeImprovement("Farm")
+        val tile2 = game.setTileTerrain(HexCoord(0,1), Constants.grassland)
+        tile2.setImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
         city.cityConstructions.addBuilding(emptyBuilding)
@@ -347,10 +381,10 @@ class GlobalUniquesTests {
     fun statPercentFromReligionFollowers() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, 1)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, 1)
         val religion = game.addReligion(civInfo)
         val belief = game.createBelief(BeliefType.Follower, "[+10]% [Faith] from every follower, up to [42]%")
-        religion.followerBeliefs.add(belief.name)
+        religion.addBeliefs(listOf(belief))
 
         city.religion.addPressure(religion.name, 1000000000)
 
@@ -370,12 +404,12 @@ class GlobalUniquesTests {
         val civInfo = game.addCiv()
         val cityState = game.addCiv(cityStateType = "Maritime")
 
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
-        val cityStateTile = game.getTile(Vector2(0f, 1f))
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
+        val cityStateTile = game.getTile(HexCoord(0,1))
         @Suppress("UNUSED_VARIABLE")
         val cityStateCity = game.addCity(cityState, cityStateTile, true)
         civInfo.diplomacyFunctions.makeCivilizationsMeet(cityState)
-        cityState.getDiplomacyManager(civInfo).addInfluence(100f)
+        cityState.getDiplomacyManager(civInfo)!!.addInfluence(100f)
 
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList[Constants.cityStates]!!.food == 3f)
@@ -391,14 +425,14 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv("[+30 Science] from each Trade Route", "[+100]% [Science] from Trade Routes")
         civInfo.tech.addTechnology("The Wheel") // Required to form trade routes
-        val tile1 = game.getTile(Vector2.Zero)
-        val tile2 =  game.getTile(Vector2(0f, 2f))
+        val tile1 = game.getTile(HexCoord.Zero)
+        val tile2 =  game.getTile(HexCoord(0,2))
         tile1.roadStatus = RoadStatus.Road
         tile2.roadStatus = RoadStatus.Road
         @Suppress("UNUSED_VARIABLE")
         val city1 = game.addCity(civInfo, tile1)
         val city2 = game.addCity(civInfo, tile2)
-        val inBetweenTile =  game.getTile(Vector2(0f, 1f))
+        val inBetweenTile =  game.getTile(HexCoord(0,1))
         inBetweenTile.roadStatus = RoadStatus.Road
 
         civInfo.cache.updateCitiesConnectedToCapital()
@@ -417,7 +451,7 @@ class GlobalUniquesTests {
     fun nullifiesStat() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies [Faith] [in this city]", "[+10 Gold, +10 Faith] [in this city]")
@@ -431,7 +465,7 @@ class GlobalUniquesTests {
     fun nullifiesGrowth() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies Growth [in this city]", "[+10 Food, +10 Gold] [in this city]")
@@ -452,7 +486,7 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionBuildings() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val buildingToConstruct = game.createBuilding()
@@ -466,7 +500,7 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionUnits() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val unitToConstruct = game.createBaseUnit()
@@ -480,7 +514,7 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionWonders() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val buildingToConstruct = game.createBuilding()
@@ -499,9 +533,9 @@ class GlobalUniquesTests {
     fun percentProductionBuildingsInCapital() {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv("[+300]% Production towards any buildings that already exist in the Capital")
-        val tile = game.getTile(Vector2(0f,2f))
+        val tile = game.getTile(HexCoord(0,2))
         val city = game.addCity(civInfo, tile, true, 0)
-        val city2 = game.addCity(civInfo, game.getTile(Vector2(0f, -2f)), initialPopulation = 0)
+        val city2 = game.addCity(civInfo, game.getTile(HexCoord(0,-2)), initialPopulation = 0)
 
         val buildingToConstruct = game.createBuilding()
         city2.cityConstructions.addToQueue(buildingToConstruct.name)
@@ -522,8 +556,9 @@ class GlobalUniquesTests {
     @Test
     fun growthPercentBonusTest() {
         val civInfo = game.addCiv()
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
-        val building = game.createBuilding("[+100]% growth [in all cities]")
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
+        // City has 2 food from center -2 from pop, so total of 0
+        val building = game.createBuilding("[+100]% growth [in all cities]", "[+2 Food]")
         city.cityConstructions.addBuilding(building)
 
         city.cityStats.update()
@@ -533,7 +568,7 @@ class GlobalUniquesTests {
     @Test
     fun carryOverFoodTest() {
         val civInfo = game.addCiv("[50]% Food is carried over after population increases [in all cities]")
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
 
         val foodNecessary = city.population.getFoodToNextPopulation()
         city.population.nextTurn(foodNecessary)
@@ -545,7 +580,7 @@ class GlobalUniquesTests {
     @Test
     fun foodConsumptionBySpecialistsTest() {
         val civInfo = game.addCiv("[-50]% Food consumption by specialists [in all cities]")
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 1)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 1)
 
         val building = game.createBuilding()
         val specialistName = game.createSpecialist()
@@ -564,26 +599,21 @@ class GlobalUniquesTests {
     @Test
     fun unhappinessFromCitiesPercentageTest() {
         val civInfo = game.addCiv("[+100]% unhappiness from the number of cities")
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
 
         city.cityStats.update()
-        // Because of some weird design choices, -3.6 is correct, though I would expect it to be -6 instead.
-        // As I'm not certain enough in my feeling that it should be -6, I'm changing the test to fit the code instead of the other way around.
-        // I've also written some text about this in CityStats.updateCityHappiness(). ~xlenstra
         println(city.cityStats.happinessList)
-        Assert.assertTrue(abs(city.cityStats.happinessList["Cities"]!! - -3.6f) < epsilon) // Float rounding errors
+        Assert.assertTrue(abs(city.cityStats.happinessList["Cities"]!! - -6f) < epsilon) // Float rounding errors
     }
 
     @Test
     fun unhappinessFromPopulationTypePercentageChangeTest() {
         val civInfo = game.addCiv("[-50]% Unhappiness from [Population] [in all cities]")
-        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 4)
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true, initialPopulation = 4)
 
         city.cityStats.update()
-        // This test suffers from the same problems as `unhappinessFromCitiesPercentageTest()`.
-        // This should be -2, I believe, as every pop should add -1 happiness and 4 pop with -50% unhappiness = -2 unhappiness.
         println(city.cityStats.happinessList)
-        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -1.2f) < epsilon)
+        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -2f) < epsilon)
 
         val building = game.createBuilding("[-50]% Unhappiness from [Specialists] [in all cities]")
         val specialist = game.createSpecialist()
@@ -593,22 +623,96 @@ class GlobalUniquesTests {
 
         city.cityStats.update()
         println(city.cityStats.happinessList)
-        // This test suffers from the same problems as above. It should be -1, I believe.
-        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -0.6f) < epsilon)
+        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -1f) < epsilon)
     }
 
 
     // endregion happiness
 
+    // region Conditionals
+
+    @Test
+    fun conditionalDifficulty() {
+        val civInfo = game.addCiv()
+        val tile = game.getTile(HexCoord.Zero)
+        val city = game.addCity(civInfo, tile, true)
+
+        val tests = listOf(
+            "<on [Settler] difficulty>" to 0,
+            "<on [Chieftain] difficulty>" to 0,
+            "<on [Prince] difficulty>" to 1, // TestGame is Prince
+            "<on [King] difficulty>" to 0,
+            "<on [Emperor] difficulty>" to 0,
+            "<on [Settler] difficulty or higher>" to 1,
+            "<on [Chieftain] difficulty or higher>" to 1,
+            "<on [Prince] difficulty or higher>" to 1,
+            "<on [King] difficulty or higher>" to 0,
+            "<on [Emperor] difficulty or higher>" to 0,
+            "<on [Immortal] difficulty or higher>" to 0,
+            "<on [Deity] difficulty or higher>" to 0,
+            "<on [Settler] difficulty or lower>" to 0,
+            "<on [Chieftain] difficulty or lower>" to 0,
+            "<on [Prince] difficulty or lower>" to 1,
+            "<on [King] difficulty or lower>" to 1,
+            "<on [Emperor] difficulty or lower>" to 1,
+            "<on [Immortal] difficulty or lower>" to 1,
+            "<on [Deity] difficulty or lower>" to 1,
+            "<on [Not Found] difficulty>" to 0,
+            "<on [Not Found] difficulty or higher>" to 0,
+            "<on [Not Found] difficulty or lower>" to 0
+        )
+
+        Assert.assertEquals(civInfo.gold, 0)
+        for ((test, expected) in tests) {
+            val building = game.createBuilding("Gain [1] [Gold] $test")
+            city.cityConstructions.addBuilding(building)
+            Assert.assertEquals("Conditional `$test` should be: $expected", civInfo.gold, expected)
+            civInfo.addGold(-civInfo.gold) // Reset the gold
+        }
+    }
+
+    @Test
+    fun conditionalFirstCivToAdopt() {
+        val civInfo = game.addCiv()
+        val city = game.addCity(civInfo, game.getTile(HexCoord.Zero), true)
+        // Policy
+        civInfo.policies.run {
+            freePolicies++
+            adopt(getPolicyByName("Tradition"))
+        }
+        // Belief
+        game.addReligion(civInfo)
+        civInfo.religionManager.religion?.run {
+            addBelief("Ancestor Worship")
+            addBelief("Not A Belief")
+        }
+        val tests = listOf(
+            "<if no Civilization has adopted [Oligarchy]>" to 1,
+            "<if no Civilization has adopted [Tradition]>" to 0,
+            "<if no Civilization has adopted [God of Craftsman]>" to 1,
+            "<if no Civilization has adopted [Not A Policy]>" to 1,
+            "<if no Civilization has adopted [Ancestor Worship]>" to 0,
+            "<if no Civilization has adopted [Not A Belief]>" to 1,
+        )
+        Assert.assertEquals(civInfo.gold, 0)
+        for ((test, expected) in tests) {
+            val building = game.createBuilding("Gain [1] [Gold] $test")
+            city.cityConstructions.addBuilding(building)
+            Assert.assertEquals("Conditional `$test` should be: $expected", civInfo.gold, expected)
+            civInfo.addGold(-civInfo.gold) // Reset the gold
+        }
+    }
+
+    // endregion
 
     // region Great Persons
     @Test
     fun statsSpendingGreatPeople() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2.Zero)
+        val tile = game.getTile(HexCoord.Zero)
         val city = game.addCity(civInfo, tile, true)
         val unit = game.addUnit("Great Engineer", civInfo, tile)
-        val building = game.createBuilding("[+250 Gold] whenever a Great Person is expended")
+        val building = game.createBuilding("Gain [250] [Gold] <upon expending a [Great Person] unit>")
         city.cityConstructions.addBuilding(building)
 
         civInfo.addGold(-civInfo.gold) // reset gold just to be sure
@@ -618,6 +722,39 @@ class GlobalUniquesTests {
     }
     // endregion
 
+    // region Gain control over tiles
+
+    @Test
+    fun takeOverTilesInCity() {
+        game.makeHexagonalMap(5)
+        val civInfo = game.addCiv()
+        val tile = game.getTile(HexCoord.Zero)
+        val city = game.addCity(civInfo, tile, true)
+        Assert.assertEquals(7, city.getTiles().count())
+        val building = game.createBuilding("Gain control over [8] tiles [in this city]")
+        city.cityConstructions.addBuilding(building)
+        Assert.assertEquals(15, city.getTiles().count())
+    }
+
+    // endregion
+
+    // region Can carry air units
+
+    @Test
+    fun testCarryExtraAirUnits() {
+        game.makeHexagonalMap(5)
+        val civInfo = game.addCiv()
+        val tile = game.getTile(HexCoord.Zero)
+        val city = game.addCity(civInfo, tile, true)
+        Assert.assertEquals(6, city.getMaxAirUnits())
+        city.cityConstructions.addBuilding(game.createBuilding("Can carry [6] extra [Air] units <in this city>"))
+        Assert.assertEquals(12, city.getMaxAirUnits())
+        city.cityConstructions.addBuilding(game.createBuilding("Can carry [-3] extra [Air] units <in this city>"))
+        Assert.assertEquals(9, city.getMaxAirUnits())
+    }
+
+    // endregion
+
     // endregion
 
 }
